package com.focustech.baseandroid.adapter;

import static com.focustech.baseandroid.general.BaseAnnotation.getAnnotation;
import android.content.Context;
import android.database.Cursor;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;

import java.util.HashMap;
import java.util.List;

import com.focustech.baseandroid.db.bean.DBean;
import com.focustech.baseandroid.general.BaseAnnotation.setContentView;
import com.focustech.baseandroid.view.ExceptionView;

/**
 * 对BaseAdapter进行二次封装，内部默认实现Holder缓存，和Item的复用
 * 不对外暴露，外部使用可以继承BasicAdapter FilterAdapter BaseDbAdapter
 * Created by zhangzheng on 2015/11/9.
 */
abstract class AbsBasicAdapter<M,T extends DBean> extends BaseAdapter{
	
	public AbsBasicAdapter(Context context ,List<M> datas){
		setContext(context);
		setAdapterData(datas);
	}
	
	public AbsBasicAdapter(Context context ,T bean){
		setContext(context);
		cursor = bean.getCursor();
		this.bean=bean;
	}
	
    private View currenShowView;

    protected List<M> dataBean;

    protected Context context;
    
    protected Cursor cursor;
    
    private T bean;
    @Override
    public int getCount() {
    	if(cursor!=null){
    		return cursor.getCount();
    	}
        return dataBean==null?0:dataBean.size();
    }

    @Override
    public Object getItem(int position) {
    	if(cursor!=null){
    		return bean;
    	}
        return dataBean==null?null:dataBean.get(position);
    }

    @Override
    public long getItemId(int position) {
        return 0;
    }

    /**
     * 设置Adapter显示的数据源
     * @param dateBean
     */
    protected void setAdapterData(List<M> dateBean){
        this.dataBean=dateBean;
    }

    /**
     * 设置Adapter的上下文对象
     * @param context
     */
    protected void setContext(Context context){
        this.context=context;
    }

    /**
     * 为View绑定数据
     * @param position
     * @param data
     * @param convertView
     * @return
     */
    protected abstract View initItemViewData(int position,M data,View convertView);

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        if(convertView!=null&&!(convertView instanceof ExceptionView)){
        	convertView=initItemDataGetView(position, (M) getItem(position), convertView);
            return convertView;
        }
        
        if(context==null){
            throw  new RuntimeException("请调用setContext（）");
        }
        
        convertView=LayoutInflater.from(context)
        		.inflate(getAnnotation(this, setContentView.class).value(), null);
        
        convertView=initItemDataGetView(position, (M) getItem(position), convertView);
        
        return convertView;
    }
    
    //初始化数据并且得到初始化后的View
    private View initItemDataGetView(int position,M data,View convertView){
    	this.currenShowView=convertView;
        convertView=initItemViewData(position, (M) getItem(position), convertView);
        this.currenShowView=convertView;
        return convertView;
    }
    
    /**
     * 节约了再次查找控件的事件  代替Holder优化
     * @param id 控件ID
     * @param <T>
     * @return
     */
    protected <T extends View> T fv(int id){
        if(currenShowView==null){
            return null;
        }
        HashMap<Integer,View> hashmapViews = (HashMap<Integer, View>) currenShowView.getTag();

        T view;
        if(hashmapViews==null||!hashmapViews.containsKey(id)) {
            view = (T) currenShowView.findViewById(id);
        }else{
            view= (T) hashmapViews.get(id);
        }
        return view;
    }
    
    /**
     * 节约了再次查找控件的事件  代替Holder优化
     * @param id 控件ID
     * @param <T> 指定找到控件的类型
     * @return
     */
    protected <T extends View> T fv(int id,Class<T> cla){
    	return fv(id);
    }
}
