package com.focustech.baseandroid.adapter;

import java.lang.reflect.Field;
import java.util.List;

import com.focustech.baseandroid.view.ExceptionView;

import android.content.Context;
import android.view.View;
import android.widget.TextView;

/**
 * 带有过滤功能的Adapter 更方便的实现过滤
 * 内部默认实现Holder缓存，和Item的复用
 * @author zhangzheng
 *
 * @param <M>
 */
public abstract class FilterAdapter<M> extends BasicAdapter<M> {

	public FilterAdapter(Context context, List<M> datas) {
		super(context, datas);
	}

	private int tvId;

	private String fieldName;

	private String filterContent = "";
	
	
	protected View initItemViewData(int position, M data, View convertView) {
		convertView=initFilterItemViewData(position,data,convertView);
		
		if (tvId != 0) {
			if (!fv(tvId, TextView.class).getText().toString().contains(filterContent)) {
				return new ExceptionView(context);
			}
		}
		
		if(fieldName!=null){
			 try {
				Field declaredField = data.getClass().getDeclaredField(fieldName);
				String content=(String) declaredField.get(data);
				if(!content.contains(filterContent)){
					return new ExceptionView(context); 
				}
			} catch (NoSuchFieldException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			}
		}

		return convertView;
	}
	
	/**
	 * 初始化Item的视图和数据
	 * @param position 位置
	 * @param data 数据 
	 * @param convertView item视图
	 * @return
	 */
	protected abstract View initFilterItemViewData(int position, M data, View convertView);
	
	 /**
	  * 根据布局item中textView的值进行过滤
	  * @param tvId  布局中TestView的id
	  * @param filterContent
	  */
	public void filter(int tvId, String filterContent) {
		this.tvId = tvId;
		this.filterContent = filterContent;
		this.fieldName=null;
		notifyDataSetChanged();
	}

	/**
	 * 根据bean的参数名称和值进行过滤
	 * @param fieldName 变量名
	 * @param filterContent 变量值
	 */
	public void filter(String fieldName, String filterContent) {
		this.fieldName = fieldName;
		this.filterContent = filterContent;
		this.tvId=0;
		notifyDataSetChanged();
	}

}
