package com.focustech.baseandroid.db;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.List;

import com.focustech.baseandroid.BaseApp;
import com.focustech.baseandroid.db.interfaces.DbAble;
import com.focustech.baseandroid.general.BaseAnnotation;
import com.focustech.baseandroid.general.BaseAnnotation.isNewDbField;
import com.focustech.baseandroid.util.BaseViewUtils;
import com.focustech.baseandroid.util.PropertyDescriptor;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

/**
 * 操作数据库的帮助类，系统默认的操作数据库方法，我进行了封装，将初始数据库通过参数进行配置
 * SQLite 仅仅支持 ALTER TABLE 语句的一部分功能，我们可以用 ALTER TABLE 语句来更改一个表的名字，
 * 也可向表中增加一个字段（列），但是我们不能删除一个已经存在的字段，或者更改一个已经存在的字段的名称、数据类型、限定符等等。
 * 
 * @author zhangzheng
 * 
 */
public class BaseSqliteOpenHelp extends SQLiteOpenHelper {

	private final DbAble dbBean;

	private BaseSqliteOpenHelp(Context context, String name, int version,
			DbAble dbBean) {
		super(context, name, null, version);
		this.dbBean = dbBean;
	}

	@Override
	public void onCreate(SQLiteDatabase db) {
		db.execSQL(getSql());
	}

	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		if (!BaseApp.getAPP().onDbUpgrade(dbBean.getTabName(),db, oldVersion, newVersion)) {
			dbTableChange(db,oldVersion,newVersion);
		}
		BaseApp.getAPP().onDbUpdateCompute();
	}

	@Override
	public void onDowngrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		if (!BaseApp.getAPP().onDbDowngrade(dbBean.getTabName(),db, oldVersion, newVersion)) {
			dbTableChange(db,oldVersion,newVersion);
		}
		BaseApp.getAPP().onDbUpdateCompute();
	}

	/**
	 * 数据库升级或者降级进行处理
	 * alter table 表名 add 字段名 类型
	 * @param db
	 */
	private void dbTableChange(SQLiteDatabase db, int oldVersion, int newVersion) {
		List<Field> fieldsHasAnnotation = BaseAnnotation.getFieldsHasAnnotation(dbBean, isNewDbField.class);
		for (Field field : fieldsHasAnnotation) {
			if(field.getAnnotation(isNewDbField.class).startDbVerson()>oldVersion){
				db.execSQL("alter table "+dbBean.getTabName()+" add "+field.getName()+" varchar");
			}
		}
//		List<DbAble> selectAll = SqliteHelp.selectAll(dbBean, db);
//		db.execSQL("drop table if exists " + dbBean.getTabName());
//		onCreate(db);
//		SqliteHelp.inserts(selectAll, db);
	}

	/**
	 * 获得实例
	 * 
	 * @param dbBean
	 * @return
	 */
	public static BaseSqliteOpenHelp getSqlHelp(DbAble dbBean) {
		Context context = BaseApp.getContext();
		String nameVerson = BaseViewUtils.getMetaData("dbNameVerson", context);
		String name = nameVerson.split("-")[0];
		String verson = nameVerson.split("-")[1];
		BaseSqliteOpenHelp baseSqliteOpenHelp = new BaseSqliteOpenHelp(context,
				name, Integer.valueOf(verson), dbBean);

		return baseSqliteOpenHelp;
	}

	@Override
	public SQLiteDatabase getWritableDatabase() {
		SQLiteDatabase writableDatabase = super.getWritableDatabase();
		onCreate(writableDatabase);
		return writableDatabase;
	}

	/**
	 * 获得bean的建表语句
	 * 
	 * @return
	 */
	public String getSql() {
		return "create table if not exists " + dbBean.getTabName() + "("
				+ dbBean.getprimaryKey() + " integer primary key,"
				+ getNormalFieldSql();
	}

	private String getNormalFieldSql() {
		List<Field> usedFields = SqliteHelp.getUsedFields(dbBean);
		StringBuilder sql = new StringBuilder();
		for (Field field : usedFields) {
			sql.append(decodeFiledToSql(field));
		}
		sql.deleteCharAt(sql.length() - 1);
		return sql.toString() + ")";
	}

	private String decodeFiledToSql(Field field) {

		Method getMethod = PropertyDescriptor.getGetMethod(dbBean.getClass(),
				field.getName());

		if (getMethod != null) {
			return field.getName() + " varchar,";
		}

		return "";
	}

}
