package com.focustech.baseandroid.db.bean;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

import com.alibaba.fastjson.annotation.JSONCreator;
import com.alibaba.fastjson.annotation.JSONField;
import com.alibaba.fastjson.annotation.JSONType;
import com.focustech.baseandroid.bean.BaseBean;
import com.focustech.baseandroid.db.BaseSqliteOpenHelp;
import com.focustech.baseandroid.db.SqliteHelp;
import com.focustech.baseandroid.db.interfaces.DbAble;
import com.focustech.baseandroid.general.BaseAnnotation;
import com.focustech.baseandroid.general.BaseAnnotation.isPrimaryKey;
import com.focustech.baseandroid.general.BaseAnnotation.setTabName;
import com.focustech.baseandroid.util.PropertyDescriptor;

/**
 * 具有操作数据库能力的高级功能数据模型
 * @author zhangzheng
 *
 */
public class DBean extends BaseBean implements DbAble{

	@JSONField(serialize=false)
	private SQLiteDatabase writableDatabase;
	
	@JSONField(serialize=false)
	private SqliteHelp<? extends DbAble> sqliteHelp;
	
	@JSONField(serialize=false)
	private static HashMap<Class<? extends DBean>, SQLiteDatabase> dbmaps;
	static{
		dbmaps=new HashMap<>();
	}
	
	/**
	 * 强制关闭所有的数据库对象
	 */
	@JSONField(serialize=false)
	public static void forcedClosed(){
		Collection<SQLiteDatabase> values = dbmaps.values();
		for (SQLiteDatabase sqLiteDatabase : values) {
			if(sqLiteDatabase.isOpen()){
				sqLiteDatabase.close();
			}
		}
		dbmaps.clear();
	}
	
	@JSONField(serialize=false)
	private static boolean isSupportTransaction=false;
	
	/**
	 * 设置打开支持事物
	 * @param isSupportTransaction
	 */
	@JSONField(serialize=false)
	public void openSupportTransaction() {
		
		if(writableDatabase==null){
			writableDatabase=getDb();
		}
		if(isSupportTransaction&&writableDatabase.isOpen()){
			writableDatabase.beginTransaction();
		}
		DBean.isSupportTransaction = true;
	}

	/**
	 * 得到数据库操作对象
	 * @return
	 */
	@Override
	@JSONField(serialize=false)
	public SQLiteDatabase getDb() {
		if(writableDatabase==null){
			writableDatabase=dbmaps.get(getClass());
		}
		
		if (writableDatabase != null && writableDatabase.isOpen()) {
			return writableDatabase;
		}
		
		BaseSqliteOpenHelp baseSqliteOpenHelp = BaseSqliteOpenHelp.getSqlHelp(this);
		writableDatabase = baseSqliteOpenHelp.getWritableDatabase();
		dbmaps.put(getClass(), writableDatabase);
		return writableDatabase;
	}

	@SuppressWarnings("rawtypes")
	@JSONField(serialize=false)
	private SqliteHelp getSqliteHelp() {
		if (sqliteHelp == null) {
			sqliteHelp = new SqliteHelp<DBean>(this);
		}
		return sqliteHelp;
	}

	/**
	 * 根据表名 和值获得一个填充好数据的Bean
	 * 
	 * @param fieldName
	 * @param value
	 */
	@SuppressWarnings("unchecked")
	@JSONField(serialize=false)
	public <T extends BaseBean> T fillData(String fieldName, String value) {
		getSqliteHelp().select(fieldName, value);
		return (T) this;
	}
	
	/**
	 * 查找数据库
	 * @return 返回的结果集
	 */
	@SuppressWarnings("unchecked")
	@JSONField(serialize=false)
	public <T extends BaseBean> List<T> select(){
		return getSqliteHelp().select();
	}
	
	/**
	 * 根据SQL语句查找数据库  只需要Sql语句的条件
	 * @param sql
	 * @return
	 */
	@SuppressWarnings("unchecked")
	@JSONField(serialize=false)
	public <T extends BaseBean> List<T> select(String wheresql){
		return getSqliteHelp().select(wheresql);
	}
	
	/**
	 * 获得数据库结果集游标
	 * @return
	 */
	@JSONField(serialize=false)
	public Cursor getCursor(){
		return getSqliteHelp().getCursor();
	}
	
	
	/**
	 * 查找指定个数的
	 * @param cla
	 * @param start 开始的位置
	 * @param count 个数
	 * @return
	 */
	@JSONField(serialize=false)
	public static <T extends DBean> List<T> selectLimit(Class<T> cla,int start,int count){
		T bean = null;
		try {
			bean = cla.newInstance();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}
		return SqliteHelp.selectLimit(bean, start, count);
		
	}

	/**
	 * 插入数据到数据库
	 */
	@JSONField(serialize=false)
	public void insert() {
		getSqliteHelp().insert();
	}
	
	/**
	 * 更新数据到数据库
	 * @param updateBean 更新的数据Bean
	 */
	@SuppressWarnings("unchecked")
	@JSONField(serialize=false)
	public <T extends DBean> void update(T updateBean) {
		getSqliteHelp().update(updateBean);
	}

	/**
	 * 更新数据 根据条件
	 * @param fieldName 变量名
	 * @param FieldValue 变量值
	 */
	@JSONField(serialize=false)
	public  void update(String fieldName,String FieldValue) {
		getSqliteHelp().update(fieldName,FieldValue);
	}

	/**
	 * 插入多条数据
	 * 
	 * @param beans
	 */
	@JSONField(serialize=false)
	public static <T extends DBean> void inserts(List<T> beans) {
		SqliteHelp.inserts(beans);
	}
	
	/**
	 * 删除数据
	 */
	@JSONField(serialize=false)
	public void delete(){
		getSqliteHelp().delete();
	}

	/**
	 * 删除所有数据
	 * 
	 * @param cla
	 */
	@JSONField(serialize=false)
	public static <T extends DBean> int deleteAll(Class<T> cla) {
		T bean = null;
		try {
			bean = cla.newInstance();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}
		return SqliteHelp.deleteAll(bean);
	}

	/**
	 * 查找所有的bean从数据库
	 * 
	 * @param bean
	 * @return
	 */
	@JSONField(serialize=false)
	public static <T extends DBean> List<T> selectAll(Class<T> cla) {
		T bean = null;
		try {
			bean = cla.newInstance();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}
		return SqliteHelp.selectAll(bean);
	}

	/**
	 * 获得表名
	 * 
	 * @return
	 */
	@JSONField(serialize=false)
	@Override
	public String getTabName() {
		setTabName annotation = BaseAnnotation.getAnnotation(this,
				setTabName.class);

		if (annotation == null) {
			throw new RuntimeException("请为DBean的子类设置setTabName");
		}
		return annotation.value();
	}

	/**
	 * 获得主键
	 * 
	 * @return
	 */
	@JSONField(serialize=false)
	public String getprimaryKey() {
		Field fieldHasAnnotation = BaseAnnotation.getFieldHasAnnotation(this,
				isPrimaryKey.class);

		if (fieldHasAnnotation == null) {
			throw new RuntimeException("请为DBean的子类设置主键isPrimaryKey");
		}

		return fieldHasAnnotation.getName();
	}

	/**
	 * 获得bean的建表语句
	 * 
	 * @return
	 */
	@JSONField(serialize=false)
	public String getSql() {
		return "create table if not exists " + getTabName() + "("
				+ getprimaryKey() + " integer primary key,"
				+ getNormalFieldSql();
	}
	
	
	@JSONField(serialize=false)
	private String getNormalFieldSql() {
		List<Field> usedFields = SqliteHelp.getUsedFields(this);
		StringBuilder sql = new StringBuilder();
		for (Field field : usedFields) {
			sql.append(decodeFiledToSql(field));
		}
		sql.deleteCharAt(sql.length() - 1);
		return sql.toString() + ")";
	}

	@JSONField(serialize=false)
	private String decodeFiledToSql(Field field) {

		Method getMethod = PropertyDescriptor.getGetMethod(this.getClass(),
				field.getName());

		if (getMethod != null) {
			return field.getName() + " varchar,";
		}

		return "";
	}

	/**
	 * 关闭对数据库的资源引用
	 */
	@JSONField(serialize=false)
	public void close() {
		
		if(isSupportTransaction&&writableDatabase.isOpen()){
			writableDatabase.setTransactionSuccessful();
			writableDatabase.endTransaction();
			isSupportTransaction=false;
		}
		
		if (writableDatabase!=null&&writableDatabase.isOpen()) {
			writableDatabase.close();
		}
		writableDatabase = null;
		sqliteHelp = null;
		dbmaps.remove(getClass());
	}
}
