package com.focustech.baseandroid.general;

import java.lang.annotation.Annotation;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import com.focustech.baseandroid.manager.NetStateManager.NetState;

/**
 * Created by zhangzheng on 2015/10/15.
 * 基础工程用到的注解
 */
public class BaseAnnotation {

    /**
     * 设置Activity的contentVIew
     */
    @Target(ElementType.TYPE)
    @Retention(RetentionPolicy.RUNTIME)
    public @interface setContentView {
        int value() default 0;
    }

    /**
     * 是否为点击事件
     * @author zhangzheng
     *
     */
    @Target(ElementType.METHOD)
    @Retention(RetentionPolicy.RUNTIME)
    public @interface isClickMethod {
    	int [] value() default {};
    }
    
    /**
     * 是否为请求结果
     * @author zhangzheng
     *
     */
    @Target(ElementType.METHOD)
    @Retention(RetentionPolicy.RUNTIME)
    public @interface isWebResult {
    	boolean receiveNull() default true;
    	
    	boolean receiveUnqualified() default true;
    }

    /**
     * 是否为EvenBus的方法
     * @author zhangzheng
     *
     */
    @Target(ElementType.METHOD)
    @Retention(RetentionPolicy.RUNTIME)
    public @interface isEventBusMethod{
        int type() default -1;
        boolean isMainThread() default false;
    }
    
    /**
     * 设置数据库表名
     * @author zhangzheng
     *
     */
    @Target(ElementType.TYPE)
    @Retention(RetentionPolicy.RUNTIME)
    public @interface setTabName {
        String value();
    }
    
    /**
     * 表示为数据库主键
     * @author zhangzheng
     *
     */
    @Target(ElementType.FIELD)
    @Retention(RetentionPolicy.RUNTIME)
    public @interface isPrimaryKey {
    	
    }
    
    @Target(ElementType.FIELD)
    @Retention(RetentionPolicy.RUNTIME)
    public @interface isNewDbField {
    	int startDbVerson();
    }
    
    /**
     * 设置数据库字段是否被忽略
     * @author zhangzheng
     *
     */
    @Target(ElementType.FIELD)
    @Retention(RetentionPolicy.RUNTIME)
    public @interface isIgnore {
    	
    }
    
    /**
     * 是否进行捕获异常
     * @author zhangzheng
     *
     */
    @Target(ElementType.FIELD)
    @Retention(RetentionPolicy.RUNTIME)
    public @interface isCatchException {
    	//是否在错误退出后，恢复任务栈
    	boolean recovery() default false;
    }
    
    /**
     * 当前网络的状态改变
     * @author zhangzheng
     *
     */
    @Target(ElementType.METHOD)
    @Retention(RetentionPolicy.RUNTIME)
    public @interface NETSTATE{
    	NetState state();
    }

    /**
     * 获得类上注解的实体
     * @param position
     * @param cla 注解类
     * @return
     */
    public static <T extends Annotation> T getAnnotation(Object position, Class<T> cla) {
        T annotation = position.getClass().getAnnotation(cla);
        return annotation;
    }
    
    /**
     * 获得方法上注解的实体
     * @param position
     * @param cla 注解类
     * @return
     */
	public static <T extends Annotation> T getAnnotation(Object position, Class<T> cla, String methodName, Class... params) {
        try {
            Method method = null;
            if (params != null) {
                method = position.getClass().getDeclaredMethod(methodName, params);
            } else {
                method = position.getClass().getMethod(methodName);
            }
            return method.getAnnotation(cla);
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        }
        return null;
    }

	/**
	 * 获得被标示注解的方法
	 * @param position
	 * @param cla 注解类
	 * @return
	 */
    public static Method getAnnotationMethod(Object position, Class cla) {
        Method[] methods = position.getClass().getMethods();
        for (Method method : methods) {
            if (method.getAnnotation(cla) != null) {
                return method;
            }
        }

        return null;
    }
    
    /**
     * 获得被标示注解的方法集合
     * @param position 需要反射的对象
     * @param cla 注解类
     * @return
     */
    public static List<Method> getAnnotationMethods(Object position, Class cla) {
        Method[] methods = position.getClass().getMethods();
        List<Method> saveMethod=new ArrayList<>();
        for (Method method : methods) {
            if (method.getAnnotation(cla) != null) {
            	saveMethod.add(method);
            }
        }

        return saveMethod;
    }
    
    /**
     * 获得注解的方法，根据参数
     * @param position
     * @param cla
     * @param parm
     * @return
     */
    public static Method getAnnotationMethodByParm(Object position, Class cla,Class... parm) {
        Method[] methods = position.getClass().getMethods();
        for (Method method : methods) {
        	if( method.getAnnotation(cla)!=null){
        		Class<?>[] parameterTypes = method.getParameterTypes();
        		List<Class> parmList = Arrays.asList(parm);
        		if(Arrays.asList(parameterTypes).containsAll(parmList)){
        			return method;
        		}
        	}
        }
        return null;
    }
    
    /**
     * 获得被注解的变量字段
     * @param position
     * @param cla
     * @return
     */
    public static Field getFieldHasAnnotation(Object position, Class cla){
    	Field[] declaredFields = position.getClass().getDeclaredFields();
    	for (Field field : declaredFields) {
			if(field.getAnnotation(cla)!=null){
				return field;
			}
		}
    	return null;
    }
    
    /**
     * 获得被注解的变量字段集合
     * @param position
     * @param cla
     * @return
     */
    public static List<Field> getFieldsHasAnnotation(Object position, Class cla){
    	Field[] declaredFields = position.getClass().getDeclaredFields();
    	List<Field> saveFields=new ArrayList<>();
    	for (Field field : declaredFields) {
			if(field.getAnnotation(cla)!=null){
				saveFields.add(field);
			}
		}
    	return saveFields;
    }

}
