package com.focustech.baseandroid.listener;

import android.view.View;
import android.view.View.OnClickListener;

/**
 * 所有点击事件的基类，主要为了解决重复点击的错误
 * 重复点击是在极短的时间内点击同一个按钮两个，但是其中过程也点击了其他按钮不算
 * @author zhangzheng
 * 
 */
public abstract class OnBaseClickListener implements OnClickListener {
	// 其他的点击事件被点击了的标识符
	private static String otherFastDoubleClick = "";

	@Override
	public void onClick(View v) {
		if (isFastDoubleClick(1000)) {
			if (toString().equals(otherFastDoubleClick)) {
				onFastDoubleClick(v);
				return;
			}
		}

		otherFastDoubleClick = toString();

		onViewClick(v);
	}

	private long lastClickTime;

	
	/**
	 * 是否被快速的点击了
	 * @param offsetTime
	 * @return
	 */
	public boolean isFastDoubleClick(int offsetTime) {
		long time = System.currentTimeMillis();
		long timeD = time - lastClickTime;
		if (0 < timeD && timeD < offsetTime) {
			return true;
		}
		lastClickTime = time;
		return false;
	}

	/**
	 * 被点击了按钮
	 * @param v
	 */
	public abstract void onViewClick(View v);

	/**
	 * 快速的被点击两次回调
	 * @param view
	 */
	public void onFastDoubleClick(View view) {

	}
}
