package com.focustech.baseandroid.manager;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;

import android.content.Intent;
import android.os.Bundle;
import android.view.View;

import com.focustech.baseandroid.BaseApp;
import com.focustech.baseandroid.activity.BaseActivity;
import com.focustech.baseandroid.general.BaseAnnotation;
import com.focustech.baseandroid.general.BaseAnnotation.isWebResult;
import com.focustech.baseandroid.net.Response;

/**
 * 控制层用到的重复方法，抽离出来
 * @author zhangzheng
 *
 */
public class ControlManager {
	
	/**
	 * 处理网络请求返回结果
	 * @param obj
	 * @param resp
	 * @param cla
	 */
	public static void onCompute(Object obj,Response resp, Class cla){
		Method method = BaseAnnotation.getAnnotationMethodByParm(obj,
				BaseAnnotation.isWebResult.class, cla);
		if (method == null || !method.isAccessible()) {
			return;
		}

		isWebResult annotation = method
				.getAnnotation(BaseAnnotation.isWebResult.class);

		if (!annotation.receiveNull() && resp == null) {
			return;
		}

		if (!annotation.receiveUnqualified()
				&& (resp == null || !resp.validate())) {
			return;
		}

		try {
			method.invoke(obj, resp);
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 需找控件 自带缓存 第二次直接从缓存中取
	 * @param rootView
	 * @param id
	 * @return
	 */
	public static <T extends View> T fv(View rootView,int id) {
		View view = null;
		HashMap<Integer, View> viewsHashMap = (HashMap) rootView.getTag();
		if (viewsHashMap == null) {
			viewsHashMap = new HashMap<>();
		} else {
			view = viewsHashMap.get(id);
		}

		if (view == null) {
			view = rootView.findViewById(id);
			viewsHashMap.put(id, view);
		}

		rootView.setTag(viewsHashMap);
		return (T) view;
	}
	
	/**
	 * 跳转到下一个Activity页面
	 * @param cla
	 * @param requestCode
	 * @param bundler
	 * @param mActivity
	 */
	public static void turnTo(Class<? extends BaseActivity> cla, Integer requestCode,
			Bundle bundler,BaseActivity mActivity){
		Intent intent = new Intent(mActivity, cla);
		bundler = (bundler == null ? new Bundle() : bundler);
		intent.putExtras(bundler);
		if (requestCode == null) {
			mActivity.startActivity(intent);
		} else {
			mActivity.startActivityForResult(intent, requestCode);
		}
		BaseApp.getAPP().recodeIntent(cla, requestCode, bundler);
	}
	

}
