package com.focustech.baseandroid.manager;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import com.focustech.baseandroid.general.BaseAnnotation;

/**
 * 网络状态的管理类
 * @author zhangzheng
 *
 */
public class NetStateManager {

	private static NetStateManager netManager = new NetStateManager();

	public static NetStateManager getInstance() {
		return netManager;
	}

	public enum NetState {
		RECONNECTION, WIFI, MOBILE, DISCONNECTION ,ALL;
	}

	public static class NetStateField {
		public Object instance;
		public NetState state;
		public Method method;
	}

	private static HashMap<String, List<NetStateField>> methodMap = new HashMap<>();

	public void register(Object obj) {
		Class<?> mClass = obj.getClass();
		Method[] methods = mClass.getMethods();
		for (Method method : methods) {
			BaseAnnotation.NETSTATE annotation = method
					.getAnnotation(BaseAnnotation.NETSTATE.class);
			if (annotation != null) {
				NetStateField mf = new NetStateField();
				mf.method = method;
				mf.instance = obj;
				mf.state = annotation.state();
				List<NetStateField> methodsList = methodMap.get(mClass
						.getName());
				if (methodsList == null) {
					methodsList = new ArrayList<>();
				}
				methodsList.add(mf);
				methodMap.put(mClass.getName(), methodsList);
			}
		}
	}
	
	public void unRegister(Object obj){
		Class<?> mClass = obj.getClass();
        methodMap.remove(mClass.getName());
	}
	
	public void post(NetState state){
		Collection<List<NetStateField>> values = methodMap.values();
		for (List<NetStateField> stateField : values) {
			for (NetStateField netStateField : stateField) {
				if(netStateField.state==state||netStateField.state==NetState.ALL){
					try {
						netStateField.method.invoke(netStateField.instance, state);
					} catch (IllegalAccessException e) {
						e.printStackTrace();
					} catch (IllegalArgumentException e) {
						e.printStackTrace();
					} catch (InvocationTargetException e) {
						e.printStackTrace();
					}
				}
			}
		}
	}
}
