package com.focustech.baseandroid.net;

import android.os.Handler;
import android.os.Looper;
import android.os.Message;

import com.alibaba.fastjson.JSON;
import com.focustech.baseandroid.net.params.Result;
import com.focustech.baseandroid.net.params.Url;
import com.loopj.android.http.AsyncHttpClient;
import com.loopj.android.http.AsyncHttpResponseHandler;

import org.apache.http.Header;

/**
 * 请求发起类
 *
 * @param <T>
 * @author wangxiulong
 */
class Executer<T extends Result> {
    private static final String TAG = "Executer";

    private Class<T> mClass;
    private Callback<T> httpCallback;
    private ExecuteHandler executeHandler;

    public Executer(Class<T> cls, Callback<T> callback) {
        mClass = cls;
        httpCallback = callback;
        executeHandler = new ExecuteHandler(Looper.getMainLooper());
    }

    public void get(Url url) {
        if (url == null) {
            HttpLog.logE(TAG, "url is null.");
            return;
        }

        HttpLog.logD("Request", AsyncHttpClient.getUrlWithQueryString(true, url.getApi(), url.getParams()));

        AsyncHttpClient client = new AsyncHttpClient();
        client.get(url.getApi(), url.getParams(), responseHandler);
    }

    public void post(Url url) {
        if (url == null) {
            HttpLog.logE(TAG, "url is null.");
            return;
        }

        HttpLog.logD("Request", url.getApi() + "?" + url.getParams().toString());

        AsyncHttpClient client = new AsyncHttpClient();
        client.post(url.getApi(), url.getParams(), responseHandler);
    }

    private AsyncHttpResponseHandler responseHandler = new AsyncHttpResponseHandler() {
        @Override
        public void onSuccess(int statusCode, Header[] headers, byte[] responseBody) {
            new Thread(new SubThreadSucessRunnable(statusCode,headers,responseBody)).start();
        }

        @Override
        public void onFailure(int statusCode, Header[] headers, byte[] responseBody, Throwable error) {
            error();
        }
    };

    private class SubThreadSucessRunnable implements Runnable {

        private final int statusCode;
        private final Header[] headers;
        private final byte[] responseBody;

        public SubThreadSucessRunnable(int statusCode, Header[] headers, byte[] responseBody) {

            this.statusCode = statusCode;
            this.headers = headers;
            this.responseBody = responseBody;
        }

        @Override
        public void run() {
            String json = new String(responseBody);
            HttpLog.logD("Response", json);

            T resp = parseResponse(json);
            if (resp == null) {
                try {
                    failure(mClass.newInstance());
                } catch (InstantiationException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            } else {
                if (resp.validate()) {
                    success(resp);
                } else {
                    failure(resp);
                }
            }
        }
    }

    private T parseResponse(String json) {
        try {
            return JSON.parseObject(json, mClass);
        } catch (Exception e) {
            return null;
        }
    }

    private void error() {
        if (executeHandler == null) {
            HttpLog.logE(TAG, "executeHandler is null.");
            return;
        }

        Message msg = Message.obtain(executeHandler, 0);
        msg.sendToTarget();
    }

    private void failure(T resp) {
        if (executeHandler == null) {
            HttpLog.logE(TAG, "executeHandler is null.");
            return;
        }

        Message msg = Message.obtain(executeHandler, 1, resp);
        msg.sendToTarget();
    }

    private void success(T resp) {
        if (executeHandler == null) {
            HttpLog.logE(TAG, "executeHandler is null.");
            return;
        }

        Message msg = Message.obtain(executeHandler, 2, resp);
        msg.sendToTarget();
    }

    private void clear() {
        mClass = null;
        httpCallback = null;
        executeHandler = null;
        responseHandler = null;
    }

    private class ExecuteHandler extends Handler {
        public ExecuteHandler(Looper looper) {
            super(looper);
        }

        @SuppressWarnings("unchecked")
        @Override
        public void handleMessage(Message msg) {
            if (httpCallback == null) {
                HttpLog.logE(TAG, "httpCallback is null.");
                return;
            }

            switch (msg.what) {
                case 0: {
                    httpCallback.onError();
                    httpCallback.onComplete();
                    clear();
                    break;
                }
                case 1: {
                    T resp = (T) msg.obj;
                    httpCallback.onFailure(resp);
                    httpCallback.onComplete();
                    clear();
                    break;
                }
                case 2: {
                    T resp = (T) msg.obj;
                    httpCallback.onSuccess(resp);
                    httpCallback.onComplete();
                    clear();
                    break;
                }
            }
        }
    }
}