package com.focustech.baseandroid.net.params;



import android.os.Handler;
import android.os.Looper;

import com.alibaba.fastjson.JSON;
import com.focustech.baseandroid.activity.BaseActivity;
import com.focustech.baseandroid.net.Response;
import com.focustech.baseandroid.net.WHttp;
import com.focustech.baseandroid.util.BasePropertiesUtil;
import com.loopj.android.http.RequestParams;

/**
 * 请求Url
 * 
 * @author zhangzheng
 * 
 */
@SuppressWarnings("rawtypes")
public abstract class Url {
	/**
	 * 获取此次请求的Api地址
	 * 
	 * @return
	 */
	public abstract String getApi();

	/**
	 * 获取请求参数
	 * 
	 * @return
	 */
	public abstract RequestParams getParams();

	public onRequestListener<?> onRequestListener;
	private BaseActivity ac;
	private boolean isShowProgress = true;

	public void setOnRequestListener(onRequestListener<?> onRequestListener) {
		this.onRequestListener = onRequestListener;
	}

	public interface onRequestListener<T extends Response> {
		/** 网络请求成功 并返回结果 */
		public void onCompute(Response resp,Class cla);
	}

	public void setProgressShow(BaseActivity ac) {
		this.ac = ac;
	}

	@SuppressWarnings("unchecked")
	public void onRequest(Class cla) {
		if (ac != null && isShowProgress) {
			new Handler().postDelayed(new Runnable() {
				@Override
				public void run() {
					ac.showProgressDialog();
				}
			},50);
		}

		new MWhttp<Response>(cla).get(this);
	}

	public void onRequest(Class<? extends Response> cla, onRequestListener lis) {
		this.onRequestListener = lis;
		onRequest(cla);
	}

	/**
	 * 带进度条
	 * 
	 * @param ac
	 *            BaseActivity 用来显示进度条
	 * @param cla
	 *            返回类的类型
	 * @param lis
	 *            请求成功后的监听
	 */
	public void onRequest(BaseActivity ac, Class<? extends Response> cla,
			onRequestListener lis) {
		this.onRequestListener = lis;
		this.ac = ac;
		onRequest(cla);
	}

	@SuppressWarnings("unchecked")
	private class MWhttp<T extends Response> extends WHttp<T> {

		private final Class<T> cla;

		public MWhttp(Class<T> cla) {
			this.cla = cla;
		}

		@Override
		public Class<T> getType() {
			return cla;

		}

		@Override
		public void onError() {
			if (BasePropertiesUtil.hasKey(BasePropertiesUtil.NET_TEST,
					cla.getSimpleName())) {
				String json = BasePropertiesUtil.getValue(BasePropertiesUtil.NET_TEST,
						cla.getSimpleName());

				Response bean = JSON.parseObject(json, cla);
				onSuccess((T) bean);
				return;
			} else {
				Url.this.onError();
			}

			if (onRequestListener != null) {
				onRequestListener.onCompute(null,cla);
			}
		}

		@Override
		public void onFailure(T resp) {

			if (BasePropertiesUtil.hasKey(BasePropertiesUtil.NET_TEST,
					cla.getSimpleName())) {
				String json = BasePropertiesUtil.getValue(BasePropertiesUtil.NET_TEST,
						cla.getSimpleName());

				Response bean = JSON.parseObject(json, cla);
				onSuccess((T) bean);
				return;
			} else {
				Url.this.onFailure(resp);
			}

			if (onRequestListener != null) {
				onRequestListener.onCompute(resp,cla);
			}

		}

		@Override
		public void onSuccess(T resp) {
			if (BasePropertiesUtil.hasKey(BasePropertiesUtil.NET_TEST,
					cla.getSimpleName())) {
				String json = BasePropertiesUtil.getValue(BasePropertiesUtil.NET_TEST,
						cla.getSimpleName());

				resp = JSON.parseObject(json, cla);
			}
			boolean sucess = Url.this.onSuccess(resp);

			if (onRequestListener != null && sucess) {
				onRequestListener.onCompute(resp,cla);
			}
		}

		@Override
		public void onComplete() {
			Url.this.onComplete();
		}

	}

	protected boolean onSuccess(Response resp) {
		return true;
	}

	protected void onError() {
//		showToast(R.string.toast_common_net_error);
	}

	protected void onFailure(Response resp) {
		if (resp != null&&!resp.getMsg().equals("")) {
			showToast(resp.getMsg());
		}
	}

	protected void onComplete() {
		dismissProgress();
	}

	protected void showToast(String content) {
		if (ac != null)
			ac.showToast(content);
	}

	protected void showToast(int resourceId) {
		if (ac != null)
			ac.showToast(resourceId);
	}

	protected void showProgress() {
		if (ac != null)
			ac.showProgressDialog();
	}

	protected void dismissProgress() {
		if (ac != null) {
			new Handler(Looper.myLooper()).post(new Runnable() {
				@Override
				public void run() {
					ac.dismissProgressDialog();
				}
			});
		}
	}

	public void setShowProgress(boolean isShowProgress) {
		this.isShowProgress = isShowProgress;
	}
}