package com.focustech.baseandroid.util;

import android.content.Context;
import android.text.ClipboardManager;
import android.text.Html;
import android.text.Spanned;
import android.util.Base64;


import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Locale;
import java.util.regex.Pattern;


/**
 * Created by jinjianxin on 15/6/1.
 */
public class BaseStringUtils {

	/**
	 * 判断是否为中文字符
	 * @param c
	 * @return
	 */
    public static boolean isChinese(char c) {
        Character.UnicodeBlock ub = Character.UnicodeBlock.of(c);
        if (ub == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS
                || ub == Character.UnicodeBlock.CJK_COMPATIBILITY_IDEOGRAPHS
                || ub == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_A
                || ub == Character.UnicodeBlock.GENERAL_PUNCTUATION
                || ub == Character.UnicodeBlock.CJK_SYMBOLS_AND_PUNCTUATION
                || ub == Character.UnicodeBlock.HALFWIDTH_AND_FULLWIDTH_FORMS) {
            return true;
        }
        return false;
    }

    /**
     * 判断字符串是否为空  Null和“” 都属于空
     *
     * @param judgeContent
     * @return
     */
    public static boolean isNull(String judgeContent) {
        if (judgeContent == null || judgeContent.trim().equals("")) {
            return true;
        }
        return false;
    }

    /**
     * 将NULL替换成“”
     * @param value
     * @return
     */
    public static String parseNotNull(String value){
        if(isNull(value)){
            return "";
        }
        return value;
    }

    /**
     * 去除前后的空字符串
     * @param str
     * @return
     */
    public static String removeStringSpace(String str) {
        str = str.trim();
        while (str.startsWith(" ")) {
            str = str.substring(1, str.length()).trim();
        }
        while (str.endsWith(" ")) {
            str = str.substring(0, str.length() - 1).trim();
        }
        return str;
    }

    /**
     * 检测字符串的长度 是否越界
     * @param str
     * @param len
     * @return
     */
    public static boolean checkStringLen(String str, int len) {
        if (str==null||str.trim().equals("")) {
            return len == 0;
        }
        if (str.length() <= len) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 分割字符串，并返回指定索引的字符
     *
     * @param source 字符串
     * @param split  分割符
     * @param index  索引
     * @return
     */
    public static String split(String source, String split, int index) {
        if (source==null || split==null) {
            return null;
        }

        String[] array = source.split(split);
        if (index >= 0 && index < array.length) {
            return array[index];
        } else {
            return null;
        }
    }


    /**
     * base64加密
     *
     * @param str
     * @return
     */
    public static String encodeBase64(String str) {
        try {
            byte[] b = Base64.encode(str.getBytes("UTF-8"), Base64.DEFAULT);
            return new String(b, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();

            return str;
        }
    }

    /**
     * base64解密
     *
     * @param str
     * @return
     */
    public static String decodeBase64(String str) {
        try {
            byte[] b = Base64.decode(str.getBytes("UTF-8"), Base64.DEFAULT);
            return new String(b, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();

            return str;
        }
    }

    /**
     * 过滤字符串中的html标签
     *
     * @param htmlString
     * @return
     */
    public static String filterHtmlCode(String htmlString) {
        if (htmlString == null) {
            return "";
        } else {
            htmlString = htmlString.replaceAll("</?[^<>]*>", "");
            htmlString = htmlString.replaceAll("&#?[A-Za-z0-9]+;", "");
            return htmlString;
        }
    }

    /**
     * 获得网页效果字
     *
     * @param str
     * @return
     */
    public static Spanned getHtml(String str) {
        return Html.fromHtml("<u>" + str + "</u>");
    }

    /**
     * md5编码
     *
     * @param inVal
     * @return
     */
    public static String md5(String inVal) {
        MessageDigest digested;
        try {
            digested = MessageDigest.getInstance("MD5");
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
            return "";
        }
        byte[] bytes = inVal.getBytes();
        digested.update(bytes);
        byte[] digest = digested.digest();

        return toHexString(digest);
    }

    /**
     * 十六进制大写
     *
     * @param src
     * @return
     */
    private static String toHexString(byte[] src) {
        StringBuilder hexString = new StringBuilder("");
        if (src == null || src.length <= 0) {
            return null;
        }
        for (int i = 0; i < src.length; i++) {
            String h = Integer.toHexString(0xFF & src[i]);
            while (h.length() < 2)
                h = "0" + h;
            hexString.append(h);
        }
        return hexString.toString().toUpperCase(Locale.getDefault());
    }

    public static byte[] hexStringToBytes(String src) {
        if (src == null || src.equals("")) {
            return null;
        }
        src = src.toUpperCase(Locale.getDefault());
        int length = src.length() / 2;
        char[] hexChars = src.toCharArray();
        byte[] d = new byte[length];
        for (int i = 0; i < length; i++) {
            int pos = i * 2;
            d[i] = (byte) (charToByte(hexChars[pos]) << 4 | charToByte(hexChars[pos + 1]));
        }
        return d;
    }

    private static byte charToByte(char c) {
        return (byte) "0123456789ABCDEF".indexOf(c);
    }

    
    /**
     * 解析输入流变成字符串
     * @param in
     * @return
     */
    public static String inputStream2String(InputStream in) {
        try {
            StringBuffer out = new StringBuffer();
            byte[] b = new byte[4096];
            for (int n; (n = in.read(b)) != -1; ) {
                out.append(new String(b, 0, n));
            }
            return out.toString();
        } catch (Exception e) {
        }

        return "";
    }

    /**
     * 获取字符个数
     *
     * @param strName
     * @return
     */
    public static int getCharSize(String strName) {
        int strSize = 0;
        char[] ch = strName.toCharArray();
        for (int i = 0; i < ch.length; i++) {
            char c = ch[i];
            if (isChinese(c) == true) {
                strSize = strSize + 2;
            } else {
                strSize = strSize + 1;
            }
        }
        return strSize;
    }





    /**
     * 检查字符串的字符是否都相等
     *
     * @param numOrStr
     * @return
     */
    public static boolean equalStr(String numOrStr) {
        boolean flag = true;
        char str = numOrStr.charAt(0);
        for (int i = 0; i < numOrStr.length(); i++) {
            if (str != numOrStr.charAt(i)) {
                flag = false;
                break;
            }
        }
        return flag;
    }

    /**
     * 检查是否是连续的数字
     *
     * @param str
     * @return
     */
    public static boolean isOrderNumbers(String str) {
        final String up = "01234567890123456789";
        final String down = "98765432109876543210";
        if (up.contains(str) || down.contains(str)) {
            return true;
        } else {
            return false;
        }

    }

    /**
     * 检查连续字符
     *
     * @param str
     * @return
     */
    public static boolean isOrderWords(String str) {
        final String aa = "abcdefghijklmnopqrstuvwxyz";
        final String aA = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
        final String aa_ = "zyxwvutsrqponmlkjihgfedcba";
        final String aA_ = "ZYXWVUTSRQPONMLKJIHGFEDCBA";
        if (aa.contains(str) || aA.contains(str) || aa_.contains(str) || aA_.contains(str)) {
            return true;
        } else {
            return false;
        }
    }

    public static boolean checkAllNum(String value) {
        String regexN = "[0-9]{1,}";
        return Pattern.matches(regexN, value);
    }

    /**
     * 实现文本复制功能
     * add by wangqianzhou
     *
     * @param content
     */
    public static void copyToClipboard(String content, Context context) {
        // 得到剪贴板管理器
        ClipboardManager cmb = (ClipboardManager) context.getSystemService(Context.CLIPBOARD_SERVICE);
        cmb.setText(content.trim());
    }

    /**
     * 实现粘贴功能
     * add by wangqianzhou
     *
     * @param context
     * @return
     */
    public static String pasteFromClipboard(Context context) {
        // 得到剪贴板管理器
        ClipboardManager cmb = (ClipboardManager) context.getSystemService(Context.CLIPBOARD_SERVICE);
        return cmb.getText().toString().trim();
    }

}
