package com.focustech.baseandroid.util;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.regex.Pattern;

import com.focustech.baseandroid.BaseApp;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.util.Log;
import android.util.SparseArray;
import android.provider.Settings.Secure;

;

public class BaseSysUtil {

	public static boolean isEmpty(Object obj) {
		return obj == null;
	}

	public static boolean isEmpty(String str) {
		return str == null || "".equals(str);
	}

	public static boolean isEmpty(List<?> list) {
		return list == null || list.size() == 0;
	}

	public static boolean isEmpty(SparseArray<?> array) {
		return array == null || array.size() == 0;
	}

	/**
	 * 获取应用版本号
	 * 
	 * @param context
	 * @return 版本号
	 */
	public static String getAppVersion(Context context) {
		String versionName = "";
		try {
			// ---get the package info---
			PackageManager pm = context.getPackageManager();
			PackageInfo pi = pm.getPackageInfo(context.getPackageName(), 0);
			versionName = pi.versionName;
		} catch (Exception e) {
			Log.e("VersionInfo", "Exception", e);
		}
		return versionName;
	}

	/**
	 * 网络是否可用
	 * 
	 * @param context
	 * @return
	 */
	public static boolean isConnect(Context context) {
		ConnectivityManager connectivity = (ConnectivityManager) context
				.getSystemService(Context.CONNECTIVITY_SERVICE);
		if (connectivity == null) {
			return false;
		} else {
			NetworkInfo[] info = connectivity.getAllNetworkInfo();
			if (info != null) {
				for (int i = 0; i < info.length; i++) {
					if (info[i].isAvailable()) {
						return true;
					}
				}
			}
		}
		return false;
	}

	public static boolean isWifi(Context context) {
		ConnectivityManager connectivityManager = (ConnectivityManager) context
				.getSystemService(Context.CONNECTIVITY_SERVICE);
		NetworkInfo activeNetInfo = connectivityManager.getActiveNetworkInfo();
		if (activeNetInfo != null
				&& activeNetInfo.getType() == ConnectivityManager.TYPE_WIFI) {
			return true;
		}
		return false;
	}

	public static boolean is3G(Context context) {
		ConnectivityManager connectivityManager = (ConnectivityManager) context
				.getSystemService(Context.CONNECTIVITY_SERVICE);
		NetworkInfo activeNetInfo = connectivityManager.getActiveNetworkInfo();
		if (activeNetInfo != null
				&& activeNetInfo.getType() == ConnectivityManager.TYPE_MOBILE) {
			return true;
		}
		return false;
	}

	public static boolean validateIDCard(String value) {
		if (isEmpty(value)) {
			return false;
		}
		String[] area = { "北京", "11", "天津", "12", "河北", "13", "山西",
				"14",
				"内蒙古",
				"15",// 华北
				"辽宁", "21", "吉林", "22",
				"黑龙江",
				"23",// 东北
				"上海", "31", "江苏", "32", "浙江", "33", "安徽", "34", "福建", "35",
				"江西", "36", "山东",
				"37",// 华东
				"河南", "41", "湖北", "42", "湖南", "43", "广东", "44", "广西", "45",
				"海南", "46", "重庆",// 中南
				"50", "四川", "51", "贵州", "52", "云南", "53", "西藏", "54",// 西南
				"陕西", "61", "甘肃", "62", "青海", "63", "宁夏", "64", "新疆", "65",// 西北
				"台湾", "71", "香港", "81", "澳门", "82", "国外", "91" };

		if (value.length() != 15 && value.length() != 18) {
			return false;
		}
		String province = value.substring(0, 2);
		int provinceCount = area.length / 2;
		boolean isProvince = false;
		for (int i = 0; i < provinceCount; i++) {
			if (area[i * 2 + 1].equals(province)) {
				isProvince = true;
				break;
			}
		}
		if (!isProvince) {
			return false;
		}
		String ereg = null;
		if (value.length() == 15) {
			ereg = "^\\d{15}";
			if (!Pattern.matches(ereg, value)) {
				return false;
			}
			Integer year = 1900 + Integer.valueOf(value.substring(6, 8));
			if ((year % 4 == 0) || ((year % 100 == 0) && (year % 4 == 0))) {
				ereg = "^[1-9][0-9]{5}[0-9]{2}((01|03|05|07|08|10|12)(0[1-9]|[1-2][0-9]|3[0-1])|(04|06|09|11)(0[1-9]|[1-2][0-9]|30)|02(0[1-9]|[1-2][0-9]))[0-9]{3}$";
			} else {
				ereg = "^[1-9][0-9]{5}[0-9]{2}((01|03|05|07|08|10|12)(0[1-9]|[1-2][0-9]|3[0-1])|(04|06|09|11)(0[1-9]|[1-2][0-9]|30)|02(0[1-9]|1[0-9]|2[0-8]))[0-9]{3}$";
			}

			if (Pattern.matches(ereg, value)) {
				return true;
			} else {
				return false;
			}
		} else if (value.length() == 18) {
			ereg = "^\\d{17}[0-9xX]{1}$";
			if (!Pattern.matches(ereg, value)) {
				return false;
			}
			String birthday = value.substring(6, 14);
			SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
			Date date = null;
			try {
				date = sdf.parse(birthday);
			} catch (ParseException e) {
				e.printStackTrace();
			}

			String[] idCards = value.substring(1, 18).split("");
			if (date != null) {
				Integer S = (Integer.valueOf(value.substring(0, 1)) + Integer
						.valueOf(idCards[10]))
						* 7
						+ (Integer.valueOf(idCards[1]) + Integer
								.valueOf(idCards[11]))
						* 9
						+ (Integer.valueOf(idCards[2]) + Integer
								.valueOf(idCards[12]))
						* 10
						+ (Integer.valueOf(idCards[3]) + Integer
								.valueOf(idCards[13]))
						* 5
						+ (Integer.valueOf(idCards[4]) + Integer
								.valueOf(idCards[14]))
						* 8
						+ (Integer.valueOf(idCards[5]) + Integer
								.valueOf(idCards[15]))
						* 4
						+ (Integer.valueOf(idCards[6]) + Integer
								.valueOf(idCards[16]))
						* 2
						+ Integer.valueOf(idCards[7])
						* 1
						+ Integer.valueOf(idCards[8])
						* 6
						+ Integer.valueOf(idCards[9]) * 3;
				Integer Y = S % 11;
				String JYM = "10X98765432";
				String M = JYM.substring(Y, Y + 1);
				if (M.equalsIgnoreCase(idCards[17])) {
					return true;
				} else {
					return false;
				}
			} else {
				return false;
			}
		} else {
			return false;
		}
	}

	/**
	 * 检查手机号的有效性
	 * 
	 * @param mobile
	 * @return
	 */
	public static boolean validateMobile(String mobile) {
		if (isEmpty(mobile)) {
			return false;
		}

		return mobile.matches("^1\\d{10}$");
	}

	/**
	 * 检查邮箱的有效性
	 * 
	 * @param email
	 * @return
	 */
	public static boolean validateEmail(String email) {
		if (isEmpty(email)) {
			return false;
		}

		String str = "^([a-zA-Z0-9_\\-\\.]+)@((\\[[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.)|(([a-zA-Z0-9\\-]+\\.)+))([a-zA-Z]{2,4}|[0-9]{1,3})(\\]?)$";
		return email.matches(str);
	}

	/**
	 * 检查是否是英文名
	 * 
	 * @param engName
	 * @return
	 */
	public static boolean isValidEngName(String engName) {
		engName = engName.trim();
		String regex = "^[a-zA-Z0-9-_.'\\s]+$";
		return Pattern.matches(regex, engName);
	}

	/**
	 * 获得唯一的机器码
	 * 
	 * @return
	 */
	public static String getAndroidID() {
		String androidID = Secure.getString(BaseApp.getAPP().getContentResolver(), Secure.ANDROID_ID);
		return androidID == null ? "" : androidID;
	}
}
