package com.focustech.baseandroid.util;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffXfermode;
import android.graphics.Rect;
import android.graphics.RectF;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.os.Handler;
import android.os.Looper;
import android.text.SpannableString;
import android.text.Spanned;
import android.text.style.StrikethroughSpan;
import android.util.DisplayMetrics;
import android.util.Log;
import android.util.SparseArray;
import android.util.TypedValue;
import android.view.Display;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewTreeObserver;
import android.view.WindowManager;
import android.view.animation.DecelerateInterpolator;
import android.view.animation.TranslateAnimation;
import android.view.inputmethod.InputMethodManager;
import android.widget.Adapter;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ListAdapter;
import android.widget.ListView;
import android.widget.Toast;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import com.focustech.baseandroid.activity.BaseActivity;

/**
 * Created by zhangzheng
 */
public class BaseViewUtils {

	private static int width;
	private static int height;

	public static void init(Context context) {
		Resources res = context.getResources();
		DisplayMetrics metrics = res.getDisplayMetrics();
		width = metrics.widthPixels;
		height = metrics.heightPixels;
	}

	public static int getScreenWidth(Context context) {
		if (width == 0) {
			init(context);
		}
		return width;
	}

	public static int getScreenHeight(Context context) {
		if (height == 0) {
			init(context);
		}
		return height;
	}

	public static Bitmap getRoundedCornerBitmap(Bitmap bitmap, int pixels) {
		Bitmap output = Bitmap.createBitmap(bitmap.getWidth(),
				bitmap.getHeight(), Bitmap.Config.ARGB_8888);
		Canvas canvas = new Canvas(output);

		final int color = 0xff424242;
		final Paint paint = new Paint();
		final Rect rect = new Rect(0, 0, bitmap.getWidth(), bitmap.getHeight());
		final RectF rectF = new RectF(rect);
		final float roundPx = pixels;

		paint.setAntiAlias(true);
		canvas.drawARGB(0, 0, 0, 0);
		paint.setColor(color);
		canvas.drawRoundRect(rectF, roundPx, roundPx, paint);

		paint.setXfermode(new PorterDuffXfermode(PorterDuff.Mode.SRC_IN));
		canvas.drawBitmap(bitmap, rect, rect, paint);

		bitmap.recycle();

		return output;
	}

	public static Bitmap getBitmapByPix(Bitmap bm, Context context) {

		int width = bm.getWidth();
		int height = bm.getHeight();
		Log.e("bm.getWidth()", bm.getWidth() + "");
		int newWidth = dip2px(context, 66);
		int newHeight = dip2px(context, 66);

		float scaleWidth = ((float) newWidth) / width;
		float scaleHeight = ((float) newHeight) / height;

		Matrix matrix = new Matrix();
		matrix.postScale(scaleWidth, scaleHeight);

		Bitmap bmp = Bitmap.createBitmap(bm, 0, 0, width, height, matrix, true);

		bm.recycle();
		return bmp;
	}

	/**
	 * change dip value to pixel value for certain screen size
	 * 
	 * @param dipValue
	 * @param context
	 * @return
	 */
	public static int getPixels(int dipValue, Context context) {
		int result = 0;
		Resources res = context.getResources();
		result = (int) TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP,
				dipValue, res.getDisplayMetrics());
		return result;
	}

	/**
	 * change View to Bitmap Object
	 * 
	 * @param v
	 * @return
	 */
	public static Bitmap loadBitmapFromView(View v) {
		if (v == null) {
			return null;
		}
		Bitmap screenshot;
		screenshot = Bitmap.createBitmap(v.getWidth(), v.getHeight(),
				Bitmap.Config.ARGB_8888);
		Canvas c = new Canvas(screenshot);
		c.translate(-v.getScrollX(), -v.getScrollY());
		v.draw(c);
		return screenshot;
	}

	/**
	 * compute sample size you want
	 * 
	 * @param options
	 * @param minSideLength
	 * @param maxNumOfPixels
	 * @return
	 */
	public static int computeSampleSize(BitmapFactory.Options options,
			int minSideLength, int maxNumOfPixels) {
		int initialSize = computeInitialSampleSize(options, minSideLength,
				maxNumOfPixels);
		int roundedSize;
		if (initialSize <= 8) {
			roundedSize = 1;
			while (roundedSize < initialSize) {
				roundedSize <<= 1;
			}
		} else {
			roundedSize = (initialSize + 7) / 8 * 8;
		}
		return roundedSize;
	}

	public static Drawable getImage(Context context, int id) {
		return context.getResources().getDrawable(id);
	}

	/**
	 * compute bitmap initial sample size
	 * 
	 * @param options
	 * @param minSideLength
	 * @param maxNumOfPixels
	 * @return
	 */
	private static int computeInitialSampleSize(BitmapFactory.Options options,
			int minSideLength, int maxNumOfPixels) {
		double w = options.outWidth;
		double h = options.outHeight;
		int lowerBound = (maxNumOfPixels == -1) ? 1 : (int) Math.ceil(Math
				.sqrt(w * h / maxNumOfPixels));
		int upperBound = (minSideLength == -1) ? 128 : (int) Math.min(
				Math.floor(w / minSideLength), Math.floor(h / minSideLength));
		if (upperBound < lowerBound) {
			// return the larger one when there is no overlapping zone.
			return lowerBound;
		}
		if ((maxNumOfPixels == -1) && (minSideLength == -1)) {
			return 1;
		} else if (minSideLength == -1) {
			return lowerBound;
		} else {
			return upperBound;
		}
	}

	/**
	 * 获得状态栏的高度
	 * 
	 * @param context
	 * @return
	 */
	public static int getStatusBarHeight(Activity context) {
		Rect rect = new Rect();
		context.getWindow().getDecorView().getWindowVisibleDisplayFrame(rect);
		return rect.top;
	}

	public static void showDialog(Activity context, String strTitle,
			String strText, int icon) {
		if (!context.isFinishing()) {
			AlertDialog.Builder tDialog = new AlertDialog.Builder(context);
			tDialog.setIcon(icon);
			tDialog.setTitle(strTitle);
			tDialog.setMessage(strText);
			tDialog.setPositiveButton(android.R.string.ok, null);
			tDialog.show();
		}
	}

	public static Dialog createCancelDialog(Context mContext, String title,
			DialogInterface.OnClickListener click, boolean isNeedCancelButton) {
		AlertDialog.Builder builder = new AlertDialog.Builder(mContext)
				.setMessage(title).setCancelable(false)
				.setPositiveButton("确认", click);
		if (isNeedCancelButton) {
			builder.setNegativeButton("取消", null);
		}
		return builder.create();
	}

	public static Dialog createMessageDialog(Context context, int rid) {
		AlertDialog.Builder builder = new AlertDialog.Builder(context);
		builder.setMessage(rid);
		builder.setCancelable(false);
		builder.setPositiveButton("确认", null);
		return builder.create();
	}

	public static int dip2px(Context context, float dipValue) {
		final float scale = context.getResources().getDisplayMetrics().density;
		return (int) (dipValue * scale + 0.5f);
	}

	public static int px2dip(Context context, float pxValue) {
		final float scale = context.getResources().getDisplayMetrics().density;
		return (int) (pxValue / scale + 0.5f);
	}

	/**
	 * 将px值转换为sp值，保证文字大小不变
	 * 
	 * @param pxValue
	 * @param context
	 * @return
	 */
	public static int px2sp(Context context, float pxValue) {
		float fontScale = context.getResources().getDisplayMetrics().scaledDensity;
		return (int) (pxValue / fontScale + 0.5f);
	}

	/**
	 * 将sp值转换为px值，保证文字大小不变
	 * 
	 * @param spValue
	 * @param context
	 *            （
	 * @return
	 */
	public static int sp2px(Context context, float spValue) {
		float fontScale = context.getResources().getDisplayMetrics().scaledDensity;
		return (int) (spValue * fontScale + 0.5f);
	}

	public static void lockCancelable(DialogInterface dialog) {
		try {
			Field field = dialog.getClass().getSuperclass()
					.getDeclaredField("mShowing");
			field.setAccessible(true);
			field.set(dialog, false);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void unlockCancelable(DialogInterface dialog) {
		try {
			Field field = dialog.getClass().getSuperclass()
					.getDeclaredField("mShowing");
			field.setAccessible(true);
			field.set(dialog, true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * 设置View的宽高为0 另类的隐藏控件方法
	 */
	public static void setViewSizeToZero(View view) {
		ViewGroup.LayoutParams lp = view.getLayoutParams();
		lp.width = 0;
		lp.height = 0;
		view.setLayoutParams(lp);
	}

	public static void setViewSize(View view, int width, int height) {
		ViewGroup.LayoutParams lp = view.getLayoutParams();
		lp.width = width;
		lp.height = height;
		view.setLayoutParams(lp);
	}

	public static Bitmap getScreenWidthBitMap(Activity ac, Bitmap preBitmap,
			float memoryRatio, boolean isOptimization) {
		DisplayMetrics dm = new DisplayMetrics();
		ac.getWindowManager().getDefaultDisplay().getMetrics(dm);
		int sw = dm.widthPixels;
		float scale = (float) sw / (float) preBitmap.getWidth();
		if (scale > 1 && isOptimization) {
			scale = 1;
			sw = preBitmap.getWidth();
		}
		int imh = (int) ((float) preBitmap.getHeight() * scale);

		Bitmap img = Bitmap.createScaledBitmap(preBitmap,
				(int) (sw * memoryRatio), (int) (imh * memoryRatio), true);
		if (img != preBitmap) {
			preBitmap.recycle();
		}

		return img;
	}

	public static Bitmap getScreenWidthBitMap(Activity ac, Bitmap preBitmap) {
		return getScreenWidthBitMap(ac, preBitmap, 1, false);
	}

	public static SpannableString getSpanText(String str) {
		SpannableString ss = new SpannableString(str);
		ss.setSpan(new StrikethroughSpan(), 0, ss.length(),
				Spanned.SPAN_EXCLUSIVE_EXCLUSIVE);
		return ss;
	}

	public static <T extends View> T get(View view, int id) {
		SparseArray<View> viewHolder = ((SparseArray<View>) view.getTag());
		if (viewHolder == null) {
			viewHolder = new SparseArray<View>();
			view.setTag(viewHolder);
		}

		View childView = viewHolder.get(id);
		if (childView == null) {
			childView = view.findViewById(id);
			viewHolder.put(id, childView);
		}

		return (T) childView;
	}

	public static void controlFullScreen(Activity activity, boolean isFullScreen) {
		if (isFullScreen) {
			activity.getWindow().setFlags(
					WindowManager.LayoutParams.FLAG_FULLSCREEN,
					WindowManager.LayoutParams.FLAG_FULLSCREEN);
			activity.getWindow().clearFlags(
					WindowManager.LayoutParams.FLAG_FORCE_NOT_FULLSCREEN);
		} else {
			activity.getWindow().setFlags(
					WindowManager.LayoutParams.FLAG_FORCE_NOT_FULLSCREEN,
					WindowManager.LayoutParams.FLAG_FORCE_NOT_FULLSCREEN);
			activity.getWindow().clearFlags(
					WindowManager.LayoutParams.FLAG_FULLSCREEN);
		}
	}

	public static class ListViewUtil {

		public static boolean isLastItemVisible(ListView listView) {
			final Adapter adapter = listView.getAdapter();

			if (null == adapter || adapter.isEmpty()) {
				return true;
			}

			final int lastItemPosition = adapter.getCount() - 1;
			final int lastVisiblePosition = listView.getLastVisiblePosition();
			if (lastVisiblePosition >= lastItemPosition - 1) {
				final int childIndex = lastVisiblePosition
						- listView.getFirstVisiblePosition();
				final int childCount = listView.getChildCount();
				final int index = Math.min(childIndex, childCount - 1);
				final View lastVisibleChild = listView.getChildAt(index);
				if (lastVisibleChild != null) {
					return lastVisibleChild.getBottom() <= listView.getBottom();
				}
			}
			return false;
		}

		public static boolean isFirstItemVisible(ListView listView) {
			final Adapter adapter = listView.getAdapter();
			if (null == adapter || adapter.isEmpty()) {
				return true;
			}
			final int firstVisiblePosition = listView.getFirstVisiblePosition();
			if (firstVisiblePosition == 0) {
				final View firstVisibleChild = listView.getChildAt(0);
				if (firstVisibleChild != null) {
					return firstVisibleChild.getTop() >= listView.getTop();
				}
			}
			return false;
		}

		/**
		 * 动态设置ListView的高度
		 * 
		 * @param listView
		 */
		public static int setListViewHeightBasedOnChildren(ListView listView) {
			if (listView == null)
				return 0;
			ListAdapter listAdapter = listView.getAdapter();
			if (listAdapter == null) {
				// pre-condition
				return 0;
			}
			int totalHeight = 0;
			for (int i = 0; i < listAdapter.getCount(); i++) {
				View listItem = listAdapter.getView(i, null, listView);
				listItem.measure(0, 0);
				totalHeight += listItem.getMeasuredHeight();
			}
			ViewGroup.LayoutParams params = listView.getLayoutParams();
			params.height = totalHeight
					+ (listView.getDividerHeight() * (listAdapter.getCount() - 1));
			listView.setLayoutParams(params);
			return params.height;
		}

		public static int getOnlyListViewHeight(ListView listView) {
			if (listView == null)
				return 0;
			ListAdapter listAdapter = listView.getAdapter();
			if (listAdapter == null) {
				// pre-condition
				return 0;
			}
			int totalHeight = 0;
			for (int i = 0; i < listAdapter.getCount(); i++) {
				View listItem = listAdapter.getView(i, null, listView);
				listItem.measure(0, 0);
				totalHeight += listItem.getMeasuredHeight();
			}
			return totalHeight
					+ (listView.getDividerHeight() * (listAdapter.getCount() - 1));
		}

	}

	public static class AnimUtil {
		public static void offsetTranslateAnim(ViewGroup content,
				int offsetTime, int rangeDp) {
			List<View> views = new ArrayList<View>();
			for (int i = 0; i < content.getChildCount(); i++) {
				views.add(content.getChildAt(i));
			}
			offsetTranslateAnim(views.toArray(new View[views.size()]),
					offsetTime, rangeDp);
		}

		public static void offsetTranslateAnim(ViewGroup content) {
			offsetTranslateAnim(content, 100, 200);
		}

		public static void offsetTranslateAnim(View[] views, int offsetTime,
				int rangeDp) {
			int position = 0;
			for (int i = 0; i < views.length; i++) {
				View view = views[i];
				if (view.getClass() == View.class) {
					continue;
				}
				TranslateAnimation anim = new TranslateAnimation(dip2px(
						view.getContext(), rangeDp), 0, 0, 0);
				anim.setDuration(offsetTime + position * 100);
				view.startAnimation(anim);
				position++;
			}
		}

		public static void offsetTranslateAnim(View... views) {
			offsetTranslateAnim(views, 100, 200);
		}

	}

	public static void hideSoftInput(final Activity context) {
		new Handler(Looper.getMainLooper()).post(new Runnable() {
			@Override
			public void run() {
				InputMethodManager imm = (InputMethodManager) context
						.getSystemService(Context.INPUT_METHOD_SERVICE);
				View view = context.getCurrentFocus();
				if (view != null) {
					view.clearFocus();
					imm.hideSoftInputFromWindow(view.getWindowToken(), 0);
				}
			}
		});
	}

	public static void hideSoftInput(final Dialog dialog) {
		InputMethodManager imm = (InputMethodManager) dialog.getContext()
				.getSystemService(Context.INPUT_METHOD_SERVICE);
		imm.hideSoftInputFromWindow(dialog.getCurrentFocus().getWindowToken(),
				0);
	}

	/**
	 * 获得当前Activity的屏幕截图
	 * 
	 * @param activity
	 * @return
	 */
	public static Bitmap getActivityViewShot(Activity activity) {
		// 获取windows中最顶层的view
		View view = activity.getWindow().getDecorView();
		view.buildDrawingCache();

		// 获取状态栏高度
		Rect rect = new Rect();
		view.getWindowVisibleDisplayFrame(rect);
		int statusBarHeights = rect.top;
		Display display = activity.getWindowManager().getDefaultDisplay();

		// 获取屏幕宽和高
		int widths = display.getWidth();
		int heights = display.getHeight();

		// 允许当前窗口保存缓存信息
		view.setDrawingCacheEnabled(true);

		// 去掉状态栏
		Bitmap bmp = Bitmap.createBitmap(view.getDrawingCache(), 0,
				statusBarHeights, widths, heights - statusBarHeights);
		// 销毁缓存信息
		view.destroyDrawingCache();
		return bmp;
	}

	public static Bitmap getViewBitmap(View resourceView) {
		resourceView.setDrawingCacheEnabled(true);
		resourceView.buildDrawingCache();
		return resourceView.getDrawingCache();
	}

	private static long lastClickTime;

	public static boolean isFastDoubleClick(int offsetTime) {
		long time = System.currentTimeMillis();
		long timeD = time - lastClickTime;
		if (0 < timeD && timeD < offsetTime) {
			return true;
		}
		lastClickTime = time;
		return false;
	}

	/**
	 * 获得Meta中添加的值
	 * 
	 * @param key
	 * @param context
	 * @return
	 */
	public static String getMetaData(String key, Context context) {
		ApplicationInfo appInfo;
		try {
			appInfo = context.getPackageManager().getApplicationInfo(
					context.getPackageName(), PackageManager.GET_META_DATA);
		} catch (NameNotFoundException e) {
			return null;
		}
		String msg = appInfo.metaData.getString(key);
		return msg;
	}
	
	/**
	 * 显示吐司  避免多次吐司  避免子线程异常
	 */
	private static Toast myToast = null;
	public static void showToast(final Context context, final String text) {

		new Handler(Looper.getMainLooper()).post(new Runnable() {

			@Override
			public void run() {
				if (myToast == null) {
					myToast = Toast.makeText(context, text, Toast.LENGTH_SHORT);
				} else {
					myToast.setText(text);
				}
				myToast.show();
			}
		});

	}

	public static void showToast(Context context, int rid) {
		String text = context.getString(rid);
		showToast(context, text);
	}
}
