/*------------------------------------------------------------------------

  Copyright 2012-2013 Nuance Communications, Inc.

------------------------------------------------------------------------*/
package com.nuance.dragon.toolkit.audio.bluetooth;

import android.os.Build;

/**
 * Relevant Bluetooth capabilities detected at run-time:
 * The capabilities may differ from one Android platform to another.
 * This object is here to set the expectations accordingly. Then this is up
 * to the application to drive {@link BluetoothManager} accordingly.
 */
public final class BluetoothCapability
{
    /** The Bluetooth adapter needs to be enabled to call {@link BluetoothManager#initialize(Context context, BluetoothListener listener)}.
     * Tested with Android 4.4. Later releases will require testing. */
    public static boolean REQUIRES_ADAPTER_ENABLED = requiresAdapterEnabled();

    /** @deprecated {@link BluetoothListener#onDeviceConnected(int status)} and {@link BluetoothListener#onDeviceDisconnected(int status)} are call backed reliably.
     * Tested with Android 4.4. Later releases will require testing. */
    @Deprecated
    public static boolean REPORTS_DEVICE_CONNECTION = reportsDeviceConnection();

    // Helpers to figure out the values based on the Android SDK #
    private static boolean requiresAdapterEnabled()
    {
        if (Build.VERSION.SDK_INT >= 17) { // JELLY BEAN OR GREATER
            return true;
        }
        else {
            return false;
        }
    }

    private static boolean reportsDeviceConnection()
    {
        if (Build.VERSION.SDK_INT >= 17) { // JELLY BEAN OR GREATER
            return false;
        }
        else {
            return true;
        }
    }
}
