package com.nuance.dragon.toolkit.audio.nat;

import com.nuance.dragon.toolkit.audio.AudioChunk;

/**
 * Interface to the JNI audio functionality
 */
public interface NativeAudio
{
    public interface Recorder
    {
        /**
         * Starts the recorder
         * @return false on error
         */
        boolean start();

        /**
         * Stops the recorder
         * @return false on error TODO: does that make sense?
         */
        boolean stop();

        /**
         * @return The duration of each buffer. Every recorded buffer will have
         * the same size. Use this to determine how frequently to poll {@link
         * #read}.
         */
        int getBufferSizeMs();

        /**
         * Read a chunk of audio. The duration will be whatever is returned by
         * {@link #getBufferSizeMs}.
         * @return An audio chunk, or null if no chunk is available yet.
         */
        AudioChunk read();

        /**
         * Release the recorder. You will leak memory if you don't call this
         * when finished.
         */
        void release();
    }

    public interface Player
    {
        /**
         * Starts the player
         * @return false on error
         */
        boolean start();

        /**
         * Stops the player
         * @return false on error TODO: does that make sense?
         */
        boolean stop();

        /**
         * Write audio to the player
         * @param audio
         * @return The number of shorts written.
         */
        int write(short[] audio);

        /**
         * @return The next chunk of audio that was played, or null if none is
         * available yet.
         */
        AudioChunk getPlayedAudio();

        /**
         * @return true if the player is stalled, i.e. it has played all the
         * audio it was given
         */
        boolean isStalled();

        /**
         * Release the player. You will leak memory if you don't call this when
         * finished.
         */
        void release();
    }

    /**
     * Initialize the JNI audio
     * @return true if successful
     */
    public boolean initialize();

    /**
     * Make sure to call this when finished with the audio, or else you'll leak
     * memory
     */
    public void release();

    /**
     * @param sampleRate
     * @return A JNI recorder
     */
    public Recorder createRecorder(int sampleRate);

    /**
     * @param sampleRate
     * @return A JNI player
     */
    public Player createPlayer(int sampleRate);
}
