package com.nuance.dragon.toolkit.audio.nat;

import android.os.Handler;

import com.nuance.dragon.toolkit.audio.AudioChunk;
import com.nuance.dragon.toolkit.audio.AudioType;
import com.nuance.dragon.toolkit.audio.sources.RecorderSource;
import com.nuance.dragon.toolkit.oem.api.Logger;
import com.nuance.dragon.toolkit.oem.api.NMTHandler;

/**
 * A recorder implementation that uses Android's native audio API (OpenSLES),
 * for slightly improved latency.
 * <p>
 * TODO: This is still in progress.
 */
class NativeRecorderSource extends RecorderSource<AudioChunk>
{
    private final NativeAudio _audio;
    private NativeAudio.Recorder _recorder;
    private boolean _recording;
    private boolean _started;

    NativeRecorderSource(AudioType audioType, NativeAudio nativeAudio, NMTHandler workerThreadHandler)
    {
        super(audioType, workerThreadHandler);
        _audio = nativeAudio;
    }

    @Override
    protected boolean isCodecSupported(AudioType audioType)
    {
        if (audioType.encoding ==  AudioType.Encoding.PCM_16 &&
                (audioType.frequency ==  AudioType.Frequency.FREQ_16KHZ))
            return true;

        return false;
    }

    @Override
    protected boolean startRecordingInternal(AudioType audioType)
    {
        if (_recorder != null)
        {
            Logger.error(this, "Recorder already active");
            return false;
        }

        final NativeAudio.Recorder recorder = _audio.createRecorder(audioType.frequency);
        _recorder = recorder;

        final int bufferSizeMs = _recorder.getBufferSizeMs();
        final Handler handler = new Handler();

        // TODO: Propagate error if we fail to record.
        _recorder.start();
        handler.postDelayed(new Runnable()
        {
            @Override
            public void run()
            {
                AudioChunk readChunk;
                do
                {
                    if (recorder != _recorder)
                        return;

                    readChunk = _recorder.read();
                    if (readChunk != null)
                    {
                        if (!_started)
                        {
                            _started = true;
                            handleStarted();
                        }
                        handleNewAudio(readChunk);
                    }
                } while (readChunk != null);

                if (_recording)
                {
                    handler.postDelayed(this, bufferSizeMs / 2);
                }
            }
        }, bufferSizeMs / 2);

        _recording = true;
        return true;
    }

    @Override
    protected void stopRecordingInternal()
    {
        if (!_recording)
            return;

        AudioChunk readChunk;
        do {
            readChunk = _recorder.read();
            if (readChunk != null)
            {
                handleNewAudio(readChunk);
            }
        } while (readChunk != null);

        _recorder.stop();
        _recorder.release();
        _recorder = null;
        _recording = false;
        handleSourceClosed();
    }
}
