package com.nuance.dragon.toolkit.audio.pipes;

import com.nuance.dragon.toolkit.audio.AbstractAudioChunk;
import com.nuance.dragon.toolkit.audio.AudioSink;
import com.nuance.dragon.toolkit.oem.api.Logger;

/**
 * A special implementation of a {@link BufferingPipe}. It just discards audio when
 * there is not sink attached to it. This might useful when application implements
 * an always recording audio chain where the recorder is always enabled when
 * application is in focus. The audio only gets piped to the connected sink which
 * can be a recognition back-end (Cloud, Elvis, VoCon) upon recording button press.
 */
public class AutoClearBufferingPipe<AudioChunkType extends AbstractAudioChunk> extends BufferingPipe<AudioChunkType>
{
    private boolean _sinkConnected = false;

    @Override
    protected void onChunkBuffered(AudioChunkType chunk)
    {
        super.onChunkBuffered(chunk);

        if (_sinkConnected)
        {
            Logger.verbose(this, "[LATCHK] Buffering chunk:" + chunk);
        }
        else
        {
            Logger.verbose(this, "[LATCHK] Ignoring chunk:" + chunk);
            clearBuffers();
        }
    }

    @Override
    protected void onSinkConnected(AudioSink<AudioChunkType> sink)
    {
        Logger.debug(this, "[LATCHK] onSinkConnected()");
        super.onSinkConnected(sink);
        _sinkConnected = true;
    }

    @Override
    protected void onSinkDisconnected(AudioSink<AudioChunkType> sink)
    {
        Logger.debug(this, "[LATCHK] onSinkDisconnected()");
        super.onSinkDisconnected(sink);
        _sinkConnected = false;
    }
}
