package com.nuance.dragon.toolkit.audio.pipes;

import com.nuance.dragon.toolkit.audio.AbstractAudioChunk;
import com.nuance.dragon.toolkit.oem.api.Logger;
import com.nuance.dragon.toolkit.oem.api.internal.Checker;

/**
 * A {@link BufferingPipe} that drops the first passed audio chunks up to a certain duration.
 * This pipe may be useful to filter out static noises when the platform starts capturing audio.
 */
public class DropChunksPipe<AudioChunkType extends AbstractAudioChunk> extends BufferingPipe<AudioChunkType>
{
    private final int _durationToDiscard;
    private int _durationBuffered;

    /**
     * DropChunksPipe constructor
     */
    public DropChunksPipe()
    {
        this(200);
    }

    /**
     * DropChunksPipe constructor
     * @param durationToDiscard The duration to discard (in ms). Default is 200 ms.
     */
    public DropChunksPipe(int durationToDiscard)
    {
        super();
        Checker.checkArgForCondition("durationToDiscard", ">= 0", durationToDiscard >= 0);
        _durationToDiscard = durationToDiscard;
        _durationBuffered = 0;
    }

    @Override
    protected void onChunkBuffered(AudioChunkType chunk)
    {
        super.onChunkBuffered(chunk);
        _durationBuffered += chunk.audioDuration;
        if (_durationBuffered <= _durationToDiscard)
        {
            Logger.debug(this, _durationBuffered + " ms of audio dropped so far.");
            clearBuffers();
        }
    }
}
