package com.nuance.dragon.toolkit.audio.pipes;

import com.nuance.dragon.toolkit.audio.AudioChunk;
import com.nuance.dragon.toolkit.audio.SpeechDetectionListener;
import com.nuance.dragon.toolkit.oem.api.internal.Checker;

/**
 * A simple pipe that provides speech detection notifications, based on the
 * {@link AudioChunk#audioSpeechStatus}. This does not automatically close the
 * stream on end-of-speech--it just calls {@link SpeechDetectionListener#onEndOfSpeech}.
 */
public class EndPointerPipe extends BufferingPipe<AudioChunk>
{
    private final SpeechDetectionListener _listener;

    private boolean _speechStarted;
    private boolean _speechStopped;

    /**
     * EndPointerPipe constructor
     *
     * @param listener A SpeechDetectionListener that receives notifications
     */
    public EndPointerPipe(SpeechDetectionListener listener)
    {
        Checker.checkArgForNull("listener", listener);
        _listener = listener;
        _speechStarted = false;
        _speechStopped = false;
    }

    @Override
    protected void onChunkBuffered(AudioChunk chunk)
    {
        if (!_speechStarted)
        {
            if (chunk.audioSpeechStatus == AudioChunk.SpeechStatus.SPEECH)
            {
                _speechStarted = true;
                _listener.onStartOfSpeech();
            }
        } else if (!_speechStopped)
        {
            if (chunk.audioSpeechStatus == AudioChunk.SpeechStatus.NO_SPEECH)
            {
                _speechStopped = true;
                _listener.onEndOfSpeech();
            }
        }

        super.onChunkBuffered(chunk);
    }
}
