package com.nuance.dragon.toolkit.audio.pipes;

import junit.framework.Assert;

import com.nuance.dragon.toolkit.audio.AudioChunk;
import com.nuance.dragon.toolkit.audio.AudioSource;
import com.nuance.dragon.toolkit.audio.AudioType;
import com.nuance.dragon.toolkit.audio.util.OpusDecoder;
import com.nuance.dragon.toolkit.oem.api.Logger;

/**
 * Implementation of audio pipe that reads from a single audio source, converts
 * the audio to PCM, and provides the Opus-decoded audio data to a single
 * audio sink.
 */
public class OpusDecoderPipe extends ConverterPipe<AudioChunk, AudioChunk>
{
    private final OpusDecoder _decoder;

    /**
     * SpeexDecoderPipe constructor
     */
    public OpusDecoderPipe() {
        _decoder = new OpusDecoder();
    }

    private AudioType getDecodedAudioType(AudioType audioType) {
        Assert.assertTrue(isAudioSourceTypeSupported(audioType));
        if (audioType.frequency == AudioType.Frequency.FREQ_8KHZ)
            return AudioType.PCM_8k;
        else
            return AudioType.PCM_16k;
    }

    @Override
    protected boolean isAudioSourceTypeSupported(AudioType type)
    {
        return type.encoding == AudioType.Encoding.OPUS;
    }

    @Override
    protected void initialize(AudioType audioType)
    {
        _decoder.initialize(audioType);
    }

    @Override
    protected AudioChunk[] convert(AudioChunk raw)
    {
        Logger.verbose(this, "[LATCHK] convert() start raw:" + raw);
        AudioChunk[] chunks = _decoder.decode(raw);
        if (com.nuance.dragon.toolkit.Build.DEBUG)
        {
            StringBuilder log = new StringBuilder();
            for (AudioChunk ac : chunks)
            {
                log.append(ac);
                log.append("; ");
            }
            Logger.verbose(this, "[LATCHK] convert() end chunk:" + log);
        }
        return chunks;
    }

    @Override
    public AudioType getAudioType()
    {
        AudioSource<AudioChunk> connectedSource = getConnectedSource();
        if (connectedSource != null)
        {
            AudioType sourceType = connectedSource.getAudioType();
            if (sourceType != null)
            {
                return getDecodedAudioType(sourceType);
            }
        }
        return AudioType.UNKNOWN;
    }

    @Override
    protected void cleanup()
    {
        _decoder.release();
    }
}
