package com.nuance.dragon.toolkit.audio.pipes;

import com.nuance.dragon.toolkit.audio.AbstractAudioChunk;
import com.nuance.dragon.toolkit.audio.AudioSink;
import com.nuance.dragon.toolkit.audio.AudioSource;
import com.nuance.dragon.toolkit.audio.AudioType;
import com.nuance.dragon.toolkit.oem.api.NMTHandler;

/**
 * Passes audio through, from one audio source to another. This class can be
 * used for queuing audio from another audio source before connecting an audio
 * sink.
 */
public class SimplePipe<AudioChunkType extends AbstractAudioChunk> extends SingleSinkPipe<AudioChunkType, AudioChunkType>
{
    public SimplePipe()
    {
        this(null);
    }
    
    public SimplePipe(NMTHandler mainThreadHandler)
    {
        super(mainThreadHandler);
    }
    
    @Override
    public AudioType getAudioType()
    {
        return getAudioTypeFromSource();
    }

    @Override
    public int getChunksAvailable()
    {
        return getChunksAvailableFromSource();
    }

    @Override
    protected AudioChunkType getAudioChunk()
    {
        return getAudioChunkFromSource();
    }

    @Override
    public void chunksAvailable(AudioSource<AudioChunkType> fromSource, AudioSink<AudioChunkType> internalSink)
    {
        notifyChunksAvailable();
    }

    @Override
    public void framesDropped(AudioSource<AudioChunkType> fromSource, AudioSink<AudioChunkType> internalSink)
    {
        notifyFramesDropped();
    }

    @Override
    protected void sourceClosed(AudioSource<AudioChunkType> fromSource, AudioSink<AudioChunkType> internalSink)
    {
        notifySourceClosed();
    }

    @Override
    public boolean isActive()
    {
        AudioSource<AudioChunkType> connectedSource = getConnectedSource();
        return (connectedSource != null && connectedSource.isActive());
    }
}
