package com.nuance.dragon.toolkit.audio.sources;

import com.nuance.dragon.toolkit.audio.AudioChunk;
import com.nuance.dragon.toolkit.audio.AudioType;
import com.nuance.dragon.toolkit.oem.api.NMTHandler;

/**
 * Implementation of {@link RecorderSource} that records from the device
 * microphone.
 */
public class MicrophoneRecorderSource extends MicrophoneRecorderSourceBase<AudioChunk> {

    /**
     * Create a recorder that uses its own dedicated worker thread.
     * This is a shortcut constructor that uses default values of:
     * sourceType = MediaRecorder.AudioSource.VOICE_RECOGNITION
     * defaultBufferLengthInMs = 400
     * workerThreadHandler = null
     * @param audioType The audio type to record.
     */
    public MicrophoneRecorderSource(AudioType audioType)
    {
        super(audioType);
    }

    /**
     * Create a recorder that uses its own dedicated worker thread.
     * This is a shortcut constructor that uses default values of:
     * defaultBufferLengthInMs = 400
     * workerThreadHandler = null
     * @param sourceType The source type to use. Default value is
     * MediaRecorder.AudioSource.VOICE_RECOGNITION.
     * @param audioType The audio type to record.
     */
    public MicrophoneRecorderSource(int sourceType, AudioType audioType)
    {
        super(sourceType, audioType);
    }

    /**
     * Create a recorder that uses the given worker thread.
     * This is a shortcut constructor that uses default values of:
     * sourceType = MediaRecorder.AudioSource.VOICE_RECOGNITION
     * defaultBufferLengthInMs = 400
     * @param audioType The audio type to record.
     * @param workerThreadHandler The external worker thread handler. If null,
     * the recorder will use its own dedicated worker thread. Default value is null.
     */
    public MicrophoneRecorderSource(AudioType audioType, NMTHandler workerThreadHandler)
    {
        super(audioType, workerThreadHandler);
    }

    /**
     * Create a recorder that uses the given worker thread.
     * This is a shortcut constructor that uses default values of:
     * sourceType = MediaRecorder.AudioSource.VOICE_RECOGNITION
     * @param audioType The audio type to record.
     * @param bufferLengthInMs The total size (in ms) of the buffer where
     * audio data is written to during the recording. Setting it to a higher value
     * can help record audio smoothly under load. Default value is 400.
     * @param workerThreadHandler The external worker thread handler. If null,
     * the recorder will use its own dedicated worker thread. Default value is null.
     */
    public MicrophoneRecorderSource(AudioType audioType,  int bufferLengthInMs, NMTHandler workerThreadHandler)
    {
        super(audioType, bufferLengthInMs, workerThreadHandler);
    }

    /**
     * Create a recorder that uses the given worker thread.
     * Other constructors are supplied that leave out some of these
     * arguments and use a default value instead.
     * @param sourceType The source type to use. Default value is
     * MediaRecorder.AudioSource.VOICE_RECOGNITION.
     * @param audioType The audio type to record.
     * @param bufferLengthInMs The total size (in ms) of the buffer where
     * audio data is written to during the recording. Setting it to a higher value
     * can help record audio smoothly under load. Default value is 400.
     * @param workerThreadHandler The external worker thread handler. If null,
     * the recorder will use its own dedicated worker thread. Default value is null.
     */
    public MicrophoneRecorderSource(int sourceType, AudioType audioType,  int bufferLengthInMs, NMTHandler workerThreadHandler)
    {
        super(sourceType, audioType, bufferLengthInMs, workerThreadHandler);
    }

    /**
     * Create a recorder that uses the given worker thread.
     * Other constructors are supplied that leave out some of these
     * arguments and use a default value instead.
     * @param sourceType The source type to use. Default value is
     * MediaRecorder.AudioSource.VOICE_RECOGNITION.
     * @param audioType The audio type to record.
     * @param bufferLengthInMs The total size (in ms) of the buffer where
     * audio data is written to during the recording. Setting it to a higher value
     * can help record audio smoothly under load. Default value is 400.
     * @param chunkLengthInMs The size (in ms) of the chunks that
     * are propagated to the rest of the system during the recording. Default value is 20.
     * @param workerThreadHandler The external worker thread handler. If null,
     * the recorder will use its own dedicated worker thread. Default value is null.
     */
    public MicrophoneRecorderSource(int sourceType, AudioType audioType, int bufferLengthInMs, int chunkLengthInMs, NMTHandler workerThreadHandler)
    {
        super(sourceType, audioType, bufferLengthInMs, chunkLengthInMs, workerThreadHandler);
    }

    @Override
    protected AudioChunk createNewAudioChunk(AudioType type, short[] data, long timestamp)
    {
        return new AudioChunk(type, data, timestamp);
    }
}
