package com.nuance.dragon.toolkit.audio.sources;

import com.nuance.dragon.toolkit.audio.AudioChunk;
import com.nuance.dragon.toolkit.audio.AudioType;
import com.nuance.dragon.toolkit.oem.api.Logger;
import com.nuance.dragon.toolkit.oem.api.NMTHandler;
import com.nuance.dragon.toolkit.oem.impl.HandlerOem;

/**
 * Impersonates an audio recorder.
 */
public class RecorderSourceImpersonator extends RecorderSource<AudioChunk>
{
    private boolean _active;
    private boolean _stopping;

    public RecorderSourceImpersonator(AudioType audioType)
    {
        super(audioType, new HandlerOem(), null);
    }

    @Override
    protected boolean startRecordingInternal(AudioType audioType)
    {
        if (!audioType.equals(AudioType.PCM_16k))
        {
            return false;
        }

        _active = true;
        _stopping = false;

        final NMTHandler fakeAudioThreadHandler = new HandlerOem();
        Runnable addFakeChunk = new Runnable()
        {
            @Override
            public void run()
            {
                // Logger.debug(this, "Fake audio chunk generated!");

                AudioChunk chunk = new AudioChunk(AudioType.PCM_16k, new short[320]);
                handleNewAudio(chunk);

                if (!_stopping)
                {
                    fakeAudioThreadHandler.postDelayed(this, 100);
                }
                else
                {
                    Logger.debug(RecorderSourceImpersonator.this, "Source closed.");
                    _stopping = false;
                    _active = false;
                    handleSourceClosed();
                }
            }
        };
        fakeAudioThreadHandler.postDelayed(addFakeChunk, 100);
        return true;
    }

    @Override
    protected void stopRecordingInternal()
    {
        _stopping = true;
    }

    @Override
    public boolean isActive()
    {
        return _active;
    }

    @Override
    protected boolean isCodecSupported(AudioType audioType) {
         if (audioType.equals(AudioType.PCM_16k))
             return true;
         return false;
    }
}
