package com.nuance.dragon.toolkit.cloudservices;

import java.util.Date;

import org.json.JSONException;

import com.nuance.dragon.toolkit.oem.api.Logger;
import com.nuance.dragon.toolkit.oem.api.json.JSONObject;

/**
 * A session is required to access server resources.
 *
 */
class ASSession
{
    static interface SessionListener
    {
        void onGetSessionSucceeded(ASSession session);
        void onGetSessionFailed(ASError error);
    }

    static final String ACCESS_TOKEN_PREFIX = "Authorization jwt-bearer ";

    private static final String KEY_ACCESS_TOKEN = "access_token";
    private static final String KEY_TOKEN_TYPE = "token_type";
    private static final String KEY_REFRESH_TOKEN = "refresh_token";
    private static final String KEY_EXPIRES_IN = "expires_in";
    private static final String KEY_SCOPE = "scope";
    private static final String KEY_NUANCE_USER_ID = "nuanceUserId";

    ThirdPartyLogin _login;
    final String AccessToken;
    final String TokenType;
    final String RefreshToken;
    final Date   AccessTokenExpirationDate;
    final String Scope;
    final String NuanceUserId;

    ASSession(ThirdPartyLogin login, String accessToken, String tokenType, String refreshToken, Date accessTokenExpirationDate, String scope, String nuanceUserId)
    {
        _login = login;
        AccessToken = accessToken;
        TokenType = tokenType;
        RefreshToken = refreshToken;
        AccessTokenExpirationDate = accessTokenExpirationDate;
        Scope = scope;
        NuanceUserId = nuanceUserId;
    }

    boolean isExpired()
    {
        long curTime = System.currentTimeMillis();
        long expTime = AccessTokenExpirationDate.getTime();
        boolean ret = curTime > expTime - 10 * 1000;
        Logger.debug(ASSession.class, "isExpired() ret:" + ret + " (curTime:" + curTime + ", expTime:" + expTime + ")");
        return ret;
    }

    static String constructSessionRequest(String refreshToken, ThirdPartyLogin login, String nuanceUserId)
    {
        StringBuffer sb = new StringBuffer("grant_type=");
        if (refreshToken != null)
        {
            sb.append("refresh_token&refresh_token=").append(refreshToken);
        }
        else if (login == null)
        {
            sb.append("anonymous");
        }
        else
        {
            sb.append("application/json&json=");

            JSONObject o = login.toJSON();
            o.tryPut(KEY_NUANCE_USER_ID, nuanceUserId);

            sb.append(o.toString());
        }

        return sb.toString();
    }

    public static ASSession fromJSON(org.json.JSONObject json) throws JSONException
    {
        String accessToken = json.getString(KEY_ACCESS_TOKEN);
        String tokenType = json.getString(KEY_TOKEN_TYPE);
        String refreshToken = json.getString(KEY_REFRESH_TOKEN);
        long expiresIn = json.getLong(KEY_EXPIRES_IN);
        String scope = json.getString(KEY_SCOPE);
        String nuanceUserId = json.getString(KEY_NUANCE_USER_ID);

        return new ASSession(null, accessToken, tokenType, refreshToken, new Date(System.currentTimeMillis() + expiresIn * 1000), scope, nuanceUserId);
    }
}
