package com.nuance.dragon.toolkit.cloudservices;

import com.nuance.dragon.toolkit.cloudservices.CloudServices.ThirdPartyLoginRetriever;
import com.nuance.dragon.toolkit.oem.api.FileManager;
import com.nuance.dragon.toolkit.oem.api.Logger;
import com.nuance.dragon.toolkit.oem.api.NMTContext;
import com.nuance.dragon.toolkit.oem.api.NMTHandler;

class AccountManager {
    private final String _nmaid;
    private final String _password;
    private final String _asHost;
    private final int    _asPort;
    private final String _deviceId;
    private final String _platform;
    private final ThirdPartyLoginRetriever _loginRetriever;
    private final boolean _reset;
    private final FileManager _fileMgr;
    private final boolean _enableSelfSignedCert;
    private final String  _certSummary;
    private final String  _certData;
    private final NMTContext _context;
    private final NMTHandler _mainHandler;

    private ASRegistration _reg;
    private ThirdPartyLogin _login;
    // At the moment this ref count is primarily to allow CalllogSender to work after the CloudServices has been released.
    private int _refCount;
    
    private boolean _released = false;

    AccountManager(String nmaid, String password, String asHost, int asPort, String deviceId, String platform, ThirdPartyLoginRetriever loginRetriever, boolean reset, FileManager fileMgr, boolean enableSelfSignedCert, String certSummary, String certData, NMTContext context, NMTHandler mainHandler)
    {
        _nmaid = nmaid;
        _password = password;
        _asHost = asHost;
        _asPort = asPort;
         _deviceId = deviceId;
        _platform = platform;
        _loginRetriever = loginRetriever;
        _reset = reset;
        _fileMgr = fileMgr;
        _enableSelfSignedCert = enableSelfSignedCert;
        _certSummary = certSummary;
        _certData = certData;
        _context = context;
        _mainHandler = mainHandler;

        _login = null;
        _refCount = 1;

        _mainHandler.postToLooper(new Runnable()
        {
            @Override
            public void run() {
                createRegistration(null, null);
            }
        });
    }

    void getSession(final ASSession.SessionListener listener)
    {
        Runnable onSucceededRunnable = new Runnable()
        {
            @Override
            public void run() {
                if (_loginRetriever != null)
                {
                    ThirdPartyLogin newLogin = _loginRetriever.fetchThirdPartyLogin();
                    if (newLogin == null && _login == null)
                    {
                        Logger.error(AccountManager.this, "third party login is null!!!");
                        listener.onGetSessionFailed(new ASError(ASError.INTERNAL_ERROR_THIRD_PARTY_LOGIN_MISSING));
                        return;
                    }
                    _login = newLogin == null ? _login : newLogin;
                }
                _reg.getSession(_login, listener);
            }
        };

        if (_reg == null)
        {
            createRegistration(onSucceededRunnable, listener);
        }
        else
        {
            onSucceededRunnable.run();
        }
    }

    String getNuanceUserId()
    {
        return _reg != null ? _reg.getNuanceUserId(_login) : null;
    }

    String getClientId()
    {
        return _reg != null ? _reg.getClientId() : null;
    }
    
    void acquire()
    {
    	_refCount++;
    }
    
    void release()
    {
        if (!_released)
        {
            _refCount--;
            if(_refCount == 0){
                _released = true;
                if (_reg != null)
                    _reg.release();
            }
        }
    }

    private void createRegistration(final Runnable onSucceededRunnable, final ASSession.SessionListener onFailedlistener)
    {
        ASRegistration.newRegistration(_nmaid, _password, _deviceId, _platform, _asHost, _asPort, new ASRegistration.RegistrationListener()
        {
            @Override
            public void onNewRegistrationSucceeded(ASRegistration reg) {
                if (_released)
                {
                    reg.release();
                    return;
                }
                
                if (_reg != null)
                    reg.release();
                else
                    _reg = reg;
                if (onSucceededRunnable != null)
                    onSucceededRunnable.run();
            }

            @Override
            public void onNewRegistrationFailed(ASError error) {
                Logger.error(AccountManager.this, "createRegistration(" + error.toString() + ")");
                if (onFailedlistener != null)
                    onFailedlistener.onGetSessionFailed(error);
            }
        }, _reset, _fileMgr, _enableSelfSignedCert, _certSummary, _certData, _context);
    }
}
