package com.nuance.dragon.toolkit.cloudservices;

import com.nuance.dragon.toolkit.core.calllog.CalllogSender;
import com.nuance.dragon.toolkit.oem.api.NMTHandler;

public class CalllogSenderImpl extends CalllogSender {

    private final CalllogSender.SenderListener _senderListener;
    private final boolean _newCop;
    private final com.nuance.nmsp.client.sdk.components.core.calllog.CalllogSender  _nmspCalllogSender1;
    private final com.nuance.nmsp.client2.sdk.components.core.calllog.CalllogSender _nmspCalllogSender2;
    private final AccountManager _accManager;
    private final NMTHandler     _mainThreadHandler;

    CalllogSenderImpl(CalllogSender.SenderListener senderListener, boolean newCop, AccountManager accManager, NMTHandler mainThreadHandler) {
        _senderListener = senderListener;
        _newCop = newCop;
        _accManager = accManager;
        _mainThreadHandler = mainThreadHandler;
        

        
        class SucceededRunnable implements Runnable
        {
            private byte[] _data;
            private SucceededRunnable(byte[] data)
            {
                _data = data;
            }
            @Override
            public void run() {
                _mainThreadHandler.post(new Runnable()
                {
                    @Override
                    public void run() {
                        if (_senderListener != null)
                            _senderListener.succeeded(_data);
                    }
                });
            }
        }
        class FailedRunnable implements Runnable
        {
            private short _errorCode;
            private byte[] _data;
            private FailedRunnable(short errorCode, byte[] data)
            {
                _errorCode = errorCode;
                _data = data;
            }
            @Override
            public void run() {
                _mainThreadHandler.post(new Runnable()
                {
                    @Override
                    public void run() {
                        if (_senderListener != null)
                            _senderListener.failed(_errorCode, _data);
                    }
                });
            }
        }
        if (!_newCop)
        {
            com.nuance.nmsp.client.sdk.components.core.calllog.CalllogSender.SenderListener nmspSenderListener1 =
                    new com.nuance.nmsp.client.sdk.components.core.calllog.CalllogSender.SenderListener() {

                        @Override
                        public void succeeded(byte[] data) {
                            (new SucceededRunnable(data)).run();
                        }

                        @Override
                        public void failed(short errorCode, byte[] data) {
                            (new FailedRunnable(errorCode, data)).run();
                        }
                    };
            _nmspCalllogSender1 = com.nuance.nmsp.client.sdk.components.core.calllog.CalllogSenderFactory.createCalllogSender(nmspSenderListener1);
            _nmspCalllogSender2 = null;
        } else
        {
            com.nuance.nmsp.client2.sdk.components.core.calllog.CalllogSender.SenderListener nmspSenderListener2 =
                    new com.nuance.nmsp.client2.sdk.components.core.calllog.CalllogSender.SenderListener() {

                        @Override
                        public void succeeded(byte[] data) {
                            (new SucceededRunnable(data)).run();
                        }

                        @Override
                        public void failed(short errorCode, byte[] data) {
                            (new FailedRunnable(errorCode, data)).run();
                        }
                    };
            _nmspCalllogSender2 = com.nuance.nmsp.client2.sdk.components.core.calllog.CalllogSenderFactory.createCalllogSender(nmspSenderListener2);
            _nmspCalllogSender1 = null;
            
            _accManager.acquire();
        }
    }

    @Override
    public void send(final byte[] data) {
        _mainThreadHandler.postToLooper(new Runnable()
        {
            @Override
            public void run() {
                if (!_newCop)
                {
                    _nmspCalllogSender1.send(data);
                }
                else
                {
                    _accManager.getSession(new ASSession.SessionListener()
                    {
                        @Override
                        public void onGetSessionSucceeded(ASSession session) {
                            _nmspCalllogSender2.send(data, session.NuanceUserId, ASSession.ACCESS_TOKEN_PREFIX + session.AccessToken);
                        }
                        @Override
                        public void onGetSessionFailed(ASError error) {
                            _senderListener.failed(error.InternalErrorCode, data);
                        }
                    });
                }
            }
        });
    }
    
    @Override
    public void shutdown()
    {
        if (!_newCop)
            _nmspCalllogSender1.shutdown();
        else{
        	_accManager.release();
            _nmspCalllogSender2.shutdown();
        }
    }
    
}
