package com.nuance.dragon.toolkit.cloudservices;

import com.nuance.dragon.toolkit.core.calllog.SessionEvent;
import com.nuance.dragon.toolkit.core.calllog.SessionEventBuilder;
import com.nuance.dragon.toolkit.oem.api.Logger;
import com.nuance.dragon.toolkit.oem.api.internal.Checker;

public class CalllogSessionEventBuilderImpl implements SessionEventBuilder {

    private final boolean _newCop;
    private final com.nuance.nmsp.client.sdk.components.core.calllog.SessionEventBuilder  _builder1;
    private final com.nuance.nmsp.client2.sdk.components.core.calllog.SessionEventBuilder _builder2;

    public CalllogSessionEventBuilderImpl(Object builder, boolean newCop) {
        _newCop = newCop;
        if (!_newCop)
        {
            _builder1 = (com.nuance.nmsp.client.sdk.components.core.calllog.SessionEventBuilder)builder;
            _builder2 = null;
        }
        else
        {
            _builder2 = (com.nuance.nmsp.client2.sdk.components.core.calllog.SessionEventBuilder)builder;
            _builder1 = null;
        }
    }

    @Override
    public SessionEventBuilder putBinary(String key, byte[] value)
    {
        Checker.checkArgForNull("key", key);
        Checker.checkArgForNull("value", value);

        if ((!_newCop ? _builder1 : _builder2) != null) {
            try {
                if (!_newCop)
                    _builder1.putBinary(key, value);
                else
                    _builder2.putBinary(key, value);
            } catch (com.nuance.nmsp.client.sdk.components.core.calllog.SessionEventAlreadyCommittedException e) {
                Logger.warn(this, "Cannot add value into committed event!");
                return null;
            } catch (com.nuance.nmsp.client2.sdk.components.core.calllog.SessionEventAlreadyCommittedException e) {
                Logger.warn(this, "Cannot add value into committed event!");
                return null;
            }
        }
        return this;
    }

    @Override
    public SessionEventBuilder putBoolean(String key, boolean value)
    {
        Checker.checkArgForNull("key", key);

        if ((!_newCop ? _builder1 : _builder2) != null) {
            try {
                if (!_newCop)
                    _builder1.putBoolean(key, value);
                else
                    _builder2.putBoolean(key, value);
            } catch (com.nuance.nmsp.client.sdk.components.core.calllog.SessionEventAlreadyCommittedException e) {
                Logger.warn(this, "Cannot add value into committed event!");
                return null;
            } catch (com.nuance.nmsp.client2.sdk.components.core.calllog.SessionEventAlreadyCommittedException e) {
                Logger.warn(this, "Cannot add value into committed event!");
                return null;
            }
        }
        return this;
    }

    @Override
    public SessionEventBuilder putFloat(String key, double value)
    {
        Checker.checkArgForNull("key", key);

        if ((!_newCop ? _builder1 : _builder2) != null) {
            try {
                if (!_newCop)
                    _builder1.putFloat(key, value);
                else
                    _builder2.putFloat(key, value);
            } catch (com.nuance.nmsp.client.sdk.components.core.calllog.SessionEventAlreadyCommittedException e) {
                Logger.warn(this, "Cannot add value into committed event!");
                return null;
            } catch (com.nuance.nmsp.client2.sdk.components.core.calllog.SessionEventAlreadyCommittedException e) {
                Logger.warn(this, "Cannot add value into committed event!");
                return null;
            }
        }
        return this;
    }

    @Override
    public SessionEventBuilder putInteger(String key, int value)
    {
        Checker.checkArgForNull("key", key);

        if ((!_newCop ? _builder1 : _builder2) != null) {
            try {
                if (!_newCop)
                    _builder1.putInteger(key, value);
                else
                    _builder2.putInteger(key, value);
            } catch (com.nuance.nmsp.client.sdk.components.core.calllog.SessionEventAlreadyCommittedException e) {
                Logger.warn(this, "Cannot add value into committed event!");
                return null;
            } catch (com.nuance.nmsp.client2.sdk.components.core.calllog.SessionEventAlreadyCommittedException e) {
                Logger.warn(this, "Cannot add value into committed event!");
                return null;
            }
        }
        return this;
    }

    @Override
    public SessionEventBuilder putLong(String key, long value)
    {
        Checker.checkArgForNull("key", key);

        if ((!_newCop ? _builder1 : _builder2) != null) {
            try {
                if (!_newCop)
                    _builder1.putLong(key, value);
                else
                    _builder2.putLong(key, value);
            } catch (com.nuance.nmsp.client.sdk.components.core.calllog.SessionEventAlreadyCommittedException e) {
                Logger.warn(this, "Cannot add value into committed event!");
                return null;
            } catch (com.nuance.nmsp.client2.sdk.components.core.calllog.SessionEventAlreadyCommittedException e) {
                Logger.warn(this, "Cannot add value into committed event!");
                return null;
            }
        }
        return this;
    }

    @Override
    public SessionEventBuilder putString(String key, String value)
    {
        Checker.checkArgForNull("key", key);
        Checker.checkArgForNull("value", value);

        if ((!_newCop ? _builder1 : _builder2) != null) {
            try {
                if (!_newCop)
                    _builder1.putString(key, value);
                else
                    _builder2.putString(key, value);
            } catch (com.nuance.nmsp.client.sdk.components.core.calllog.SessionEventAlreadyCommittedException e) {
                Logger.warn(this, "Cannot add value into committed event!");
                return null;
            } catch (com.nuance.nmsp.client2.sdk.components.core.calllog.SessionEventAlreadyCommittedException e) {
                Logger.warn(this, "Cannot add value into committed event!");
                return null;
            }
        }
        return this;
    }

    @Override
    public SessionEventBuilder createRemoteEvent(String eventName)
    {
        Checker.checkArgForNull("eventName", eventName);

        if ((!_newCop ? _builder1 : _builder2) != null) {
            try {
                if (!_newCop)
                    _builder1.createRemoteEvent(eventName);
                else
                    _builder2.createRemoteEvent(eventName);
            } catch (com.nuance.nmsp.client.sdk.components.core.calllog.SessionEventAlreadyCommittedException e) {
                Logger.warn(this, "Cannot create remote event, event is already committed!");
                return null;
            } catch (com.nuance.nmsp.client2.sdk.components.core.calllog.SessionEventAlreadyCommittedException e) {
                Logger.warn(this, "Cannot create remote event, event is already committed!");
                return null;
            }
        }
        return this;
    }

    @Override
    public SessionEvent commit() {
        if ((!_newCop ? _builder1 : _builder2) != null) {
            Object event = !_newCop ? _builder1.commit() : _builder2.commit();
            if (event != null)
                return new CalllogSessionEventImpl(event, _newCop);
        }
        return null;
    }
}
