package com.nuance.dragon.toolkit.cloudservices;

import java.util.Enumeration;
import java.util.Hashtable;

import com.nuance.dragon.toolkit.oem.api.internal.Checker;
import com.nuance.nmsp.client2.sdk.common.defines.NMSPDefines;

/**
 * Represents a configuration for {@link CloudConfig}. It sets the CloudServices idle session timeout.
 */
public class ConnectionTimeoutConfig implements NMTConfig
{
    private Hashtable<String, Object> _parameters = new Hashtable<String, Object>();

    /**
     * Make a IdleSessionTimeoutConfig object.
     *
     * @param timeoutS timeout value in seconds
     */
    public ConnectionTimeoutConfig(int timeoutS)
    {
        Checker.checkArgForCondition("timeoutS", "greater than 0", timeoutS > 0);
        _parameters.put(NMSPDefines.ConnectionTimeout, ("" + timeoutS).getBytes());
    }

    @Override
    public int size() {
        return _parameters.size();
    }

    @Override
    public Enumeration<String> keys() {
        return _parameters.keys();
    }

    @Override
    public Object getValue(String key) {
        return _parameters.get(key);
    }
}
