package com.nuance.dragon.toolkit.cloudservices;

import org.json.JSONException;

import com.nuance.dragon.toolkit.oem.api.JSONCompliant;
import com.nuance.dragon.toolkit.oem.api.json.JSONObject;

/**
 * Base class for params that represent just data, and can be converted to/from
 * JSON.
 */
public abstract class DataParam extends Param implements JSONCompliant
{
    static final String KEY_KEY = "key";
    static final String KEY_TYPE = "type";
    static final String KEY_VALUE = "value";

    /**
     * @param type  the type defined in {@link Param}
     * @param key   the name of the parameter
     */
    DataParam(int type, String key)
    {
        super(type, key);
    }

    /**
     * @param json The input data with specific fields to build a DataParam
     * @throws JSONException if some mandatory fields are missing from the input
     *                       json object.
     */
    DataParam(org.json.JSONObject json) throws JSONException
    {
        super(json.getInt(KEY_TYPE), json.getString(KEY_KEY));
    }

    @Override
    public org.json.JSONObject toJSON()
    {
        JSONObject o = new JSONObject();
        o.tryPut(KEY_KEY, key);
        o.tryPut(KEY_TYPE, type);
        return o;
    }

    /**
     * @param json The input data with specific fields to build a DataParam
     * @return DataParam
     * @throws JSONException if some mandatory fields are missing from the input json object
     */
    public static DataParam createFromJSON(org.json.JSONObject json) throws JSONException
    {
        int type = json.getInt(KEY_TYPE);
        DataParam ret = null;
        switch (type)
        {
        case TYPE_STRING:
            ret = new StringParam(json);
            break;
        case TYPE_DICTIONARY:
            ret = new DictionaryParam(json);
            break;
        case TYPE_SEQUENCE:
        case TYPE_INTEGER:
        case TYPE_BYTES:
        case TYPE_SEQUENCE_START:
        case TYPE_SEQUENCE_CHUNK:
        case TYPE_SEQUENCE_END:
        case TYPE_AUDIO:
        case TYPE_TTS:
        default:
            throw new JSONException("Can't create from JSON, type=" + type);
        }
        return ret;
    }
}
