package com.nuance.dragon.toolkit.cloudservices;

import org.json.JSONException;

import com.nuance.dragon.toolkit.core.data.Data;
import com.nuance.dragon.toolkit.oem.api.internal.Checker;

/**
 * A dictionary parameter to be sent via {@link Transaction#addParam}.
 */
public class DictionaryParam extends DataParam
{
    protected final Data.Dictionary _dict;

    /**
     * Creates a parameter to send PDX Dictionary data to the Nuance Mobile
     * Application Server network command. The Dictionary contains one or
     * more key-value pairs.
     *
     * @param key   The name of the Dictionary Parameter expected by the command
     * @param dict  The Dictionary contains one or more key-value pairs.
     */
    public DictionaryParam(String key, Data.Dictionary dict)
    {
        this(TYPE_DICTIONARY, key, dict);
    }

    DictionaryParam(int type, String key, Data.Dictionary dict)
    {
        super(type, key);
        Checker.checkArgForNull("dict", dict);
        _dict = dict;
    }

    DictionaryParam(org.json.JSONObject json) throws JSONException
    {
        super((org.json.JSONObject) Checker.checkArgForNull("json", json));
        _dict = Data.Dictionary.createFromJSON(json.getJSONObject(KEY_VALUE));
    }

    @Override
    void send(CommandContext commandContext) throws Exception
    {
        commandContext.sendDictParam(key, _dict);
    }
}
