package com.nuance.dragon.toolkit.cloudservices;

import java.util.Vector;

import com.nuance.dragon.toolkit.cloudservices.CloudServices.MdsMessageListener;
import com.nuance.dragon.toolkit.cloudservices.util.PdxConverter;
import com.nuance.dragon.toolkit.oem.api.Logger;
import com.nuance.dragon.toolkit.oem.api.NMTHandler;
import com.nuance.nmsp.client2.sdk.components.core.internal.pdx.PDXDictionary;
import com.nuance.nmsp.client2.sdk.components.resource.common.Manager;
import com.nuance.nmsp.client2.sdk.components.resource.common.Resource;
import com.nuance.nmsp.client2.sdk.components.resource.common.ResourceException;
import com.nuance.nmsp.client2.sdk.components.resource.nmas.NMASResourceFactory;
import com.nuance.nmsp.client2.sdk.components.resource.nmas.NMASResourceListener;

class MdsContextImpl implements MdsContext {
    private NMTHandler           _mainThreadHandler;
    private MdsMessageListener   _mdsListener;
    private Manager              _nmspManager;
    private CloudServicesConfig  _config;
    private AccountManager       _accManager;
    private NMASResourceListener _resourceListener;
    private Resource             _resource = null;

    MdsContextImpl(MdsMessageListener listener, Manager nmspManager, CloudServicesConfig config, AccountManager accManager, NMTHandler mainHandler)
    {
        _mainThreadHandler = mainHandler;
        _mdsListener = listener;
        _nmspManager = nmspManager;
        _config = config;
        _accManager = accManager;
        _resourceListener = createMdsResourceListener();
    }
    
    @Override
    public void createOrReloadMdsResource() {
        _accManager.getSession(new ASSession.SessionListener()
        {
            @Override
            public void onGetSessionSucceeded(ASSession session) {
                if (_resource == null)
                {
                    _resource = NMASResourceFactory.createNMASResource(_nmspManager, _resourceListener, null, _config.applicationName(), session.NuanceUserId, ASSession.ACCESS_TOKEN_PREFIX + session.AccessToken);
                }
                else
                {
                    //TODO _mdsResource.loadResource();
                }
            }
            @Override
            public void onGetSessionFailed(ASError error) {
            }
        });
    }
    
    @Override
    public void release() {
        if (_resource != null)
        {
	        try {
	            _resource.freeResource(0);
	        } catch (ResourceException e) {
	        }
        }
    }
    
    private NMASResourceListener createMdsResourceListener()
    {
        return new NMASResourceListener()
        {
            @Override
            public void PDXCommandCreated(final String sessionId)
            {
            }

            @Override
            public void PDXCreateCommandFailed(short reasonCode)
            {
            }

            @SuppressWarnings("rawtypes")
            @Override
            public void getParameterCompleted(short arg0, Vector arg1, long arg2)
            {
            }

            @Override
            public void getParameterFailed(short arg0, short arg1, long arg2)
            {
            }

            @Override
            public void resourceUnloaded(short arg0)
            {
            }

            @SuppressWarnings("rawtypes")
            @Override
            public void setParameterCompleted(short arg0, Vector arg1, long arg2)
            {
            }

            @Override
            public void setParameterFailed(short arg0, short arg1, long arg2)
            {
            }

            @Override
            public void sendMdsMessageStatus(int statusCode, int transactionId) {
                // TODO
            }

            @Override
            public boolean receiveMdsMessageStatus(int statusCode,
                    final int transactionId, final PDXDictionary message) {
                Logger.debug(MdsContextImpl.this, "receiveMdsMessageStatus(" + statusCode + ", " + transactionId + ", " + message + ")");
                
                // For phase one we only pass on successful MDS messages
                if (statusCode != STATUS_CODE_MDS_MESSAGE_RECEIVED)
                    return true;
                
                _mainThreadHandler.post(new Runnable()
                {
                    public void run()
                    {
                        _mdsListener.onMessageReceived(transactionId, message == null ? null : PdxConverter.dictionaryFromPdx(message));
                    }
                });
                return true;
            }
        };
    }
}
