package com.nuance.dragon.toolkit.cloudservices;

import com.nuance.dragon.toolkit.oem.api.internal.Checker;

/**
 * Represents a parameter that is sent in an NMAS command. Contains the logic
 * for sending the specific parameter type.
 */
public abstract class Param
{
    public static final int TYPE_AUDIO = 1;
    public static final int TYPE_TTS = 2;
    public static final int TYPE_STRING = 3;
    public static final int TYPE_INTEGER = 4;
    public static final int TYPE_BYTES = 5;
    public static final int TYPE_DICTIONARY = 6;
    public static final int TYPE_SEQUENCE = 7;
    public static final int TYPE_SEQUENCE_START = 8;
    public static final int TYPE_SEQUENCE_CHUNK = 9;
    public static final int TYPE_SEQUENCE_END = 10;

    final String key;
    final int type;
    boolean isStream;

    Param(int type, String key)
    {
        Checker.checkArgForCondition("type", "should be one of Param.TYPE_AUDIO, Param.TYPE_TTS, Param.TYPE_STRING, " +
                "Param.TYPE_INTEGER, Param.TYPE_BYTES, Param.TYPE_DICTIONARY, Param.TYPE_SEQUENCE, " +
                "Param.TYPE_SEQUENCE_START, Param.TYPE_SEQUENCE_CHUNK, Param.TYPE_SEQUENCE_END",
                (type >= TYPE_AUDIO && type <= TYPE_SEQUENCE_END));
        Checker.checkStringArgForNullOrEmpty("key", key);
        this.type = type;
        this.key = key;
        isStream = false;
    }

    /**
     * Send the parameter.
     * @param commandContext Used to send the parameter via the NMSP SDK.
     * @throws Exception
     */
    abstract void send(CommandContext commandContext) throws Exception;

    /**
     * Cancel an asynchronous send that is in progress (if any). Default
     * implementation does nothing.
     */
    void cancelSend()
    {
        // Nothing by default
    }

    boolean isStreamParam()
    {
        return isStream;
    }
}
