package com.nuance.dragon.toolkit.cloudservices;

import java.util.Enumeration;
import java.util.Hashtable;

import com.nuance.dragon.toolkit.oem.api.internal.Checker;
import com.nuance.nmsp.client2.sdk.common.defines.NMSPDefines;

/**
 * The SSL Configuration is always enabled, the extra parameters are:
 * self signed or not, the SSL certificate summary and SSL certificate Data.
 */
public class SSLConfig implements NMTConfig
{
    private Hashtable<String, Object> _parameters = new Hashtable<String, Object>();

    /**
     * Make a SSLConfig object.
     *
     * @param selfSigned
     * @param certSummary Certificate Summary could be null, if non-null
     *                    it should not be empty
     * @param certData    Certificate Data could be null, if non-null
     *                    it should not be empty
     */
    public SSLConfig(boolean selfSigned, String certSummary, String certData)
    {
        Checker.checkStringArgForEmpty("certSummary", certSummary);
        Checker.checkStringArgForEmpty("certData", certData);

        _parameters.put(NMSPDefines.NMSP_DEFINES_SSL_SOCKET, "TRUE".getBytes());

        if (selfSigned) {
            _parameters.put(NMSPDefines.NMSP_DEFINES_SSL_SELFSIGNED_CERT, "TRUE".getBytes());
        }

        if (certSummary != null) {
            _parameters.put(NMSPDefines.NMSP_DEFINES_SSL_CERT_SUMMARY, certSummary.getBytes());
        }

        if (certData != null) {
            _parameters.put(NMSPDefines.NMSP_DEFINES_SSL_CERT_DATA, certData.getBytes());
        }
    }

    @Override
    public int size()
    {
        return _parameters.size();
    }

    @Override
    public Enumeration<String> keys()
    {
        return _parameters.keys();
    }

    @Override
    public Object getValue(String key)
    {
        return _parameters.get(key);
    }
}
