package com.nuance.dragon.toolkit.cloudservices;

/**
 * Streaming parameter (e.g. audio)
 */
public abstract class StreamParam extends Param {

    /**
     * Listener that notifies when an asynchronous parameter has finished streaming.
     */
    public interface StreamListener
    {
        void onStreamComplete(Param param, boolean error);
    }

    StreamParam(int type, String key)
    {
        super(type, key);
        isStream = true;
    }

    /**
     * Stream the parameter. If the stream is asynchronous, the streamListener will
     * be called when the stream has completed, this function will return false,
     * and {@link #isStreaming} will return true until the stream has completed.
     * Otherwise, this function will return true.
     * @param commandContext Used to send the parameter via the NMSP SDK.
     * @param streamListener If returns false, the streamListener will be called
     * later when the streaming of the parameter is complete.
     * @return true if done streaming, false if asynchronous streaming needs to
     * continue (e.g. audio)
     * @throws Exception
     */
    abstract boolean stream(CommandContext commandContext, StreamListener streamListener) throws Exception;

    /**
     * Cancel an asynchronous streaming that is in progress (if any).
     */
    abstract void cancelStreaming();

    /**
     * Is this param streaming?
     * @return true if the param is still streaming.
     */
    abstract boolean isStreaming();
}
