package com.nuance.dragon.toolkit.cloudservices;

import org.json.JSONException;

import com.nuance.dragon.toolkit.oem.api.internal.Checker;

/**
 * A text parameter to be sent via {@link Transaction#addParam}.
 */
public class StringParam extends DataParam
{
    protected final String _value;

    /**
     * @param key   The name of the String Parameter
     * @param value The value of the String Parameter
     */
    public StringParam(String key, String value)
    {
        super(TYPE_STRING, key);
        Checker.checkArgForNull("value", value);
        _value = value;
    }

    /**
     * @param json the JSON object containing the specific fields for building
     *             the String Parameter
     * @throws JSONException if the input JSON object missing the mandatory fields for building
     *                       the String Parameter
     */
    StringParam(org.json.JSONObject json) throws JSONException
    {
        super(json);
        _value = json.getString(KEY_VALUE);
    }

    @Override
    void send(CommandContext commandContext) throws Exception
    {
        commandContext.sendTextParam(key, _value);
    }
}
