package com.nuance.dragon.toolkit.cloudservices;

import com.nuance.dragon.toolkit.oem.api.JSONCompliant;
import com.nuance.dragon.toolkit.oem.api.internal.Checker;
import com.nuance.dragon.toolkit.oem.api.json.JSONObject;

/**
 * A third party login, which is validated by the authorization server
 * in order to gain access to Nuance services. 
 */
public class ThirdPartyLogin implements JSONCompliant
{
    private static final String KEY_PROVIDER_ID = "providerId";
    private static final String KEY_USER_ID = "userId";
    private static final String KEY_ACCESS_TOKEN = "accessToken";

	public final String ProviderId;
	public final String UserId;
	public final String AccessToken;
	
	public ThirdPartyLogin(String providerId, String userId, String accessToken)
	{
	    Checker.checkStringArgForNullOrEmpty("providerId", providerId);
	    Checker.checkStringArgForNullOrEmpty("userId", userId);
	    Checker.checkStringArgForNullOrEmpty("accessToken", accessToken);
	    
		ProviderId = providerId;
		UserId = userId;
		AccessToken = accessToken;
	}
	
	@Override
	public boolean equals(Object obj)
	{
	    if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        ThirdPartyLogin other = (ThirdPartyLogin)obj;
        if (!ProviderId.equals(other.ProviderId))
            return false;
        if (!UserId.equals(other.UserId))
            return false;
        if (!AccessToken.equals(other.AccessToken))
            return false;
        return true;
	}

	@Override
    public JSONObject toJSON()
    {
        JSONObject o = new JSONObject();
        o.tryPut(KEY_PROVIDER_ID, ProviderId);
        o.tryPut(KEY_USER_ID, UserId);
        o.tryPut(KEY_ACCESS_TOKEN, AccessToken);
        return o;
    }
}
