package com.nuance.dragon.toolkit.cloudservices;

import java.util.ArrayList;
import java.util.List;

import com.nuance.dragon.toolkit.core.data.Data;
import com.nuance.dragon.toolkit.oem.api.Logger;


class TransactionStateStreaming extends TransactionState {

    private final List<StreamParam>   _streamingParams;
    private final ArrayList<StreamParam> _remainingStreamingParams;
    private final StreamParam.StreamListener _streamingListener;

    TransactionStateStreaming(CommandContext commandContext, Transaction transaction, List<StreamParam> params) {
        super(commandContext, true, false, transaction);
        _remainingStreamingParams = new ArrayList<StreamParam>();
        _streamingParams = params;

        _streamingListener = new StreamParam.StreamListener()
        {
            @Override
            public void onStreamComplete(Param param, boolean error) {
                if (error)
                {
                    handleError(new TransactionError(_transaction, TransactionError.Types.OTHER_ERROR, null, param.key, null, 0));
                }
            }
        };
    }

    @Override
    void enter()
    {
        super.enter();
        for (StreamParam p : _streamingParams) {
            boolean done;
            try
            {
                done = p.stream(_context, _streamingListener);
            } catch (Exception e)
            {
                Logger.error(this, "Failed to stream param [" + p.key + "] command [" + _transaction.getCommandName() + "]", e);
                handleError(new TransactionError(_transaction, TransactionError.Types.OTHER_ERROR, e.getMessage(), p.key, null, 0));
                return;
            }

            if (!done)
                _remainingStreamingParams.add(p);
        }
    }

    @Override
    void leave()
    {
        for (StreamParam p : _remainingStreamingParams)
        {
            Logger.debug(this, "Streaming parameter: " + p.key + " is cancelled.");
            p.cancelStreaming();
        }
        _streamingParams.clear();
        _remainingStreamingParams.clear();
        super.leave();
    }

    @Override
    void queryResult(Data.Dictionary queryResult)
    {
        super.queryResult(queryResult);
        TransactionResult qResult = new TransactionResult(_transaction, queryResult);
        boolean isFinal = qResult.isFinal();
        Logger.debug(this, "Result in streaming: " + qResult.toString() + " is final: " + isFinal);
        if (isFinal)
        {
            switchToState(new TransactionStateFinished(_context, _transaction));
        }
        _transaction.handleResult(qResult, isFinal);
    }
}
