package com.nuance.dragon.toolkit.cloudservices.datadownloader;

import com.nuance.dragon.toolkit.core.data.Data.Dictionary;
import com.nuance.dragon.toolkit.oem.api.internal.Checker;

/**
 * Represents a configuration for {@link NcsCloudDataDownloader}. It allows the application
 * to override a couple of the NVC download command and download logic parameters.
 */
public class NcsCloudDataDownloaderConfig
{
    public final String commandName;
    public final Dictionary commandSettings;
    public final int commandTimeoutMs;

    /**
     * Create a {@link NcsCloudDataDownloaderConfig} object with the default parameters.
     */
    public NcsCloudDataDownloaderConfig()
    {
        this("NCS_DOWNLOAD_DATA_CMD", null, 40000);
    }

    /**
     * Create a {@link NcsCloudDataDownloaderConfig} object with the given parameters.
     *
     * @param commandName The command name.
     * @param commandSettings The command settings. Can be null.
     * @param commandTimeoutMs The command timeout (in milliseconds).
     */
    public NcsCloudDataDownloaderConfig(String commandName, Dictionary commandSettings,  int commandTimeoutMs)
    {
        Checker.checkStringArgForNullOrEmpty("commandName", commandName);
        Checker.checkArgForCondition("commandTimeoutMs", "stricly greater than 0", commandTimeoutMs > 0);

        this.commandName = commandName;
        this.commandSettings = commandSettings;
        this.commandTimeoutMs = commandTimeoutMs;
    }
}
