package com.nuance.dragon.toolkit.cloudservices.datauploader;

import com.nuance.dragon.toolkit.core.data.Data.Dictionary;
import com.nuance.dragon.toolkit.oem.api.internal.Checker;

/**
 * Represents a configuration for {@link CloudDataUploader}. It allow the application
 * to override a couple of the NVC upload command and upload logic parameters.
 */
public class CloudDataUploaderConfig
{
    public final String commandName;
    public final Dictionary commandSettings;
    public final int commandTimeoutMs;
    public final int commandChunkSize, commandTotalSize;

    /**
     * Create a {@link CloudDataUploaderConfig} object with the default parameters.
     */
    public CloudDataUploaderConfig()
    {
        this("NVC_DATA_UPLOAD_CMD", null, 40000, 50, 200);
    }

    /**
     * Create a {@link CloudDataUploaderConfig} object with the given parameters.
     *
     * @param commandName The command name.
     * @param commandSettings The command settings. Can be null.
     * @param commandTimeoutMs The command timeout (in milliseconds).
     * @param commandChunkSize The command chunk size (i.e. the number of items per data block).
     * @param commandTotalSize The command total size (i.e. the number of items per NVC command).
     */
    public CloudDataUploaderConfig(String commandName, Dictionary commandSettings,  int commandTimeoutMs, int commandChunkSize, int commandTotalSize)
    {
        Checker.checkStringArgForNullOrEmpty("commandName", commandName);
        Checker.checkArgForCondition("commandTimeoutMs", "stricly greater than 0", commandTimeoutMs > 0);
        Checker.checkArgForCondition("commandChunkSize", "stricly greater than 0", commandChunkSize > 0);
        Checker.checkArgForCondition("commandTotalSize", "stricly greater than 0", commandTotalSize > 0);

        this.commandName = commandName;
        this.commandSettings = commandSettings;
        this.commandTimeoutMs = commandTimeoutMs;
        this.commandChunkSize = commandChunkSize;
        this.commandTotalSize = commandTotalSize;
    }
}
