package com.nuance.dragon.toolkit.cloudservices.recognizer;

import junit.framework.Assert;

import org.json.JSONException;

import com.nuance.dragon.toolkit.cloudservices.TransactionError;
import com.nuance.dragon.toolkit.oem.api.JSONCompliant;
import com.nuance.dragon.toolkit.oem.api.json.JSONObject;

/**
 * A cloud recognition error.
 */
public class CloudRecognitionError implements JSONCompliant
{
    /**
     * Possible error types.
     */
    public static final class Errors
    {
        /**
         * The error came from the
         * {@link com.nuance.dragon.toolkit.cloudservices.Transaction}.
         */
        public static final int TRANSACTION = 0;

        /** TODO: more error types? */
    }

    private static final String    KEY_TRANSACTION_ERROR = "transaction_error";
    private static final String    KEY_TYPE              = "type";

    private final TransactionError _transactionError;
    private final int              _type;

    CloudRecognitionError(int type)
    {
        this(type, null);
    }

    /**
     * Create a CloudRecognitionError with a TransactionError object
     * (for internal usage only)
     * @param transactionError
     */
    public CloudRecognitionError(TransactionError transactionError)
    {
        this(Errors.TRANSACTION, transactionError);
    }

    private CloudRecognitionError(int type, TransactionError transactionError)
    {
        Assert.assertEquals(type == Errors.TRANSACTION, transactionError != null);
        _type = type;
        _transactionError = transactionError;
    }

    /**
     * @return The type of the error, from {@link Errors}.
     */
    public int getType()
    {
        return _type;
    }

    /**
     * Get the transaction error, if there is one.
     *
     * @return The transaction error if the type is {@link Errors#TRANSACTION};
     *         null otherwise.
     */
    public TransactionError getTransactionError()
    {
        return _transactionError;
    }

    @Override
    public org.json.JSONObject toJSON()
    {
        JSONObject o = new JSONObject();
        o.tryPut(KEY_TYPE, _type);
        o.tryPut(KEY_TRANSACTION_ERROR, _transactionError);
        return o;
    }

    public static CloudRecognitionError createFromJSON(org.json.JSONObject json) throws JSONException
    {
        TransactionError tError = null;
        int type = json.getInt(KEY_TYPE);
        if (type == Errors.TRANSACTION)
            tError = TransactionError.createFromJSON(json.getJSONObject(KEY_TRANSACTION_ERROR));
        return new CloudRecognitionError(type, tError);
    }
}
