package com.nuance.dragon.toolkit.cloudservices.recognizer;

import org.json.JSONException;

import com.nuance.dragon.toolkit.cloudservices.TransactionResult;
import com.nuance.dragon.toolkit.core.data.Data;
import com.nuance.dragon.toolkit.oem.api.JSONCompliant;
import com.nuance.dragon.toolkit.oem.api.json.JSONObject;

/**
 * A cloud recognition result.
 */
public class CloudRecognitionResult implements JSONCompliant
{
    protected static final String   KEY_RESULT     = "res";

    private final TransactionResult _results;

    /**
     * Create a CloudRecognitionResult with a TransactionResult object
     * (for internal usage only)
     * @param results
     */
    public CloudRecognitionResult(TransactionResult results)
    {
        _results = results;
    }

    /**
     * @return the cloud recognition result type (never be null)
     */
    public String getType()
    {
        return _results.getResultType();
    }

    /**
     * @return the cloud recognition result format (never be null)
     */
    public String getFormat()
    {
        return _results.getResultFormat();
    }

    /**
     * @return the complete result in dictionary format (never null)
     */
    public Data.Dictionary getDictionary()
    {
        return _results.getContents();
    }

    @Override
    public org.json.JSONObject toJSON()
    {
        JSONObject o = new JSONObject();
        o.tryPut(KEY_RESULT, _results.toJSON());
        return o;
    }

    /**
     * Is this the final result, or are we expecting more?
     * @return true if there will be no more results.
     */
    public boolean isFinal()
    {
        return _results.isFinal();
    }
    
    /**
     * Returns whether the utterance is complete or not. This method
     * should only be caleld if the intermediate response mode is enabled
     * and set to "UtteranceDetectionWithPartialRecognition".
     * @return true if the utterance is complete
     */
    public boolean isUtteranceComplete(){
    	return _results.isUtteranceComplete();
    }

    public static CloudRecognitionResult createFromJSON(org.json.JSONObject json) throws JSONException
    {
        return new CloudRecognitionResult(TransactionResult.createFromJSON(json.getJSONObject(KEY_RESULT)));
    }
}
