package com.nuance.dragon.toolkit.core;

import java.util.Iterator;
import java.util.List;

import com.nuance.dragon.toolkit.oem.api.internal.Checker;

public class SimpleWordList extends WordList
{
    private class SimpleWordIterator implements WordIterator
    {
        private final Iterator<WordAction> _it;

        public SimpleWordIterator()
        {
            _it = _list.iterator();
        }

        @Override
        public boolean hasNext() {
            return _it.hasNext();
        }

        @Override
        public WordAction next() {
            return _it.next();
        }

        @Override
        public void remove() {
            //_it.remove();
        }

        @Override
        public void acceptChanges() {
        }

        @Override
        public void discardChanges() {
        }

        @Override
        public int getCount() {
            return _list.size();
        }

        @Override
        public int getChecksum() {
            return _checksum;
        }

    }

    private final List<WordAction> _list;
    private final boolean _clearAll;
    private final int _acceptedChecksum;
    private final int _checksum;

    public SimpleWordList(List<WordAction> list, boolean clearAll, int currentChecksum, int newChecksum)
    {
        Checker.checkArgForNull("list", list);
        _list = list;
        _clearAll = clearAll;
        _acceptedChecksum = currentChecksum;
        _checksum = newChecksum;
    }

    @Override
    public WordIterator getFullIterator() {
        return _clearAll ? new SimpleWordIterator() : null;
    }

    @Override
    public WordIterator getModifiedIterator() {
        return _clearAll ? null : new SimpleWordIterator();
    }

    @Override
    public boolean fullUpdateRequired() {
        return _clearAll;
    }

    @Override
    public int getAcceptedChecksum() {
        return _acceptedChecksum;
    }
}
