package com.nuance.dragon.toolkit.core.calllog;

import java.util.Map;

public interface SessionEvent {
    /**
     * Method to create a new session event builder as a child of the current session event.
     * @param eventName
     * @return SessionEventBuilder
     */
    SessionEventBuilder createChildEventBuilder(String eventName);

    /**
     * Method to create a new session event builder as a child of the current session event.
     *
     * @param eventName
     * @param schemaVersion The call log schema Version used by identifying the current session
     *                      event's schema, could be NULL or Empty string if there is no schema
     *                      version to use.
     * @return SessionEventBuilder
     */
    SessionEventBuilder createChildEventBuilder(String eventName, String schemaVersion);

    /**
     * Retrieve the session event Id. This id is the concatenation of the root parent id and
     * the sequence id of the event.
     *
     * @return session event id. (must not be NULL).
     */
    String getId();

    /**
     * Retrieve the session event name.
     *
     * @return session event name. (must not be NULL).
     */
    String getName();

    /**
     * Retrieve the remote reference ids and their associated event names that have been created
     * for this event.
     *
     * The returned the map object contains the list of name, id pairs. If the current session
     * event does not have any remote references. The map object will contain 0 elements.
     *
     * @return List of all the reference <name, id> pairs generated for this SessionEvent
     */
    Map<String, String> getRefIds();
}
