package com.nuance.dragon.toolkit.core.calllog;

public interface SessionEventBuilder {
    /**
     * Puts a binary value at the specified key.
     *
     * @param key
     *          The key where to store the value
     * @param value
     *          The value to add to the vector for the given key
     * @return This SessionEventBuilder if key-pair value is successfully, otherwise null.
     *         When null is return, the SessionEvenBuilder is obsoleted.
     */
    SessionEventBuilder putBinary(String key, byte[] value);

    /**
     * Puts a boolean value at the specified key.
     *
     * @param key
     *          The key where to store the value
     * @param value
     *          The value to add to the vector for the given key
     * @return This SessionEventBuilder if key-pair value is successfully, otherwise null.
     *         When null is return, the SessionEvenBuilder is obsoleted.
     */
    SessionEventBuilder putBoolean(String key, boolean value);

    /**
     * Puts a float value at the specified key.
     *
     * @param key
     *          The key where to store the value
     * @param value
     *          The value to add to the vector for the given key
     * @return This SessionEventBuilder if key-pair value is successfully, otherwise null.
     *         When null is return, the SessionEvenBuilder is obsoleted.
     */
    SessionEventBuilder putFloat(String key, double value);

    /**
     * Puts a integer value at the specified key.
     *
     * @param key
     *          The key where to store the value
     * @param value
     *          The value to add to the vector for the given key
     * @return This SessionEventBuilder if key-pair value is successfully, otherwise null.
     *         When null is return, the SessionEvenBuilder is obsoleted.
     */
    SessionEventBuilder putInteger(String key, int value);

    /**
     * Puts a long value at the specified key.
     *
     * @param key
     *          The key where to store the value
     * @param value
     *          The value to add to the vector for the given key
     * @return This SessionEventBuilder if key-pair value is successfully, otherwise null.
     *         When null is return, the SessionEvenBuilder is obsoleted.
     */
    SessionEventBuilder putLong(String key, long value);

    /**
     * Puts a string value at the specified key.
     *
     * @param key
     *          The key where to store the value
     * @param value
     *          The value to add to the vector for the given key
     * @return This SessionEventBuilder if key-pair value is successfully, otherwise null.
     *         When null is return, the SessionEvenBuilder is obsoleted.
     */
    SessionEventBuilder putString(String key, String value);

    /**
     * Put a remote event into the builder.
     *
     * @param eventName
     *          Name of the new remote event
     * @return This SessionEventBuilder, otherwise null.
     *         When null is return, session event is already committed.
     */
    SessionEventBuilder createRemoteEvent(String eventName);

    /**
     * Method to conclude the new session event and return the SessionEvent object.
     *
     * @return The committed SessionEvent object or null if the commitment fails, the current
     *         SessionEventBuilder is already committed or the current call log session has been
     *         closed already.
     */
    SessionEvent commit();
}
