package com.nuance.dragon.toolkit.core.data;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.nuance.dragon.toolkit.oem.api.JSONCompliant;
import com.nuance.dragon.toolkit.oem.api.Logger;
import com.nuance.dragon.toolkit.oem.api.internal.Checker;

/**
 * Represents a basic data type that is used by the {@link com.nuance.dragon.toolkit.cloudservices.CloudServices} and
 * other modules.
 */
public abstract class Data implements JSONCompliant, Serializable
{
    /**
     *
     */
    private static final long serialVersionUID = 1L;
    public static final short  TYPE_INT           = (short) (0xC0 & 0xFF); // 192 //com.nuance.nmsp.client.sdk.components.resource.nmas.NMASResource.PDX_DATA_TYPE_INT;
    public static final short  TYPE_BYTES         = (short) (0x04);        // 004 //com.nuance.nmsp.client.sdk.components.resource.nmas.NMASResource.PDX_DATA_TYPE_BYTES;
    public static final short  TYPE_UTF8          = (short) (0xFF & 0xC1); // 193 //com.nuance.nmsp.client.sdk.components.resource.nmas.NMASResource.PDX_DATA_TYPE_UTF8;
    public static final short  TYPE_ASCII         = (short) (0x16);        // 022 //com.nuance.nmsp.client.sdk.components.resource.nmas.NMASResource.PDX_DATA_TYPE_ASCII;
    public static final short  TYPE_NULL          = (short) (0x05);        // 005 //com.nuance.nmsp.client.sdk.components.resource.nmas.NMASResource.PDX_DATA_TYPE_NULL;
    public static final short  TYPE_DICT          = (short) (0xFF & 0xE0); // 224 //com.nuance.nmsp.client.sdk.components.resource.nmas.NMASResource.PDX_DATA_TYPE_DICT;
    public static final short  TYPE_SEQ           = (short) (0x10);               //com.nuance.nmsp.client.sdk.components.resource.nmas.NMASResource.PDX_DATA_TYPE_SEQ;
    public static final short  TYPE_STRING        = TYPE_UTF8;
    public static final short  TYPE_DOUBLE        = (short) (0x06);               //com.nuance.nmsp.client.sdk.components.resource.nmas.NMASResource.PDX_DATA_TYPE_DOUBLE;
    public static final short  TYPE_BOOLEAN       = (short) (0x07);               //com.nuance.nmsp.client.sdk.components.resource.nmas.NMASResource.PDX_DATA_TYPE_BOOLEAN;

    private static final java.lang.String KEY_TYPE = "type";
    private static final java.lang.String KEY_VALUE = "value";

    /**
     * An integer.
     */
    public static final class Integer extends Data
    {
        /**
         *
         */
        private static final long serialVersionUID = 1L;
        public final int value;

        /**
         * @param value The integer value.
         */
        public Integer(int value)
        {
            super(TYPE_INT);
            this.value = value;
        }

        public int getValue() {
            return this.value;
        }

        @Override
        public java.lang.String toString(java.lang.String indent)
        {
            return java.lang.String.valueOf(value);
        }

        @Override
        public int hashCode() {
            final int prime = 31;
            int result = super.hashCode();
            result = prime * result + value;
            return result;
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj)
                return true;
            if (!super.equals(obj))
                return false;
            if (getClass() != obj.getClass())
                return false;
            Integer other = (Integer) obj;
            if (value != other.value)
                return false;
            return true;
        }

        @Override
        protected void putInJSON(JSONObject o) throws JSONException
        {
            o.put(KEY_VALUE, value);
        }

        public static Integer createFromJSON(JSONObject o) throws JSONException
        {
            Checker.checkJsonForNull("o", o);
            return new Data.Integer(o.getInt(KEY_VALUE));
        }
    }

    /**
     * A boolean.
     */
    public static final class Boolean extends Data
    {
        /**
         *
         */
        private static final long serialVersionUID = 1L;
        public final boolean value;

        /**
         * @param value The boolean value.
         */
        public Boolean(boolean value)
        {
            super(TYPE_BOOLEAN);
            this.value = value;
        }

        public boolean getValue() {
            return this.value;
        }

        @Override
        public java.lang.String toString(java.lang.String indent)
        {
            return java.lang.String.valueOf(value);
        }

        @Override
        public int hashCode() {
            final int prime = 31;
            int result = super.hashCode();
            result = prime * result + new java.lang.Boolean(value).hashCode();
            return result;
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj)
                return true;
            if (!super.equals(obj))
                return false;
            if (getClass() != obj.getClass())
                return false;
            Boolean other = (Boolean) obj;
            if (value != other.value)
                return false;
            return true;
        }

        @Override
        protected void putInJSON(JSONObject o) throws JSONException
        {
            o.put(KEY_VALUE, value);
        }

        public static Boolean createFromJSON(JSONObject o) throws JSONException
        {
            Checker.checkJsonForNull("o", o);
            return new Data.Boolean(o.getBoolean(KEY_VALUE));
        }
    }

    /**
     * A double.
     */
    public static final class Double extends Data
    {
        /**
         *
         */
        private static final long serialVersionUID = 1L;
        public final double value;

        /**
         * @param value The double value.
         */
        public Double(double value)
        {
            super(TYPE_DOUBLE);
            this.value = value;
        }

        public double getValue() {
            return this.value;
        }

        @Override
        public java.lang.String toString(java.lang.String indent)
        {
            return java.lang.String.valueOf(value);
        }

        @Override
        public int hashCode() {
            final int prime = 31;
            int result = super.hashCode();
            result = prime * result + new java.lang.Double(value).hashCode();
            return result;
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj)
                return true;
            if (!super.equals(obj))
                return false;
            if (getClass() != obj.getClass())
                return false;
            Double other = (Double) obj;
            if (value != other.value)
                return false;
            return true;
        }

        @Override
        protected void putInJSON(JSONObject o) throws JSONException
        {
            o.put(KEY_VALUE, value);
        }

        public static Double createFromJSON(JSONObject o) throws JSONException
        {
            Checker.checkJsonForNull("o", o);
            return new Data.Double(o.getDouble(KEY_VALUE));
        }
    }

    /**
     * A byte array.
     */
    public static final class Bytes extends Data
    {
        /**
         *
         */
        private static final long serialVersionUID = 1L;
        public byte[] value;

        /**
         * @param bytes The byte-array value.
         */
        public Bytes(byte[] bytes)
        {
            super(TYPE_BYTES);
            Checker.checkArgForNull("bytes", bytes);
            value = new byte[bytes.length];
            System.arraycopy(bytes, 0, value, 0, bytes.length);
        }

        private static final char [] _hexMap = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
                                                'a', 'b', 'c', 'd', 'e', 'f'};
        @Override
        public java.lang.String toString(java.lang.String indent)
        {

            StringBuilder builder = new StringBuilder();
            builder.append("0x");

            for (byte b : value)
            {
                byte high =(byte) ((b >> 4) & 0xf);;
                byte low = (byte) (b & 0xf);

                builder.append(_hexMap[high]);
                builder.append(_hexMap[low]);
            }
            return builder.toString();
        }

        @Override
        public int hashCode() {
            final int prime = 31;
            int result = super.hashCode();
            result = prime * result + Arrays.hashCode(value);
            return result;
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj)
                return true;
            if (!super.equals(obj))
                return false;
            if (getClass() != obj.getClass())
                return false;
            Bytes other = (Bytes) obj;
            if (!Arrays.equals(value, other.value))
                return false;
            return true;
        }

        @Override
        protected void putInJSON(JSONObject o) throws JSONException
        {
            // This is ugly. We could use Base64, but it's only available on OS 2.2
            // and greater. (TODO)
            StringBuilder sb = new StringBuilder();
            boolean first = true;
            for (byte b : value)
            {
                if (!first)
                    sb.append(',');

                sb.append(b);
                first = false;
            }

            o.put(KEY_VALUE, sb.toString());
        }

        public static Bytes createFromJSON(JSONObject o) throws JSONException
        {
            Checker.checkJsonForNull("o", o);
            java.lang.String byteString = o.getString(KEY_VALUE);
            if (byteString.length() == 0) {
                return new Bytes(new byte[]{}); // An empty byte array.
            }
            java.lang.String [] byteStrings = byteString.split(",");
            byte [] bytes = new byte[byteStrings.length];

            for (int i = 0; i < bytes.length; i++)
            {
                bytes[i] = Byte.parseByte(byteStrings[i]);
            }
            return new Bytes(bytes);
        }
    }

    /**
     * A string (UTF8).
     */
    public static final class String extends Data
    {
        /**
         *
         */
        private static final long serialVersionUID = 1L;
        public java.lang.String value;

        /**
         * @param value The string value.
         */
        public String(java.lang.String value)
        {
            super(TYPE_STRING);
            Checker.checkArgForNull("value", value);
            this.value = value;
        }

        @Override
        public java.lang.String toString(java.lang.String indent)
        {
            return value;
        }

        @Override
        public int hashCode() {
            final int prime = 31;
            int result = super.hashCode();
            result = prime * result + ((value == null) ? 0 : value.hashCode());
            return result;
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj)
                return true;
            if (!super.equals(obj))
                return false;
            if (getClass() != obj.getClass())
                return false;
            String other = (String) obj;
            if (value == null) {
                if (other.value != null)
                    return false;
            } else if (!value.equals(other.value))
                return false;
            return true;
        }

        @Override
        protected void putInJSON(JSONObject o) throws JSONException
        {
            o.put(KEY_VALUE, value);
        }

        public static String createFromJSON(JSONObject o) throws JSONException
        {
            Checker.checkJsonForNull("o", o);
            return new String(o.getString(KEY_VALUE));
        }
    }

    /**
     * A null value.
     */
    public static final class Null extends Data
    {
        /**
         *
         */
        private static final long serialVersionUID = 1L;

        public Null()
        {
            super(TYPE_NULL);
        }

        @Override
        public java.lang.String toString(java.lang.String indent)
        {
            return "[null]";
        }

        @Override
        protected void putInJSON(JSONObject o) throws JSONException
        {
        }
    }

    /**
     * A dictionary.
     */
    public static final class Dictionary extends Data
    {
        /**
         *
         */
        private static final long serialVersionUID = 1L;
        private final Map<java.lang.String, Data> _values;

        public Dictionary()
        {
            this(null);
        }

        /**
         * @param values The initial values for the dictionary.
         */
        public Dictionary(Map<java.lang.String, Data> values)
        {
            super(TYPE_DICT);
            _values = new LinkedHashMap<java.lang.String, Data>();
            if (values != null)
                _values.putAll(values);
        }

        public void put(java.lang.String key, Data value)
        {
            Checker.checkArgForNull("key", key);
            _values.put(key, value);
        }

        public void put(java.lang.String key, java.lang.String value)
        {
            Checker.checkArgForNull("key", key);
            _values.put(key, new String(value));
        }

        public void put(java.lang.String key, int value)
        {
            Checker.checkArgForNull("key", key);
            _values.put(key, new  Data.Integer(value));
        }

        public void put(java.lang.String key, byte[] value)
        {
            Checker.checkArgForNull("key", key);
            _values.put(key, new Bytes(value));
        }

        public void putAll(Dictionary dict)
        {
            _values.putAll(dict._values);
        }

        public Set<Map.Entry<java.lang.String, Data>> getEntries()
        {
            return _values.entrySet();
        }

        /**
         * @param key
         * @return null if not found
         */
        public Data get(java.lang.String key)
        {
            return _values.get(key);
        }

        /**
         * @param key
         * @return The bytes data, or null if the key isn't found or isn't bytes
         */
        public Data.Bytes getBytes(java.lang.String key)
        {
            Data d = get(key);
            if (d != null && d._type == Data.TYPE_BYTES)
                return (Data.Bytes)d;
            return null;
        }

        /**
         * @param key
         * @return The integer data, or null if the key isn't found or isn't an integer
         */
        public Data.Integer getInt(java.lang.String key)
        {
            Data d = get(key);
            if (d != null && d._type == Data.TYPE_INT)
                return (Data.Integer)d;
            return null;
        }

        /**
         * @param key
         * @return The boolean data, or null if the key isn't found or isn't a boolean
         */
        public Data.Boolean getBoolean(java.lang.String key)
        {
            Data d = get(key);
            if (d != null && d._type == Data.TYPE_BOOLEAN)
                return (Data.Boolean)d;
            return null;
        }

        /**
         * @param key
         * @return The double data, or null if the key isn't found or isn't a double
         */
        public Data.Double getDouble(java.lang.String key)
        {
            Data d = get(key);
            if (d != null && d._type == Data.TYPE_DOUBLE)
                return (Data.Double)d;
            return null;
        }

        /**
         * @param key
         * @return The string data, or null if the key isn't found or isn't a string
         */
        public Data.String getString(java.lang.String key)
        {
            Data d = get(key);
            if (d != null && d._type == Data.TYPE_STRING)
                return (Data.String)d;
            return null;
        }

        /**
         * @param key
         * @return The sequence data, or null if the key isn't found or isn't a sequence
         */
        public Data.Sequence getSequence(java.lang.String key)
        {
            Data d = get(key);
            if (d != null && d._type == Data.TYPE_SEQ)
                return (Data.Sequence)d;
            return null;
        }

        /**
         * @param key
         * @return The dictionary data, or null if the key isn't found or isn't a dictionary
         */
        public Data.Dictionary getDictionary(java.lang.String key)
        {
            Data d = get(key);
            if (d != null && d._type == Data.TYPE_DICT)
                return (Data.Dictionary)d;
            return null;
        }

        @Override
        protected java.lang.String toString(java.lang.String indent)
        {
            StringBuilder builder = new StringBuilder();
            builder.append("{\n");

            java.lang.String itemIndent = indent + "  ";
            Set<Map.Entry<java.lang.String, Data>> entries = _values.entrySet();
            for (Map.Entry<java.lang.String, Data> entry : entries)
            {
                builder.append(itemIndent);
                builder.append(entry.getKey());
                builder.append(" : ");

                builder.append(entry.getValue().toString(itemIndent + "  "));
                builder.append(",\n");
            }

            builder.append(indent);
            builder.append("}");
            return builder.toString();
        }

        @Override
        public int hashCode() {
            final int prime = 31;
            int result = super.hashCode();
            result = prime * result
                    + ((_values == null) ? 0 : _values.hashCode());
            return result;
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj)
                return true;
            if (!super.equals(obj))
                return false;
            if (getClass() != obj.getClass())
                return false;
            Dictionary other = (Dictionary) obj;
            if (_values == null) {
                if (other._values != null)
                    return false;
            } else if (!_values.equals(other._values))
                return false;
            return true;
        }

        public void putInJSON(JSONObject o, boolean insertPdxTypes) throws JSONException {
            JSONObject jsonDict = new JSONObject();
            if (insertPdxTypes) {
                try
                {
                    for (Entry<java.lang.String, Data> entry : _values.entrySet())
                    {
                        jsonDict.put(entry.getKey(), entry.getValue().toJSON());
                    }
                    o.put(KEY_VALUE, jsonDict);
                }
                catch (JSONException e)
                {
                }
            } else {
                try
                {
                    for (Entry<java.lang.String, Data> entry : _values.entrySet())
                    {
                        short dataType = entry.getValue().getType();
                        switch(dataType)
                        {
                            case Data.TYPE_STRING:
                            case Data.TYPE_ASCII:
                                o.put(entry.getKey(), entry.getValue().toString());
                                break;
                            case Data.TYPE_DOUBLE:
                                o.put(entry.getKey(), ((Data.Double)entry.getValue()).getValue());
                                break;
                            case Data.TYPE_INT:
                                o.put(entry.getKey(), ((Data.Integer)entry.getValue()).getValue());
                                break;
                            case Data.TYPE_DICT:
                                JSONObject tempDictionary = new JSONObject();
                                ((Data.Dictionary)entry.getValue()).putInJSON(tempDictionary, false);
                                o.put(entry.getKey(), tempDictionary);
                                break;
                            case Data.TYPE_NULL:
                                o.put(entry.getKey(), null);
                                break;
                            case Data.TYPE_BOOLEAN:
                                o.put(entry.getKey(), ((Data.Boolean)entry.getValue()).getValue());
                                break;
                            case Data.TYPE_SEQ:
                                JSONArray tempArray = new JSONArray();
                                ((Data.Sequence)entry.getValue()).putInJSONArray(tempArray);
                                o.put(entry.getKey(), tempArray);
                                break;
                            case Data.TYPE_BYTES:
                                break;
                            default:
                                // TODO: Error, unknown type.
                                break;
                        }

                    }

                }
                catch (JSONException e)
                {
                }
            }
        }

        @Override
        protected void putInJSON(JSONObject o) throws JSONException
        {
            putInJSON(o, true);
        }

        public static Dictionary createFromJSON(JSONObject o) throws JSONException
        {
            Checker.checkJsonForNull("o", o);
            JSONObject jsonDict = o.getJSONObject(KEY_VALUE);
            JSONArray names = jsonDict.names();
            Dictionary ret = new Dictionary();
            if (names != null)
            {
                for (int i = 0; i < names.length(); i++)
                {
                    java.lang.String name = names.getString(i);
                    JSONObject item = jsonDict.getJSONObject(name);
                    ret.put(name, Data.createFromJSON(item));
                }
            }
            return ret;
        }
    }

    /**
     * A sequence.
     */
    public static final class Sequence extends Data
    {
        /**
         *
         */
        private static final long serialVersionUID = 1L;
        private final List<Data> _values;

        public Sequence()
        {
            this(null);
        }

        /**
         * @param values The initial values for the sequence.
         */
        public Sequence(List<Data> values)
        {
            super(TYPE_SEQ);
            _values = new ArrayList<Data>();
            if (values != null)
                _values.addAll(values);
        }

        public void add(Data value)
        {
            _values.add(value);
        }

        public void add(java.lang.String value)
        {
            _values.add(new String(value));
        }

        public void add(int value)
        {
            _values.add(new  Data.Integer(value));
        }

        public void add(byte[] value)
        {
            _values.add(new Bytes(value));
        }

        public List<Data> getValues()
        {
            return _values;
        }

        /**
         * @param index
         * @return The data at the given index
         */
        public Data get(int index)
        {
            return _values.get(index);
        }

        /**
         * @param index
         * @return The bytes data, or null if not bytes
         */
        public Data.Bytes getBytes(int index)
        {
            Data d = _values.get(index);
            if (d._type == Data.TYPE_BYTES)
                return (Data.Bytes)d;
            return null;
        }

        /**
         * @param index
         * @return The integer data, or null if not an integer
         */
        public Data.Integer getInt(int index)
        {
            Data d = _values.get(index);
            if (d._type == Data.TYPE_INT)
                return (Data.Integer)d;
            return null;
        }

        /**
         * @param index
         * @return The boolean data, or null if not an boolean
         */
        public Data.Boolean getBoolean(int index)
        {
            Data d = _values.get(index);
            if (d._type == Data.TYPE_BOOLEAN)
                return (Data.Boolean)d;
            return null;
        }

        /**
         * @param index
         * @return The double data, or null if not an double
         */
        public Data.Double getDouble(int index)
        {
            Data d = _values.get(index);
            if (d._type == Data.TYPE_DOUBLE)
                return (Data.Double)d;
            return null;
        }

        /**
         * @param index
         * @return The string data, or null if not a string
         */
        public Data.String getString(int index)
        {
            Data d = _values.get(index);
            if (d._type == Data.TYPE_STRING)
                return (Data.String)d;
            return null;
        }

        /**
         * @param index
         * @return The sequence data, or null if not a sequence
         */
        public Data.Sequence getSequence(int index)
        {
            Data d = _values.get(index);
            if (d._type == Data.TYPE_SEQ)
                return (Data.Sequence)d;
            return null;
        }

        /**
         * @param index
         * @return The dictinoary data, or null if not a dictionary
         */
        public Data.Dictionary getDictionary(int index)
        {
            Data d = _values.get(index);
            if (d._type == Data.TYPE_DICT)
                return (Data.Dictionary)d;
            return null;
        }

        public int size()
        {
            return _values.size();
        }

        @Override
        public java.lang.String toString(java.lang.String indent)
        {
            StringBuilder builder = new StringBuilder();
            builder.append("[\n");

            java.lang.String itemIndent = indent + "  ";
            for (Data val : _values)
            {
                builder.append(itemIndent);
                builder.append(val.toString(itemIndent));
                builder.append(",\n");
            }

            builder.append(indent);
            builder.append("]");
            return builder.toString();
        }

        @Override
        public int hashCode() {
            final int prime = 31;
            int result = super.hashCode();
            result = prime * result
                    + ((_values == null) ? 0 : _values.hashCode());
            return result;
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj)
                return true;
            if (!super.equals(obj))
                return false;
            if (getClass() != obj.getClass())
                return false;
            Sequence other = (Sequence) obj;
            if (_values == null) {
                if (other._values != null)
                    return false;
            } else if (!_values.equals(other._values))
                return false;
            return true;
        }

        public void putInJSONArray(JSONArray o) throws JSONException {
            for (Data d : _values) {
                short dataType = d.getType();
                switch(dataType)
                {
                    case Data.TYPE_STRING:
                    case Data.TYPE_ASCII:
                        o.put(d.toString());
                        break;
                    case Data.TYPE_DOUBLE:
                        o.put(((Data.Double)d).getValue());
                        break;
                    case Data.TYPE_INT:
                        o.put(((Data.Integer)d).getValue());
                        break;
                    case Data.TYPE_DICT:
                        JSONObject tempDictionary = new JSONObject();
                        ((Data.Dictionary)d).putInJSON(tempDictionary, false);
                        o.put(tempDictionary);
                        break;
                    case Data.TYPE_NULL:
                        o.put(null);
                        break;
                    case Data.TYPE_BOOLEAN:
                        o.put(((Data.Boolean)d).getValue());
                        break;
                    case Data.TYPE_SEQ:
                        JSONArray tempArray = new JSONArray();
                        ((Data.Sequence)d).putInJSONArray(tempArray);
                        o.put(tempArray);
                        break;
                    case Data.TYPE_BYTES:
                        break;
                    default:
                        // TODO: Error, unknown type.
                        break;
                }
            }
        }

        @Override
        protected void putInJSON(JSONObject o) throws JSONException
        {
            JSONArray a = new JSONArray();
            for (Data d : _values)
            {
                a.put(d.toJSON());
            }
            o.put(KEY_VALUE, a);
        }

        public static Sequence createFromJSON(JSONObject o) throws JSONException
        {
            Checker.checkJsonForNull("o", o);
            JSONArray a = o.getJSONArray(KEY_VALUE);
            ArrayList<Data> values = new ArrayList<Data>(a.length());
            for (int i = 0; i < a.length(); i++)
            {
                values.add(Data.createFromJSON(a.getJSONObject(i)));
            }
            return new Sequence(values);
        }
    }

    private short _type;

    Data(short type)
    {
        _type = type;
    }

    public final short getType()
    {
        return _type;
    }

    @Override
    public final java.lang.String toString()
    {
        return toString("");
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + _type;
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        Data other = (Data) obj;
        if (_type != other._type)
            return false;
        return true;
    }

    protected abstract java.lang.String toString(java.lang.String indent);

    protected abstract void putInJSON(JSONObject o) throws JSONException;

    @Override
    public JSONObject toJSON()
    {
        JSONObject o = new JSONObject();
        try
        {
            putInJSON(o);
            o.put(KEY_TYPE, (short)_type);
        }
        catch (JSONException e)
        {
            Logger.error(this, "Error adding to JSON", e);
        }
        return o;
    }

    public static Data createFromJSON(JSONObject json) throws JSONException
    {
        Checker.checkJsonForNull("json", json);
        int type = json.getInt(KEY_TYPE);
        Data data = null;
        switch(type)
        {
            case TYPE_INT:
                data = Integer.createFromJSON(json);
                break;
            case TYPE_UTF8:
            case TYPE_ASCII:
                data = String.createFromJSON(json);
                break;
            case TYPE_BYTES:
                data = Bytes.createFromJSON(json);
                break;
            case TYPE_DICT:
                data = Dictionary.createFromJSON(json);
                break;
            case TYPE_SEQ:
                data = Sequence.createFromJSON(json);
                break;
            default:
                data = new Null();
                break;
        }
        return data;
    }
}
