package com.nuance.dragon.toolkit.edr;

/**
 * Represents a recognition result.
 */
public abstract class EdrResult {

    /**
     * Returns the choice count for a given utterance.
     * @return the choice count for a given utterance.
     */
    public abstract int getChoiceCount();

    /**
     * Returns a human readable string that represents a complete recognition transcription.
     * Multiple choices are accessible and are ordered from the most probable (at index 0) to the least probable recognition transcription.
     * @param index index of the chosen choice. The top choice can be accessed from index 0.
     * @return a human readable string that represents a complete recognition transcription.
     * @throws IndexOutOfBoundsException
     */
    public abstract String getChoiceAtIndex(int index) throws IndexOutOfBoundsException;

    /**
     * Returns true if there is no more results to report.
     * Note that it is important to wait for this boolean to be true before performing
     * a new recognition session except if the recognition has been cancelled.
     * @return true if there is no more results to report and the recognizer is ready to perform a new recognition session.
     */
    public abstract boolean isAtEndOfRecognition();
}
