package com.nuance.dragon.toolkit.edr;

import com.nuance.dragon.toolkit.edr.internal.WordSetContactImpl;
import com.nuance.dragon.toolkit.edr.internal.WordSetImpl;
import com.nuance.dragon.toolkit.grammar.content.ContentManager;
import com.nuance.dragon.toolkit.grammar.content.StructuredContactManager;
import com.nuance.dragon.toolkit.oem.api.FileManager;
import com.nuance.dragon.toolkit.oem.api.internal.Checker;

/**
 * Represents a set of words that are contextually bound.
 */
public abstract class WordSet {

    /**
     * Returns the associated id.
     * @return the associated id.
     */
    public abstract String getId();

    /**
     * Returns a WordSet compatible object.
     * This default WordSet type can be used to enrich the default language model vocabulary.
     * @param id a unique identification string that will be associated with the returned WordSet.
     * @param contentManager The content manager used to carry the word list.
     * @param fileManager The file manager used to store the content list cache.
     * @return a WordSet compatible object.
     */
    public static WordSet createWordSet(String id, ContentManager contentManager, FileManager fileManager) {
        Checker.checkArgForNull("id", id);
        Checker.checkArgForNull("contentManager", contentManager);
        Checker.checkArgForNull("fileManager", fileManager);

        return new WordSetImpl(id, contentManager, fileManager);
    }

    /**
     * Returns a WordSet compatible object.
     * This word set object should be used when using a contact list as a word set.
     * @param id a unique identification string that will be associated with the returned WordSet.
     * @param contentManager The content manager used to carry the word list.
     * @param fileManager The file manager used to store the content list cache.
     * @return a WordSet compatible object.
     */
    public static WordSet createWordSetContact(String id, StructuredContactManager contentManager, FileManager fileManager) {
        Checker.checkArgForNull("id", id);
        Checker.checkArgForNull("contentManager", contentManager);
        Checker.checkArgForNull("fileManager", fileManager);

        return new WordSetContactImpl(id, contentManager, fileManager);
    }
}
