package com.nuance.dragon.toolkit.edr.internal.jni;

public class DictationRecognizer {
  private long javaCPtr;
  protected boolean javaCMemOwn;

  protected DictationRecognizer(long cPtr, boolean cMemoryOwn) {
    javaCMemOwn = cMemoryOwn;
    javaCPtr = cPtr;
  }

  protected static long getCPtr(DictationRecognizer obj) {
    return (obj == null) ? 0 : obj.javaCPtr;
  }

  public synchronized void delete() 
  {
    if (javaCPtr != 0) {
      if (javaCMemOwn) {
        this.destroy();
        javaCMemOwn = false;
      }
      javaCPtr = 0;
    }
  }


  public DictationRecognizer() {
    javaCMemOwn = false;
    javaCPtr = 0;
    edr.createDictationRecognizer(this);
  }
  
  protected synchronized void setCPtr(long cPtr) {
    this.delete();
    if(cPtr != 0) {
      javaCPtr = cPtr;
      javaCMemOwn = true;
    }
  }
  
  protected synchronized void setCPtrNotOwn(long cPtr) {
    this.delete();
    if(cPtr != 0) {
      javaCPtr = cPtr;
      javaCMemOwn = false;
    }
  }
  
  protected void finalize() {
    this.delete();
  }

  private EDRReturnCode destroy() { 
    return EDRReturnCode.intToEnum(edrJNI.DictationRecognizer_destroy(javaCPtr, this)); 
  }

  public EDRReturnCode init(StringMap initialSettings) { 
    return EDRReturnCode.intToEnum(edrJNI.DictationRecognizer_init(javaCPtr, this, StringMap.getCPtr(initialSettings), initialSettings)); 
  }

  public EDRReturnCode finish() { 
    return EDRReturnCode.intToEnum(edrJNI.DictationRecognizer_finish(javaCPtr, this)); 
  }

  public EDRReturnCode startRecognition(IDictationRecognizerListener callback) { 
    return EDRReturnCode.intToEnum(edrJNI.DictationRecognizer_startRecognition__JNI_0(javaCPtr, this, callback)); 
  }

  public EDRReturnCode startRecognition(StringMap recognitionSettings, IDictationRecognizerListener callback) { 
    return EDRReturnCode.intToEnum(edrJNI.DictationRecognizer_startRecognition__JNI_1(javaCPtr, this, StringMap.getCPtr(recognitionSettings), recognitionSettings, callback)); 
  }

  public EDRReturnCode stopRecognition() { 
    return EDRReturnCode.intToEnum(edrJNI.DictationRecognizer_stopRecognition(javaCPtr, this)); 
  }

  public EDRReturnCode cancelRecognition() { 
    return EDRReturnCode.intToEnum(edrJNI.DictationRecognizer_cancelRecognition(javaCPtr, this)); 
  }

  public EDRReturnCode pushAudio(byte[] audioBuffer) { 
    return EDRReturnCode.intToEnum(edrJNI.DictationRecognizer_pushAudio(javaCPtr, this, audioBuffer)); 
  }

  public EDRReturnCode resetSpeakerProfile() { 
    return EDRReturnCode.intToEnum(edrJNI.DictationRecognizer_resetSpeakerProfile(javaCPtr, this)); 
  }

  public EDRReturnCode registerWordSet(WordSet content) { 
    return EDRReturnCode.intToEnum(edrJNI.DictationRecognizer_registerWordSet(javaCPtr, this, WordSet.getCPtr(content), content)); 
  }

  public EDRReturnCode unregisterWordSet(String id) { 
    return EDRReturnCode.intToEnum(edrJNI.DictationRecognizer_unregisterWordSet(javaCPtr, this, id)); 
  }

  public EDRReturnCode getRegisteredWordSets(StringArray ids) { 
    return EDRReturnCode.intToEnum(edrJNI.DictationRecognizer_getRegisteredWordSets(javaCPtr, this, StringArray.getCPtr(ids), ids)); 
  }

  public EDRReturnCode buildWordSets(StringArray ids) { 
    return EDRReturnCode.intToEnum(edrJNI.DictationRecognizer_buildWordSets(javaCPtr, this, StringArray.getCPtr(ids), ids)); 
  }

  public EDRReturnCode unbuildWordSets() { 
    return EDRReturnCode.intToEnum(edrJNI.DictationRecognizer_unbuildWordSets(javaCPtr, this)); 
  }

  public EDRReturnCode sleep() { 
    return EDRReturnCode.intToEnum(edrJNI.DictationRecognizer_sleep(javaCPtr, this)); 
  }

  public EDRReturnCode wake() { 
    return EDRReturnCode.intToEnum(edrJNI.DictationRecognizer_wake(javaCPtr, this)); 
  }

}
