package com.nuance.dragon.toolkit.elvis;

import org.json.JSONException;

import com.nuance.dragon.toolkit.oem.api.JSONCompliant;
import com.nuance.dragon.toolkit.oem.api.json.JSONObject;


/**
 * An Elvis Adaptation Result.
 */
public class ElvisAdaptationResult implements JSONCompliant
{
    private static final String KEY_STATUS = "status";
    private static final String KEY_MISCLOGGING = "misclogging";

    private final boolean _status;
    private final String _miscLogging;

    ElvisAdaptationResult(boolean status, String miscLogging)
    {
        _status = status;
        _miscLogging = miscLogging;
    }

    /**
     * Get the result status
     * @return success or not
     */
    public boolean getStatus()
    {
        return _status;
    }

    /**
     * Returns the result log to be eventually sent to the server for data mining purposes.
     * @return The log buffer. null if not available.
     */
    public String getMiscLogging()
    {
        return _miscLogging;
    }

    @Override
    public org.json.JSONObject toJSON()
    {
        JSONObject o = new JSONObject();
        o.tryPut(KEY_STATUS, _status);
        o.tryPut(KEY_MISCLOGGING, _miscLogging);
        return o;
    }

    // TODO never null?
    public static ElvisAdaptationResult createFromJSON(org.json.JSONObject json) throws JSONException
    {
        boolean status = json.getBoolean(KEY_STATUS);
        String miscLogging = json.getString(KEY_MISCLOGGING);
        return new ElvisAdaptationResult(status, miscLogging);
    }

    @Override
    public String toString()
    {
        return _status ? "true" : "false";
    }
}
