package com.nuance.dragon.toolkit.elvis;

import org.json.JSONException;

import com.nuance.dragon.toolkit.oem.api.JSONCompliant;
import com.nuance.dragon.toolkit.oem.api.json.JSONObject;

/**
 * An Elvis error.
 */
public class ElvisError implements JSONCompliant
{
    /**
     * Possible reasons for Elvis errors.
     */
    public static class Reason
    {
        public static final int CANCELED              = 0;
        public static final int AUDIO_FRAMES_DROPPED  = 1;
        public static final int NO_RESULT             = 2;
        public static final int CANT_START            = 3;
        public static final int TRANSITION_DEF_ERROR  = 4;
        public static final int CONSTRAINT_DEF_ERROR  = 5;
        public static final int WORDSLOT_DEF_ERROR    = 6;
        public static final int FILE_ERROR            = 7;
        public static final int MISSING_WORDLIST      = 8;
        public static final int NOT_INITIALIZED       = 9;

        private static final String [] REASON_TEXTS = {"Recognition canceled",
                                                       "Audio frames dropped",
                                                       "No recognition result",
                                                       "Can't start",
                                                       "Transition definition error",
                                                       "Constraint definition error",
                                                       "WordSlot definition error",
                                                       "File error",
                                                       "Missing wordlist",
                                                       "Elvis is not initialized."};
    }

    private static final String KEY_CODE = "code";

    private final int _code;
    private final String _reason;

    ElvisError(int code)
    {
        _code = code;
        _reason = Reason.REASON_TEXTS[code];
    }

    /**
     * Get the reason code, from {@link Reason}.
     * @return The reason code
     */
    public int getReasonCode()
    {
        return _code;
    }

    /**
     * @return A description corresponding with the reason code
     */
    public String getReason()
    {
        return _reason;
    }

    @Override
    public org.json.JSONObject toJSON()
    {
        JSONObject o = new JSONObject();
        o.tryPut(KEY_CODE, _code);
        return o;
    }

    // TODO never null?
    public static ElvisError createFromJSON(org.json.JSONObject json) throws JSONException
    {
        int code = json.getInt(KEY_CODE);
        if (code < 0 || code >= Reason.REASON_TEXTS.length)
            throw new JSONException("Invalid code");
        return new ElvisError(code);
    }

    @Override
    public String toString()
    {
        return _reason;
    }
}
