package com.nuance.dragon.toolkit.file;

import java.io.File;
import java.io.FileDescriptor;
import java.io.IOException;

import android.content.res.AssetFileDescriptor;

import com.nuance.dragon.toolkit.oem.api.Logger;

/**
 * A file object that can be used from JNI code.
 */
public final class FileJni {

    private final File _file;
    private AssetFileDescriptor _fd;

    // We don't bother with the AssetFileDescriptor for internal/external files
    // (we just need the full file name).
    public FileJni(File file) {
        _fd = null;
        _file = file;
    }

    // We only bother with the AssetFileDescriptor for asset files.
    public FileJni(AssetFileDescriptor fd) {
        _fd = fd;
        _file = null;
    }

    public FileDescriptor getFD() {
        FileDescriptor fd = null;
        if (_fd != null) {
            fd = _fd.getFileDescriptor();
        }
        return fd;
    }

    public String getFileName() {
        String fileName = null;
        if (_file != null) {
            fileName = _file.getAbsolutePath();
        }
        return fileName;
    }

    /**
     * @return The start offset for this file.
     */
    public int getStartOffset() {
        int offset = 0;
        if (_fd != null) {
            offset = (int) _fd.getStartOffset();
        }
        return offset;
    }

    /**
     * @return The declared length for this file. Not accurate for normal files!
     */
    public int getLength() {
        int length = -1;
        if (_fd != null) {
            length = (int) _fd.getDeclaredLength();
        }
        return length;
    }

    // TODO What happens if already closed?
    public void close() {
        if (_fd != null) {
            try {
                _fd.close();
            } catch (IOException e) {
                // ignore
                Logger.error(this, "Error closing file", e);
            }
            _fd = null;
        }
    }
}
