package com.nuance.dragon.toolkit.grammar;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.json.JSONException;

import com.nuance.dragon.toolkit.cloudservices.recognizer.AsrSpec;
import com.nuance.dragon.toolkit.vocon.VoconContext;

/**
 * Defines a recognition scenario that uses {@link com.nuance.dragon.toolkit.elvis.ElvisRecognizer Elvis}
 * and {@link com.nuance.dragon.toolkit.cloudservices.CloudServices CloudServices}.
 */
public class NcsScenario extends Scenario
{
    public NcsScenario(String name,
                       List<String> elvisConstraints,
                       List<VoconContext> voconContexts,
                       AsrSpec asrSpec) {
        super(name, elvisConstraints, voconContexts, asrSpec);
    }
    
    protected NcsScenario(Scenario s)
    {
        super(s._name, s._elvisConstraints, s._voconContexts, s._asrSpec);
    }
    
    /**
     * Use this function to determine the IDs of the word lists required by this object, if any.
     *
     * @return A list of word list IDs that are required by this object.
     * Will be an empty list (never null) if no word lists are required.
     */
    @Override
    public Set<String> getRequiredWordListIds() {
        if (_asrSpec != null) {
            Set<String> requiredWordListIds = _asrSpec.getRequiredCustomWordListIds();
            return requiredWordListIds;
        }
        return new HashSet<String>();
    }
    
    /**
     * Use this function to determine the category of a word list.
     *
     * @param id The word list ID.
     * @return The category of the word list.
     */
    @Override
    public String getWordListCategory(String id) {
        return "Not Supported!!!";
    }
    
    public static NcsScenario createFromJSON(org.json.JSONObject json) throws JSONException
    {
        return new NcsScenario(Scenario.createFromJSON(json));
    }
}
