package com.nuance.dragon.toolkit.grammar.content;

import android.content.Context;
import android.database.Cursor;
import android.net.Uri;

import com.nuance.dragon.toolkit.core.Word;
import com.nuance.dragon.toolkit.core.WordAction;
import com.nuance.dragon.toolkit.oem.api.FileManager;

/**
 * A {@link ContentManager} that provides the full list of songs in the
 * device's music library.
 */
public class SongManager extends UriContentManager
{
    private final static Uri URI = android.provider.MediaStore.Audio.Media.EXTERNAL_CONTENT_URI;
    private static final String[] PROJECTION =  {
        android.provider.MediaStore.MediaColumns.TITLE,
        android.provider.MediaStore.Audio.AudioColumns.IS_MUSIC
    };

    /**
     * Like {@link #SongManager(String, FileManager, boolean, Context)} using auto
     * content sync-up without resetting the cache file.
     */
    public SongManager(String fileName, FileManager fileManager, Context context)
    {
        this(fileName, fileManager, false, false, context);
    }

    /**
     * Like {@link #SongManager(String, FileManager, boolean, Context)} without
     * resetting the cache file.
     */
    public SongManager(String fileName, FileManager fileManager, boolean manualPlatformFullUpdate, Context context)
    {
        this(fileName, fileManager, false, manualPlatformFullUpdate, context);
    }

    /**
     * @param fileName The name of the file to store the full content list cache
     * @param fileManager The file manager used to store the content list cache
     * @param resetFile True if the cache file should be reset
     * @param manualPlatformFullUpdate True if the auto content sync-up with
     * platform is not required. Therefore, {@link #forceRefresh()} needs to be
     * triggered manually to force a sync-up
     * @param context The app context
     */
    public SongManager(String fileName, FileManager fileManager, boolean resetFile, boolean manualPlatformFullUpdate, Context context)
    {
        super(fileName, fileManager, URI, resetFile, manualPlatformFullUpdate, context);
    }

    @Override
    protected PlatformFullIterator getPlatformFullIterator()
    {
        return new UriPlatformFullIterator(_context, _uri, PROJECTION)
        {
            private int colTitle;
            private int colIsMusic;

            @Override
            protected void init(Cursor cursor)
            {
                colTitle   = cursor.getColumnIndex(PROJECTION[0]);
                colIsMusic = cursor.getColumnIndex(PROJECTION[1]);
            }

            @Override
            protected WordAction getNext(Cursor cursor)
            {
                while (cursor.moveToNext()) {
                    String name = cursor.getString(colTitle);
                    int isMusic = cursor.getInt(colIsMusic);

                    if (name != null && name.length() == 0)
                        name = null;
                    if (name == null || isMusic == 0) continue;

                    return new WordAction(new Word(name, null), true);
                }
                return null;
            }
        };
    }
}
