package com.nuance.dragon.toolkit.hybrid;

import java.util.List;

import com.nuance.dragon.toolkit.cloudservices.recognizer.CloudRecognitionError;
import com.nuance.dragon.toolkit.cloudservices.recognizer.CloudRecognitionResult;
import com.nuance.dragon.toolkit.recognition.InterpretException;
import com.nuance.dragon.toolkit.recognition.LocalRecognitionResult;

/**
 * A hybrid recognition composite result (i.e. it aggregates VoCon, Elvis and cloud results).
 */
public class CompositeRecognitionResult extends HybridRecognitionResult
{
    private final LocalRecognitionResult _rawLocalResult;
    private final List<CloudRecognitionResult> _rawRemoteResults;
    private final CloudRecognitionError _eventualRemoteError;

    CompositeRecognitionResult(LocalRecognitionResult rawLocalResult,
                               List<CloudRecognitionResult> rawRemoteResults,
                               CloudRecognitionError eventualRemoteError,
                               List<String> slotsToUpdate) throws InterpretException
    {
        super(null, null, null, slotsToUpdate);

        _rawLocalResult = rawLocalResult;
        _rawRemoteResults = rawRemoteResults;
        _eventualRemoteError = eventualRemoteError;
    }

    @Override
    public LocalRecognitionResult getRawLocalResult()
    {
        return _rawLocalResult;
    }

    @Override
    public List<CloudRecognitionResult> getRawRemoteResults()
    {
        return _rawRemoteResults;
    }

    @Override
    public CloudRecognitionError getEventualRemoteError()
    {
        return _eventualRemoteError;
    }
}
