package com.nuance.dragon.toolkit.hybrid;

import com.nuance.dragon.toolkit.recognition.InterpretException;
import com.nuance.dragon.toolkit.recognition.InterpretedRecognition;
import com.nuance.dragon.toolkit.recognition.InterpretedRecognition.Choice;
import com.nuance.dragon.toolkit.recognition.InterpretedRecognitionCombiner;

/**
 * Default combiner for Local + Cloud recognition results.
 */
public class HybridRecognitionCombiner implements InterpretedRecognitionCombiner
{
    private static final int MAX_RESULTS = 10;

    @Override
    public InterpretedRecognition combine(InterpretedRecognition localResult,
                                          InterpretedRecognition cloudResult) throws InterpretException
    {
        if ((localResult == null || localResult.getChoiceCount() == 0) &&
            (cloudResult == null || cloudResult.getChoiceCount() == 0))
        {
            throw new InterpretException("No results to combine");
        }

        // If only one is present, just use it
        if (localResult == null || localResult.getChoiceCount() == 0)
        {
            return cloudResult;
        }

        if (cloudResult == null || cloudResult.getChoiceCount() == 0)
        {
            return localResult;
        }

        // To merge, start with the Local choice, and copy from the Cloud choice only
        // what isn't in the Local choice
        InterpretedRecognition combined = new InterpretedRecognition();
        for (int i = 0; i < localResult.getChoiceCount() && combined.getChoiceCount() < MAX_RESULTS; i++)
        {
            Choice localChoice = localResult.getChoice(i);

            Choice merged = localChoice.resolveUnrecognizedPhrases(cloudResult, true);
            if (merged != null)
            {
                combined.addChoice(merged.toString(), merged.getRecognitionType(), merged.getScore(), merged.getPhrases(), merged.getExtra());
            }
        }

        if (combined.getChoiceCount() == 0)
            return localResult;

        return combined;
    }
}
