package com.nuance.dragon.toolkit.language;

import java.util.Locale;

public class GrammarLanguage
{
    /**
     * Get the Grammar language, which can be passed to a Grammar Depot, from a Locale object.
     * @param l The input Locale object.
     * @return The Grammar language.
     */
    public static String getGrammarLanguage(Locale l)
    {
        if (l == null)
            return null;

        String lang = l.getLanguage();
        String country = l.getCountry();
        String ret = "";

        if (lang == null || lang.length() != 2 ||
            country == null || country.length() != 2)
            return null;

        // map lang code
        if (lang.equals("de"))
            ret += "deu";
        else if (lang.equals("en"))
            ret += "eng";
        else if (lang.equals("es"))
            ret += "spa";
        else if (lang.equals("fr"))
            ret += "fra";
        else if (lang.equals("it"))
            ret += "ita";
        else if (lang.equals("ja"))
            ret += "jpn";
        else if (lang.equals("ko"))
            ret += "kor";
        else if (lang.equals("zh"))
        {
            if (country.equals("CT") ||
                country.equals("GD") ||
                country.equals("HK"))
                ret += "yue";
            else
                ret += "cmn";
        }
        else if (lang.equals("tr"))
            ret += "tur";
        else if (lang.equals("no"))
            ret += "nor";
        else if (lang.equals("sv"))
            ret += "swe";
        else if (lang.equals("pl"))
            ret += "pol";
        else if (lang.equals("pt"))
            ret += "por";
        else if (lang.equals("fi"))
            ret += "fin";
        else if (lang.equals("da"))
            ret += "dan";
        else if (lang.equals("nl"))
            ret += "nld";
        else if (lang.equals("uk"))
            ret += "ukr";
        else if (lang.equals("el"))
            ret += "ell";
        else if (lang.equals("ru"))
            ret += "rus";
        else
            return null;

        ret += "-";

        // map country code
        if (country.equals("CA"))
            ret += "CAN";
        else if (country.equals("CN"))
            ret += "CHN";
        else if (country.equals("DE"))
            ret += "DEU";
        else if (country.equals("ES"))
            ret += "ESP";
        else if (country.equals("FR"))
            ret += "FRA";
        else if (country.equals("GB"))
            ret += "GBR";
        else if (country.equals("IT"))
            ret += "ITA";
        else if (country.equals("JP"))
            ret += "JPN";
        else if (country.equals("KR"))
            ret += "KOR";
        else if (country.equals("TW"))
            ret += "TWN";
        else if (country.equals("US"))
            ret += lang.equals("es") ? "XLA" : "USA";
        else if (country.equals("TR"))
            ret += "TUR";
        else if (country.equals("NO"))
            ret += "NOR";
        else if (country.equals("SE"))
            ret += "SWE";
        else if (country.equals("AU"))
            ret += "AUS";
        else if (country.equals("PL"))
            ret += "POL";
        else if (country.equals("BR"))
            ret += "BRA";
        else if (country.equals("PT"))
            ret += "PRT";
        else if (country.equals("FI"))
            ret += "FIN";
        else if (country.equals("DK"))
            ret += "DNK";
        else if (country.equals("NL"))
            ret += "NLD";
        else if (country.equals("UA"))
            ret += "UKR";
        else if (country.equals("GR"))
            ret += "GRC";
        else if (country.equals("RU"))
            ret += "RUS";
        else if (country.equals("CT") ||
                 country.equals("GD"))
            ret += "CHS";
        else if (country.equals("HK"))
            ret += "CHN";
        else
            return null;

        return ret;
    }
}
