package com.nuance.dragon.toolkit.language;

import com.nuance.dragon.toolkit.elvis.ElvisLanguage;
import com.nuance.dragon.toolkit.vocalizer.VocalizerLanguage;
import com.nuance.dragon.toolkit.vocon.VoconLanguage;

/**
 * All supported DMT languages.
 */
public class Languages extends LangSpecs
{
    /**
     * This method finds the {@link Language} object that represents the same language as the given
     * (@link ElvisLanguage} object.
     * @param elvisLanguage The {@link ElvisLanguage} that represents the desired language.
     * @return The requested (@link Language} or null if the language is not found.
     */
    public static Language lookupFromElvisLanguage(ElvisLanguage elvisLanguage)
    {
        for (Language language : Languages.ALL_LANGUAGES) {
            if (language.getElvisLanguage().equals(elvisLanguage)) {
                return language;
            }
        }
        return null;
    }

    /**
     * This method finds the {@link Language} object that represents the same language as the given
     * (@link VocalizerLanguage} object.
     * @param vocalizerLanguage The {@link VocalizerLanguage} that represents the desired language.
     * @return The requested (@link Language} or null if the language is not found.
     */
    public static Language lookupFromVocalizerLanguage(VocalizerLanguage vocalizerLanguage)
    {
        for (Language language : Languages.ALL_LANGUAGES) {
            if (language.getVocalizerLanguage().equals(vocalizerLanguage)) {
                return language;
            }
        }
        return null;
    }

    /**
     * This method finds the {@link Language} object that represents the same language as the given
     * {@link VoconLanguage} object.
     * @param voconLanguage The {@link VoconLanguage} that represents the desired language.
     * @return The requested (@link Language} or null if the language is not found.
     */
    public static Language lookupFromVoconLanguage(VoconLanguage voconLanguage)
    {
        for (Language language : Languages.ALL_LANGUAGES) {
            if (language.getVoconLanguage().equals(voconLanguage)) {
                return language;
            }
        }
        return null;
    }

    /**
     * This method finds the {@link Language} object that represents the same language as the given
     * cloudLanguage string.
     * @param cloudLanguage A string that represents the desired cloud language.
     * @return The requested (@link Language} or null if the language is not found.
     */
    public static Language lookupFromCloudLanguage(String cloudLanguage)
    {
        for (Language language : Languages.ALL_LANGUAGES) {
            if (language.getCloudLanguage().equals(cloudLanguage)) {
                return language;
            }
        }
        return null;
    }

    /**
     * This method finds the {@link Language} object that represents the same language as the given
     * grammarLanguage string.
     * @param grammarLanguage A string that represents the desired grammar language.
     * @return The requested (@link Language} or null if the language is not found.
     */
    public static Language lookupFromGrammarLanguage(String grammarLanguage)
    {
        for (Language language : Languages.ALL_LANGUAGES) {
            if (language.getGrammarLanguage().equals(grammarLanguage)) {
                return language;
            }
        }
        return null;
    }
}
