package com.nuance.dragon.toolkit.nvsl;

import org.json.JSONException;

import com.nuance.dragon.toolkit.oem.api.JSONCompliant;
import com.nuance.dragon.toolkit.oem.api.json.JSONObject;

public class NVSLAudioInfo implements JSONCompliant
{
    private static final String KEY_PROCESSEDAUDIO = "processedaudio";
    private static final String KEY_PROCESSEDSPEECH = "processedspeech";
    private static final String KEY_TOTALAUDIODURATION = "totalaudioduration";
    private static final String KEY_SPEECHLEVEL = "speechlevel";
    private static final String KEY_SATURATIONLEVEL = "saturationlevel";
    private static final String KEY_SNR = "snr";

    /**
     * The seconds of audio processed by the engine since the beginning of the current task
     * (enrollment, speaker identification, language detection).
     */
    public float processedAudio;

    /**
     * The seconds of speech processed by the engine since the beginning of the current task
     * (enrollment, speaker identification, language detection).
     */
    public float processedSpeech;

    /**
     * The total audio duration (in seconds) the engine has processed in the current task.
     * -1.0 means the duration is unknown, maybe because audio provider doesn't supports
     * this feature.
     */
    public float totalAudioDuration;

    /**
     * The speech level is a differential measure that takes into account the estimated speech and
     * noise energies. The returned value is typically in the range 0.0 to 10.0, where lower values
     * mean that the signal level is more soft. Very soft audio recordings can reduce overall
     * accuracy because they make it difficult for the system to detect voice activity.
     */
    public float speechLevel;

    /**
     * Use the saturation level to detect distorted (or "clipped") audio utterances that might
     * degrade performance. The value is the ratio of saturated audio samples for each minute.
     * It will always be 0.0, if no saturation.
     */
    public float saturationLevel;

    /**
     * The estimated Signal-to-Noise Ratio of the audio processed by the engine in the current task.
     * The range is -40 (dB) to +100 (dB). It will always be -40.0, if no SNR available.
     */
    public float snr;

    @Override
    public org.json.JSONObject toJSON()
    {
        JSONObject o = new JSONObject();
        o.tryPut(KEY_PROCESSEDAUDIO, processedAudio);
        o.tryPut(KEY_PROCESSEDSPEECH, processedSpeech);
        o.tryPut(KEY_TOTALAUDIODURATION, totalAudioDuration);
        o.tryPut(KEY_SPEECHLEVEL, speechLevel);
        o.tryPut(KEY_SATURATIONLEVEL, saturationLevel);
        o.tryPut(KEY_SNR, snr);

        return o;
    }

    public static NVSLAudioInfo createFromJSON(org.json.JSONObject json) throws JSONException
    {
        NVSLAudioInfo audioInfo = new NVSLAudioInfo();

        audioInfo.processedAudio = (float)json.getDouble(KEY_PROCESSEDAUDIO);
        audioInfo.processedSpeech = (float)json.getDouble(KEY_PROCESSEDSPEECH);
        audioInfo.totalAudioDuration = (float)json.getDouble(KEY_TOTALAUDIODURATION);
        audioInfo.speechLevel = (float)json.getDouble(KEY_SPEECHLEVEL);
        audioInfo.saturationLevel = (float)json.getDouble(KEY_SATURATIONLEVEL);
        audioInfo.snr = (float)json.getDouble(KEY_SNR);

        return audioInfo;
    }

    @Override
    public int hashCode()
    {
        final int prime = 31;
        int result = 1;
        result = prime * result + (int)(processedAudio*100);
        result = prime * result + (int)(processedSpeech*100);
        result = prime * result + (int)(totalAudioDuration*100);
        result = prime * result + (int)(speechLevel*100);
        result = prime * result + (int)(saturationLevel*100);
        result = prime * result + (int)(snr*100);

        return result;
    }

    @Override
    public boolean equals(Object obj)
    {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        NVSLAudioInfo other = (NVSLAudioInfo) obj;
        if (processedAudio != other.processedAudio)
            return false;
        if (processedSpeech != other.processedSpeech)
            return false;
        if (totalAudioDuration != other.totalAudioDuration)
            return false;
        if (speechLevel != other.speechLevel)
            return false;
        if (saturationLevel != other.saturationLevel)
            return false;
        if (snr != other.snr)
            return false;

        return true;
    }
}
