package com.nuance.dragon.toolkit.nvsl;

import com.nuance.dragon.toolkit.oem.api.Factory;
import com.nuance.dragon.toolkit.oem.api.FileManager;
import com.nuance.dragon.toolkit.oem.api.NMTHandler;
import com.nuance.dragon.toolkit.oem.api.WorkerThread;
import com.nuance.dragon.toolkit.oem.api.internal.Checker;

final class NVSLIdentifierImpl extends NVSLIdentifierBase
{
    private final boolean _hasWorkerThread;

    // Only allow 1 worker thread and file factory for all NVSL instances.
    private static WorkerThread _workerThread;
    private static int _workerThreadCount;

    private boolean _released;

    /**
     * Create a NVSL Identifier. The NVSL will use a dedicated worker thread. <p>
     * NOTE: Due to NVSL engine limitations, multiple NVSL instances must use
     * the same worker thread ?
     * @param fileManager The file manager where NVSL should look for files.
     */
    public NVSLIdentifierImpl(FileManager fileManager) {
        this(fileManager, null);
    }

    /**
     * Create a NVSL Identifier. The NVSL will use the given worker thread. <p>
     * NOTE: Due to NVSL engine limitations, multiple NVSL instances must use
     * the same worker thread. ?<p>
     * NOTE #2: Due to the audio pull model of NVSL engine, audio capture
     * must run on a different thread than the NVSL worker thread.
     * @param fileManager The file manager where NVSL should look for files.
     * @param workerThread
     */
    public NVSLIdentifierImpl(FileManager fileManager, NMTHandler workerThread) {
        super(fileManager, workerThread);
        Checker.checkArgForNull("fileManager", fileManager);
        _hasWorkerThread = workerThread == null;
    }

    @Override
    protected NMTHandler getWorkerThreadHandler() {
        if (_workerThread == null) {
            _workerThread = Factory.createWorkerThread("com.nuance.dragon.toolkit.nvsl.NVSLIdentifierImpl");
            _workerThread.start();
        }
        _workerThreadCount++;
        return _workerThread.getHandler();
    }

    @Override
    public void release()
    {
        if (!_released)
        {
            _released = true;
            super.release();

            if (_hasWorkerThread)
            {
                _workerThreadCount--;
                if (_workerThreadCount == 0)
                {
                    _workerThread.stop();
                    _workerThread = null;
                }
            }
        }
    }
}
