package com.nuance.dragon.toolkit.nvsl;

import java.util.List;

import com.nuance.dragon.toolkit.audio.AudioChunk;

/**
 * An interface that provides access to the NVSL engine. All methods here
 * are expected to be called from a single worker thread unless otherwise specified.
 */

interface NativeNVSL
{
    interface NVSLAudioListener
    {
        void onStartAudioPull(String audioFileName);

        void onStopAudioPull(String audioFileName);

        AudioChunk onGetAudioChunk(int maxLen);

        void onCanceled();

        void onSignalUpdate(AudioInfoJni audioInfo);
    }

    interface NVSLResultListener
    {
        void onIdentificationResult(ResultJni result);
    }

    boolean createNVSL(NVSLConfig config);

    void destroyNVSL();

    void cancel();

    boolean startEnrolling(String vpFilePath, List<String> audioFiles, int freqHz, NVSLAudioListener audioListener);

    boolean startAdapting(String vpFilePath, String audioFile, int freqHz, NVSLAudioListener audioListener);

    boolean startIdentifying(List<String> vpFilePaths, int freqHz, int maxResults, boolean isMultiIdent, int partialResultTimerMs, NVSLAudioListener audioListener, NVSLResultListener resultListener);

    boolean startDetectingLanguage(int freqHz, NVSLAudioListener audioListener, NVSLResultListener resultListener);

    /*
     * Returns true if all audioes in audio files are consistent, otherwise false
     */
    boolean startCheckConsistency(List<String> audioFiles, int freqHz, NVSLAudioListener audioListener);

    boolean startAudioCheck(int freqHz, int taskType, String audioDumpFilePath, NVSLAudioListener audioListener);

    boolean startAudioCheck(int freqHz, int taskType, String audioDumpFilePath, String noiseInFilePath, int rand, int degradationLevel, String audioPlusNoiseFilePath, NVSLAudioListener audioListener);

    void enableVoconVerboseLogging(boolean verbose);
}
