package com.nuance.dragon.toolkit.nvsl;

import java.util.ArrayList;


class ResultJni
{
    private final ArrayList<NVSLResult.Language> _languages;
    private final ArrayList<NVSLResult.Speaker> _speakers;
    private final boolean _isFinal;

    ResultJni(boolean isFinal)
    {
        _languages = new ArrayList<NVSLResult.Language>();
        _speakers = new ArrayList<NVSLResult.Speaker>();
        _isFinal = isFinal;
    }

    void addSpeakers(int NumOfSpeaker)
    {
        for (int i = 0; i < NumOfSpeaker; i++)
            _speakers.add(new NVSLResult.Speaker());
    }

    void setGenderResult(int index, char cGender, int genderConf)
    {
        _speakers.get(index).setGenderResult(cGender, genderConf);
    }

    void addLanguage(String langTag, String langDescr, int score)
    {
        _languages.add(new NVSLResult.Language(langTag, langDescr, score));
    }

    void addEntry(int index, String voiceprintTag, int rawScore, int mappedScore)
    {
        _speakers.get(index).addIdentEntry(new NVSLResult.IdentEntry(voiceprintTag, rawScore, mappedScore));
    }

    void addInterval(int index, float start, float end)
    {
        _speakers.get(index).addInterval(new NVSLResult.Interval(start, end));
    }

    NVSLResult createNVSLResult(String confSetName) {
        if (_speakers.isEmpty() && _languages.isEmpty())
            return null;

        NVSLResult realResult = new NVSLResult(_isFinal);

        for (NVSLResult.Speaker speaker : _speakers)
        {
            for (NVSLResult.IdentEntry entry : speaker.getOrgIdentEntries())
            {
                // voiceId in raw entry includes confSetName
                String voiceId = entry.voiceId.replace(("_" + confSetName), "");
                entry.voiceId = voiceId;
            }
            realResult.addSpeaker(speaker);
        }

        for (NVSLResult.Language lang : _languages)
        {
            realResult.addLanguage(lang);
        }

        return realResult;

    }
}
