package com.nuance.dragon.toolkit.oem.api;

import java.io.File;

public interface NMTContext
{
    /**
     * Open a file descriptor for reading a raw resource.
     *
     * @param id The resource identifier to open, as generated by the appt tool.
     *
     * @return NMTAssetFileDescriptor A new file descriptor you can use to read the resource.
     *
     * @throws RuntimeException
     */
    NMTAssetFileDescriptor openRawResourceFd(int id);

    NMTAssetFileDescriptor openFd (String fileName);

    /**
     * Returns the absolute path to the directory on the internal file system.
     * @return The path of the directory holding application files.
     */
    File getFilesDir();

    /**
     * Return the primary external storage directory.
     */
    File getExternalStorageDirectory();

    /**
     * Return a String array of all the assets at the given path.
     *
     * @param path A relative path within the assets.
     * @return String[] Array of strings, one for each asset.
     */
    String[] listAssets(String path);

    /**
     * Return the context of the single, global Application object of the current process.
     * This generally should only be used if you need a Context whose lifecycle is separate from the current context,
     * that is tied to the lifetime of the process rather than the current component.
     */
    NMTContext getAppContext();

    /**
     * Return a platform dependent context such as android.content.Context
     */
    Object getNativeContext();
}
