package com.nuance.dragon.toolkit.oem.api;

public interface NMTHandler
{
    /**
     * Causes the Runnable r to be added to the message queue.
     * The runnable will be run on the thread to which this handler is attached.
     *
     * @param r The Runnable that will be executed.
     *
     * @return Returns true if the Runnable was successfully placed in to the message queue.
     * 		   Returns false on failure, usually because the looper processing the message queue is exiting.
     */
    boolean post(Runnable r);

    /**
     * Causes the Runnable r to be added to the message queue, to be run after the specified amount of time elapses.
     * The runnable will be run on the thread to which this handler is attached.
     *
     * @param r The Runnable that will be executed.
     * @param delayMillis The delay (in milliseconds) until the Runnable will be executed.
     *
     * @return Returns true if the Runnable was successfully placed in to the message queue.
     * 		   Returns false on failure, usually because the looper processing the message queue is exiting.
     */
    boolean postDelayed(Runnable r, long delayMillis);

    boolean postToLooper(Runnable r);

    /**
     * Remove any pending posts of Runnable r that are in the message queue.
     *
     * @param r The pending runnable to be removed
     */
    void removeCallbacks(Runnable r);

    /**
     * Return a platform dependent handler. On Android it is android.os.Handler; On J2SE it is null
     */
    Object getNativeHandler();
}
