package com.nuance.dragon.toolkit.oem.api;

/**
 * A simple worker thread wrapper. This class starts a dedicated thread with
 * a {@link NMTHandler} attached for posting messages.
 */
public interface WorkerThread
{
    /**
     * Start the thread.
     */
    void start();

    /**
     * Stop the thread.
     */
    void stop();

    /**
     * Get the handler for the thread. This will never return null. On Android,
     * this handler is not actually attached to the worker thread; rather, it
     * is an AttachableHandler that passes messages to the correct
     * handler.
     * @return The handler.
     */
    NMTHandler getHandler();

    /**
     * @return The thread
     */
    Thread getThread();

    /**
     * Add a runnable to the queue, no effect on Android
     */
    void addRunnable(Runnable r);

    /**
     * Remove a runnable from the queue, no effect on Android
     */
    void removeRunnable(Runnable r);
}
