package com.nuance.dragon.toolkit.oem.api.json;

import java.util.List;

import org.json.JSONException;

import com.nuance.dragon.toolkit.oem.api.JSONCompliant;
import com.nuance.dragon.toolkit.oem.api.Logger;


/**
 * JSONObject extension that does some typical exception handling, to simplify
 * application code.
 */
public class JSONObject extends org.json.JSONObject
{
    public JSONObject(org.json.JSONObject json)
    {
        org.json.JSONArray names = json.names();
        for (int i = 0; i < names.length(); i++)
        {
            try
            {
                String name = names.getString(i);
                Object o = json.get(name);

                // TODO: will this work for integers?
                put(name, o);
            }
            catch (JSONException e)
            {
                Logger.error(this, "Error creating JSONObject", e);
            }
        }
    }

    public JSONObject(String text) throws JSONException
    {
        super(text);
    }

    public JSONObject()
    {
        super();
    }

    public JSONObject(String paramName, Object paramValue)
    {
        super();
        tryPut(paramName, paramValue);
    }

    /**
     * Converts the {@link JSONCompliant} object to JSON prior to adding it.
     * @param name
     * @param value
     * @return true if successful, false otherwise.
     */
    public boolean tryPut(String name, JSONCompliant value)
    {
        if (value != null)
        {
            try
            {
                put(name, value.toJSON());
                return true;
            }
            catch (JSONException e)
            {
                Logger.error(this, "Error adding to JSON");
            }
        }
        return false;
    }

    /**
     * Like {@link org.json.JSONObject#put(String, Object)}, but reports errors
     * via return value instead of exception.
     * @param name
     * @param value
     * @return true if successful, false otherwise.
     */
    public boolean tryPut(String name, Object value)
    {
        if (value != null)
        {
            try
            {
                put(name, value);
                return true;
            }
            catch (JSONException e)
            {
                Logger.error(this, "Error adding to JSON");
            }
        }
        return false;
    }

    /**
     * Add the contents of a list to a JSONArray, and add the JSONArray to
     * this JSONObject.
     * @param name
     * @param list
     * @return true if successful, false otherwise.
     */
    public boolean tryPutList(String name, List<? extends JSONCompliant> list)
    {
        if (list != null)
        {
            org.json.JSONArray a = new org.json.JSONArray();
            for (JSONCompliant j : list)
            {
                org.json.JSONObject o = j.toJSON();
                if (o == null)
                    return false;
                a.put(o);
            }
            return tryPut(name, a);
        }
        return false;
    }

    /**
     * Add the contents of a list to a JSONArray, and add the JSONArray to
     * this JSONObject.
     * @param name
     * @param list
     * @return true if successful, false otherwise.
     */
    public boolean tryPutStringList(String name, List<String> list)
    {
        if (list != null)
        {
            org.json.JSONArray a = new org.json.JSONArray();
            for (String s : list)
            {
                a.put(s);
            }
            return tryPut(name, a);
        }
        return false;
    }
}
