package com.nuance.dragon.toolkit.oem.impl;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import android.content.res.AssetFileDescriptor;

import com.nuance.dragon.toolkit.oem.api.Logger;
import com.nuance.dragon.toolkit.oem.api.NMTAssetFileDescriptor;
import com.nuance.dragon.toolkit.oem.api.internal.Checker;

public class AssetFileDescriptorOem implements NMTAssetFileDescriptor {

    private final AssetFileDescriptor _assetFileDescriptor;

    public AssetFileDescriptorOem(Object obj)
    {
        Checker.checkState(obj, (obj instanceof AssetFileDescriptor), "AssetFileDescriptorOem is expecting android.content.res.AssetFileDescriptor in constructor");
        _assetFileDescriptor = (AssetFileDescriptor)obj;
    }

    @Override
    public FileInputStream createInputStream()
    {
        try
        {
            return _assetFileDescriptor.createInputStream();
        }
        catch (IOException e)
        {
            Logger.error(this, "error on createInputStream: " + e.getMessage());
        }
        return null;
    }

    @Override
    public FileOutputStream createOutputStream()
    {
        try
        {
            return _assetFileDescriptor.createOutputStream();
        }
        catch (IOException e)
        {
            Logger.error(this, "error on createOutputStream: " + e.getMessage());
        }
        return null;
    }

    @Override
    public void close()
    {
        try
        {
            _assetFileDescriptor.close();
        }
        catch (IOException e)
        {
            Logger.error(this, "error on close: " + e.getMessage());
        }
    }

    @Override
    public Object getNativeDescriptor() {
        return _assetFileDescriptor;
    }

    @Override
    public long getDeclaredLength()
    {
        return _assetFileDescriptor.getDeclaredLength();
    }

}
