package com.nuance.dragon.toolkit.oem.impl;

import java.io.File;
import java.io.IOException;

import android.content.Context;
import android.os.Environment;

import com.nuance.dragon.toolkit.oem.api.Logger;
import com.nuance.dragon.toolkit.oem.api.NMTAssetFileDescriptor;
import com.nuance.dragon.toolkit.oem.api.NMTContext;
import com.nuance.dragon.toolkit.oem.api.internal.Checker;

public class ContextOem implements NMTContext
{
    private final Context _context;

    public ContextOem(Object obj)
    {
        Checker.checkState(obj, (obj instanceof Context), "ContextOem is expecting android.content.Context in constructor");
        _context = (Context)obj;
    }

    @Override
    public NMTAssetFileDescriptor openFd(String fileName)
    {
        if (_context != null)
        {
            try
            {
                return new AssetFileDescriptorOem(_context.getAssets().openFd(fileName));
            }
            catch (IOException e)
            {
                Logger.error(this, "error when opening asset file descriptor: " + e.getMessage());
            }
        }
        return null;
    }

    @Override
    public String[] listAssets(String path)
    {
        try
        {
            return _context.getAssets().list(path);
        }
        catch (IOException e)
        {
            Logger.error(this, "error on returning all the assets: " + e.getMessage());
        }
        return null;
    }

    @Override
    public NMTAssetFileDescriptor openRawResourceFd(int id)
    {
        if (_context != null)
        {
            return new AssetFileDescriptorOem(_context.getResources().openRawResourceFd(id));
        }
        return null;
    }

    @Override
    public NMTContext getAppContext()
    {
        if (_context != null)
        {
            return new ContextOem(_context.getApplicationContext());
        }
        return null;
    }

    @Override
    public Object getNativeContext()
    {
        return _context;
    }

    @Override
    public File getFilesDir()
    {
        return _context.getFilesDir();
    }

    @Override
    public File getExternalStorageDirectory()
    {
        return Environment.getExternalStorageDirectory();
    }
}
