package com.nuance.dragon.toolkit.oem.impl;

import java.io.UnsupportedEncodingException;

import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.telephony.TelephonyManager;

import com.nuance.dragon.toolkit.oem.api.DeviceInfo;
import com.nuance.dragon.toolkit.oem.api.NMTContext;
import com.nuance.nmsp.client2.sdk.common.util.Checker;
import com.nuance.nmsp.client2.sdk.components.general.ResourceUtil;

public class DeviceInfoOem implements DeviceInfo
{
	public static DeviceInfoOem _instance;
	public Context _context;

	public static DeviceInfo getDeviceInfoOem(NMTContext context)
	{
		if (_instance == null)
		{
			_instance = new DeviceInfoOem(context);
		}
		return _instance;
	}

	private DeviceInfoOem(NMTContext context)
	{
		Checker.checkArgForNull("context", context);

		_context = (Context) context.getNativeContext();
	}


	@Override
	public String getCarrierName()
	{
        TelephonyManager telephony = (TelephonyManager) _context.getSystemService(Context.TELEPHONY_SERVICE);
        return telephony.getNetworkOperatorName();
	}

	@Override
	public String getNetworkType()
	{
		ConnectivityManager cm = (ConnectivityManager) _context.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo ni = cm.getActiveNetworkInfo();
        if (ni != null)
        {
            return ni.getTypeName();
        }
        return null;
	}

	@Override
	public String getAppName()
	{
        try
        {
            PackageManager pm = _context.getPackageManager();
            String packageName = _context.getPackageName();

            String label = (String) pm.getApplicationLabel(pm.getApplicationInfo(packageName, 0));
            String version = pm.getPackageInfo(packageName, 0).versionName;
            return (label + " " + version);
        }
        catch (Exception e)
        {
        }

		return null;
	}

	@Override
	public byte[] getPackageName()
	{
        try
        {
            ApplicationInfo info = _context.getApplicationInfo();
            if (info.packageName != null)
            {
                return info.packageName.getBytes("UTF-8");
            }
        } catch (UnsupportedEncodingException ex)
        {
        }
        return new byte[0];
	}

	@Override
	public String getUid()
	{
		return ResourceUtil.getUniqueID(_context);
	}

    @Override
    public String getPlatform() {
        return "android";
    }
}