package com.nuance.dragon.toolkit.oem.impl;

import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSession;

import com.android.volley.AuthFailureError;
import com.android.volley.NetworkResponse;
import com.android.volley.NoConnectionError;
import com.android.volley.ParseError;
import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.Response.ErrorListener;
import com.android.volley.Response.Listener;
import com.android.volley.TimeoutError;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.HttpHeaderParser;
import com.android.volley.toolbox.HurlStack;
import com.android.volley.toolbox.Volley;
import com.nuance.dragon.toolkit.oem.api.HttpUtil;
import com.nuance.dragon.toolkit.oem.api.Logger;
import com.nuance.dragon.toolkit.oem.api.NMTContext;
import com.nuance.nmsp.client2.sdk.oem.socket.ssl.NmspSSLSocketFactory;
import com.nuance.nmsp.client2.sdk.oem.socket.ssl.SSLSettings;

public class HttpUtilOem implements HttpUtil
{
    private static class VolleyRequest extends Request<String>
    {
        private final Listener<String> mListener;
        private final String _requestBodyContentType;
        private final String mRequestBody;
        private final Map<String, String> _extraHeaders;

        public VolleyRequest(int method, String url, String requestBodyContentType, String requestBody, Map<String, String> extraHeaders, Listener<String> listener, ErrorListener errorListener) {
            super(method, url, errorListener);
            mListener = listener;
            mRequestBody = requestBody;
            _requestBodyContentType = requestBodyContentType;
            _extraHeaders = extraHeaders;
        }

        @Override
        protected void deliverResponse(String response)
        {
            mListener.onResponse(response);
        }

        @Override
        public String getBodyContentType()
        {
            return _requestBodyContentType;
        }

        @Override
        public byte[] getBody()
        {
            try
            {
                return mRequestBody == null ? null : mRequestBody.getBytes(PROTOCOL_CHARSET);
            } catch (UnsupportedEncodingException uee)
            {
                Logger.error(this, "Unsupported Encoding while trying to get the bytes of " + mRequestBody + " using " + PROTOCOL_CHARSET);
                return null;
            }
        }

        @Override
        protected Response<String> parseNetworkResponse(NetworkResponse response)
        {
            try
            {
                String s = new String(response.data, HttpHeaderParser.parseCharset(response.headers));
                return Response.success(s, HttpHeaderParser.parseCacheHeaders(response));
            } catch (UnsupportedEncodingException e)
            {
                return Response.error(new ParseError(e));
            }
        }

        @Override
        public Map<String, String> getHeaders() throws AuthFailureError
        {
            if (_extraHeaders == null)
            {
                return super.getHeaders();
            }

            Map<String, String> headers = super.getHeaders();
            HashMap<String, String> retHeaders = new HashMap<String, String>();
            retHeaders.putAll(headers);
            retHeaders.putAll(_extraHeaders);
            return retHeaders;
        }
    }

    private RequestQueue _reqQueue;
    private SSLSettings  _ssl = new SSLSettings();

    public HttpUtilOem(boolean enableSelfSignedCert, String certSummary, String certData, NMTContext context)
    {
        android.content.Context navContext = (android.content.Context) context.getNativeContext();
        _ssl.enableSelfSignedCert = enableSelfSignedCert;
        _ssl.certSummary = certSummary;
        _ssl.certData = certData;
        if (_ssl.enableSelfSignedCert)
        {
            HttpsURLConnection.setDefaultHostnameVerifier(new HostnameVerifier() {
                @Override
                public boolean verify(String hostname, SSLSession session) {
                    return true;
                }
            });
        }
        _reqQueue = Volley.newRequestQueue(navContext, new HurlStack(null, NmspSSLSocketFactory.getSocketFactory(_ssl)));
    }

    @Override
    public void request(int method, String url, String requestBodyContentType, String requestBody, ResponseCallback responseCallback)
    {
        request(method, url, requestBodyContentType, requestBody, responseCallback, null);
    }

    @Override
    public void request(int method, String url, String requestBodyContentType, String requestBody, final ResponseCallback responseCallback, final Map<String, String> extraHeaders)
    {
        Response.Listener<String> rListener = new Response.Listener<String>() {
            @Override
            public void onResponse(String response)
            {
                responseCallback.okCallback(response);
            }
        };

        Response.ErrorListener eListener = new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                try {
                    HttpConnectionStatus status = HttpConnectionStatus.HTTP_CONNECTION_OK;
                    if (error instanceof TimeoutError ||
                            error instanceof NoConnectionError)
                        status = HttpConnectionStatus.HTTP_CONNECTION_ERROR;
                    responseCallback.errorCallback(status, (error.networkResponse != null ? error.networkResponse.statusCode : 0), (error.networkResponse != null && error.networkResponse.data != null ? new String(error.networkResponse.data, PROTOCOL_CHARSET) : error.getMessage()));
                } catch (UnsupportedEncodingException e) {
                    Logger.error(HttpUtilOem.this, "Unsupported Encoding while trying to get the String of " + error.networkResponse.data + " using " + PROTOCOL_CHARSET);
                }
            }
        };

        Request<String> r = new VolleyRequest(method == METHOD_GET ? Request.Method.GET : Request.Method.POST,
                                              url,
                                              requestBodyContentType,
                                              requestBody,
                                              extraHeaders,
                                              rListener,
                                              eListener);
        _reqQueue.add(r);
    }

    @Override
    public void release()
    {
        _reqQueue.stop();
    }
    /*public void jsonRequest(int method, String url, JSONObject jsonRequest, final ResponseCallback responseCallback, final Map<String, String> extraHeaders)
    {
        JsonObjectRequest jr = new JsonObjectRequest(method == METHOD_GET ? Request.Method.GET : Request.Method.POST,
                url,
                jsonRequest,
                new Response.Listener<JSONObject>() {
                    @Override
                    public void onResponse(JSONObject response) {
                        responseCallback.okCallback(response);
                    }
                },
                new Response.ErrorListener() {
                    @Override
                    public void onErrorResponse(VolleyError error) {
                        responseCallback.errorCallback(HttpConnectionStatus.HTTP_CONNECTION_OK*//*TODO*//*, error.networkResponse.statusCode, error.networkResponse.data.toString());
                    }
                })
                {
                    @Override
                    public Map<String, String> getHeaders() throws AuthFailureError {
                        Map<String, String> headers = super.getHeaders();
                        extraHeaders.putAll(headers);
                        return extraHeaders;
                    }
                };
        _reqQueue.add(jr);
    }*/
}
