package com.nuance.dragon.toolkit.oem.impl;

import android.util.Log;

import com.nuance.dragon.toolkit.util.internal.ClassUtils;

public class LoggerOem
{
	private static final String TAG = "NMT";

    private static final class Obf
    {
        private static final boolean IS_OBFUSCATED = !"Obf".equals(Obf.class.getSimpleName());
    }

    public static void verbose(Object source, String text)
    {
        Log.v(TAG, appendToName(source, text));
    }

    public static void debug(Object source, String text)
    {
        Log.d(TAG, appendToName(source, text));
    }

    public static void info(Object source, String text)
    {
        Log.i(TAG, appendToName(source, text));
    }

    public static void warn(Object source, String text)
    {
        Log.w(TAG, appendToName(source, text));
    }

    public static void error(Object source, String text)
    {
        Log.e(TAG, appendToName(source, text));
    }

    public static void error(Object source, String text, Throwable tr)
    {
        Log.e(TAG, appendToName(source, text), tr);
    }

    public static void fatal(Object source, String text)
    {
        Log.wtf(TAG, appendToName(source, text));
    }

    public static void fatal(Object source, String text, Throwable tr)
    {
        Log.wtf(TAG, appendToName(source, text), tr);
    }

    private static String appendToName(Object o, String text)
    {
        if (o == null)
        {
            return text;
        }
        else
        {
            String name;
            if (Obf.IS_OBFUSCATED)
            {
                name = null;
            }
            else if (o instanceof String)
            {
                name = (String) o;
            }
            else if (o instanceof Class)
            {
                name = ClassUtils.getShortClassName((Class<?>) o);
            }
            else
            {
                name = ClassUtils.getShortClassName(o.getClass());
            }

            if (name == null || name.length() == 0)
            {
                return text;
            }
            else
            {
                return name + ": " + text;
            }
        }
    }
}
