package com.nuance.dragon.toolkit.recognition;

import org.json.JSONException;
import org.json.JSONObject;

import com.nuance.dragon.toolkit.oem.api.Logger;
import com.nuance.dragon.toolkit.oem.api.internal.Checker;

/**
 * The base class for local recognition results (Elvis, VoCon).
 */
public abstract class LocalRecognitionResult
{
    protected final JSONObject _rawResult;

    public LocalRecognitionResult(String rawResultStr)
    {
        Checker.checkStringArgForEmpty("rawResultStr", rawResultStr);

        Logger.debug(this, "" + rawResultStr);
        org.json.JSONObject json = null;
        try {
            json = new JSONObject(rawResultStr);
        } catch (JSONException e) {
            Logger.error(this, "Unable to parse raw result.", e);
        }
        _rawResult = json;
    }

    public LocalRecognitionResult(JSONObject rawResult)
    {
        Checker.checkArgForNull("rawResult", rawResult);

        Logger.debug(this, "" + rawResult);
        _rawResult = rawResult;
    }

    /**
     * Get the raw JSON result as emitted by the recognition engine. The object will never be null.
     * @return The raw engine result.
     */
    public JSONObject getRawResult()
    {
        return _rawResult;
    }

    @Override
    public String toString() {
        return "LocalRecognitionResult [_rawResult=" + _rawResult + "]";
    }
}
