package com.nuance.dragon.toolkit.recognition;

import java.util.List;

import com.nuance.dragon.toolkit.cloudservices.DataParam;

/**
 * @param <RecogType> The type of the raw recognition result.
 */
public interface RecognitionInterpreter<RecogType extends Object>
{
    /**
     * Get an interpreted result from the recognition.
     * @param recognitionResult The raw recognition result.
     * @return An interpreted recognition result--never null.
     * @throws InterpretException Thrown if there was an error interpreting the result.
     */
    InterpretedRecognition getInterpretedResult(RecogType recognitionResult) throws InterpretException;

    /**
     * Get the network parameters to be sent to the other recognizer(s) after a
     * recognition.
     * @param recognitionResult
     * @param params Empty list of parameters, to be filled in by this method.
     * @return true if cloud processing is required.
     */
    boolean processForCloud(RecogType recognitionResult, List<DataParam> params);

    /**
     * After a recognition, get a list of the names of any WordSlots that were flagged as
     * requiring an update on the server.  This can be necessary if the state of the WordList
     * on the server does not match the state on the client, for example if a user cleared the
     * application data on the phone.
     * @param recognitionResult The raw recognition result.
     * @return A list of WordSlot names that require an update on the server.
     */
    List<String> getUpdateRequiredList(RecogType recognitionResult);
}
