package com.nuance.dragon.toolkit.recognition.dictation;

/**
 * This class represents an alternative for a given text in the edit buffer.
 * It is an aggregate of the <code>EditorItem</code> objects that constitute
 * the alternative. It is returned by the <code>AlterChoices#getChoiceAt(int)</code>
 * method.
 * @see AlterChoices#getChoiceAt(int)
 */
public interface AlterChoice {

    /**
     * Returns the string representation of this <code>AlterChoice</code>.
     *
     * @return A string representation of this <code>AlterChoice</code>.
     */
    public String toString();

    /**
     * Returns the number of <code>EditorItem</code> objects contained in
     * this <code>Alternative</code>.
     *
     * The <code>AlterChoice</code> object is constituted of one or more
     * <code>EditorItem</code> objects. This method returns the total number
     * of editor items that constitute this <code>AlterChoice</code>.
     *
     * @return  number of EditorItems
     */
    public int size();

    /**
     * Returns the <code>EditorItem</code> at the specified index.
     *
     * The <code>AlterChoice</code> object is constituted of one or
     * more <code>EditorItem</code> objects. This method returns the
     * <code>EditorItem</code> at the specified index.
     *
     * @param index Index of the requested <code>EditorItem</code>
     * @return <code>EditorItem</code> at the indicated index, or null if index is invalid
     */
    public EditorItem editorItemAt(int index);
}
