package com.nuance.dragon.toolkit.recognition.dictation;

/**
 * The <code>EditorItem</code> is the basic component for representing text
 * within a <code>DictionEditBuffer</code> class and the <code>AlterChoice</code> class.
 * @see DictationEditBuffer
 * @see AlterChoice
 */
public interface EditorItem
{
    /**
     * The Type enum indicates the of the EditorItem.
     */
    public enum Type
    {
        /**
         * The type TOKEN indicates that this EditorItem represents a result
         * as received from the server. Typically a TOKEN EditorItem can have
         * alternatives
         */
        TOKEN,
        /**
         * The type WORD indicates that this EditorItem is inserted text or
         * a TOKEN that has been manually edited. A WORD EditorItem does not
         * have alternatives.
         */
        WORD,
        /**
         * The type WHITE_SPACE indicates that this EditorItem is a white space.
         * A WHITE_SPACE EditorItem does not have alternatives.
         */
        WHITE_SPACE
    }

    /**
     * Returns the <code>Type</code> of this <code>EditorItem</code>
     *
     * @return Type of <code>EditorItem</code>
     */
    public Type getType();

    /**
     * Retrieve length of <code>EditorItem</code> in number of characters
     *
     * @return length of edit buffer
     */
    public int length();

    /**
     * Returns the string representation of the current <code>EditorItem</code>
     *
     * @return string representation of the current <code>EditorItem</code>
     */
    public String toString();
}
