/**
 *  Copyright 2006-2012, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.dragon.toolkit.recognition.dictation;

import java.util.Hashtable;

/**
 * Represents a <code>Token</code> object, which is contained by
 * a <code>Sentence</code> or an <code>Alternative</code>. A <code>Token</code> carries
 * the content as a string and the confidence score as a value of type <code>double</code>.
 *
 * Call the {@link #toString()} method to get the string representation of the <code>Token</code>.
 *
 * @author Nuance Communication Inc.
 * @since 1.1
 **/
public interface Token extends EditorItem
{
    /**
     * Retrieves the confidence score of the current
     * <code>Token</code>.
     *
     * <p>The confidence range and format are configurable on Nuance Speech Server.
     *
     * @return the confidence score
     */
    public double getConfidenceScore();


    /**
     * Retrieves the start time in milliseconds of the current
     * <code>Token</code>.
     *
     * @return the start time
     */
    public long getStartTime();

    /**
     * Retrieves the end time in milliseconds of the current
     * <code>Token</code>.
     *
     * @return the end time
     */
    public long getEndTime();

    /**
     * Checks if the <code>Token</code> has a <code>\*no-space-before</code> directive.
     *
     * @return true if there is a <code>\*no-space-before</code> directive for this <code>Token</code>
     *         false if there is no <code>\*no-space-before</code> directive for this <code>Token</code>
     */
    public boolean hasNoSpaceBeforeDirective();

    /**
     * Checks if the <code>Token</code> has a <code>\*no-space-after</code> directive.
     *
     * @return true if there is a <code>\*no-space-after</code> directive for this <code>Token</code>
     *         false if there is no <code>\*no-space-after</code> directive for this <code>Token</code>
     */
    public boolean hasNoSpaceAfterDirective();

   /** Retrieves the written form of the current
   * <code>Token</code>.
   *
   * @return the written form
   */
    public String getWrittenForm();

    /**
     * Retrieves the spoken forms of the current
     * <code>Token</code>.
     *
     * @return the spoken forms in a hashtable
     * where keys are the pronunciation types and
     * values are corresponding contents.
     */
    public Hashtable<String, String> getSpokenForms();

//    /**
//     * Retrieves the homophones of the current
//     * <code>Token</code>.
//     *
//     * @return the homophones
//     */
//    public List<String> getHomophones();

    /**
     * Retrieves the string representation of the current
     * <code>Token</code>.
     *
     * @return the string content of the <code>Token</code>
     */
    public String toString();
}
