package com.nuance.dragon.toolkit.recognition.dictation.internal;

import java.util.ArrayList;

import com.nuance.dragon.toolkit.recognition.dictation.AlterChoice;
import com.nuance.dragon.toolkit.recognition.dictation.EditorItem;
import com.nuance.dragon.toolkit.recognition.dictation.EditorItem.Type;
import com.nuance.dragon.toolkit.recognition.dictation.Token;

class AlterChoiceImpl implements AlterChoice
{
    private ArrayList<EditorItem> _items;

    public AlterChoiceImpl ()
    {
        _items = new ArrayList<EditorItem>();
    }

    public AlterChoiceImpl (AlterChoice choice)
    {
        _items = new ArrayList<EditorItem>();
        for (int i = 0; i < choice.size(); ++ i) {
            _items.add(choice.editorItemAt(i));
        }
    }

    @Override
    public String toString()
    {
        if (_items == null || _items.size() == 0)
            return "";

        String text = new String();
        for (int i = 0; i < _items.size(); ++ i) {
            text += _items.get(i).toString();
        }
        return text;
    }

    @Override
    public int size()
    {
        return _items.size();
    }

    @Override
    public EditorItem editorItemAt(int index)
    {
        if (index < 0 || index >= _items.size()) {
            return null;
        }
        return _items.get(index);
    }

    public void add(EditorItem toAdd) {
        _items.add(toAdd);
    }

    public void addTokens(ArrayList<Token> tokens) {

        //skip duplicated tokens
        int itemIndex = _items.lastIndexOf(tokens.get(0));
        int tokenIndex = 0;

        if (itemIndex != -1) {
            while (itemIndex < _items.size()) {
                EditorItem item = _items.get(itemIndex);
                if (item.getType() == EditorItem.Type.WHITE_SPACE) {
                    ++ itemIndex;
                }
                else if (item.getType() == EditorItem.Type.WORD) {
                    tokenIndex = 0;
                    break;
                }
                else {
                    if (tokenIndex < tokens.size()) {
                        if (item.equals(tokens.get(tokenIndex))){
                            ++ itemIndex;
                            ++ tokenIndex;
                        }
                        else {
                            tokenIndex = 0;
                            break;
                        }
                    }
                    else {
                        tokenIndex = 0;
                        break;
                    }
                }
            }

            //patch: removed last white-space if no token will be inserted.
            if (tokenIndex == tokens.size()) {
                if (_items.size() > 1 &&
                    _items.get(_items.size() - 1).getType() == EditorItem.Type.WHITE_SPACE &&
                    _items.get(_items.size() - 2).getType() == EditorItem.Type.TOKEN) {
                    _items.remove(_items.size() - 1);
                }
            }
        }


        boolean no_space_after = true;
        if (_items.size() > 0) {
            EditorItem lastItem = _items.get(_items.size() - 1);
            if(lastItem.getType() == EditorItem.Type.TOKEN) {
                no_space_after = ((Token)lastItem).hasNoSpaceAfterDirective();
            }
        }

        boolean no_space_before = false;
        for (int index = tokenIndex; index < tokens.size(); ++ index) {

            TokenImpl token = (TokenImpl)tokens.get(index);

            no_space_before = token.hasNoSpaceBeforeDirective();
            if ( !no_space_before && !no_space_after) {
                _items.add(new Word(" ", Type.WHITE_SPACE));
             }

            _items.add((EditorItem) token);

            no_space_after = token.hasNoSpaceAfterDirective();
        }
    }

    public ArrayList<EditorItem> getItems()
    {
        return _items;
    }
}
