package com.nuance.dragon.toolkit.recognition.dictation.internal;

import java.util.ArrayList;

import com.nuance.dragon.toolkit.recognition.dictation.AlterChoice;
import com.nuance.dragon.toolkit.recognition.dictation.AlterChoices;
import com.nuance.dragon.toolkit.recognition.dictation.EditorItem;
import com.nuance.dragon.toolkit.recognition.dictation.Token;

class AlterChoicesImpl implements AlterChoices {

    private ArrayList<AlterChoice> _alternativesList;
    private DictationEditBufferImpl _parent;
    private long _firstPos;
    private long _lastPos;
    private boolean _clean = false;

    private final long _stamp;

    public AlterChoicesImpl(DictationEditBufferImpl parent, int size, long firstPos, long lastPos, long stamp) {
        _stamp = stamp;
        _alternativesList = new ArrayList<AlterChoice>(size);
        for (int i = 0; i < size; i++)
            _alternativesList.add(i, new AlterChoiceImpl());

        _parent = parent;
        _firstPos = firstPos;
        _lastPos = lastPos;
    }

    @Override
    public void choose(int alternativeId) {
        if (alternativeId < 0 || alternativeId >= _alternativesList.size())
            throw new IndexOutOfBoundsException("Index provided is not within range.");

        AlterChoiceImpl choice = (AlterChoiceImpl)_alternativesList.get(alternativeId);
        if (_parent.compareStamp(_stamp))
            _parent.replaceItems(_firstPos, _lastPos, choice.getItems());
        else
            throw new IllegalStateException("The DictationEditBuffer has been modified since this AlterChoices object was created.");
    }

    @Override
    public String[] toStringArray() {
        String[] ret = new String[size()];
        for (int i = 0; i < size(); i++){
            ret[i] = getChoiceAt(i).toString();
        }
        return ret;
    }

    @Override
    public int size() {
        return _alternativesList.size();
    }

    @Override
    public AlterChoice getChoiceAt(int index)
    {
        if (index < 0 || index >= _alternativesList.size()) {
            return null;
        }
        return _alternativesList.get(index);
    }

    public void addEntry(EditorItem entry) {
        _clean = false;
        addToAll((EditorItem) entry);
    }

//    public void addEntry(ArrayList< ArrayList<Token> > entry) {
//        _clean = false;
//        for (int alternativeIndex = 0; alternativeIndex < entry.size(); ++alternativeIndex) {
//            ArrayList<Token> tokens = entry.get(alternativeIndex);
//
//            boolean no_space_before = false;
//            boolean no_space_after = true;
//            for (int tokenIndex = 0; tokenIndex < tokens.size(); ++ tokenIndex) {
//                TokenImpl token = (TokenImpl)tokens.get(tokenIndex);
//
//                no_space_before = token.hasNoSpaceBeforeDirective();
//                if ( !no_space_before && !no_space_after) {
//                    ((AlterChoiceImpl)_alternativesList.get(alternativeIndex)).add(new Word(" ", Type.WHITE_SPACE));
//                }
//                ((AlterChoiceImpl)_alternativesList.get(alternativeIndex)).add((EditorItem) token);
//                no_space_after = token.hasNoSpaceAfterDirective();
//            }
//        }
//    }

    public void addTokens(int index, ArrayList<Token> tokens)
    {
        _clean = false;

        AlterChoiceImpl alterChoice = (AlterChoiceImpl)_alternativesList.get(index);
        alterChoice.addTokens(tokens);
    }

    private void addToAll(EditorItem toAdd) {
        for (AlterChoice alt : _alternativesList){
            ((AlterChoiceImpl)alt).add((EditorItem) toAdd);
        }
    }

    public void removeDuplicates()
    {
        if (!_clean && _alternativesList.size() > 1)
        {
            ArrayList<AlterChoice> cleanAlternativesList = new ArrayList<AlterChoice>();
            ArrayList<String> alreadyAdded = new ArrayList<String>();

            //Always add the first one
            cleanAlternativesList.add(_alternativesList.get(0));
            alreadyAdded.add(_alternativesList.get(0).toString());

            for (int i = 1; i<_alternativesList.size(); i++)
            {
                AlterChoice choice = _alternativesList.get(i);

                if (!alreadyAdded.contains(choice.toString()))
                {
                    cleanAlternativesList.add(choice);
                    alreadyAdded.add(choice.toString());
                }
            }

            _alternativesList = cleanAlternativesList;
            _clean = true;
        }
    }
}
