/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.dragon.toolkit.recognition.dictation.internal;

import java.util.ArrayList;

import com.nuance.dragon.toolkit.oem.api.Logger;
import com.nuance.dragon.toolkit.recognition.dictation.Alternative;
import com.nuance.dragon.toolkit.recognition.dictation.Token;

class AlternativeImpl implements Alternative
{
    private ArrayList<Token> tokenList;

    public AlternativeImpl()
    {
        Logger.debug(this, "AlternativeImpl()");
        tokenList = new ArrayList<Token>();
    }

    public AlternativeImpl(ArrayList<Token> tokens)
    {
        Logger.debug(this, "AlternativeImpl(tokens)");
        tokenList = tokens;
    }

    public String toString()
    {
        if(tokenList.size() == 0)
            return "";

        StringBuffer sb = new StringBuffer("");
        boolean no_space_before = false;
        boolean no_space_after = true;
        for (int i = 0; i < tokenList.size(); ++ i) {

            TokenImpl token = (TokenImpl)(tokenList.get(i));
            no_space_before = token.hasNoSpaceBeforeDirective();

            if ( !no_space_before && !no_space_after) {
                sb.append(' ');
            }

            sb.append(this.tokenAt(i).toString());
            no_space_after = token.hasNoSpaceAfterDirective();
        }

        return sb.toString();
    }

    public int size()
    {
        int size = this.tokenList.size();
        return size;
    }

    public Token tokenAt(int index)
    {
        TokenImpl token = (TokenImpl)this.tokenList.get(index);
        return (Token) token;
    }
}
