/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.dragon.toolkit.recognition.dictation.internal;

import java.io.UnsupportedEncodingException;

import com.nuance.dragon.toolkit.oem.api.Logger;

/**
 * @author Pierre-Luc Bertrand ing jr
 */
public class Util
{
    public static String constructByteEncodedString(byte[] buffer, int offset, int len,
            String encoding)
    {
        String firstChars;
        try
        {
            //do not trim string because we have to support blankspace token.
            firstChars = new String(buffer, offset, len, encoding);
        } catch (UnsupportedEncodingException ex)
        {
            Logger.warn("dictation.Util", encoding + " character encoding is not available in your VM. Using the default one.");

            firstChars = (new String(buffer, offset, len));
        }
        return firstChars;
    }

    public static int bytesToUShort(byte[] from, int pos)
    {
        int low = from[pos] & 0xFF;
        int high = from[pos + 1] & 0xFF;
        return ((high << 8) | low);
    }

    public static int bytesToInt(byte[] from, int pos)
    {
        int low = bytesToUShort(from, pos) & 0x00FFFF;
        int high = bytesToUShort(from, pos + 2) & 0x00FFFF;
        return (high << 16) | low;
    }

    public static long bytesToUInt(byte[] from, int pos)
    {
        int firstByte = (0x000000FF & ((int)from[pos+3]));
        int secondByte = (0x000000FF & ((int)from[pos+2]));
        int thirdByte = (0x000000FF & ((int)from[pos+1]));
        int fourthByte = (0x000000FF & ((int)from[pos]));

        return ((long) (firstByte << 24
                | secondByte << 16
                | thirdByte << 8
                | fourthByte))
                & 0xFFFFFFFFL;
    }

    /**
         * Computes the length of a null terminated string contained in
         * a byte array by starting at the provided index.
         *
         * @param buffer
         *            The byte array containing the null terminated string
         * @param beginIndex
         *            The beginning index
         *
         * @return The length of a string inside the byte array
         **/
        public static int computeStrLen(byte[] buffer, int beginIndex)
        {
            Logger.debug("dictation.Util", "Computing string length");

            int index = beginIndex;
            while(index < buffer.length && buffer[index] != 0)
                index++;

            int len = index - beginIndex;

            Logger.debug("dictation.Util", "String length: [" + len + "]");
            return len;
        }
}
