/**
 *  Copyright 2006-2012, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.dragon.toolkit.recognition.dictation.parser;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import com.nuance.dragon.toolkit.recognition.dictation.DictationResult;
import com.nuance.dragon.toolkit.recognition.dictation.internal.DictationResultImpl;
import com.nuance.dragon.toolkit.recognition.interpretation.InterpretationResult;

//TODO need comments
public class XMLSaxParser implements XMLParser
{
    //private static final String DEFAULT_ENCODING = "cp1252";
    //private static final String UTF8_ENCODING = "UTF8";
    private byte[] buffer;
    private DictationResultImpl dictationResult;
    private InterpretationResult _interpResult;
    private String errorMsg = "";

    public interface ResultsHandler extends ContentHandler
    {
        public DictationResultImpl getDictationResult();
        public InterpretationResult getInterpretationResult();
    }

    protected ResultsHandler getResultHandler()
    {
        return new XMLResultsHandler();
    }

    public XMLSaxParser(byte[] buffer)
    {
        if(buffer == null)
            throw new IllegalArgumentException("Buffer cannot be null");
        this.buffer = buffer;
    }

    @Override
    public final void parse() throws IOException
    {
        try
        {
            System.setProperty("org.xml.sax.driver", "org.xmlpull.v1.sax2.Driver");
            InputSource is = new InputSource(new ByteArrayInputStream(this.buffer));

            XMLReader xmlReader = XMLReaderFactory.createXMLReader();
            ResultsHandler xmlHandler = getResultHandler();
            xmlReader.setContentHandler(xmlHandler);
            xmlReader.parse(is);

            dictationResult = xmlHandler.getDictationResult();
            _interpResult = xmlHandler.getInterpretationResult();
            // TODO: can retrieve the dictation result from the interpretation result later
        }
        catch(SAXException e)
        {
            errorMsg = e.getMessage();
        }
    }

    @Override
    public final DictationResult getDictationResult()
    {
        return this.dictationResult;
    }

    @Override
    public final InterpretationResult getInterpretationResult()
    {
        return _interpResult;
    }

    @Override
    public final String getErrorMsg()
    {
        return errorMsg;
    }

    @Override
    public final boolean resultIsValid()
    {
        return (this.dictationResult != null) && (this.dictationResult.size() > 0);
    }

}
