/**
 *  Copyright 2006-2012, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.dragon.toolkit.recognition.dictation.parser;

import java.io.IOException;

import com.nuance.dragon.toolkit.oem.api.Logger;
import com.nuance.dragon.toolkit.recognition.dictation.DictationResult;

//TODO need comments
public class XmlResultParser implements ResultParser
{
    private byte[] buffer;

    public XmlResultParser(byte[] buffer)
    {
        this.buffer = buffer;
    }

    @Override
    public DictationResult parse()
    {
        Logger.debug(this, "Unpacking XML dictation results.");

        try
        {
            XMLParser parserNLSML = new NLSMLSaxParser(this.buffer);
            parserNLSML.parse();
            if(parserNLSML.resultIsValid())
                return parserNLSML.getDictationResult();

            Logger.debug(this, "Could not parse NLSML dictation results: " +
                    parserNLSML.getErrorMsg() +
                    ". Trying to parse XML results.");

            XMLParser parserXML = new XMLSaxParser(this.buffer);
            parserXML.parse();
            if(parserXML.resultIsValid())
                return parserXML.getDictationResult();


            String errorMsg = "Could not parse XML neither " +
                    "NLSML dictation results. Error from XML Parser: " +
                    parserXML.getErrorMsg() + ". Error from NLSML Parser: " +
                    parserNLSML.getErrorMsg();

            Logger.error(this, errorMsg);

            throw new IllegalArgumentException(errorMsg);
        }
        catch(IOException e)
        {
            String errorMsg = "Received IOException while parsing XML/NLSML.";

            Logger.error(this, errorMsg, e);

            throw new IllegalArgumentException(errorMsg, e);
        }
    }
}
