package com.nuance.dragon.toolkit.recognition.interpretation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.nuance.dragon.toolkit.recognition.dictation.DictationResult;
import com.nuance.dragon.toolkit.recognition.dictation.internal.SentenceImpl;

// TODO Need comments
public class InterpretationImpl extends SentenceImpl implements Interpretation
{
    private SentenceImpl _innerSentence;
    private final List<RawPiece> _pieceList;
    private final List<Slot> _slots;
    private final Map<String, Slot> _slotMap;
    private String _slotType;
    private final double _confidenceScore;

    public InterpretationImpl(double confidenceScore)
    {
        _pieceList = new ArrayList<RawPiece>();
        _slots = new ArrayList<Slot>();
        _slotMap = new HashMap<String, Slot>();
        _confidenceScore = confidenceScore;
    }

    @Override
    public List<RawPiece> getRecognitionPieces()
    {
        return _pieceList;
    }

    public SentenceImpl getInnerSentence()
    {
        return _innerSentence;
    }

    public void setInnerSentence(SentenceImpl sentence)
    {
        _innerSentence = sentence;
    }

    public void addPiece(RawPiece piece)
    {
        _pieceList.add(piece);
    }

    public void addSlot(String key, String value, double confidenceScore)
    {
        Slot slot = new Slot(key, value, confidenceScore);
        _slots.add(slot);
        _slotMap.put(key,  slot);
    }

    public void addSlot(String key, DictationResult dictationResult)
    {
        Slot slot = new Slot(key, dictationResult);
        _slots.add(slot);
        _slotMap.put(key, slot);
    }

    public void setSlotType(String slotType)
    {
        _slotType = slotType;
    }

    @Override
    public List<Slot> getSlots()
    {
        return _slots;
    }

    @Override
    public Slot getSlot(String name)
    {
        return _slotMap.get(name);
    }

    @Override
    public String getRecogType()
    {
        return _slotType;
    }

    @Override
    public double getConfidenceScore()
    {
        return _confidenceScore;
    }

    @Override
    public String toString()
    {
        return _innerSentence != null ? _innerSentence.toString() : "";
    }
}
