package com.nuance.dragon.toolkit.recognition.interpretation;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.nuance.dragon.toolkit.recognition.dictation.DictationResult;
import com.nuance.dragon.toolkit.recognition.dictation.internal.DictationResultImpl;
import com.nuance.dragon.toolkit.recognition.dictation.internal.SentenceImpl;
import com.nuance.dragon.toolkit.util.internal.ReadOnlyList;

/**
 * A result from a server ASR command containing possible interpretations.
 */
public class InterpretationResult
{
    /**
     * In case we want to treat this interpretation result as a dictation
     * result, we can just use this inner dictation result.
     */
    private final DictationResultImpl _innerDictationResult;

    /**
     * The open dictation pieces mapped by the argument keywords, we only
     * save one open dictation result all the open dictation pieces in each
     * interpretation object.
     *
     * For example: "searcharg" for WEB_SEARCH
     *              "subject" for SEND_MESSAGE
     */
//    private final Map<String, DictationResult> _openDictationDepot;
    private final List<Interpretation> _nInterpList;

    public InterpretationResult(List<InterpretationImpl> nInterpList,  Map<String, String> extraInformation)
    {
        ArrayList<SentenceImpl> nbestList = new ArrayList<SentenceImpl>();
        for (InterpretationImpl interp : nInterpList)
            nbestList.add(interp.getInnerSentence());

        _innerDictationResult = new DictationResultImpl(nbestList, extraInformation);
        _nInterpList = new ReadOnlyList<Interpretation>(nInterpList);
    }

    /**
     * Get the list of interpretations, in order of confidence (i.e. element 0 will be the best choice).
     * @return The interpretations. Will always have at least 1 element.
     */
    public List<Interpretation> getInterpretations()
    {
        return _nInterpList;
    }

    /**
     * Get a dictation result that represents the same recognition as this interpretation result.
     * @return The dictation result. Never null.
     */
    public DictationResult asDictationResult()
    {
        return _innerDictationResult;
    }
}
