package com.nuance.dragon.toolkit.remoting;

import com.nuance.dragon.toolkit.oem.api.json.JSONObject;


/**
 * A service that is capable of sending and handling messages.
 */
public abstract class Service
{
    private final String _name;
    protected final Connection _messageConnection;

    /**
     * @param name The unique name of this service.
     * @param messageConnection The connection that this service should use when
     * sending messages to other services.
     */
    public Service(String name, Connection messageConnection)
    {
        _name = name;
        _messageConnection = messageConnection;
    }

    /**
     * Get the name of the service.
     * @return The name.
     */
    public final String getName()
    {
        return _name;
    }

    /**
     * Send a message to another service.
     * @param to The destination service.
     * @param messageId The ID of the message to send.
     * @param message The contents of the message.
     * @return true if a receiver was found for the message; false otherwise.
     */
    protected boolean sendMessage(String to, String messageId, JSONObject message)
    {
        return _messageConnection.sendMessage(to, _name, messageId, message);
    }

    /**
     * Handle a message that was sent to this service.
     * @param from Where the message came from.
     * @param messageId The ID of the message.
     * @param message The contents of the message.
     */
    public abstract void handleMessage(String from, String messageId, JSONObject message);
}
