package com.nuance.dragon.toolkit.remoting;

import java.util.HashMap;

import com.nuance.dragon.toolkit.oem.api.Logger;
import com.nuance.dragon.toolkit.oem.api.json.JSONObject;

/**
 * A connection implementation that is terminal, i.e. has services directly
 * attached to it.
 */
public class TerminalConnection extends Connection
{
    private final HashMap<String, Service> _services;

    public TerminalConnection(String name)
    {
        super(name);
        _services = new HashMap<String, Service>();
    }

    /**
     * Add a service to be monitored by this terminal connection.
     * @param service
     */
    public void addService(Service service)
    {
        String name = service.getName();
        if (_services.containsValue(service) || _services.containsKey(name)) {
            Logger.error(this, "Service '" + name + "' already added");
            return;
        }

        Logger.debug(this, "Adding service '" + name + "'");
        _services.put(name, service);

        onServiceConnected(name, this);
    }

    /**
     * Remove a service.
     * @param service
     */
    public void removeService(Service service)
    {
        String name = service.getName();
        if (_services.remove(name) == null)
        {
            Logger.warn(this, "Couldn't find service '" + name + "' to remove");
        }

        onServiceDisconnected(name, this);
    }

    @Override
    public boolean sendMessage(String to, String from, String messageId, JSONObject message)
    {
        if (_services.containsKey(to))
        {
            Service s = _services.get(to);
            s.handleMessage(from, messageId, message);
            return true;
        }
        return super.sendMessage(to, from, messageId, message);
    }
}
