package com.nuance.dragon.toolkit.speechkit;

import java.util.ArrayList;

import com.nuance.dragon.toolkit.audio.AudioType;
import com.nuance.dragon.toolkit.cloudservices.CalllogConfig;
import com.nuance.dragon.toolkit.cloudservices.CloudConfig;
import com.nuance.dragon.toolkit.cloudservices.NMTConfig;
import com.nuance.dragon.toolkit.cloudservices.SSLConfig;

/**
 * A cloud configuration for SpeechKit. Based on {@link CloudConfig}, but:
 * <br> a) Always uses {@link AudioType#SPEEX_WB}, and
 * <br> b) Uses a self-signed certificate for SSL
 * <br> c) Uses a randomly generated device ID.
 */
public final class SKCloudConfig extends CloudConfig
{
    public SKCloudConfig(String host, int port, String appId, byte[] appKey, final boolean useSsl)
    {
        super(null, host, port, appId, appKey,
              new ArrayList<NMTConfig>() {
                  private static final long serialVersionUID = 1L; {
                      add(new CalllogConfig(true));
                      if (useSsl)
                          add(new SSLConfig(true, null, null));
              }},
              null, AudioType.SPEEX_WB, AudioType.SPEEX_WB);
    }
}
