package com.nuance.dragon.toolkit.sse;

public interface NativeSse
{
    boolean createSse();

    void destroySse();

    /**
     * Initialize with the given sample rate
     * @param sampleRate 8000 or 16000
     * @param rawConfigData raw configData
     * @return true if successful
     */
    boolean initialize(int sampleRate,
                       int micNum,
                       boolean useEchoCancel,
                       boolean useNoiseSuppression,
                       boolean useBeamForming,
                       boolean usePic,
                       byte[] rawConfigData);

    /**
     * Process using reference audio
     * @param microphone
     * @param reference If null, SSE will match the microphone audio against
     * silence
     * @return The output audio, or null if an error occurred
     */
    short[] processRef(short[] microphone, short[] reference);

    short[][] processRecv(short[] microphone, short[] receive);

    short[] processPic(short[][] microphoneIns);

    /**
     * Get VoCon configuration data (only call when doing ASR / WUW / EP using VoCon).
     * @return blob of data only meaningful to VoCon.
     */
    public byte[] getVoconConfiguration();

    /**
     * Get embedded SSE library version
     * @return SSE library version string
     */
    public String getVersion();
}
