package com.nuance.dragon.toolkit.util;

import android.os.Bundle;

import com.nuance.dragon.toolkit.audio.util.OpusEncoder;
import com.nuance.dragon.toolkit.audio.util.SpeexEncoder;
import com.nuance.dragon.toolkit.edr.internal.EdrRecognizerImpl;
import com.nuance.dragon.toolkit.elvis.ElvisRecognizer;
import com.nuance.dragon.toolkit.nvsl.NVSLIdentifier;
import com.nuance.dragon.toolkit.oem.api.Logger;
import com.nuance.dragon.toolkit.oem.api.internal.Checker;
import com.nuance.dragon.toolkit.util.internal.NativeUtils;
import com.nuance.dragon.toolkit.vocalizer.Vocalizer;
import com.nuance.dragon.toolkit.vocon.VoconRecognizer;

/**
 * Convenience diagnostic object that indicates if there is something fundamentally
 * wrong with the toolkit (example: native libraries not found). Essentially useful for
 * client applications distributed through Google Play.
 */
public final class Diagnostic
{
    /**
     * Indicate if the toolkit is installed correctly
     * (i.e. all required native components are present).
     * @param requirement The application requirement object.
     * @return True if the toolkit is installed correctly on the device, else false.
     */
    public static boolean checkInstallation(Bundle requirement)
    {
        Checker.checkArgForNull("requirement", requirement);
        boolean ok = true;

        if (requirement.getBoolean("elvis", false) && !ElvisRecognizer.LOAD_NATIVE_LIBRARY_SUCCESS) {
            Logger.error(Diagnostic.class, "Elvis is not installed.");
            ok = false;
        }
        if (requirement.getBoolean("vocon", false) && !VoconRecognizer.LOAD_NATIVE_LIBRARY_SUCCESS) {
            Logger.error(Diagnostic.class, "VoCon is not installed.");
            ok = false;
        }
        if (requirement.getBoolean("nvsl", false) && !NVSLIdentifier.LOAD_NATIVE_LIBRARY_SUCCESS) {
            Logger.error(Diagnostic.class, "NVSL is not installed.");
            ok = false;
        }
        if (requirement.getBoolean("vfa", false) && !Vocalizer.LOAD_NATIVE_VFA_LIBRARY_SUCCESS) {
            Logger.error(Diagnostic.class, "Vocalizer for Automotive is not installed.");
            ok = false;
        }
        if (requirement.getBoolean("vex", false) && !Vocalizer.LOAD_NATIVE_VEX_LIBRARY_SUCCESS) {
            Logger.error(Diagnostic.class, "Vocalizer Expressive is not installed.");
            ok = false;
        }
        if (requirement.getBoolean("speex", false) && !SpeexEncoder.LOAD_NATIVE_LIBRARY_SUCCESS) {
            Logger.error(Diagnostic.class, "SPEEX is not installed.");
            ok = false;
        }
        if (requirement.getBoolean("opus", false) && !OpusEncoder.LOAD_NATIVE_LIBRARY_SUCCESS) {
            Logger.error(Diagnostic.class, "OPUS is not installed.");
            ok = false;
        }
        if (requirement.getBoolean("edr", false) && !EdrRecognizerImpl.LOAD_NATIVE_LIBRARY_SUCCESS) {
            Logger.error(Diagnostic.class, "Embeded Dictation Recognizer is not installed.");
            ok = false;
        }
        if (!NativeUtils.LOAD_NATIVE_LIBRARY_SUCCESS) {
            Logger.error(Diagnostic.class, "Native utilities are not installed.");
            ok = false;
        }
        // TODO: check more stuff?
        return ok;
    }
}
