package com.nuance.dragon.toolkit.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;

import org.json.JSONException;

import com.nuance.dragon.toolkit.oem.api.internal.Checker;


public class JSONUtils
{
    /**
     * Read a JSON object from a file
     * @param file
     * @return The JSON object (never null)
     * @throws IOException
     * @throws JSONException
     */
    public static org.json.JSONObject readFromFile(File file) throws IOException, JSONException
    {
        Checker.checkArgForNull("file", file);

        return readFromStream(new FileInputStream(file));
    }

    /**
     * Read a JSON object from a stream. Will close the stream when finished!
     * @param is
     * @return The JSON object (never null)
     * @throws IOException
     * @throws JSONException
     */
    public static org.json.JSONObject readFromStream(InputStream is) throws IOException, JSONException
    {
        Checker.checkArgForNull("is", is);

        InputStreamReader isr = null;

        StringBuffer buffer = new StringBuffer();
        try {
            isr = new InputStreamReader(is);

            char [] chars = new char[1024];
            do
            {
                int read = isr.read(chars);
                if (read <= 0)
                    break;

                buffer.append(chars, 0, read);
            } while (true);
            isr.close();
        }
        finally {
            if (isr != null) isr.close();
            else if (is != null) is.close();
        }

        return new org.json.JSONObject(buffer.toString());
    }

    /**
     * Save a JSON object to a file
     * @param json
     * @param file
     * @throws IOException
     * @throws JSONException
     */
    public static void saveToFile(org.json.JSONObject json, File file)  throws IOException, JSONException
    {
        Checker.checkArgForNull("file", file);

        saveToStream(json, new FileOutputStream(file));
    }

    /**
     * Write a JSON object to a stream. Will close the stream when finished!
     * @param json
     * @param os
     * @throws IOException
     * @throws JSONException
     */
    public static void saveToStream(org.json.JSONObject json, OutputStream os) throws IOException, JSONException
    {
        Checker.checkArgForNull("json", json);
        Checker.checkArgForNull("os", os);

        OutputStreamWriter osw = null;
        try {
            osw = new OutputStreamWriter(os);
            osw.write(json.toString());
        } finally {
            if (osw != null) osw.close();
            else if (os != null) os.close();
        }
    }
}
