package com.nuance.dragon.toolkit.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.IntentSender;
import android.content.IntentSender.SendIntentException;
import android.content.ServiceConnection;
import android.content.SharedPreferences;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.content.res.AssetManager;
import android.content.res.Resources;
import android.content.res.Resources.Theme;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteDatabase.CursorFactory;
import android.graphics.Bitmap;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;

import com.nuance.dragon.toolkit.oem.api.NMTAssetFileDescriptor;
import com.nuance.dragon.toolkit.oem.api.NMTContext;

public class NMTContextAndroid extends Context implements NMTContext
{
    private final NMTContext _nmtContext;
    private final Context _context;
    
    NMTContextAndroid(NMTContext nmtContext)
    {
        _nmtContext = nmtContext;
        _context = (Context)nmtContext.getNativeContext();
    }

    /******************* Pass-thru to ContextOem methods *******************/
    @Override
    public NMTAssetFileDescriptor openRawResourceFd(int id) {
        return _nmtContext.openRawResourceFd(id);
    }

    @Override
    public NMTAssetFileDescriptor openFd(String fileName) {
        return _nmtContext.openFd(fileName);
    }

    @Override
    public File getFilesDir() {
        return _nmtContext.getFilesDir();
    }

    @Override
    public File getExternalStorageDirectory() {
        return _nmtContext.getExternalStorageDirectory();
    }

    @Override
    public String[] listAssets(String path) {
        return _nmtContext.listAssets(path);
    }

    @Override
    public NMTContext getAppContext() {
        return _nmtContext.getAppContext();
    }

    @Override
    public Object getNativeContext() {
        return _nmtContext.getNativeContext();
    }
    
    /******************* Pass-thru to Android Context methods *******************/
    @Override
    public boolean bindService(Intent service, ServiceConnection conn, int flags) {
        return _context.bindService(service, conn, flags);
    }

    @Override
    public int checkCallingOrSelfPermission(String permission) {
        return _context.checkCallingOrSelfPermission(permission);
    }

    @Override
    public int checkCallingOrSelfUriPermission(Uri uri, int modeFlags) {
        return _context.checkCallingOrSelfUriPermission(uri, modeFlags);
    }

    @Override
    public int checkCallingPermission(String permission) {
        return _context.checkCallingPermission(permission);
    }

    @Override
    public int checkCallingUriPermission(Uri uri, int modeFlags) {
        return _context.checkCallingUriPermission(uri, modeFlags);
    }

    @Override
    public int checkPermission(String permission, int pid, int uid) {
        return _context.checkPermission(permission, pid, uid);
    }

    @Override
    public int checkUriPermission(Uri uri, int pid, int uid, int modeFlags) {
        return _context.checkUriPermission(uri, pid, uid, modeFlags);
    }

    @Override
    public int checkUriPermission(Uri uri, String readPermission,
            String writePermission, int pid, int uid, int modeFlags) {
        return _context.checkUriPermission(uri, readPermission, writePermission, pid, uid, modeFlags);
    }

    @Override
    public void clearWallpaper() throws IOException {
        _context.clearWallpaper();
    }

    @Override
    public Context createPackageContext(String packageName, int flags)
            throws NameNotFoundException {
        return _context.createPackageContext(packageName, flags);
    }

    @Override
    public String[] databaseList() {
        return _context.databaseList();
    }

    @Override
    public boolean deleteDatabase(String name) {
        return _context.deleteDatabase(name);
    }

    @Override
    public boolean deleteFile(String name) {
        return _context.deleteFile(name);
    }

    @Override
    public void enforceCallingOrSelfPermission(String permission, String message) {
        _context.enforceCallingOrSelfPermission(permission, message);
    }

    @Override
    public void enforceCallingOrSelfUriPermission(Uri uri, int modeFlags,
            String message) {
        _context.enforceCallingOrSelfUriPermission(uri, modeFlags, message);
    }

    @Override
    public void enforceCallingPermission(String permission, String message) {
        _context.enforceCallingPermission(permission, message);
    }

    @Override
    public void enforceCallingUriPermission(Uri uri, int modeFlags,
            String message) {
        _context.enforceCallingUriPermission(uri, modeFlags, message);
    }

    @Override
    public void enforcePermission(String permission, int pid, int uid,
            String message) {
        _context.enforcePermission(permission, pid, uid, message);
    }

    @Override
    public void enforceUriPermission(Uri uri, int pid, int uid, int modeFlags,
            String message) {
        _context.enforceUriPermission(uri, pid, uid, modeFlags, message);
    }

    @Override
    public void enforceUriPermission(Uri uri, String readPermission,
            String writePermission, int pid, int uid, int modeFlags,
            String message) {
        _context.enforceUriPermission(uri, readPermission, writePermission, pid, uid, modeFlags, message);
    }

    @Override
    public String[] fileList() {
        return _context.fileList();
    }

    @Override
    public Context getApplicationContext() {
        return _context.getApplicationContext();
    }

    @Override
    public ApplicationInfo getApplicationInfo() {
        return _context.getApplicationInfo();
    }

    @Override
    public AssetManager getAssets() {
        return _context.getAssets();
    }

    @Override
    public File getCacheDir() {
        return _context.getCacheDir();
    }

    @Override
    public ClassLoader getClassLoader() {
        return _context.getClassLoader();
    }

    @Override
    public ContentResolver getContentResolver() {
        return _context.getContentResolver();
    }

    @Override
    public File getDatabasePath(String name) {
        return _context.getDatabasePath(name);
    }

    @Override
    public File getDir(String name, int mode) {
        return _context.getDir(name, mode);
    }

    @Override
    public File getExternalCacheDir() {
        return _context.getExternalCacheDir();
    }

    @Override
    public File getExternalFilesDir(String type) {
        return _context.getExternalFilesDir(type);
    }

    @Override
    public File getFileStreamPath(String name) {
        return _context.getFileStreamPath(name);
    }

    @Override
    public Looper getMainLooper() {
        return _context.getMainLooper();
    }

    @Override
    public String getPackageCodePath() {
        return _context.getPackageCodePath();
    }

    @Override
    public PackageManager getPackageManager() {
        return _context.getPackageManager();
    }

    @Override
    public String getPackageName() {
        return _context.getPackageName();
    }

    @Override
    public String getPackageResourcePath() {
        return _context.getPackageResourcePath();
    }

    @Override
    public Resources getResources() {
        return _context.getResources();
    }

    @Override
    public SharedPreferences getSharedPreferences(String name, int mode) {
        return _context.getSharedPreferences(name, mode);
    }

    @Override
    public Object getSystemService(String name) {
        return _context.getSystemService(name);
    }

    @Override
    public Theme getTheme() {
        return _context.getTheme();
    }

    @Override
    public Drawable getWallpaper() {
        return _context.getWallpaper();
    }

    @Override
    public int getWallpaperDesiredMinimumHeight() {
        return _context.getWallpaperDesiredMinimumHeight();
    }

    @Override
    public int getWallpaperDesiredMinimumWidth() {
        return _context.getWallpaperDesiredMinimumWidth();
    }

    @Override
    public void grantUriPermission(String toPackage, Uri uri, int modeFlags) {
        _context.grantUriPermission(toPackage, uri, modeFlags);
    }

    @Override
    public FileInputStream openFileInput(String name)
            throws FileNotFoundException {
        return _context.openFileInput(name);
    }

    @Override
    public FileOutputStream openFileOutput(String name, int mode)
            throws FileNotFoundException {
        return _context.openFileOutput(name, mode);
    }

    @Override
    public SQLiteDatabase openOrCreateDatabase(String name, int mode,
            CursorFactory factory) {
        return _context.openOrCreateDatabase(name, mode, factory);
    }

    @Override
    public Drawable peekWallpaper() {
        return _context.peekWallpaper();
    }

    @Override
    public Intent registerReceiver(BroadcastReceiver receiver,
            IntentFilter filter) {
        return _context.registerReceiver(receiver, filter);
    }

    @Override
    public Intent registerReceiver(BroadcastReceiver receiver,
            IntentFilter filter, String broadcastPermission, Handler scheduler) {
        return _context.registerReceiver(receiver, filter, broadcastPermission, scheduler);
    }

    @Override
    public void removeStickyBroadcast(Intent intent) {
        _context.removeStickyBroadcast(intent);
    }

    @Override
    public void revokeUriPermission(Uri uri, int modeFlags) {
        _context.revokeUriPermission(uri, modeFlags);
    }

    @Override
    public void sendBroadcast(Intent intent) {
        _context.sendBroadcast(intent);
    }

    @Override
    public void sendBroadcast(Intent intent, String receiverPermission) {
        _context.sendBroadcast(intent, receiverPermission);
    }

    @Override
    public void sendOrderedBroadcast(Intent intent, String receiverPermission) {
        _context.sendOrderedBroadcast(intent, receiverPermission);
    }

    @Override
    public void sendOrderedBroadcast(Intent intent, String receiverPermission,
            BroadcastReceiver resultReceiver, Handler scheduler,
            int initialCode, String initialData, Bundle initialExtras) {
        _context.sendOrderedBroadcast(intent, receiverPermission, resultReceiver, scheduler, initialCode, initialData, initialExtras);
    }

    @Override
    public void sendStickyBroadcast(Intent intent) {
        _context.sendStickyBroadcast(intent);
    }

    @Override
    public void sendStickyOrderedBroadcast(Intent intent,
            BroadcastReceiver resultReceiver, Handler scheduler,
            int initialCode, String initialData, Bundle initialExtras) {
        _context.sendStickyOrderedBroadcast(intent, resultReceiver, scheduler, initialCode, initialData, initialExtras);
    }

    @Override
    public void setTheme(int resid) {
        _context.setTheme(resid);
    }

    @Override
    public void setWallpaper(Bitmap bitmap) throws IOException {
        _context.setWallpaper(bitmap);
    }

    @Override
    public void setWallpaper(InputStream data) throws IOException {
        _context.setWallpaper(data);
    }

    @Override
    public void startActivity(Intent intent) {
        _context.startActivity(intent);
    }

    @Override
    public boolean startInstrumentation(ComponentName className,
            String profileFile, Bundle arguments) {
        return _context.startInstrumentation(className, profileFile, arguments);
    }

    @Override
    public void startIntentSender(IntentSender intent, Intent fillInIntent,
            int flagsMask, int flagsValues, int extraFlags)
            throws SendIntentException {
        _context.startIntentSender(intent, fillInIntent, flagsMask, flagsValues, extraFlags);
    }

    @Override
    public ComponentName startService(Intent service) {
        return _context.startService(service);
    }

    @Override
    public boolean stopService(Intent service) {
        return _context.stopService(service);
    }

    @Override
    public void unbindService(ServiceConnection conn) {
        _context.unbindService(conn);
    }

    @Override
    public void unregisterReceiver(BroadcastReceiver receiver) {
        _context.unregisterReceiver(receiver);
    }
}
