package com.nuance.dragon.toolkit.util.internal;

import java.util.AbstractList;
import java.util.ArrayList;
import java.util.List;

/**
 * A simple read-only list.
 * @param <T> The list type
 */
public class ReadOnlyList<T extends Object> extends AbstractList<T>
{
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = super.hashCode();
        result = prime * result + ((_array == null) ? 0 : _array.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (!super.equals(obj))
            return false;
        if (getClass() != obj.getClass())
            return false;
        ReadOnlyList<?> other = (ReadOnlyList<?>) obj;
        if (_array == null) {
            if (other._array != null)
                return false;
        } else if (!_array.equals(other._array))
            return false;
        return true;
    }

    private final ArrayList<T> _array;

    public ReadOnlyList(List<? extends T> items)
    {
        _array = new ArrayList<T>(items.size());
        _array.addAll(items);
    }

    public ReadOnlyList(T... items)
    {
        _array = new ArrayList<T>(items.length);
        for (T item : items)
            _array.add(item);
    }

    public ReadOnlyList(T item)
    {
        _array = new ArrayList<T>(1);
        _array.add(item);
    }

    public ReadOnlyList()
    {
        _array = new ArrayList<T>(0);
    }

    protected ReadOnlyList(int initialCapacity)
    {
        _array = new ArrayList<T>(initialCapacity);
    }

    /**
     * Subclasses may add to the list by calling this method.
     * @param item
     */
    protected final void addInternal(T item)
    {
        _array.add(item);
    }

    @Override
    public T get(int location)
    {
        return _array.get(location);
    }

    @Override
    public int size()
    {
        return _array.size();
    }
}
