package com.nuance.dragon.toolkit.vocalizer;

import com.nuance.dragon.toolkit.oem.api.Factory;
import com.nuance.dragon.toolkit.oem.api.FileManager;
import com.nuance.dragon.toolkit.oem.api.NMTHandler;
import com.nuance.dragon.toolkit.oem.api.WorkerThread;
import com.nuance.dragon.toolkit.oem.api.internal.Checker;

/**
 * Implementation of {@link Vocalizer}.
 * <p>
 * Don't create this directly. Use {@link Vocalizer#createVocalizer}.
 */
final class VocalizerImpl extends VocalizerBase
{
    private WorkerThread _workerThread;

    /**
     * Create a Vocalizer that uses a dedicated worker thread.
     * @param engine The engine used for TTS generation, see {@link Engine}.
     * @param fileManager The file manager where Vocalizer should look for files.
     */
    public VocalizerImpl(Engine engine, FileManager fileManager)
    {
        this(engine, fileManager, null);
    }

    /**
     * Create a Vocalizer that uses the given worker thread handler.
     * @param engine The engine used for TTS generation, see {@link Engine}.
     * @param fileManager The file manager where Vocalizer should look for files.
     * @param workerThread If this is null, will create a new one
     */
    public VocalizerImpl(Engine engine, FileManager fileManager, NMTHandler workerThread)
    {
        super(engine == Engine.VOCALIZER_EXPRESSIVE ?
              new NativeVocalizerExpressiveImpl(fileManager) :
              new NativeVocalizerAutomotiveImpl(fileManager),
              workerThread);
        Checker.checkArgForNull("fileManager", fileManager);
    }

    @Override
    public void release()
    {
        super.release();
        if (_workerThread != null)
        {
            _workerThread.stop();
            _workerThread = null;
        }
    }

    @Override
    protected NMTHandler getWorkerThreadHandler()
    {
        _workerThread = Factory.createWorkerThread("com.nuance.dragon.toolkit.vocalizer.VocalizerImpl");
        _workerThread.start();
        return _workerThread.getHandler();
    }
}
