package com.nuance.dragon.toolkit.vocalizer;

import org.json.JSONException;

import com.nuance.dragon.toolkit.oem.api.JSONCompliant;
import com.nuance.dragon.toolkit.oem.api.json.JSONObject;

/**
 * A Vocalizer voice. Note that not all voices are necessarily supported--supported
 * languages and voices are dependent on the original build configuration of the
 * Vocalizer engine, and the model files present on the device.
 */
public final class VocalizerVoice implements JSONCompliant
{
    private static final String KEY_NAME = "name";

    /** The voice name. This is the string passed to the Vocalizer engine. */
    public final String name;
    /** A description of the voice. */
    public final String description;
    /** The language that this voice belongs to */
    public final VocalizerLanguage language;

    /**
     * @param voice The name of the voice
     * @param desc The description of the voice
     * @param language The supported language
     */
    VocalizerVoice(String voice, String desc, VocalizerLanguage language)
    {
        this.name = voice;
        this.description = desc;
        this.language = language;
    }

    @Override
    public String toString()
    {
        return description;
    }

    @Override
    public boolean equals(Object item)
    {
        if (item == null)
            return false;

        if (!(item instanceof VocalizerVoice))
            return false;

        VocalizerVoice voice = (VocalizerVoice)item;
        return name.equals(voice.name);
    }

    @Override
    public int hashCode()
    {
        return name.hashCode();
    }

    @Override
    public org.json.JSONObject toJSON()
    {
        // Only store 'name'
        JSONObject o = new JSONObject();
        o.tryPut(KEY_NAME, name);
        return o;
    }

    public static VocalizerVoice createFromJSON(org.json.JSONObject json) throws JSONException
    {
        // Don't create a new Voice; rather, always make sure we return one of the valid ones
        String name = json.getString(KEY_NAME);
        VocalizerVoice v = findByName(name);
        if (v != null)
        {
            return v;
        }
        throw new JSONException("Invalid voice name: " + name);
    }

    static VocalizerVoice findByName(String name)
    {
        for (VocalizerVoice voice : VoiceSpecs.ALL_VOICES)
        {
            if (voice.name.equalsIgnoreCase(name))
                return voice;
        }
        return null;
    }
}
