package com.nuance.dragon.toolkit.vocon;

public class GrammarLoadParam
{

    /**
     * Builder for creating instances of {@link GrammarLoadParam}.
     */
    public static class Builder
    {
        private boolean onlineCompile;
        private String delimiters;

        /**
         * Creates the builder with mandatory parameter(s).
         */
        public Builder()
        {
            this.onlineCompile = false;
            this.delimiters = null;
        }

        /**
         * Sets true to indicate dynamic words will be compiled online.
         * By default (false), dynamic words will be added to word slot one by one.
         * <p>
         * Note: This option is only applicable to domain "name" {@link DynamicSlot.Domains}
         * </p>
         */
        public Builder setOnlineCompile(boolean onlineCompile)
        {
            this.onlineCompile = onlineCompile;
            return this;
        }

        /**
         * Sets delimiters which will be used to split dynamic words into multiple ones.
         * Then they are added back to VoCon engine in one shot. It will increase transcription
         * combination and improve recognition accuracy.
         * By default, delimiters is set to null, meaning no word will be split.
         *
         * <p>
         * Note: '#' should not be used as delimiter
         */
        public Builder setDelimiters(String delimiters)
        {
            this.delimiters = delimiters;
            return this;
        }

        /**
         * Builds the parameters.
         * @return The {@link RecognitionParam} instance.
         */
        public GrammarLoadParam build()
        {
            return new GrammarLoadParam(this);
        }
    }

    private final boolean _isOnlineCompile;
    private final String _delimiters;

    private GrammarLoadParam(Builder builder)
    {
        this._isOnlineCompile = builder.onlineCompile;
        this._delimiters = builder.delimiters;
    }

    boolean isOnlineCompile()
    {
        return _isOnlineCompile;
    }

    String getDelimiters()
    {
        return _delimiters;
    }

    @Override
    public String toString()
    {
        return "RecognitionParam [_delimiters=" + _delimiters
                + ", _isOnlineCompile=" + _isOnlineCompile
                + "]";
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + (_isOnlineCompile ? 1231 : 1237);
        if (_delimiters != null)
            result = prime * result + _delimiters.hashCode();
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        GrammarLoadParam other = (GrammarLoadParam) obj;
        if (_isOnlineCompile != other._isOnlineCompile)
            return false;
        if (_delimiters == null) {
            if (other._delimiters != null)
                return false;
        } else if (!_delimiters.equals(other._delimiters))
            return false;
        return true;
    }
}
