package com.nuance.dragon.toolkit.vocon;

import java.util.Map;

public class RecognitionParam {

    /** Default parameters */
    public static final boolean IS_BLOCKING_ISSUE_RESULTS = false;

    /**
     * Builder for creating instances of {@link RecognitionParam}.
     */
    public static class Builder
    {
        private Map<VoconParam, Integer> fxParams;
        private boolean isBlockingIssueResults;

        /**
         * Creates the builder with mandatory parameter(s).
         */
        public Builder(Map<VoconParam, Integer> fxParams)
        {
            this.fxParams = fxParams;
            this.isBlockingIssueResults = IS_BLOCKING_ISSUE_RESULTS;
        }

        /**
         * Sets to allow blocking issue of results. If multiple contexts are loaded,
         * VoCon's worker thread will block after giving each result to allow user to call {@link VoconRecognizer#stopRecognitionResults()}
         * to stop VoCon from providing the following results. Since this would slow down worker thread,
         * it is recommended to use this flag only if the recognition uses multiple contexts.
         */
        public Builder setBlockingIssueResults()
        {
            isBlockingIssueResults = true;
            return this;
        }

        /**
         * Builds the parameters.
         * @return The {@link RecognitionParam} instance.
         */
        public RecognitionParam build()
        {
            return new RecognitionParam(this);
        }
    }

    private final Map<VoconParam, Integer> _fxParams;
    private final boolean _isBlockingIssueResults;

    private RecognitionParam(Builder builder)
    {
        this._fxParams = builder.fxParams;
        this._isBlockingIssueResults = builder.isBlockingIssueResults;
    }

    Map<VoconParam, Integer> getFxParams()
    {
        return _fxParams;
    }

    boolean isBlockingIssueResults()
    {
        return _isBlockingIssueResults;
    }

    @Override
    public String toString()
    {
        return "RecognitionParam [_fxParams=" + _fxParams.toString()
                + ", _isBlockingIssueResults=" + _isBlockingIssueResults
                + "]";
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + (_isBlockingIssueResults ? 1231 : 1237);
        result = prime * result + _fxParams.hashCode();
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        RecognitionParam other = (RecognitionParam) obj;
        if (_isBlockingIssueResults != other._isBlockingIssueResults)
            return false;
        if (_fxParams == null) {
            if (other._fxParams != null)
                return false;
        } else if (!_fxParams.equals(other._fxParams))
            return false;
        return true;
    }
}