package com.nuance.dragon.toolkit.vocon;

import org.json.JSONException;

import com.nuance.dragon.toolkit.oem.api.JSONCompliant;
import com.nuance.dragon.toolkit.oem.api.json.JSONObject;

/**
 * A Vocon language.
 */
public class VoconLanguage implements JSONCompliant
{
    private static final String KEY_NAME = "name";

    /** The language name. This is the string passed to the Vocon engine. */
    public final String name;
    /** A description of the language. */
    public final String description;

    VoconLanguage(String language, String description)
    {
        this.name = language;
        this.description = description;
    }

    @Override
    public String toString()
    {
        return description;
    }

    @Override
    public boolean equals(Object other)
    {
        if (other == null)
            return false;

        if (!(other instanceof VoconLanguage))
            return false;

        VoconLanguage lang = (VoconLanguage)other;
        if (!lang.name.equals(name))
            return false;

        return true;
    }

    @Override
    public org.json.JSONObject toJSON()
    {
        JSONObject o = new JSONObject();
        o.tryPut(KEY_NAME, name);
        return o;
    }

    public static VoconLanguage createFromJSON(org.json.JSONObject json) throws JSONException
    {
        String name = json.getString(KEY_NAME);
        for (VoconLanguage l : LangSpecs.ALL_LANGUAGES)
        {
            if (l.name.equals(name))
            {
                return l;
            }
        }
        throw new JSONException("Invalid language name: " + name);
    }
}
