package com.nuance.dragon.toolkit.vocon;

/**
 * Represents a table query object, for fuzzy-match context tuning.
 */
public class VoconTableQuery
{
    final String query;
    final int activationValue;

    /**
     * Constructs a VoconTableQuery object.
     * @param query The query string (ex: "city_id=0x1000", "category=restaurant").
     * Passing null identifies the whole database.
     * @param activationValue A value between 0 (entry is not recognizable) and
     * 10000 ("rude-activation" i.e. the rest of the database is automatically activated to 0).
     * Values outside this range will be clamped.
     */
    public VoconTableQuery(String query, int activationValue)
    {
        if (activationValue < 0) activationValue = 0;
        else if (activationValue > 10000) activationValue = 10000;
        this.query = query;
        this.activationValue = activationValue;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + activationValue;
        result = prime * result + ((query == null) ? 0 : query.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        VoconTableQuery other = (VoconTableQuery) obj;
        if (activationValue != other.activationValue)
            return false;
        if (query == null) {
            if (other.query != null)
                return false;
        } else if (!query.equals(other.query))
            return false;
        return true;
    }

    @Override
    public String toString() {
        return "VoconTableQuery [query=" + query + ", activationValue="
                + activationValue + "]";
    }
}
