package com.nuance.dragon.toolkit.vocon;

import org.json.JSONException;

import com.nuance.dragon.toolkit.oem.api.JSONCompliant;
import com.nuance.dragon.toolkit.oem.api.json.JSONObject;

/**
 * A Vocon warning.
 */
public class VoconWarning implements JSONCompliant
{
    /**
     * Possible reasons for Vocon warnings.
     */
    public static class Reason
    {
        public static final int NONE               = 0;
        public static final int BAD_SNR            = 1;
        public static final int OVERLOAD           = 2;
        public static final int TOO_QUIET          = 3;
        public static final int NO_SIGNAL          = 4;
        public static final int POOR_MIC           = 5;
        public static final int NO_LEADING_SILENCE = 6;

        private static final String [] REASON_TEXTS = {"None",
                                                       "The signal to noise ratio is too low",
                                                       "The speech level is too loud",
                                                       "The speech level is too weak",
                                                       "No or very low input signal",
                                                       "The AGC cannot adjust the gain to get the input signal in the proper range",
                                                       "The recording apparantly does not start with leading silence"};
    }

    private static final String KEY_CODE = "code";

    private final int _code;
    private final String _reason;

    VoconWarning(int code)
    {
        _code = code;
        _reason = Reason.REASON_TEXTS[code];
    }

    /**
     * Get the reason code, from {@link Reason}.
     * @return The reason code
     */
    public int getReasonCode()
    {
        return _code;
    }

    /**
     * @return A description corresponding with the reason code
     */
    public String getReason()
    {
        return _reason;
    }

    @Override
    public org.json.JSONObject toJSON()
    {
        JSONObject o = new JSONObject();
        o.tryPut(KEY_CODE, _code);
        return o;
    }

    // TODO never null?
    public static VoconWarning createFromJSON(org.json.JSONObject json) throws JSONException
    {
        int code = json.getInt(KEY_CODE);
        if (code < 0 || code >= Reason.REASON_TEXTS.length)
            throw new JSONException("Invalid code");
        return new VoconWarning(code);
    }

    @Override
    public String toString()
    {
        return _reason;
    }
}
