package com.nuance.dragon.toolkit.vocon;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class WakeupCheckResult
{
    /**
     * Wakeup word quality returned by a WakeupCheckResult
     */
    public class VoconWuwQuality
    {
        /** Value returned when trying to lookup a quality for an unchecked phrase. */
        public static final int INVALID = -1;

        /** The checked phrase is OK and has a good quality. */
        public static final int GOOD = 0;

        /** The checked phrase is OK but has a weak quality. */
        public static final int WEAK = 1;

        /** The checked phrase is OK but has a weak quality. */
        public static final int BAD = 2;

    }

    /**
     * Contains a checked phrase and it's associated quality as a wakeup phrase.
     */
    public class PhraseInfo
    {
        private String _phrase;
        private int _quality;

        public String getPhrase() {
            return _phrase;
        }
        public int getQuality() {
            return _quality;
        }

        public PhraseInfo(String phrase, int quality) {
            _phrase = phrase;
            _quality = quality;
        }
    }

    /**
     * Get the phrases info regarding their wakeup check.
     * @return A list of @link PhraseInfo for all phrases passed through the checkWakeupPhrases call.
     */
    public List<PhraseInfo> getPhrasesInfo()
    {
        ArrayList<PhraseInfo> phraseInfo = new ArrayList<PhraseInfo>();
        for (Map.Entry<String, Integer> entry :_innerPhraseQualityList.entrySet())
        {
            phraseInfo.add(new PhraseInfo(entry.getKey(), entry.getValue()));
        }
        return phraseInfo;
    }

    WakeupCheckResult()
    {
        _innerPhraseQualityList = new HashMap<String, Integer>();
    }

    void AddPhraseQuality(String phrase, int quality)
    {
        _innerPhraseQualityList.put(phrase, quality);
    }

    private HashMap<String, Integer> _innerPhraseQualityList;
}
