/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client.sdk.common.oem.api;

import java.util.Vector;

import com.nuance.nmsp.client.sdk.common.util.Enum;

public interface AudioSystem {
	//#ifdef NMSP_CPL_ENABLED
//@    public static class CPLAudioPathContext {
//@        public String audioPath = "n/a";
//@    }
    //#endif
    
    static class InputDevice extends Enum {
        private static final int ID_MICROPHONE        = 0;
        private static final int ID_NETWORK           = 1;
        private static final int ID_BLUETOOTH_HEADSET = 2;
        private static final int ID_WIRED_HEADSET     = 3;
        private InputDevice(int id) { super(id); }
        
        public static InputDevice MICROPHONE        = new InputDevice(InputDevice.ID_MICROPHONE);        /*!< Record from microphone */
        public static InputDevice NETWORK           = new InputDevice(InputDevice.ID_NETWORK);           /*!< Record from network (i.e. from a phone call) */
        public static InputDevice BLUETOOTH_HEADSET = new InputDevice(InputDevice.ID_BLUETOOTH_HEADSET); /*!< Record from Bluetooth headset */
        public static InputDevice WIRED_HEADSET     = new InputDevice(InputDevice.ID_WIRED_HEADSET);     /*!< Record from wired headset */
    }
    
    static class OutputDevice extends Enum {
        private static final int ID_SPEAKER           = 0;
        private static final int ID_NETWORK           = 1;
        private static final int ID_BLUETOOTH_HEADSET = 2;
        private static final int ID_BLUETOOTH_A2DP    = 3;
        private static final int ID_WIRED_HEADSET     = 4;
        private OutputDevice(int id) { super(id); }
        
        public static OutputDevice SPEAKER           = new OutputDevice(OutputDevice.ID_SPEAKER);           /*!< Playback to speaker */
        public static OutputDevice NETWORK           = new OutputDevice(OutputDevice.ID_NETWORK);           /*!< Playback to network (i.e. to a phone call) */
        public static OutputDevice BLUETOOTH_HEADSET = new OutputDevice(OutputDevice.ID_BLUETOOTH_HEADSET); /*!< Playback to Bluetooth headset */
        public static OutputDevice BLUETOOTH_A2DP    = new OutputDevice(OutputDevice.ID_BLUETOOTH_A2DP);    /*!< Playback to Bluetooth A2DP device */
        public static OutputDevice WIRED_HEADSET     = new OutputDevice(OutputDevice.ID_WIRED_HEADSET);     /*!< Playback to wired headset */
    }
    
    static class AudioStatus extends Enum {
        private static final int ID_AUDIO_OK           = 0;
        private static final int ID_AUDIO_ERROR        = 1;
        private AudioStatus(int id) { super(id); }
        
        public static final AudioStatus AUDIO_OK    = new AudioStatus(ID_AUDIO_OK);   /*!< Requested operation succeeded */
        public static final AudioStatus AUDIO_ERROR = new AudioStatus(ID_AUDIO_ERROR);   /*!< Requested operation failed */
    }

    static class IntegerMutable {
        public int i;
        public IntegerMutable(int i) {
            this.i = i;
        }
    }
    
    interface AudioCallback {
        void audioCallback(byte[] pcmData, Object vocoderData, IntegerMutable pcmDataLen, IntegerMutable vocoderDataLen, Float avgAndPeakGain, Object context);
        void audioCallback(Vector pcmData, Vector vocoderData, IntegerMutable pcmDataLen, IntegerMutable vocoderDataLen, Float avgAndPeakGain, Object context);
    }
    
    interface DoneCallback {
        void doneCallback(AudioStatus status, Object context);
    }
    
    interface StopCallback {
        void stopCallback(AudioStatus status, Object context);
    }
    
    interface ErrorCallback {
        void errorCallback(Object context);
    }
    
    interface StartOfSpeechCallback {
    	void startOfSpeechCallback(Object context);
	}
    
    interface NoSpeechCallback {
        void noSpeechCallback(Object context);
    }

    interface EndOfSpeechCallback {
        void endOfSpeechCallback(Object context);
    }
    
    interface EndPointerStartedCallback {
        void endPointerStartedCallback(Object context);
    }
    
    interface EndPointerStoppedCallback {
        void endPointerStoppedCallback(Object context);
    }

    boolean startRecording(
            InputDevice inputDevice,
            boolean useEndPointer,
            AudioCallback audioCallback,
            StopCallback stopCallback,
            ErrorCallback errorCallback,
            StartOfSpeechCallback startOfSpeechCallback,
            NoSpeechCallback noSpeechCallback,
            EndOfSpeechCallback endOfSpeechCallback,
            EndPointerStartedCallback endPointerStartedCallback,
            EndPointerStoppedCallback endPointerStoppedCallback,
            Object context
            );
    
    void turnOnEndPointer(EndPointerStartedCallback callback, Object context);
    
    void turnOffEndPointer(EndPointerStoppedCallback callback, Object context);
    
    void pauseRecording(int pause);

    void stopRecording(StopCallback stopCallback, Object context);

    boolean startPlayback(
            OutputDevice outputDevice, 
            AudioCallback audioCallback,
            DoneCallback doneCallback,
            Object context);
    
    boolean pausePlayback(int pause);
    
    void stopPlayback(StopCallback stopCallback, Object context);
    
    boolean previousPlayback();
    
    boolean nextPlayback();
    
    InputDevice getInputDevice();
    
    OutputDevice getOutputDevice();

    Vector getParams(Vector parameters);

}
