/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client.sdk.common.oem.api;

import java.net.SocketAddress;


public interface DeviceInfo {
    
    /// <summary> Return the phone's Unique ID number (either the
    /// ESN/MEID for a CDMA phone, or the IMEI for a GSM phone) .  </summary>
    public String getUniqueID();

    /// <summary> Returns the phone number from the phone's (e.g "7815551212"). 
    /// If the phone number is not available, an empty strign will be returned.</summary>
    public String getPhoneNumber();

    /// <summary> Returns the network operator name from the phone's. If the network operator 
    /// name is not available, an empty string will be returned. (i.e. "") </summary>
    public String getOperatorName();

    /// <summary> Returns the device model from. If the device model is not available, 
    /// an empty string will be returned. (i.e. "")</summary>
    public String getDeviceModel();

    /// <summary> Returns the device submodel from. If the device submodel is not available, 
    /// an empty string will be returned. (i.e. "")</summary>
    public String getDeviceSubModel();

    /// <summary> Returns the OS Type. If the OS Type is not available, 
    /// an empty string will be returned. (i.e. "")</summary>
    public String getOSType();
    
    /// <summary> Returns the OS version. If the OS version is not available, 
    /// an empty string will be returned. (i.e. "")</summary>
    public String getOSVersion();
    
    /// <summary> Returns the proxy address for the current connection. If the proxy
    /// information is unavailable null will be returned.</summary>
    public SocketAddress getProxyAddress();
}
