/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client.sdk.common.oem.api;

import java.io.UnsupportedEncodingException;

import com.nuance.nmsp.client.sdk.oem.LogFactoryOEM;

public class LogFactory
{
    public static Log getLog(Class klass)
    {
        return LogFactoryOEM.getLog(klass);
    }
    
    public static abstract class Log
    {
        public abstract void trace(Object message);
        public abstract void trace(Object message, Throwable t);
        public abstract void debug(Object message);
        public abstract void debug(Object message, Throwable t);
        public abstract void info(Object message);
        public abstract void info(Object message, Throwable t);
        public abstract void warn(Object message);
        public abstract void warn(Object message, Throwable t);
        public abstract void error(Object message);
        public abstract void error(Object message, Throwable t);
        public abstract void fatal(Object message);
        public abstract void fatal(Object message, Throwable t);
        
        public abstract boolean isTraceEnabled();
        public abstract boolean isDebugEnabled();
        public abstract boolean isInfoEnabled();
        public abstract boolean isWarnEnabled();
        public abstract boolean isErrorEnabled();
        public abstract boolean isFatalEnabled();
        
        public abstract void setCurrentSession(String sessionId);
        public abstract void unsetCurrentSession();
        
        public void traceBuffer(byte[] buf)
        {
            if (isTraceEnabled())
            {
                trace("Buffer dump:");
                String[] lines = bufferStr(buf);
                for (int i = 0; i < lines.length; i++)
                    trace(lines[i]);
            }
        }
        
        public static String[] bufferStr(byte[] buffer)
        {
            int numOfLines = buffer.length / 8;
            if (buffer.length % 8 != 0)
                numOfLines++;
            String[] ret = new String[numOfLines];
            
            String str;
            try
            {
                str = new String(buffer, "ISO-8859-1");
            } catch (UnsupportedEncodingException e)
            {
                str = new String(buffer);
            }
            
            int curOffset = 0;
            for(int i = 0; i < numOfLines; i++)
            {
                char[] line = new char[8 * 2 + 7 + 3 + 8 + 7];
                for(int j=0;j<line.length;j++)
                    line[j] = ' ';
                
                for(int k=0;k<8 && curOffset<buffer.length;curOffset++, k++)
                {
                    String hex = getHex(buffer[curOffset]);
                    line[k * 3] = hex.charAt(0);
                    line[k * 3 + 1] = hex.charAt(1);
                    line[8 * 2 + 7 + 3 + k * 2] = str.charAt(curOffset);
                }
                
                ret[i] = new String(line);
            }
            
            return ret;
        }
        
        public static String FormatUuid(byte[] uuid)
        {
            if (uuid == null)
                return "";
            
            StringBuffer buf = new StringBuffer();
            
            for (int i = 0; i < 16; i++)
            {
                buf.append(getHex(uuid[i]));
                if (i == 3 || i == 5 || i == 7 || i == 9)
                    buf.append("-");
            }
            
            return buf.toString().toLowerCase();
        }
        
        private static String pseudo[] = { "0", "1", "2", "3", "4", "5", "6",
                "7", "8", "9", "A", "B", "C", "D", "E", "F" };
        
        private static String getHex(byte b)
        {
            byte ch1 = (byte) (b & 0xF0);
            ch1 = (byte) (ch1 >>> 4);
            ch1 = (byte) (ch1 & 0x0F);
            
            byte ch2 = (byte) (b & 0x0F);
            
            return pseudo[(int) ch1] + pseudo[(int) ch2];
        }
    }
}
