/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client.sdk.common.protocols;

//#ifdef NMSP_LOG
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory.Log;
//#endif


public class ProtocolBuilder
{
	//#ifdef NMSP_LOG
	 private final static Log log = LogFactory.getLog(ProtocolBuilder.class);
	//#endif 
    public static byte[] buildXModeBuf(byte protocol, byte version, short cmd, byte[] payload)
    {
       //#ifdef NMSP_LOG
         if(log.isDebugEnabled())
        {
            log.debug("Building XMode buffer: protocol=" + protocol + 
                    " version=" + version + 
                    " cmd=" + cmd + 
                    " payload len:" + payload.length);
            log.traceBuffer(payload);
       }
       //#endif

        int payloadLen = payload.length;
        int bufferLen = ProtocolDefines.XMODE_PACKET_HEADER_SIZE + payloadLen;
        byte[] buffer = new byte[bufferLen];
        int offset = 0;
        
        XModeMsgHeader header = new XModeMsgHeader(protocol, version, cmd, payloadLen);
        System.arraycopy(header.getBytes(), 0, buffer, offset, ProtocolDefines.XMODE_PACKET_HEADER_SIZE);
        offset += ProtocolDefines.XMODE_PACKET_HEADER_SIZE;     
        System.arraycopy(payload, 0, buffer, offset, payloadLen);
        
        return buffer;
    }
    
    public static byte[] appendBCPSessionUUID(byte[] payload, byte[] sessionId)
    {
    	//#ifdef NMSP_LOG
        if(log.isDebugEnabled())
        {
            log.debug("Appending session ID");
            log.traceBuffer(sessionId);
            log.traceBuffer(payload);
        }
        //#endif
        byte[] newPayload = new byte[ProtocolDefines.XMODE_SERVER_SESSION_UUID_SIZE + payload.length];
        System.arraycopy(sessionId, 0, newPayload, 0, ProtocolDefines.XMODE_SERVER_SESSION_UUID_SIZE);
        System.arraycopy(payload, 0, newPayload, ProtocolDefines.XMODE_SERVER_SESSION_UUID_SIZE, payload.length);
        return newPayload;
    }
}
