/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client.sdk.common.protocols;

//#ifdef NMSP_LOG
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory.Log;
//#endif
import com.nuance.nmsp.client.sdk.common.util.ByteConversion;

public class XModeMsgHeader
{
	//#ifdef NMSP_LOG
     private final static Log log = LogFactory.getLog(XModeMsgHeader.class);
    //#endif
    public byte protocol;
    public byte version;
    public short cmd;
    public int len;
    
    public XModeMsgHeader(byte[] buffer)
    {
//#ifdef NMSP_LOG    	
        if(log.isDebugEnabled())
       {
            log.debug("Constructing XModeMsgHeader(byte[])");
            log.traceBuffer(buffer);
        }
//#endif        
        protocol = buffer[0];
        version  = buffer[1];
        cmd      = ByteConversion.bytesToShort(buffer, 2);
        len      = ByteConversion.bytesToInt(buffer, 4);
    }
    
    public XModeMsgHeader(byte protocol, byte version, short cmd, int len)
    {
//#ifdef NMSP_LOG    	
        if(log.isDebugEnabled())
        {
            log.debug("Constructing XModeMsgHeader(protocol=" + protocol + 
                    ", version=" + version + 
                    ", cmd=" + cmd + 
                    ", len=" + len + ")");
        }
//#endif        
        this.protocol = protocol;
        this.version  = version;
        this.cmd      = cmd;
        this.len      = len;
    }
    
    public byte[] getBytes()
    {
//#ifdef NMSP_LOG    
        if(log.isDebugEnabled())
            log.debug("XModeMsgHeader.getBytes()");
//#endif        
        byte[] buf = new byte[ProtocolDefines.XMODE_PACKET_HEADER_SIZE];
        buf[0] = protocol;
        buf[1] = version;
        ByteConversion.shortToBytes(cmd, buf, 2);
        ByteConversion.intToBytes(len, buf, 4);
//#ifdef NMSP_LOG          
        if(log.isDebugEnabled())
        {
            log.trace("Generated: ");
            log.traceBuffer(buf);
       }
//#endif        
        return buf;
    }
}
