/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client.sdk.common.util;

//#ifdef NMSP_LOG
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory.Log;
//#endif

public class ByteConversion
{
//#ifdef NMSP_LOG	
    private final static Log log = LogFactory.getLog(ByteConversion.class);
//#endif    
    /* store s as 2 bytes in to at position pos */
    public static void shortToBytes(short s, byte[] to, int pos)
    {
        byte low = (byte) (s & 0xFF);
        byte high = (byte) ((s >> 8) & 0xFF);
        to[pos] = low;
        to[pos + 1] = high;
//#ifdef NMSP_LOG        
        if(log.isTraceEnabled())
            log.trace("Converted short " + s + " to [" + to[pos] + ", "
                    + to[pos + 1] + "]");
//#endif        
    }
    
    /* read a short at position pos in from */
    public static short bytesToShort(byte[] from, int pos)
    {
        int low = from[pos] & 0xFF;
        int high = from[pos + 1] & 0xFF;
        short s = (short) ((high << 8) | low);
//#ifdef NMSP_LOG        
        if(log.isTraceEnabled())
            log.trace("Converted [" + from[pos] + ", " + from[pos + 1]
                    + "] to short " + s);
//#endif        
        return s;
    }
    
    /* store i as 4 bytes in to at position pos */
    public static void intToBytes(int i, byte[] to, int pos)
    {
        short low = (short) (i & 0xFFFF);
        short high = (short) ((i >> 16) & 0xFFFF);
        shortToBytes(low, to, pos);
        shortToBytes(high, to, pos + 2);
//#ifdef NMSP_LOG        
        if(log.isTraceEnabled())
            log.trace("Converted int " + i + " to [" + to[pos] + ", "
                    + to[pos + 1] + ", " + to[pos + 2] + ", " + to[pos + 3]
                    + "]");
//#endif        
    }
    
    /* read an int at position pos in from */
    public static int bytesToInt(byte[] from, int pos)
    {
        int low = bytesToShort(from, pos) & 0x00FFFF;
        int high = bytesToShort(from, pos + 2) & 0x00FFFF;
        int i = (high << 16) | low;
//#ifdef NMSP_LOG        
        if(log.isTraceEnabled())
            log.trace("Converted [" + from[pos] + ", " + from[pos + 1] + ", "
                    + from[pos + 2] + ", " + from[pos + 3] + "] to int " + i);
//#endif        
        return i;
    }
}
