/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client.sdk.common.util;

import com.nuance.nmsp.client.sdk.common.defines.NMSPDefines;
import com.nuance.nmsp.client.sdk.common.defines.NMSPDefines.Codec;
//#ifdef NMSP_LOG
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory.Log;
//#endif

public class Util
{
//#ifdef NMSP_LOG	
    private final static Log log = LogFactory.getLog(Util.class);
//#endif    
    //#ifdef NMSP_LOG
    private static final String E_ENC_AMR_0      = "E_ENC_AMR_0";
    
    //#ifndef NMSP_PLATFORM_BLACKBERRY
    private static final String E_ENC_PCM_16_11K = "E_ENC_PCM_16_11K";
    private static final String E_ENC_PCM_16_8K  = "E_ENC_PCM_16_8K";
    //#endif
    
    public static String getCodecStr(NMSPDefines.Codec codec) {
        short codecValue = codec.getValue();
        //#ifndef NMSP_PLATFORM_ANDROID
//@        if (codecValue >= NMSPDefines.Codec.AMR_00.getValue() && codecValue <= NMSPDefines.Codec.AMR_07.getValue())
//@            return E_ENC_AMR_0 + ((short) (codecValue - NMSPDefines.Codec.AMR_00.getValue()));
        //#endif
        //#ifndef NMSP_PLATFORM_BLACKBERRY
        if (codecValue == NMSPDefines.Codec.PCM_16_8K.getValue())
            return E_ENC_PCM_16_8K;
        if (codecValue == NMSPDefines.Codec.PCM_16_11K.getValue())
            return E_ENC_PCM_16_11K;
        //#endif
            return "UNKOWN_CODEC";
    }
    //#endif
    
    public static boolean isAMRCodec(NMSPDefines.Codec codec) {
    	//#if !NMSP_PLATFORM_ANDROID        
//@        short codecValue = codec.getValue();
//@        return (codecValue >= NMSPDefines.Codec.AMR_00.getValue()
//@                && codecValue <= NMSPDefines.Codec.AMR_07.getValue());
        //#else
     return false;
        //#endif
    }
    
    public static boolean isMP3Codec(NMSPDefines.Codec codec) {
        //#if !NMSP_PLATFORM_ANDROID
//@        short codecValue = codec.getValue();
//@        return (codecValue == NMSPDefines.Codec.MP3.getValue());
        //#else
     return false;
        //#endif
    }
    
    public static boolean isGSMCodec(NMSPDefines.Codec codec) {
        //#if NMSP_PLATFORM_J2SE || NMSP_PLATFORM_ANDROID
        short codecValue = codec.getValue();
        return (codecValue == NMSPDefines.Codec.MS_GSM_FR.getValue());
        //#else
//@        return false;
        //#endif
    }

    public static boolean isPCMCodec(NMSPDefines.Codec codec) {
        //#ifdef NMSP_PLATFORM_BLACKBERRY
//@    	return (codec.getValue() == NMSPDefines.Codec.PCM_16_8K.getValue());
    	//#elifdef NMSP_PLATFORM_ANDROID
    	short codecValue = codec.getValue();
    	return (codecValue == NMSPDefines.Codec.PCM_16_8K.getValue()
                 || codecValue == NMSPDefines.Codec.PCM_16_11K.getValue()
                 || codecValue == NMSPDefines.Codec.PCM_16_16K.getValue()
                 || codecValue == NMSPDefines.Codec.PCM_16_32K.getValue());
    	//#else
//@    	short codecValue = codec.getValue();
//@    	return (codecValue == NMSPDefines.Codec.PCM_16_8K.getValue()
//@                || codecValue == NMSPDefines.Codec.PCM_16_11K.getValue());
        //#endif
    }
    
    public static boolean isSpeexCodec(NMSPDefines.Codec codec) {
		//#if NMSP_PLATFORM_J2SE || NMSP_PLATFORM_ANDROID 	
 	     short codecValue = codec.getValue();
 	     return(codecValue == Codec.SPEEX_8K.getValue() || 
 	     	    codecValue == Codec.SPEEX_11K.getValue()||
 	     	    codecValue == Codec.SPEEX_16K.getValue());
		//#else
//@			return false;
		//#endif
    }
    
    public static boolean isOpusCodec(NMSPDefines.Codec codec) {
		//#if NMSP_PLATFORM_J2SE || NMSP_PLATFORM_ANDROID 	
 	     short codecValue = codec.getValue();
 	     return(codecValue == Codec.OPUS_8K.getValue() ||
 	    		codecValue == Codec.OPUS_16K.getValue());
		//#else
//@			return false;
		//#endif
    }
    public static String escapeXML(String text)
    {
        //text.replaceAll("&","&amp;").replaceAll("<","&lt;").replaceAll(">","&gt;").replaceAll("\"","&quot;").replaceAll("'","&apos;");
//#ifdef NMSP_LOG        
        if(log.isTraceEnabled())
            log.trace("Escaping XML reserved tokens (&, <, >, \" and \') of: " + text);
//#endif        
        int index = 0;
        StringBuffer str = new StringBuffer(text);
        while(index < str.length())
        {
            char c = str.charAt(index);
            if(c == '&')
            {
                index++;
                str.insert(index, "amp;");
                index += 4;
            }
            else if(c == '<')
            {
                str.deleteCharAt(index);
                str.insert(index, "&lt;");
                index += 4;
            }
            else if(c == '>')
            {
                str.deleteCharAt(index);
                str.insert(index, "&gt;");
                index += 4;
            }
            else if(c == '"')
            {
                str.deleteCharAt(index);
                str.insert(index, "&quot;");
                index += 6;
            }
            else if(c == '\'')
            {
                str.deleteCharAt(index);
                str.insert(index, "&apos;");
                index += 6;
            }
            else
                index++;
        }
        
        String ret = str.toString();
 //#ifdef NMSP_LOG        
        if(log.isTraceEnabled())
            log.trace("Final output: " + ret);
//#endif        
        return ret;
        
//        public static void main(String[] args)
//        {
//            String test = "\"\"\" ''' >>> <<< &&& &><\'\"& dsfa < dsfa > \"test thyat\" dfsafd \'&\'\"\'<>& &&& <<< >>> ''' \"\"\"";
//            System.out.println(escapeXML(test));
//        }
    }
    
    public static NMSPDefines.Codec adjustCodecForBluetooth(NMSPDefines.Codec codec) {
        NMSPDefines.Codec ret;
        if (codec == NMSPDefines.Codec.SPEEX_16K ||
            codec == NMSPDefines.Codec.SPEEX_11K) {
            ret = NMSPDefines.Codec.SPEEX_8K;
        }
        else if (codec == NMSPDefines.Codec.PCM_16_11K ||
                 codec == NMSPDefines.Codec.PCM_16_16K ||
                 codec == NMSPDefines.Codec.PCM_16_22K ||
                 codec == NMSPDefines.Codec.PCM_16_32K) {
            ret = NMSPDefines.Codec.PCM_16_8K;
        }
        else {
            ret = codec;
        }
        //#ifdef NMSP_LOG
        if (log.isInfoEnabled())
            log.info("adjustCodecForBluetooth() " + codec.getValue() + " -> " + ret.getValue());
        //#endif
        return ret;
    }
}
