package com.nuance.nmsp.client.sdk.components.core.calllog;

import java.util.Vector;

import com.nuance.nmsp.client.sdk.components.core.calllog.CalllogSender.SenderListener;
import com.nuance.nmsp.client.sdk.components.core.internal.calllog.CalllogSenderImpl;
import com.nuance.nmsp.client.sdk.components.general.Parameter;

/**
 * Factory class to create the {@link CalllogSender} object. The {@link SenderConfig}
 * contains the basic configuration including the network information and application
 * credentials for the call log server.
 * 
 * @see {@link CalllogSender}
 * @see {@link com.nuance.nmsp.client.sdk.components.resource.common.Manager}
 */
public class CalllogSenderFactory
{
    /**
     * Create the {@link CallogSender}
     * 
     * @param config  The configuration for the {@link CallogSender}, must not be NULL
     * 
     * @return  The created CalllogSender object
     */
    public static CalllogSender createCalllogSender(SenderListener listener)
    {
        return new CalllogSenderImpl(listener, null); 
    }
    
    /**
     * Create the {@link CallogSender} with the given override parameters list. This list of parameters
     * will override any matching parameters serialized in the call log data. This is useful in case some
     * of those parameters cannot be properly serialized during the data generation process. 
     * 
     * @param config  The configuration for the {@link CallogSender}, must not be NULL
     * @param overrideParameters The parameters list {@link Parameter} to override the existing ones
     * 
     * @return  The created CalllogSender object
     */
    public static CalllogSender createCalllogSender(SenderListener listener, Vector overrideParameters)
    {
        return new CalllogSenderImpl(listener, overrideParameters); 
    }
}
