package com.nuance.nmsp.client.sdk.components.core.calllog;

public interface SessionEventBuilder
{
    /**
     * Puts a binary value at the specified key.
     * 
     * @param key
     *          The key where to store the value
     * @param value
     *          The value to add to the vector for the given key
     *          
     * @return The SessionEventBuilder itself.
     * 
     * @throws SessionEventAlreadyCommittedException 
     * @throws IllegalArgumentException
     *           If the value added to the vector is null or is not of the same type as a previous addition.
     */
    SessionEventBuilder putBinary(String key, byte[] value) throws SessionEventAlreadyCommittedException;

    /**
     * Puts a boolean value at the specified key.
     * 
     * @param key
     *          The key where to store the value
     * @param value
     *          The value to add to the vector for the given key
     *          
     * @return The SessionEventBuilder itself.
     * 
     * @throws SessionEventAlreadyCommittedException 
     * @throws IllegalArgumentException
     *           If the value added to the vector is null or is not of the same type as a previous addition.
     */
    SessionEventBuilder putBoolean(String key, boolean value) throws SessionEventAlreadyCommittedException;

    /**
     * Puts a float value at the specified key.
     * 
     * @param key
     *          The key where to store the value
     * @param value
     *          The value to add to the vector for the given key
     *          
     * @return The SessionEventBuilder itself.
     * 
     * @throws SessionEventAlreadyCommittedException 
     * @throws IllegalArgumentException
     *           If the value added to the vector is null or is not of the same type as a previous addition.
     */
    SessionEventBuilder putFloat(String key, double value) throws SessionEventAlreadyCommittedException;

    /**
     * Puts a integer value at the specified key.
     * 
     * @param key
     *          The key where to store the value
     * @param value
     *          The value to add to the vector for the given key
     *          
     * @return The SessionEventBuilder itself.
     * 
     * @throws SessionEventAlreadyCommittedException 
     * @throws IllegalArgumentException
     *           If the value added to the vector is null or is not of the same type as a previous addition.
     */
    SessionEventBuilder putInteger(String key, int value) throws SessionEventAlreadyCommittedException;

    /**
     * Puts a long value at the specified key.
     * 
     * @param key
     *          The key where to store the value
     * @param value
     *          The value to add to the vector for the given key
     *          
     * @return The SessionEventBuilder itself.
     * 
     * @throws SessionEventAlreadyCommittedException 
     * @throws IllegalArgumentException
     *           If the value added to the vector is null or is not of the same type as a previous addition.
     */
    SessionEventBuilder putLong(String key, long value) throws SessionEventAlreadyCommittedException;
    
    /**
     * Puts a string value at the specified key.
     * 
     * @param key
     *          The key where to store the value
     * @param value
     *          The value to add to the vector for the given key
     *          
     * @return The SessionEventBuilder itself.
     *          
     * @throws SessionEventAlreadyCommittedException 
     * @throws IllegalArgumentException
     *           If the value added to the vector is null or is not of the same type as a previous addition.
     */
    SessionEventBuilder putString(String key, String value) throws SessionEventAlreadyCommittedException;

    /**
     * 
     * @param eventName
     * 			Name of the new remote event
     * 
     * @return The SessionEventBuilder itself.
     * 
     * @throws SessionEventAlreadyCommittedException 
     * 		   IllegalArgumentException
     * 			If the eventName is null
     */
    SessionEventBuilder createRemoteEvent(String eventName) throws SessionEventAlreadyCommittedException;
    
    /**
     * Method to conclude the new session event and return the SessionEvent object. 
     * 
     * @return The committed SessionEvent object or null if the commitment fails, the current
     *         SessionEventBuilder is already committed or the current call log session has been
     *         closed already.
     */
    SessionEvent commit();
}
