/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client.sdk.components.core.internal.pdx;

import java.io.ByteArrayOutputStream;
import java.util.Enumeration;
import java.util.Hashtable;

import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory.Log;
import com.nuance.nmsp.client.sdk.components.core.pdx.Dictionary;
import com.nuance.nmsp.client.sdk.components.core.pdx.Sequence;

/**
 * PDX dictionary representation
 * 
 */
public class PDXDictionary extends PDXClass implements Dictionary {
    //#ifdef NMSP_LOG
    private static final Log log = LogFactory.getLog(PDXDictionary.class);
    //#endif

    private Hashtable dictionary;

    public PDXDictionary() {
        super(PDXClass.DICT);
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary()");
        //#endif
        dictionary = new Hashtable();
    }

    public PDXDictionary(byte[] message, boolean isMessage) {
        super(PDXClass.DICT);
        //#ifdef NMSP_LOG3
//@        String inputHex = "";
//@        for(int i = 0; i < message.length; i++) {
//@            inputHex += Integer.toString( ( message[i] & 0xff ) + 0x100, 16).substring( 1 );
//@        }
//@        log.debug("PDXDictionary() " + inputHex);
        //#endif
        dictionary = new Hashtable();
        if (message.length > 0) {
            int classType = message[0] & 0xFF;
            if (classType != PDXClass.DICT) {
                //#ifdef NMSP_LOG
                log.error("PDXDictionary() Expected a dictionary. ");
                //#endif
                return;
            }

            int length = getLength(message, 1);

            byte[] dictionaryContent = new byte[length];
            System.arraycopy(message, message.length - length, dictionaryContent, 0, dictionaryContent.length);
            setContent(dictionaryContent);
        }
    }

    public PDXDictionary(byte[] byteDictionary) {
        super(PDXClass.DICT);
        //#ifdef NMSP_LOG3
//@        String inputHex = "";
//@        for(int i = 0; i < byteDictionary.length; i++) {
//@            inputHex += Integer.toString( ( byteDictionary[i] & 0xff ) + 0x100, 16).substring( 1 );
//@        }
//@        log.debug("PDXDictionary() " + inputHex);
        //#endif
        dictionary = new Hashtable();
        setContent(byteDictionary);
    }
    
//    protected void addAll(Dictionary dict)
//    {
//        Enumeration keys = dict.keys();
//        
//        while (keys.hasMoreElements())
//        {
//            String key = (String)keys.nextElement();
//            dictionary.put(key, (PDXClass) getObject(key));
//        }
//    }
//
//    private Object getObject(String key)
//    {
//        return dictionary.get(key);
//    }
    
    public void put(String name, String value, short type) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.put() name: " + name + " value: " + value + " type: " + type);
        //#endif
        switch (type) {
        case PDXClass.UTF8:
            dictionary.put(name, new PDXUTF8String(value));
            break;
        case PDXClass.ASCII:
            dictionary.put(name, new PDXAsciiString(value));
            break;
        default:
            //#ifdef NMSP_LOG
            //log.error("PDXDictionary.put() type should be UTF8 or ASCII. ");
            //#endif
            break;
        }
    }

    protected void put(String name, boolean value, short type) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.put() name: " + name + " value: " + value + " type: " + type);
        //#endif
        if (type == PDXClass.BOOLEAN) {
            dictionary.put(name, new PDXBoolean(value));
        }
        //#ifdef NMSP_LOG
        else {
            log.error("PDXDictionary.put() type should be BOOLEAN. ");
        }
        //#endif
    }
    
    protected void put(String name, double value, short type) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.put() name: " + name + " value: " + value + " type: " + type);
        //#endif
        if (type == PDXClass.DOUBLE) {
            dictionary.put(name, new PDXDouble(value));
        }
        //#ifdef NMSP_LOG
        else {
            log.error("PDXDictionary.put() type should be DOUBLE. ");
        }
        //#endif
    }    
    
    protected void put(String name, int value, short type) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.put() name: " + name + " value: " + value + " type: " + type);
        //#endif
        if (type == PDXClass.INT) {
            dictionary.put(name, new PDXInteger(value));
        }
        //#ifdef NMSP_LOG
        else {
            log.error("PDXDictionary.put() type should be INT. ");
        }
        //#endif
    }

    protected void put(String name, byte[] value, short type) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.put() name: " + name + " value: " + value + " type: " + type);
        //#endif
        switch (type) {
        case PDXClass.BOOLEAN:
        	dictionary.put(name, new PDXBoolean(value));
        	break;
        case PDXClass.DOUBLE:
        	dictionary.put(name,  new PDXDouble(value));
        	break;
        case PDXClass.INT:
            dictionary.put(name, new PDXInteger(value));
            break;
        case PDXClass.BYTES:
            dictionary.put(name, new PDXByteString(value));
            break;
        case PDXClass.UTF8:
            dictionary.put(name, new PDXUTF8String(value));
            break;
        case PDXClass.ASCII:
            dictionary.put(name, new PDXAsciiString(value));
            break;
        case PDXClass.NULL:
            dictionary.put(name, new PDXNull());
            break;
        case PDXClass.DICT:
            dictionary.put(name, new PDXDictionary(value));
            break;
        case PDXClass.SEQ:
            dictionary.put(name, new PDXSequence(value));
            break;
        default:
            //#ifdef NMSP_LOG
            log.error("PDXDictionary.put() Unknown PDXClass type: " + type + ". ");
            //#endif
            break;
        }
    }

    /**
     * Adds a dictionary element.
     * 
     * @param key
     *            element name
     * 
     * @param pdxClass
     *            element
     * 
     * @return
     */
    // public void put(String name, PDXClass pdxClass)
    // {
    // dictionary.put(name, pdxClass);
    // }
    // public PDXClass get(String name)
    // {
    // if(dictionary == null) {
    // dictionary = getValue();
    // }
    //        
    // return (PDXClass) dictionary.get(name);
    // }
    
    /**
     * Retrieves an enumeration of the keys in this dictionary.
     * 
     * 
     * @return Enumeration
     */
    public Enumeration keys()
    {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.keys()");
        //#endif
        return dictionary.keys();
    }
    
    public boolean containsKey(String key)
    {
        if (key == null) {
            throw new NullPointerException("PDXDictionary.containsKey key is null");
        }
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.keys()");
        //#endif
        return dictionary.containsKey(key);
    }
    
    public PDXClass getClass(String key) {
        return (PDXClass) dictionary.get(key);
    }
    
    public short getType(String key) {
        if(key == null) throw new IllegalArgumentException("key is null.");
        PDXClass pdxClass = (PDXClass)dictionary.get(key);
        return pdxClass.getType();
    }
    
    /**
     * Retrieves a dictionary boolean element.
     * 
     * @param key
     *            element name
     * 
     * @throws RuntimeException
     * 
     * @return boolean
     *
     * @see com.nuance.nmsp.client.sdk.components.pdx.Dictionary#getBoolean(java.lang.String)
     */
    public boolean getBoolean(String key) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.getBoolean() key: " + key);
        //#endif
        if(key == null) throw new IllegalArgumentException("key is null.");
        PDXClass pdxClass = (PDXClass) dictionary.get(key);
        if (pdxClass == null) {
            //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("PDXDictionary.getBoolean() " + key + " does not exist. ");
            //#endif
            throw new RuntimeException(Dictionary.KEY_DOES_NOT_EXIST);
        }
        if (pdxClass.getType() != PDXClass.BOOLEAN) {
            //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("PDXDictionary.getBoolean() " + key + " is not a PDXBoolean. ");
            //#endif
            throw new RuntimeException(Dictionary.KEY_IS_OF_WRONG_TYPE);
        }

        return ((PDXBoolean) pdxClass).getValue();
    }
    
    /**
     * Retrieves a dictionary double element.
     * 
     * @param key
     *            element name
     * 
     * @throws RuntimeException
     * 
     * @return double
     *
     * @see com.nuance.nmsp.client.sdk.components.pdx.Dictionary#getDouble(java.lang.String)
     */
    public double getDouble(String key) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.getDouble() key: " + key);
        //#endif
        if(key == null) throw new IllegalArgumentException("key is null.");
        PDXClass pdxClass = (PDXClass) dictionary.get(key);
        if (pdxClass == null) {
            //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("PDXDictionary.getDouble() " + key + " does not exist. ");
            //#endif
            throw new RuntimeException(Dictionary.KEY_DOES_NOT_EXIST);
        }
        if (pdxClass.getType() != PDXClass.DOUBLE) {
            //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("PDXDictionary.getDouble() " + key + " is not a PDXDouble. ");
            //#endif
            throw new RuntimeException(Dictionary.KEY_IS_OF_WRONG_TYPE);
        }

        return ((PDXDouble) pdxClass).getValue();
    }
    
    /**
     * Retrieves a dictionary integer element.
     * 
     * @param key
     *            element name
     * 
     * @throws RuntimeException
     * 
     * @return int
     *
     * @see com.nuance.nmsp.client.sdk.components.pdx.Dictionary#getInteger(java.lang.String)
     */
    public int getInteger(String key) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.getInteger() key: " + key);
        //#endif
        if(key == null) throw new IllegalArgumentException("key is null.");
        PDXClass pdxClass = (PDXClass) dictionary.get(key);
        if (pdxClass == null) {
            //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("PDXDictionary.getInteger() " + key + " does not exist. ");
            //#endif
            throw new RuntimeException(Dictionary.KEY_DOES_NOT_EXIST);
        }
        if (pdxClass.getType() != PDXClass.INT) {
            //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("PDXDictionary.getInteger() " + key + " is not a PDXInteger. ");
            //#endif
            throw new RuntimeException(Dictionary.KEY_IS_OF_WRONG_TYPE);
        }

        return ((PDXInteger) pdxClass).getValue();
    }

    /**
     * Retrieves a dictionary byte string element.
     * 
     * @param key
     *            element name
     * 
     * @throws RuntimeException
     *
     * @return byte[]
     */
    public byte[] getByteString(String key) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.getByteString() key: " + key);
        //#endif
        if(key == null) throw new IllegalArgumentException("key is null.");
        PDXClass pdxClass = (PDXClass) dictionary.get(key);
        if (pdxClass == null) {
          //#ifdef NMSP_LOG
          log.error("PDXDictionary.getByteString() " + key + " does not exist. ");
          //#endif
          throw new RuntimeException(Dictionary.KEY_DOES_NOT_EXIST);
        }
        if (pdxClass.getType() != PDXClass.BYTES) {
          //#ifdef NMSP_LOG
          log.error("PDXDictionary.getByteString() " + key + " is not a PDXByteString. ");
          //#endif
          throw new RuntimeException(Dictionary.KEY_IS_OF_WRONG_TYPE);
        }

        return ((PDXByteString) pdxClass).getValue();
    }

    /**
     * Retrieves a dictionary UTF8 String element.
     * 
     * @param key
     *            element name
     * 
     * @throws RuntimeException
     * 
     * @return String
     */
    public String getUTF8String(String key) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.getUTF8String() key: " + key);
        //#endif
        if(key == null) throw new IllegalArgumentException("key is null.");
        PDXClass pdxClass = (PDXClass) dictionary.get(key);
        if (pdxClass == null) {
          //#ifdef NMSP_LOG
          log.error("PDXDictionary.getUTF8String() " + key + " does not exist. ");
          //#endif
          throw new RuntimeException(Dictionary.KEY_DOES_NOT_EXIST);
        }
        if (pdxClass.getType() != PDXClass.UTF8) {
          //#ifdef NMSP_LOG
          log.error("PDXDictionary.getUTF8String() " + key + " is not a PDXUTF8String. ");
          //#endif
          throw new RuntimeException(Dictionary.KEY_IS_OF_WRONG_TYPE);
        }

        return ((PDXUTF8String) pdxClass).getValue();
    }

    /**
     * Retrieves a dictionary ASCII String element.
     * 
     * @param key
     *            element name
     * 
     * @throws RuntimeException
     * 
     * @return String
     */
    public String getAsciiString(String key) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.getAsciiString() key: " + key);
        //#endif
        if(key == null) throw new IllegalArgumentException("key is null.");
        PDXClass pdxClass = (PDXClass) dictionary.get(key);
        if (pdxClass == null) {
          //#ifdef NMSP_LOG
          log.error("PDXDictionary.getAsciiString() " + key + " does not exist. ");
          //#endif
          throw new RuntimeException(Dictionary.KEY_DOES_NOT_EXIST);
        }
        if (pdxClass.getType() != PDXClass.ASCII) {
          //#ifdef NMSP_LOG
          log.error("PDXDictionary.getAsciiString() " + key + " is not a PDXAsciiString. ");
          //#endif
          throw new RuntimeException(Dictionary.KEY_IS_OF_WRONG_TYPE);
        }

        return ((PDXAsciiString) pdxClass).getValue();
    }

    /**
     * Retrieves a dictionary Dictionary element.
     * 
     * @param key
     *            element name
     * 
     * @throws RuntimeException
     * 
     * @return Dictionary
     */
    public Dictionary getDictionary(String key) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.getDictionary() key: " + key);
        //#endif
        if(key == null) throw new IllegalArgumentException("key is null.");
        PDXClass pdxClass = (PDXClass) dictionary.get(key);
        if (pdxClass == null) {
          //#ifdef NMSP_LOG
          log.error("PDXDictionary.getDictionary() " + key + " does not exist. ");
          //#endif
          throw new RuntimeException(Dictionary.KEY_DOES_NOT_EXIST);
        }
        if (pdxClass.getType() != PDXClass.DICT) {
          //#ifdef NMSP_LOG
          log.error("PDXDictionary.getDictionary() " + key + " is not a PDXDictionary. ");
          //#endif
          throw new RuntimeException(Dictionary.KEY_IS_OF_WRONG_TYPE);
        }

        return ((PDXDictionary) pdxClass);
    }

    /**
     * Retrieves a dictionary Sequence element.
     * 
     * @param key
     *            element name
     * 
     * @throws RuntimeException
     * 
     * @return Sequence
     */
    public Sequence getSequence(String key) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.getSequence() key: " + key);
        //#endif
        if(key == null) throw new IllegalArgumentException("key is null.");
        PDXClass pdxClass = (PDXClass) dictionary.get(key);
        if (pdxClass == null) {
          //#ifdef NMSP_LOG
          log.error("PDXDictionary.getSequence() " + key + " does not exist. ");
          //#endif
          throw new RuntimeException(Dictionary.KEY_DOES_NOT_EXIST);
        }
        if (pdxClass.getType() != PDXClass.SEQ) {
          //#ifdef NMSP_LOG
          log.error("PDXDictionary.getSequence() " + key + " is not a PDXSequence. ");
          //#endif
          throw new RuntimeException(Dictionary.KEY_IS_OF_WRONG_TYPE);
        }

        return ((PDXSequence) pdxClass);
    }

    public void addAsciiString(String key, String value) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.addAsciiString() key: " + key + " value: " + value);
        //#endif
        if(key == null || value == null)
        {
            throw new IllegalArgumentException("key is : " + key + " value is : " + value);
        }
        PDXAsciiString string = new PDXAsciiString(value);
        dictionary.put(key, string);
    }

    public void addByteString(String key, byte[] value) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.addByteString() key: " + key + " value: " + value);
        //#endif
        if(key == null || value == null)
        {
            throw new IllegalArgumentException("key is : " + key + " value is : " + value);
        }
        PDXByteString string = new PDXByteString(value);
        dictionary.put(key, string);
    }

    public void addDictionary(String key, Dictionary value) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.addDictionary() key: " + key + " value: " + value);
        //#endif
        if(key == null || value == null)
        {
            throw new IllegalArgumentException("key is : " + key + " value is : " + value);
        }
        if (((PDXClass) value).getType() != PDXClass.DICT) {
            //#ifdef NMSP_LOG
            log.error("PDXDictionary.addDictionary() value is not a valid dictionary.");
            //#endif
            throw new IllegalArgumentException("value is not a valid dictionary. ");
        }
        dictionary.put(key, value);
    }
    
    public void addBoolean(String key, boolean value) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.addBoolean() key: " + key + " value: " + value);
        //#endif
        if(key == null) throw new IllegalArgumentException("key is null.");
        PDXBoolean booleanValue = new PDXBoolean(value);
        dictionary.put(key, booleanValue);
    }

    public void addDouble(String key, double value) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.addDouble() key: " + key + " value: " + value);
        //#endif
        if(key == null) throw new IllegalArgumentException("key is null.");
        PDXDouble doubleValue = new PDXDouble(value);
        dictionary.put(key, doubleValue);
    }


    public void addInteger(String key, int value) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.addInteger() key: " + key + " value: " + value);
        //#endif
        if(key == null) throw new IllegalArgumentException("key is null.");
        PDXInteger integer = new PDXInteger(value);
        dictionary.put(key, integer);
    }

    public void addSequence(String key, Sequence value) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.addSequence() key: " + key + " value: " + value);
        //#endif
        if(key == null || value == null)
        {
            throw new IllegalArgumentException("key is : " + key + " value is : " + value);
        }
        if (((PDXClass) value).getType() != PDXClass.SEQ) {
            //#ifdef NMSP_LOG
            log.error("PDXDictionary.addSequence() value is not a valid sequence.");
            //#endif
            throw new IllegalArgumentException("value is not a valid sequence. ");
        }
        dictionary.put(key, value);
    }

    public void addUTF8String(String key, String value) {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.addUTF8String() key: " + key + " value: " + value);
        //#endif
        if(key == null || value == null)
        {
            throw new IllegalArgumentException("key is : " + key + " value is : " + value);
        }
        PDXUTF8String string = new PDXUTF8String(value);
        dictionary.put(key, string);
    }

    private void setContent(byte[] byteDictionary) {
        //#ifdef NMSP_LOG3
//@        String dictionaryHex = "";
//@        for(int i = 0; i < byteDictionary.length; i++) {
//@            dictionaryHex += Integer.toString( ( byteDictionary[i] & 0xff ) + 0x100, 16).substring( 1 );
//@        }
//@        log.debug("PDXDictionary.setContent() " + dictionaryHex);
        //#endif
        int index = 0;
        int length;
        int classType;
        byte[] classContent;

        while (index < byteDictionary.length) {
            String name;
            // The key is always an ASCII string
            classType = byteDictionary[index++] & 0xFF;
            if (classType != PDXClass.ASCII) {
                //#ifdef NMSP_LOG
                log.error("PDXDictionary.setContent() Expected an ASCII string but got " + classType + ". ");
                //#endif
                return;
            }

            length = getLength(byteDictionary, index);
            index += getLengthSize(length);

            classContent = new byte[length];
            System.arraycopy(byteDictionary, index, classContent, 0, classContent.length);
            index += length;

            name = new String(classContent);

            // Get the value class type
            classType = byteDictionary[index++] & 0xFF;
            length = getLength(byteDictionary, index);
            index += getLengthSize(length);

            classContent = new byte[length];
            System.arraycopy(byteDictionary, index, classContent, 0, classContent.length);
            index += length;

            // Build the dictionary by adding a key/value
            switch (classType) {
            case PDXClass.BOOLEAN:
            	dictionary.put(name, new PDXBoolean(classContent));
            	break;
            case PDXClass.DOUBLE:
            	dictionary.put(name,  new PDXDouble(classContent));
            	break;
            case PDXClass.INT:
                dictionary.put(name, new PDXInteger(classContent));
                break;
            case PDXClass.BYTES:
                dictionary.put(name, new PDXByteString(classContent));
                break;
            case PDXClass.UTF8:
                dictionary.put(name, new PDXUTF8String(classContent));
                break;
            case PDXClass.ASCII:
                dictionary.put(name, new PDXAsciiString(classContent));
                break;
            case PDXClass.NULL:
                dictionary.put(name, new PDXNull());
                break;
            case PDXClass.DICT:
                dictionary.put(name, new PDXDictionary(classContent));
                break;
            case PDXClass.SEQ:
                dictionary.put(name, new PDXSequence(classContent));
                break;
            default:
                //#ifdef NMSP_LOG
                log.error("PDXDictionary.setContent() Unknown PDXClass type: " + classType + ". ");
                //#endif
                break;
            }
        }
    }

    public byte[] getContent() {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.getContent()");
        //#endif
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        Enumeration names = dictionary.keys();
        while (names.hasMoreElements()) {
            String name = (String) names.nextElement();
            PDXAsciiString pdxKey = new PDXAsciiString(name);
            try {
                output.write(pdxKey.toByteArray());
                PDXClass pdxValue = (PDXClass) dictionary.get(name);
                switch (pdxValue.getType()) {
                case PDXClass.BOOLEAN:
                	output.write(((PDXBoolean) pdxValue).toByteArray());
                	break;
                case PDXClass.DOUBLE:
                	output.write(((PDXDouble) pdxValue).toByteArray());
                	break;
                case PDXClass.INT:
                    output.write(((PDXInteger) pdxValue).toByteArray());
                    break;
                case PDXClass.BYTES:
                    output.write(((PDXByteString) pdxValue).toByteArray());
                    break;
                case PDXClass.UTF8:
                    output.write(((PDXUTF8String) pdxValue).toByteArray());
                    break;
                case PDXClass.ASCII:
                    output.write(((PDXAsciiString) pdxValue).toByteArray());
                    break;
                case PDXClass.NULL:
                    output.write(((PDXNull) pdxValue).toByteArray());
                    break;
                case PDXClass.DICT:
                    output.write(((PDXDictionary) pdxValue).toByteArray());
                    break;
                case PDXClass.SEQ:
                    output.write(((PDXSequence) pdxValue).toByteArray());
                    break;
                }
            } catch (java.io.IOException ioe) {
                //#ifdef NMSP_LOG
                log.error("PDXDictionary.getContent() " + ioe.toString() + ". ");
                //#endif
            }
        }
        return output.toByteArray();
    }

    public byte[] toByteArray() {
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.toByteArray()");
        //#endif
        return super.toByteArray(getContent());
    }
    
    public String toString()
    {
        return this.toString(0);
    }
    
    protected String toString(int indentLevel)
    {
        String stringDictionary = "";
        String baseIndent = "";
        String indent = "";
        for(int i = 0; i < indentLevel - 1; i++) {
            baseIndent += "    ";
        }
        if(indentLevel > 0) {
            indent = baseIndent + "    ";
        }
        
        Enumeration keys = dictionary.keys();
        if(indentLevel != 0) {
            stringDictionary += "{ \n";
        }
        while(keys.hasMoreElements()) {
            String key = (String)keys.nextElement();
            PDXClass value = (PDXClass)dictionary.get(key);
            switch (value.getType()) {
            case PDXClass.BOOLEAN:
            	stringDictionary += indent + key + ": <BOOLEAN> " + ((PDXBoolean)value).getValue() + "\n";
            	break;
            case PDXClass.DOUBLE:
            	stringDictionary += indent + key + ": <DOUBLE> " + ((PDXDouble)value).getValue() + "\n";
            	break;
            case PDXClass.INT:
                stringDictionary += indent + key + ": <INT> " + ((PDXInteger)value).getValue() + "\n";
                break;
            case PDXClass.BYTES:
                stringDictionary += indent + key + ": <BYTES> \"" + ((PDXByteString)value).getValue() + "\"\n";
                break;
            case PDXClass.UTF8:
                stringDictionary += indent + key + ": <UTF8> \"" + ((PDXUTF8String)value).getValue() + "\"\n";
                break;
            case PDXClass.ASCII:
                stringDictionary += indent + key + ": <ASCII> \"" + ((PDXAsciiString)value).getValue() + "\"\n";
                break;
            case PDXClass.NULL:
                stringDictionary += indent + key + ": <NULL> \n";
                break;
            case PDXClass.DICT:
                stringDictionary += indent + key + ": " + ((PDXDictionary)value).toString(indentLevel + 1) + "\n";
                break;
            case PDXClass.SEQ:
                stringDictionary += indent + key + ": " + ((PDXSequence)value).toString(indentLevel + 1) + "\n";
                break;
            }
        }
        if(indentLevel != 0) {
            stringDictionary += baseIndent + "} ";
        }
        //#ifdef NMSP_LOG2
//@        log.debug("PDXDictionary.toString()\n" + stringDictionary);
        //#endif
        return stringDictionary;
    }
}
