/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client.sdk.components.resource.internal.common;

import java.util.HashMap;
import java.util.List;
import java.util.Vector;
import java.util.concurrent.Callable;

//#ifdef NMSP_PLATFORM_BLACKBERRY
//@import net.rim.device.api.system.RuntimeStore;
//#endif


import com.nuance.nmsp.client.sdk.common.defines.NMSPDefines;
import com.nuance.nmsp.client.sdk.components.core.calllog.SessionEvent;
import com.nuance.nmsp.client.sdk.components.core.calllog.SessionEventBuilder;
import com.nuance.nmsp.client.sdk.components.core.internal.calllog.CalllogImpl;
import com.nuance.nmsp.client.sdk.components.core.internal.calllog.CalllogListener;
import com.nuance.nmsp.client.sdk.components.general.NMSPManager;
import com.nuance.nmsp.client.sdk.components.general.Parameter;
import com.nuance.nmsp.client.sdk.components.resource.common.Manager;
import com.nuance.nmsp.client.sdk.components.resource.common.ManagerListener;
import com.nuance.nmsp.client.sdk.common.oem.api.MiscSystem;
import com.nuance.nmsp.client.sdk.common.oem.api.MessageSystem.MessageHandler;
import com.nuance.nmsp.client.sdk.oem.BluetoothSystemOEM;
//#ifdef NMSP_LOG
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory.Log;
//#endif


public class ManagerImpl extends NMSPManager implements Manager, CalllogListener {
    //#ifdef NMSP_LOG
    private static final Log log = LogFactory.getLog(ManagerImpl.class);
    //#endif

    private NMSPSession session;
    private Object resLogsToServer;
    public String applicationId;
    //#ifdef NMSP_PLATFORM_BLACKBERRY
//@    private static final long ID_NMSP_SDK_RESOURCE_LOGS_TO_SERVER = 0x725721789465ab9L; // "ID_NMSP_SDK_RESOURCE_LOGS_TO_SERVER_MOTOSPEAK"
    //#endif
/**
 * This is the old key used int NMSP 3.0 SDK for BB V2
 * We need the building process to generate a different KEY for each Application later
 * "ID_NMSP_SDK_RESOURCE_LOGS_TO_SERVER_NVC2"
 * private static final long ID_NMSP_SDK_RESOURCE_LOGS_TO_SERVER = 0xf21d914ceff3845dL;
 **/

    //#ifdef NMSP_PLATFORM_BLACKBERRY
//@    public String _applicationId = null;
//@    public byte[] _appKey = null;
//@    public Vector _params = null;
//@    public ManagerListener _managerListener = null;
    //#endif
    //public static final String CALLLOG_FILENAME = "Calllog_Filename";
    //public static final String CALLLOG_APPKEY = "Calllog_AppKey";
    private CalllogImpl _calllog;
    // This is to hold the parameters used when creating the Manager. This way subsequent
    // createCalllog calls can reuse those parameters.
    private HashMap calllogParams;
    private Vector _params;
    private ManagerListener _managerListener;

    public ManagerImpl(String gatewayIP,
            short gatewayPort,
            String applicationId,
            byte[] appKey,
            String uid,
            NMSPDefines.Codec inputCodec,
            NMSPDefines.Codec outputCodec,
            Vector parameters,
            ManagerListener managerListener)
    {
        super(gatewayIP, gatewayPort, uid, inputCodec, outputCodec);

        if (applicationId == null)
            throw new IllegalArgumentException(" application id is null.");

        this.applicationId = applicationId;

        if (appKey == null)
            throw new IllegalArgumentException(" application key is null");

        _managerListener = managerListener;

        Vector params;
        // Defaults parameters for the call log
        int retentionDays = CalllogImpl.CALLLOG_RETENTION_DAYS_DEFAULT;
        String rootId = null;
        int chunkSize = CalllogImpl.CALLLOG_CHUNK_SIZE_DEFAULT;


        // This is by JIRA ticket NMSPSDK-147
        // always disable call logging for all the Java platforms
        boolean calllogDisable = true;

        if (parameters == null || parameters.size() == 0) {
            params = new Vector();
        } else {
            params = new Vector(parameters.size());
            for (int i = 0; i < parameters.size(); i++) {
                Parameter param = (Parameter) parameters.elementAt(i);
                if (param.getType() == Parameter.Type.SDK) {
                    if (param.getName().equals(NMSPDefines.DEVICE_CMD_LOG_TO_SERVER_ENABLED)) {
                        //#ifdef NMSP_PLATFORM_BLACKBERRY
//@                        RuntimeStore runtimeStore = RuntimeStore.getRuntimeStore();
//@                        Object tmpObject = runtimeStore.get(ID_NMSP_SDK_RESOURCE_LOGS_TO_SERVER);
//@                        if ((tmpObject != null) && (tmpObject instanceof Vector)) {
//@                            resLogsToServer = (Vector) tmpObject;
//@                        } else {
                        //#endif

                        resLogsToServer = new Vector();

                        //#ifdef NMSP_PLATFORM_BLACKBERRY
//@                        }
//@                        runtimeStore.replace(ID_NMSP_SDK_RESOURCE_LOGS_TO_SERVER, resLogsToServer);
                        //#endif
                    }
                    else if (param.getName().equals(NMSPDefines.NMSP_DEFINES_CALLLOG_DISABLE)) {
                        if ((new String(param.getValue())).equals("TRUE") || (new String(param.getValue())).equals("true")) {
                            calllogDisable = true;
                        }
                        else if ((new String(param.getValue())).equals("FALSE") || (new String(param.getValue())).equals("false")){
                            calllogDisable = false;
                        }
                    }
                    else if (param.getName().equals(NMSPDefines.NMSP_DEFINES_CALLLOG_ROOT_ID))
                    {
                        rootId = new String(param.getValue());
                    }
                    else if (param.getName().equals(NMSPDefines.NMSP_DEFINES_CALLLOG_RETENTION_DAYS))
                    {
                        retentionDays = Integer.parseInt(new String(param.getValue()));
                        if ( ( ( retentionDays == -1 ) || ( retentionDays > 0 ) ) == false )
                        {
                            throw new IllegalArgumentException("The NMSP_DEFINES_CALLLOG_RETENTION_DAYS parameter must be greater than 0 or equals to -1.");
                        }
                    }
                    else if (param.getName().equals(NMSPDefines.NMSP_DEFINES_CALLLOG_CHUNK_SIZE))
                    {
                        chunkSize = Integer.parseInt(new String(param.getValue()));
                        if(chunkSize < CalllogImpl.CALLLOG_CHUNK_SIZE_MINIMUM)
                        {
                            throw new IllegalArgumentException("Minimum chunk size for calllog is " + CalllogImpl.CALLLOG_CHUNK_SIZE_MINIMUM);
                        }
                        if(chunkSize > CalllogImpl.CALLLOG_CHUNK_SIZE_MAXIMUM){
                            throw new IllegalArgumentException("Maximum chunk size for calllog is " + CalllogImpl.CALLLOG_CHUNK_SIZE_MAXIMUM);
                        }
                    }
                    else if ( param.getName().equals(NMSPDefines.NMSP_DEFINES_ENABLE_USER_ID_ENCRYPTION) )
                    {
                        if ((new String(param.getValue())).equals("TRUE") || (new String(param.getValue())).equals("true"))
                        {
                            byte[] hashedUid = MiscSystem.HMAC_SHA1(applicationId.getBytes(), uid.getBytes());
                            StringBuilder sb = new StringBuilder();
                            for ( int j = 0; j < hashedUid.length; j++ )
                            {
                                String hex = Integer.toHexString(hashedUid[j]);
                                if (hex.length() == 1)
                                {
                                    sb.append(0);
                                    sb.append(hex.charAt(hex.length() - 1));
                                }
                                else
                                {
                                    sb.append(hex.substring(hex.length() - 2));
                                }
                            }
                            this.uid = sb.toString();
                        }
                    }
                }
                params.addElement(param);
            }
        }

        _params = params;
        this.session = new NMSPSession(getGatewayIP(), getGatewayPort(), applicationId, appKey, this.uid, params, getMsgSys(), managerListener);
        // If we have enabled calllog, create a sdk calllog
        if(!calllogDisable)
        {
            _calllog = new CalllogImpl( gatewayIP,
                                        gatewayPort,
                                        this.applicationId,
                                        appKey,
                                        chunkSize,
                                        retentionDays,
                                        rootId,
                                        parameters,
                                        this.uid,
                                        getMsgSys(),
                                        this);
            session.setCalllog(_calllog);
        }

        //#ifdef NMSP_PLATFORM_BLACKBERRY
//@        _applicationId = applicationId;
//@        _appKey = appKey;
//@        _params = params;
//@        _managerListener = managerListener;
        //#endif
    }

    public NMSPSession getSession() {
        return this.session;
    }

    public Object getResourceLogs() {
        return this.resLogsToServer;
    }

    public void clearResourceLogs() {
        this.resLogsToServer = null;
    }

//    //Override
    public CalllogImpl getCalllog() {
        return _calllog;
    }

    // This returns the current session event that is active within the session.
    // This allows for sdk subtrees (branches)
    public SessionEvent getCalllogSession() {
        return session.getSessionEvent();
    }


    //Override
    public void shutdown() {
        //#ifdef NMSP_LOG
        log.debug("shutdown");
        //#endif

        if (_calllog != null) {
            SessionEvent root = _calllog.getRootSessionEvent();
            if (root != null) {
                root.createChildEventBuilder(SessionEvent.NMSP_CALLLOG_SHUTDOWN_EVENT).commit();
            }
            _calllog.flushLogsToListener(false);
            _calllog.stopCalllog();
        }
        session.disconnectAndShutdown();
    }

    public SessionEventBuilder logAppEvent(String appEventName, String schemaVersion) {
        if(_calllog == null) {
            //#ifdef NMSP_LOG
                        log.error("Trying to add log while calllog is disabled");
            //#endif

            return null;
        }
        
        SessionEventBuilder newBuilder = _calllog.getRootSessionEvent().createChildEventBuilder(appEventName, schemaVersion);
        return newBuilder;
    }

    public String getCalllogRootParentId() {
        if(_calllog == null){
            //#ifdef NMSP_LOG
                        log.error("Trying to get root parent id while calllog is disabled");
            //#endif
            
            return "";
        }
        return _calllog.getRootParentId();
    }
    
    public void flushCallLogData() {
        if(_calllog == null){
            //#ifdef NMSP_LOG
                        log.error("Trying to flush logs while calllog is disabled");
            //#endif
            
            return;
        }        
        _calllog.flushLogsToListener(true);
    }

    public void calllogDataGenerated(byte[] data, List sessionEventIds) {
        _managerListener.callLogDataGenerated(data, sessionEventIds);
    }

}
