/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client.sdk.components.resource.internal.common;

import java.io.ByteArrayOutputStream;
import java.util.Hashtable;
import java.util.Map;
import java.util.Vector;

//#ifdef NMSP_PLATFORM_BLACKBERRY
//@import net.rim.device.api.system.RuntimeStore;
//#endif



import com.nuance.nmsp.client.sdk.common.defines.NMSPDefines;
import com.nuance.nmsp.client.sdk.common.oem.api.MessageSystem;
import com.nuance.nmsp.client.sdk.common.oem.api.MessageSystem.MessageData;
import com.nuance.nmsp.client.sdk.common.protocols.ProtocolBuilder;
import com.nuance.nmsp.client.sdk.common.protocols.ProtocolDefines;
import com.nuance.nmsp.client.sdk.common.protocols.XModeMsgHeader;
import com.nuance.nmsp.client.sdk.common.util.ByteConversion;
import com.nuance.nmsp.client.sdk.common.util.Util;
import com.nuance.nmsp.client.sdk.components.resource.common.Resource;
import com.nuance.nmsp.client.sdk.components.core.XMode;
import com.nuance.nmsp.client.sdk.components.core.calllog.SessionEvent;
import com.nuance.nmsp.client.sdk.components.core.calllog.SessionEventAlreadyCommittedException;
import com.nuance.nmsp.client.sdk.components.core.calllog.SessionEventBuilder;
import com.nuance.nmsp.client.sdk.components.core.internal.calllog.SessionEventBuilderImpl;
import com.nuance.nmsp.client.sdk.components.core.internal.calllog.CalllogImpl;
import com.nuance.nmsp.client.sdk.components.core.internal.calllog.SessionEventImpl;
import com.nuance.nmsp.client.sdk.components.general.NMSPAudioSink;
import com.nuance.nmsp.client.sdk.components.general.TransactionProcessingException;
import com.nuance.nmsp.client.sdk.components.resource.common.ManagerListener;
//#ifdef NMSP_LOG
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory.Log;
//#endif
import com.nuance.nmsp.client.sdk.components.resource.common.ResourceListener;

//TODO Should we start a timer for all/some methods? Should we put this in the Resource code or in the Session?
public class NMSPSession implements XMode.XModeListener, MessageSystem.MessageHandler {
    //#ifdef NMSP_LOG
    private static final Log log = LogFactory.getLog(NMSPSession.class);
    //#endif

    private Hashtable requestIds;
    private Hashtable audioSinks;
    private Hashtable audioRelatedSessions;
    private XMode xmode;

    private String server;
    private short port;

    private Vector sessionListeners; // Vector of NMSPSessionListener

    private MessageSystem messageSystem;
    private ManagerListener managerListener;
    private Vector messageQueue;
    private byte[] sessionId = null;
    private static byte[] emptySessionId = new byte[16];
    private NMSPDefines.Codec inputCodec;
    private NMSPDefines.Codec outputCodec;

//    private int defaultAudioId = 0;
    private static int audioIdCount = 1;

    private long defaultReqId;

    private boolean shutdownUponDisconnect = false;

    private Resource resource;
    private long resourceIdCounter = 0x0001;
    private byte tranIdCounter = 0x01;

    private SessionEvent sessionEvent;

    // We need to track the sdkCalllog to notify it for flush before disconnect
    private CalllogImpl _calllog;

    private static final byte CMD_CONNECT                 = 0x01;
    private static final byte CMD_DISCONNECT              = 0x02;
    private static final byte CMD_DISCONNECT_AND_SHUTDOWN = 0x03;
    private static final byte CMD_BCP_FREE_RESOURCE       = 0x04;
    private static final byte CMD_VAP_RECORD_BEGIN        = 0x05;
    private static final byte CMD_VAP_RECORD              = 0x06;
    private static final byte CMD_VAP_RECORD_END          = 0x07;
    private static final byte CMD_BCP_SEND_MSG            = 0x08;

    //#ifdef NMSP_CPL_ENABLED
//@    private Vector sessionWriteMsgs;
    //#endif

    //#ifdef NMSP_PLATFORM_BLACKBERRY
//@    private static final long ID_NMSP_SDK_TRANSACTION_ID_COUNTER = 0x8526bd4362d5e3bL; // "ID_NMSP_SDK_TRANSACTION_ID_COUNTER_MOTOSPEAK"
    //#endif
    
    // All methods in this class are called within the sdk thread
    public NMSPSession(String server, short port, String applicationId, byte[] appKey, String uid, Vector parameters, MessageSystem messageSystem, ManagerListener managerListener) {
        this.server = server;
        this.port = port;
        this.messageSystem = messageSystem;
        this.managerListener = managerListener;
        this.resource = null;

        messageQueue = new Vector();

        requestIds = new Hashtable();
        audioSinks = new Hashtable();
        audioRelatedSessions = new Hashtable();
        sessionListeners = new Vector();

        xmode = new XMode(this.server, this.port, applicationId, appKey, uid, this, parameters, messageSystem);

        //#ifdef NMSP_CPL_ENABLED
//@        sessionWriteMsgs = new Vector();
        //#endif
    }

    public XMode getXmode()
    {
        return xmode;
    }

    public void setResource(Resource resource) {
        this.resource = resource;
    }

    public synchronized int getNewAudioId() {
        int audioId = audioIdCount++;
        if (audioIdCount == Integer.MIN_VALUE)
            audioIdCount = 0x0001;

        return audioId;
    }

//    public int getDefaultAudioId() {
//        return defaultAudioId;
//    }

    public void setSessionListener(NMSPSessionListener sessionListener) {
        if (!sessionListeners.contains(sessionListener))
            sessionListeners.addElement(sessionListener);
    }

    public void removeSessionListener(NMSPSessionListener sessionListener) {
        sessionListeners.removeElement(sessionListener);
    }

    public void setDefaultReqId(long defaultReqId) {
        this.defaultReqId = defaultReqId;
    }

    /******** COP CONNECT ********/
    public void connect(NMSPDefines.Codec inputCodec, NMSPDefines.Codec outputCodec) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("connect()");
        //#endif

    	// We create a new branch upon connection
    	createSessionEvent(_calllog);

    	createChildEvent(SessionEvent.NMSP_CALLLOG_CONNECT_EVENT);

        this.inputCodec = inputCodec;
        this.outputCodec = outputCodec;
        sendCmdMessage(CMD_CONNECT, null);
    }

    private void handleConnect() {
    	String refId = null;
    	if(sessionEvent != null){
    		Map refIdMap = sessionEvent.getRefIds();
	    	//assert(refIdMap != null && refIdMap.size() == 1);
	    	if(refIdMap.size() > 0){
	    		refId = (String) refIdMap.values().toArray()[0];
	    	}else{
	            //#ifdef NMSP_LOG
	    				log.error("handleConnect: there is no ref id to connect with");
	    		//#endif
	    	}
    	}
        xmode.connect(inputCodec.getValue(), outputCodec.getValue(), refId);
    }

    /******** COP DISCONNECT ********/
    public void disconnect() {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("disconnect");
        //#endif

        sendCmdMessage(CMD_DISCONNECT, null);
    }

    /******** COP DISCONNECT & SHUTDOWN ********/
    public void disconnectAndShutdown() {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("disconnectAndShutdown");
        //#endif

    	sendCmdMessage(CMD_DISCONNECT_AND_SHUTDOWN, null);
    }

    public void handleDisconnect() {
        xmode.disconnect();
    }

    public void handleDisconnectAndShutdown() {
        shutdownUponDisconnect = true;
        xmode.disconnect();
    }

    public void socketClosed(short reason, short subReason) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("socketClosed, reason: " + reason);
        //#endif

        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("socketClosed() sessionListeners.size():" + sessionListeners.size());
        //#endif
        for (int i = 0; i < sessionListeners.size(); i++) {
            NMSPSessionListener sessionListener = (NMSPSessionListener) sessionListeners.elementAt(i);
            sessionListener.onSessionDisconnected(reason);
        }
        clearMessageQueue();
        sessionListeners.removeAllElements();
        
        if ( reason == XMode.XMODE_SOCKET_CLOSED_OPEN_SOCKET_FAILED )
        {
            managerListener.connectionFailed(resource, ManagerListener.NMSP_REMOTE_DISCONNECT_UNKNOWN);
            cancelGwRemoteEvent(SessionEventImpl.NMSP_CALLLOG_EVENT_CANCEL_CAUSE_CODE_CONNECTION_ERROR, "open socket failed");
        }
        else if ( reason == XMode.XMODE_SOCKET_CLOSED_OPEN_SOCKET_FAILED_SECURITY )
        {
            managerListener.connectionFailed(resource, ManagerListener.NMSP_REMOTE_DISCONNECT_SECURITY);
            cancelGwRemoteEvent(SessionEventImpl.NMSP_CALLLOG_EVENT_CANCEL_CAUSE_CODE_CONNECTION_ERROR, "open socket failed due to security");
        }
        else if ( reason == XMode.XMODE_SOCKET_CLOSED_COP_CONNECT_TIMEOUT )
        {
            managerListener.connectionFailed(resource, ManagerListener.NMSP_REMOTE_DISCONNECT_UNKNOWN);
            cancelGwRemoteEvent(SessionEventImpl.NMSP_CALLLOG_EVENT_CANCEL_CAUSE_CODE_CONNECTION_ERROR, "timed out on sending COP_CONNECT");
        }
        else if ( reason == XMode.XMODE_SOCKET_CLOSED_COP_CONNECT_FAILED )
        {
            managerListener.connectionFailed(resource, subReason);
            cancelGwRemoteEvent(SessionEventImpl.NMSP_CALLLOG_EVENT_CANCEL_CAUSE_CODE_CONNECTION_ERROR, "COP_CONNECT_FAILED");
        }
        // TODO this is a hack for bug 22390. The definition of disconnected() callback has to be revisited.
        else if (    (    reason == XMode.XMODE_SOCKET_CLOSED_NETWORK_ERROR )
                  && ( sessionId == null                                    ) )
        {
            managerListener.connectionFailed(resource, ManagerListener.NMSP_REMOTE_DISCONNECT_UNKNOWN);
            cancelGwRemoteEvent(SessionEventImpl.NMSP_CALLLOG_EVENT_CANCEL_CAUSE_CODE_CONNECTION_ERROR, "NETWORK ERROR and remote_disconnect_unknown");
        }
        else
        {
            if ( sessionId != null )
            { // if session was connected
                managerListener.disconnected(resource, subReason);
            }
        }

        if(reason == XMode.XMODE_SOCKET_CLOSED_DISCONNECT && shutdownUponDisconnect) {
            messageSystem.stop();
            managerListener.shutdownCompleted();
        }
        sessionId = null;
    }

    /******** BCP FREE RESOURCE ********/
    public void freeResource(byte tranId, int disconnectTimeout, NMSPSessionListener sessionListener) {
        //#ifdef NMSP_LOG
        log.debug("freeResource, TID: " + tranId + ", disconnect timeout: " + disconnectTimeout);
        //#endif
        sessionListeners.removeElement(sessionListener);
        Object array[] = new Object[2];
        array[0] = new Byte(tranId);
        array[1] = new Integer(disconnectTimeout);
        if (sessionId != null && messageQueue.isEmpty()) {
            sendCmdMessage(CMD_BCP_FREE_RESOURCE, array);
        } else {
            messageQueue.addElement(new MessageData(CMD_BCP_FREE_RESOURCE, array));
        }
    }

    private void handleFreeResource(byte tranId, int disconnectTimeout) {
        int offset = 0;
        byte[] payload = new byte[ProtocolDefines.XMODE_BCP_TRANSACTION_ID_SIZE
                + ProtocolDefines.XMODE_SERVER_BCP_FREE_RESOURCE_TIMEOUT_LENGTH];
        payload[offset] = tranId;
        offset += ProtocolDefines.XMODE_BCP_TRANSACTION_ID_SIZE;
        ByteConversion.intToBytes(disconnectTimeout, payload, offset);
        offset += ProtocolDefines.XMODE_SERVER_BCP_FREE_RESOURCE_TIMEOUT_LENGTH;

        payload = ProtocolBuilder.appendBCPSessionUUID(payload, sessionId);

        byte[] buffer = ProtocolBuilder.buildXModeBuf(
                ProtocolDefines.XMODE_PROTOCOL_BCP,
                ProtocolDefines.XMODE_VERSION_BCP,
                ProtocolDefines.XMODE_BCP_COMMAND_FREE_RESOURCE, payload);
        xmode.sendXModeMsg(buffer, XMode.SEND_MSG_PRIORITY_HIGH, XMode.NET_CONTEXT_SEND_BCP_FREE_RESOURCE);
    }

    /******** VAP RECORD BEGIN ********/
    public void vapRecordBegin(int audioId) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("vapRecordBegin, AID: " + audioId);
        //#endif
        Integer data = new Integer(audioId);
        if (sessionId != null && messageQueue.isEmpty()) {
            sendCmdMessage(CMD_VAP_RECORD_BEGIN, data);
        } else {
            messageQueue.addElement(new MessageData(CMD_VAP_RECORD_BEGIN, data));
        }
    }

    private void handleVapRecordbegin(int audioId) {
        xmode.startStreaming(audioId);
    }

    /******** VAP RECORD ********/
    public void vapRecord(byte[] audio, int audioId) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("vapRecord, AID: " + audioId);
        //#endif
        Object array[] = new Object[2];
        array[0] = audio;
        array[1] = new Integer(audioId);
        if (sessionId != null && messageQueue.isEmpty()) {
            sendCmdMessage(CMD_VAP_RECORD, array);
        } else {
            messageQueue.addElement(new MessageData(CMD_VAP_RECORD, array));
        }
    }

    private void handleVapRecord(byte[] buffer, int audioId) {
        for(int i = 0; i < sessionListeners.size(); i++) {
            NMSPSessionListener sessionListener = (NMSPSessionListener)sessionListeners.elementAt(i);
            sessionListener.onVapSending();
            //#ifdef NMSP_LOG
         	log.debug("onVapSending is called to reset command time out task");
            //#endif
        }
        xmode.sendVapRecordMsg(buffer, audioId);
    }

    /******** VAP RECORD END ********/
    public void vapRecordEnd(int audioId) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("vapRecordEnd, AID: " + audioId);
        //#endif
        Integer data = new Integer(audioId);
        if (sessionId != null && messageQueue.isEmpty()) {
            sendCmdMessage(CMD_VAP_RECORD_END, data);
        } else {
            messageQueue.addElement(new MessageData(CMD_VAP_RECORD_END, data));
        }
    }

    private void handleVapRecordEnd(int audioId) {
        xmode.sendVapRecordEnd(audioId);
    }

    /******** BCP MESSAGE ********/
    public void postBcpMessage(short bcpCommand, String netContext, byte[] primaryPayload, byte[] binaryPayload, byte tranId, long requestId, NMSPSessionListener sessionListener, boolean ackImmediately) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("postBcpMessage, BCP: " + bcpCommand + ", TID: " + tranId + ", RID: " + requestId);
        //#endif
        Object array_set_params[] = new Object[8];
        array_set_params[0] = new Short(bcpCommand);
        array_set_params[1] = netContext;
        array_set_params[2] = primaryPayload;
        array_set_params[3] = binaryPayload;
        array_set_params[4] = new Byte(tranId);
        array_set_params[5] = new Long(requestId);
        array_set_params[6] = sessionListener;
        array_set_params[7] = new Boolean(ackImmediately);
        if (sessionId != null && messageQueue.isEmpty()) {
            sendCmdMessage(CMD_BCP_SEND_MSG, array_set_params);
        } else {
            messageQueue.addElement(new MessageData(CMD_BCP_SEND_MSG, array_set_params));
        }
    }

    private void sendBcpMessage(short bcpCommand, String netContext, byte[] primaryPayload, byte[] binaryPayload, byte tranId, long requestId, NMSPSessionListener sessionListener, boolean ackImmediately) {
        requestIds.put(new Long(requestId), sessionListener);
        int offset = 0;
        int payloadSize = ProtocolDefines.XMODE_BCP_TRANSACTION_ID_SIZE +
                          ProtocolDefines.XMODE_XML_PAYLOAD_SIZE +
                          primaryPayload.length;
        if(bcpCommand == ProtocolDefines.XMODE_BCP_COMMAND_BCP_DATA) {
            payloadSize += ProtocolDefines.XMODE_BCP_REQUEST_ID_SIZE;
        }
        byte[] payload = new byte[payloadSize];

        payload[offset] = tranId;
        offset += ProtocolDefines.XMODE_BCP_TRANSACTION_ID_SIZE;

        if(bcpCommand == ProtocolDefines.XMODE_BCP_COMMAND_BCP_DATA) {
            ByteConversion.intToBytes((int)requestId, payload, offset);
            offset += ProtocolDefines.XMODE_BCP_REQUEST_ID_SIZE;
        }

        ByteConversion.intToBytes(primaryPayload.length, payload, offset);
        offset += ProtocolDefines.XMODE_XML_PAYLOAD_SIZE;

        System.arraycopy(primaryPayload, 0, payload, offset, primaryPayload.length);

        payload = ProtocolBuilder.appendBCPSessionUUID(payload, ((sessionId != null)? sessionId : emptySessionId));

        ByteArrayOutputStream message = new ByteArrayOutputStream();
        message.write(payload, 0, payload.length);
        if (binaryPayload != null)
            message.write(binaryPayload, 0, binaryPayload.length);

        byte[] buffer = ProtocolBuilder.buildXModeBuf(ProtocolDefines.XMODE_PROTOCOL_BCP,
                                                      ProtocolDefines.XMODE_VERSION_BCP,
                                                      bcpCommand,
                                                      message.toByteArray());
        xmode.sendXModeMsg(buffer, XMode.SEND_MSG_PRIORITY_HIGH, netContext);

        if(ackImmediately) {
            if(bcpCommand == ProtocolDefines.XMODE_BCP_COMMAND_SET_PARAMS) {
                if(sessionListener != null) {
                    sessionListener.onBcpSetParamsComplete(tranId, requestId, ResourceListener.STATUS_CODE_SUCCESS, null);
                }
            }
            else if(bcpCommand == ProtocolDefines.XMODE_BCP_COMMAND_FREE_RESOURCE_ID) {
                if(sessionListener != null) {
                    sessionListeners.removeElement(sessionListener);
                    sessionListener.onBcpFreeResourceId();
                }
            }
        }
    }

    private void parseXModeMsgBcpRecognitionComplete(byte[] payload) {
        byte transactionId;
        long requestId;
        short completionCause;
        int resultLength;
        byte recResult[] = null;
        int offset = 0;

        offset += ProtocolDefines.XMODE_SERVER_SESSION_UUID_SIZE;

        transactionId = payload[offset];
        offset += ProtocolDefines.XMODE_BCP_TRANSACTION_ID_SIZE;

        requestId = ByteConversion.bytesToInt(payload, offset);
        offset += ProtocolDefines.XMODE_BCP_REQUEST_ID_SIZE;

        completionCause = ByteConversion.bytesToShort(payload, offset);
        offset += ProtocolDefines.XMODE_SERVER_BCP_COMPLETION_CAUSE_SIZE;

        resultLength = ByteConversion.bytesToInt(payload, offset);
        offset += ProtocolDefines.XMODE_SERVER_BCP_RECOGNITION_RESULT_LENGTH;

        //HG: Fix for Android NMDP crash:
        // If the length is more than buffer size, ignore this buffer
        // TODO: investigate if we need to read more data
        // The issue was, we were reading command as len header in rear situations.
        if ((resultLength > 0) && (resultLength <= (payload.length - offset))) {
            recResult = new byte[resultLength];
            System.arraycopy(payload, offset, recResult, 0, resultLength);
        }

        NMSPSessionListener sessionListener = (NMSPSessionListener)requestIds.remove(new Long(requestId));

        if(sessionListener != null) {
            sessionListener.onBcpRecognitionComplete(transactionId, requestId, completionCause, recResult);
        }
    }

    private void parseXModeMsgBcpRecognitionIntermediateResults(byte[] payload) {
        byte transactionId;
        long requestId;
        short completionCause;
        int resultLength;
        byte recResult[] = null;
        int offset = 0;

        offset += ProtocolDefines.XMODE_SERVER_SESSION_UUID_SIZE;

        transactionId = payload[offset];
        offset += ProtocolDefines.XMODE_BCP_TRANSACTION_ID_SIZE;

        requestId = ByteConversion.bytesToInt(payload, offset);
        offset += ProtocolDefines.XMODE_BCP_REQUEST_ID_SIZE;

        completionCause = ByteConversion.bytesToShort(payload, offset);
        offset += ProtocolDefines.XMODE_SERVER_BCP_COMPLETION_CAUSE_SIZE;

        resultLength = ByteConversion.bytesToInt(payload, offset);
        offset += ProtocolDefines.XMODE_SERVER_BCP_RECOGNITION_RESULT_LENGTH;

          //HG: Fix for Android NMDP crash:
        // If the length is more than buffer size, ignore this buffer
        // TODO: investigate if we need to read more data
        // The issue was, we were reading command as len header in rear situations.
        if ((resultLength > 0) && (resultLength <= (payload.length - offset))) {
            recResult = new byte[resultLength];
            System.arraycopy(payload, offset, recResult, 0, resultLength);
        }

        NMSPSessionListener sessionListener = (NMSPSessionListener)requestIds.get(new Long(requestId));

        if(sessionListener != null) {
            sessionListener.onBcpRecognitionIntermediateResults(transactionId, requestId, completionCause, recResult);
        }
    }

    private void parseXModeMsgBcpGenerateAudioComplete(byte[] payload) {
        byte transactionId;
        long requestId;
        long audioId;
        short completionCause;
        int offset = 0;

        offset += ProtocolDefines.XMODE_SERVER_SESSION_UUID_SIZE;

        transactionId = payload[offset];
        offset += ProtocolDefines.XMODE_BCP_TRANSACTION_ID_SIZE;

        requestId = ByteConversion.bytesToInt(payload, offset);
        offset += ProtocolDefines.XMODE_BCP_REQUEST_ID_SIZE;

        audioId = ByteConversion.bytesToInt(payload, offset);
        offset += ProtocolDefines.XMODE_BCP_GENERATE_AUDIO_AUDIO_ID_SIZE;

        completionCause = ByteConversion.bytesToShort(payload, offset);
        offset += ProtocolDefines.XMODE_SERVER_BCP_COMPLETION_CAUSE_SIZE;

        NMSPSessionListener sessionListener = (NMSPSessionListener)requestIds.remove(new Long(requestId));

        if(sessionListener != null) {
            sessionListener.onBcpGenerateAudioComplete(transactionId, requestId, audioId, completionCause);
        }
    }

    private void parseXModeMsgBcpGetParamsComplete(byte[] payload) {
        byte transactionId;
        int requestId;
        short statusCode;
        int resultLength;
        byte result[] = null;
        int offset = 0;
        // Vector parameters = null;
        offset += ProtocolDefines.XMODE_SERVER_SESSION_UUID_SIZE;

        transactionId = payload[offset];
        offset += ProtocolDefines.XMODE_BCP_TRANSACTION_ID_SIZE;

        requestId = ByteConversion.bytesToInt(payload, offset);
        offset += ProtocolDefines.XMODE_BCP_REQUEST_ID_SIZE;

        statusCode = ByteConversion.bytesToShort(payload, offset);
        offset += ProtocolDefines.XMODE_SERVER_BCP_RESPONSE_STATUS_CODE_SIZE;

        resultLength = ByteConversion.bytesToInt(payload, offset);
        offset += ProtocolDefines.XMODE_BCP_GETPARAM_LENGTH_SIZE;

          //HG: Fix for Android NMDP crash:
        // If the length is more than buffer size, ignore this buffer
        // TODO: investigate if we need to read more data
        // The issue was, we were reading command as len header in rear situations.
        if ((resultLength > 0) && (resultLength <= (payload.length - offset))) {
            result = new byte[resultLength];
            System.arraycopy(payload, offset, result, 0, resultLength);
        }

        NMSPSessionListener sessionListener = (NMSPSessionListener)requestIds.remove(new Long(requestId));

        if(sessionListener != null) {
            sessionListener.onBcpGetParamsComplete(transactionId, requestId, statusCode, result);
        }
    }

    private void parseXModeMsgBcpSetParamsComplete(byte[] payload) {
        byte transactionId;
        int requestId;
        short statusCode;
        int resultLength;
        byte result[] = null;
        int offset = 0;
        // Vector parameters = null;
        offset += ProtocolDefines.XMODE_SERVER_SESSION_UUID_SIZE;

        transactionId = payload[offset];
        offset += ProtocolDefines.XMODE_BCP_TRANSACTION_ID_SIZE;

        requestId = ByteConversion.bytesToInt(payload, offset);
        offset += ProtocolDefines.XMODE_BCP_REQUEST_ID_SIZE;

        statusCode = ByteConversion.bytesToShort(payload, offset);
        offset += ProtocolDefines.XMODE_SERVER_BCP_RESPONSE_STATUS_CODE_SIZE;

        resultLength = ByteConversion.bytesToInt(payload, offset);
        offset += ProtocolDefines.XMODE_BCP_SETPARAM_LENGTH_SIZE;

        //HG: Fix for Android NMDP crash:
        // If the length is more than buffer size, ignore this buffer
        // TODO: investigate if we need to read more data
        // The issue was, we were reading command as len header in rear situations.
        if ((resultLength > 0) && (resultLength <= (payload.length - offset))) {
            result = new byte[resultLength];
            System.arraycopy(payload, offset, result, 0, resultLength);
        }

        NMSPSessionListener sessionListener = (NMSPSessionListener)requestIds.remove(new Long(requestId));

        if(sessionListener != null) {
            sessionListener.onBcpSetParamsComplete(transactionId, requestId, statusCode, result);
        }
    }

    private void parseXModeMsgBcpResponse(byte[] payload) {
        byte transactionId;
        int requestId;
        short statusCode;
        short requestState;
        short completionCause;
        int offset = 0;

        offset += ProtocolDefines.XMODE_SERVER_SESSION_UUID_SIZE;

        transactionId = payload[offset];
        offset += ProtocolDefines.XMODE_BCP_TRANSACTION_ID_SIZE;

        requestId = ByteConversion.bytesToInt(payload, offset);
        offset += ProtocolDefines.XMODE_BCP_REQUEST_ID_SIZE;

        statusCode = ByteConversion.bytesToShort(payload, offset);
        offset += ProtocolDefines.XMODE_SERVER_BCP_RESPONSE_STATUS_CODE_SIZE;

        requestState = ByteConversion.bytesToShort(payload, offset);
        offset += ProtocolDefines.XMODE_SERVER_BCP_RESPONSE_REQUEST_STATE_SIZE;

        completionCause = ByteConversion.bytesToShort(payload, offset);
        offset += ProtocolDefines.XMODE_SERVER_BCP_COMPLETION_CAUSE_SIZE;

        // We're not removing the request id, the resource will remove it if the bcp response is a final response.
        NMSPSessionListener sessionListener = (NMSPSessionListener)requestIds.get(new Long(requestId));

        if(sessionListener != null) {
            sessionListener.onBcpResponse(transactionId, requestId, statusCode, requestState, completionCause);
            if(statusCode != ProtocolDefines.XMODE_BCP_RESPONSE_STATUS_OK) {
                requestIds.remove(new Long(requestId));
            }
        }
    }

    private void parseXModeMsgBcpEvent(byte[] payload) {
        byte transactionId;
        // int requestId;
        short eventCode;
//        int payloadLength;
        int offset = 0;

        offset += ProtocolDefines.XMODE_SERVER_SESSION_UUID_SIZE;

        transactionId = payload[offset];
        offset += ProtocolDefines.XMODE_BCP_TRANSACTION_ID_SIZE;

        eventCode = ByteConversion.bytesToShort(payload, offset);
        offset += ProtocolDefines.XMODE_SERVER_BCP_EVENT_CODE_SIZE;

//        payloadLength = ByteConversion.bytesToInt(payload, offset);
//        offset += ProtocolDefines.XMODE_SERVER_BCP_EVENT_PAYLOAD_LENGTH;

        for(int i = 0; i < sessionListeners.size(); i++) {
            NMSPSessionListener sessionListener = (NMSPSessionListener)sessionListeners.elementAt(i);
            sessionListener.onBcpEvent(transactionId, eventCode);
        }
        //sessionListeners.removeAllElements();
    }

    private void parseVapPlayBegin(byte[] payload)
    {
        int audioId;
        int offset = 0;
        audioId = ByteConversion.bytesToInt(payload, offset);

        // No AudioSink available if SDK only sent TEXTParam not TTSParam
        if (audioRelatedSessions.size() == 0)
            return;

        NMSPSessionListener sessionListener = (NMSPSessionListener) audioRelatedSessions.get(new Integer(audioId));
        if (sessionListener == null)
        {
            // couldn't find an audio sink matching the audio id, trying with the NMAS default audio id
            //#ifdef NMSP_LOG
            log.error("parseVapPlayBegin:: Could not find the session listener associated with AID: " + audioId);
            //#endif
            return;
        } else
        {
            sessionListener.onVapPlayBeginReceived();
        }
    }

    private void parseXModeMsgVapPlay(byte[] payload) {
        int audioId;
        int payloadLength;
        int offset = 0;
        NMSPAudioSink audioSink;

        audioId = ByteConversion.bytesToInt(payload, offset);
        offset += ProtocolDefines.XMODE_CLIENT_VAP_AUDIO_ID_SIZE;

        //#ifdef NMSP_LOG
        log.debug("parseXModeMsgVapPlay:: VAP play bytes [" + payload.length + "]");
        //#endif

		//No AudioSink available if SDK only sent TEXTParm not TTSParam
        if (audioSinks.size() == 0)
        	return;

        audioSink = (NMSPAudioSink) audioSinks.get(new Integer(audioId));
        if (audioSink == null) {
            // couldn't find an audio sink matching the audio id, trying with the NMAS default audio id
//            audioSink = (NMSPAudioSink) audioSinks.get(new Integer(defaultAudioId));
//            if (audioSink == null) {
                //#ifdef NMSP_LOG
                log.error("Could not find the audio sink associated with AID: " + audioId);
                //#endif
                return;
//            }
        }

        payloadLength = ByteConversion.bytesToInt(payload, offset);
        offset += ProtocolDefines.XMODE_CLIENT_VAP_AUDIO_LEN_SIZE;

        //
        // This chunk of code is to take off the Nuance header from
        // VAP PLAY buffer, and then drop the Nuance header and export
        // the raw buffer to the application.
        //
        // Note that one VAP_PLAY message can carry one or more
        // than one Vocoded data packet.
        //

        if (Util.isSpeexCodec(outputCodec) || Util.isOpusCodec(outputCodec))
        {
        	while (offset < payload.length)
        	{
                //
                // Nuance Header description:
                // 
                // The highest bit of each length byte indicates whether
                // it is the last length byte and the lowest seven are bits
                // of the length field.
                //
                // Lengths less than 128 are coded with 1 byte e.g.:
                // 127 is: 01111111
                //
                // 128 to 16256 require 2 bytes :
                // 128 is : 10000001
                //          00000000
                //
                // We built in one error fall back logics to avoid crashing
                // with an invalid buffer:
                // 
        	    // The counter should not be greater than 5. ( 5 x 7 bits = 35 bits, too big for int )
        	    //
                int packetLen = 0;
                int oneByte   = 0;
                int count     = 0;
                boolean errorEncountered = false;
                do
                {
                    if ( count >= 5 )
                    {
                        //#ifdef NMSP_LOG
                        log.error("NMSP_COMP_CORE_XMODE_PLAY, Too many audio frame size bytes! This buffer is invalid.");
                        //#endif
                        errorEncountered = true;
                        break;
                    }

                    oneByte = payload[offset] & 0xFF;
                    
                    offset++;
                    count++;
                    packetLen = (packetLen << 7) | (oneByte & 0x7F);
                    
                    //#ifdef NMSP_LOG
                    log.debug("--------> offset " + offset + " packet length " + packetLen + " one byte (" + new Byte((byte) oneByte) + ")");
                    //#endif
                } while ((oneByte & 0x80) != 0);

                if ( packetLen == 0 )
                {
                    errorEncountered = true;
                }

                //#ifdef NMSP_LOG
                log.debug("extract buffer [" + packetLen + "] bytes! offset [" + offset + "]");
                //#endif

                if ( errorEncountered == true )
                {
                    // report NULL as the buffer, 0 as the packet length in the addAudioBuf callback
                    // to indicate this error to the upper layer
                    
                    try
                    {
                        //#ifdef NMSP_CPL_ENABLED
//@                        audioSink.addAudioBuf(null, false, 0, 0);
                        //#else
                        audioSink.addAudioBuf(null, 0, 0, false);
                        //#endif
                    } catch (TransactionProcessingException tpe)
                    {
                        //#ifdef NMSP_LOG
                        log.error(tpe.getMessage());
                        //#endif                        
                    }
                    
                    return;                        
                }
                
                // HG: Fix for Android NMDP crash:
                // If the length is more than buffer size, ignore this buffer
                // TODO: investigate if we need to read more data
                // The issue was, we were reading command as len header in rear situations.
                if (    ( packetLen > 0                            )
                     && ( packetLen <= ( payload.length - offset ) ) )
                {
                    try
                    {
                        //#ifdef NMSP_CPL_ENABLED
//@                         audioSink.addAudioBuf(audio, false, 0, 0);
                        //#else
                        audioSink.addAudioBuf(payload, offset, packetLen, false);
                        //#endif
                    } catch (TransactionProcessingException tpe)
                    {
                        //#ifdef NMSP_LOG
                        log.error(tpe.getMessage());
                        //#endif
                    }
                    offset += packetLen ;
                }
        	}
        	// No AudioSink available if SDK only sent TEXTParam not TTSParam
        	if (audioRelatedSessions.size() == 0)
        		return;

        	NMSPSessionListener sessionListener = (NMSPSessionListener) audioRelatedSessions.get(new Integer(audioId));
        	if (sessionListener == null)
        	{
        		// couldn't find an audio sink matching the audio id, trying with the NMAS default audio id
        		//#ifdef NMSP_LOG
        		            log.error("parseXModeMsgVapPlay:: Could not find the session listener associated with AID: " + audioId);
        		//#endif
        		return;
        	} else
        	{
        		sessionListener.onVapPlayReceived();
        	}
        } else
        {
            //#ifdef NMSP_LOG
            log.debug("extract buffer [" + payloadLength + "] bytes! offset [" + offset + "]");
            //#endif

        	//HG: Fix for Android NMDP crash:
        	// If the length is more than buffer size, ignore this buffer
        	// TODO: investigate if we need to read more data
        	// The issue was, we were reading command as len header in rear situations.
        	if ((payloadLength > 0) && (payloadLength <= (payload.length - offset))) {
        		try {
        			//#ifdef NMSP_CPL_ENABLED
//@                 audioSink.addAudioBuf(audio, false, 0, 0);
        			//#else
        			audioSink.addAudioBuf(payload, offset, payloadLength, false);
        			//#endif
        		} catch (TransactionProcessingException tpe) {
        			//#ifdef NMSP_LOG
                log.error(tpe.getMessage());
                //#endif
            }
        }

        // No AudioSink available if SDK only sent TEXTParam not TTSParam
        if (audioRelatedSessions.size() == 0)
            return;

        NMSPSessionListener sessionListener = (NMSPSessionListener) audioRelatedSessions.get(new Integer(audioId));
        if (sessionListener == null)
        {
        		// couldn't find an audio sink matching the audio id, trying with the NMAS default audio id
        		//#ifdef NMSP_LOG
            log.error("parseXModeMsgVapPlay:: Could not find the session listener associated with AID: " + audioId);
        		//#endif
        		return;
        	} else
        	{
        		sessionListener.onVapPlayReceived();
        	}
        }
    }

    private void parseXModeMsgVapPlayEnd(byte[] payload) {
        int audioId = ByteConversion.bytesToInt(payload, 0);

        if (audioSinks.size() == 0)
        	return;

        NMSPAudioSink audioSink = (NMSPAudioSink)audioSinks.remove(new Integer(audioId));
        if (audioSink == null) {
            // couldn't find an audio sink matching the audio id, trying with the NMAS default audio id
//            audioSink = (NMSPAudioSink) audioSinks.remove(new Integer(defaultAudioId));
//            if (audioSink == null) {
                //#ifdef NMSP_LOG
                log.error("Could not find the audio sink associated with AID: " + audioId);
                //#endif
                return;
//            }
        }

        try {
            // #ifdef NMSP_CPL_ENABLED
//@             audioSink.addAudioBuf(null, true, 0, 0);
            // #else
            audioSink.addAudioBuf(null, 0, 0, true);
            // #endif
        } catch (TransactionProcessingException tpe) {
            //#ifdef NMSP_LOG
            log.error(tpe.getMessage());
            //#endif
        }

        NMSPSessionListener sessionListener = (NMSPSessionListener) audioRelatedSessions.remove(new Integer(audioId));
        if (sessionListener == null)
        {
            // couldn't find an audio sink matching the audio id, trying with the NMAS default audio id
            //#ifdef NMSP_LOG
            log.error("parseXModeMsgVapPlayEnd:: Could not find the session listener associated with AID: " + audioId);
            //#endif
            return;
        } else
        {
            sessionListener.onVapPlayEndReceived();
        }
    }

    private void parseXModeMsgBcpData(byte[] payload) {
        byte transactionId;
//        long requestId;
        int payloadLength;
        int offset = 0;
        offset += ProtocolDefines.XMODE_SERVER_SESSION_UUID_SIZE;

        transactionId = payload[offset];
        offset += ProtocolDefines.XMODE_BCP_TRANSACTION_ID_SIZE;

//        requestId = ByteConversion.bytesToInt(payload, offset);
        offset += ProtocolDefines.XMODE_BCP_REQUEST_ID_SIZE;

        payloadLength = ByteConversion.bytesToInt(payload, offset);
        offset += ProtocolDefines.XMODE_SERVER_BCP_DATA_LENGTH;

          //HG: Fix for Android NMDP crash:
        // If the length is more than buffer size, ignore this buffer
        // TODO: investigate if we need to read more data
        // The issue was, we were reading command as len header in rear situations.
        if ((payloadLength > 0) && (payloadLength <= (payload.length - offset))) {
            byte[] data = new byte[payloadLength];
            System.arraycopy(payload, offset, data, 0, payloadLength);

            // NMAS is not returning the right request id so we always
            // use the same request id for all NMAS requests.
            // In addition, NMAS may return multiple BcpDatas upon one
            // request, for example, intermediate results, so we do not
            // remove the sessionListener from requestIds. Since we always
            // use the same request id, requestIds will not grow.
            NMSPSessionListener sessionListener = (NMSPSessionListener)requestIds.get(new Long(defaultReqId));

            if(sessionListener != null) {
                sessionListener.onBcpData(transactionId, defaultReqId, data);
            }
        }
    }

    public void xmodeMsgCallback(XModeMsgHeader header, byte[] payloadBuf) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("xmodeMsgCallback, protocol: " + header.protocol + ", command: " + header.cmd);
        //#endif
        switch (header.protocol) {
        case ProtocolDefines.XMODE_PROTOCOL_COP:
            switch (header.cmd) {
            case ProtocolDefines.XMODE_COP_COMMAND_CONNECTED:
                sessionId = xmode.sessionId;

                //#ifdef NMSP_LOG
                if(log.isDebugEnabled())
                    log.debug("connected(" + FormatUuid(sessionId) + ") called on " + managerListener);
                //#endif

                createChildEvent(SessionEvent.NMSP_CALLLOG_CONNECTED_EVENT);

                managerListener.connected(FormatUuid(sessionId), resource);

                for(int i = 0; i < sessionListeners.size(); i++) {
                    NMSPSessionListener sessionListener = (NMSPSessionListener)sessionListeners.elementAt(i);
                    sessionListener.onSessionConnected(sessionId);
                }

                emptyMessageQueue();
                break;
            case ProtocolDefines.XMODE_COP_COMMAND_DISCONNECT:
                // Wait for socketClosed() before notifying the listeners
                break;
            case ProtocolDefines.XMODE_COP_COMMAND_CONNECT_FAILED:
                // Wait for socketClosed() before notifying the listeners
                break;
            default: // XMODE_COP_COMMAND_PING_REQUEST
            }
            break;
        case ProtocolDefines.XMODE_PROTOCOL_BCP:
            switch (header.cmd) {
            case ProtocolDefines.XMODE_BCP_COMMAND_RECOGNITION_COMPLETE:
                parseXModeMsgBcpRecognitionComplete(payloadBuf);
                break;
            case ProtocolDefines.XMODE_BCP_COMMAND_RECOGNITION_INTERMRESULTS:
                parseXModeMsgBcpRecognitionIntermediateResults(payloadBuf);
                break;
            case ProtocolDefines.XMODE_BCP_COMMAND_GET_PARAMS_COMPLETE:
                parseXModeMsgBcpGetParamsComplete(payloadBuf);
                break;
            case ProtocolDefines.XMODE_BCP_COMMAND_SET_PARAMS_COMPLETE:
                parseXModeMsgBcpSetParamsComplete(payloadBuf);
                break;
            case ProtocolDefines.XMODE_BCP_COMMAND_GENERATE_AUDIO_COMPLETE:
                parseXModeMsgBcpGenerateAudioComplete(payloadBuf);
                break;
            case ProtocolDefines.XMODE_BCP_COMMAND_EVENT:
                parseXModeMsgBcpEvent(payloadBuf);
                break;
            case ProtocolDefines.XMODE_BCP_COMMAND_RESPONSE:
                parseXModeMsgBcpResponse(payloadBuf);
                break;
            case ProtocolDefines.XMODE_BCP_COMMAND_START_OF_SPEECH:
                break;
            case ProtocolDefines.XMODE_BCP_COMMAND_BCP_DATA:
                parseXModeMsgBcpData(payloadBuf);
                break;
            default:
                //#ifdef NMSP_LOG
                log.error("Unknown BCP command");
                //#endif
                break;
            }
            break;
        case ProtocolDefines.XMODE_PROTOCOL_VAP:
            switch (header.cmd) {
                case ProtocolDefines.XMODE_VAP_COMMAND_PLAY_BEGIN:
                    parseVapPlayBegin(payloadBuf);
                    break;
                case ProtocolDefines.XMODE_VAP_COMMAND_PLAY_END:
                    parseXModeMsgVapPlayEnd(payloadBuf);
                    break;
                case ProtocolDefines.XMODE_VAP_COMMAND_PLAY:
                    parseXModeMsgVapPlay(payloadBuf);
                    break;
            }
            break;
        default:
            //#ifdef NMSP_LOG
            log.error("Unknown Xmode protocol");
            //#endif
            break;
        }
    }

    public void handleMessage(Object msgData, Object msgSender) {
        MessageData sessionMsgData = (MessageData) msgData;
        Object array[];

        switch (sessionMsgData.command) {
        case CMD_CONNECT:
            handleConnect();
            break;
        case CMD_DISCONNECT:
            handleDisconnect();
            break;
        case CMD_DISCONNECT_AND_SHUTDOWN:
            handleDisconnectAndShutdown();
            break;
        case CMD_BCP_FREE_RESOURCE:
            if (sessionId == null) {
                break;
            }
            array = (Object[]) sessionMsgData.data;
            handleFreeResource(((Byte)array[0]).byteValue(), ((Integer)array[1]).intValue());
            break;
        case CMD_VAP_RECORD_BEGIN:
            if (sessionId == null) {
                break;
            }
            handleVapRecordbegin(((Integer) sessionMsgData.data).intValue());
            break;
        case CMD_VAP_RECORD:
            if (sessionId == null) {
                break;
            }
            array = (Object[]) sessionMsgData.data;
            handleVapRecord(((byte[]) array[0]), ((Integer) array[1]).intValue());
            break;
        case CMD_VAP_RECORD_END:
            if (sessionId == null) {
                break;
            }
            handleVapRecordEnd(((Integer) sessionMsgData.data).intValue());
            break;
        case CMD_BCP_SEND_MSG:
            if (sessionId == null) {
                break;
            }
            Object[] array_bcp_send_msg = (Object[]) sessionMsgData.data;
            sendBcpMessage(((Short)array_bcp_send_msg[0]).shortValue(),
                           ((String)array_bcp_send_msg[1]),
                           (byte[]) array_bcp_send_msg[2],
                           (byte[]) array_bcp_send_msg[3],
                           ((Byte) array_bcp_send_msg[4]).byteValue(),
                           ((Long) array_bcp_send_msg[5]).longValue(),
                           (NMSPSessionListener) array_bcp_send_msg[6],
                           ((Boolean) array_bcp_send_msg[7]).booleanValue());
            break;
        default:
            //#ifdef NMSP_LOG
            log.error("Unknown command");
            //#endif
            break;
        }
    }

    private void sendCmdMessage(byte cmd, Object data) {
        messageSystem.send(new MessageData(cmd, data), this, Thread.currentThread(), (messageSystem.getVRAddr())[0]);
    }

    private void emptyMessageQueue() {
        MessageData sessionMsgData;

        while (!messageQueue.isEmpty()) {
            sessionMsgData = (MessageData) messageQueue.firstElement();
            messageQueue.removeElementAt(0);
            switch (sessionMsgData.command) {
            case CMD_CONNECT:
                sendCmdMessage(CMD_CONNECT, sessionMsgData.data);
                break;
            case CMD_DISCONNECT:
                sendCmdMessage(CMD_DISCONNECT, sessionMsgData.data);
                break;
            case CMD_BCP_FREE_RESOURCE:
                sendCmdMessage(CMD_BCP_FREE_RESOURCE, sessionMsgData.data);
                break;
            case CMD_VAP_RECORD_BEGIN:
                sendCmdMessage(CMD_VAP_RECORD_BEGIN, sessionMsgData.data);
                break;
            case CMD_VAP_RECORD:
                sendCmdMessage(CMD_VAP_RECORD, sessionMsgData.data);
                break;
            case CMD_VAP_RECORD_END:
                sendCmdMessage(CMD_VAP_RECORD_END, sessionMsgData.data);
                break;
            case CMD_BCP_SEND_MSG:
                sendCmdMessage(CMD_BCP_SEND_MSG, sessionMsgData.data);
                break;
            }
        }
    }

    private void clearMessageQueue() {
        if (!messageQueue.isEmpty())
            messageQueue.removeAllElements();
    }

    public void socketOpened() {
        // TODO Auto-generated method stub
    }

    public void copConnected() {
     // TODO Auto-generated method stub
    }

    public void addAudioSink(int audioId, NMSPAudioSink audioSink, NMSPSessionListener sessionListener) {
        this.audioSinks.put(new Integer(audioId), audioSink);
        this.audioRelatedSessions.put(new Integer(audioId), sessionListener);
    }

    public byte[] getSessionId() {
        return this.sessionId;
    }

    // app thread
    public long getResourceId() {
        long resId = resourceIdCounter++;
        if (resourceIdCounter == Long.MIN_VALUE)
            resourceIdCounter = 0x0001;

        return resId;
    }

    // app thread
    public byte getTransactionId() {
        //#ifdef NMSP_PLATFORM_BLACKBERRY
//@        RuntimeStore runtimeStore = RuntimeStore.getRuntimeStore();
//@        Object tmpObject = runtimeStore.get(ID_NMSP_SDK_TRANSACTION_ID_COUNTER);
//@        if ((tmpObject != null) && (tmpObject instanceof Byte)) {
//@            tranIdCounter = ((Byte) tmpObject).byteValue();
//@        }
        //#endif
        byte transactionId = tranIdCounter++;
        /**
         * in java, byte is signed, should round between (0, Byte.MAX_VALUE]
         * byte is in the range [-128, 127], we will take [1, 127] byte's serial
         * range is [0, 1, 2 ... 127, -128, -127 ... -2, -1]
         **/
        if (tranIdCounter == Byte.MIN_VALUE)
            tranIdCounter = 0x01;

        //#ifdef NMSP_PLATFORM_BLACKBERRY
//@        runtimeStore.replace(ID_NMSP_SDK_TRANSACTION_ID_COUNTER, new Byte(tranIdCounter));
        //#endif

        return transactionId;
    }

    public NMSPDefines.Codec getInputCodec() {
        return inputCodec;
    }

    public MessageSystem getMsgSystem() {
        return this.messageSystem;
    }

    public Vector getMsgQueue() {
        return this.messageQueue;
    }

    public boolean isNetworkHealthy() {
        return xmode.isNetworkHealthy();
    }

    public void xmodeMsgSent(String context, Object data) {
        // do nothing for now
        // more can be added in the future if we want to do something in the
        // message sent callback
    }

    public void xmodeMsgNotSent(String context, Object data) {
        // do nothing for now
        // more can be added in the future if we want to do something in the
        // message not sent callback
    }

    public void createSessionEvent(CalllogImpl calllog) {
        //#ifdef NMSP_LOG
            log.debug("Creating new SDK calllog tree");
        //#endif
    	if(_calllog == null){
    		return;
    	}

    	SessionEvent parent = _calllog.getRootSessionEvent();
    	if(parent != null){
	        SessionEventBuilder builder = parent.createChildEventBuilder(SessionEvent.NMSP_CALLLOG_NMSPSESSION);
    		try {
				builder.createRemoteEvent(SessionEvent.NMSP_CALLLOG_NMSPGW_EVENT);
			} catch (SessionEventAlreadyCommittedException e) {
	            //#ifdef NMSP_LOG
                if(log.isErrorEnabled())
                    log.error("XMode.createSessionEvent() parent is already committed");
                //#endif
			}
	        this.sessionEvent = builder.commit();
	        xmode.createSessionEvent(this.sessionEvent);
    	}
    }
    public SessionEvent getSessionEvent() {
        return sessionEvent;
    }

    private void createChildEvent(String eventName)
    {
    	if (sessionEvent != null) {
    		SessionEventBuilder bldr = sessionEvent.createChildEventBuilder(eventName);
    		if (eventName.compareTo(SessionEvent.NMSP_CALLLOG_CONNECTED_EVENT) == 0) {
    			try {
					bldr.putString("SessionID", FormatUuid(sessionId));
				} catch (SessionEventAlreadyCommittedException e) {}
    		}
    		bldr.commit();
    	}
    }
    
    private void cancelGwRemoteEvent(int causeCode, String causeMessage)
    {
        if ( sessionEvent != null )
        {
            String eventIdToCancel = null;
            Map map = sessionEvent.getRefIds();
            if ( map != null )
            {
                if ( map.containsKey(SessionEvent.NMSP_CALLLOG_NMSPGW_EVENT) == true )
                {
                    Object obj = map.get(SessionEvent.NMSP_CALLLOG_NMSPGW_EVENT);
                    if ( obj instanceof String )
                    {
                        eventIdToCancel = (String) obj;
                    }
                }
            }

            if (    (            eventIdToCancel != null  )
                 && ( eventIdToCancel.equals("") == false ) )
            {
                SessionEventBuilder builder = sessionEvent.createChildEventBuilder(SessionEventImpl.NMSP_CALLLOG_CANCEL_LOG_EVENT);
                
                try
                {
                    builder.putInteger (    "CauseCode",  causeCode    );
                    builder.putString  ( "CauseMessage",  causeMessage );
                    
                    ((SessionEventBuilderImpl) builder).setEventToCancel(eventIdToCancel, SessionEvent.NMSP_CALLLOG_NMSPGW_EVENT);
                } catch (SessionEventAlreadyCommittedException e)
                {
                    //#ifdef NMSP_LOG
                if ( log.isErrorEnabled() )
                    log.error("NMSPSession.cancelGwRemoteEvent() the builder is already committed.");
                    //#endif
                }
                builder.commit();
            }
        }
    }
   
    //#ifdef NMSP_CPL_ENABLED
//@    public long getConnectionEstablishedTime() {
//@      return xmode.connectionEstablishedTime;
//@    }
//@
//@    public long getOpenSocketTime() {
//@      return xmode.openSocketTime;
//@    }
//@
//@    public long getSocketOpenedTime() {
//@      return xmode.socketOpenedTime;
//@    }
//@
//@    public Vector getSocketReadRecords() {
//@      return xmode.cplSocketReadRecords;
//@    }
//@
//@    public Vector getSocketWriteRecords() {
//@      return xmode.cplSocketWriteRecords;
//@    }
//@
//@    public Vector getXmodeReadMsgs() {
//@      return xmode.cplXmodeReadMsgs;
//@    }
//@
//@    public Vector getXmodeWriteMsgs() {
//@      return xmode.cplXmodeWriteMsgs;
//@    }
//@
//@    public Vector getSessionWriteMsgs() {
//@      return sessionWriteMsgs;
//@    }
//@
//@    public Vector getXmodeReadRequests() {
//@      return xmode.cplXmodeReadRequests;
//@    }
//@
//@    public void clearCPL() {
//@      sessionWriteMsgs.removeAllElements();
//@      xmode.clearCPL();
//@    }
    //#endif


    // Formats the sessionid to a String
    public static String FormatUuid(byte[] uuid)
    {
        if (uuid == null)
            return "";

        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < 16; i++)
        {
            buf.append(getHex(uuid[i]));
            if (i == 3 || i == 5 || i == 7 || i == 9)
                buf.append("-");
        }

        return buf.toString().toLowerCase();
    }

    private static String pseudo[] = { "0", "1", "2", "3", "4", "5", "6",
        "7", "8", "9", "A", "B", "C", "D", "E", "F" };

    private static String getHex(byte b)
    {
        byte ch1 = (byte) (b & 0xF0);
        ch1 = (byte) (ch1 >>> 4);
        ch1 = (byte) (ch1 & 0x0F);

        byte ch2 = (byte) (b & 0x0F);

        return pseudo[(int) ch1] + pseudo[(int) ch2];
    }
    
    void setCalllog(CalllogImpl calllog)
    { 
        _calllog = (CalllogImpl) calllog;
    }

}
