/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client.sdk.components.resource.internal.nmas;

import java.util.Vector;

//#ifdef NMSP_PLATFORM_BLACKBERRY
//@import net.rim.device.api.system.ApplicationDescriptor;
//@import net.rim.device.api.system.CodeModuleManager;
//@import com.nuance.nmsp.client.sdk.components.resource.internal.common.NMSPSession;
//#endif

import com.nuance.nmsp.client.sdk.common.defines.NMSPDefines;
import com.nuance.nmsp.client.sdk.common.oem.api.MessageSystem;
import com.nuance.nmsp.client.sdk.common.oem.api.MessageSystem.MessageData;
import com.nuance.nmsp.client.sdk.components.core.calllog.SessionEvent;
import com.nuance.nmsp.client.sdk.components.core.calllog.SessionEventBuilder;
import com.nuance.nmsp.client.sdk.components.core.internal.pdx.PDXDictionary;
import com.nuance.nmsp.client.sdk.components.core.internal.pdx.PDXSequence;
import com.nuance.nmsp.client.sdk.components.core.pdx.Dictionary;
import com.nuance.nmsp.client.sdk.components.core.pdx.Sequence;
import com.nuance.nmsp.client.sdk.components.general.NMSPAudioSink;
import com.nuance.nmsp.client.sdk.components.resource.common.ResourceException;
import com.nuance.nmsp.client.sdk.components.resource.common.ResourceListener;
import com.nuance.nmsp.client.sdk.components.resource.internal.common.ManagerImpl;
import com.nuance.nmsp.client.sdk.components.resource.internal.common.ResourceImpl;
import com.nuance.nmsp.client.sdk.components.resource.nmas.AudioParam;
import com.nuance.nmsp.client.sdk.components.resource.nmas.Command;
import com.nuance.nmsp.client.sdk.components.resource.nmas.NMASResource;
import com.nuance.nmsp.client.sdk.components.resource.nmas.NMASResourceListener;
import com.nuance.nmsp.client.sdk.components.resource.nmas.PDXCommandListener;
import com.nuance.nmsp.client.sdk.components.resource.nmas.Parameter;
import com.nuance.nmsp.client.sdk.components.resource.internal.nmas.PDXTransactionImpl.TransactionLogEntry;
//#ifdef NMSP_LOG
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory.Log;
//#endif

public class NMASResourceImpl extends ResourceImpl implements NMASResource {
    //#ifdef NMSP_LOG
    private static final Log log = LogFactory.getLog(NMASResourceImpl.class);
    //#endif
    

    private MessageSystem messageSystem;
    private NMASResourceListener nmasListener;
    private PDXTransactionImpl pdxTransaction;
    
   
//    private static final byte CMD_CONNECT  = 0x04;

    public static final byte BCP_BEGIN = 0x04;
    public static final byte BCP_DATA = 0x05;
    
    public NMASResourceImpl(ManagerImpl manager, NMASResourceListener nmasListener, Vector parameters, String application) {
        super(manager, application, nmasListener, parameters, ResourceImpl.NMAS);
        
        this.nmasListener = nmasListener;
        this.messageSystem = manager.getMsgSys();
        //#ifdef NMSP_PLATFORM_BLACKBERRY
//@        this.session = new NMSPSession(manager.getGatewayIP(), manager.getGatewayPort(), manager._applicationId, manager._appKey, manager.getUid(), manager._params, manager.getMsgSys(), manager._managerListener);
        //#else
        this.session = manager.getSession();
        //#endif
        this.session.setResource(this);
        this.pdxTransaction = null;
        
        messageSystem.send(new MessageData(CMD_LOAD_RESOURCE, null),
                this, Thread.currentThread(), messageSystem.getVRAddr()[0]);
    }
    
    //app thread
    public Command createCommand(PDXCommandListener commandListener,
            String cmd, String scriptVersion, String dictationLanguage, String carrier, String phoneModel,
            Dictionary optionalKeys, long commandTimeout) throws ResourceException {

        /* verify parameters */
        String errorMsg = "";
        if(commandListener == null)
            errorMsg += "commandListener is invalid; ";
        if (cmd == null || cmd.equals(""))
            errorMsg += "cmd should be non-null; ";
        if (scriptVersion == null || scriptVersion.equals(""))
            errorMsg += "scriptVersion should be non-null; ";
        if (dictationLanguage == null || dictationLanguage.equals(""))
            errorMsg += "dictationLanguage should be non-null; ";
        // do not validate carrier since 3.5 server can handle this condition gracefully
        /**if (carrier == null || carrier.equals(""))
            errorMsg += "carrier should be non-null; ";**/
        if (phoneModel == null || phoneModel.equals(""))
            errorMsg += "phoneModel should be non-null; ";
        if (commandTimeout <= 0)
            errorMsg += "commandTimeout is invalid; ";

        if (!errorMsg.equals("")) {
            //#ifdef NMSP_LOG
            log.error("NMASResourceImpl.createCommand() " + errorMsg);
            //#endif
            throw new IllegalArgumentException(errorMsg);
        }
        
        synchronized (syncObject) {
            if(pdxTransaction != null) {
                pdxTransaction.transactionOver();
            }

            session.setDefaultReqId(defaultReqId);

            if(resourceState == STATE_RESOURCE_UNLOADED) {
                messageSystem.send(new MessageData(CMD_LOAD_RESOURCE, null),
                        this, Thread.currentThread(), messageSystem.getVRAddr()[0]);
            }
            
            this.tranId = session.getTransactionId();
            
            pdxTransaction = new PDXTransactionImpl(messageSystem, commandListener, cmd,
            		                               ((ManagerImpl)manager).applicationId, session, ((ManagerImpl)manager).getUid(), "1", NMSPDefines.VERSION, CLIENT_VERSION,
            		                               scriptVersion, "enus", "ne", ((ManagerImpl)manager).getInputCodec(),
            		                               dictationLanguage,(short) 1, (short)1, carrier,
            		                               phoneModel, ((ManagerImpl)manager).getUid(), "", 
            		                               /*session.getSessionId(), */commandTimeout, optionalKeys, this, nmasListener, tranId);

            return pdxTransaction;
        }
    }

    /**
    //sdk thread
    private void handleConnect(Codec inputCodec, Codec outputCodec) {
        session.setSessionListener(this);
        session.connect(inputCodec, outputCodec);
    }**/
    
    private void handleLoadResource()
    {
        super.loadResource(this, null);
    }
    
    public long getRequestId() {
        return super.getRequestId();
    }
    
    //app thread
    public AudioParam createPDXAudioParam(String name) {
        if(name == null) {
            throw new NullPointerException("name can not be null");
        }
        return new PDXAudioParam(name, session, this.messageSystem);
    }

    //app thread
    public Parameter createPDXChoiceParam(String name, String choicename) {
        if(name == null) {
            throw new NullPointerException("name can not be null");
        }
        if(choicename == null) {
            throw new NullPointerException("choicename can not be null");
        }
        return new PDXChoiceParam(name, choicename);
    }

    //app thread
    public Parameter createPDXDataParam(String name, byte[] data) {
        if(name == null) {
            throw new NullPointerException("name can not be null");
        }
        if(data == null) {
            throw new NullPointerException("data can not be null");
        }
        return new PDXDataParam(name, data);
    }

    //app thread
    public Parameter createPDXDictParam(String name, Dictionary dict) {
        if(name == null) {
            throw new NullPointerException("name can not be null");
        }
        if(dict == null) {
            throw new NullPointerException("dict can not be null");
        }
        return new PDXDictParam(name, (PDXDictionary) dict);
    }

    //app thread
    public Dictionary createPDXDictionary() {
        return new PDXDictionary();
    }

    //app thread
    public Parameter createPDXSeqChunkParam(String name, Dictionary dict) {
        if(name == null) {
            throw new NullPointerException("name can not be null");
        }
        if(dict == null) {
            throw new NullPointerException("dict can not be null");
        }
        return new PDXSeqParam(name, (PDXDictionary) dict, PDXSeqParam.SEQ_CHUNK);
    }

    //app thread
    public Parameter createPDXSeqEndParam(String name, Dictionary dict) {
        if(name == null) {
            throw new NullPointerException("name can not be null");
        }
        if(dict == null) {
            throw new NullPointerException("dict can not be null");
        }
        return new PDXSeqParam(name, (PDXDictionary) dict, PDXSeqParam.SEQ__END);
    }

    //app thread
    public Parameter createPDXSeqStartParam(String name, Dictionary dict) {
        if(name == null) {
            throw new NullPointerException("name can not be null");
        }
        if(dict == null) {
            throw new NullPointerException("dict can not be null");
        }
        return new PDXSeqParam(name, (PDXDictionary) dict, PDXSeqParam.SEQ_START);
    }

    //app thread
    public Sequence createPDXSequence() {
        return new PDXSequence();
    }

    //app thread
    public Parameter createPDXTTSParam(String name, Dictionary tts_dict, NMSPAudioSink audioSink) {
    	if(name == null) {
            throw new NullPointerException("name can not be null");
        }
    	if(tts_dict == null) {
            throw new NullPointerException("tts_dict can not be null");
        }
        if(audioSink == null) {
            throw new NullPointerException("audioSink can not be null");
        }
        return new PDXTTSParam(name, session, (PDXDictionary)tts_dict, audioSink);
    }

    //app thread
    public Parameter createPDXTextParam(String name, String text) {
        if(name == null) {
            throw new NullPointerException("name can not be null");
        }
        if(text == null) {
            throw new NullPointerException("text can not be null");
        }
        return new PDXTextParam(name, text);
    }

    //app thread
    public void freeResource(int disconnectTimeout) throws ResourceException {
        //#ifdef NMSP_LOG
        log.debug("freeResource() disconnectTimeout:" + disconnectTimeout);
        //#endif
        if(pdxTransaction != null) {
            pdxTransaction.transactionOver();
        }
        synchronized (syncObject) {
            // resourceLoaded is not relevant to NMAS resource.
            if (resourceState == STATE_RESOURCE_LOADED) {
            	resourceState = STATE_RESOURCE_UNLOADED;
                Integer dTimeout = new Integer(disconnectTimeout);
                messageSystem.send(new MessageData(CMD_FREE_RESOURCE, dTimeout), this,
                                   Thread.currentThread(), messageSystem.getVRAddr()[0]);
            } else {
                throw new ResourceException(RESOURCE_UNLOADED_EXCEPTION);
            }
        }
    }
    
    public void onSessionConnected(byte[] sessionId) {
        super.onSessionConnected(sessionId);
        if(pdxTransaction != null)
            pdxTransaction.onSessionConnected(sessionId);
    }
    
    public void onSessionDisconnected(short reasonCode) {
        //#ifdef NMSP_LOG
        log.debug("onSessionDisconnected() reasonCode:" + reasonCode);
        //#endif
        if(pdxTransaction != null)
            pdxTransaction.onSessionDisconnected(reasonCode);
        super.onSessionDisconnected(reasonCode);
    }
    
    public void onBcpData(byte transactionId, long requestId, byte[] data) {
        PDXMessage pdxMessage = PDXMessageFactory.createMessage(data);
        switch (pdxMessage.getCommandCode()) {
        // case PDXMessage.GetScriptUpdate:
        // break;
        // case PDXMessage.QueryBegin:
        // break;
        // case PDXMessage.QueryParameter:
        // break;
        // case PDXMessage.EnrollementSegment:
        // break;
        // case PDXMessage.EnrollementComplete:
        // break;
        // case PDXMessage.DiagnosticLogUpdate:
        // break;
        case PDXMessage.QueryResult:
            clearResLogsToServer(transactionId);
            if(pdxTransaction != null)
                pdxTransaction.onQueryResults((PDXQueryResult) pdxMessage, transactionId);
            break;
        case PDXMessage.QueryError:
            if(pdxTransaction != null)
                pdxTransaction.onQueryError((PDXQueryError) pdxMessage, transactionId);
            break;
        case PDXMessage.QueryRetry:
            clearResLogsToServer(transactionId);
            if(pdxTransaction != null)
                pdxTransaction.onQueryRetry((PDXQueryRetry) pdxMessage, transactionId);
            break;
        // case PDXMessage.QueryDisambiguate:
        // pdxTransaction.onQueryDisambiguate((PDXQueryDisambiguate)pdxMessage);
        // break;
        // case PDXMessage.QueryNarrow:
        // pdxTransaction.onQueryNarrow((PDXQueryNarrow)pdxMessage);
        // break;
        // case PDXMessage.QueryRetry:
        // break;
        // case PDXMessage.EnrollementText:
        // break;
        // case PDXMessage.ScriptUpdateAvailable:
        // pdxTransaction.onScriptUpdateAvailable((PDXScriptUpdateAvailable)pdxMessage);
        // break;
        // case PDXMessage.UpdateChunk:
        // pdxTransaction.onUpdateChunk((PDXUpdateChunk)pdxMessage);
        // break;
        // case PDXMessage.NoScriptUpdateAvailable:
        // pdxTransaction.onNoScriptUpdateAvailable();
        // break;
        default:
            //#ifdef NMSP_LOG
            log.error("Session.parseXModeMsgBcpData() Unknown command: " + pdxMessage.getCommandCode() + ". ");
            //#endif
            break;
        }
    }
    
    private void clearResLogsToServer(byte transactionId) {
    	if(pdxTransaction == null){
    		return;
    	}
    	
        if (transactionId == pdxTransaction.tranId) {
            Vector v = (Vector) ((ManagerImpl) getManager()).getResourceLogs();
            if (v != null && v.size() > 0) {
                //#ifdef NMSP_LOG
                log.debug("clearResLogsToServer() before clean the log vector tranId[" + transactionId + "] log list size [" + v.size() + "]");
                //#endif
                for (int i = 0; i < v.size();) {
                    TransactionLogEntry entry = (TransactionLogEntry) v.elementAt(i);
                    if (entry.isSent())
                        v.removeElement(entry);
                    else
                        i++;                
                }
                //#ifdef NMSP_LOG
                log.debug("clearResLogsToServer() after clean the log vector tranId[" + transactionId + "] log list size [" + v.size() + "]");
                //#endif
            }
        }
    }
    
    public void onBcpEvent(byte transactionId, short eventCode) {
        /** this is only for nmas command, it will be always 0 for other BCP response message **/
        if (transactionId != this.tranId) {
            return; // not for this nmas resource
        }
        if(pdxTransaction != null) {
            pdxTransaction.transactionOver();
        }
        super.onBcpEvent(transactionId, eventCode);
    }
    
    public void onMsgSent(byte transactionId, String context, Object data) {
        // do nothing for now
        // normally we can do something based on the context
    }
    
    public void onMsgNotSent(byte transactionId, String context, Object data) {
        // do nothing for now
        // normally we can do something based on the context
    }
    
    public void onVapSending() {
        // reset command timeout task upon sending a VAP_Record
	    if(pdxTransaction != null){
	        pdxTransaction.resetCommandTimeoutTask();
	    }
    }

    public void onVapPlayBeginReceived()
    {
        // reset command timeout task upon receiving a VAP_Play_Begin
    	if(pdxTransaction != null){	
            pdxTransaction.onVapPlayBeginReceived();
	    }
    }

    public void onVapPlayReceived()
    {
        // reset command timeout task upon receiving a VAP_Play
    	if(pdxTransaction != null){	
            pdxTransaction.onVapPlayReceived();
	    }
    }

    public void onVapPlayEndReceived()
    {
        // reset command timeout task upon receiving a VAP_Play_End
    	if(pdxTransaction != null){	
            pdxTransaction.onVapPlayEndReceived();
	    }
    }

    public void handleMessage(Object msgData, Object msgSender) {
        MessageData messageData = (MessageData) msgData;
        
        switch (messageData.command) {
            case CMD_LOAD_RESOURCE:
                handleLoadResource();
                break;
//call loadResource() in createComand, loadResource will reconnect                 
//            case CMD_CONNECT:
//                Object[] params = (Object[])messageData.data;
//                Codec inputCodec = (Codec)params[0];
//                Codec outputCodec = (Codec)params[1];
//                handleConnect(inputCodec, outputCodec);
//                break;
            case CMD_FREE_RESOURCE:
                session.removeSessionListener(this);
                nmasListener.resourceUnloaded(ResourceListener.EVENT_CODE_CLIENT_DISCONNECTED_NORMALLY);
                //#ifdef NMSP_PLATFORM_BLACKBERRY
//@                // disconnect the session upon freeResource, prevent receiving more
//@                // bcp messages upon cancelling the current transaction
//@                // the freeResource behaves more like a cancel for blackberry nmas resource
//@                session.disconnect();
                //#endif
                break;
            default:
                super.handleMessage(msgData, msgSender);
                break;
        }
    }
    

    
    //#ifdef NMSP_PLATFORM_BLACKBERRY
//@    // Method to retrieve the Blackberry Device Software
//@    // Version
//@    private static String CLIENT_VERSION = getRIMOSVersion();
//@    public static String getRIMOSVersion() {
//@        int ribbonHandle = CodeModuleManager.getModuleHandle("net_rim_bb_ribbon_app");
//@        ApplicationDescriptor[] ribbonDesps = CodeModuleManager.getApplicationDescriptors(ribbonHandle);
//@        if (ribbonDesps != null && ribbonDesps.length > 0) {
//@            return ribbonDesps[0].getVersion();
//@        }
//@        return "RIM_OS_VERSION";
//@    }
    //#elif NMSP_PLATFORM_ANDROID
    private static String CLIENT_VERSION = android.os.Build.VERSION.RELEASE;
    //#else
//@    private static String CLIENT_VERSION = "J2SE_OR_J2ME_OS_VERSION";
    //#endif
}
