/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client.sdk.components.resource.internal.nmas;

//#ifdef NMSP_LOG
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory.Log;
//#endif
import com.nuance.nmsp.client.sdk.components.resource.nmas.QueryResult;

//<DICT>
//<ASCII>result_type</ASCII><UTF8>(name of results)</UTF8>
//(Query specific results)
//</DICT>

public class PDXQueryResult extends PDXMessage implements QueryResult {
    //#ifdef NMSP_LOG
    private static final Log log = LogFactory.getLog(PDXQueryResult.class);
    //#endif

    private final byte[] input;

    public PDXQueryResult(byte[] input) {
        super(PDXMessage.QueryResult, input);
        this.input = input;
        //#ifdef NMSP_LOG3
//@        String inputHex = "";
//@        for(int i = 0; i < input.length; i++) {
//@            inputHex += Integer.toString( ( input[i] & 0xff ) + 0x100, 16).substring( 1 );
//@        }
//@        log.debug("PDXQueryResult() " + inputHex);
        //#endif
    }

    public String getResultType() {
        //#ifdef NMSP_LOG
        log.debug("PDXQueryResult.getResultType()");
        //#endif
        String resultType = getUTF8String("result_type");
        if (resultType != null) {
            return resultType;
        } else {
            return "";
        }
    }

    public byte[] getRawInput() {
        return input;
    }
    
    public boolean isFinalResponse() {
        boolean finalResponse = true;
        try {
            if (containsKey("final_response")) {
                if (getInteger("final_response") == 0){
                    finalResponse = false;
                };
            }
            else {
            //#ifdef NMSP_LOG
         if(log.isInfoEnabled())
             log.info("final_response does not exist. ");
            //#endif
            }
        }catch(Exception e){}
    
        return finalResponse;
    }
}
