/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client.sdk.components.resource.internal.nmas;

//#ifdef NMSP_LOG
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory.Log;
//#endif

//#ifdef NMSP_CPL_ENABLED
//@import com.nuance.nmsp.client.sdk.common.oem.api.NetworkSystem.CPLSocketReadRecord;
//@import com.nuance.nmsp.client.sdk.common.oem.api.NetworkSystem.CPLSocketReadContext;
//@import com.nuance.nmsp.client.sdk.common.oem.api.NetworkSystem.CPLSocketWriteContext;
//@import com.nuance.nmsp.client.sdk.common.oem.api.log.CPLRecord;
//@
//@import java.io.ByteArrayOutputStream;
//@import java.util.Date;
//@import java.util.Enumeration;
//#endif

import java.util.Vector;
import com.nuance.nmsp.client.sdk.components.resource.common.ManagerListener;
import com.nuance.nmsp.client.sdk.components.resource.internal.common.ResourceImpl;
import com.nuance.nmsp.client.sdk.components.resource.internal.common.ManagerImpl;

import com.nuance.nmsp.client.sdk.common.defines.NMSPDefines;
import com.nuance.nmsp.client.sdk.common.oem.api.MessageSystem;
import com.nuance.nmsp.client.sdk.common.oem.api.MessageSystem.MessageData;
import com.nuance.nmsp.client.sdk.common.oem.api.TimerSystem.TimerSystemTask;
import com.nuance.nmsp.client.sdk.common.protocols.ProtocolDefines;
import com.nuance.nmsp.client.sdk.common.util.Util;
import com.nuance.nmsp.client.sdk.components.core.XMode;
import com.nuance.nmsp.client.sdk.components.core.calllog.SessionEvent;
import com.nuance.nmsp.client.sdk.components.core.calllog.SessionEventAlreadyCommittedException;
import com.nuance.nmsp.client.sdk.components.core.calllog.SessionEventBuilder;
import com.nuance.nmsp.client.sdk.components.core.internal.pdx.PDXClass;
import com.nuance.nmsp.client.sdk.components.core.internal.pdx.PDXDictionary;
import com.nuance.nmsp.client.sdk.components.core.internal.pdx.PDXSequence;
import com.nuance.nmsp.client.sdk.components.core.pdx.Dictionary;
import com.nuance.nmsp.client.sdk.components.general.TransactionAlreadyFinishedException;
import com.nuance.nmsp.client.sdk.components.general.TransactionExpiredException;
import com.nuance.nmsp.client.sdk.components.resource.internal.common.NMSPSession;
import com.nuance.nmsp.client.sdk.components.resource.nmas.Command;
import com.nuance.nmsp.client.sdk.components.resource.nmas.NMASResourceListener;
import com.nuance.nmsp.client.sdk.components.resource.nmas.PDXCommandListener;
import com.nuance.nmsp.client.sdk.components.resource.nmas.Parameter;
import com.nuance.nmsp.client.sdk.oem.BluetoothSystemOEM;

public class PDXTransactionImpl implements Command, MessageSystem.MessageHandler {
    //#ifdef NMSP_LOG
    private static final Log log = LogFactory.getLog(PDXTransactionImpl.class);
    //#endif
    
    private NMASResourceImpl nmasResource;
    private NMASResourceListener nmasListener;
    private PDXCommandListener pdxCommandListener = null;
    private NMSPSession session = null;

    private SessionEvent sessionEvent;
    
    /* transaction state definitions */
    public final static short STATE_IDLE    = -1;
    public final static short STATE_BEGUN   = 0;
    public final static short STATE_ENDED   = 1;
    public final static short STATE_EXPIRED = 2;

    private short state = STATE_IDLE;

    private MessageSystem msgSys;
    
    /* command timeout definitions */
    private TimerSystemTask commandTask;
    private long commandTimeout;
    private boolean resultReceived;

    /* transaction identification definitions */
    protected byte tranId;
    private String commandName;
    
    // bug 22727 a temp fix for now. The whole class needs to be refactored.
    private boolean commandCreated = false;
    
    //#ifdef NMSP_CPL_ENABLED
//@    private String cplCommandName;
//@    private Vector cplAudioParamsRecords;
//@    private boolean cplAlreadyConnected;
//@    private Vector cplQueryParamRecords;
//@    private Vector cplRecords;
//@    private boolean queryResultReceived;
    //#endif
    
    /* transaction log entry */
    private TransactionLogEntry logEntry;
    public static final String INTERNAL_COMPLETION_CAUSE_CONN_FAILED    = "CONN_FAILED";
    public static final String INTERNAL_COMPLETION_CAUSE_QUERY_ERROR    = "QUERY_ERROR";
    public static final String INTERNAL_COMPLETION_CAUSE_QUERY_RETRY    = "QUERY_RETRY";
    public static final String INTERNAL_COMPLETION_CAUSE_FINAL_RESULT   = "FINAL_RESULT";
    public static final String INTERNAL_COMPLETION_CAUSE_REMOTE_DISC    = "REMOTE_DISC";
    public static final String INTERNAL_COMPLETION_CAUSE_TIMEOUT_CMD    = "TIMEOUT_CMD";
    public static final String INTERNAL_COMPLETION_CAUSE_TIMEOUT_IDLE   = "TIMEOUT_IDLE";
    public static final String INTERNAL_COMPLETION_CAUSE_INTERNAL_ERROR = "INTERNAL_ERROR";
    public static final String INTERNAL_COMPLETION_CAUSE_UNKNOWN_ERROR  = "UNKNOWN_ERROR";
    
    private static final byte CMD_TRANS_INIT                  = 0x01;
    private static final byte CMD_TRANS_SEND_PARAM            = 0x02;
    private static final byte CMD_TRANS_SEND_ENROLLMENT_AUDIO = 0x03;
    private static final byte CMD_TRANS_END                   = 0x04;    
    private static final byte CMD_TRANS_SEND_COMMAND_EVENT    = 0x05;
    private static final byte CMD_TRANS_SET_LOG               = 0x06;

    /* synchronization object for pdx transaction states */
    private Object syncObjState;

    protected PDXTransactionImpl(MessageSystem msgSys, PDXCommandListener commandListener, String command,
                                 String application, NMSPSession session, String uid,
                                 String pdxVersion, String sdkVersion, String clientVersion, String scriptVersion,
                                 String language, String region, NMSPDefines.Codec deviceCodec,
                                 String dictationLanguage, short lcdWidth, short lcdHeight,
                                 String carrier, String phoneModel, String phoneNumber,
                                 String originalSessionId/*, byte[] sessionId*/, long commandTimeout, Dictionary optionalKeys,
                                 NMASResourceImpl nmasResource, NMASResourceListener nmasListener, byte tranId) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("PDXTransactionImpl()");
        //#endif
        
        syncObjState = new Object();
        this.msgSys = msgSys;
        this.session = session;
        this.pdxCommandListener = commandListener;
        this.commandTimeout = commandTimeout;
        this.nmasResource = nmasResource;
        this.nmasListener = nmasListener;
        this.tranId = tranId;
        this.commandName = command;
        this.resultReceived = false; 

        if (((ManagerImpl) nmasResource.getManager()).getResourceLogs() != null) {
            logEntry = new TransactionLogEntry(this.tranId, this);
            logEntrySetCompCause(logEntry, INTERNAL_COMPLETION_CAUSE_INTERNAL_ERROR);
        }
        
        synchronized (syncObjState) {
            state = STATE_BEGUN;
        }
        
        //#ifdef NMSP_CPL_ENABLED
//@        this.cplCommandName = command;
//@        this.cplAudioParamsRecords = new Vector();
//@        this.cplAlreadyConnected = false;
//@        this.cplQueryParamRecords = new Vector();
//@        this.session.clearCPL();
//@        this.cplRecords = new Vector();
//@        this.queryResultReceived = false;
        //#endif
        
        Object[] params = new Object[19];
        int index = 0;
        params[index++] = uid;                  //0
        params[index++] = pdxVersion;           //1
        params[index++] = sdkVersion;			//2
        params[index++] = clientVersion;        //3
        params[index++] = scriptVersion;        //4
        params[index++] = language;             //5
        params[index++] = region;               //6
        params[index++] = deviceCodec;          //7
        params[index++] = dictationLanguage;    //8
        params[index++] = new Short(lcdWidth);  //9
        params[index++] = new Short(lcdHeight); //10
        params[index++] = carrier;              //11
        params[index++] = phoneModel;           //12
        params[index++] = phoneNumber;          //13
        params[index++] = originalSessionId;    //14
        //params[index++] = sessionId;            //15
        params[index++] = application;          //16
        params[index++] = command;              //17
        params[index++] = optionalKeys;         //18
        
        msgSys.send(new MessageData(CMD_TRANS_INIT, params), this, Thread.currentThread(), msgSys.getVRAddr()[0]);
    }

    /** *************************************************************** */
    /* Command interface methods */
    /** *************************************************************** */
    public void sendParam(Parameter param) throws TransactionAlreadyFinishedException, TransactionExpiredException {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("PDXTransactionImpl.sendParam()");
        //#endif
    	if (param == null){
    		throw new NullPointerException("Parameter cannot be null");
    	}
        synchronized (syncObjState) {
            if (state == STATE_IDLE) {
                //#ifdef NMSP_LOG
                log.error("PDXTransactionImpl.sendParam() " + MESG_TRANSACTION_ALREADY_FINISHED);
                //#endif
                throw new TransactionAlreadyFinishedException(MESG_TRANSACTION_ALREADY_FINISHED);
            } else if (state == STATE_BEGUN) {
                msgSys.send(new MessageData(CMD_TRANS_SEND_PARAM, param),
                        this, Thread.currentThread(), msgSys.getVRAddr()[0]);
            } else if (state == STATE_ENDED) {
                //#ifdef NMSP_LOG
                log.error("PDXTransactionImpl.sendParam() " + MESG_TRANSACTION_ALREADY_FINISHED);
                //#endif
                throw new TransactionAlreadyFinishedException(MESG_TRANSACTION_ALREADY_FINISHED);
            } else if (state == STATE_EXPIRED) {
                //#ifdef NMSP_LOG
                log.error("PDXTransactionImpl.sendParam() " + MESG_TRANSACTION_EXPIRED);
                //#endif
                throw new TransactionExpiredException(MESG_TRANSACTION_EXPIRED);
            }
        }
    }

    public void sendEnrollmentAudio(byte[] audio) throws TransactionAlreadyFinishedException, TransactionExpiredException {
        //#ifdef NMSP_LOG
        log.debug("PDXTransactionImpl.sendEnrollmentAudio()");
        //#endif
        synchronized (syncObjState) {
            if (state == STATE_IDLE) {
                //#ifdef NMSP_LOG
                log.error("PDXTransactionImpl.sendEnrollmentAudio() " + MESG_TRANSACTION_ALREADY_FINISHED);
                //#endif
                throw new TransactionAlreadyFinishedException(MESG_TRANSACTION_ALREADY_FINISHED);
            } else if (state == STATE_BEGUN) {
                if (audio == null) {
                    throw new NullPointerException("enrollment audio is null");
                } else if(audio.length == 0) {
                    throw new IllegalArgumentException("enrollment audio is empty");
                } else {
                    byte[] audio_copy = new byte[audio.length];
                    System.arraycopy(audio, 0, audio_copy, 0, audio_copy.length);
                    msgSys.send(new MessageData(CMD_TRANS_SEND_ENROLLMENT_AUDIO, audio_copy),
                            this, Thread.currentThread(), msgSys.getVRAddr()[0]);
                }
            } else if (state == STATE_ENDED) {
                //#ifdef NMSP_LOG
                log.error("PDXTransactionImpl.sendEnrollmentAudio() " + MESG_TRANSACTION_ALREADY_FINISHED);
                //#endif
                throw new TransactionAlreadyFinishedException(MESG_TRANSACTION_ALREADY_FINISHED);
            } else if (state == STATE_EXPIRED) {
                //#ifdef NMSP_LOG
                log.error("PDXTransactionImpl.sendEnrollmentAudio() " + MESG_TRANSACTION_EXPIRED);
                //#endif
                throw new TransactionExpiredException(MESG_TRANSACTION_EXPIRED);
            }
        }
    }

    public void end() throws TransactionAlreadyFinishedException, TransactionExpiredException {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("PDXTransactionImpl.end()");
        //#endif
        synchronized (syncObjState) {
            if (state == STATE_IDLE) {
                //#ifdef NMSP_LOG
                log.error("PDXTransactionImpl.end() " + MESG_TRANSACTION_ALREADY_FINISHED);
                //#endif
                throw new TransactionAlreadyFinishedException(MESG_TRANSACTION_ALREADY_FINISHED);
            } else if (state == STATE_BEGUN) {
                state = STATE_ENDED;
                msgSys.send(new MessageData(CMD_TRANS_END, null),
                        this, Thread.currentThread(), msgSys.getVRAddr()[0]);
            } else if (state == STATE_ENDED) {
                //#ifdef NMSP_LOG
                log.error("PDXTransactionImpl.end() " + MESG_TRANSACTION_ALREADY_FINISHED);
                //#endif
                throw new TransactionAlreadyFinishedException(MESG_TRANSACTION_ALREADY_FINISHED);
            } else if (state == STATE_EXPIRED) {
                //#ifdef NMSP_LOG
                log.error("PDXTransactionImpl.end() " + MESG_TRANSACTION_EXPIRED);
                //#endif
                throw new TransactionExpiredException(MESG_TRANSACTION_EXPIRED);
            }
        }
    }
    
    public void setLog(Command.CompletionCause compCause) {
        msgSys.send(new MessageData(CMD_TRANS_SET_LOG, compCause),
                this, Thread.currentThread(), msgSys.getVRAddr()[0]);
    }

    /**
     * SessionListener interface methods
     */
    public void onQueryResults(PDXQueryResult pdxQueryResult, byte transactionId) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("PDXTransactionImpl.onQueryResults()");
        //#endif
        if(transactionId != tranId) { //The result is for another transaction
            return;
        }
        synchronized (syncObjState) {
            if(state == STATE_IDLE || state == STATE_EXPIRED) // Timed out waiting for results or canceled
                return;
//            state = STATE_IDLE; //we can receive more than one QueryResult
            //#ifdef NMSP_CPL_ENABLED
//@            queryResultReceived = true;
//@            addRecord(cplRecords, new CPLRecord(System.currentTimeMillis(), "recv_query_result"));
//@            addRecord(cplRecords, new CPLRecord(System.currentTimeMillis(), "CLIENT_RESPONSE_RECEIVED_" + this.cplCommandName));
            //#endif

            this.resultReceived = true; 
            createResponseEvent("QUERY_RESULT", pdxQueryResult.isFinalResponse());
            
            if (pdxQueryResult.isFinalResponse()) {
            	if (commandTask != null)
            		msgSys.cancelTask(commandTask);
            } else {
            	resetCommandTimeoutTask();
            }
            
            if (pdxCommandListener != null) {
                try {
                    if (pdxQueryResult.isFinalResponse()) {
                        logEntrySetCompCause(logEntry, INTERNAL_COMPLETION_CAUSE_FINAL_RESULT);
                    }
                    pdxCommandListener.PDXQueryResultReturned(pdxQueryResult);
                } catch (Throwable t) {
                    //#ifdef NMSP_LOG
                    log.error("got exp in PDXCommandListener.PDXQueryResultReturned() ["
                              + t.getClass().getName() + "] msg [" + t.getMessage() + "]");
                    //#endif
                }
            }
//#ifdef NMSP_CPL_ENABLED
//@         sendStats();
//#endif
//            session = null; //we can receive QueryResults before sending QueryEnd
            //#ifdef NMSP_PLATFORM_BLACKBERRY
//@            int expectMore = 0;
//@            try {
//@                if (pdxQueryResult.containsKey("expect_more_result"))
//@                    expectMore = pdxQueryResult.getInteger("expect_more_result");
//@            } catch (Exception e) {}
//@            
//@            if(expectMore == 0) {
//@                session.disconnect();
//@            }
            //#endif
        }
    }

    public void onQueryError(PDXQueryError pdxQueryError, byte transactionId) {
        //#ifdef NMSP_LOG
        log.debug("PDXTransactionImpl.onQueryError()");
        //#endif
        if(transactionId != tranId) { //The result if for another transaction
            return;
        }
        synchronized (syncObjState) {
            if(state == STATE_IDLE || state == STATE_EXPIRED) // Timed out waiting for results or canceled
                return;
            state = STATE_IDLE;
            //#ifdef NMSP_CPL_ENABLED
//@            addRecord(cplRecords, new CPLRecord(System.currentTimeMillis(), "recv_query_error"));
//@            addRecord(cplRecords, new CPLRecord(System.currentTimeMillis(), "CLIENT_RESPONSE_RECEIVED_" + this.cplCommandName));
            //#endif

            createResponseEvent("QUERY_ERROR", true);
            
            if(commandTask != null)
                msgSys.cancelTask(commandTask);
            if (pdxCommandListener != null) {
                try {
                    logEntrySetCompCause(logEntry, INTERNAL_COMPLETION_CAUSE_QUERY_ERROR);
                    pdxCommandListener.PDXQueryErrorReturned(pdxQueryError);
                } catch (Throwable t) {
                    //#ifdef NMSP_LOG
                    log.error("got exp in PDXCommandListener.PDXQueryErrorReturned() ["
                              + t.getClass().getName() + "] msg [" + t.getMessage() + "]");
                    //#endif
                }
            }
            //#ifdef NMSP_CPL_ENABLED
//@            sendStats();
            //#endif
            //#ifdef NMSP_PLATFORM_BLACKBERRY
//@            session.disconnect();
            //#endif
        }
    }

    public void onQueryRetry(PDXQueryRetry pdxQueryRetry, byte transactionId) {
        //#ifdef NMSP_LOG
        log.debug("PDXTransactionImpl.onQueryRetry()");
        //#endif
        if(transactionId != tranId) { //The result if for another transaction
            return;
        }
        synchronized (syncObjState) {
            if(state == STATE_IDLE || state == STATE_EXPIRED) // Timed out waiting for results or canceled
                return;
            state = STATE_IDLE;
            //#ifdef NMSP_CPL_ENABLED
//@            addRecord(cplRecords, new CPLRecord(System.currentTimeMillis(), "recv_query_retry"));
//@            addRecord(cplRecords, new CPLRecord(System.currentTimeMillis(), "CLIENT_RESPONSE_RECEIVED_" + this.cplCommandName));
            //#endif
            
            createResponseEvent("QUERY_RETRY", true);
            
            if(commandTask != null)
                msgSys.cancelTask(commandTask);
            if (pdxCommandListener != null) {
                try {
                    logEntrySetCompCause(logEntry, INTERNAL_COMPLETION_CAUSE_QUERY_RETRY);
                    pdxCommandListener.PDXQueryRetryReturned(pdxQueryRetry);
                } catch (Throwable t) {
                    //#ifdef NMSP_LOG
                    log.error("got exp in PDXCommandListener.PDXQueryRetryReturned() ["
                              + t.getClass().getName() + "] msg [" + t.getMessage() + "]");
                    //#endif
                }
            }
            //#ifdef NMSP_CPL_ENABLED
//@            sendStats();
            //#endif
            //#ifdef NMSP_PLATFORM_BLACKBERRY
//@            session.disconnect();
            //#endif
        }
    }
    
    public void onVapPlayBeginReceived()
    {
        resetCommandTimeoutTask();
    }
    
    public void onVapPlayReceived()
    {
        resetCommandTimeoutTask();
    }
    
    public void onVapPlayEndReceived()
    {
        resetCommandTimeoutTask();
    }

    public void onSessionDisconnected(short reasonCode) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("PDXTransactionImpl.onSessionDisconnected() " + reasonCode);
        //#endif
        synchronized (syncObjState) {
            if (state == STATE_ENDED) {
                if(commandTask != null)
                    msgSys.cancelTask(commandTask);
            }
            if (state == STATE_EXPIRED) {
                return;
            }
            
            try {
                if (reasonCode == XMode.XMODE_SOCKET_CLOSED_REMOTE_DISCONNECT) {
                    if (state != STATE_IDLE) {
                        logEntrySetCompCause(logEntry, INTERNAL_COMPLETION_CAUSE_REMOTE_DISC);
                        pdxCommandListener.PDXCommandEvent(PDXCommandListener.REMOTE_DISCONNECTION);
                    }
                }
                else if (reasonCode == XMode.XMODE_SOCKET_CLOSED_DISCONNECT) {
                    if (state != STATE_IDLE) {
                        pdxCommandListener.PDXCommandEvent(PDXCommandListener.COMMAND_ENDED);
                    }
                }
                else if (reasonCode == XMode.XMODE_SESSION_IDLE_FOR_TOO_LONG) {
                    if (state != STATE_IDLE) {
                        logEntrySetCompCause(logEntry, INTERNAL_COMPLETION_CAUSE_TIMEOUT_IDLE);
                        pdxCommandListener.PDXCommandEvent(PDXCommandListener.COMMAND_IDLE_FOR_TOO_LONG);
                    }
                }
                else if (reasonCode == XMode.XMODE_SOCKET_CLOSED_OPEN_SOCKET_FAILED) {
                    if (state != STATE_IDLE) {
                        logEntrySetCompCause(logEntry, INTERNAL_COMPLETION_CAUSE_CONN_FAILED);
                        nmasListener.PDXCreateCommandFailed(ManagerListener.NMSP_REMOTE_DISCONNECT_UNKNOWN);
                    }
                }
                else if (reasonCode == XMode.XMODE_SOCKET_CLOSED_OPEN_SOCKET_FAILED_SECURITY) {
                    if (state != STATE_IDLE) {
                        logEntrySetCompCause(logEntry, INTERNAL_COMPLETION_CAUSE_CONN_FAILED);
                        nmasListener.PDXCreateCommandFailed(ManagerListener.NMSP_REMOTE_DISCONNECT_SECURITY);
                    }
                }
                else if (reasonCode == XMode.XMODE_SOCKET_CLOSED_COP_CONNECT_TIMEOUT) {
                    if (state != STATE_IDLE) {
                        logEntrySetCompCause(logEntry, INTERNAL_COMPLETION_CAUSE_CONN_FAILED);
                        nmasListener.PDXCreateCommandFailed(ManagerListener.NMSP_REMOTE_DISCONNECT_TIMEOUT);
                    }
                }
                else if (reasonCode == XMode.XMODE_SOCKET_CLOSED_COP_DISCONNECT) {
                    if (state != STATE_IDLE) {
                        logEntrySetCompCause(logEntry, INTERNAL_COMPLETION_CAUSE_REMOTE_DISC);
                        pdxCommandListener.PDXCommandEvent(PDXCommandListener.REMOTE_DISCONNECTION);
                    }
                }
                else if (reasonCode == XMode.XMODE_SOCKET_CLOSED_COP_CONNECT_FAILED) {
                    if (state != STATE_IDLE) {
                        logEntrySetCompCause(logEntry, INTERNAL_COMPLETION_CAUSE_CONN_FAILED);
                        nmasListener.PDXCreateCommandFailed(ManagerListener.NMSP_REMOTE_DISCONNECT_UNKNOWN);
                    }
                }
                else if (reasonCode == XMode.XMODE_SOCKET_CLOSED_NETWORK_ERROR) {
                    if (state != STATE_IDLE) {
                        logEntrySetCompCause(logEntry, INTERNAL_COMPLETION_CAUSE_REMOTE_DISC);
                        pdxCommandListener.PDXCommandEvent(PDXCommandListener.REMOTE_DISCONNECTION);
                    }
                }
            } catch (Throwable t) {
                //#ifdef NMSP_LOG
                log.error("got exp in PDXCommandListener.PDXCommandEvent() or" +
                          " PDXManagerListener.PDXManagerError() or" +
                          " PDXManagerListener.PDXManagerDisconnected() reasonCode [" + reasonCode + "] ["
                          + t.getClass().getName() + "] msg [" + t.getMessage() + "]");
                //#endif
            }
            
            //#ifdef NMSP_CPL_ENABLED
//@            addRecord(cplRecords, new CPLRecord(System.currentTimeMillis(),
//@                    "session_disconnected_[reason:" + reasonCode + "]"));
            //#endif
            
            state = STATE_EXPIRED;
            //#ifdef NMSP_PLATFORM_BLACKBERRY
//@            session.disconnect();
            //#endif
        }
    }

    public void onSessionConnected(byte[] sessionId) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("PDXTransactionImpl.onSessionConnected()");
        //#endif
        synchronized (syncObjState) {
            if (state == STATE_BEGUN || state == STATE_ENDED) { //end() can be called before cop connected is received because of queuing
                try {
                    //#ifdef NMSP_CPL_ENABLED
//@                    addRecord(cplRecords, new CPLRecord(System.currentTimeMillis(), "sessionId[" + sessionIdToString(sessionId) + "]"));
                    //#endif
                    String sessionIdStr = sessionIdToString(sessionId);
                    logEntrySetSessionId(logEntry, sessionIdStr);
                    if(!commandCreated) {
                        commandCreated = true;
                        //#ifdef NMSP_LOG
                        log.debug("PDXCommandCreated() called from onSessionConnected()" + sessionIdStr + ":" + tranId + " (" + this + "," + nmasListener + ")");
                        //#endif
                        nmasListener.PDXCommandCreated(sessionIdStr + ":" + tranId);
                    }
                } catch (Throwable t) {
                    //#ifdef NMSP_LOG
                    log.error("got exp in PDXCommandListener.PDXCommandCreated() ["
                              + t.getClass().getName() + "] msg [" + t.getMessage() + "]");
                    //#endif
                }
            }
        }
    }

    public void transactionOver() {
        synchronized (syncObjState) {
            if (state == STATE_ENDED) {
                if(commandTask != null)
                    msgSys.cancelTask(commandTask);
            }
            if(state == STATE_EXPIRED) {
                return;
            }
            if(state != STATE_IDLE) {
            	
            	if (!resultReceived) {
            		createChildEvent(SessionEvent.NMSP_CALLLOG_COMMANDABORT_EVENT);
            	}
                
//                pdxCommandListener.PDXCommandEvent(PDXCommandListener.COMMAND_ENDED);
                msgSys.send(new MessageData(CMD_TRANS_SEND_COMMAND_EVENT, null), this, Thread.currentThread(), msgSys.getVRAddr()[0]);
            }
        //#ifdef NMSP_CPL_ENABLED
//@        if (state == STATE_BEGUN || state == STATE_ENDED) {
//@            if(!queryResultReceived) {
//@                addRecord(cplRecords, new CPLRecord(System.currentTimeMillis(), "CLIENT_REQUEST_ABORTED_" + cplCommandName));
//@                sendStats();
//@            }
//@        }
        //#endif
            state = STATE_EXPIRED;
        }
    }
    
//    public short getState() {
//        //#ifdef NMSP_LOG
////@        log.debug("PDXTransactionImpl.getState()");
//        //#endif
//        synchronized (syncObjState) {
//            return state;
//        }
//    }
    
    public void handleMessage(Object msgData, Object msgSender) {
        MessageData tranMsgData = (MessageData) msgData;
        Object data = tranMsgData.data;
        //#ifndef NMSP_LOG
//@        //log.debug("\n handleMessage - [" + getMessageCmd(xmodeMsgData.command) + "]");
        //#endif
        switch (tranMsgData.command) {
            case CMD_TRANS_INIT:
                handleInit(data);
                break;
            case CMD_TRANS_SEND_PARAM:
                handleSendParam(data);
                break;
            case CMD_TRANS_SEND_ENROLLMENT_AUDIO:
                handleSendEnrollmentAudio(data);
                break;
            case CMD_TRANS_END:
                handleEnd(data);
                break;
            case CMD_TRANS_SEND_COMMAND_EVENT:
                handleSendCommandEvent();
                break;
            case CMD_TRANS_SET_LOG:
                handleSetLog(data);
                break;
        }
    }
    
    private void handleInit(Object data) {
        Object[] params = (Object[]) data;
        //int index = 0;
        String msgData_uid                    = (String) params[0];
        String msgData_pdxVersion             = (String) params[1];
        String msgData_sdkVersion			  = (String) params[2];
        String msgData_clientVersion          = (String) params[3];
        String msgData_scriptVersion          = (String) params[4];
        String msgData_language               = (String) params[5];
        String msgData_region                 = (String) params[6];
        NMSPDefines.Codec msgData_deviceCodec = (NMSPDefines.Codec) params[7];
        String msgData_dictationLanguage      = (String) params[8];
        short msgData_lcdWidth                = ((Short) params[9]).shortValue();
        short msgData_lcdHeight               = ((Short) params[10]).shortValue();
        String msgData_carrier                = (String) params[11];
        String msgData_phoneModel             = (String) params[12];
        String msgData_phoneNumber            = (String) params[13];
        String msgData_originalSessionId      = (String) params[14];
        byte[] msgData_sessionId              = /*(byte[]) params[14]*/this.session.getSessionId();
        String msgData_application            = (String) params[/*15*/15];
        String msgData_command                = (String) params[/*16*/16];
        Dictionary msgData_dict 			  = (Dictionary)params[/*17*/17];

        /* Checking BT to adjust codecs */
        if ((new BluetoothSystemOEM(nmasResource.parameters)).isBluetoothHeadsetConnected()) {
            msgData_deviceCodec = Util.adjustCodecForBluetooth(msgData_deviceCodec);
        }
        
        // NMAS is not returning the right request id so we always 
        // use the same request id for all NMAS requests
        long requestId = ResourceImpl.getDefaultRequestId();

    	createSessionEvent(session.getSessionEvent());
        
        PDXQueryBegin pdxQueryBegin = new PDXQueryBegin(msgData_uid, msgData_pdxVersion, msgData_sdkVersion,
                msgData_clientVersion, msgData_scriptVersion, msgData_language, msgData_region, msgData_deviceCodec,
                msgData_dictationLanguage, msgData_lcdWidth, msgData_lcdHeight, msgData_carrier, msgData_phoneModel,
                msgData_phoneNumber, msgData_originalSessionId, msgData_sessionId, msgData_application, msgData_command, msgData_dict);
        //#if NMSP_CPL_ENABLED
//@        addRecord(cplRecords, new CPLRecord(System.currentTimeMillis(), "send_Query_Begin"));
//@        addRecord(cplRecords, new CPLRecord(System.currentTimeMillis(), "transaction_id[" + this.tranId + "]"));
        //#endif
        appendLogToQueryBegin(pdxQueryBegin);
        session.postBcpMessage(ProtocolDefines.XMODE_BCP_COMMAND_BCP_BEGIN, XMode.NET_CONTEXT_SEND_BCP_BEGIN + tranId, pdxQueryBegin.getMessage(), null, tranId, requestId, nmasResource, false);

        if (msgData_sessionId != null) {
            //#ifdef NMSP_CPL_ENABLED
//@            this.cplAlreadyConnected = true;
//@            addRecord(cplRecords, new CPLRecord(System.currentTimeMillis(), "sessionId[" + sessionIdToString(msgData_sessionId) + "]"));
            //#endif

        	
            try {
                if(!commandCreated) {
                    commandCreated = true;
                    //#ifdef NMSP_LOG
                    log.debug("PDXCommandCreated() called from handleInit()" + sessionIdToString(msgData_sessionId) + ":" + tranId + " (" + this + "," + nmasListener + ")");
                    //#endif
                    nmasListener.PDXCommandCreated(sessionIdToString(msgData_sessionId) + ":" + tranId);
                }
            } catch (Throwable t) {
                //#ifdef NMSP_LOG
                log.error("got exp in PDXCommandListener.PDXCommandCreated() ["
                          + t.getClass().getName() + "] msg [" + t.getMessage() + "]");
                //#endif
            }
        }
    }
    
    private void handleSendParam(Object data) {
        Parameter msgData_param = (Parameter) data;
        if (((PDXParam) msgData_param).getType() == PDXParam.TTS_PARAMETER) {
//            int audioId = session.getDefaultAudioId();
            session.addAudioSink(((PDXTTSParam) msgData_param).getAudioId(), ((PDXTTSParam) msgData_param).getAudioSink(), nmasResource);
        }
        //#ifdef NMSP_CPL_ENABLED
//@                else if(((PDXParam) msgData_param).getType() == PDXParam.AUDIO_PARAMETER) {
//@                    cplAudioParamsRecords.addElement(((PDXAudioParam) msgData_param));
//@                }
        //#endif
        // NMAS is not returning the right request id so we always 
        // use the same request id for all NMAS requests
        long requestId = ResourceImpl.getDefaultRequestId();
        
        if (msgData_param instanceof PDXAudioParam) {
            createAudioParamEvent(((PDXAudioParam)msgData_param).getName(), ((PDXAudioParam)msgData_param).getBufferId());
        }
        
        PDXQueryParameter pdxQueryParam = new PDXQueryParameter((PDXParam) msgData_param);
        session.postBcpMessage(ProtocolDefines.XMODE_BCP_COMMAND_BCP_DATA, XMode.NET_CONTEXT_SEND_BCP_DATA, pdxQueryParam.toByteArray(), null, tranId, requestId, nmasResource, false);

        //#ifdef NMSP_CPL_ENABLED
//@                String cplParamType = "";
//@                if (msgData_param instanceof PDXAudioParam)
//@                    cplParamType = "audio";
//@                else if (msgData_param instanceof PDXChoiceParam)
//@                    cplParamType = "choice";
//@                else if (msgData_param instanceof PDXDataParam)
//@                    cplParamType = "data";
//@                else if (msgData_param instanceof PDXDictParam)
//@                    cplParamType = "dict";
//@                else if (msgData_param instanceof PDXSeqParam)
//@                    cplParamType = "seq";
//@                else if (msgData_param instanceof PDXTextParam)
//@                    cplParamType = "text";
//@                else if (msgData_param instanceof PDXTTSParam)
//@                    cplParamType = "tts";
//@                else
//@                    cplParamType = "unknown";
//@                cplQueryParamRecords.addElement(new CPLRecord(System.currentTimeMillis(), cplParamType));
        //#endif
    }
    
    private void handleSendEnrollmentAudio(Object data) {
        byte[] msgData_audio = (byte[]) data;
        // NMAS is not returning the right request id so we always 
        // use the same request id for all NMAS requests
        long requestId = ResourceImpl.getDefaultRequestId();
        
        PDXEnrollmentAudio pdxEnrollmentAudio = new PDXEnrollmentAudio(msgData_audio);
        session.postBcpMessage(ProtocolDefines.XMODE_BCP_COMMAND_BCP_DATA, XMode.NET_CONTEXT_SEND_BCP_DATA, pdxEnrollmentAudio.toByteArray(), null, tranId, requestId, nmasResource, false);

    }
    
    private void handleEnd(Object data) {
        // NMAS is not returning the right request id so we always 
        // use the same request id for all NMAS requests
        long requestId = ResourceImpl.getDefaultRequestId();
        
        PDXQueryEnd pdxQueryEnd = new PDXQueryEnd();
        session.postBcpMessage(ProtocolDefines.XMODE_BCP_COMMAND_BCP_DATA, XMode.NET_CONTEXT_SEND_BCP_DATA, pdxQueryEnd.toByteArray(), null, tranId, requestId, nmasResource, false);

        //#ifdef NMSP_CPL_ENABLED
//@        addRecord(cplRecords, new CPLRecord(System.currentTimeMillis(), "send_Query_End"));
        //#endif
        

        commandTask = new TimerSystemTask() {
            public void run() {
                try {
                    synchronized (syncObjState) {
                        state = STATE_IDLE;
                    }
                    //#ifdef NMSP_CPL_ENABLED
//@                    addRecord(cplRecords, new CPLRecord(System.currentTimeMillis(), "Command_timeout_[" + cplCommandName + "]"));
//@                    addRecord(cplRecords, new CPLRecord(System.currentTimeMillis(), "CLIENT_RESPONSE_TIMEOUT_" + cplCommandName));
                    //#endif

                    createChildEvent(SessionEvent.NMSP_CALLLOG_COMMANDEXPIRED_EVENT);

                    if (pdxCommandListener != null) {
                        //#ifdef NMSP_LOG
                        log.debug("PDXTransactionImpl.end() timed out waiting for results. ");
                        //#endif
                        try {
                            logEntrySetCompCause(logEntry, INTERNAL_COMPLETION_CAUSE_TIMEOUT_CMD);
                            pdxCommandListener.PDXCommandEvent(PDXCommandListener.TIMED_OUT_WAITING_FOR_RESULT);
                        } catch (Throwable t) {
                            //#ifdef NMSP_LOG
                            log.error("got exp in PDXCommandListener.PDXCommandEvent(TIMED_OUT_WAITING_FOR_RESULT) ["
                                      + t.getClass().getName() + "] msg [" + t.getMessage() + "]");
                            //#endif
                        }
                        //#ifdef NMSP_PLATFORM_BLACKBERRY
//@                        session.disconnect();
                        //#endif
                    }
                    //#ifdef NMSP_CPL_ENABLED
//@                    sendStats();
                    //#endif
                }
                catch (Exception e) {
                    //#ifdef NMSP_LOG
                    log.error("PDXTransactionImpl.run() " + e.getClass().getName() + " " + e.getMessage());
                    //#endif
                }
            }
        };
        msgSys.scheduleTask(commandTask, commandTimeout);
    }
    
    // the command timeout task will be reset upon sending a VAP_Record
    protected void resetCommandTimeoutTask() {
        if (commandTask != null) {
            if (msgSys.cancelTask(commandTask))
                commandTask = new TimerSystemTask() {
            	public void run() {
                    try {
                        synchronized (syncObjState) {
                            state = STATE_IDLE;
                        }
                        //#ifdef NMSP_CPL_ENABLED
    //@                    addRecord(cplRecords, new CPLRecord(System.currentTimeMillis(), "Command_timeout_[" + cplCommandName + "]"));
    //@                    addRecord(cplRecords, new CPLRecord(System.currentTimeMillis(), "CLIENT_RESPONSE_TIMEOUT_" + cplCommandName));
                        //#endif

                        createChildEvent(SessionEvent.NMSP_CALLLOG_COMMANDEXPIRED_EVENT);

                        if (pdxCommandListener != null) {
                            //#ifdef NMSP_LOG
                            log.debug("PDXTransactionImpl.end() timed out waiting for results. ");
                            //#endif
                            try {
                                logEntrySetCompCause(logEntry, INTERNAL_COMPLETION_CAUSE_TIMEOUT_CMD);
                                pdxCommandListener.PDXCommandEvent(PDXCommandListener.TIMED_OUT_WAITING_FOR_RESULT);
                            } catch (Throwable t) {
                                //#ifdef NMSP_LOG
                                log.error("got exp in PDXCommandListener.PDXCommandEvent(TIMED_OUT_WAITING_FOR_RESULT) ["
                                          + t.getClass().getName() + "] msg [" + t.getMessage() + "]");
                                //#endif
                            }
                            //#ifdef NMSP_PLATFORM_BLACKBERRY
    //@                        session.disconnect();
                            //#endif
                        }
                        //#ifdef NMSP_CPL_ENABLED
    //@                    sendStats();
                        //#endif
                    }
                    catch (Exception e) {
                        //#ifdef NMSP_LOG
                        log.error("PDXTransactionImpl.run() " + e.getClass().getName() + " " + e.getMessage());
                        //#endif
                    }
                }
            };
            msgSys.scheduleTask(commandTask, commandTimeout);
        }
    }

    private void handleSendCommandEvent()
    {
        pdxCommandListener.PDXCommandEvent(PDXCommandListener.COMMAND_ENDED);
    }
    
    private void handleSetLog(Object data) {
        CompletionCause compCause = (CompletionCause) data;
        logEntrySetCompCause(logEntry, compCause);
    }    
    
    //#ifdef NMSP_CPL_ENABLED
//@    private void sendStats() {
//@        try {
//@            if(!cplAlreadyConnected) {
//@                addRecord(cplRecords, new CPLRecord(session.getOpenSocketTime(), "request_open_socket"));
//@                addRecord(cplRecords, new CPLRecord(session.getSocketOpenedTime(), "socket_opened"));
//@                addRecord(cplRecords, new CPLRecord(session.getConnectionEstablishedTime(), "session_established"));
//@                addRecord(cplRecords, new CPLRecord(session.getConnectionEstablishedTime(), "CLIENT_CONNECTION_ESTABLISHED"));
//@            } else {
//@                addRecord(cplRecords, new CPLRecord(session.getOpenSocketTime(), "request_open_socket"));
//@                addRecord(cplRecords, new CPLRecord(session.getSocketOpenedTime(), "prev_socket_opened"));
//@                addRecord(cplRecords, new CPLRecord(session.getConnectionEstablishedTime(), "prev_session_established"));
//@            }
//@            
//@            Enumeration audioParam = cplAudioParamsRecords.elements();
//@            while(audioParam.hasMoreElements()) {
//@                PDXAudioParam param = (PDXAudioParam)audioParam.nextElement();
//@                if(param.recordStartTime != 0) {
//@                    addRecord(cplRecords, new CPLRecord(param.recordStartTime, "CLIENT_PTT_START_" + param.getName()));
//@                }
//@                if(param.streamStartTime != 0) {
//@                    addRecord(cplRecords, new CPLRecord(param.streamStartTime, "CLIENT_STREAM_START_" + param.getName()));
//@                }    
//@                if(param.recordStopTime != 0) {
//@                    addRecord(cplRecords, new CPLRecord(param.recordStopTime, "CLIENT_PTT_END_" + param.getName()));
//@                }
//@                if(param.streamEndTime != 0) {
//@                    addRecord(cplRecords, new CPLRecord(param.streamEndTime, "CLIENT_STREAM_END_" + param.getName()));
//@                }
//@            }
//@            Enumeration cplQueryParamTimesEnu = cplQueryParamRecords.elements();
//@            while (cplQueryParamTimesEnu.hasMoreElements()) {
//@                CPLRecord record = (CPLRecord) cplQueryParamTimesEnu.nextElement();
//@                addRecord(cplRecords, new CPLRecord(record.time, "send_Query_Param_[" + record.type + "]"));
//@            }
//@            
//@            // socket read records
//@            Enumeration cplSocketReadRecords = session.getSocketReadRecords().elements();
//@            while (cplSocketReadRecords.hasMoreElements()) {
//@                CPLSocketReadContext record = (CPLSocketReadContext) cplSocketReadRecords.nextElement();
//@                
//@                Enumeration cplSocketMultipleReadAvailRecords = record.multipleAvails.elements();
//@                while (cplSocketMultipleReadAvailRecords.hasMoreElements()) {
//@                    CPLSocketReadRecord record_read_avail = (CPLSocketReadRecord) cplSocketMultipleReadAvailRecords.nextElement();
//@                    long time_avail = record_read_avail.cplBeforeCheckSocketReadAvailableTime;
//@                    long duration_avail = record_read_avail.cplAfterCheckSocketReadAvailableTime - record_read_avail.cplBeforeCheckSocketReadAvailableTime;
//@                    addRecord(cplRecords, new CPLRecord(time_avail,
//@                            "socket_read_available_[" + (duration_avail) + "_ms][" + record_read_avail.cplAvailableBytes + "_bytes][" + record_read_avail.signalStrengthLevel + "]"));
//@                }
//@                
//@                Enumeration cplSocketMultipleReadRecords = record.multipleReads.elements();
//@                while (cplSocketMultipleReadRecords.hasMoreElements()) {
//@                    CPLSocketReadRecord record_read = (CPLSocketReadRecord) cplSocketMultipleReadRecords.nextElement();
//@                    long time_read = record_read.cplBeforeSocketReadTime;
//@                    long duration_read = record_read.cplAfterSockeReadTime - record_read.cplBeforeSocketReadTime;
//@                    if (record_read.cplInternalAvaBytes != 0)
//@                        addRecord(cplRecords, new CPLRecord(time_read,
//@                            "socket_read_[" + (duration_read) + "_ms][" + record_read.bytesRead + "_bytes][intern " + record_read.cplInternalAvaBytes + "_bytes][" + record_read.signalStrengthLevel + "]"));
//@                    else
//@                        addRecord(cplRecords, new CPLRecord(time_read,
//@                            "socket_read_[" + (duration_read) + "_ms][" + record_read.bytesRead + "_bytes][" + record_read.signalStrengthLevel + "]"));    
//@                }
//@            }
//@            
//@            // socket write records
//@            Enumeration cplSocketWriteRecords = session.getSocketWriteRecords().elements();
//@            while (cplSocketWriteRecords.hasMoreElements()) {
//@                CPLSocketWriteContext record = (CPLSocketWriteContext) cplSocketWriteRecords.nextElement();
//@                long time_write = record.cplBeforeSocketWriteTime;
//@                long duration_write = record.cplAfterSocketWriteTime - record.cplBeforeSocketWriteTime;
//@                addRecord(cplRecords, new CPLRecord(time_write,
//@                        "socket_write_[" + (duration_write) + "_ms][" + record.bytesWritten + "_bytes][" + record.signalStrengthLevel + "]"));
//@            }
//@            
//@            // xmode read message records
//@            Enumeration cplXmodeReadMsgs = session.getXmodeReadMsgs().elements();
//@            while (cplXmodeReadMsgs.hasMoreElements()) {
//@                CPLRecord record = (CPLRecord) cplXmodeReadMsgs.nextElement();
//@                long time = record.time;
//@                addRecord(cplRecords, new CPLRecord(time, record.type));
//@            }
//@            
//@            // xmode write message records cop + vap
//@            Enumeration cplXmodeWriteMsgs = session.getXmodeWriteMsgs().elements();
//@            while (cplXmodeWriteMsgs.hasMoreElements()) {
//@                CPLRecord record = (CPLRecord) cplXmodeWriteMsgs.nextElement();
//@                long time = record.time;
//@                addRecord(cplRecords, new CPLRecord(time, record.type));
//@            }
//@            
//@            // session write message records bcp
//@            Enumeration cplSessionWriteMsgs = session.getSessionWriteMsgs().elements();
//@            while (cplSessionWriteMsgs.hasMoreElements()) {
//@                CPLRecord record = (CPLRecord) cplSessionWriteMsgs.nextElement();
//@                long time = record.time;
//@                addRecord(cplRecords, new CPLRecord(time, record.type));
//@            }
//@            
//@            // xmode read request
//@            Enumeration cplXmodeReadRequest = session.getXmodeReadRequests().elements();
//@            while (cplXmodeReadRequest.hasMoreElements()) {
//@                CPLRecord record = (CPLRecord) cplXmodeReadRequest.nextElement();
//@                addRecord(cplRecords, record);
//@            }
//@            
//@            buildAndFlushStats(cplRecords);
//@            buildAndFlushStatsFormal(cplRecords);
//@        } catch (Exception e) {
//@            byte binaryLength[] = new byte[4];
//@            ByteArrayOutputStream binaryPayload = new ByteArrayOutputStream();
//@            String payloadStr = "<sp>" + "<rid>0</rid>";
//@            payloadStr += "<gwlog p=\"got exp while sending the cpls\" v=\"1\"/>";
//@            payloadStr += "</sp>";
//@            byte buf[] = ("[" + e.getClass().getName() + "] msg[" + e.getMessage() + "]").getBytes();
//@            ByteConversion.intToBytes(buf.length, binaryLength, 0);
//@            binaryPayload.write(binaryLength, 0, 4);
//@            binaryPayload.write(buf, 0, buf.length);
//@            session.postBcpMessage(ProtocolDefines.XMODE_BCP_COMMAND_SET_PARAMS, XMode.NET_CONTEXT_SEND_BCP_SET_PARAMS, payloadStr.getBytes(), binaryPayload.toByteArray(), tranId, nmasResource.getRequestId(), nmasResource);
//@
//@            try {
//@                binaryPayload.close();
//@            } catch (Exception ee) {}
//@        }
//@    }
//@    
//@    private void addRecord(Vector records, CPLRecord record) {
//@        int size = records.size();
//@        for (int i = 0; i < size; i++) {
//@            CPLRecord record_tmp = (CPLRecord) records.elementAt(i);
//@            if (record_tmp.time < record.time) {
//@                continue;
//@            } else {
//@                records.insertElementAt(record, i);
//@                return;
//@            }
//@        }
//@        records.addElement(record);
//@        return;
//@    }
//@    
//@    private static String getDateTime(long miliTime) {
//@        if (miliTime <= 0) {
//@            return "         ," + miliTime;
//@        } else {
//@            String timeStr = new Date(miliTime).toString();
//@            timeStr = timeStr.substring(0, timeStr.length() - 9);
//@            String miliTimeStr = "" + miliTime;
//@            if (miliTimeStr.length() > 3) {
//@                miliTimeStr = miliTimeStr.substring(miliTimeStr.length() - 3);
//@            }
//@            return timeStr + "," + miliTimeStr;
//@        }
//@    }
//@    
//@    private void buildAndFlushStats(Vector records) {
//@        ByteArrayOutputStream binaryPayload = new ByteArrayOutputStream();
//@        byte binaryLength[] = new byte[4];
//@        byte valueBytes[];
//@        String payloadStr = "<sp>" + "<rid>0</rid>";
//@        payloadStr += "<gwlog p=\"NVC_SDK_CPL\" v=\"1\"/>";
//@        payloadStr += "</sp>";
//@        String value = "";
//@        int size = records.size();
//@        for (int i = 0; i < size; i++) {
//@            CPLRecord record = (CPLRecord) records.elementAt(i);
//@            String recordStr = ("[" + getDateTime(record.time).substring(4) + "]:[" + record.type + "]");
//@            value += "\n" + recordStr;
//@            log.error("NotError_CPL_LOG:" + recordStr);
//@        }
//@        byte[] buf = value.getBytes();
//@        ByteConversion.intToBytes(buf.length, binaryLength, 0);
//@        binaryPayload.write(binaryLength, 0, 4);
//@        binaryPayload.write(buf, 0, buf.length);
//@        
//@        session.postBcpMessage(ProtocolDefines.XMODE_BCP_COMMAND_SET_PARAMS, XMode.NET_CONTEXT_SEND_BCP_SET_PARAMS, payloadStr.getBytes(), binaryPayload.toByteArray(), tranId, nmasResource.getRequestId(), nmasResource);
//@        try {
//@            binaryPayload.close();
//@        } catch (Exception e) {}
//@    }
//@    
//@    private void buildAndFlushStatsFormal(Vector records) {
//@        ByteArrayOutputStream binaryPayload = new ByteArrayOutputStream();
//@        byte binaryLength[] = new byte[4];
//@        byte valueBytes[];
//@        String payloadStr = "<sp>" + "<rid>0</rid>";
//@        String value = "";
//@        int size = records.size();
//@        long openSocketTime = 0;
//@        for (int i = 0; i < size; i++) {
//@            CPLRecord record = (CPLRecord) records.elementAt(i);
//@            if (record.type.startsWith("request_open_socket")) {
//@                openSocketTime = record.time;
//@            }
//@            if (record.type.startsWith("CLIENT_")) {
//@                payloadStr += "<gwlog p=\"" + record.type + "\" v=\"" + (i + 1) + "\"/>";    
//@            }
//@            byte[] buf = Long.toString(record.time - openSocketTime).getBytes();
//@            ByteConversion.intToBytes(buf.length, binaryLength, 0);
//@            binaryPayload.write(binaryLength, 0, 4);
//@            binaryPayload.write(buf, 0, buf.length);
//@        }
//@        payloadStr += "</sp>";
//@        
//@        session.postBcpMessage(ProtocolDefines.XMODE_BCP_COMMAND_SET_PARAMS, XMode.NET_CONTEXT_SEND_BCP_SET_PARAMS, payloadStr.getBytes(), binaryPayload.toByteArray(), tranId, nmasResource.getRequestId(), nmasResource);
//@        try {
//@            binaryPayload.close();
//@        } catch (Exception e) {}
//@    }
    //#endif
    
    /* exception reasons */
    private final static String MESG_TRANSACTION_EXPIRED = "transaction already expired!";
    private final static String MESG_TRANSACTION_ALREADY_FINISHED = "transaction already finished!";
    
    private String sessionIdToString(byte[] sessionId)
    {
        // a2b83195-3f06-4c67-9a49-9757c4664768
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < sessionId.length; i++)
        {
            //#ifdef NMSP_PLATFORM_BLACKBERRY
//@            net.rim.device.api.util.NumberUtilities.appendNumber(sb, sessionId[i], 16, 2);
            //#else
            String strDigit = Integer.toHexString((int) sessionId[i]);
            if (strDigit.length() > 1) {
                sb.append(strDigit.substring(strDigit.length() - 2));
            } else {
            	sb.append("0");
                sb.append(strDigit);
            }
            //#endif
            if (i == 3 || i == 5 || i == 7 || i == 9)
            {
                sb.append('-');
            }
        }
        return sb.toString();
    }
    
    public boolean isNetworkHealthy() {
        return session.isNetworkHealthy();
    }
    
    /**** device log to server functions ****/
    protected static class TransactionLogEntry {
        private int transactionId;
        private String sessionId;
        private String compCause;
        private PDXTransactionImpl tran;
        private boolean isSent;
        
        protected TransactionLogEntry(int tranId, PDXTransactionImpl tran) {
            transactionId = tranId;
            if (tran.session.getSessionId() != null)
                sessionId = NMSPSession.FormatUuid(tran.session.getSessionId());
            else
                sessionId = "";
            compCause = INTERNAL_COMPLETION_CAUSE_INTERNAL_ERROR;
            this.tran = tran;
            isSent = false;
        }
        
        protected int getTranId() {
            return transactionId;
        }
        
        protected String getSessionId() {
            return sessionId;
        }
        
        protected String getCompCause() {
            return compCause;
        }
        
        protected void setSessionId(String sessionId) {
            this.sessionId = sessionId;
        }
        
        protected void setCompCause(CompletionCause compCause) {
        	if (this.compCause.equals(INTERNAL_COMPLETION_CAUSE_INTERNAL_ERROR)) {
        		this.compCause = compCause.toString();
        	} else {
        		this.compCause += ":" + compCause.toString();	
        	}
            tran.appendLogToResLogs(this);
        }
        
        private void setCompCause(String compCauseInternal) {
        	if (this.compCause.equals(INTERNAL_COMPLETION_CAUSE_INTERNAL_ERROR)) {
        		this.compCause = compCauseInternal;
        	} else {
        		this.compCause += ":" + compCauseInternal;
        	}
            tran.appendLogToResLogs(this);
        }
        
        protected void setSent(boolean sent) {
            this.isSent = sent;
        }
        
        protected boolean isSent() {
            return isSent;
        }
    };
    
    private void logEntrySetSessionId(TransactionLogEntry logEntry, String sessionId) {
        if (logEntry != null)
            logEntry.setSessionId(sessionId);
    }
    
    private void logEntrySetCompCause(TransactionLogEntry logEntry, CompletionCause compCause) {
        if (logEntry != null)
            logEntry.setCompCause(compCause);
    }
    
    private void logEntrySetCompCause(TransactionLogEntry logEntry, String compCauseInternal) {
        if (logEntry != null)
            logEntry.setCompCause(compCauseInternal);
    }
    
    private void appendLogToQueryBegin(PDXQueryBegin queryBegin) {
        Vector nmasResLogsToServer = (Vector) ((ManagerImpl) ((ResourceImpl) nmasResource).getManager()).getResourceLogs();
        if (nmasResLogsToServer == null) {
            //#ifdef NMSP_LOG
            log.info("appendLogToQueryBegin: NMSPDefines.DEVICE_CMD_LOG_TO_SERVER_ENABLED is disabled");
            //#endif
            return;
        } else if (nmasResLogsToServer.size() == 0) {
            //#ifdef NMSP_LOG
            log.info("appendLogToQueryBegin: nmasResLogsToServer is empty, nothing to log to server");
            //#endif
            return;
        }
        int size = nmasResLogsToServer.size();
        PDXDictionary appInfoDict = new PDXDictionary();
        PDXSequence logSeq = new PDXSequence();
        
        for (int i = 0; i < size; i++) {
            TransactionLogEntry entry = (TransactionLogEntry) nmasResLogsToServer.elementAt(i);
            if (entry.getTranId() != tranId) {
                // should send all the old entries, even those marked sent
                // it's possible they haven't been modifed back because the
                // MsgNotSent callback has no chance to be called
            //OLD_LOGIC if (!entry.isSent() && entry.getTranId() != tranId) {
                PDXDictionary dict = new PDXDictionary();
                dict.put("id", entry.getSessionId() + ":" + entry.getTranId(), PDXClass.UTF8);
                dict.put("status", entry.getCompCause().toString(), PDXClass.UTF8);
                logSeq.addDictionary(dict);
                entry.setSent(true);
            }
        }

        // later maybe separate these two steps in case we want to append more app info
        // other than device log
        appInfoDict.addSequence("device_log", logSeq);
        queryBegin.addDictionary("app_info", appInfoDict);
    }
    
    private void appendLogToResLogs(TransactionLogEntry entry) {
        Vector nmasResLogsToServer = (Vector) ((ManagerImpl) ((ResourceImpl) nmasResource).getManager()).getResourceLogs();
        if (nmasResLogsToServer == null) {
            //#ifdef NMSP_LOG
            log.info("appendLogToResLogs: NMSPDefines.DEVICE_CMD_LOG_TO_SERVER_ENABLED is disabled.");
            //#endif
            return;
        }
        // in case the user log the completion cause twice
        // we will keep one entry in the list and append the
        // second completion cause after the first completion
        // cause, it is up to the 3rd party application to adjust
        // the logging
        if (!nmasResLogsToServer.contains(entry))
        	nmasResLogsToServer.addElement(entry);
    }
    
    private void createSessionEvent(SessionEvent parent)
    {
        if (parent != null)
        {
            SessionEventBuilder builder = parent.createChildEventBuilder(SessionEvent.NMSP_CALLLOG_NMASCOMMAND_EVENT);
            try {
                sessionEvent = builder.putString("Name", this.commandName)
                                      .putString("Tid", (new Integer(this.tranId)).toString()).commit();
            } catch (SessionEventAlreadyCommittedException e) {}
        }
    }
    
    private void createResponseEvent(String resultType, boolean isFinal) 
    {
        if (sessionEvent != null)
        {
            SessionEventBuilder bldr = sessionEvent.createChildEventBuilder(SessionEvent.NMSP_CALLLOG_NMASRESPONSE_EVENT);
            try {
                bldr = bldr.putString("ResultType", resultType);

                if (resultType.compareTo("QUERY_RESULT") == 0) {
                    bldr = bldr.putBoolean("IsFinal", isFinal);
                }
            } catch (SessionEventAlreadyCommittedException e) {}

            bldr.commit();
        }
    }

    private void createAudioParamEvent(String name, int audioId)
    {
        if (sessionEvent != null)
        {
            SessionEventBuilder bldr = sessionEvent.createChildEventBuilder(SessionEvent.NMSP_CALLLOG_NMASAUDIOPARAM_EVENT);
            try {
                bldr.putString("Name", name).putInteger("AudioId", audioId).commit();
            } catch (SessionEventAlreadyCommittedException e) {}
        }
    }
    
    private void createChildEvent(String eventName)
    {
        if (sessionEvent != null)
        {
            sessionEvent.createChildEventBuilder(eventName).commit();
        }
    }    
}
