/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client.sdk.components.resource.nmas;

/**
 * Receives results (in the form of PDX <i>Query</i> objects) from  
 * Nuance Mobile Application Server and delivers command events.
 * 
 * @author Nuance Communication Inc.
 */
public interface PDXCommandListener {
    /**The Command timed out while waiting for a result. */
    public static final short TIMED_OUT_WAITING_FOR_RESULT = 0x0001;
    /**The connection between the client and the server was terminated from the server side. */
    public static final short REMOTE_DISCONNECTION         = 0x0003;
    /**Processing of the command has completed. */
    public static final short COMMAND_ENDED                = 0x0004;
    /**Processing of the command has not completed, and the command has been idle too long. 
     * @see com.nuance.nmsp.client.sdk.common.defines.NMSPDefines#IdleSessionTimeout
     */
    public static final short COMMAND_IDLE_FOR_TOO_LONG    = 0x0005;

    /**
     * Called when Nuance Mobile Application Server returns a <code>QueryResult</code> 
     * that contains an application logic result or a response to the client.
     * 
     * @param result QueryResult
     */
    public void PDXQueryResultReturned(QueryResult result);

    /**
     * Called when a <code>QueryError</code> is returned by 
     * Nuance Mobile Application Server to indicate that an error
     * occurred either in the command or on the server side.
     * 
     * @param error QueryError
     */
    public void PDXQueryErrorReturned(QueryError error);

    /**
     * Called when Nuance Mobile Application Server returns a
     * <code>QueryRetry</code> to indicate that the 
     * quality or content of the received audio is insufficient
     * for a recognition and that Nuance Mobile Application Server
     * is requesting to "retry" recording and streaming the audio.  
     * 
     * @param retry QueryRetry
     */
    public void PDXQueryRetryReturned(QueryRetry retry);

    // /**
    // * Called when a <code>QueryDisambiguate</code> is returned by Nuance Mobile Application Server.
    // * @param disambiguation QueryDisambiguate
    // */
    // public void PDXQueryDisambiguateReturned(QueryDisambiguate disambiguation);

    // /**
    // * Called when a <code>QueryNarrow</code> is returned by Nuance Mobile Application Server.
    // * @param narrow QueryNarrow
    // */
    // public void PDXQueryNarrowReturned(QueryNarrow narrow);

    // /**
    // * Called when a <code>ScriptUpdateAvailable</code> is returned by Nuance Mobile Application Server.
    // * @param update ScriptUpdateAvailable
    // */
    // public void PDXScriptUpdateAvailableReturned(ScriptUpdateAvailable update);

    // /**
    // * Called when a <code>UpdateChunk</code> is returned by Nuance Mobile Application Server.
    // * @param update UpdateChunk
    // */
    // public void PDXUpdateChunkReturned(UpdateChunk update);

    // /**
    // * Called when 'No script available' is returned by NMAS.
    // */
    // public void PDXNoScriptUpdateAvailableReturned();

//    /**
//     * Called when the command has been created.
//     * 
//     * @param sessionId     NMSP Gateway session ID. This information is used for debugging
//     *                      purposes only.  
//     */
//    public void PDXCommandCreated(String sessionId);

    /**
     * Called to notify the application of an event.
     * 
     * @param eventCode  a short indicating the event; 
     *                   possible values are:
     *                   <ul>
     *                   <li> {@link PDXCommandListener#TIMED_OUT_WAITING_FOR_RESULT}</li>
     *                   <li> {@link PDXCommandListener#REMOTE_DISCONNECTION}</li>
     *                   <li> {@link PDXCommandListener#COMMAND_IDLE_FOR_TOO_LONG}</li>
     *                   <li> {@link PDXCommandListener#COMMAND_ENDED}</li>
     *                   </ul>
     */
    public void PDXCommandEvent(short eventCode);
}
