package com.nuance.nmsp.client.sdk.oem;

import java.util.Vector;

import android.content.Context;
import android.content.pm.PackageManager;

import com.nuance.nmsp.client.sdk.common.defines.NMSPDefines;
import com.nuance.nmsp.client.sdk.common.oem.api.BluetoothSystem;
import com.nuance.nmsp.client.sdk.common.oem.api.MessageSystem;
import com.nuance.nmsp.client.sdk.components.general.Parameter;
import com.nuance.nmsp.client.sdk.oem.bluetooth.Bluetooth;
//#ifdef NMSP_LOG
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory.Log;
//#endif

public class BluetoothSystemOEM implements BluetoothSystem
{
    //#ifdef NMSP_LOG
    // Logger
    private Log log = LogFactory.getLog(this.getClass());
    //#endif

    private Context _context = null;
    private boolean _bluetoothDisabled = false;
    
    public BluetoothSystemOEM(Vector parameters)  {
        if (parameters != null) {
            for (int i = 0; i < parameters.size() ; i++) {
                Parameter curParam = (Parameter) parameters.get(i);
                String curParamName = curParam.getName();
                if(curParam.getType() == Parameter.Type.SDK)
                {
                    if (curParamName.equals(NMSPDefines.NMSP_DEFINES_ANDROID_CONTEXT)) {
                        _context = (Context)curParam.getValueRaw();
                        //#ifdef NMSP_LOG
                        if(log.isDebugEnabled())
                            log.debug("NMSP_DEFINES_ANDROID_CONTEXT is passed in as" + _context);
                        //#endif
                    }
                    else if (curParamName.equals(NMSPDefines.NMSP_DEFINES_DISABLE_BLUETOOTH)) {
                        String curParamValue = new String(curParam.getValue());
                        if (curParamValue.equalsIgnoreCase("TRUE")) {
                            //#ifdef NMSP_LOG
                            if(log.isDebugEnabled())
                                log.debug(NMSPDefines.NMSP_DEFINES_DISABLE_BLUETOOTH + " is true.");
                            //#endif
                            _bluetoothDisabled = true;
                        }
                    }
                }
            }
        }
        if (_context != null) {
        	String permission = "android.permission.BLUETOOTH";
        	_bluetoothDisabled = !(_context.checkCallingOrSelfPermission(permission) == PackageManager.PERMISSION_GRANTED);
        }
    }
    
    public boolean isBluetoothHeadsetConnected() {
        if (_bluetoothDisabled)
            return false;
        if (_context == null) {
            //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("ANDROID_CONTEXT parameter is not passed in!!!");
            //#endif
            return false;
        }
        Bluetooth bt = Bluetooth.createInstance(_context);
        boolean connected = bt.isHeadsetConnected();
        bt.close();
        return connected;
    }
}
