package com.nuance.nmsp.client.sdk.oem;

import java.util.Vector;

import com.nuance.nmsp.client.sdk.common.defines.NMSPDefines;
import com.nuance.nmsp.client.sdk.components.general.Parameter;
import com.nuance.nmsp.client.sdk.oem.EndPointer.EpType;


public class ElvisEndPointer implements EndPointer{
	
    public final static int NGF_ELVIS_NOSPEECH = 0;     /*!< FE has not yet detected speech */
	public final static int NGF_ELVIS_SPEECH = 1;           /*!< FE has detected onset of speech */
	public final static int NGF_ELVIS_DONE = 2;             /*!< FE thinks utt is done or wakeup phrase was detected. */
	public final static int NGF_ELVIS_SPEECH_ERROR = 3;     /*!< An error condition occurred. */
	
	
	private EpType elvisResultToEpType(int epResult){
		EpType type = EpType.DETECT_NOTHING;
		switch(epResult){
		case 0:
			type = EpType.DETECT_NOTHING;
			break;
		case 1:
			type = EpType.SPEECH_ONGOING;
			break;
		case 2:
			type = EpType.SPEECH_END;
			break;
		case 3:
			type = EpType.DETECT_ERROR;
			break;
		}
		return type;
	}
    
	boolean _isEndPointingActive = false;
	NativeElvisImpl _nativeElvis = null;
	int _samplesNeeded = 50000; 
	boolean _enableEndPointing = true;
	int _samplingRate = 0;
	
	public ElvisEndPointer(int samplingRate){
		_samplingRate = samplingRate;
	}
	
	@Override
	public void initialize(Vector audioParams) {

	
		if (audioParams != null) {

			for (int i = 0; i < audioParams.size() ; i++) {
				Parameter curParam = (Parameter) audioParams.get(i);
				String curParamName = curParam.getName();
				
				if (curParamName.equals(VST_OEM_AUDIO_VAD_STOP_ON_END_OF_SPEECH)) {
	                String curParamValue = new String(curParam.getValue());
					if (curParamValue.equalsIgnoreCase("TRUE")) {
//						//#ifdef NMSP_LOG
//						if(log.isDebugEnabled())
//							log.debug("Stop on end of speech is activated.");
//						//#endif
//						vadparams.stopOnEndOfSpeech = 1;
					}
				}
				else if (curParamName.equals(VST_OEM_AUDIO_VAD_END_THRESHOLD)) {
	                String curParamValue = new String(curParam.getValue());
	                _samplesNeeded = Integer.parseInt(curParamValue);
				}
				else if (curParamName.equals(NMSPDefines.NMSP_DEFINES_ENDPOINTER)) {
	                String curParamValue = new String(curParam.getValue());
	                if (curParamValue.equalsIgnoreCase("TRUE")) {
	                	_enableEndPointing = true;

	                }
				}
			}
		}		
//		isEndPointingActive = (vadparams.stopOnEndOfSpeech == 1);
//		if (isEndPointingActive) {
//			initializeEndPointer(vadparams.stopOnEndOfSpeech, vadparams.vadLongUtterance, vadparams.vadHistoryLength, 
//					vadparams.vadBeginLength, vadparams.vadBeginThreshold, vadparams.vadBeginDelay, vadparams.vadEndLength, 
//					vadparams.vadEndThreshold, vadparams.vadInterSpeechLength);
//			resetEndpointingDetection();
//		}
		if(_enableEndPointing){
    		_nativeElvis = new NativeElvisImpl(_samplingRate);
		}
		
	}

	@Override
	public boolean isEndPointingActive() {
		return _isEndPointingActive;
	}

	@Override
	public EpType detectEndPointing(short[] data) {
		if(!_enableEndPointing){
			return EpType.DETECT_ERROR;
		}
		if(!_isEndPointingActive){
			return EpType.DETECT_ERROR;
		}
		int endPoint = _nativeElvis.processEndpointingAudio(data);
		return elvisResultToEpType(endPoint);
	}

	@Override
	public void resetEndpointingDetection() {
		if(!_enableEndPointing){
			return;
		}
		_nativeElvis.endpointingMode(_samplesNeeded);
		_isEndPointingActive = true;
		// TODO Auto-generated method stub
		
	}

}
