package com.nuance.nmsp.client.sdk.oem;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

//#ifdef NMSP_PLATFORM_ANDROID
import android.os.Environment;
//#endif

import com.nuance.nmsp.client.sdk.common.oem.api.FileSystem;
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client.sdk.common.oem.api.MessageSystem;
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory.Log;

public class FileSystemOEM implements FileSystem
{
    private final static Log log = LogFactory.getLog(NetworkSystemOEM.class);
    
    private FileMode _fileMode;
    private File _file = null;
    private FileInputStream _is = null;
    private FileOutputStream _os = null;
    
    public FileSystemOEM()
    {
    }
    
    public boolean open(String fileName, FileMode fileMode)
    {
        _fileMode = fileMode;
        //#ifdef NMSP_PLATFORM_ANDROID
    	File dataDir = Environment.getDataDirectory();
        _file = new File(dataDir, fileName);       
        //#else
//@        	_file = new File(fileName);
        //#endif

        boolean error = false;
        if (fileMode == FileMode.READ)
            error = !_file.exists();
        /*else if (fileMode == FileMode.WRITE || fileMode == FileMode.APPEND)
            error = !_file.canWrite();*/
        
        if (error)
            _file = null;
        
        return !error;
    }

    public void close()
    {
        if (_is != null)
            try { _is.close(); } catch (IOException e) {}
        if (_os != null)
            try { _os.close(); } catch (IOException e) {}
        _is = null;
        _os = null;
        _file = null;
    }

    public int read(byte[] b, int off, int len)
    {
        if (_file == null)
            return -1;
        
        if (_is == null)
        {
            try
            {
                _is = new FileInputStream(_file);
            } catch (FileNotFoundException e)
            {
                // should not happen
                _file = null;
                return -1;
            }
        }
        
        try
        {
            return _is.read(b, off, len);
        } catch (Exception e)
        {
            try { _is.close(); } catch (IOException e1) {}
            _is = null;
            _file = null;
            return -1;
        }
    }

    public int write(byte[] b, int off, int len)
    {
        if (_file == null)
            return -1;
        
        if (_os == null)
        {
            try
            {
                _os = new FileOutputStream(_file, _fileMode == FileMode.APPEND);
            } catch (FileNotFoundException e)
            {
                // should not happen
                _file = null;
                return -1;

            }
        }
        
        try
        {
            _os.write(b, off, len);
            return len;
        } catch (IOException e)
        {
            try { _os.close(); } catch (IOException e1) {}
            _os = null;
            _file = null;
            return -1;
        }
        
    }

    public long skip(long n)
    {
        if (_file != null && _is != null)
        {
            try
            {
                return _is.skip(n);
            } catch (IOException e) {}
        }
        return 0;
    }

    public long size()
    {
        return _file == null ? -1 : _file.length();
    }

    public boolean exist()
    {
        return _file != null && _file.exists();
    }

    public boolean rename(String newName)
    {
        return _file == null ? false : _file.renameTo(new File(newName));
    }

    public boolean remove()
    {
        if (_file != null)
        {
            if (_is != null)
                try { _is.close(); } catch (IOException e) {}
            if (_os != null)
                try { _os.close(); } catch (IOException e) {}
            _is = null;
            _os = null;
            File file = _file;
            _file = null;
            return file.delete();
        }
        return false;
    }
}
