package com.nuance.nmsp.client.sdk.oem;

//package com.nuance.dragon.toolkit.elvis;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.json.JSONObject;

import android.os.SystemClock;


//#ifdef NMSP_LOG
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory.Log;
//#endif

//
//import com.nuance.dragon.toolkit.elvis.ElvisRecognizer.Logging;
//import com.nuance.dragon.toolkit.file.FileManager;
//import com.nuance.dragon.toolkit.grammar.Word;
//import com.nuance.dragon.toolkit.util.JSONUtils;
//import com.nuance.dragon.toolkit.util.log;

/**
 * Internal class that directly interfaces with the Elvis engine through JNI.
 * This is split from ElvisRecognizerImpl so that ElvisRecognizerImpl can be
 * obfuscated (this class doesn't need to be, since it isn't public).
 */
public class NativeElvisImpl
{
	//#ifdef NMSP_LOG
    private final static Log log = LogFactory.getLog(MessageSystemOEM.class);
    //#endif
    
    static final int SILENCE = 0;
    static final int SPEECH = 1;
    static final int SPEECH_ENDED = 2;
    static final int SPEECH_ERROR = 3;
    
    static boolean LOAD_LIBRARY_SUCCESS = true;
    static {
        try {
            System.loadLibrary("nmsp_elvis");
            System.out.println("Elvis loaded successfully");
        }
        catch (UnsatisfiedLinkError e) {
            log.error(NativeElvisImpl.class.getName() + " Failed to load native library. + " + e.getMessage());
            LOAD_LIBRARY_SUCCESS = false;
        }
    }
    private native long elvisCreate(int sampleRate);
    private native void elvisDestroy(long elvis);

    private native int elvisStartEndpointing(long elvis, int neededSamples);
    private native int elvisStopEndpointing(long elvis);
    private native int elvisProcessEndpointingAudio(long elvis, short data[], int numSamples);

    private long _elvis;
    private int _samplingRate;

    /* Note: The value of these constants must match the values in ngf_elvis.h */
    private static final int ELVIS_OK = 0;                     /*!< Normal return. */
    private static final int ELVIS_OOM = 1;                    /*!< Heap exhausted, or internal buffer full.*/
    private static final int ELVIS_INVALID_LANGUAGE = 2;       /*!< Invalid language selected.*/
    private static final int ELVIS_FILE_ERROR = 3;             /*!< Corrupt, unavailable, or non-existent file.*/
    private static final int ELVIS_ARGUMENT_ERROR = 4;         /*!< An invalid argument was passed to native Elvis. */
    private static final int ELVIS_UNSUPPORTED_OPERATION = 5;  /*!< Requested operation is not supported at this time. */
    private static final int ELVIS_NO_PRONUNCIATION = 6;       /*!< Unable to add word because Elvis has no pronunciation. */
    private static final int ELVIS_VOCABULARY_FULL = 7;        /*!< Unable to add word because Elvis's vocabulary is full. */
    private static final int ELVIS_OTHER_ERROR = 8;            /*!< An error occurred.*/
    private static final int ELVIS_NOSPEECH = 0;     /*!< FE has not yet detected speech */
    private static final int ELVIS_SPEECH = 1;       /*!< FE has detected onset of speech */
    private static final int ELVIS_DONE = 2;         /*!< FE thinks utt is done or wakeup phrase was detected. */
    private static final int ELVIS_SPEECH_ERROR = 3; /*!< An error condition occurred. */


    public NativeElvisImpl(int samplingRate)
    {
    	this.createElvis(samplingRate);
        //_fileManager = fileManager;
    }


//Override
    private boolean createElvis(int samplingRate)
    {
    	_elvis = elvisCreate(samplingRate);

        if (_elvis != 0) {
            log.debug("Created native Elvis");
            _samplingRate = samplingRate;
            //log.debug("starting end pointing : " + elvisStartEndpointing(_elvis, 50));
            //log.debug("stopping end pointing : " + elvisStopEndpointing(_elvis));
            
            
            return true;
        } else {
            log.error("Unable to create native Elvis");
            return false;
        }
    }


    public void destroyElvis()
    {
        if (_elvis == 0) {
            log.error("destroyElvis: native Elvis does not exist!");
            return;
        }

        elvisDestroy(_elvis);
        _elvis = 0;
        log.debug("Destroyed native Elvis");
    }

    public boolean endpointingMode(int neededSamples)
    {
        if (_elvis == 0) {
            log.error("Error starting endpointing mode: native Elvis does not exist.");
            return false;
        }
        log.debug("Starting Endpointing Mode.");
        return (ELVIS_OK == elvisStartEndpointing(_elvis, neededSamples));
    }


    public int processEndpointingAudio(short[] shorts)
    {
        if (_elvis == 0) {
            log.error("processEndpointingAudio: native Elvis does not exist.");
            return SPEECH_ERROR;
        }

        // Feed to Elvis
        int speech = SILENCE;
        if (shorts != null) {
        	//log.debug("Processing audio for endpointer");
            speech = elvisProcessEndpointingAudio(_elvis, shorts, shorts.length);
        }
        return speech;
    }


    public boolean finishEndpointing()
    {
        if (_elvis != 0)
        {
            return (ELVIS_OK == elvisStopEndpointing(_elvis));
        }
        return false;
    }

////Override
//    public boolean adapt(String phrase)
//    {
//        if (_elvis == 0) {
//            log.error("Error starting adaptation: native Elvis does not exist.");
//            return false;
//        }
//        if (phrase == null || phrase.length() == 0) {
//            log.error("Error starting adaptation: invalid phrase.");
//            return false;
//        }
//        return (ELVIS_OK == elvisStartAdaptation(_elvis, phrase));
//    }
//
////Override
//    public boolean processAdaptation()
//    {
//        if (_elvis != 0)
//        {
//            return (ELVIS_OK == elvisProcessAdaptation(_elvis));
//        }
//        return false;
//    }
//
////Override
//    public boolean loadAdaptation(String fileName)
//    {
//        String adpFile = fileName + _language + _samplingRate + ".adp";
//        // TODO manage file name and extension
//        if (_elvis == 0) {
//            log.error("loadAdaptation(" + fileName + "): native Elvis does not exist.");
//            return false;
//        }
//
//        if (_fileManager.exists(fileName))
//        {
//            int stat = elvisLoad(_elvis, null, adpFile);
//            if (stat != ELVIS_OK) {
//                log.error("loadAdaptation (" + adpFile + ") error loading Elvis with adaptation data: " + stat);
//                return false;
//            }
//        }
//        return true;
//    }
//
////Override
//    public boolean finishAdapting(String fileName, boolean abort)
//    {
//        String adpFile = fileName + _language + _samplingRate + ".adp";
//        if (_elvis != 0)
//        {
//            return (ELVIS_OK == elvisEndAdaptationSession(_elvis, adpFile, abort));
//        }
//        return false;
//    }
//
////Override
//    public boolean checkAdaptationStatus(String fileName)
//    {
//        if (_elvis == 0) {
//            log.error("Error checking adaptation status: native Elvis does not exist.");
//            return false;
//        }
//        if (_fileManager.exists(fileName + _language + _samplingRate + ".adp")) {
//            return elvisCheckAdaptationStatus(_elvis);
//        }
//        return false;
//    }
//
////Override
//    public boolean clearAdaptation(String fileName, boolean all)
//    {
//        final String ext = ".adp";
//        if (_elvis == 0) {
//            log.error("clearAdaptation: native Elvis does not exist!");
//            return false;
//        }
//        elvisDestroy(_elvis);
//        _elvis = 0;
//
//        if (all) {
//            String[] files = _fileManager.findFiles(ext);
//            if (files != null) {
//                for (String f : files) {
//                    _fileManager.delete(f);
//                }
//            }
//        }
//        else {
//            _fileManager.delete(fileName + _language + _samplingRate + ext);
//        }
//        return createElvis(_language, _samplingRate, _binFile);
//    }
//
////Override
//    public int processAudio(short[] shorts)
//    {
//        if (_elvis == 0) {
//            log.error("processAudio: native Elvis does not exist.");
//            return SPEECH_ERROR;
//        }
//
//        // Feed to Elvis
//        int speech = SILENCE;
//        if (shorts != null) {
//            speech = elvisProcessAudio(_elvis, shorts, shorts.length);
//        }
//        return speech;
//    }
//
////Override
//    public int processWakeupAudio(short[] shorts)
//    {
//        if (_elvis == 0) {
//            log.error("processWakeupAudio: native Elvis does not exist.");
//            return SPEECH_ERROR;
//        }
//
//        // Feed to Elvis
//        int speech = SILENCE;
//        if (shorts != null) {
//            if (_refTimestamp == -1)
//                _refTimestamp = SystemClock.uptimeMillis();
//            speech = elvisProcessWakeupAudio(_elvis, shorts, shorts.length);
//        }
//        return speech;
//    }
//
////Override
//    public ElvisResult getResult(Grammar grammar)
//    {
//        ElvisResult result = null;
//        ElvisNbestList nbest = null;
//        if (_elvis == 0) {
//            log.error("getResult: native Elvis does not exist.");
//            return null;
//        }
//        int choices = elvisGetChoiceCount(_elvis);
//        int gateConfidence = elvisGetGateConfidence(_elvis);
//
//        if (choices < 0) {
//            log.error("getResult: error retrieving choices.");
//        }
//        else if (choices == 0) {
//            log."getResult: no choices available.");
//        }
//        else if (choices > 0) {
//            log."getResult: " + choices + " choices.");
//            ArrayList<NbestEntryJni> jniEntries = new ArrayList<NbestEntryJni>(choices);
//            for (int i = 0; i < choices; i ++) {
//                int status = elvisGetChoiceListEntry(_elvis, i, jniEntries);
//                if (status != ELVIS_OK) {
//                    log.error("elvis result error.");
//                    break;
//                }
//            }
//
//            ArrayList<ElvisNbestList.Entry> entries = new ArrayList<ElvisNbestList.Entry>(choices);
//
//            for (int i = 0; i < jniEntries.size(); i++)
//            {
//                entries.add(jniEntries.get(i).createRealEntry(grammar));
//            }
//
//            nbest = new ElvisNbestList(entries);
//        }
//        if (nbest != null  && !nbest.isEmpty()) {
//            result = new ElvisResult(nbest, gateConfidence, ElvisResult.WAKEUP_TIMESTAMP_UNAVAILABLE, ElvisResult.WAKEUP_TIMESTAMP_UNAVAILABLE, false);
//            DefaultResultFilter drf = new DefaultResultFilter();
//            result = drf.applyFilter(result);
//        } else {
//            log.warn(this, "No nbest results.");
//        }
//        return result;
//    }
//
////Override
//    public ElvisResult getWakeupResult()
//    {
//        ElvisResult result = null;
//        ElvisNbestList nbest = null;
//        if (_elvis == 0) {
//            log.error("getWakeupResult: native Elvis does not exist.");
//            return null;
//        }
//        int start = elvisGetUtteranceStartTime(_elvis);
//        int end = elvisGetUtteranceEndTime(_elvis);
//
//        ArrayList<NbestEntryJni> jniEntries = new ArrayList<NbestEntryJni>(1);
//        int status = elvisGetWakeupEntry(_elvis, jniEntries);
//        if (status != ELVIS_OK || jniEntries.isEmpty()) {
//            log.error("elvis result error.");
//        } else
//        {
//            long wakeupStartTimestamp = ElvisResult.WAKEUP_TIMESTAMP_UNAVAILABLE;
//            if (start != -1) {
//                wakeupStartTimestamp = _refTimestamp + start;
//            }
//            else {
//                log.error("getWakeupResult: error retrieving utterance start time.");
//            }
//            long wakeupEndTimestamp = ElvisResult.WAKEUP_TIMESTAMP_UNAVAILABLE;
//            if (end != -1) {
//                wakeupEndTimestamp = _refTimestamp + end;
//            }
//            else {
//                log.error("getWakeupResult: error retrieving utterance end time.");
//            }
//
//            nbest = new ElvisNbestList(jniEntries.get(0).createRealEntry(null));
//            result = new ElvisResult(nbest, ElvisResult.GATE_CONFIDENCE_UNAVAILABLE, wakeupStartTimestamp, wakeupEndTimestamp, false);
//        }
//        return result;
//    }
//
////Override
//    public boolean checkWakeupPhrase(String wakeupPhrase)
//    {
//        if (_elvis == 0) {
//            log.error("Error checking wakeup phrase: native Elvis does not exist.");
//            return false;
//        }
//        return elvisCheckWakeupPhrase(_elvis, wakeupPhrase);
//    }
//
////Override
//    public void releaseUnusedMemory()
//    {
//        if (_elvis == 0) {
//            log.error("Error releasing unused memory: native Elvis does not exist.");
//            return;
//        }
//        elvisReleaseUnusedMemory(_elvis);
//    }
//
////Override
//    public void enableElvisVerboseLogging(boolean verbose)
//    {
//        elvisEnableVerboseLogging(verbose);
//    }
//
////Override
//    public boolean enableElvisRecognitionLogging(String logPrefix, int logMillis)
//    {
//        if (_elvis == 0) {
//            log.error("Error enabling recognition logging: native Elvis does not exist.");
//            return false;
//        }
//        return (elvisEnableRecognitionLogging(_elvis, logPrefix, logMillis) == 0);
//    }
//
////Override
//    public int getElvisRecognitionLoggingStatus()
//    {
//        if (_elvis == 0) {
//            log.error("Error getting logging status: native Elvis does not exist.");
//            return Logging.LOGGING_ERROR;
//        }
//        return elvisGetRecognitionLoggingStatus(_elvis);
//    }
//
////Override
//    public boolean copyElvisRecognitionLogs(String destinationFolder, String destinationPrefix)
//    {
//        String sourcePsaFile = destinationPrefix + "elvisdump.psa";
//        String absDestPsaFile = destinationFolder + File.separator + sourcePsaFile;
//        String sourceRprFile = destinationPrefix + "elvisdump.rpr";
//        String absDestRprFile = destinationFolder + File.separator + sourceRprFile;
//        String sourceUttFile = destinationPrefix + "elvisdump.utt";
//        String absDestUttFile = destinationFolder + File.separator + sourceUttFile;
//        String sourceWavFile = destinationPrefix + "elvisdump.sph";
//        String absDestWavFile = destinationFolder + File.separator + sourceWavFile;
//
//        File folder = new File(destinationFolder);
//        if (!folder.exists()) {
//            if (!folder.mkdir()) {
//                log.error("Destination folder cannot be created.");
//                return false;
//            }
//        } else if (!folder.isDirectory()) {
//            log.error("Destination folder is not a valid directory.");
//            return false;
//        }
//        if (copyFileToAbsoluteLocation(sourcePsaFile, absDestPsaFile))
//            if(copyFileToAbsoluteLocation(sourceRprFile, absDestRprFile))
//                if (copyFileToAbsoluteLocation(sourceUttFile, absDestUttFile))
//                    return copyFileToAbsoluteLocation(sourceWavFile, absDestWavFile);
//        return false;
//    }
//
//    private boolean copyFileToAbsoluteLocation(String source, String destination)
//    {
//        FileInputStream fis = _fileManager.openFileForReading(source);
//        if (fis == null) {
//            log.error("Unable to read file:" + source);
//            return false;
//        }
//        FileOutputStream fos = null;
//
//        File destFile = new File (destination);
//        try {
//            if (destFile.exists()) {
//                if (!destFile.delete()) {
//                    log.error("Can't delete file: " + destination);
//                    fis.close();
//                    return false;
//                }
//            }
//            if (!destFile.createNewFile()) {
//                log.error("Could not create log output file:" + destination);
//                fis.close();
//                return false;
//            }
//            fos = new FileOutputStream(destFile);
//            byte[] buff = new byte[1024];
//            int bytesRead = 0;
//            while((bytesRead = fis.read(buff)) > -1 )
//            {
//                fos.write(buff, 0, bytesRead);
//            }
//            fis.close();
//            fis = null;
//            fos.close();
//            fos = null;
//        } catch (IOException e) {
//            log.error("Error copying file:" + e.toString());
//            try {
//                if (fis != null)
//                    fis.close();
//            } catch (IOException ioe) {
//                log.error("Error closing file:" + ioe.toString());
//            }
//            try {
//                if (fos != null)
//                    fos.close();
//            } catch (IOException ioe) {
//                log.error("Error closing file:" + ioe.toString());
//            }
//            return false;
//        }
//        return true;
//    }
}
