/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client.sdk.oem;

//#ifdef NMSP_LOG
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory.Log;
//#endif
import com.nuance.nmsp.client.sdk.common.defines.NMSPDefines;
import com.nuance.nmsp.client.sdk.common.oem.api.NetworkSystem;
import com.nuance.nmsp.client.sdk.common.oem.api.MessageSystem;
import com.nuance.nmsp.client.sdk.common.oem.api.MessageSystem.MessageHandler;
import com.nuance.nmsp.client.sdk.components.general.Parameter;
import com.nuance.nmsp.client.sdk.oem.socket.NmspSocketFactory;
import com.nuance.nmsp.client.sdk.oem.socket.SocketSetting;
import com.nuance.nmsp.client.sdk.oem.socket.SocketSetting.ProxyOption;
import com.nuance.nmsp.client.sdk.oem.socket.ssl.SSLSettings;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Enumeration;
import java.util.Vector;

import javax.net.ssl.SSLHandshakeException;
import javax.net.ssl.SSLSocket;
import android.content.Context;

public class NetworkSystemOEM implements NetworkSystem, MessageHandler {

    //#ifdef NMSP_SOCKET_HANDSHAKE
//@    private static final int XMODE_PACKET_HEADER_SIZE = 8;
	//#endif
	
	//#ifdef NMSP_LOG
	private final static Log log = LogFactory.getLog(NetworkSystemOEM.class);
	//#endif
	
	private MessageSystem msgSys = null;

    private final static Integer CALLBACK_TYPE_OPEN  = new Integer(0);
    private final static Integer CALLBACK_TYPE_CLOSE = new Integer(1);
    private final static Integer CALLBACK_TYPE_READ  = new Integer(2);
    private final static Integer CALLBACK_TYPE_WRITE = new Integer(3);
   	
    private Object syncObj = new Object();

    private static class OEMSocket {
        Socket socket;
        CloseSocketCallback closeCallback;
        InputStream is = null;
        OutputStream os = null;
        Object context = null;
        SocketReadThread sockReadThd = null;
        public OEMSocket(Socket s, CloseSocketCallback closeCb, Object con, InputStream i, OutputStream o, SocketReadThread sr) {
            socket = s;
            closeCallback = closeCb;
            context = con;
            is = i;
            os = o;
            sockReadThd = sr;
        }
    }
    
    public NetworkSystemOEM(MessageSystem msgSys) {
        this.msgSys = msgSys;
    }
   
    private void postCallbackMsg(Object[] params) {
        msgSys.send(params, this, msgSys.getMyAddr(), msgSys.getVRAddr()[0]);
    }
    
    public void handleMessage(Object msgData, Object msgSender) {
        Object[] params = (Object[])msgData;
        if(params[0] == CALLBACK_TYPE_OPEN) {
            ((OpenSocketCallback)params[1]).openSocketCallback((NetworkStatus)params[2], params[3], params[4]);
        }
        else if(params[0] == CALLBACK_TYPE_CLOSE) {
            ((CloseSocketCallback)params[1]).closeSocketCallback((NetworkStatus)params[2], params[3], params[4]);
        }
        else if(params[0] == CALLBACK_TYPE_READ) {
            ((ReadSocketCallback)params[1]).readSocketCallback((NetworkStatus)params[2], params[3], (byte[])params[4],
                                                         ((Integer)params[5]).intValue(), ((Integer)params[6]).intValue(),
                                                         ((Integer)params[7]).intValue(), params[8]);
        }
        else if(params[0] == CALLBACK_TYPE_WRITE) {
            ((WriteSocketCallback)params[1]).writeSocketCallback((NetworkStatus)params[2], params[3], (byte[])params[4],
                                                         ((Integer)params[5]).intValue(), ((Integer)params[6]).intValue(),
                                                         ((Integer)params[7]).intValue(), params[8]);
        }
    }
	
       
    public void openSocket(String hostName, int port, Vector params,
    		OpenSocketCallback openCallback,
    		CloseSocketCallback closeCallback,
    		Object context) {
    	
    	SocketSetting socketSetting = new SocketSetting();
	    SSLSettings newSSLSettings = new SSLSettings();
		Enumeration elements = params.elements();
		while (elements.hasMoreElements())
		{
			Parameter element = (Parameter) elements.nextElement();
			if (element.getName().equals(NMSPDefines.NMSP_DEFINES_USING_DEVICE_PROXY_SETTING_IF_AVAILABLE))
			{
				if ((new String(element.getValue())).equals("TRUE") || (new String(element.getValue())).equals("true") )
				{
					socketSetting.proxyOption = ProxyOption.PROXY_IF_AVAIL;
				}
			}
			// this order is important since we allow the PROXY_ONLY to override the PROXY_IF_AVAIL
			if (element.getName().equals(NMSPDefines.NMSP_DEFINES_USING_DEVICE_PROXY_SETTING_ONLY))
			{
				if ((new String(element.getValue())).equals("TRUE") || (new String(element.getValue())).equals("true") )
				{
					socketSetting.proxyOption = ProxyOption.PROXY_ONLY;
				}
			}
			if (element.getName().equals(NMSPDefines.NMSP_DEFINES_ANDROID_CONTEXT))
			{
				socketSetting.context = (Context) element.getValueRaw();
			}
		}

    	SocketReadThread sockReadThd = new SocketReadThread(hostName, port, socketSetting, openCallback, closeCallback, context);
    	
    	if (sockReadThd == null)
    	{
    		//#ifdef NMSP_LOG
    		if (log.isErrorEnabled())
    		{
    			log.error("new SocketReadThread failed");
    		}
    		//#endif
    		postCallbackMsg(new Object[] {CALLBACK_TYPE_OPEN, openCallback, NetworkStatus.NETWORK_ERROR, null, context});
    		return;
    	}

    	try
    	{
    		sockReadThd.start();
    	}
    	catch (Exception e)
    	{
    		//#ifdef NMSP_LOG
    		if (log.isErrorEnabled())
    		{
    			log.error("Open Socket Exception - [" + e.getClass().getName() + "] Message - [" + e.getMessage() + "]");
    		}
    		//#endif
    		postCallbackMsg(new Object[] {CALLBACK_TYPE_OPEN, openCallback, NetworkStatus.NETWORK_ERROR, null, context});
    		return;
    	}

    }
	
    public void openSSLSocket(String hostName, int port, Vector params,
			OpenSocketCallback openCallback, CloseSocketCallback closeCallback,
			Object context) 
    {
    	SocketSetting socketSetting = new SocketSetting();
    	socketSetting.isSSL = true;
    	socketSetting.sslSetting = new SSLSettings();
		Enumeration elements = params.elements();
		while (elements.hasMoreElements())
		{
			Parameter element = (Parameter) elements.nextElement();
			if (element.getName().equals(NMSPDefines.NMSP_DEFINES_SSL_SELFSIGNED_CERT))
			{
				if ((new String(element.getValue())).equals("TRUE") || 
					(new String(element.getValue())).equals("true") )
				{
					socketSetting.sslSetting.enableSelfSignedCert = true;
				}
			}

			if (element.getName().equals(NMSPDefines.NMSP_DEFINES_SSL_CERT_SUMMARY))
			{
				socketSetting.sslSetting.certSummary = new String(element.getValue());
			}

			if (element.getName().equals(NMSPDefines.NMSP_DEFINES_SSL_CERT_DATA)) 
			{
				socketSetting.sslSetting.certData = new String(element.getValue());
			}
			
			if (element.getName().equals(NMSPDefines.NMSP_DEFINES_USING_DEVICE_PROXY_SETTING_IF_AVAILABLE))
			{
				if ((new String(element.getValue())).equals("TRUE") || (new String(element.getValue())).equals("true") )
				{
					socketSetting.proxyOption = ProxyOption.PROXY_IF_AVAIL;
				}
			}
			// this order is important since we allow the PROXY_ONLY to override the PROXY_IF_AVAIL
			if (element.getName().equals(NMSPDefines.NMSP_DEFINES_USING_DEVICE_PROXY_SETTING_ONLY))
			{
				if ((new String(element.getValue())).equals("TRUE") || (new String(element.getValue())).equals("true") )
				{
					socketSetting.proxyOption = ProxyOption.PROXY_ONLY;
				}
			}
			if (element.getName().equals(NMSPDefines.NMSP_DEFINES_ANDROID_CONTEXT))
			{
				socketSetting.context = (Context) element.getValueRaw();
			}
		}

		SocketReadThread sockReadThd = new SocketReadThread(hostName, port, socketSetting, openCallback, closeCallback, context);
		if (sockReadThd == null) {
// #ifdef NMSP_LOG
			 if(log.isErrorEnabled())
			 log.error("new SocketReadThread failed");
// #endif
			postCallbackMsg(new Object[] { CALLBACK_TYPE_OPEN, openCallback,
					NetworkStatus.NETWORK_ERROR, null, context });
			return;
		}

		try {
			sockReadThd.start();
		} catch (Exception e) {
// #ifdef NMSP_LOG
				            if(log.isErrorEnabled())
				                log.error("Open Socket Exception - [" + e.getClass().getName() + "] Message - [" + e.getMessage() + "]");
// #endif
			postCallbackMsg(new Object[] { CALLBACK_TYPE_OPEN, openCallback,
					NetworkStatus.NETWORK_ERROR, null, context });
			return;
		}	    
	    
    };
	
	public void setSessionId(Object socketStruct, byte[] sessionId) {
	}
	
    public void closeSocket(Object socket) {
        OEMSocket oemSock = (OEMSocket)socket;
        boolean toStop = false;

        if(oemSock == null) {
            return;
        }
        synchronized(syncObj) {
            if(oemSock.sockReadThd == null) {
                return;
            }
            if(!oemSock.sockReadThd.stopping) {
                toStop = true;
                oemSock.sockReadThd.stopThread();
                oemSock.sockReadThd = null;
            }
        }
        
        if(oemSock.socket == null) {
            return;
        }
        if(toStop) {
            try {
                oemSock.is.close();
                oemSock.is = null;
                oemSock.socket.close();
                oemSock.socket = null;
            } catch (Throwable ioe) {
//#ifdef NMSP_LOG
                if(log.isErrorEnabled())
                    log.error("Socket Close Expception - [" + ioe.getClass().getName() + "] Message - [" + ioe.getMessage() + "]");
//#endif
                postCallbackMsg(new Object[] {CALLBACK_TYPE_CLOSE, oemSock.closeCallback, NetworkStatus.NETWORK_ERROR, socket, oemSock.context});
                return;
            }
            postCallbackMsg(new Object[] {CALLBACK_TYPE_CLOSE, oemSock.closeCallback, NetworkStatus.NETWORK_OK, socket, oemSock.context});
        }
    }
    
    public NetworkStatus socketRead(Object socket,
                NetworkReadMode mode,
                byte[] buffer, int offset, int bufferLen,
                ReadSocketCallback socketCallback,
                Object context) {
        if(mode != NetworkReadMode.NETWORK_READ_FULL) {
//#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("Blackberry NetworkSystem only supports NETWORK_READ_FULL");
//#endif
            return NetworkStatus.NETWORK_ERROR;
        }
        OEMSocket oemSock = ((OEMSocket)socket);
        if(oemSock.sockReadThd != null) {
            if(!oemSock.sockReadThd.stopping) {
                SocketReadJob j = new SocketReadJob(mode, buffer, offset, bufferLen, socketCallback, context);
                return oemSock.sockReadThd.addNewJob(j);
            }
            else {
//#ifdef NMSP_LOG
                if(log.isErrorEnabled())
                    log.error("socket read thread is stopping");
//#endif
                return NetworkStatus.NETWORK_ERROR;
            }
        }
        else {
//#ifdef NMSP_LOG
            if(log.isDebugEnabled())
                log.debug("SOCKET READ ERROR: socket read thread is null");
//#endif
            return NetworkStatus.NETWORK_ERROR;
        }
    }
    
    public NetworkStatus socketWrite(Object socket, 
            byte[] buffer, int offset, int bufferLen,
            WriteSocketCallback socketCallback,
            Object context) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("socketWrite(bufferLen:" + bufferLen + ") start");
        //#endif
        
        OEMSocket oemSock = (OEMSocket)socket;
        if(oemSock.socket == null || oemSock.os == null) {
            return NetworkStatus.NETWORK_ERROR;
        }
        
        OutputStream os = oemSock.os;
        try {
            os.write(buffer, offset, bufferLen);
            os.flush();
        } catch (Exception ioe) {
//#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("Socket Write Exception - [" + ioe.getClass().getName() + "] Message - [" + ioe.getMessage() + "]");
//#endif
            postCallbackMsg(new Object[] {CALLBACK_TYPE_WRITE, socketCallback, NetworkStatus.NETWORK_ERROR, socket, buffer, new Integer(offset), new Integer(bufferLen), new Integer(0), context});
            closeSocket(socket);
            return NetworkStatus.NETWORK_ERROR;
        }
       
        postCallbackMsg(new Object[] {CALLBACK_TYPE_WRITE, socketCallback, NetworkStatus.NETWORK_OK, socket, buffer, new Integer(offset), new Integer(bufferLen), new Integer(bufferLen), context});
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("socketWrite(bufferLen:" + bufferLen + ") end");
        //#endif
        return NetworkStatus.NETWORK_OK;
    }
    
    // TODO : Fix this, network cannot always be healthy
    public boolean isNetworkHealthy() {
        return true;
    }
  
    public void clearPendingOps(Object socket) {
        OEMSocket oemSock = ((OEMSocket)socket);
        if (oemSock.sockReadThd != null) {
            oemSock.sockReadThd.cleanPendingJobs();
        } else {
//#ifdef NMSP_LOG
            if(log.isDebugEnabled())
                log.debug("SOCKET WRITE ERROR: socket read thread is null");
//#endif
            closeSocket(socket);
        }
    }	
	
    private class SocketReadJob {
        NetworkReadMode mode;
        byte[] buffer;
        int offset;
        int bufferLen;
        ReadSocketCallback socketCallback;
        Object context;
        
        public SocketReadJob(
                NetworkReadMode mode,
                byte[] buffer, int offset, int bufferLen,
                ReadSocketCallback socketCallback,
                Object context) {
            this.mode = mode;
            this.buffer = buffer;
            this.offset = offset;
            this.bufferLen = bufferLen;
            this.socketCallback = socketCallback;
            this.context = context;
        }
    }
    	
	
    private class SocketReadThread extends Thread {
        
        private String mHostName = null;
        private int mPort;
        private OpenSocketCallback openCallback = null;
        private CloseSocketCallback closeCallback = null;
        private Object context = null;
        
        //private OEMSocket socket = null;
        private NetworkStatus status = NetworkStatus.NETWORK_OK;
        private boolean stopping = false;
        private Vector queue = null;
        private SocketSetting _socketSetting = null;
        
        public SocketReadThread(String hostName, 
        						int port, 
        						SocketSetting socketSetting,
        						OpenSocketCallback openCb, 
        						CloseSocketCallback closeCb,
        						Object ctx) {
			mHostName = hostName;
			mPort = port;
			openCallback = openCb;
			closeCallback = closeCb;
			context = ctx;
			//socket = null;
			queue = new Vector();
			_socketSetting = socketSetting;
			
			// validate the proxy related parameters the upper layer needs to pass
			// the android context in order to use the device's proxy setting directly
			// the context is required to retrieve the proxy setting from the device
			if ( ( _socketSetting.proxyOption == ProxyOption.PROXY_ONLY || _socketSetting.proxyOption == ProxyOption.PROXY_IF_AVAIL )
					&& _socketSetting.context == null )
			{
				//#ifdef NMSP_LOG
				if ( log.isErrorEnabled() )
				{
					log.error("Trying to use device's proxy setting, " +
							"the upper layer needs to pass the android context object. " +
							"Check the parameter NMSP_DEFINES_ANDROID_CONTEXT.");
				}
				//#endif
			}
		}
        
        public NetworkStatus addNewJob(SocketReadJob j) {
            if(!stopping) {
                NetworkStatus st = NetworkStatus.NETWORK_OK;
                //#ifdef NMSP_LOG
                String errMsg = null;
                //#endif
                synchronized(syncObj) {
                    if (queue==null) {
                        st = NetworkStatus.NETWORK_ERROR;
                        //#ifdef NMSP_LOG
                        errMsg = "SocketReadThread: queue is null!!";
                        //#endif
                     }
                    else
                    {
                        queue.addElement(j);
                    }
                    //#ifdef NMSP_LOG
                    if(log.isDebugEnabled())
                        log.debug("addNewJob() notifying");
                    //#endif
                    syncObj.notify(); 
                }
//#ifdef NMSP_LOG
                if(log.isErrorEnabled() && errMsg != null)
                    log.error(errMsg);
//#endif
                return st;
            }
            else {
//#ifdef NMSP_LOG
                if(log.isErrorEnabled())
                    log.error("SocketReadThread is already stopping!");
//#endif
                return NetworkStatus.NETWORK_ERROR;
            }
        }
        public void stopThread() {
            synchronized(syncObj) {
                stopping = true;
                queue.removeAllElements();
                queue = null;
                syncObj.notify();
            }
                                    
        }
        public void run() {
            
            // ------------------------------------------------------------------------------
            // Open the socket
            // ------------------------------------------------------------------------------
            Socket sock = null;
            InputStream is = null;
            OutputStream os = null;

//#ifdef NMSP_LOG
            if(log.isDebugEnabled())
                log.debug("Open Socket connection on host:- [" + mHostName + "] and port[" + mPort + "]");
//#endif
            try {
            	sock = NmspSocketFactory.createSocket(_socketSetting, mHostName, mPort);
            	
            	if ( sock == null )
            	{
            		throw new IOException("socket couldn't be opened!");
            	}
                
                if (!sock.isConnected()) {
     	    	   	//#ifdef NMSP_LOG
                	if(log.isErrorEnabled())
                		log.error("SocketReadThread NETWORK_ERROR. !sock.isConnected()");
     	    	   	//#endif            	
                    postCallbackMsg(new Object[] {CALLBACK_TYPE_OPEN, openCallback, NetworkStatus.NETWORK_ERROR, null, context});
                    return;            	
                }
            } catch (UnknownHostException uhe) {
            	//#ifdef NMSP_LOG
            	                if(log.isErrorEnabled())
            	                    log.error("Open Streams Exception - [" + uhe.getClass().getName() + "] Message - [" + uhe.getMessage() + "]");
            	//#endif            	
                postCallbackMsg(new Object[] {CALLBACK_TYPE_OPEN, openCallback, NetworkStatus.NETWORK_ERROR, null, context});
                return;
            } catch (IOException ioe) {
            	//#ifdef NMSP_LOG
            	                if(log.isErrorEnabled())
            	                    log.error("Open Streams Exception - [" + ioe.getClass().getName() + "] Message - [" + ioe.getMessage() + "]");
            	//#endif            	
                postCallbackMsg(new Object[] {CALLBACK_TYPE_OPEN, openCallback, NetworkStatus.NETWORK_ERROR, null, context});
                return;
            } catch (SecurityException se) {
            	//#ifdef NMSP_LOG
            	                if(log.isErrorEnabled())
            	                    log.error("Open Streams Exception - [" + se.getClass().getName() + "] Message - [" + se.getMessage() + "]");
            	//#endif            	
                postCallbackMsg(new Object[] {CALLBACK_TYPE_OPEN, openCallback, NetworkStatus.NETWORK_SECURITY_ERROR, null, context});
                return;
            } catch (Throwable th) {
            // trying to catch everything including some internal socket failures
			// we do see EBADF and ENOTSOCK errors on some devices:
			// E/AndroidRuntime( 9713): java.lang.AssertionError: libcore.io.ErrnoException: getsockname failed: EBADF (Bad file number)
			// E/AndroidRuntime(11202): Caused by: libcore.io.ErrnoException: getsockname failed: ENOTSOCK (Socket operation on non-socket)
            	//#ifdef NMSP_LOG
            	                if(log.isErrorEnabled())
            	                    log.error("Open Streams Exception - [" + th.getClass().getName() + "] Message - [" + th.getMessage() + "]");
            	//#endif            	
                postCallbackMsg(new Object[] {CALLBACK_TYPE_OPEN, openCallback, NetworkStatus.NETWORK_ERROR, null, context});
                return;
            }

            
            try {
                sock.setTcpNoDelay(true);
                sock.setKeepAlive(true);
                sock.setSoLinger(true, 40);
            } catch (Exception e) {
            	//#ifdef NMSP_LOG
            	                if(log.isErrorEnabled())
            	                    log.error("Set Socket Option Exception - [" + e.getClass().getName() + "] Message - [" + e.getMessage() + "]");
            	//#endif            	
            }
            
            try {
                is = sock.getInputStream();
                os = sock.getOutputStream();
            } 
            catch (SSLHandshakeException she) {
                try {
                    sock.close();
                } catch (IOException ioe1) {
                }
                //#ifdef NMSP_LOG
                 if(log.isErrorEnabled())
                     log.error("Set Socket Option Exception - [" + she.getClass().getName() + "] Message - [" + she.getMessage() + "]");
                //#endif          
                postCallbackMsg(new Object[] {CALLBACK_TYPE_OPEN, openCallback, NetworkStatus.NETWORK_SECURITY_ERROR, null, context});
                return;
            }
            catch (IOException ioe) {
                try {
                    sock.close();
                } catch (IOException ioe1) {
                }
            	//#ifdef NMSP_LOG
            	                if(log.isErrorEnabled())
            	                    log.error("Set Socket Option Exception - [" + ioe.getClass().getName() + "] Message - [" + ioe.getMessage() + "]");
            	//#endif          
                postCallbackMsg(new Object[] {CALLBACK_TYPE_OPEN, openCallback, NetworkStatus.NETWORK_ERROR, null, context});
                return;
            }

                
            //#ifdef NMSP_SOCKET_HANDSHAKE
//@            try {
//@                byte[] msg = new byte[XMODE_PACKET_HEADER_SIZE];
//@                is.read(msg, 0, XMODE_PACKET_HEADER_SIZE);
//@                os.write(msg, 0, XMODE_PACKET_HEADER_SIZE);
//@                os.flush();
//@            } catch (IOException ioe) {
//@                try {
//@                    sock.close();
//@                } catch (IOException ioe1) {
//@                }
        	//#ifdef NMSP_LOG
//@        	                if(log.isErrorEnabled())
//@        	                    log.error("Set Socket Option Exception - [" + ioe.getClass().getName() + "] Message - [" + ioe.getMessage() + "]");
        	//#endif          
//@                postCallbackMsg(new Object[] {CALLBACK_TYPE_OPEN, openCallback, NetworkStatus.NETWORK_ERROR, null, context});
//@                return;
//@            }
            //#endif            
            
            OEMSocket socket = new OEMSocket(sock, closeCallback, context, is, os, this);
            postCallbackMsg(new Object[] {CALLBACK_TYPE_OPEN, openCallback, NetworkStatus.NETWORK_OK, socket, context});
            
            // ------------------------------------------------------------------------------
            // Start reading loop 
            // ------------------------------------------------------------------------------
            SocketReadJob j = null;
            try {
                while(!stopping) {
                    synchronized(syncObj) {
                        while (!stopping && queue != null && queue.isEmpty() ) {
                            //#ifdef NMSP_LOG
                            if(log.isDebugEnabled())
                                log.debug("socket read thread. waiting syncObj in.");
                            //#endif
                            try {
                                syncObj.wait();
                            } catch(Exception ie) {
//#ifdef NMSP_LOG
                                if(log.isErrorEnabled())
                                    log.error("InterruptedException ie:" + ie);
//#endif
                            }
                            //#ifdef NMSP_LOG
                            if(log.isDebugEnabled())
                                log.debug("socket read thread. waiting syncObj out.");
                            //#endif
                        }
                        if(stopping || queue == null) break;
                        j = (SocketReadJob) queue.elementAt(0);
                        queue.removeElementAt(0);
                    }
                
                    try {
                        //#ifdef NMSP_LOG
                        if(log.isDebugEnabled())
                            log.debug("socket read thread. reading socket in.");
                        //#endif
                        int totalBytesRead = 0;
                        while(totalBytesRead < j.bufferLen) {
                            int bytesRead = is.read(j.buffer, j.offset + totalBytesRead, j.bufferLen - totalBytesRead);
                            //#ifdef NMSP_LOG
                            if(log.isDebugEnabled())
                                log.debug("socket read thread. bytesRead:" + bytesRead);
                            //#endif
                            if(bytesRead < 0) {
                            	
                            	//if socket is not closing, send back NETWORK_ERROR
                            	if (!stopping) 
                                status = NetworkStatus.NETWORK_ERROR;
                            	
                                break;
                            }
                            else {
                                totalBytesRead += bytesRead;
                            }
                        }
                        //#ifdef NMSP_LOG
                        if(log.isDebugEnabled())
                            log.debug("socket read thread. reading socket out.");
                        //#endif
                    } catch (IOException ioe) {
//#ifdef NMSP_LOG
                        if(log.isErrorEnabled())
                            log.error("Socket Read/Available Expception - [" + ioe.getClass().getName() + "] Message - [" + ioe.getMessage() + "]");
//#endif
                        status = NetworkStatus.NETWORK_ERROR;
                    }
                  
                    if(status == NetworkStatus.NETWORK_OK) {
                        postCallbackMsg(new Object[] {CALLBACK_TYPE_READ, j.socketCallback, NetworkStatus.NETWORK_OK, socket, j.buffer, new Integer(j.offset), new Integer(j.bufferLen), new Integer(j.bufferLen), j.context});
                    } else {
                        postCallbackMsg(new Object[] {CALLBACK_TYPE_READ, j.socketCallback, NetworkStatus.NETWORK_ERROR, socket, j.buffer, new Integer(j.offset), new Integer(j.bufferLen), new Integer(0), j.context});
                        closeSocket(socket);
                    }
                }
            } catch (Exception e) {
//#ifdef NMSP_LOG
                if(log.isErrorEnabled())
                    log.error("NetworkSystemJ2me.run() " + e.getClass().getName() + " " + e.getMessage());
//#endif
            }
        }
        
        public void cleanPendingJobs() {
            //#ifdef NMSP_OEM_NETWORK_EXTRA_EVENTLOG
//@            if(log.isErrorEnabled())
//@                log.error("[NOT ERROR] [cleanPendingJobs() start]");
            //#endif
            //#ifdef NMSP_LOG
            String errMsg = null;
            //#endif
            synchronized(syncObj) {
                if (queue == null) {
                  //#ifdef NMSP_LOG
                    errMsg = "SocketReadThread.cleanPendingJobs(): queue is null!!";
                  //#endif
                } else {
                    queue.removeAllElements();
                }
            }

//#ifdef NMSP_LOG
            if(log.isErrorEnabled() && errMsg != null)
                log.error(errMsg);
//#endif
        }
    }
	
	public static void terminate()
	{
	}
}
