package com.nuance.nmsp.client.sdk.oem.bluetooth;

import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client.sdk.common.oem.api.LogFactory.Log;
import com.nuance.nmsp.client.sdk.oem.bluetooth.BluetoothHeadset.ServiceListener;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;

/**
 * Abstract base class for the Bluetooth functionality.
 * Derived classes are using platform specific Bluetooth APIs
 * for implementing functionality defined by this class. 
 */
public abstract class Bluetooth {
    //#ifdef NMSP_LOG    
    private final static Log log = LogFactory.getLog(Bluetooth.class);
    //#endif    

    protected Context mContext;
    protected BluetoothHeadsetOEM mHeadset;
    private Object mHeadsetScoSyncObj = new Object();

    public static Bluetooth createInstance(Context context) {
        int sdk = AndroidVersion.SDK;
        if ( (sdk < AndroidVersion.FROYO) || (AndroidVersion.IS_BROKEN_HTC)) {
            return new Bluetooth_2_0(context);
        } else if (sdk < AndroidVersion.GINGERBREAD){
            return new Bluetooth_2_2(context);
        } else {
            return new BluetoothImpl_2_3(context);
        }
    }

    protected Bluetooth(Context context) {
        mContext = context;
        mHeadset = new BluetoothHeadsetOEM(mContext);
    }
    
    public void startBluetoothSco()
    {
        IntentFilter filter = new IntentFilter(BluetoothHeadsetOEM.ACTION_AUDIO_STATE_CHANGED);
        BroadcastReceiver receiver = new BroadcastReceiver()
        {
            @Override
            public void onReceive(Context context, Intent intent)
            {
                int value = intent.getIntExtra(BluetoothHeadsetOEM.EXTRA_AUDIO_STATE, -1);
                //#ifdef NMSP_LOG
                if(log.isInfoEnabled())
                    log.info("BluetoothHeadset BroadcastReceiver " + value);
                //#endif
                synchronized(mHeadsetScoSyncObj)
                {
                    if (value == BluetoothHeadsetOEM.AUDIO_STATE_CONNECTED)
                    {
                        //#ifdef NMSP_LOG
                        if(log.isInfoEnabled())
                            log.info("BluetoothHeadset SCO connected. Notify wait lock.");
                        //#endif
                        mHeadsetScoSyncObj.notify();
                    }
                }
            }
        };
        synchronized(mHeadsetScoSyncObj)
        {
            mContext.registerReceiver(receiver, filter);
            long startTime = System.currentTimeMillis();
            startBluetoothScoInternal();
            try
            {
                mHeadsetScoSyncObj.wait(3000);
            } catch (InterruptedException e)
            {
                //#ifdef NMSP_LOG
                if(log.isErrorEnabled())
                    log.error("BluetoothHeadset mHeadsetScoSyncObj.wait() threw exception:" + e);
                //#endif
            }
            //#ifdef NMSP_LOG
            if(log.isInfoEnabled())
                log.info("BluetoothHeadset " + (System.currentTimeMillis() - startTime) + "ms to wait for SCO");
            //#endif
        }
        mContext.unregisterReceiver(receiver);
    }
    
    public boolean isHeadsetConnected() {
        boolean headsetConnected = mHeadset.getConnectedDevice() != null;
        return headsetConnected;
    }

    public void close() {
        if(mHeadset != null) {
            try {
                mHeadset.close();
            } catch(Throwable t) {
            }
            mHeadset = null;
        }
    }
    
    public abstract void startBluetoothScoInternal();

    public abstract void stopBluetoothSco();

    public abstract int getPlaybackStream();

    public abstract int getRecordingSource();
}
