/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/
package com.nuance.nmsp.client.sdk.oem.socket.ssl;

import java.io.IOException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.security.GeneralSecurityException;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.Provider;
import java.security.Security;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;

public class NmspSSLSocketFactory
{
	private static final String PROTOCOL = "TLS";
	
	// push OpenSSL to the end of Security preference list to have minimum expose to OpenSSL
//	static {
//	    Provider[] providers = Security.getProviders();
//        for (Provider p : providers)
//        {
//            String name = p.getName();
//            if (name.toLowerCase().contains("openssl"))
//            {
//                Security.removeProvider(name);
//                Security.addProvider(p);
//                break;
//            }
//        }
//    }
	
	private static SSLSocketFactory getSocketFactory(SSLSettings sslSettings) throws SecurityException
	{
		try
		{
			final TrustManager[] trustManager = new TrustManager[] { new NmspX509TrustManager(sslSettings) };
			final SSLContext context = SSLContext.getInstance(PROTOCOL);
			context.init(null, trustManager, null);

			return context.getSocketFactory();
		}
		catch (NoSuchAlgorithmException e)
		{
			throw new SecurityException( "No such algorithm exception " + e );
		}
		catch (KeyManagementException e)
		{
			throw new SecurityException( "Failed to initialize the client-side SSLContext " + e );
		}
		catch (GeneralSecurityException e)
		{
			throw new SecurityException( "General security exception " + e );
		}
	}

	public static Socket createSocket(String hostName, int port, SSLSettings sslSettings) throws UnknownHostException, IOException, SecurityException
	{
		return getSocketFactory(sslSettings).createSocket(hostName, port);
	}
	
	public static Socket createSocket(Socket proxySocket, String hostName, int port, SSLSettings sslSettings) throws UnknownHostException, IOException, SecurityException
	{
		return getSocketFactory(sslSettings).createSocket(proxySocket, hostName, port, true);
	}
}
