/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client.sdk.oem.socket.ssl;

import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.X509TrustManager;

import com.nuance.nmsp.client.sdk.common.util.Base64;

public class NmspX509TrustManager implements X509TrustManager {
	private SSLSettings sslSettings;
    private static X509TrustManager defaultManager;

    public NmspX509TrustManager(SSLSettings sslSettings) throws GeneralSecurityException
	{
		this.sslSettings = sslSettings;

		TrustManagerFactory trustManagerFactory = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
		trustManagerFactory.init((KeyStore)null);
		defaultManager = (X509TrustManager) trustManagerFactory.getTrustManagers()[0];
	}

	public void checkClientTrusted(X509Certificate[] chain, String authType)
			throws CertificateException {
		//not implemented
	}

	public void checkServerTrusted(X509Certificate[] certs, String authType)
			throws CertificateException {

		try {
			defaultManager.checkServerTrusted(certs, authType);
		}
		catch (CertificateException exception){
			if (!sslSettings.enableSelfSignedCert) {
				throw exception;
			}
		}
		

		if (sslSettings.certSummary != null) {
			boolean summaryVerified = false;
			for (int i = 0; i < certs.length; ++ i) {
				X509Certificate cert = certs[i];

				String subject = cert.getSubjectDN().getName();
				int start = subject.indexOf("CN=");
                if (start != -1) {
                    start += 3;
                    
                    String summary = null;
                    int end = subject.indexOf(',', start);
                    if (end == -1) {
                        summary = subject.substring(start);
                    }
                    else {
                        summary = subject.substring(start, end);
                    }
                    
                    if (sslSettings.certSummary.equals(summary)) {
                        summaryVerified = true;
                        break;
                    }
                }
			}
			
			if (!summaryVerified) {
				throw new CertificateException("certificate summary is not identical");
			}
		}

		if (sslSettings.certData != null) {
			boolean certDataVerified = false;
			for (int i = 0; i < certs.length; ++ i) {
				X509Certificate cert = certs[i];

				String encodedData = Base64.encode(cert.getEncoded());
				if (sslSettings.certData.equals(encodedData)) {
					certDataVerified = true;
                    break;
				}	
			}
			
			if (!certDataVerified) {
				throw new CertificateException("certificate data is not identical");
			}
		}
	}

	public X509Certificate[] getAcceptedIssuers() {
		return defaultManager.getAcceptedIssuers();
	}
}
