/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client2.sdk.common.defines;

import com.nuance.nmsp.client2.sdk.common.util.Enum;
import com.nuance.nmsp.client2.sdk.common.util.ShortConstant;


/**
 * Provides common definitions that
 * are shared by both the third-party application and the SDK.
 *
 * </br></br>
 * <strong><em>Note:</em></strong> Currently, recording in AMR7 is supported only on BlackBerry devices. 
 **/
public class NMSPDefines {

    private static final boolean ANDROID =
//#ifdef NMSP_PLATFORM_ANDROID
        true;
//#else
//@        false;
//#endif

    private static final boolean BLACKBERRY =
//#ifdef NMSP_PLATFORM_BLACKBERRY
//@        true;
//#else
        false;
//#endif

    private static final boolean J2SE =
//#ifdef NMSP_PLATFORM_J2SE
//@        true;
//#else
        false;
//#endif


    /**
     * <code>NMSPDefines.Codec</code> provides a complete list of codec definitions.
     * (See the Release Notes for currently supported platforms.)
     * </br></br>
     * <strong><em>Note:</em></strong>
     * Not all codecs are supported on all platforms. For example, recording in AMR7
     * is supported only on BlackBerry devices currently.
     *
     **/
    public static class Codec extends ShortConstant {
        private Codec(short value, boolean supported) {
            super(supported ? value : 0);
        }

        /**
         * Determine whether this codec is supported on the current configuration.
         * </br></br>
         * @return true if the codec is supported, false if it is not.
         *
         **/
        public final boolean isSupported() {
            return value != 0;
        }

//if !NMSP_PLATFORM_ANDROID        
        public static final Codec AMR_00 = new Codec((short) 0x0050, !ANDROID);
        public static final Codec AMR_01 = new Codec((short) 0x0051, !ANDROID);
        public static final Codec AMR_02 = new Codec((short) 0x0052, !ANDROID);
        public static final Codec AMR_03 = new Codec((short) 0x0053, !ANDROID);
        public static final Codec AMR_04 = new Codec((short) 0x0054, !ANDROID);
        public static final Codec AMR_05 = new Codec((short) 0x0055, !ANDROID);
        public static final Codec AMR_06 = new Codec((short) 0x0056, !ANDROID);
        public static final Codec AMR_07 = new Codec((short) 0x0057, !ANDROID);
        
        public static final Codec ULAW   = new Codec((short) 0x0008, !ANDROID);

        public static final Codec MP3    = new Codec((short) 0x0260, J2SE || ANDROID || BLACKBERRY);

        public static final Codec MP3_8KBPS    = new Codec((short) 0x0261, J2SE || ANDROID || BLACKBERRY);
        public static final Codec MP3_16KBPS    = new Codec((short) 0x0262, J2SE || ANDROID || BLACKBERRY);
        public static final Codec MP3_24KBPS    = new Codec((short) 0x0263, J2SE || ANDROID || BLACKBERRY);
        public static final Codec MP3_32KBPS    = new Codec((short) 0x0264, J2SE || ANDROID || BLACKBERRY);
        public static final Codec MP3_40KBPS    = new Codec((short) 0x0265, J2SE || ANDROID || BLACKBERRY);
        public static final Codec MP3_48KBPS    = new Codec((short) 0x0266, J2SE || ANDROID || BLACKBERRY);
        public static final Codec MP3_56KBPS    = new Codec((short) 0x0267, J2SE || ANDROID || BLACKBERRY);
        public static final Codec MP3_64KBPS    = new Codec((short) 0x0268, J2SE || ANDROID || BLACKBERRY);
        public static final Codec MP3_80KBPS    = new Codec((short) 0x0269, J2SE || ANDROID || BLACKBERRY);
        public static final Codec MP3_96KBPS    = new Codec((short) 0x026A, J2SE || ANDROID || BLACKBERRY);
        public static final Codec MP3_112KBPS    = new Codec((short) 0x026B, J2SE || ANDROID || BLACKBERRY);
        public static final Codec MP3_128KBPS    = new Codec((short) 0x026C, J2SE || ANDROID || BLACKBERRY);
        public static final Codec MP3_144KBPS    = new Codec((short) 0x026D, J2SE || ANDROID || BLACKBERRY);
        public static final Codec MP3_160KBPS    = new Codec((short) 0x026E, J2SE || ANDROID || BLACKBERRY);



        public static final Codec MS_GSM_FR = new Codec((short) 0x0290, J2SE || ANDROID);
        public static final Codec GSM_FR = new Codec((short) 0x0291, J2SE || ANDROID);
//endif
        
//if NMSP_PLATFORM_J2SE
//        public static final Codec DSR_ES201108 = new Codec((short) 0x0030);
//        public static final Codec DSR_ES201108_11K = new Codec((short) 0x0031);
        
//        public static final Codec G729_R0 = new Codec((short) 0x0040);
//        public static final Codec G729_R1 = new Codec((short) 0x0041);
//        public static final Codec G729_R2 = new Codec((short) 0x0042);
//        
        public static final Codec AMR_NBO_R0 = new Codec((short) 0x0080, J2SE);
        public static final Codec AMR_NBO_R1 = new Codec((short) 0x0081, J2SE);
        public static final Codec AMR_NBO_R2 = new Codec((short) 0x0082, J2SE);
        public static final Codec AMR_NBO_R3 = new Codec((short) 0x0083, J2SE);
        public static final Codec AMR_NBO_R4 = new Codec((short) 0x0084, J2SE);
        public static final Codec AMR_NBO_R5 = new Codec((short) 0x0085, J2SE);
        public static final Codec AMR_NBO_R6 = new Codec((short) 0x0086, J2SE);
        public static final Codec AMR_NBO_R7 = new Codec((short) 0x0087, J2SE);
        
        public static final Codec QCELP13 = new Codec((short) 0x0062, J2SE);
        /**
         * Not supported
         */
        public static final Codec QCELPVar = new Codec((short) 0x0063, J2SE);
        
        public static final Codec EVRC = new Codec((short) 0x0070, J2SE);
        
        /**
         * Not supported
         */
        public static final Codec ILBC_R0 = new Codec((short) 0x0090, J2SE);
        /**
         * Not supported
         */
        public static final Codec ILBC_R1 = new Codec((short) 0x0091, J2SE);
//endif
//if NMSP_PLATFORM_J2SE || NMSP_PLATFORM_ANDROID        
        public static final Codec SPEEX_8K = new Codec((short) 0x0106, J2SE || ANDROID);
        public static final Codec SPEEX_16K = new Codec((short) 0x0116, J2SE || ANDROID);
        public static final Codec SPEEX_11K = new Codec((short) 0x0168, J2SE || ANDROID);
        
        public static final Codec OPUS_8K = new Codec((short) 0x040A, J2SE || ANDROID);
        public static final Codec OPUS_16K = new Codec((short) 0x0462, J2SE || ANDROID);
//endif
 
        /**
         * Bit rate:          16x8 bps;
         * Audio sample rate: 8 KHz;
         * Channels:          1 (mono);
         * Audio format:     PCM
         **/
        public static final Codec PCM_16_8K  = new Codec((short) 0x0002, true); /* 8 KHz PCM format */
//if !NMSP_PLATFORM_BLACKBERRY
        /**
         * Bit rate:          16x11 bps;
         * Audio sample rate: 11 KHz;
         * Channels:          1 (mono);
         * Audio format:     PCM 
         */ 
        public static final Codec PCM_16_11K = new Codec((short) 0x0004, !BLACKBERRY); /* 16 KHz PCM format */
//endif

//if NMSP_PLATFORM_J2SE || NMSP_PLATFORM_ANDROID
//        public static final Codec ALAW = new Codec((short) 0x0012);
        
        public static final Codec PCM_16_16K = new Codec((short) 0x0005, J2SE || ANDROID);
        public static final Codec PCM_16_32K = new Codec((short) 0x0006, J2SE || ANDROID);
        public static final Codec PCM_16_22K = new Codec((short) 0x0007, J2SE || ANDROID);
        
//endif
    }
    

    // SDK Parameters
    /**
     * Specifies the maximum time, in seconds, starting from the creation of the 
     * Command until the connection to the dictation server is established. 
     * Can be used as a key for the properties list used to create an NMSPManager object. 
     * <br><br>
     * The value must be an integer greater than 0.<br>
     * Default = 30 seconds
     *  
     **/
    public static final String ConnectionTimeout = "ConnectionTimeout";
    
    /**
     * Specifies the maximum time, in seconds, that a session can remain idle.
     * A session is considered to be idle when no messages are either sent to 
     * or received by the NMSP Gateway from the mobile device.
     * Can be used as a key for the properties list used to create an NMSPManager object.
     * <br /><br />
     * The value must be an integer greater than 0. <br />
     * Default = 45 seconds
     **/
    public static final String IdleSessionTimeout = "IdleSessionTimeout";
    
//ifdef NMSP_PLATFORM_BLACKBERRY
    /**
     * The Access Point Name (APN) string used to establish a socket connection
     * on BlackBerry devices. Can be used as a key for the properties list used to create 
     * an NMSPPDXManager object.
     * <br />
     * The value must be a <code>String</code>.
     **/
    public static final String SocketConnectionSetting = "SocketConnectionSetting";
//endif
    
    /** 
     * Defines whether the recorder should continue recording on end-pointer end-of-speech.
    **/
    public static final String NMSP_DEFINES_RECORDER_CONTINUES_ON_ENDPOINTER_AND_TIMER_STOPPING = "NMSP_DEFINES_RECORDER_CONTINUES_ON_ENDPOINTER_AND_TIMER_STOPPING";

    /** 
     * Defines whether the capturing should continue recording on end-pointer end-of-speech.
    **/
    public static final String NMSP_DEFINES_CAPTURING_CONTINUES_ON_ENDPOINTER = "NMSP_DEFINES_CAPTURING_CONTINUES_ON_ENDPOINTER";

    /** 
     * Defines the energy level parameter, which specifies if a notification must be sent when 
     * a change in the audio energy level occurs.
     */
    public static final String NMSP_DEFINES_USE_ENERGY_LEVEL	 = "USE_ENERGY_LEVEL";
    
    
    /**
     * Allows the client to pass to the server information about the status of the
     * previous transaction so that the server can add it to a database and report
     * on the pass rate from the client's point of view. This is specific to the <code>NMASResource</code>.
     **/
    public static final String DEVICE_CMD_LOG_TO_SERVER_ENABLED = "DEVICE_CMD_LOG_TO_SERVER_ENABLED";
    
    /**
     * Allows users to specify the input audio source.
     **/
    public static final String NMSP_DEFINES_AUDIO_INPUTSOURCE = "Audio_Source";
    
    
    // End Pointing
    /**
     * Enables the end pointer.
     **/
    public static final String NMSP_DEFINES_ENDPOINTER = "ep.enable";

    /**
     * Endpointer fine-tuning parameter.
     **/
    public static final String NMSP_DEFINES_ENDPOINTER_SPEEX_VAD_LONG_UTTERANCE = "ep.VadLongUtterance";
    
    /**
     * Endpointer fine-tuning parameter.
     **/
    public static final String NMSP_DEFINES_ENDPOINTER_SPEEX_VAD_BEGIN_DELAY    = "ep.VadBeginDelay";

    /**
     * Endpointer fine-tuning parameter.
     **/
    public static final String NMSP_DEFINES_ENDPOINTER_SPEEX_VAD_HISTORY_LENGTH = "ep.VadHistoryLength";
    
    /**
     * Endpointer fine-tuning parameter.
     **/
    public static final String NMSP_DEFINES_ENDPOINTER_SPEEX_VAD_END_THRESHOLD 	= "ep.VadEndThreshold";
    
    /**
     * Endpointer fine-tuning parameter.
     **/
    public static final String NMSP_DEFINES_ENDPOINTER_SPEEX_VAD_END_LENGTH 	= "ep.VadEndLength";
    
    /**
     * AMR endpointer fine-tuning parameter.
     **/
    public static final String NMSP_DEFINES_ENDPOINTER_AMR_MEDIAN_ORDER       = "ep.amr.MEDIAN_ORDER";
    
    /**
     * AMR endpointer fine-tuning parameter.
     **/
    public static final String NMSP_DEFINES_ENDPOINTER_AMR_VOWEL_THRESHOLD    = "ep.amr.VOWEL_THRESHOLD";
    
    /**
     * AMR endpointer fine-tuning parameter.
     **/
    public static final String NMSP_DEFINES_ENDPOINTER_AMR_POST_VOWEL_TIMEOUT = "ep.amr.POST_VOWEL_TIMEOUT";
    
    /**
     * AMR endpointer fine-tuning parameter.
     **/
    public static final String NMSP_DEFINES_ENDPOINTER_AMR_PRE_VOWEL_TIMEOUT  = "ep.amr.PRE_VOWEL_TIMEOUT";
    
    /**
     * AMR endpointer fine-tuning parameter.
     **/
    public static final String NMSP_DEFINES_ENDPOINTER_AMR_BOS_BACKOFF        = "ep.amr.BOS_BACKOFF";
    
    /**
     * AMR endpointer fine-tuning parameter.
     **/
    public static final String NMSP_DEFINES_ENDPOINTER_AMR_EOS_BACKOFF        = "ep.amr.EOS_BACKOFF";
	
    /**
	 * Utterance detection fine tuning parameter
	 **/
	public static final String NMSP_DEFINES_ENDPOINTER_UTTERANCE_SILENCE_DURATION = "utterance_detection_silence_duration";

    // SSL settings
    /**
     * Enables the SSL connection.
     **/
    public static final String NMSP_DEFINES_SSL_SOCKET          = "SSL_Socket_Enable";
    
    /**
     * Presets the SSL certificate summary, which will be compared with the one replied by the server for verification.
     **/
    public static final String NMSP_DEFINES_SSL_CERT_SUMMARY    = "SSL_Cert_Summary";
    
    /**
     * Presets the SSL certificate data, which will be compared with the data replied by the server for verification.
     **/
    public static final String NMSP_DEFINES_SSL_CERT_DATA       = "SSL_Cert_Data";
    
    /**
     * Lets the server use its self-signed certificate for development purposes.
     **/
    public static final String NMSP_DEFINES_SSL_SELFSIGNED_CERT = "SSL_SelfSigned_Cert";

    //#ifdef NMSP_PLATFORM_ANDROID
    /**
     * This is an android specific feature, extract the device Proxy setting and use it to open
     * the network connection. Once this parameter is enabled, one has to pass in the NMSP_DEFINES_ANDROID_CONTEXT
     * parameter as well.
     * 
     * Once this parameter is enabled, the SDK will only connect via the existing proxy server. If there is no
     * proxy server or the proxy server does not work properly, the connect attempt will fail.
     */
    public static final String NMSP_DEFINES_USING_DEVICE_PROXY_SETTING_ONLY = "Use_Device_ProxySetting_Only";
    
    /**
     * This is an android specific feature, extract the device Proxy setting and use it to open
     * the network connection. Once this parameter is enabled, one has to pass in the NMSP_DEFINES_ANDROID_CONTEXT
     * parameter as well.
     * 
     * Once this parameter is enabled, the SDK will first try to connect via the existing proxy server. If there is no
     * proxy server, the SDK will fall back to open the connection to the server IP directly.
     * 
     * If both PROXY_SETTING_IF_AVAILABLE and PROXY_SETTING_ONLY are set to true, the PROXY_SETTING_ONLY will be used. 
     */
    public static final String NMSP_DEFINES_USING_DEVICE_PROXY_SETTING_IF_AVAILABLE = "Use_Device_ProxySetting_If_Available";
    //#endif
    
    /**
     * Explicitly disable Bluetooth.
     **/
    public static final String NMSP_DEFINES_DISABLE_BLUETOOTH = "Disable_Bluetooth";

//#ifdef NMSP_PLATFORM_ANDROID
    /**
     * Used to pass in the Android Context object.
     **/
    public static final String NMSP_DEFINES_ANDROID_CONTEXT = "Android_Context";
//#endif
    
    /**
     * Allows users to disable call logging by setting this parameter to "TRUE" or "True", it is enabled by default. 
     */
    public static final String NMSP_DEFINES_CALLLOG_DISABLE = "Calllog_Disable";
    
    /**
     * Allows users to pass an existing UUID as the root parent id. This ID has to be created by the NMSP SDK as a remote
     * reference ID. A typical use case is to relate the call logs generated by one NMSP SDK application A and call logs
     * generated by another NMPS SDK application B. The application B could be launched by the application A and it should
     * use a remote reference ID created by application A as its root parent ID.
     */
    public static final String NMSP_DEFINES_CALLLOG_ROOT_ID = "Calllog_Root_Id";
    
    /**
     * Specifies the suggested number of days for preserving the call log data in the cloud. The effective data retention
     * period is subject to a number of conditions on the server side and may be automatically adjusted to a higher or
     * lower value than the specified one.
     * 
     * The value of 1 indicates no specific value for the data retention and the server default value will be used. Other
     * negative values or 0 is not allowed. The default value is -1.
     */ 
    public static final String NMSP_DEFINES_CALLLOG_RETENTION_DAYS = "Calllog_Retention_Days";

    /**
     * Allows users to control the call log generation call back rate based on the chunk size. The call log data callback
     * will be called with a call log data buffer once the internal logs accumulated acceding the chunk size. The chunk
     * size is 1M by default. The minimum chunk size is 20K (1000x20), passing a parameter smaller than 20K will cause
     * the manager creation failed.
	 *
	 * The default buffer size is 100k (1000x100) and the maximum chunk size is 150K (1000x150).
	 * 
	 * \see {@link com.nuance.nmsp.client.sdk.components.resource.common.ManagerListener#callLogDataGenerated(byte[], List)}
     */
    public static final String NMSP_DEFINES_CALLLOG_CHUNK_SIZE = "Calllog_Chunk_Size";
    
    /**
     * This is the event name the application needs to use to create the root node of the transaction information log tree.
     * Before starting each transaction, the application needs to call logAppEvent function with this name, commit that
     * SessionEventBuilder right away. And then get the ID for that committed call log event. Once the application is ready
     * to start a transaction (create a NMAS command for a recognition, etc) The application needs to pass the following key
     * value pair {NMSP_DEFINES_CALLLOG_APP_TRANSACTION_REF_EVENT, ID} as one element in the optional key parameter
     * Refer to the following function: nmsp_nmas_command_create.
     */
    public static final String NMSP_DEFINES_CALLLOG_APP_TRANSACTION_REF_EVENT = "APP_TRANSACTION_REF";
    
    /**
     * Allows the user to control the start of speech timeout value. If the user starts a recognition but does not speak for
     * a duration longer than this value, the recognition will stop. This value must be specified in milliseconds.
     */
    public static final String NMSP_DEFINES_START_OF_SPEECH_TIMEOUT = "Start_Of_Speech_Timeout";

    /**
     * Allows the user to set the complexity level to the OPUS encoder. If the user does not specify this parameter, SDK
     * will use 10 as the default complexity value for OPUS encoding. 
     * Opus Complexity type integer, range [1, 10]
     */

    public static final String NMSP_DEFINES_OPUS_COMPLEXITY = "Opus_Complexity";
    
    /**
     * Allows the user to enable the VBR (variable bit rate) feature to the OPUS encoder. If the user does not specify this
     * parameter, SDK will use CBR (constant bit rate) by default. 
     * Opus Variable Bit Rate turned on, type integer, value 1, by default it is CBR
     */
    public static final String NMSP_DEFINES_OPUS_VBR_ENABLED = "Opus_Enable_VBR";
    
    /**
     * Allows the user to set the bit rate value to the OPUS encoder. If the user does not specify this parameter, SDK will
     * use 28kbps by default. 
     * Opus Bit rate, type integer, range [14k, 32k], by default it is 28Kbps
     */
    
    public static final String NMSP_DEFINES_OPUS_BITRATE = "Opus_Bitrate";
    /**
     * Allows the user to set the packet length value to the OPUS encoder. If the user does not specify this parameter, SDK
     * will use 20ms by default. 
     * Opus Packet Length, type integer, value {20, 40, 60}, by default it is 20
     */
    public static final String NMSP_DEFINES_OPUS_PACKET_LENGTH = "Opus_Packet_Length";   
    
    /**
     * Allows the client to send and receive Mds messages.
     */
    public static final String NMSP_DEFINES_ALLOW_MDS_MESSAGES = "connecttomds";
    
    
    
    
//ifndef NMSP_PLATFORM_J2SE
    public static class AudioSystem {
        /**
         * Input device types.
         *
         **/
        public static class InputDevice extends Enum {
            private static final int ID_MICROPHONE        = 0;
            private static final int ID_NETWORK           = 1;
            private static final int ID_BLUETOOTH_HEADSET = 2;
            private static final int ID_WIRED_HEADSET     = 3;
            private InputDevice(int id) { super(id); }
            
            public static InputDevice MICROPHONE        = new InputDevice(InputDevice.ID_MICROPHONE);        /*!< Record from microphone */
            public static InputDevice NETWORK           = new InputDevice(InputDevice.ID_NETWORK);           /*!< Record from network (i.e. from a phone call) */
            public static InputDevice BLUETOOTH_HEADSET = new InputDevice(InputDevice.ID_BLUETOOTH_HEADSET); /*!< Record from Bluetooth headset */
            public static InputDevice WIRED_HEADSET     = new InputDevice(InputDevice.ID_WIRED_HEADSET);     /*!< Record from wired headset */
        }
        
        /**
         * Output device types.
         *
         **/
        public static class OutputDevice extends Enum {
            private static final int ID_SPEAKER           = 0;
            private static final int ID_NETWORK           = 1;
            private static final int ID_BLUETOOTH_HEADSET = 2;
            private static final int ID_BLUETOOTH_A2DP    = 3;
            private static final int ID_WIRED_HEADSET     = 4;
            private OutputDevice(int id) { super(id); }
            
            public static OutputDevice SPEAKER           = new OutputDevice(OutputDevice.ID_SPEAKER);           /*!< Playback to speaker */
            public static OutputDevice NETWORK           = new OutputDevice(OutputDevice.ID_NETWORK);           /*!< Playback to network (i.e. to a phone call) */
            public static OutputDevice BLUETOOTH_HEADSET = new OutputDevice(OutputDevice.ID_BLUETOOTH_HEADSET); /*!< Playback to Bluetooth headset */
            public static OutputDevice BLUETOOTH_A2DP    = new OutputDevice(OutputDevice.ID_BLUETOOTH_A2DP);    /*!< Playback to Bluetooth A2DP device */
            public static OutputDevice WIRED_HEADSET     = new OutputDevice(OutputDevice.ID_WIRED_HEADSET);     /*!< Playback to wired headset */
        }
    }
//endif

    /**
     * Version string
     **/
	public static final String VERSION = "NMSP 6.0 client SDK - build 020";
}
