/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client2.sdk.common.oem.api;

import java.util.Vector;

public interface NetworkSystem {
    //#ifdef NMSP_CPL_ENABLED
//@    public static class CPLSocketReadContext {    
//@        public String normalContext = "";
//@        public Vector multipleReads = new Vector();
//@        public Vector multipleAvails = new Vector();
//@    }
//@    public static class CPLSocketReadRecord {
//@        public long cplBeforeSocketReadTime = 0;
//@        public long cplAfterSockeReadTime = 0;
//@        public long cplBeforeCheckSocketReadAvailableTime = 0;
//@        public long cplAfterCheckSocketReadAvailableTime = 0;
//@        public short bytesRead = 0;
//@        public short cplAvailableBytes = 0;
//@        public short cplInternalAvaBytes = 0;
//@        public String signalStrengthLevel = null;
//@    }
//@    public static class CPLSocketWriteContext {
//@        public long cplBeforeSocketWriteTime = 0;
//@        public long cplAfterSocketWriteTime = 0;
//@        public short bytesWritten = 0;
//@        public String normalContext = "";
//@        public String signalStrengthLevel = null;
//@    }
    //#endif
    static class NetworkStatus {
        private NetworkStatus() {}
        public static final NetworkStatus NETWORK_OK             = new NetworkStatus();   /*!< Requested operation succeeded */
        public static final NetworkStatus NETWORK_ERROR          = new NetworkStatus();   /*!< Requested operation failed */
        public static final NetworkStatus NETWORK_MEMORY_ERROR   = new NetworkStatus();   /*!< Requested operation failed due to memory error */
        public static final NetworkStatus NETWORK_SECURITY_ERROR = new NetworkStatus();   /*!< Requested operation failed due to SSL security error */
    }
    static class NetworkReadMode {
        private NetworkReadMode() {}
        public static final NetworkReadMode NETWORK_READ_PARTIAL = new NetworkReadMode(); /*!< Return first chunk of received data */
        public static final NetworkReadMode NETWORK_READ_FULL    = new NetworkReadMode(); /*!< Return only when buffer is full */
    }
    
    interface OpenSocketCallback {
        void openSocketCallback(NetworkStatus status, Object socketStruct, Object context);
    }
    interface CloseSocketCallback {
        void closeSocketCallback(NetworkStatus status, Object socketStruct, Object context);
    }
    interface ReadSocketCallback {
        void readSocketCallback(NetworkStatus status, Object socketStruct, byte[] buffer, int offset, int bufferLen, int bytesRead, Object context);
    }
    interface WriteSocketCallback {
        void writeSocketCallback(NetworkStatus status, Object socketStruct, byte[] buffer, int offset, int bufferLen, int bytesWritten, Object context);
    }
    
    void openSocket(String hostName,
    		        int port,
    		        Vector socketSettings,
                    OpenSocketCallback openCallback,
                    CloseSocketCallback closeCallback,
                    Object context);

    void openSSLSocket(String hostName, int port,
    		Vector sslSettings,
            OpenSocketCallback openCallback,
            CloseSocketCallback closeCallback,
            Object context);
    
    void setSessionId(Object socketStruct, byte[] sessionId);
    
    void closeSocket(Object socketStruct);
    
    boolean isNetworkHealthy();
    
    void clearPendingOps(Object socketStruct);
    
    NetworkStatus socketRead(Object socketStruct,
            NetworkReadMode mode,
            byte[] buffer, int offset, int readLen,
            ReadSocketCallback socketCallback,
            Object context);
    
    NetworkStatus socketWrite(Object socketStruct,
                              byte[] buffer, int offset, int writeLen,
                              WriteSocketCallback socketCallback,
                              Object context);
}
