/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client2.sdk.components.core;

//#ifdef NMSP_SOCKET_WRITE_PADDING
//@import java.io.ByteArrayOutputStream;
//#endif
import java.io.UnsupportedEncodingException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;
import java.util.Vector;

import com.nuance.nmsp.client2.sdk.common.defines.NMSPDefines;
//#ifdef NMSP_LOG
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory.Log;
//#endif
import com.nuance.nmsp.client2.sdk.common.oem.api.MessageSystem;
import com.nuance.nmsp.client2.sdk.common.oem.api.MiscSystem;
import com.nuance.nmsp.client2.sdk.common.oem.api.NetworkSystem;
import com.nuance.nmsp.client2.sdk.common.oem.api.MessageSystem.MessageData;
import com.nuance.nmsp.client2.sdk.common.oem.api.NetworkSystem.NetworkReadMode;
import com.nuance.nmsp.client2.sdk.common.oem.api.NetworkSystem.NetworkStatus;
import com.nuance.nmsp.client2.sdk.common.oem.api.TimerSystem.TimerSystemTask;
import com.nuance.nmsp.client2.sdk.common.protocols.ProtocolBuilder;
import com.nuance.nmsp.client2.sdk.common.protocols.ProtocolDefines;
import com.nuance.nmsp.client2.sdk.common.protocols.XModeMsgHeader;
import com.nuance.nmsp.client2.sdk.common.util.ByteConversion;
import com.nuance.nmsp.client2.sdk.common.util.Util;
import com.nuance.nmsp.client2.sdk.components.core.calllog.SessionEvent;
import com.nuance.nmsp.client2.sdk.components.core.calllog.SessionEventAlreadyCommittedException;
import com.nuance.nmsp.client2.sdk.components.core.calllog.SessionEventBuilder;
import com.nuance.nmsp.client2.sdk.components.core.internal.calllog.SessionEventBuilderImpl;
import com.nuance.nmsp.client2.sdk.components.core.internal.calllog.SessionEventImpl;
import com.nuance.nmsp.client2.sdk.components.general.Parameter;
import com.nuance.nmsp.client2.sdk.oem.*;

//#ifdef NMSP_CPL_ENABLED
//@import java.util.Vector;
//@import com.nuance.nmsp.client.sdk.common.oem.api.NetworkSystem.CPLSocketReadContext;
//@import com.nuance.nmsp.client.sdk.common.oem.api.NetworkSystem.CPLSocketWriteContext;
//@import com.nuance.nmsp.client.sdk.common.oem.api.LogSystem.CPLRecord;
//#endif

public class XMode implements MessageSystem.MessageHandler,
                                NetworkSystem.OpenSocketCallback,
                                NetworkSystem.CloseSocketCallback,
                                NetworkSystem.ReadSocketCallback,
                                NetworkSystem.WriteSocketCallback
{
//#ifdef NMSP_LOG
    private static Log log = LogFactory.getLog(XMode.class);
//#endif
    /** read socket scheduler **/
    private TimerSystemTask readSocketTask;

    /** read socket interval in milliseconds **/
    private static final short READ_SOCKET_INTERVAL = 20;

    /** socket connection timeout **/
    private static final short XMODE_NMSP_CONNECTION_TIMEOUT_SECS = 30;
    private int connectionTimeout = XMODE_NMSP_CONNECTION_TIMEOUT_SECS;
    private TimerSystemTask copConnectTask;

    /** session idle timeout **/
    private static final short XMODE_NMSP_IDLE_SESSION_TIMEOUT_SECS = 50;
    private int idleSessionTimeout = XMODE_NMSP_IDLE_SESSION_TIMEOUT_SECS;
    private TimerSystemTask idleSessionTask;

    //#ifdef NMSP_PLATFORM_BLACKBERRY
//@    private String apnSetting = null;
    //#endif

    /** gateway attributes **/
    protected String server = null;
    protected short port = 0;

    /** xmode parameters list **/
    private Vector parameters;

    /** socket Settings **/
    private boolean isSSL = false;
    private Vector socketParams = null;

    /** message system **/
    MessageSystem msgSys = null;

    /** private static final byte CMD_NO_CMD                  = 0x0; **/
    private static final byte CMD_CONNECT                 = 0x01;
    private static final byte CMD_DISCONNECT              = 0x02;
    private static final byte CMD_OPEN_SOCKET             = 0x03;
    private static final byte CMD_CLOSE_SOCKET            = 0x04;
    private static final byte CMD_READ_SOCKET_MSG_HEADER  = 0x05;
    private static final byte CMD_READ_SOCKET_MSG_PAYLOAD = 0x06;
    private static final byte CMD_COP_CONNECT             = 0x07;
    private static final byte CMD_COP_PING_RESPONSE       = 0x08;
    private static final byte CMD_COP_CONNECT_TIMED_OUT   = 0x09;
    private static final byte CMD_COP_CONFIRM             = 0x0A;
    private static final byte CMD_MESSAGE_SENT_CONFIRM    = 0x0B;
    private static final byte CMD_MESSAGE_NOT_SENT_NOTICE = 0x0C;
    //private static final byte CMD_GENERATE_COP_REF_ID     = 0x0D;
    private static final byte CMD_SEND_LOW_PRIORITY_MSG   = 0x0E;

    public static final short XMODE_SOCKET_CLOSED_REMOTE_DISCONNECT                 = 0;
    public static final short XMODE_SOCKET_CLOSED_DISCONNECT                        = 1;
    public static final short XMODE_SESSION_IDLE_FOR_TOO_LONG                       = 3;
    public static final short XMODE_SOCKET_CLOSED_OPEN_SOCKET_FAILED                = 4;
    public static final short XMODE_SOCKET_CLOSED_COP_CONNECT_TIMEOUT               = 5;
    public static final short XMODE_SOCKET_CLOSED_COP_DISCONNECT                    = 6;
    public static final short XMODE_SOCKET_CLOSED_COP_CONNECT_FAILED                = 7;
    public static final short XMODE_SOCKET_CLOSED_NETWORK_ERROR                     = 8;
    public static final short XMODE_SOCKET_CLOSED_OPEN_SOCKET_FAILED_SECURITY       = 9;

    public static final int XMODE_MAX_PAYLOAD_SIZE = 500*1024;

//    public static final short XMODE_BCP_LOG_SUCCESS       = 0x00;
//    public static final short XMODE_BCP_LOG_PARSE_ERROR   = 0x01;
//    public static final short XMODE_BCP_LOG_WRITE_ERROR   = 0x02;
//    public static final short XMODE_BCP_LOG_DISABLED      = 0x03;

    /** socket variables **/
    private Object socket;
    private XModeMsgHeader header = null;
    private short socketClosedReason = XMODE_SOCKET_CLOSED_REMOTE_DISCONNECT;
    private short disconnectedReason = ProtocolDefines.XMODE_DISCONNECT_CODE_UNKNOWN;

    /** network system **/
    private String appId = "Not specified";
    private String uid = "Not specified";
    private String accessToken = "Not specified";
    private byte[] applicationKey = null;
    private NetworkSystem netSys = null;
    private static final String NET_CONTEXT_SEND_COP_CONNECT          = "SEND_COP_CONNECT";
    private static final String NET_CONTEXT_SEND_COP_CONFIRM          = "SEND_COP_CONFIRM";
    private static final String NET_CONTEXT_SEND_COP_DISCONNECT       = "SEND_COP_DISCONNECT";
    private static final String NET_CONTEXT_SEND_COP_PING_RESPONSE    = "SEND_COP_PING_RESPONSE";
    private static final String NET_CONTEXT_READ_XMODE_HEADER         = "READ_XMODE_HEADER";
    private static final String NET_CONTEXT_READ_XMODE_PAYLOAD        = "READ_XMODE_PAYLOAD";
    private static final String NET_CONTEXT_SEND_VAP_RECORD_BEGIN     = "SEND_VAP_RECORD_BEGIN";
    private static final String NET_CONTEXT_SEND_VAP_RECORD           = "SEND_VAP_RECORD";
    private static final String NET_CONTEXT_SEND_VAP_RECORD_END       = "SEND_VAP_RECORD_END";
    public static final String NET_CONTEXT_SEND_BCP_RECOGNIZE         = "SEND_BCP_RECOGNIZE";
    public static final String NET_CONTEXT_SEND_BCP_LOAD_RESOURCE     = "SEND_BCP_LOAD_RESOURCE";
    public static final String NET_CONTEXT_SEND_BCP_FREE_RESOURCE     = "SEND_BCP_FREE_RESOURCE";
    public static final String NET_CONTEXT_SEND_BCP_FREE_RESOURCE_ID  = "SEND_BCP_FREE_RESOURCE_ID";
    public static final String NET_CONTEXT_SEND_BCP_CANCEL            = "SEND_BCP_CANCEL";
    public static final String NET_CONTEXT_SEND_BCP_SET_PARAMS        = "SEND_BCP_SET_PARAMS";
    public static final String NET_CONTEXT_SEND_BCP_GET_PARAMS        = "SEND_BCP_GET_PARAMS";
    public static final String NET_CONTEXT_SEND_BCP_LOG               = "SEND_BCP_LOG";
    // this context is always used with the transaction id appended at the end
    // so we normally use String.startsWith() to judge
    public static final String NET_CONTEXT_SEND_BCP_BEGIN             = "SEND_BCP_BEGIN";
    public static final String NET_CONTEXT_SEND_BCP_DATA              = "SEND_BCP_DATA";
    public static final String NET_CONTEXT_SEND_BCP_LOAD_GRAMMAR      = "SEND_BCP_LOAD_GRAMMAR";
    public static final String NET_CONTEXT_SEND_BCP_GENERATE_AUDIO    = "SEND_BCP_GENERATE_AUDIO";
    public static final String NET_CONTEXT_SEND_BCP_MDS_MESSAGE       = "SEND_BCP_MDS_MESSAGE";
    public static final String NET_CONTEXT_SEND_BCP_MDS_MESSAGE_RESPONSE = "SEND_BCP_MDS_MESSAGE_RESPONSE";
  //public static final String NET_CONTEXT_SEND_BCP_END               = "SEND_BCP_END";

    /** state machine defines **/
    public static final byte STATE_CONNECTING = 0x00;
    public static final byte STATE_CONNECTED  = 0x01;
    public static final byte STATE_CLOSING    = 0x02;
    public static final byte STATE_CLOSED     = 0x03;
    protected byte state;

    /** identifications **/
    //protected int requestId;
    public byte[] sessionId;
    //#ifdef NMSP_LOG
    private String sessionIdStr = "";
    //#endif
    protected int pingRequestId;

    //#ifdef NMSP_CPL_ENABLED
//@    public long openSocketTime;
//@    public long socketOpenedTime;
//@    public long connectionEstablishedTime;
//@    public Vector cplSocketWriteRecords;
//@    public Vector cplSocketReadRecords;
//@    public Vector cplXmodeReadMsgs;
//@    public Vector cplXmodeWriteMsgs;
//@    public Vector cplXmodeReadRequests;
    //#endif

    /** transaction attributes **/
    protected short inputCodec;
    protected short outputCodec;
    //protected short codec;

    /** xmode listener, provided by different functional modules **/
    public static interface XModeListener {
        /** pass the formal xmode message to the session object **/
        public void xmodeMsgCallback(XModeMsgHeader header, byte[] payloadBuf);
        /** pass the sent message's network context to the session object **/
        public void xmodeMsgSent(String context, Object data);
        /** pass the blocked message's network context to the session object **/
        public void xmodeMsgNotSent(String context, Object data);
        /** in case the connection is dropped abruptly, no formal xmode message will be sent **/
        /** we have to notify the Session that the socket is closed by this method **/
        public void socketClosed(short reason, short subReason);
        /** in case we send the COP_DISCONNECT to gateway, no formal xmode message will be sent **/
        /** we have to notify the Session that COP_DISCONNECT has been sent, and it can let the app quit **/
//        public void disconnected();
        public void socketOpened();
        public void copConnected();
    }
    private XModeListener xmodeListener;

    public  static final int SEND_MSG_PRIORITY_HIGH     = 3;
    private static final int SEND_MSG_PRIORITY_PROMOTED = 2;
    public  static final int SEND_MSG_PRIORITY_LOW      = 1;

    private SessionEvent sessionEvent;
    private String _refId = "";

    private class SocketWriteRequest
    {
        byte[] buffer;
        Object context;
        public SocketWriteRequest(byte[] b, Object c)
        {
            buffer = b;
            context = c;
        }
    }
    private Queue socketWritesQueue = new LinkedList();
    private Queue pendingSocketWritesQueue = new LinkedList();

    // This is to open the COP session with call logging disabled
    // It will also prevent logging to be created in this new session
    private boolean forLogOnly = false;

    public XMode(String server, short port, String appId, XModeListener listener, Vector parameters, MessageSystem msgSys) {
        this.server         = server;
        this.port           = port;
        this.appId          = appId;
        this.uid = uid;
        this.accessToken = accessToken;
        this.xmodeListener  = listener;
        this.socketParams   = new Vector();

        if(parameters != null) {
            this.parameters = parameters;
        }
        else {
            this.parameters = new Vector();
        }
        this.msgSys         = msgSys;
      //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("XMode() server: " + server + " port: " + port);
      //#endif
        Enumeration params = this.parameters.elements();
        while(params.hasMoreElements()) {
            Parameter param = (Parameter) params.nextElement();
          //#ifdef NMSP_LOG
            if(log.isDebugEnabled())
                log.debug("XMode() " + param.getType() + " : " + param.getName() + " = " + new String(param.getValue()));
          //#endif
            if(param.getType() == Parameter.Type.SDK) {
                if(param.getName().equals(NMSPDefines.IdleSessionTimeout)) {
                    int idleTimeout = Integer.parseInt(new String(param.getValue()));
                    if (idleTimeout > 0) {
                        idleSessionTimeout = idleTimeout;
                    }
                }
                else if(param.getName().equals(NMSPDefines.ConnectionTimeout))
                {
                    connectionTimeout = Integer.parseInt(new String(param.getValue()));
                }
                else if(    param.getName().equals(NMSPDefines.NMSP_DEFINES_SSL_SOCKET)
                         || param.getName().equals(NMSPDefines.NMSP_DEFINES_SSL_CERT_SUMMARY)
                         || param.getName().equals(NMSPDefines.NMSP_DEFINES_SSL_CERT_DATA)
                         || param.getName().equals(NMSPDefines.NMSP_DEFINES_SSL_SELFSIGNED_CERT)
                         //#ifdef NMSP_PLATFORM_ANDROID
                         || param.getName().equals(NMSPDefines.NMSP_DEFINES_ANDROID_CONTEXT)
                         || param.getName().equals(NMSPDefines.NMSP_DEFINES_USING_DEVICE_PROXY_SETTING_ONLY)
                         || param.getName().equals(NMSPDefines.NMSP_DEFINES_USING_DEVICE_PROXY_SETTING_IF_AVAILABLE)
                         //#endif
                         )
                {

                    if (param.getName().equals(NMSPDefines.NMSP_DEFINES_SSL_SOCKET)
                        && ((new String(param.getValue())).equals("TRUE") || (new String(param.getValue())).equals("true")))
                    {
                        isSSL = true;
                    }

                    socketParams.addElement(param);
                }

                //#ifdef NMSP_PLATFORM_BLACKBERRY
//@                if(param.getName().equals(NMSPDefines.SocketConnectionSetting)) {
//@                    apnSetting = new String(param.getValue());
//@                }
                //#endif
            }
        }

        state = STATE_CLOSED;

        //#ifdef NMSP_CPL_ENABLED
//@        cplSocketWriteRecords = new Vector();
//@        cplSocketReadRecords = new Vector();
//@        openSocketTime = 0;
//@        socketOpenedTime = 0;
//@        connectionEstablishedTime = 0;
//@        cplXmodeReadMsgs = new Vector();
//@        cplXmodeWriteMsgs = new Vector();
//@        cplXmodeReadRequests = new Vector();
        //#endif
    }

    //#ifdef NMSP_CPL_ENABLED
//@    public void clearCPL() {
//@        if (state == STATE_CLOSED) {
//@            openSocketTime = 0;
//@            socketOpenedTime = 0;
//@            connectionEstablishedTime = 0;
//@        }
//@        cplSocketWriteRecords.removeAllElements();
//@        cplSocketReadRecords.removeAllElements();
//@        cplXmodeReadMsgs.removeAllElements();
//@        cplXmodeWriteMsgs.removeAllElements();
//@        cplXmodeReadRequests.removeAllElements();
//@    }
    //#endif

    /****************************************************************
     * API methods exposed by this module to the functional modules *
     ****************************************************************/
    public void connect(short inputCodec, short outputCodec, String refId)
    {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("XMode.connect() codec: "  + outputCodec);
        //#endif

        if (    (     STATE_CONNECTING == state )
             && ( _refId != null)
             && ( _refId.equals(refId) == false ) )
        {
            // this means the socket has not been opened and connection
            // has not been established yet, if the XMode.connect() is
            // called again with a different refId, we need to cancel the
            // old one to use the new one. Otherwise we will see unresolved
            // remote reference events in the end.
            cancelGwRemoteEvent(_refId, SessionEventImpl.NMSP_CALLLOG_EVENT_CANCEL_CAUSE_CODE_OTHER,
                    "New connecting request (New command created) while connecting, dropped the old reference Id");
        }

        _refId = refId;

        if (state == STATE_CONNECTED)
        {
        }
        else if (state == STATE_CONNECTING)
        {
            // DO NOTHING BUT WAIT FOR BEING CONNECTED
        }
        else if (state == STATE_CLOSING)
        {
            if (readSocketTask != null)
                msgSys.cancelTask(readSocketTask);
            if (idleSessionTask != null)
                msgSys.cancelTask(idleSessionTask);

            createChildEvent(SessionEvent.NMSP_CALLLOG_OPENSOCKET_EVENT);

            this.socket   = null;
            netSys        = null;
            sessionId     = null;
            //#ifdef NMSP_LOG
            sessionIdStr  = "";
            //#endif
            this.inputCodec = inputCodec;
            this.outputCodec = outputCodec;
            state = STATE_CONNECTING;
            sendCmdMsg(CMD_CONNECT, null);
        }
        else if (state == STATE_CLOSED)
        {
            createChildEvent(SessionEvent.NMSP_CALLLOG_OPENSOCKET_EVENT);

            this.inputCodec = inputCodec;
            this.outputCodec = outputCodec;
            state = STATE_CONNECTING;
            sendCmdMsg(CMD_CONNECT, null);
        }
    }

    public void disconnect() {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("XMode.disconnect() state:" + state + ", socket:" + socket);
        //#endif
        if(state == STATE_CLOSED) {
            socketClosedReason = XMODE_SOCKET_CLOSED_DISCONNECT;
            xmodeListener.socketClosed(socketClosedReason, disconnectedReason);
        }
        else if (state == STATE_CLOSING) {
            return;
        }
        else if (state == STATE_CONNECTING) {
            createChildEvent(SessionEvent.NMSP_CALLLOG_CLOSESOCKET_EVENT);

            socketClosedReason = XMODE_SOCKET_CLOSED_DISCONNECT;
            state = STATE_CLOSING;
            if (socket != null) {
                netSys.closeSocket(socket);
            }
        }
        else if (state == STATE_CONNECTED) {
            state = STATE_CLOSING;
            socketClosedReason = XMODE_SOCKET_CLOSED_DISCONNECT;
            sendCmdMsg(CMD_DISCONNECT, null);
        }
    }

    public void setAccessToken(String token) {
        accessToken = token;
    }

    public String getAccessToken() {
        return accessToken;
    }

    public void setUID(String id) {
        uid = id;
    }

    public String getUID() {
        return uid;
    }

    public void startStreaming(int audioId) {
//#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("XMode.startStreaming() audio id: " + audioId);
//#endif

        if (state != STATE_CONNECTED) {
            return;
        } else {
            sendVapRecordBegin(audioId);
        }
    }

    public void sendVapRecordMsg(byte[] audioBuf, int audioId) {
 //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("XMode.sendVapRecordMsg() audio id: " + audioId);
 //#endif

        if (state != STATE_CONNECTED) {
            return;
        }
        int audioLen = audioBuf.length;
        byte[] payload = new byte[ProtocolDefines.XMODE_CLIENT_VAP_AUDIO_ID_SIZE + ProtocolDefines.XMODE_CLIENT_VAP_AUDIO_LEN_SIZE + audioLen];
        int offset = 0;
        ByteConversion.intToBytes(audioId, payload, offset);
        offset += ProtocolDefines.XMODE_CLIENT_VAP_AUDIO_ID_SIZE;
        ByteConversion.intToBytes(audioLen, payload, offset);
        offset += ProtocolDefines.XMODE_CLIENT_VAP_AUDIO_LEN_SIZE;
        System.arraycopy(audioBuf, 0, payload, offset, audioLen);

        byte[] buffer = ProtocolBuilder.buildXModeBuf(ProtocolDefines.XMODE_PROTOCOL_VAP, ProtocolDefines.XMODE_VERSION_VAP,
                ProtocolDefines.XMODE_VAP_COMMAND_RECORD, payload);

        //#ifdef NMSP_CPL_ENABLED
//@        CPLRecord record = new CPLRecord(System.currentTimeMillis(), "request_xmode_send_vap_record[" + audioBuf.length + "_bytes]");
//@        cplXmodeWriteMsgs.addElement(record);
        //#endif
        //#ifdef NMSP_PR_MEASURE_ENABLED
//@        createRecordEvent(buffer.length);
        //#endif
        sendXModeMsg(buffer, SEND_MSG_PRIORITY_HIGH, NET_CONTEXT_SEND_VAP_RECORD);
    }

    public void sendVapRecordEnd(int audioId) {
//#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("XMode.sendVapRecordEnd() audio id: " + audioId);
//#endif

        if (state != STATE_CONNECTED) {
            return;
        }
        byte[] payload = new byte[ProtocolDefines.XMODE_CLIENT_VAP_AUDIO_ID_SIZE];
        ByteConversion.intToBytes(audioId, payload, 0);

        byte[] buffer = ProtocolBuilder.buildXModeBuf(ProtocolDefines.XMODE_PROTOCOL_VAP, ProtocolDefines.XMODE_VERSION_VAP,
                ProtocolDefines.XMODE_VAP_COMMAND_RECORD_END, payload);

        //#ifdef NMSP_CPL_ENABLED
//@        CPLRecord record = new CPLRecord(System.currentTimeMillis(), "request_xmode_send_vap_record_end");
//@        cplXmodeWriteMsgs.addElement(record);
        //#endif
        sendXModeMsg(buffer, SEND_MSG_PRIORITY_HIGH, NET_CONTEXT_SEND_VAP_RECORD_END);
    }

    public void sendXModeMsg(byte[] buffer, int priority, Object context) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("XMode.sendXModeMsg() " + context + ", buffer.length:" + buffer.length);
        //#endif
        //#ifdef NMSP_SOCKET_WRITE_PADDING
//@        if(context.equals(NET_CONTEXT_SEND_COP_PING_RESPONSE) ||
//@           context.equals(NET_CONTEXT_SEND_VAP_RECORD_BEGIN) ||
//@           context.equals(NET_CONTEXT_SEND_VAP_RECORD) ||
//@           context.equals(NET_CONTEXT_SEND_VAP_RECORD_END) ||
//@           context.equals(NET_CONTEXT_SEND_BCP_SET_PARAMS) ||
//@           context.equals(NET_CONTEXT_SEND_BCP_BEGIN) ||
//@           context.equals(NET_CONTEXT_SEND_BCP_DATA)) {
//@            StringBuffer paddingStr = new StringBuffer();
//@            for(int i = 0; i < 50; i++)
//@                paddingStr.append("PPPPPPPPPPPPPPPPPPPP");
//@            byte[] paddingBuff = preparePaddingBuffer(paddingStr.toString());
//@            byte[] tmpBuff = new byte[buffer.length + paddingBuff.length];
//@            System.arraycopy(buffer, 0, tmpBuff, 0, buffer.length);
//@            System.arraycopy(paddingBuff, 0, tmpBuff, buffer.length, paddingBuff.length);
//@            buffer = tmpBuff;
//@        }
        //#endif

        if(idleSessionTask != null && msgSys.cancelTask(idleSessionTask)) {
            idleSessionTask = new TimerSystemTask() {
                public void run() {
                    try {
                        socketClosedReason = XMODE_SESSION_IDLE_FOR_TOO_LONG;
                      //#ifdef NMSP_LOG
                        if(log.isErrorEnabled())
                            log.error("Session Idle for too long, longer than [" + idleSessionTimeout + "] ()");
                      //#endif
                        state = STATE_CLOSING;
                        sendCmdMsg(CMD_CLOSE_SOCKET, null);
                    } catch (Exception e) {
                        //#ifdef NMSP_LOG
                        if(log.isErrorEnabled())
                            log.error("XMode.sendXModeMsg() " + e.getClass().getName() + " " + e.getMessage());
                        //#endif
                    }
                }
            };
            msgSys.scheduleTask(idleSessionTask, idleSessionTimeout * 1000);
        }

        if (netSys != null && socket != null) {
            //#ifdef NMSP_CPL_ENABLED
//@            CPLSocketWriteContext cplContext = new CPLSocketWriteContext();
//@            cplContext.normalContext = (String) context;
//@            context = cplContext;
            //#endif
            if (priority == SEND_MSG_PRIORITY_HIGH || priority == SEND_MSG_PRIORITY_PROMOTED || (socketWritesQueue.size() == 0 && pendingSocketWritesQueue.size() == 0))
            {
                SocketWriteRequest req;
                if (priority == SEND_MSG_PRIORITY_PROMOTED)
                {
                    req = (SocketWriteRequest)pendingSocketWritesQueue.remove();
                    buffer = req.buffer;
                }
                else
                {
                    req = new SocketWriteRequest(null, context);
                }
                socketWritesQueue.add(req);
                netSys.socketWrite(socket, buffer, 0, buffer.length, this, req);
            }
            else
            {
                pendingSocketWritesQueue.add(new SocketWriteRequest(buffer, context));
            }
        }
        else if (priority == SEND_MSG_PRIORITY_LOW)
        {
            pendingSocketWritesQueue.add(new SocketWriteRequest(buffer, context));
        }
    }

    /******************
     * helper methods *
     ******************/
    private void sendVapRecordBegin(int audioId) {
        byte[] payload = new byte[ProtocolDefines.XMODE_CLIENT_VAP_AUDIO_ID_SIZE + ProtocolDefines.XMODE_CLIENT_VAP_AUDIO_CODEC];
        ByteConversion.intToBytes(audioId, payload, 0);
        ByteConversion.shortToBytes(inputCodec, payload, ProtocolDefines.XMODE_CLIENT_VAP_AUDIO_ID_SIZE);

        byte[] buffer = ProtocolBuilder.buildXModeBuf(ProtocolDefines.XMODE_PROTOCOL_VAP, ProtocolDefines.XMODE_VERSION_VAP,
                ProtocolDefines.XMODE_VAP_COMMAND_RECORD_BEGIN, payload);

        //#ifdef NMSP_CPL_ENABLED
//@        CPLRecord record = new CPLRecord(System.currentTimeMillis(), "request_xmode_send_vap_record_begin audioId[" + audioId + "]");
//@        cplXmodeWriteMsgs.addElement(record);
        //#endif
        sendXModeMsg(buffer, SEND_MSG_PRIORITY_HIGH, NET_CONTEXT_SEND_VAP_RECORD_BEGIN);
    }

    /***********************************
     * NetworkSystem call back methods *
     ***********************************/
    public void openSocketCallback(NetworkStatus status, Object socket, Object context) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("XMode.openSocketCallback() ");
        //#endif
        if (status == NetworkStatus.NETWORK_OK) {
            this.socket = socket;
            if (state == STATE_CONNECTING) {
                //#ifdef NMSP_CPL_ENABLED
//@                socketOpenedTime = System.currentTimeMillis();
                //#endif
                sendCmdMsg(CMD_COP_CONNECT, null);
            }
            else {
                state = STATE_CLOSING;
                netSys.closeSocket(this.socket);
            }
        } else if (status == NetworkStatus.NETWORK_ERROR || status == NetworkStatus.NETWORK_SECURITY_ERROR) {
            //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("XMode.openSocketCallback() NETWORK_ERROR");
            //#endif
            state = STATE_CLOSED;
            socketClosedReason = status == NetworkStatus.NETWORK_ERROR ? XMODE_SOCKET_CLOSED_OPEN_SOCKET_FAILED : XMODE_SOCKET_CLOSED_OPEN_SOCKET_FAILED_SECURITY;
            xmodeListener.socketClosed(socketClosedReason, disconnectedReason);
            this.socket   = null;
            netSys        = null;
            sessionId     = null;
            //#ifdef NMSP_LOG
            sessionIdStr  = "";
            //#endif
            // JUST IGNORE IT, the network system will close the socket and call closeSocketCallback
        } else if (status == NetworkStatus.NETWORK_MEMORY_ERROR) {
            socketClosedReason = XMODE_SOCKET_CLOSED_OPEN_SOCKET_FAILED;
          //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("XMode.openSocketCallback() NETWORK_MEMORY_ERROR");
          //#endif
            // JUST IGNORE IT, the network system will close the socket and call closeSocketCallback
        }
    }

    public void readSocketCallback(NetworkStatus status, Object socket, byte[] buffer, int offset, int bufferLen, int bytesRead, Object context) {
        //#ifdef NMSP_CPL_ENABLED
//@        String contextStr = ((NetworkSystem.CPLSocketReadContext) context).normalContext;
//@        cplSocketReadRecords.addElement(context);
        //#else
        String contextStr = (String) context;
        //#endif

        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("Read callback");

        if(log.isTraceEnabled())
            log.traceBuffer(buffer);
        //#endif

        if (status == NetworkStatus.NETWORK_OK) {
            if (contextStr.equals(NET_CONTEXT_READ_XMODE_HEADER)) {
                if (bytesRead == 0) {
                    readSocketTask = new TimerSystemTask(){
                            public void run() {
                                try {
                                    sendCmdMsg(CMD_READ_SOCKET_MSG_HEADER, null);
                                } catch (Exception e) {
                                    //#ifdef NMSP_LOG
                                    if(log.isErrorEnabled())
                                        log.error("XMode.readSocketCallback() " + e.getClass().getName() + " " + e.getMessage());
                                    //#endif
                                }
                            }
                        };
                        msgSys.scheduleTask(readSocketTask, (long) READ_SOCKET_INTERVAL);
                } else if (bytesRead == bufferLen) {
                    /** XMODE_PACKET_HEADER_SIZE **/
                    this.header = new XModeMsgHeader(buffer);

                    /** the header.len should be >= 0 **/
                    if (this.header.len == 0) {
                        parseXModeMsg(this.header, null);
                        sendCmdMsg(CMD_READ_SOCKET_MSG_HEADER, null);
                        /*HG Workaround 500K is the max message size for audio otherwise ignore it*/
                    } else {
                        if (this.header.len > XMODE_MAX_PAYLOAD_SIZE || this.header.len < 0) {
                            //#ifdef NMSP_LOG
                         if(log.isWarnEnabled())
                             log.warn("XMode.readSocketCallback() Received a larger than anticipated payload size.");
                            //#endif
                        }
                        sendCmdMsg(CMD_READ_SOCKET_MSG_PAYLOAD, null);
                    }
                // to be removed by wlwl <start>
                } else {
                    //#ifdef NMSP_LOG
                    if(log.isErrorEnabled())
                        log.error("----***---- readSocketCallback fatal error in readSocketCallback NET_CONTEXT_READ_XMODE_HEADER bytesRead:[" + bytesRead + "] bufferLen:[" + bufferLen + "]");
                          //#endif
                // to be removed by wlwl <end>
                }
            } else if (contextStr.equals(NET_CONTEXT_READ_XMODE_PAYLOAD)) {
                if (bytesRead == 0)
                {
                    //#ifdef NMSP_LOG
                    if(log.isDebugEnabled())
                        log.debug(Integer.toHexString(header.cmd) + " payload not read bytesRead is 0");
                    //#endif
                    readSocketTask = new TimerSystemTask(){
                            public void run() {
                                try {
                                    sendCmdMsg(CMD_READ_SOCKET_MSG_PAYLOAD, null);
                                }
                                catch (Exception e) {
                                    //#ifdef NMSP_LOG
                                    if(log.isErrorEnabled())
                                        log.error("XMode.readSocketCallback() " + e.getClass().getName() + " " + e.getMessage());
                                    //#endif
                                }
                            }
                        };
                        msgSys.scheduleTask(readSocketTask, (long) READ_SOCKET_INTERVAL);
                } else if (bytesRead == bufferLen) {
                    //HG: Fix for Android NMDP crash:
                    // If the length is more than buffer size, ignore this buffer
                    // TODO: investigate if we need to read more data
                    // The issue was, we were reading command as len header in rear situations.
                    if (header.len <= bufferLen)
                        parseXModeMsg(this.header, buffer);
                    sendCmdMsg(CMD_READ_SOCKET_MSG_HEADER, null);
                    // to be removed by wlwl <start>
                } else {
                    //#ifdef NMSP_LOG
                    if(log.isErrorEnabled())
                        log.error("----***---- readSocketCallback fatal error in readSocketCallback NET_CONTEXT_READ_XMODE_PAYLOAD bytesRead:[" + bytesRead + "] bufferLen:[" + bufferLen + "]");
                          //#endif
                // to be removed by wlwl <end>
                }
            }
        } else if (status == NetworkStatus.NETWORK_ERROR) {
            if (socketClosedReason != XMODE_SOCKET_CLOSED_DISCONNECT && socketClosedReason != XMODE_SESSION_IDLE_FOR_TOO_LONG
                    && socketClosedReason != XMODE_SOCKET_CLOSED_COP_DISCONNECT) {
                socketClosedReason = XMODE_SOCKET_CLOSED_NETWORK_ERROR;
              //#ifdef NMSP_LOG
                if(log.isErrorEnabled())
                    log.error("XMode.readSocketCallback() NETWORK_ERROR");
              //#endif
            }
            sendCmdMsg(CMD_CLOSE_SOCKET, null);
            // JUST IGNOR IT, the network system will close the socket and call closeSocketCallback
        } else if (status == NetworkStatus.NETWORK_MEMORY_ERROR) {
            if (socketClosedReason != XMODE_SOCKET_CLOSED_DISCONNECT && socketClosedReason != XMODE_SESSION_IDLE_FOR_TOO_LONG) {
                socketClosedReason = XMODE_SOCKET_CLOSED_NETWORK_ERROR;
              //#ifdef NMSP_LOG
                if(log.isErrorEnabled())
                    log.error("XMode.readSocketCallback() NETWORK_MEMORY_ERROR");
              //#endif
            }
            // JUST IGNORE IT, the network system will close the socket and call closeSocketCallback
        }
    }

    public void writeSocketCallback(NetworkStatus status, Object socket, byte[] buffer, int offset, int bufferLen, int bytesWritten, Object context) {
        String contextStr = (String)((SocketWriteRequest)context).context;
        //#ifdef NMSP_CPL_ENABLED
//@        Object realContext = ((SocketWriteRequest)context).context;
//@        cplSocketWriteRecords.addElement(realContext);
        //#endif

        if (status == NetworkStatus.NETWORK_OK && (bufferLen == bytesWritten)) {
//            LogSystem.debug("XMode.writeSocketCallback() " + context);
            if (contextStr.equals(NET_CONTEXT_SEND_COP_CONNECT)) {
                sendCmdMsg(CMD_READ_SOCKET_MSG_HEADER, null);
            // TODO maybe send an event when a write succeeds while we are connected
//            } else if (contextStr.equals(NET_CONTEXT_SEND_VAP_RECORD_BEGIN)) {
//            } else if (contextStr.equals(NET_CONTEXT_SEND_BCP_RECOGNIZE)) {
//            } else if (contextStr.equals(NET_CONTEXT_SEND_VAP_RECORD)) {
            } else if (contextStr.equals(NET_CONTEXT_SEND_COP_DISCONNECT)) {
                netSys.closeSocket(socket);
                //xmodeListener.disconnected();
//            } else if (contextStr.equals(NET_CONTEXT_SEND_COP_PING_RESPONSE)) {
//            } else if (contextStr.equals(NET_CONTEXT_SEND_VAP_RECORD_END)) {
            }

            /** message sent callback can be called here **/
            sendCmdMsg(CMD_MESSAGE_SENT_CONFIRM, contextStr);

            socketWritesQueue.remove(context);
            if (socketWritesQueue.size() == 0 && pendingSocketWritesQueue.size() > 0)
            {
                sendCmdMsg(CMD_SEND_LOW_PRIORITY_MSG, null);
            }
        } else if (status == NetworkStatus.NETWORK_ERROR) {
            if (socketClosedReason != XMODE_SOCKET_CLOSED_DISCONNECT && socketClosedReason != XMODE_SESSION_IDLE_FOR_TOO_LONG) {
                socketClosedReason = XMODE_SOCKET_CLOSED_NETWORK_ERROR;
              //#ifdef NMSP_LOG
                if(log.isErrorEnabled())
                    log.error("XMode.writeSocketCallback() NETWORK_ERROR");
              //#endif
            }

            /** message not sent callback can be called here **/
            sendCmdMsg(CMD_MESSAGE_NOT_SENT_NOTICE, contextStr);

            socketWritesQueue.remove(context);
            if (socketWritesQueue.size() == 0)
            {
                int sizeOfPendingWrites = pendingSocketWritesQueue.size();
                for (int i = 0; i < sizeOfPendingWrites; i++)
                {
                    SocketWriteRequest req = (SocketWriteRequest)pendingSocketWritesQueue.remove();
                    sendCmdMsg(CMD_MESSAGE_NOT_SENT_NOTICE, req.context);
                }
            }

            // don't close socket upon socket write error so socket read can continue to read off all messages.
            //sendCmdMsg(CMD_CLOSE_SOCKET, null);
            // JUST IGNOR IT, the network system will close the socket and call closeSocketCallback
        } else if (status == NetworkStatus.NETWORK_MEMORY_ERROR) {
            if (socketClosedReason != XMODE_SOCKET_CLOSED_DISCONNECT && socketClosedReason != XMODE_SESSION_IDLE_FOR_TOO_LONG) {
                socketClosedReason = XMODE_SOCKET_CLOSED_NETWORK_ERROR;
              //#ifdef NMSP_LOG
                if(log.isErrorEnabled())
                    log.error("XMode.writeSocketCallback() NETWORK_MEMORY_ERROR");
              //#endif
            }

            /** message notsent callback can be called here **/
            sendCmdMsg(CMD_MESSAGE_NOT_SENT_NOTICE, contextStr);

            socketWritesQueue.remove(context);
            if (socketWritesQueue.size() == 0)
            {
                int sizeOfPendingWrites = pendingSocketWritesQueue.size();
                for (int i = 0; i < sizeOfPendingWrites; i++)
                {
                    SocketWriteRequest req = (SocketWriteRequest)pendingSocketWritesQueue.remove();
                    sendCmdMsg(CMD_MESSAGE_NOT_SENT_NOTICE, req.context);
                }
            }

            // JUST IGNOR IT, the network system will close the socket and call closeSocketCallback
        }
    }

    public void closeSocketCallback(NetworkStatus status, Object socket, Object context) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("XMode.closeSocketCallback() " + sessionIdStr);
        //#endif

        state = STATE_CLOSED;

        if(readSocketTask != null) {
            msgSys.cancelTask(readSocketTask);
            readSocketTask = null;
        }

        if (idleSessionTask != null) {
            msgSys.cancelTask(idleSessionTask);
            idleSessionTask = null;
        }

        if(this.socket != null){
             xmodeListener.socketClosed(socketClosedReason, disconnectedReason);
        }

        this.socket   = null;
        netSys        = null;
        sessionId     = null;
        //#ifdef NMSP_LOG
        sessionIdStr  = "";
        //#endif
    }

    /***********************************************************
     * all the command message handler methods for XMode class *
     ***********************************************************/
    private void handleConnect() {
        if(state == STATE_CONNECTING) {
            sendCmdMsg(CMD_OPEN_SOCKET, null);
        }
        else {
            xmodeListener.socketClosed(socketClosedReason, disconnectedReason);
        }
    }

    private void handleOpenSocket() {
        //#ifdef NMSP_PLATFORM_BLACKBERRY
//@        netSys = new NetworkSystemOEM(apnSetting, msgSys);
        //#elif NMSP_PLATFORM_J2ME
//@        netSys = new NetworkSystemJ2me("", msgSys);
        //#elif NMSP_PLATFORM_J2SE
//@        netSys = new NetworkSystemOEM(msgSys);
       //#elif NMSP_PLATFORM_ANDROID
        netSys = new NetworkSystemOEM(msgSys);
        //#endif

        //#ifdef NMSP_CPL_ENABLED
//@        openSocketTime = System.currentTimeMillis();
        //#endif

        if (isSSL)
            netSys.openSSLSocket(server, port, socketParams, this, this, null);
        else
            netSys.openSocket(server, port, socketParams, this, this, null);
    }

    private void handleSendCopConnect() {
        boolean pingIntervalPresent = false;
        String payloadStr = "<?xml version=\"1.0\"?>"
            + "<cc>"
            + "<s></s>"
            + "<t>" + ProtocolDefines.DEVICE_ID + "</t>"
            + "<b>" + ProtocolDefines.XMODE_BUILD_DATE + "</b>"
            + "<bs>" + XMODE_MAX_PAYLOAD_SIZE + "</bs>"
            + "<tsc>" + inputCodec + "</tsc>"
            + "<fsc>" + outputCodec + "</fsc>"
            + "<nmaid>" + appId + "</nmaid>"
            + "<uid>" + uid + "</uid>"
            + ((_refId != null) ? "<rootlog>" + _refId.substring(0, _refId.length() - 2) + "</rootlog>" : "")
            + (forLogOnly ? "<disablecalllogging/>" : "" );

        Enumeration params = parameters.elements();
        while(params.hasMoreElements()) {
            Parameter param = (Parameter) params.nextElement();
            if(param.getType() == Parameter.Type.NMSP) {
                String valueString = new String(param.getValue());
                payloadStr += "<nmsp p=\"" + param.getName() + "\" v=\"" + Util.escapeXML(valueString) + "\"/>";
                if(param.getName().equals(ProtocolDefines.XMODE_NMSP_PARAM_PING_INTERVAL_SECS)) {
                    pingIntervalPresent = true;
                }
            }
            if(param.getType() == Parameter.Type.APP) {
                String valueString = new String(param.getValue());
                payloadStr += "<app p=\"" + param.getName() + "\" v=\"" + Util.escapeXML(valueString) + "\"/>";
            }
            if (param.getName().equals(NMSPDefines.NMSP_DEFINES_ALLOW_MDS_MESSAGES)
                    && ((new String(param.getValue())).equals("TRUE") || (new String(param.getValue())).equals("true"))) {
                payloadStr += "<" + new String(param.getName()) + "/>";
            }
        }
        // Pings are disabled unless the application sets Ping_IntervalSecs
        if(!pingIntervalPresent) {
            payloadStr += "<nmsp p=\"" + ProtocolDefines.XMODE_NMSP_PARAM_PING_INTERVAL_SECS + "\" v=\"0\"/>";
        }

        payloadStr += "</cc>";

        byte[] payloadStrBytes = payloadStr.getBytes();
        int payloadStrLen = payloadStrBytes.length;
        int dataLen = payloadStrLen + ProtocolDefines.XMODE_CLIENT_COP_CONNECT_DATA_LEN_SIZE;
        int payloadLen = ProtocolDefines.XMODE_CLIENT_COP_CONNECT_DATA_TYPE_SIZE + dataLen;

        byte[] tokenStrBytes = accessToken.getBytes();
        int tokenStrLen = tokenStrBytes.length;
        int tokenLength = tokenStrLen + ProtocolDefines.XMODE_CLIENT_ACCESS_TOKEN_DATA_SIZE +
                                ProtocolDefines.XMODE_CLIENT_ACCESS_TOKEN_VERSION_SIZE +
                                ProtocolDefines.XMODE_CLIENT_ACCESS_TOKEN_TYPE_SIZE;
        int bufferLength = payloadLen + tokenLength;

        byte[] payload = new byte[bufferLength];
        int offset = 0;
        ByteConversion.intToBytes(payloadLen - ProtocolDefines.XMODE_CLIENT_COP_CONNECT_DATA_LEN_SIZE, payload, offset);
        offset += ProtocolDefines.XMODE_CLIENT_COP_CONNECT_DATA_LEN_SIZE;
        payload[offset] = ProtocolDefines.XMODE_COP_CONNECT_DATA_TYPE_GATEWAY;
        offset += ProtocolDefines.XMODE_CLIENT_COP_CONNECT_DATA_TYPE_SIZE;
        System.arraycopy(payloadStrBytes, 0, payload, offset, payloadStrLen);
        offset += payloadStrLen;

        ByteConversion.intToBytes(tokenLength - ProtocolDefines.XMODE_CLIENT_ACCESS_TOKEN_DATA_SIZE, payload, offset);
        offset += ProtocolDefines.XMODE_CLIENT_ACCESS_TOKEN_DATA_SIZE;
        payload[offset] = ProtocolDefines.XMODE_ACCESS_TOKEN_VERSION;
        offset += ProtocolDefines.XMODE_CLIENT_ACCESS_TOKEN_VERSION_SIZE;
        payload[offset] += ProtocolDefines.XMODE_ACCESS_TOKEN_TYPE_JWT;
        offset += ProtocolDefines.XMODE_CLIENT_ACCESS_TOKEN_TYPE_SIZE;
        System.arraycopy(tokenStrBytes, 0, payload, offset, tokenStrLen);

        byte[] buffer = ProtocolBuilder.buildXModeBuf(ProtocolDefines.XMODE_PROTOCOL_COP, ProtocolDefines.XMODE_VERSION_COP,
                ProtocolDefines.XMODE_COP_COMMAND_CONNECT, payload);

        //#ifdef NMSP_CPL_ENABLED
//@        CPLRecord record = new CPLRecord(System.currentTimeMillis(), "request_xmode_send_cop_connect");
//@        cplXmodeWriteMsgs.addElement(record);
        //#endif
        sendXModeMsg(buffer, SEND_MSG_PRIORITY_HIGH, NET_CONTEXT_SEND_COP_CONNECT);

        copConnectTask = new TimerSystemTask() {
            public void run() {
                try {
                    //#ifdef NMSP_LOG
                    if(log.isErrorEnabled())
                        log.error("XMode.handleSendCopConnect() COP CONNECT timed out. ");
                    //#endif

                    sendCmdMsg(CMD_COP_CONNECT_TIMED_OUT, null);
                } catch (Exception e) {
                    //#ifdef NMSP_LOG
                    if(log.isErrorEnabled())
                        log.error("XMode.handleSendCopConnect() " + e.getClass().getName() + " " + e.getMessage());
                    //#endif
                }
            }
        };

        msgSys.scheduleTask(copConnectTask, 1000 * connectionTimeout);
    }

    private void handleSendCopConfirm() {
        byte[] hashedSessionId = MiscSystem.HMAC_SHA1(this.applicationKey, this.sessionId);

        int payloadLen = ProtocolDefines.XMODE_CLIENT_COP_CONFIRM_DATA_LEN_SIZE + hashedSessionId.length;
        byte[] payload = new byte[payloadLen];
        ByteConversion.intToBytes(hashedSessionId.length, payload, 0);
        System.arraycopy(hashedSessionId, 0, payload, ProtocolDefines.XMODE_CLIENT_COP_CONFIRM_DATA_LEN_SIZE, hashedSessionId.length);

        byte[] buffer = ProtocolBuilder.buildXModeBuf(ProtocolDefines.XMODE_PROTOCOL_COP, ProtocolDefines.XMODE_VERSION_COP,
                ProtocolDefines.XMODE_COP_COMMAND_CONFIRM, payload);
        sendXModeMsg(buffer, SEND_MSG_PRIORITY_HIGH, NET_CONTEXT_SEND_COP_CONFIRM);
    }

    private void handleSendCopDisconnect() {
        byte[] payload = new byte[ProtocolDefines.XMODE_CLIENT_COP_DISCONNECT_CODE_SIZE];
        ByteConversion.shortToBytes(ProtocolDefines.XMODE_DISCONNECT_CODE_NORMAL, payload, 0);
        byte[] buffer = ProtocolBuilder.buildXModeBuf(ProtocolDefines.XMODE_PROTOCOL_COP, ProtocolDefines.XMODE_VERSION_COP,
                ProtocolDefines.XMODE_COP_COMMAND_DISCONNECT, payload);
        //#ifdef NMSP_CPL_ENABLED
//@        CPLRecord record = new CPLRecord(System.currentTimeMillis(), "request_xmode_send_cop_disconnect");
//@        cplXmodeWriteMsgs.addElement(record);
        //#endif
        /*HG*///Clear waiting operations before disconnect so they do not cause next command to fail.
        if (netSys != null && socket != null)
            netSys.clearPendingOps(socket);
        sendXModeMsg(buffer, SEND_MSG_PRIORITY_HIGH, NET_CONTEXT_SEND_COP_DISCONNECT);
    }

    private void handleReadSocketXModeMsgHeader() {
        /** USE PARTIAL, but compare if we could read the full length **/
        /** disconnect the socket if we could not read the full length **/
        if (netSys != null && socket != null) {
            byte[] headerBuf = new byte[ProtocolDefines.XMODE_PACKET_HEADER_SIZE];
            //#ifdef NMSP_CPL_ENABLED
//@            NetworkSystem.CPLSocketReadContext context = new NetworkSystem.CPLSocketReadContext();
//@            context.normalContext = NET_CONTEXT_READ_XMODE_HEADER;
//@            cplXmodeReadRequests.addElement(new CPLRecord(System.currentTimeMillis(), "XMode_Request_Read_Header"));
            //#else
            String context = NET_CONTEXT_READ_XMODE_HEADER;
            //#endif
            netSys.socketRead(socket, NetworkReadMode.NETWORK_READ_FULL,
                    headerBuf, 0, ProtocolDefines.XMODE_PACKET_HEADER_SIZE, this, context);
        }
    }

    private void handleReadSocketXModeMsgPayload() {
        /** at this point, we will know the length of the payload is greater than zero **/
        if (this.header.len > 0 && this.header.len <= (XMODE_MAX_PAYLOAD_SIZE))
        {
            byte[] payloadBuf = new byte[this.header.len];
            if (netSys != null && socket != null) {
                //#ifdef NMSP_CPL_ENABLED
//@                NetworkSystem.CPLSocketReadContext context = new NetworkSystem.CPLSocketReadContext();
//@                context.normalContext = NET_CONTEXT_READ_XMODE_PAYLOAD;
//@                cplXmodeReadRequests.addElement(new CPLRecord(System.currentTimeMillis(), "XMode_Request_Read_Payload"));
                //#else
                String context = NET_CONTEXT_READ_XMODE_PAYLOAD;
                //#endif
                 netSys.socketRead(socket, NetworkReadMode.NETWORK_READ_FULL,
                    payloadBuf, 0, payloadBuf.length, this, context);
            }
        }
    }

    private void handleCloseSocket() {
        if (netSys != null && socket != null)
             netSys.closeSocket(socket);
    }

    private void handleSendCopPingResponse() {
        byte[] payload = new byte[ProtocolDefines.XMODE_SERVER_COP_PING_REQUEST_ID_SIZE
                                  + ProtocolDefines.XMODE_CLIENT_COP_PING_RESPONSE_LATENCY_SIZE];
        ByteConversion.intToBytes(pingRequestId, payload, 0);
        ByteConversion.intToBytes(ProtocolDefines.XMODE_PING_RESPONSE_ZERO_LATENCY, payload, ProtocolDefines.XMODE_SERVER_COP_PING_REQUEST_ID_SIZE);

        byte[] buffer = ProtocolBuilder.buildXModeBuf(ProtocolDefines.XMODE_PROTOCOL_COP, ProtocolDefines.XMODE_VERSION_COP,
                ProtocolDefines.XMODE_COP_COMMAND_PING_RESPONSE, payload);

        //#ifdef NMSP_CPL_ENABLED
//@        CPLRecord record = new CPLRecord(System.currentTimeMillis(), "request_xmode_send_cop_ping_response");
//@        cplXmodeWriteMsgs.addElement(record);
        //#endif
        sendXModeMsg(buffer, SEND_MSG_PRIORITY_HIGH, NET_CONTEXT_SEND_COP_PING_RESPONSE);
    }

    private void handleCopConnectTimeout()
    {
        if (state == STATE_CONNECTED) {
            state = STATE_CLOSING;
            socketClosedReason = XMODE_SOCKET_CLOSED_COP_CONNECT_TIMEOUT;
          //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("XMode.handleCopConnectTimeout() COP CONNECT timed out. ");
          //#endif
            sendCmdMsg(CMD_CLOSE_SOCKET, null);
        }
    }

    private void handleMessageSentConfirm(String context) {
        // pass null for now, pass more information later
        // depending on requirements
        xmodeListener.xmodeMsgSent(context, null);
    }

    private void handleMessageNotSentNotice(String context) {
        // pass null for now, pass more information later
        // depending on requirements
        xmodeListener.xmodeMsgNotSent(context, null);
    }

    /**********************************
     * XMode protocol parsing methods *
     **********************************/
    private void parseXModeMsg(XModeMsgHeader header, byte[] payloadBuf) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("XMode.parseXModeMsg() protocol: " + header.protocol + " cmd: " + header.cmd);
        //#endif
        //#ifdef NMSP_CPL_ENABLED
//@        cplXmodeReadMsg(header);
        //#endif
        switch (header.protocol) {
            case ProtocolDefines.XMODE_PROTOCOL_COP:
                switch (header.cmd) {
                    case ProtocolDefines.XMODE_COP_COMMAND_CONNECTED:
                    {
                        //#ifdef NMSP_CPL_ENABLED
//@                        connectionEstablishedTime = System.currentTimeMillis();
                        //#endif
                        parseXModeMsgCopConnected(payloadBuf);

                        createChildEvent(SessionEvent.NMSP_CALLLOG_SOCKETOPENED_EVENT);

                        idleSessionTask = new TimerSystemTask() {
                            public void run() {
                                try {
                                    socketClosedReason = XMODE_SESSION_IDLE_FOR_TOO_LONG;
                                  //#ifdef NMSP_LOG
                                    if(log.isErrorEnabled())
                                        log.error("Session Idle for too long, longer than [" + idleSessionTimeout + "] (initiated from XMode.parseCopConnected)");
                                  //#endif
                                    state = STATE_CLOSING;
                                    sendCmdMsg(CMD_CLOSE_SOCKET, null);
                                } catch (Exception e) {
                                    //#ifdef NMSP_LOG
                                    if(log.isErrorEnabled())
                                        log.error("XMode.parseXModeMsg() " + e.getClass().getName() + " " + e.getMessage());
                                    //#endif
                                }
                            }
                        };
                        msgSys.scheduleTask(idleSessionTask, idleSessionTimeout * 1000);
                        break;
                    }
                    case ProtocolDefines.XMODE_COP_COMMAND_PING_REQUEST:
                    {
                        parseXModeMsgCopPingRequest(payloadBuf);
                        break;
                    }
                    case ProtocolDefines.XMODE_COP_COMMAND_DISCONNECT:
                    {
                        if (idleSessionTask != null)
                            msgSys.cancelTask(idleSessionTask);
                        parseXModeMsgCopDisconnect(payloadBuf);
                        break;
                    }
                    case ProtocolDefines.XMODE_COP_COMMAND_CONNECT_FAILED:
                    {
                        parseXModeMsgCopConnectFailed(payloadBuf);
                        break;
                    }
                    default:
                }
                /** forward the COP xmode message to functional module **/
                xmodeListener.xmodeMsgCallback(header, payloadBuf);
                break;
            case ProtocolDefines.XMODE_PROTOCOL_BCP:
            {
                if(idleSessionTask != null && msgSys.cancelTask(idleSessionTask)) {
                    idleSessionTask = new TimerSystemTask() {
                        public void run() {
                            try {
                                socketClosedReason = XMODE_SESSION_IDLE_FOR_TOO_LONG;
                              //#ifdef NMSP_LOG
                                if(log.isErrorEnabled())
                                    log.error("Session Idle for too long, longer than [" + idleSessionTimeout + "] (initiated from XMode.parseBcp)");
                              //#endif
                                state = STATE_CLOSING;
                                sendCmdMsg(CMD_CLOSE_SOCKET, null);
                            } catch (Exception e) {
                                //#ifdef NMSP_LOG
                                if(log.isErrorEnabled())
                                    log.error("XMode.parseXModeMsg() " + e.getClass().getName() + " " + e.getMessage());
                                //#endif
                            }
                        }
                    };
                    msgSys.scheduleTask(idleSessionTask, idleSessionTimeout * 1000);
                }

                if (header.cmd == ProtocolDefines.XMODE_BCP_COMMAND_BCP_LOG_RESPONSE) {
                    //#ifdef NMSP_LOG
                    if(log.isWarnEnabled())
                        log.warn("XMode.parseXModeMsg() BCP_LOG_RESPONSE is deprecated");
                    //#endif
                    parseXModeMsgBcpLogResponse(payloadBuf);
                }else {
                    /** forward the BCP xmode message to functional module **/
                    xmodeListener.xmodeMsgCallback(header, payloadBuf);
                }
                break;
            }
            case ProtocolDefines.XMODE_PROTOCOL_VAP:
            {
                if(idleSessionTask != null && msgSys.cancelTask(idleSessionTask)) {
                    idleSessionTask = new TimerSystemTask() {
                        public void run() {
                            try {
                                socketClosedReason = XMODE_SESSION_IDLE_FOR_TOO_LONG;
                              //#ifdef NMSP_LOG
                                if(log.isErrorEnabled())
                                    log.error("Session Idle for too long, longer than [" + idleSessionTimeout + "] (initiated from XMode.parseVap)");
                              //#endif
                                state = STATE_CLOSING;
                                sendCmdMsg(CMD_CLOSE_SOCKET, null);
                            } catch (Exception e) {
                                //#ifdef NMSP_LOG
                                if(log.isErrorEnabled())
                                    log.error("XMode.parseXModeMsg() " + e.getClass().getName() + " " + e.getMessage());
                                //#endif
                            }
                        }
                    };
                    msgSys.scheduleTask(idleSessionTask, idleSessionTimeout * 1000);
                }
                /** forward the VAP xmode message to functional module **/
                xmodeListener.xmodeMsgCallback(header, payloadBuf);
                break;
            }
            case ProtocolDefines.XMODE_PROTOCOL_BB_HANDSHAKE:
                break;
            default:
                //#ifdef NMSP_LOG
                if(log.isErrorEnabled())
                    log.error("XMode.parseXModeMsg() unknown protocol: " + Integer.toHexString(header.protocol));
                //#endif
                break;
        }
    }

    private void parseXModeMsgCopConnected(byte[] payloadBuf)
    {
        msgSys.cancelTask(copConnectTask);

        int offset = 0;
        sessionId = new byte[16];
        System.arraycopy(payloadBuf, 0, sessionId, 0, ProtocolDefines.XMODE_SERVER_SESSION_UUID_SIZE);
        offset += ProtocolDefines.XMODE_SERVER_SESSION_UUID_SIZE;

        msgSys.setSessionId(sessionId);
        netSys.setSessionId(socket, sessionId);

        //#ifdef NMSP_LOG
        sessionIdStr = Log.FormatUuid(sessionId);
        log.setCurrentSession(sessionIdStr);
        try {
        if(log.isDebugEnabled())
            log.debug("Received COP_Connected " + sessionIdStr);
        //#endif

        //sendCmdMsg(CMD_COP_CONFIRM, null);

        xmodeListener.copConnected();
        //#ifdef NMSP_LOG
        } finally {
            log.unsetCurrentSession();
        }
        //#endif
    }

    private void parseXModeMsgCopPingRequest(byte[] payload) {
        pingRequestId = ByteConversion.bytesToInt(payload, 0);
        /** NO NEED to parse this param now
        int pingPreRoundTripTime =
            ByteConversion.bytesToInt(payload, XModeCore.XMODE_SERVER_COP_PING_REQUEST_ID_SIZE);
        **/
        //#ifdef NMSP_CPL_ENABLED
//@        cplXmodeReadMsgs.addElement(new CPLRecord(System.currentTimeMillis(), "read_xmode_msg[cop|ping_request|id:[" + pingRequestId + "]]"));
        //#endif
        sendCmdMsg(CMD_COP_PING_RESPONSE, null);
    }

    private void parseXModeMsgCopDisconnect(byte[] payload) {
        /** expose cop disconnected reason code **/
        disconnectedReason = ByteConversion.bytesToShort(payload, 0);

        state = STATE_CLOSING;
        socketClosedReason = XMODE_SOCKET_CLOSED_COP_DISCONNECT;
      //#ifdef NMSP_LOG
        if(log.isWarnEnabled())
            log.warn("XMode.parseXModeMsgCopDisconnect() Received COP DISCONNECT. ");
      //#endif
        sendCmdMsg(CMD_CLOSE_SOCKET, null);
    }

    private void parseXModeMsgCopConnectFailed(byte[] payload) {
        //#ifdef LOG
//@        Log.entering(Log.ClassXMode, Log.MethodParseXModeMsgCopConnectFailed);
        //#endif

        short failureCode;
        int payloadLen, offset = 0;
        failureCode = ByteConversion.bytesToShort(payload, 0);
        offset += ProtocolDefines.XMODE_SERVER_COP_CONNECT_FAILURE_CODE_SIZE;
        String failureText = "";
        payloadLen = ByteConversion.bytesToInt(payload, offset);
        offset += ProtocolDefines.XMODE_SERVER_COP_CONNECT_FAILED_PAYLOAD_SIZE;
        try
        {
            failureText = new String(payload, offset, payloadLen, "UTF-8");
        } catch (UnsupportedEncodingException uee)
        {
            //#ifdef LOG
//@            log.error(Log.MethodParseXModeMsgCopConnectFailed
//@                    + " got an exp - [" + UnsupportedEncodingException.class.getName() + "]"
//@                    + " message - [" + uee.getMessage() + "]");
            //#endif
        }
        //#ifdef LOG
//@        log.debug(Log.MethodParseXModeMsgCopConnectFailed
//@                + " failureCode - [" + failureCode + "]"
//@                + " failureText - [" + failureText + "]");
        //#endif

        socketClosedReason = XMODE_SOCKET_CLOSED_COP_CONNECT_FAILED;
        disconnectedReason = failureCode;
        state = STATE_CLOSING;
      //#ifdef NMSP_LOG
        if(log.isErrorEnabled())
            log.error("XMode.parseXModeMsgCopConnectFailed() COP CONNECT failure. ");
      //#endif
        sendCmdMsg(CMD_CLOSE_SOCKET, null);
    }

    private void parseXModeMsgBcpLogResponse(byte[] payload)
    {

    }

    /*****************************************************
     * send a command as a message to the message system *
     *****************************************************/
    private void sendCmdMsg(byte cmd, Object data) {
        try {
            msgSys.send(new MessageData(cmd, data), this, Thread.currentThread(), msgSys.getVRAddr()[0]);
        } catch (Exception e) {
            //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("XMode.sendCmdMsg() " + e.getMessage());
            //#endif
        }
    }

    /******************************************************
     * implementation of the MessageSystem.MessageHandler *
     ******************************************************/
    public void handleMessage(Object msgData, Object msgSender) {
        MessageData xmodeMsgData = (MessageData) msgData;
        switch (xmodeMsgData.command) {
            case CMD_CONNECT:
                //#ifdef NMSP_LOG
                if(log.isDebugEnabled())
                    log.debug("XMode.handleMessage() CMD_CONNECT");
                //#endif
                handleConnect();
                break;
            case CMD_OPEN_SOCKET:
                //#ifdef NMSP_LOG
                if(log.isDebugEnabled())
                    log.debug("XMode.handleMessage() CMD_OPEN_SOCKET");
                //#endif
                if(state == STATE_CONNECTING) {
                    handleOpenSocket();
                }
                else {
                    xmodeListener.socketClosed(socketClosedReason, disconnectedReason);
                }
                break;
            case CMD_COP_CONNECT:
                //#ifdef NMSP_LOG
                if(log.isDebugEnabled())
                    log.debug("XMode.handleMessage() CMD_COP_CONNECT");
                //#endif
                if(state == STATE_CONNECTING) {
                    handleSendCopConnect();
                    state = STATE_CONNECTED;
                    xmodeListener.socketOpened();
                }
                break;
            case CMD_READ_SOCKET_MSG_HEADER:
                handleReadSocketXModeMsgHeader();
                break;
            case CMD_READ_SOCKET_MSG_PAYLOAD:
                handleReadSocketXModeMsgPayload();
                break;
            case CMD_DISCONNECT:
                //#ifdef NMSP_LOG
                if(log.isDebugEnabled())
                    log.debug("XMode.handleMessage() CMD_DISCONNECT");
                //#endif
                handleSendCopDisconnect();
                break;
            case CMD_CLOSE_SOCKET:
                //#ifdef NMSP_LOG
                if(log.isDebugEnabled())
                    log.debug("XMode.handleMessage() CMD_CLOSE_SOCKET");
                //#endif
                handleCloseSocket();
                break;
            case CMD_COP_PING_RESPONSE:
                //#ifdef NMSP_LOG
                if(log.isDebugEnabled())
                    log.debug("XMode.handleMessage() CMD_COP_PING_RESPONSE");
                //#endif
                handleSendCopPingResponse();
                break;
            case CMD_COP_CONNECT_TIMED_OUT:
                //#ifdef NMSP_LOG
                if(log.isDebugEnabled())
                    log.debug("XMode.handleMessage() CMD_COP_CONNECT_TIMED_OUT");
                //#endif
                handleCopConnectTimeout();
                break;
            case CMD_COP_CONFIRM:
                //#ifdef NMSP_LOG
                if(log.isDebugEnabled())
                    log.debug("XMode.handleMessage() CMD_COP_CONFIRM");
                //#endif
                handleSendCopConfirm();
                break;
            case CMD_MESSAGE_SENT_CONFIRM:
                handleMessageSentConfirm((String) xmodeMsgData.data);
                break;
            case CMD_MESSAGE_NOT_SENT_NOTICE:
                handleMessageNotSentNotice((String) xmodeMsgData.data);
                break;
            case CMD_SEND_LOW_PRIORITY_MSG:
                //#ifdef NMSP_LOG
                if(log.isDebugEnabled())
                    log.debug("XMode.handleMessage() CMD_SEND_LOW_PRIORITY_MSG");
                //#endif
                sendXModeMsg(null, SEND_MSG_PRIORITY_PROMOTED, null);
                break;
        }
    }

    public boolean isNetworkHealthy() {
        NetworkSystem net = netSys;
        return net != null ? net.isNetworkHealthy() : false;
    }
    public void clearPendingNetworkOps() {
        netSys.clearPendingOps(socket);
    }

    public void createSessionEvent(SessionEvent parent) {
        if (parent != null) {
            this.sessionEvent = parent.createChildEventBuilder(SessionEvent.NMSP_CALLLOG_NMSPSOCKET_EVENT).commit();
        }
    }

    private void createChildEvent(String eventName)
    {
        if (sessionEvent != null && !forLogOnly) {
            SessionEventBuilder bldr = sessionEvent.createChildEventBuilder(eventName);
            bldr.commit();
        }
    }

    public void setForLogOnly()
    {
        forLogOnly = true;
    }

    private void createRecordEvent(int bufferSize)
    {
        if(sessionEvent != null){
            SessionEventBuilder builder = sessionEvent.createChildEventBuilder("AudioBufferSent");
            try {
                builder.putInteger("buffer_size", bufferSize);
            } catch (SessionEventAlreadyCommittedException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
            builder.commit();
        }
    }

    private void cancelGwRemoteEvent(String refId, int causeCode, String causeMessage)
    {
        if ( sessionEvent != null )
        {
            if (    (            refId != null  )
                 && ( refId.equals("") == false ) )
            {
                SessionEventBuilder builder = sessionEvent.createChildEventBuilder(SessionEventImpl.NMSP_CALLLOG_CANCEL_LOG_EVENT);

                try
                {
                    builder.putInteger (    "CauseCode",  causeCode    );
                    builder.putString  ( "CauseMessage",  causeMessage );

                    ((SessionEventBuilderImpl) builder).setEventToCancel(refId, SessionEvent.NMSP_CALLLOG_NMSPGW_EVENT);
                } catch (SessionEventAlreadyCommittedException e)
                {
                    //#ifdef NMSP_LOG
                if ( log.isErrorEnabled() )
                    log.error("XMode.cancelGwRemoteEvent() the builder is already committed.");
                    //#endif
                }
                builder.commit();
            }
        }
    }

    //#ifdef NMSP_CPL_ENABLED
//@    private void cplXmodeReadMsg(XModeMsgHeader header) {
//@       CPLRecord record = new CPLRecord(System.currentTimeMillis(), "read_xmode_msg[");
//@       switch (header.protocol) {
//@           case ProtocolDefines.XMODE_PROTOCOL_COP:
//@           {
//@               record.type += "cop|";
//@               switch (header.cmd) {
//@                   case ProtocolDefines.XMODE_COP_COMMAND_CONNECTED:
//@                       record.type += "connected";
//@                       break;
//@                   case ProtocolDefines.XMODE_COP_COMMAND_PING_REQUEST:
//@                       //record.type += "ping_request"; // will be logged inside the parser for this message
//@                       return;
//@                   case ProtocolDefines.XMODE_COP_COMMAND_DISCONNECT:
//@                       record.type += "disconnect";
//@                       break;
//@                   case ProtocolDefines.XMODE_COP_COMMAND_CONNECT_FAILED:
//@                       record.type += "connect_failed";
//@                       break;
//@                   default:
//@                       record.type += Integer.toHexString(header.cmd);
//@               }
//@               break;
//@           }
//@           case ProtocolDefines.XMODE_PROTOCOL_BCP:
//@           {
//@               record.type += "bcp|";
//@               switch (header.cmd) {
//@                   case ProtocolDefines.XMODE_BCP_COMMAND_BCP_DATA:
//@                       record.type += "data";
//@                       break;
//@                   case ProtocolDefines.XMODE_BCP_COMMAND_RECOGNITION_COMPLETE:
//@                       record.type += "recognition_compl";
//@                       break;
//@                   case ProtocolDefines.XMODE_BCP_COMMAND_RESPONSE:
//@                       record.type += "response";
//@                       break;
//@                   case ProtocolDefines.XMODE_BCP_COMMAND_SET_PARAMS_COMPLETE:
//@                       record.type += "set_params_compl";
//@                       break;
//@                   default:
//@                       record.type += Integer.toHexString(header.cmd);
//@               }
//@               break;
//@           }
//@           case ProtocolDefines.XMODE_PROTOCOL_VAP:
//@           {
//@               record.type += "vap|";
//@               switch (header.cmd) {
//@                   case ProtocolDefines.XMODE_VAP_COMMAND_PLAY:
//@                       record.type += "play";
//@                       break;
//@                   case ProtocolDefines.XMODE_VAP_COMMAND_PLAY_BEGIN:
//@                       record.type += "begin";
//@                       break;
//@                   case ProtocolDefines.XMODE_VAP_COMMAND_PLAY_END:
//@                       record.type += "end";
//@                       break;
//@                   default:
//@                       record.type += Integer.toHexString(header.cmd) + "";
//@               }
//@               break;
//@           }
//@           case ProtocolDefines.XMODE_PROTOCOL_BB_HANDSHAKE:
//@           {
//@               record.type += "bb|handshake";
//@               break;
//@           }
//@       }
//@       record.type = record.type + "|" + header.len + "]";
//@       cplXmodeReadMsgs.addElement(record);
//@    }
    //#endif
    //#ifdef NMSP_SOCKET_WRITE_PADDING
//@    private byte[] preparePaddingBuffer(String paddingStr) {
//@        byte binaryLength[] = new byte[4];
//@        ByteArrayOutputStream binaryPayloadOs = new ByteArrayOutputStream();
//@        String payloadStr = "<sp>" + "<rid>0</rid>";
//@        payloadStr += "<gwlog p=\"SOCKET PADDING\" v=\"1\"/>";
//@        payloadStr += "</sp>";
//@        byte buf[] = ("[" + paddingStr + "]").getBytes();
//@        ByteConversion.intToBytes(buf.length, binaryLength, 0);
//@        binaryPayloadOs.write(binaryLength, 0, 4);
//@        binaryPayloadOs.write(buf, 0, buf.length);
//@
//@        byte[] primaryPayload = payloadStr.getBytes();
//@        byte[] binaryPayload = binaryPayloadOs.toByteArray();
//@        byte tranId = (byte)0x0;
//@
//@        int offset = 0;
//@        byte[] payload = new byte[ProtocolDefines.XMODE_BCP_TRANSACTION_ID_SIZE
//@                + ProtocolDefines.XMODE_XML_PAYLOAD_SIZE + primaryPayload.length];
//@        payload[offset] = tranId;
//@        offset += ProtocolDefines.XMODE_BCP_TRANSACTION_ID_SIZE;
//@        ByteConversion.intToBytes(primaryPayload.length, payload, offset);
//@        offset += ProtocolDefines.XMODE_XML_PAYLOAD_SIZE;
//@        System.arraycopy(primaryPayload, 0, payload, offset, primaryPayload.length);
//@
//@        payload = ProtocolBuilder.appendBCPSessionUUID(payload, sessionId);
//@
//@        ByteArrayOutputStream message = new ByteArrayOutputStream();
//@        message.write(payload, 0, payload.length);
//@        message.write(binaryPayload, 0, binaryPayload.length);
//@
//@        byte[] buffer = ProtocolBuilder.buildXModeBuf(
//@                ProtocolDefines.XMODE_PROTOCOL_BCP,
//@                ProtocolDefines.XMODE_VERSION_BCP,
//@                ProtocolDefines.XMODE_BCP_COMMAND_SET_PARAMS, message.toByteArray());
//@
//@        return buffer;
//@    }
    //#endif
}
