package com.nuance.nmsp.client2.sdk.components.core.calllog;

import java.util.Map;

public interface SessionEvent
{
	public final static String NMSP_CALLLOG_NMSPSDK_EVENT            = "CLIENT";
	public final static String NMSP_CALLLOG_SHUTDOWN_EVENT           = "NMSP_ShutDown";


	public final static String NMSP_CALLLOG_NMSPSESSION              = "NMSPSession";
	public final static String NMSP_CALLLOG_CONNECT_EVENT            = "Connect";
	public final static String NMSP_CALLLOG_CONNECTED_EVENT          = "ConnectionEstablished";

	public final static String NMSP_CALLLOG_NMSPGW_EVENT             = "NMSP_GATEWAY";

	public final static String NMSP_CALLLOG_NMSPSOCKET_EVENT         = "NMSPSocket";
	public final static String NMSP_CALLLOG_OPENSOCKET_EVENT         = "OpenSocket";
	public final static String NMSP_CALLLOG_SOCKETOPENED_EVENT       = "SocketOpened";
	public final static String NMSP_CALLLOG_CLOSESOCKET_EVENT        = "CloseSocket";
	public final static String NMSP_CALLLOG_SOCKETCLOSED_EVENT       = "SocketClosed";

	public final static String NMSP_CALLLOG_NMASCOMMAND_EVENT        = "NMASCommand";
	public final static String NMSP_CALLLOG_NMASRESPONSE_EVENT       = "NMASResponse";
	public final static String NMSP_CALLLOG_COMMANDABORT_EVENT       = "CommandAbort";
	public final static String NMSP_CALLLOG_COMMANDEXPIRED_EVENT     = "CommandExpired";
	public final static String NMSP_CALLLOG_NMASAUDIOPARAM_EVENT     = "NMASAudioParameter";


	public final static String NMSP_CALLLOG_RECORDER_EVENT           = "NMSPRecorder";
	public final static String NMSP_CALLLOG_RECORDERSTART_EVENT      = "RecorderStart";
	public final static String NMSP_CALLLOG_RECORDERSTOP_EVENT       = "RecorderStop";
	public final static String NMSP_CALLLOG_RECORDSTREAMSTART_EVENT  = "StreamStart";
	public final static String NMSP_CALLLOG_RECORDSTREAMSTOP_EVENT   = "StreamStop";

	public final static String NMSP_CALLLOG_PLAYER_EVENT             = "NMSPPlayer";
	public final static String NMSP_CALLLOG_PLAYERSTART_EVENT        = "PlayerStart";
	public final static String NMSP_CALLLOG_PLAYERSTOP_EVENT         = "PlayerStop";
	public final static String NMSP_CALLLOG_PLAYSTREAMSTART_EVENT    = "StreamStart";
	public final static String NMSP_CALLLOG_PLAYSTREAMSTOP_EVENT     = "StreamStop";

	public final static String NMSP_CALLLOG_APP_REMOTEEVENT          = "APP_REMOTEEVENT";
	
	public final static String NMSP_CALLLOG_ORIGIN_CLIENT            = "Client";
	
    /**
     * Method to create a new session event builder as a child of the current session event.
     * 
     * @param eventName
     * 
     * @return SessionEventBuilder
     */
    SessionEventBuilder createChildEventBuilder(String eventName);
       
    /**
     * Method to create a new session event builder as a child of the current session event.
     * 
     * @param eventName
     * 
     * @param schemaVersion The call log schema Version used by identifying the current session
     *                      event's schema, could be NULL or Empty string if there is no schema
     *                      version to use.
     * 
     * @return SessionEventBuilder
     */
    SessionEventBuilder createChildEventBuilder(String eventName, String schemaVersion);    

    /**
     * Retrieve the session event Id. This id is the concatenation of the root parent id and
     * the sequence id of the event.
     *
     * @return session event id. (must not be NULL).
     */
    String getId();
    
    /**
     * Retrieve the session event name.
     * 
     * @return session event name. (must not be NULL).
     */
    String getName();
    
    /**
     * Retrieve the remote reference ids and their associated event names that have been created
	 * for this event.
	 * 
	 * The returned the map object contains the list of name, id pairs. If the current session
	 * event does not have any remote references. The map object will contain 0 elements.
     * 
     * @return List of all the reference <name, id> pairs generated for this SessionEvent
     */
    Map getRefIds();
}
