package com.nuance.nmsp.client2.sdk.components.core.internal.calllog;

import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Iterator;

import com.nuance.nmsp.client2.sdk.components.core.calllog.SessionEvent;
import com.nuance.nmsp.client2.sdk.components.core.calllog.SessionEventBuilder;
import com.nuance.nmsp.client2.sdk.components.core.internal.pdx.PDXDictionary;
import com.nuance.nmsp.client2.sdk.components.core.internal.pdx.PDXSequence;
import com.nuance.nmsp.client2.sdk.components.core.pdx.Dictionary;
import com.nuance.nmsp.client2.sdk.components.core.pdx.Sequence;

//#ifdef NMSP_LOG
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory.Log;
//#endif

public class SessionEventImpl implements SessionEvent
{
    //#ifdef NMSP_LOG
    private static final Log log = LogFactory.getLog(CalllogImpl.class);
    //#endif
	
    public final static String  NMSP_CALLLOG_META_MERGED_NAME           = "Merged";
    public final static String  NMSP_CALLLOG_META_APPLICATION_NAME      = "Application";
    public final static String  NMSP_CALLLOG_META_SCHEMA_VERSION        = "SchemaVersion";
    public final static String  NMSP_CALLLOG_META_CHANNEL               = "Channel";
    public final static String  NMSP_CALLLOG_META_COMPLETED_TIMESTAMP   = "CompletedTimestamp";
    public final static String  NMSP_CALLLOG_META_HOSTNAME              = "Hostname";
    public final static String  NMSP_CALLLOG_META_RETENTION_DAYS        = "RetentionDays";
    public final static String  NMSP_CALLLOG_META_ROOT_PARENT_ID        = "RootParentId";
    public final static String  NMSP_CALLLOG_META_SEQUENCE_ID           = "SeqId";
    public final static String  NMSP_CALLLOG_META_PARENT_ID             = "ParentSeqId";
    public final static String  NMSP_CALLLOG_META_NAME                  = "Name";
    public final static String  NMSP_CALLLOG_META_TIMESTAMP             = "Timestamp";
    public final static String  NMSP_CALLLOG_META_REFERENCE_ID          = "RefId";
    public final static String  NMSP_CALLLOG_META_CANCEL                = "Cancel";
    public final static String  NMSP_CALLLOG_META_EVENT_ID_TO_CANCEl    = "EventIdToCancel";
    public final static String  NMSP_CALLLOG_META_EVENT_NAME_TO_CANCEl  = "EventNameToCancel";
    
    // Cancels a specific call log event regardless of its type. Cancellation of the event
    // does not remove the original event from the call log. It only indicates that the event
    // originator requests to disregard it. The original event may be complete or not.
    public final static String NMSP_CALLLOG_CANCEL_LOG_EVENT         = "CancelLogEvent";
    
    public final static int NMSP_CALLLOG_EVENT_CANCEL_CAUSE_CODE_OTHER                = 0;
    public final static int NMSP_CALLLOG_EVENT_CANCEL_CAUSE_CODE_CONNECTION_ERROR     = 1;
    public final static int NMSP_CALLLOG_EVENT_CANCEL_CAUSE_CODE_USER_INITIATED       = 2;
    public final static int NMSP_CALLLOG_EVENT_CANCEL_CAUSE_CODE_INTERNAL_APP_ERROR   = 500;

    SessionEventImpl _parent;
    RootSessionEventImpl _rootParent;
    CalllogImpl _calllog;
    // This holds the remote events objects
    List _remoteEvents = new LinkedList(); 
    // This holds only the remote event names until we can generate sequence ids
    List _remoteEventsNames = new LinkedList();
    // This holds the ref ids generated
    Map _refIds = new TreeMap();
    String _schemaVersion;
    
    // properties unknown until later
    int    _sequenceId;
    String _name;
    long   _timestamp;
    
    // properties for cancel an event
    String   _eventIdToCancel = null;
    String _eventNameToCancel = null;
    
    // properties for committing
    protected Hashtable _table;
    

    SessionEventImpl(String eventName, String schemaVersion, SessionEventImpl parent, RootSessionEventImpl rootParent, CalllogImpl resource)
    {
    	_name = eventName;
    	_schemaVersion = schemaVersion;
        _parent = parent;
        _rootParent = rootParent;
        _calllog = resource;
    }
    
    public CalllogImpl getCalllog() 
    {
        return _calllog;
    }
    
	public SessionEvent getParent() {
		return _parent;
	}
	
    public void done(Hashtable table)
    {
        _table = table;
        
        _timestamp = System.currentTimeMillis();
        _calllog.addSessionEvent(this);
    }

    // sdk thread
    public byte[] getBinaryFormat()
    {
        // construct PDX binary
        Dictionary logDict = new PDXDictionary();

        // Add "meta" data
        Sequence metaSeq = buildMetaSeq();
    	logDict.addSequence("meta", metaSeq);
 
    	// Add "attr" data
        Sequence attrSeq = new PDXSequence();
        Iterator keyIt = _table.keySet().iterator();
        while (keyIt.hasNext())
        {
            String key = (String)keyIt.next();
            Object value = _table.get(key);
        
            addEntryToSequence(attrSeq, key, value);
        }
    	if (attrSeq.size() > 0) {
    		logDict.addSequence("attr", attrSeq);
    	}
    	
    	// Add "refids" data
    	_timestamp = System.currentTimeMillis();
    	if(_remoteEvents.size() > 0){
    		Sequence remoteEventsSeq = new PDXSequence();
    		while(_remoteEvents.size() > 0){
    			RemoteSessionEventImpl event = (RemoteSessionEventImpl) _remoteEvents.remove(0);
    			
    			Dictionary remoteEventDict = new PDXDictionary();
    			remoteEventDict.addUTF8String("rootParentId", _calllog.getRootParentId());
    			remoteEventDict.addUTF8String("seqId", event.getSeqId());
    			remoteEventDict.addUTF8String("eventName", event.getEventName());
    			remoteEventDict.addUTF8String("timestamp", ((Long)_timestamp).toString());
    			remoteEventDict.addUTF8String("refId", event.getRefId());
    			
    			remoteEventsSeq.addDictionary(remoteEventDict);
    		}
    		logDict.addSequence("refids", remoteEventsSeq);
    	}
    	else
    	{
    		Sequence remoteEventsSeq = new PDXSequence();
    		logDict.addSequence("refids", remoteEventsSeq);
    	}
        
    	return ((PDXDictionary) logDict).toByteArray();
    	
    }
    
    public Dictionary getDictionaryFormat()
    {    	
        // construct PDX binary
        Dictionary logDict = new PDXDictionary();

        // Add "meta" data
        Sequence metaSeq = buildMetaSeq();
    	logDict.addSequence("meta", metaSeq);
 
    	// Add "attr" data
        Sequence attrSeq = new PDXSequence();
        Iterator keyIt = _table.keySet().iterator();
        while (keyIt.hasNext())
        {
            String key = (String)keyIt.next();
            Object value = _table.get(key);
        
            addEntryToSequence(attrSeq, key, value);
        }
    	if (attrSeq.size() > 0) {
    		logDict.addSequence("attr", attrSeq);
    	}
    	
    	// Add "refids" data
    	_timestamp = System.currentTimeMillis();
		Sequence remoteEventsSeq = new PDXSequence();
    	if ( _remoteEvents.size() > 0 )
    	{

    		while ( _remoteEvents.size() > 0 )
    		{
    			RemoteSessionEventImpl event = (RemoteSessionEventImpl) _remoteEvents.remove(0);
    			
    			Dictionary remoteEventDict = new PDXDictionary();
    			remoteEventDict.addUTF8String("rootParentId", _calllog.getRootParentId());
    			remoteEventDict.addUTF8String("seqId", event.getSeqId());
    			remoteEventDict.addUTF8String("eventName", event.getEventName());
    			remoteEventDict.addUTF8String("timestamp", ((Long)_timestamp).toString());
    			remoteEventDict.addUTF8String("refId", event.getRefId());
    			
    			remoteEventsSeq.addDictionary(remoteEventDict);
    		}
    	}
		logDict.addSequence("refids", remoteEventsSeq);
    	return logDict;    	
    }
    
    /**private void printKeyValue(int numOfTabs, StringBuffer strBuf, Object key, String value)
    {
        for (int i = 0; i < numOfTabs; i++)
            strBuf.append("    ");
        strBuf.append(key + ":" + value + "\n");
    }**/
    
    // helper functions
    private String attrTypeToPDXType(Object value)
    {
    	if (value instanceof byte []) {
            return "bin";
    	}
    	else if (value instanceof Boolean) {
            return "bool";
    	}
    	else if (value instanceof Double) {
            return "float";
    	}
    	else if (value instanceof Integer) {
            return "int";
    	}
    	else if (value instanceof Long) {
            return "long";
    	}
    	else if (value instanceof String) {
            return "str";
    	}
    	else if (value instanceof SessionEvent) {
            return "str";
    	}
    	
    	return "unknown";
    }
    
    protected boolean addEntryToSequence(Sequence seq, String key, Object value)
    {
        Dictionary dict = new PDXDictionary();
        String type = attrTypeToPDXType(value);
        
        if (type.compareTo("unknown") == 0) {
        	return false;
        }
        
        dict.addAsciiString("t", type);
        dict.addAsciiString("k", key);

    	if (value instanceof byte []) {
    		 dict.addByteString("v", (byte[])value);
    	}
    	else if (value instanceof Boolean) {
    		dict.addAsciiString("v", ((Boolean) value).toString());
    	}
    	else if (value instanceof Double) {
    		dict.addAsciiString("v", ((Double)value).toString());
    	}
    	else if (value instanceof Integer) {
    		dict.addInteger("v", ((Integer)value).intValue());
    	}
    	else if (value instanceof Long) {
    		dict.addAsciiString("v", ((Long)value).toString());
    	}
    	else if (value instanceof String) {
    		dict.addUTF8String("v", (String)value);
    	}
    	else if (value instanceof SessionEvent) {
    		String id = ((SessionEventImpl)value)._rootParent.getRootParentId() + "." + ((SessionEventImpl)value)._sequenceId;
    		dict.addAsciiString("v", (String)id);
    	}
    	else {
    		return false;
    	}

    	seq.addDictionary(dict);
    	return true;
    }
    
    protected void genSeqId()
    {
        // settle unknown properties
        _sequenceId = _rootParent.getSequenceId();
    }
    
    protected Sequence buildMetaSeq()
    {
        Sequence metaSeq = new PDXSequence();

        addEntryToSequence(metaSeq, NMSP_CALLLOG_META_ROOT_PARENT_ID, _rootParent.getRootParentId());
        
        if (_parent != null) {
        	addEntryToSequence(metaSeq, NMSP_CALLLOG_META_PARENT_ID, new Integer(_parent._sequenceId));
        }
        
    	addEntryToSequence(metaSeq, NMSP_CALLLOG_META_SEQUENCE_ID, new Integer(_sequenceId));
        
    	addEntryToSequence(metaSeq, NMSP_CALLLOG_META_NAME, _name);

    	addEntryToSequence(metaSeq, NMSP_CALLLOG_META_TIMESTAMP, new Long(_timestamp));
    	
    	if (_schemaVersion != null && !_schemaVersion.equals(""))
    	{
        	addEntryToSequence(metaSeq, NMSP_CALLLOG_META_SCHEMA_VERSION, _schemaVersion);	
    	}
    	
    	// Added the event cancellation meta properties
    	if (    (              _eventIdToCancel != null  )
    	     && (   _eventIdToCancel.equals("") == false )
    	     && (            _eventNameToCancel != null  )
    	     && ( _eventNameToCancel.equals("") == false ) )
    	{
    	    addEntryToSequence(metaSeq, NMSP_CALLLOG_META_EVENT_ID_TO_CANCEl, _eventIdToCancel);
    	    addEntryToSequence(metaSeq, NMSP_CALLLOG_META_EVENT_NAME_TO_CANCEl, _eventNameToCancel);
    	}
        
        return metaSeq;
    }
    
    public void addRemoteEventRefid(String eventName)
    {
    	_remoteEventsNames.add(eventName);
    }
    
    public void generateSeqIds()
    {
    	while ( _remoteEventsNames.size() > 0 )
    	{
    		String eventName = (String) _remoteEventsNames.remove(0);
    		RemoteSessionEventImpl remoteEvent = new RemoteSessionEventImpl(_rootParent.getRootParentId(),
    																	 _rootParent.getSequenceId(),
    																	 eventName);
    		_remoteEvents.add(remoteEvent);
    		_refIds.put(eventName, remoteEvent.getRefId());
    	}
    }
    
    //Override
    public Map getRefIds()
    {
    	return _refIds;
    }

    //Override
    public SessionEventBuilder createChildEventBuilder(String eventName)
    {
        return SessionEventBuilderImpl.createRegularSessionEventBuilder(this, eventName, null);
    }
    
    //Override
    public SessionEventBuilder createChildEventBuilder(String eventName, String schemaVersion)
    {
        return SessionEventBuilderImpl.createRegularSessionEventBuilder(this, eventName, schemaVersion);
    }

    //Override
	public String getId()
	{
		if ( _sequenceId == 0 )
		{
            //#ifdef NMSP_LOG
                    if (log.isErrorEnabled())
                        log.error("getSessionEventId() has been called before builder commit has completed");
            //#endif
			return null;
		}
		return _rootParent.getRootParentId() + "." + _sequenceId;
	}

	//Override
	public String getName()
	{
	    return _name;
	}
}
