/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client2.sdk.components.general;

//#ifdef NMSP_LOG
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory.Log;
//#endif
import com.nuance.nmsp.client2.sdk.common.defines.NMSPDefines;
import com.nuance.nmsp.client2.sdk.common.oem.api.MessageSystem;
import com.nuance.nmsp.client2.sdk.oem.MessageSystemOEM;


//#ifdef NMSP_PLATFORM_BLACKBERRY
//@import net.rim.blackberry.api.phone.Phone;
//@import net.rim.device.api.system.*;
//@import javax.microedition.io.*;
//@import java.io.*;
//@import java.util.*;
//#endif

/**
 * Manages the connection with the NMSP Gateway.
 * 
 * @author Nuance Communications Inc.
 */
public abstract class NMSPManager {
    //#ifdef NMSP_LOG	
    private final static Log log = LogFactory.getLog(NMSPManager.class);
    //#endif   
    /** The IP address of the NMSP Gateway */
    protected String gatewayIP;
    /** The port of the NMSP Gateway */
    protected short gatewayPort;
    protected MessageSystem msgSys;
    protected NMSPDefines.Codec inputCodec;
    protected NMSPDefines.Codec outputCodec;

    protected NMSPManager(String gatewayIP, short gatewayPort, NMSPDefines.Codec inputCodec, NMSPDefines.Codec outputCodec) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("in NMSPManager() gateway IP [" + gatewayIP   + "]" +
                    " Port ["       + gatewayPort + "]");
        //#endif

        /* verify parameters */
        String errorMsg = null;
        if (gatewayIP == null) {
            errorMsg = " gatewayIP is null";
        } else if (gatewayIP.length() == 0) {
            errorMsg = " gatewayIP is empty";
        } else if (gatewayIP.indexOf('.') != -1) {
            /* TODO */
            /* regexp "[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}" */
            /* errorMsg = " gatewayIP should be in format xxx.xxx.xxx.xxx"; */
        } else if (gatewayIP.indexOf('.') == -1) {
            /* TODO */
            /* && !gatewayIP.matches("[a-zA-Z0-9\\-]+")) */ 
            /* errorMsg = " gateway machine name should only contains [A-Z][a-z][0-9]and-"; */
        }

        if (gatewayPort <= 0)
            errorMsg = " gatewayPort should be greater than 0";

        if (errorMsg != null) {
            //#ifdef NMSP_LOG
            log.error("NMSPManager " + IllegalArgumentException.class.getName() + errorMsg);
            //#endif
            throw new IllegalArgumentException(errorMsg);
        }

        this.gatewayIP   = gatewayIP;
        this.gatewayPort = gatewayPort;
        this.inputCodec = inputCodec;
        this.outputCodec = outputCodec;
        msgSys = new MessageSystemOEM();
    }

    /**
     * Returns the MessageSystem object.
     */
    public MessageSystem getMsgSys() {
        return msgSys;
    }

    /**
     * Returns the NMSP Gateway IP address. 
     */    
    public String getGatewayIP() {
        return gatewayIP;    
    }

    /**
     * Returns the NMSP Gateway port number. 
     */        
    public short getGatewayPort() {
        return gatewayPort;
    }

    /**
     * Returns the input codec.
     */
    public NMSPDefines.Codec getInputCodec() {
        return this.inputCodec;
    }

    /**
     * Returns the output codec.
     */
    public NMSPDefines.Codec getOutputCodec() {
        return this.outputCodec;
    }
    
    /**
     * Sets input codec.
     */
    public void setInputCodec(NMSPDefines.Codec codec) {
        this.inputCodec = codec;
    }
    
    /**
     * Sets output codec.
     */
    public void setOutputCodec(NMSPDefines.Codec codec) {
        this.outputCodec = codec;
    }

    /**
     * Closes the connection between the manager object and the NMSP Gateway. 
     * Each instance of a manager must implement this method.
     */    
    public abstract void shutdown();

//#ifndef NMSP_PLATFORM_BLACKBERRY
    /**
     * Unsupported on this platform. Throws RuntimeException.
     */
    public static String discoverAPN(String gatewayIP, short port) throws RuntimeException {
        throw new RuntimeException("Unsupported");
    }

    /**
     * Unsupported on this platform.
     */
    public static boolean verifyAPN(String apnSetting, String gatewayIP, short port) {
        return false;
    }
//#else
//@    /**
//@     * Use this method to discover which APN string should be used to
//@     * establish a connection, in case RuntimeException is thrown and
//@     * the working APN string is not found, the user should go to find
//@     * out the valid apn setting from the carrier and manually enter
//@     * them in "Options->Advanced Options->TCP" on the blackberry
//@     * device, use "socket://IP:Port;deviceside=true" as the url to
//@     * open the socket.
//@     * 
//@     * @param gatewayIP the IP address of the official deployed gateway
//@     * @param port      the port to communicate with the gateway
//@     *
//@     * @return APN string that should be passed when creating the NMSPManager
//@     * @throws RuntimException if the working APN string is not found
//@     */
//@    public static String discoverAPN(String gatewayIP, short port) throws RuntimeException {
//@        return DiscoverApnUtil.discoverAPN(gatewayIP, port);
//@    }
//@
//@    /**
//@     * Use this method to verify if a certain APN setting will work on the
//@     * current handset. For instance, the third-party application lets the user
//@     * specify the APN settings, and lets the application call this method
//@     * to verify the if the APN settings entered by the user work.
//@     *  
//@     * @param apnSetting the APN string used to open a socket, including the access point IP
//@     *                   username and password: for example:
//@     *                   ";deviceside=true;apn=apnvalue;TunnelAuthUsername=username;TunnelAuthPassword=password",
//@     *                   or ";deviceside=true"
//@     * @param gatewayIP  the IP address of the deployed gateway
//@     * @param port       the port to communicate with the gateway
//@     * @return
//@     */
//@    public static boolean verifyAPN(String apnSetting, String gatewayIP, short port) {
//@        return DiscoverApnUtil.verifyAPN(apnSetting, gatewayIP, port);
//@    }
//@
//@    private static class DiscoverApnUtil {
//@        public static final String DEVICESIDE_TRUE  = "deviceside=true";
//@        public static final String DEVICESIDE_FALSE = "deviceside=false";
//@        public static final char   SEMICOLON        = ';';
//@        public static final String APN_EQUALS       = "apn=";
//@        public static final String USERNAME_EQUALS  = "TunnelAuthUsername=";
//@        public static final String PW_EQUALS        = "TunnelAuthPassword=";
//@        private final static String APN_NOT_FOUND   = "APN_NOT_FOUND";
//@
//@        private static final Hashtable APN_TABLE = new Hashtable();
//@        private static int apnIndex = 0;
//@        private static final String MOBILE_NUMBER = "#MOBILE_NUMBER#";
//@
//@        /** United States **/
//@        private static final int MCC_US_0x310 = 0x310;
//@        private static final int MCC_US_0x311 = 0x311;
//@        private static final int CARRIER_US_TMOBILE  = apnIndex++;
//@        private static final int CARRIER_US_CINGULAR = apnIndex++;
//@        private static final int CARRIER_US_ATNT     = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_US_TMOBILE),  "wap.voicestream.com||");                         /** verified **/
//@            subTable.put(new Integer(CARRIER_US_ATNT),     "WAP.CINGULAR|WAP@CINGULARGPRS.COM|CINGULAR1");   /** verified **/
//@            subTable.put(new Integer(CARRIER_US_CINGULAR), "isp.cingular|ISPDA@CINGULARGPRS.COM|CINGULAR1");
//@            APN_TABLE.put(new Integer(MCC_US_0x310), subTable);
//@            APN_TABLE.put(new Integer(MCC_US_0x311), subTable);
//@        }
//@
//@        /** Canada **/
//@        private static final int MCC_CA_0x302 = 0x302;
//@        private static final int CARRIER_CA_ROGERS = apnIndex++;
//@        private static final int CARRIER_CA_FIDO   = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_CA_ROGERS), "internet.com|wapuser1|wap");  /** verified **/
//@            subTable.put(new Integer(CARRIER_CA_FIDO),   "internet.fido.ca|fido|fido");
//@            APN_TABLE.put(new Integer(MCC_CA_0x302), subTable);
//@        }
//@
//@        /** United Kingdom **/
//@        private static final int MCC_UK_0x234 = 0x234;
//@        private static final int CARRIER_UK_VODAFONE_CONTRACT  = apnIndex++;
//@        private static final int CARRIER_UK_VODAFONE_PREPAID   = apnIndex++;
//@        private static final int CARRIER_UK_VODAFONE_2         = apnIndex++;
//@        private static final int CARRIER_UK_O2_CONTRACT        = apnIndex++;
//@        private static final int CARRIER_UK_O2_CONTRACT_2      = apnIndex++;
//@        private static final int CARRIER_UK_O2_PREPAID         = apnIndex++;
//@        private static final int CARRIER_UK_ORANGE             = apnIndex++;
//@        private static final int CARRIER_UK_ORANGE_CONTRACT    = apnIndex++;
//@        private static final int CARRIER_UK_ORANGE_PREPAID     = apnIndex++;
//@        private static final int CARRIER_UK_TMOBILE            = apnIndex++;
//@        private static final int CARRIER_UK_TMOBILE_ONE2ONE    = apnIndex++;
//@        private static final int CARRIER_UK_VIRGIN             = apnIndex++;
//@        private static final int CARRIER_UK_JERSEY_TELECOM     = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_UK_VODAFONE_CONTRACT), "wap.vodafone.co.uk|wap|wap");         /** verified **/
//@            subTable.put(new Integer(CARRIER_UK_VODAFONE_PREPAID),  "pp.vodafone.co.uk|wap|wap");
//@            subTable.put(new Integer(CARRIER_UK_VODAFONE_2),        "internet|web|web");
//@            subTable.put(new Integer(CARRIER_UK_O2_CONTRACT),       "mobile.o2.co.uk|web|password");       /** verified **/
//@            subTable.put(new Integer(CARRIER_UK_O2_CONTRACT_2),     "mobile.o2.co.uk|mobileweb|password");
//@            subTable.put(new Integer(CARRIER_UK_O2_PREPAID),        "payandgo.o2.co.uk|payandgo|payandgo");
//@            subTable.put(new Integer(CARRIER_UK_ORANGE),            "orangeinternet||");
//@            subTable.put(new Integer(CARRIER_UK_ORANGE_CONTRACT),   "orangewap|Orange|Multimedia");
//@            subTable.put(new Integer(CARRIER_UK_ORANGE_PREPAID),    "paygwap|Orange|Multimedia");
//@            subTable.put(new Integer(CARRIER_UK_TMOBILE),           "general.t-mobile.uk|user|mms");
//@            subTable.put(new Integer(CARRIER_UK_TMOBILE_ONE2ONE),   "general.t-mobile.uk|Username|one2one");
//@            subTable.put(new Integer(CARRIER_UK_VIRGIN),            "goto.virginmobile.uk|user|");
//@            subTable.put(new Integer(CARRIER_UK_JERSEY_TELECOM),    "pepper||");
//@            APN_TABLE.put(new Integer(MCC_UK_0x234), subTable);
//@        }
//@
//@        /** Australia **/
//@        private static final int MCC_AU_0x505 = 0x505;
//@        private static final int CARRIER_AU_TELSTRA = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_AU_TELSTRA), "telstra.internet||"); /** verified **/
//@            APN_TABLE.put(new Integer(MCC_AU_0x505), subTable);
//@        }
//@
//@        /** India **/
//@        private static final int MCC_IN_0x404 = 0x404;
//@        private static final int MCC_IN_0x405 = 0x405;
//@        private static final int CARRIER_IN_VODAFONE   = apnIndex++;
//@        private static final int CARRIER_IN_AIRTEL     = apnIndex++;
//@        private static final int CARRIER_IN_RELIANCE   = apnIndex++;
//@        private static final int CARRIER_IN_VODAFONE_2 = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_IN_VODAFONE),   "www||");            /** verified **/
//@            subTable.put(new Integer(CARRIER_IN_AIRTEL),     "airtelgprs.com||"); /** verified **/
//@            subTable.put(new Integer(CARRIER_IN_RELIANCE),   "rcomnet||");        /** verified **/
//@            subTable.put(new Integer(CARRIER_IN_VODAFONE_2), "portalnmms||");
//@            APN_TABLE.put(new Integer(MCC_IN_0x404), subTable);
//@            APN_TABLE.put(new Integer(MCC_IN_0x405), subTable);
//@        }
//@
//@        /** South Korea **/
//@        private static final int MCC_SK_0x450 = 0x450;
//@        private static final int CARRIER_SK_TELECOM = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_SK_TELECOM), "internet||");
//@            APN_TABLE.put(new Integer(MCC_SK_0x450), subTable);
//@        }
//@
//@        /** Singapore **/
//@        private static final int MCC_SG_0x525 = 0x525;
//@        private static final int CARRIER_SG_TELECOM = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_SG_TELECOM), "internet||"); /** verified **/
//@            APN_TABLE.put(new Integer(MCC_SG_0x525), subTable);
//@        }
//@
//@        /** Spain **/
//@        private static final int MCC_ES_0x214 = 0x214;
//@        private static final int CARRIER_ES_AMENA               = apnIndex++;
//@        private static final int CARRIER_ES_TELEFONICA_MOVISTAR = apnIndex++;
//@        private static final int CARRIER_ES_VODAFONE_AIRTEL     = apnIndex++;
//@        private static final int CARRIER_ES_VODAFONE            = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_ES_AMENA),               "internet|CLIENTE|AMENA");
//@            subTable.put(new Integer(CARRIER_ES_TELEFONICA_MOVISTAR), "movistar.es|movistar|movistar");
//@            subTable.put(new Integer(CARRIER_ES_VODAFONE_AIRTEL),     "airtelnet.es|vodafone|vodafone");
//@            subTable.put(new Integer(CARRIER_ES_VODAFONE),            "airtelwap.es|wap@wap|wap125");
//@            APN_TABLE.put(new Integer(MCC_ES_0x214), subTable);
//@        }
//@
//@        /** France **/
//@        private static final int MCC_FR_0x208 = 0x208;
//@        private static final int CARRIER_FR_BOUYGUES            = apnIndex++;
//@        private static final int CARRIER_FR_BOUYGUES_B2BOUYGTEL = apnIndex++;
//@        private static final int CARRIER_FR_SFR                 = apnIndex++;
//@        private static final int CARRIER_FR_ORANGE_PRO          = apnIndex++;
//@        private static final int CARRIER_FR_ORANGE_PERSO        = apnIndex++;
//@        private static final int CARRIER_FR_ORANGE_MIB          = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_FR_BOUYGUES),            "ebouygtel.com||");
//@            subTable.put(new Integer(CARRIER_FR_BOUYGUES_B2BOUYGTEL), "b2bouygtel.com||");
//@            subTable.put(new Integer(CARRIER_FR_SFR),                 "websfr||");
//@            subTable.put(new Integer(CARRIER_FR_ORANGE_PRO),          "orange.fr|orange|orange"); /** verified **/
//@            subTable.put(new Integer(CARRIER_FR_ORANGE_PERSO),        "orange|orange|orange");
//@            subTable.put(new Integer(CARRIER_FR_ORANGE_MIB),          "orange-mib|mportail|mib");
//@            APN_TABLE.put(new Integer(MCC_FR_0x208), subTable);
//@        }
//@
//@        /** Germany **/
//@        private static final int MCC_DE_0x262 = 0x262;
//@        private static final int CARRIER_DE_D2_VODAFONE      = apnIndex++;
//@        private static final int CARRIER_DE_EPLUS            = apnIndex++;
//@        private static final int CARRIER_DE_TMOBILE          = apnIndex++;
//@        private static final int CARRIER_DE_QUAM             = apnIndex++;
//@        private static final int CARRIER_DE_O2_VIAG_INTERKOM = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_DE_D2_VODAFONE),      "web.vodafone.de||");
//@            subTable.put(new Integer(CARRIER_DE_EPLUS),            "internet.eplus.de|eplus|gprs");
//@            subTable.put(new Integer(CARRIER_DE_TMOBILE),          "internet.t-d1.de|td1|gprs");    /** verified **/
//@            subTable.put(new Integer(CARRIER_DE_QUAM),             "quam.de|quam|quam");
//@            subTable.put(new Integer(CARRIER_DE_O2_VIAG_INTERKOM), "internet||");
//@            APN_TABLE.put(new Integer(MCC_DE_0x262), subTable);
//@        }
//@
//@        /** Italy **/
//@        private static final int MCC_IT_0x222 = 0x222;
//@        private static final int CARRIER_IT_BLU_CONTRATTO    = apnIndex++;
//@        private static final int CARRIER_IT_BLU_PREPAGATA    = apnIndex++;
//@        private static final int CARRIER_IT_VODAFONE_OMNITEL = apnIndex++;
//@        private static final int CARRIER_IT_TIM              = apnIndex++;
//@        private static final int CARRIER_IT_TIM_2            = apnIndex++;
//@        private static final int CARRIER_IT_WIND             = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_IT_BLU_CONTRATTO),    "INTERNET||");
//@            subTable.put(new Integer(CARRIER_IT_BLU_PREPAGATA),    "PINTERNET||");
//@            subTable.put(new Integer(CARRIER_IT_VODAFONE_OMNITEL), "web.omnitel.it||"); /** verified **/
//@            subTable.put(new Integer(CARRIER_IT_TIM),              "ibox.tim.it||");    /** verified **/
//@            subTable.put(new Integer(CARRIER_IT_TIM_2),            "uni.tim.it||");
//@            subTable.put(new Integer(CARRIER_IT_WIND),             "internet.wind||");
//@            APN_TABLE.put(new Integer(MCC_IT_0x222), subTable);
//@        }
//@
//@        /** Switzerland **/
//@        private static final int MCC_CH_0x228 = 0x228;
//@        private static final int CARRIER_CH_ORANGE   = apnIndex++;
//@        private static final int CARRIER_CH_SUNRISE  = apnIndex++;
//@        private static final int CARRIER_CH_SWISSCOM = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_CH_ORANGE),   "internet||");
//@            subTable.put(new Integer(CARRIER_CH_SUNRISE),  "internet|internet|internet");
//@            subTable.put(new Integer(CARRIER_CH_SWISSCOM), "gprs.swisscom.ch||");
//@            APN_TABLE.put(new Integer(MCC_CH_0x228), subTable);
//@        }
//@
//@        /** Austria **/
//@        private static final int MCC_AT_0x232 = 0x232;
//@        private static final int CARRIER_AT_ONENET              = apnIndex++;
//@        private static final int CARRIER_AT_ONENET_2            = apnIndex++;
//@        private static final int CARRIER_AT_MAX_ONLINE          = apnIndex++;
//@        private static final int CARRIER_AT_MAX_ONLINE_BUSINESS = apnIndex++;
//@        private static final int CARRIER_AT_MAX_ONLINE_METRO    = apnIndex++;
//@        private static final int CARRIER_AT_MOBILKOM_A1         = apnIndex++;
//@        private static final int CARRIER_AT_MOBILKOM_A1_2       = apnIndex++;
//@        private static final int CARRIER_AT_TELE_RING           = apnIndex++;
//@        private static final int CARRIER_AT_TMOBILE             = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_AT_ONENET),              "web.one.at||");
//@            subTable.put(new Integer(CARRIER_AT_ONENET_2),            "web.one.at|web|web");
//@            subTable.put(new Integer(CARRIER_AT_MAX_ONLINE),          "gprsinternet|GPRS|");
//@            subTable.put(new Integer(CARRIER_AT_MAX_ONLINE_BUSINESS), "business.gprsinternet|GPRS|");
//@            subTable.put(new Integer(CARRIER_AT_MAX_ONLINE_METRO),    "gprsmetro|GPRS|");
//@            subTable.put(new Integer(CARRIER_AT_MOBILKOM_A1),         "A1.net|gprs@a1plus.at|");
//@            subTable.put(new Integer(CARRIER_AT_MOBILKOM_A1_2),       "a1.net|ppp@a1plus.at|ppp");
//@            subTable.put(new Integer(CARRIER_AT_TELE_RING),           "web|web@telering.at|web");
//@            subTable.put(new Integer(CARRIER_AT_TMOBILE),             "gprsmms||");
//@            APN_TABLE.put(new Integer(MCC_AT_0x232), subTable);
//@        }
//@
//@        /** Belgium **/
//@        private static final int MCC_BE_0x206 = 0x206;
//@        private static final int CARRIER_BE_MOBISTAR   = apnIndex++;
//@        private static final int CARRIER_BE_MOBISTAR_2 = apnIndex++;
//@        private static final int CARRIER_BE_PROXIMUS   = apnIndex++;
//@        private static final int CARRIER_BE_ORANGE     = apnIndex++;
//@        private static final int CARRIER_BE_BASE       = apnIndex++;
//@        private static final int CARRIER_BE_SIMYO      = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_BE_MOBISTAR),   "web.pro.be|mobistar|mobistar");
//@            subTable.put(new Integer(CARRIER_BE_MOBISTAR_2), "mworld.be|mobistar|mobistar");
//@            subTable.put(new Integer(CARRIER_BE_PROXIMUS),   "internet.proximus.be||");
//@            subTable.put(new Integer(CARRIER_BE_ORANGE),     "orangeinternet||");
//@            subTable.put(new Integer(CARRIER_BE_BASE),       "gprs.base.be|base|base");
//@            subTable.put(new Integer(CARRIER_BE_SIMYO),      "web.be|web|web");
//@            APN_TABLE.put(new Integer(MCC_BE_0x206), subTable);
//@        }
//@
//@        /** Chile **/
//@        private static final int MCC_CL_0x730 = 0x730;
//@        private static final int CARRIER_CL_ENTEL_PCS  = apnIndex++;
//@        private static final int CARRIER_CL_TELEFONICA = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_CL_ENTEL_PCS),  "imovil.entelpcs.cl|entelpcs|entelpcs");
//@            subTable.put(new Integer(CARRIER_CL_TELEFONICA), "web.tmovil.cl|web|web");
//@            APN_TABLE.put(new Integer(MCC_CL_0x730), subTable);
//@        }
//@
//@        /** Mexico **/
//@        private static final int MCC_CL_0x334 = 0x334;
//@        private static final int CARRIER_MX_TELCEL   = apnIndex++;
//@        private static final int CARRIER_MX_MOVISTAR = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_MX_TELCEL),   "internet.itelcel.com|webgprs|webgprs2002");
//@            subTable.put(new Integer(CARRIER_MX_MOVISTAR), "internet.movistar.mx||");
//@            APN_TABLE.put(new Integer(MCC_CL_0x334), subTable);
//@        }
//@
//@        /** Argentina **/
//@        private static final int MCC_AR_0x722 = 0x722;
//@        private static final int CARRIER_AR_PERSONAL   = apnIndex++;
//@        private static final int CARRIER_AR_UNIFON     = apnIndex++;
//@        private static final int CARRIER_AR_CTI        = apnIndex++;
//@        private static final int CARRIER_AR_MOVISTAR   = apnIndex++;
//@        private static final int CARRIER_AR_MOVISTAR_2 = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_AR_PERSONAL),   "gprs.personal.com|" + MOBILE_NUMBER + "|adgj");
//@            subTable.put(new Integer(CARRIER_AR_UNIFON),     "internet.gprs.unifon.com.ar||");
//@            subTable.put(new Integer(CARRIER_AR_CTI),        "internet.ctimovil.com.ar|guest|guest");
//@            subTable.put(new Integer(CARRIER_AR_MOVISTAR),   "internet.gprs.unifon.com.ar|wap|wap");
//@            subTable.put(new Integer(CARRIER_AR_MOVISTAR_2), "internet.unifon|wap|wap");
//@            APN_TABLE.put(new Integer(MCC_AR_0x722), subTable);
//@        }
//@
//@        /** Colombia **/
//@        private static final int MCC_CO_0x733 = 0x733;
//@        private static final int CARRIER_CO_COMCEL   = apnIndex++;
//@        private static final int CARRIER_CO_MOVISTAR = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_CO_COMCEL),   "internet.comcel.com.co|COMCELWEB|COMCELWEB");
//@            subTable.put(new Integer(CARRIER_CO_MOVISTAR), "internet.movistar.com.co|movistar|movistar");
//@            APN_TABLE.put(new Integer(MCC_CO_0x733), subTable);
//@        }
//@
//@        /** Costa Rica **/
//@        private static final int MCC_CR_0x712 = 0x712;
//@        private static final int CARRIER_CR_ICE_CELULAR = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_CR_ICE_CELULAR), "icecelular|guest|guest");
//@            APN_TABLE.put(new Integer(MCC_CR_0x712), subTable);
//@        }
//@
//@        /** Nicaragua **/
//@        private static final int MCC_NI_0x710 = 0x710;
//@        private static final int CARRIER_NI_ALO_PCS = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_NI_ALO_PCS), "internet.ideasalo.ni|internet|internet");
//@            APN_TABLE.put(new Integer(MCC_NI_0x710), subTable);
//@        }
//@
//@        /** Peru **/
//@        private static final int MCC_PE_0x716 = 0x716;
//@        private static final int CARRIER_PE_CLARO = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_PE_CLARO), "tim.pe|tim|tulibertad");
//@            APN_TABLE.put(new Integer(MCC_PE_0x716), subTable);
//@        }
//@
//@        /** Uruguay **/
//@        private static final int MCC_UY_0x748 = 0x748;
//@        private static final int CARRIER_UY_ANCEL    = apnIndex++;
//@        private static final int CARRIER_UY_CTI      = apnIndex++;
//@        private static final int CARRIER_UY_MOVISTAR = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_UY_ANCEL),    "gprs.ancel|guest|guest");
//@            subTable.put(new Integer(CARRIER_UY_CTI),      "internet.ctimovil.com.uy|ctiweb|ctiweb999");
//@            subTable.put(new Integer(CARRIER_UY_MOVISTAR), "webapn.movistar.com.uy|movistar|movistar");
//@            APN_TABLE.put(new Integer(MCC_UY_0x748), subTable);
//@        }
//@
//@        /** Venezuela **/
//@        private static final int MCC_VE_0x734 = 0x734;
//@        private static final int CARRIER_VE_TIM = apnIndex++;
//@        static {
//@            Hashtable subTable = new Hashtable();
//@            subTable.put(new Integer(CARRIER_VE_TIM), "gprsweb.digitel.ve|guest|guest");
//@            APN_TABLE.put(new Integer(MCC_VE_0x734), subTable);
//@        }
//@
//@        private static final int[][] MCC_MNC_TABLE = {
//@            {0x310, 0x026, CARRIER_US_TMOBILE},  /** US-T-Mobile Formerly Cook Inlet Voicestream                     **/
//@            {0x310, 0x160, CARRIER_US_TMOBILE},  /** US-T-Mobile                                                     **/  
//@            {0x310, 0x170, CARRIER_US_TMOBILE},  /** US-T-Mobile Formerly Cingular CA/NV, also Pacific Bell Wireless **/
//@            {0x310, 0x200, CARRIER_US_TMOBILE},  /** US-T-Mobile Formerly Smith Bagley                               **/
//@            {0x310, 0x210, CARRIER_US_TMOBILE},  /** US-T-Mobile Iowa                                                **/
//@            {0x310, 0x220, CARRIER_US_TMOBILE},  /** US-T-Mobile Kansas / Oklahoma                                   **/
//@            {0x310, 0x230, CARRIER_US_TMOBILE},  /** US-T-Mobile Utah                                                **/
//@            {0x310, 0x240, CARRIER_US_TMOBILE},  /** US-T-Mobile New Mexico / Texas / Arizona                        **/
//@            {0x310, 0x250, CARRIER_US_TMOBILE},  /** US-T-Mobile Hawaii                                              **/
//@            {0x310, 0x260, CARRIER_US_TMOBILE},  /** US-T-Mobile Universal USA code                                  **/
//@            {0x310, 0x270, CARRIER_US_TMOBILE},  /** US-T-Mobile Formerly Powertel                                   **/
//@            {0x310, 0x280, CARRIER_US_TMOBILE},  /** US-T-Mobile                                                     **/
//@            {0x310, 0x290, CARRIER_US_TMOBILE},  /** US-T-Mobile                                                     **/
//@            {0x310, 0x310, CARRIER_US_TMOBILE},  /** US-T-Mobile                                                     **/
//@            {0x310, 0x330, CARRIER_US_TMOBILE},  /** US-T-Mobile                                                     **/
//@            {0x310, 0x580, CARRIER_US_TMOBILE},  /** US-T-Mobile Formerly PCS One, discontinued                      **/
//@            {0x310, 0x660, CARRIER_US_TMOBILE},  /** US-T-Mobile Formerly DigiPhone PCS / DigiPH                     **/
//@            {0x310, 0x800, CARRIER_US_TMOBILE},  /** US-T-Mobile Formerly SOL Communications                         **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x310, 0x038, CARRIER_US_ATNT}, /** US-ATNT                                                             **/
//@            {0x310, 0x090, CARRIER_US_ATNT}, /** US-ATNT Formerly Edge Wireless                                      **/
//@            {0x310, 0x150, CARRIER_US_ATNT}, /** US-ATNT Formerly Bell South Mobility DCS                            **/
//@            {0x310, 0x380, CARRIER_US_ATNT}, /** US-ATNT Was Cingular Wireless                                       **/
//@            {0x310, 0x410, CARRIER_US_ATNT}, /** US-ATNT                                                             **/
//@            {0x310, 0x560, CARRIER_US_ATNT}, /** US-ATNT Formerly Cellular One DCS, Dobson                           **/
//@            {0x310, 0x680, CARRIER_US_ATNT}, /** US-ATNT Formerly Cellular One DCS, NPI Wireless                     **/
//@            {0x310, 0x980, CARRIER_US_ATNT}, /** US-ATNT                                                             **/
//@            {0x310, 0x990, CARRIER_US_ATNT}, /** US-ATNT                                                             **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x311, 0x180, CARRIER_US_CINGULAR}, /** US-Cingular                                                     **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x302, 0x720, CARRIER_CA_ROGERS},   /** CA-Rogers                                                       **/
//@            {0x302, 0x370, CARRIER_CA_FIDO},     /** CA-Fido                                                         **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x234, 0x002, CARRIER_UK_O2_CONTRACT},        /** UK-O2-Telefonica O2 UK Limited                        **/
//@            {0x234, 0x010, CARRIER_UK_O2_CONTRACT},        /** UK-O2-Telefonica O2 UK Limited                        **/
//@            {0x234, 0x011, CARRIER_UK_O2_CONTRACT},        /** UK-O2-Telefonica Europe                               **/
//@            {0x234, 0x015, CARRIER_UK_VODAFONE_CONTRACT},  /** UK-Vodafone                                           **/
//@            {0x234, 0x003, CARRIER_UK_VODAFONE_CONTRACT},  /** UK-Vodafone Jersey Airtel Limited                     **/
//@            {0x234, 0x030, CARRIER_UK_TMOBILE},            /** UK-T-Mobile                                           **/
//@            {0x234, 0x033, CARRIER_UK_ORANGE},             /** UK-Orange                                             **/
//@            {0x234, 0x034, CARRIER_UK_ORANGE},             /** UK-Orange                                             **/
//@            {0x234, 0x050, CARRIER_UK_JERSEY_TELECOM},     /** UK-Jersey Telecom                                     **/
//@            {0x234, 0x031, CARRIER_UK_VIRGIN},             /** UK-Virgin-Mobile                                      **/
//@            {0x234, 0x032, CARRIER_UK_VIRGIN},             /** UK-Virgin-Mobile                                      **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x505, 0x001, CARRIER_AU_TELSTRA}, /** AU-Telstra                                                       **/
//@            {0x505, 0x011, CARRIER_AU_TELSTRA}, /** AU-Telstra                                                       **/
//@            {0x505, 0x071, CARRIER_AU_TELSTRA}, /** AU-Telstra                                                       **/
//@            {0x505, 0x072, CARRIER_AU_TELSTRA}, /** AU-Telstra                                                       **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x404, 0x002, CARRIER_IN_AIRTEL}, /** IN-Airtel Punjab                                                  **/
//@            {0x404, 0x010, CARRIER_IN_AIRTEL}, /** IN-Airtel Delhi & NCR                                             **/
//@            {0x404, 0x031, CARRIER_IN_AIRTEL}, /** IN-Airtel Kolkata                                                 **/
//@            {0x404, 0x090, CARRIER_IN_AIRTEL}, /** IN-Airtel Maharashtra & Goa                                       **/
//@            {0x404, 0x092, CARRIER_IN_AIRTEL}, /** IN-Airtel Mumbai                                                  **/
//@            {0x404, 0x093, CARRIER_IN_AIRTEL}, /** IN-Airtel Gujrat                                                  **/
//@            {0x404, 0x096, CARRIER_IN_AIRTEL}, /** IN-Airtel Haryana                                                 **/
//@            {0x405, 0x051, CARRIER_IN_AIRTEL}, /** IN-Airtel West Bengal                                             **/
//@            {0x405, 0x052, CARRIER_IN_AIRTEL}, /** IN-Airtel Bihar & Jharkhand                                       **/
//@            {0x405, 0x054, CARRIER_IN_AIRTEL}, /** IN-Airtel Uttar Pradesh (East)                                    **/
//@            {0x405, 0x056, CARRIER_IN_AIRTEL}, /** IN-Airtel Assam                                                   **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x404, 0x009, CARRIER_IN_RELIANCE}, /** IN-Reliance Assam                                               **/
//@            {0x404, 0x036, CARRIER_IN_RELIANCE}, /** IN-Reliance Bihar & Jharkhand                                   **/
//@            {0x404, 0x052, CARRIER_IN_RELIANCE}, /** IN Reliance Orissa                                              **/
//@            {0x404, 0x067, CARRIER_IN_RELIANCE}, /** IN-Reliance GSM Madhya Pradesh & Chhattisgarh                   **/
//@            {0x404, 0x083, CARRIER_IN_RELIANCE}, /** IN-Reliance Smart GSM Kolkata                                   **/
//@            {0x404, 0x085, CARRIER_IN_RELIANCE}, /** IN-Reliance West Bengal                                         **/
//@            {0x405, 0x005, CARRIER_IN_RELIANCE}, /** IN-Reliance Delhi & NCR                                         **/
//@            {0x405, 0x010, CARRIER_IN_RELIANCE}, /** IN-Reliance Karnataka                                           **/
//@            {0x405, 0x013, CARRIER_IN_RELIANCE}, /** IN-Reliance Maharashtra & Goa                                   **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x404, 0x011, CARRIER_IN_VODAFONE}, /** IN-Vodafone Delhi & NCR                                         **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x404, 0x001, CARRIER_IN_VODAFONE_2}, /** IN-Vodafone Haryana                                           **/
//@            {0x404, 0x020, CARRIER_IN_VODAFONE_2}, /** IN-Vodafone Mumbai                                            **/
//@            {0x404, 0x015, CARRIER_IN_VODAFONE_2}, /** IN-Vodafone Uttar Pradesh (East)                              **/
//@            {0x404, 0x013, CARRIER_IN_VODAFONE_2}, /** IN-Vodafone Andhra Pradesh                                    **/
//@            {0x404, 0x005, CARRIER_IN_VODAFONE_2}, /** IN-Vodafone Gujarat                                           **/
//@            {0x404, 0x030, CARRIER_IN_VODAFONE_2}, /** IN-Vodafone Kolkata                                           **/
//@            {0x405, 0x066, CARRIER_IN_VODAFONE_2}, /** IN-Vodafone Uttar Pradesh (West)                              **/
//@            {0x405, 0x750, CARRIER_IN_VODAFONE_2}, /** IN-Vodafone Jammu & Kashmir                                   **/
//@            {0x405, 0x751, CARRIER_IN_VODAFONE_2}, /** IN-Vodafone Assam                                             **/
//@            {0x405, 0x752, CARRIER_IN_VODAFONE_2}, /** IN-Vodafone Bihar & Jharkhand                                 **/
//@            {0x405, 0x753, CARRIER_IN_VODAFONE_2}, /** IN-Vodafone Orissa                                            **/
//@            {0x405, 0x754, CARRIER_IN_VODAFONE_2}, /** IN-Vodafone Himachal Pradesh                                  **/
//@            {0x405, 0x755, CARRIER_IN_VODAFONE_2}, /** IN-Vodafone Madhya Pradesh & Chhattisgarh                     **/
//@            {0x405, 0x756, CARRIER_IN_VODAFONE_2}, /** IN-Vodafone Madhya Pradesh & Chhattisgarh                     **/
//@            {0x404, 0x027, CARRIER_IN_VODAFONE_2}, /** IN-Vodafone Maharashtra & Goa                                 **/
//@            {0x404, 0x084, CARRIER_IN_VODAFONE_2}, /** IN-Vodafone Chennai                                           **/
//@            {0x404, 0x060, CARRIER_IN_VODAFONE_2}, /** IN-Vodafone Rajasthan                                         **/
//@            {0x404, 0x086, CARRIER_IN_VODAFONE_2}, /** IN-Vodafone Essar South                                       **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x450, 0x005, CARRIER_SK_TELECOM}, /** South Korea SK Telecom                                           **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x525, 0x001, CARRIER_SG_TELECOM}, /** Singapore Telecom                                                **/
//@            {0x525, 0x002, CARRIER_SG_TELECOM}, /** Singapore SingTel-G18 Singapore Telecom                          **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x214, 0x007, CARRIER_ES_TELEFONICA_MOVISTAR}, /** Spain Telefónica Móviles España                      **/
//@            {0x214, 0x001, CARRIER_ES_VODAFONE_AIRTEL},     /** Spain Vodafone                                       **/
//@            {0x214, 0x006, CARRIER_ES_VODAFONE_AIRTEL},     /** Spain Vodafone                                       **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x208, 0x000, CARRIER_FR_ORANGE_PRO}, /** Orange France Télécom                                         **/
//@            {0x208, 0x001, CARRIER_FR_ORANGE_PRO}, /** Orange France Télécom                                         **/
//@            {0x208, 0x002, CARRIER_FR_ORANGE_PRO}, /** Orange France Télécom                                         **/
//@            {0x208, 0x010, CARRIER_FR_SFR},        /** SFR Vivendi                                                   **/
//@            {0x208, 0x011, CARRIER_FR_SFR},        /** SFR Vivendi                                                   **/
//@            {0x208, 0x013, CARRIER_FR_SFR},        /** SFR Vivendi                                                   **/
//@            {0x208, 0x020, CARRIER_FR_BOUYGUES},   /** Bouygues Telecom                                              **/
//@            {0x208, 0x021, CARRIER_FR_BOUYGUES},   /** Bouygues Telecom                                              **/
//@            {0x208, 0x088, CARRIER_FR_BOUYGUES},   /** Bouygues Telecom                                              **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x262, 0x001, CARRIER_DE_TMOBILE},          /** DE T-Mobile Deutschland GmbH                            **/
//@            {0x262, 0x002, CARRIER_DE_D2_VODAFONE},      /** DE Vodafone D2 GmbH                                     **/
//@            {0x262, 0x003, CARRIER_DE_EPLUS},            /** DE E-Plus Mobilfunk                                     **/
//@            {0x262, 0x004, CARRIER_DE_D2_VODAFONE},      /** DE Vodafone                                             **/
//@            {0x262, 0x005, CARRIER_DE_EPLUS},            /** DE E-Plus Mobilfunk Reserved                            **/
//@            {0x262, 0x006, CARRIER_DE_TMOBILE},          /** DE T-Mobile                                             **/
//@            {0x262, 0x007, CARRIER_DE_O2_VIAG_INTERKOM}, /** DE O2 (Germany) GmbH n Co.                              **/
//@            {0x262, 0x008, CARRIER_DE_O2_VIAG_INTERKOM}, /** DE Reserved                                             **/
//@            {0x262, 0x009, CARRIER_DE_D2_VODAFONE},      /** DE Vodafone                                             **/
//@            {0x262, 0x011, CARRIER_DE_O2_VIAG_INTERKOM}, /** DE Reserved                                             **/
//@            {0x262, 0x077, CARRIER_DE_EPLUS},            /** DE E-Plus                                               **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x222, 0x001, CARRIER_IT_TIM},              /** IT-Tim                                                  **/
//@            {0x222, 0x010, CARRIER_IT_VODAFONE_OMNITEL}, /** IT-Tim Omnitel N.V.	                                 **/
//@            {0x222, 0x088, CARRIER_IT_WIND},             /** IT-Tim Wind Telecomunicazioni SpA                       **/
//@            {0x222, 0x098, CARRIER_IT_BLU_CONTRATTO},    /** IT-Tim Blu                                              **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x228, 0x001, CARRIER_CH_SWISSCOM}, /** CH-Swisscom Ltd                                                 **/
//@            {0x228, 0x002, CARRIER_CH_SUNRISE},  /** CH-Sunrise Communications AG                                    **/
//@            {0x228, 0x003, CARRIER_CH_ORANGE},   /** CH-Orange Communications SA                                     **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x232, 0x001, CARRIER_AT_MOBILKOM_A1}, /** AT-A1 Telekom Austria                                        **/
//@            {0x232, 0x003, CARRIER_AT_TMOBILE},     /** AT-T-Mobile Austria                                          **/
//@            {0x232, 0x007, CARRIER_AT_TELE_RING},   /** AT-tele.ring T-Mobile Austria                                **/
//@            {0x232, 0x009, CARRIER_AT_MOBILKOM_A1}, /** AT-A1 Telekom Austria                                        **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x206, 0x001, CARRIER_BE_PROXIMUS}, /** BE-Proximus Belgacom Mobile                                     **/
//@            {0x206, 0x010, CARRIER_BE_MOBISTAR}, /** BE-Mobistar France Telecom                                      **/
//@            {0x206, 0x020, CARRIER_BE_BASE},     /** BE-BASE KPN                                                     **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x730, 0x001, CARRIER_CL_ENTEL_PCS}, /** Chile-Entel PCS Telecomunicaciones S.A.                        **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x334, 0x002, CARRIER_MX_TELCEL},   /** Mexico América Móvil                                            **/
//@            {0x334, 0x030, CARRIER_MX_MOVISTAR}, /** Mexico movistar Pegaso Comunicaciones y Sistemas                **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x722, 0x010, CARRIER_AR_MOVISTAR}, /** Argentina Telefonica Móviles Argentina SA                       **/
//@            {0x722, 0x070, CARRIER_AR_MOVISTAR}, /** Argentina Telefonica Móviles Argentina SA                       **/
//@            {0x722, 0x034, CARRIER_AR_PERSONAL}, /** Argentina Telecom Personal SA                                   **/
//@            {0x722, 0x341, CARRIER_AR_PERSONAL}, /** Argentina Telecom Personal SA                                   **/
//@            {0x722, 0x036, CARRIER_AR_PERSONAL}, /** Argentina Telecom Personal SA                                   **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x733, 0x101, CARRIER_CO_COMCEL},   /** Comcel Colombia                                                 **/
//@            {0x733, 0x102, CARRIER_CO_MOVISTAR}, /** movistar Bellsouth Colombia                                     **/
//@            {0x733, 0x123, CARRIER_CO_MOVISTAR}, /** movistar Telefónica Móviles Colombia                            **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x712, 0x001, CARRIER_CR_ICE_CELULAR}, /** ICE	Instituto Costarricense de Electricidad                  **/
//@            {0x712, 0x002, CARRIER_CR_ICE_CELULAR}, /** ICE	Instituto Costarricense de Electricidad                  **/
//@            {0x712, 0x003, CARRIER_CR_ICE_CELULAR}, /** ICE	Grupo ICE                                                **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x716, 0x010, CARRIER_PE_CLARO}, /** Claro América Móvil Perú                                           **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@            {0x748, 0x000, CARRIER_UY_ANCEL},    /** Uruguay Compania estatal (ANTEL)                                **/
//@            {0x748, 0x001, CARRIER_UY_ANCEL},    /** Uruguay Compania estatal (ANTEL)                                **/
//@            {0x748, 0x001, CARRIER_UY_MOVISTAR}, /** Movistar Telefónica Móviles Uruguay                             **/
//@            /**------------------------------------------------------------------------------------------------------**/
//@        };
//@
//@        /**
//@         * use this method to discover which APN string 
//@         * should be used to establish a connection
//@         *
//@         * @return APN string that should be passed when creating the NMSPManager
//@         */
//@        protected static String discoverAPN(String gatewayIP, short port) {
//@            /** [1] check if the current network is CDMA **/
            //#ifdef NMSP_LOG
//@            log.debug("DiscoverApnUtil ->[Option 1] check if CDMA");
            //#endif
//@            if (RadioInfo.getNetworkType() == RadioInfo.NETWORK_CDMA) {
//@                String CMDA_APN = SEMICOLON + DEVICESIDE_TRUE;
//@                if (verifyAPN(CMDA_APN, gatewayIP, port)) {
                    //#ifdef NMSP_LOG
//@                    log.debug("DiscoverApnUtil ->[Option 1] succeed apn [" + CMDA_APN + "]");
                    //#endif
//@                    return CMDA_APN;
//@                } else {
                    //#ifdef NMSP_LOG
//@                    log.error("DiscoverApnUtil ->[Option 1] failed throw exception");
                    //#endif
//@                    throw new RuntimeException("Network: CDMA; APNSetting :[" + CMDA_APN + "]; Connection Test: Failed");
//@                }
//@            }
//@
//@            /** [2] get home Mobile Country Code and home Mobile Network Code **/
            //#ifdef NMSP_LOG
//@            log.debug("DiscoverApnUtil ->[Option 2] get APN by home mcc_mnc");
            //#endif
//@            int homeMcc = GPRSInfo.getHomeMCC();
//@            int homeMnc = GPRSInfo.getHomeMNC();
//@            String apnSetting = getAPN(homeMcc, homeMnc);
//@            if (apnSetting != null && !apnSetting.equals(APN_NOT_FOUND)) {
//@                if (verifyAPN(apnSetting, gatewayIP, port)) {
                    //#ifdef NMSP_LOG
//@                    log.debug("DiscoverApnUtil ->[Option 2] succeed apn [" + apnSetting + "]");
                    //#endif
//@                    return apnSetting;
//@                }
//@            }
//@
//@            /** [3] do round robin for all the carriers in the home country **/
//@            /**     put the carriers in the current country on the top **/
            //#ifdef NMSP_LOG_APN
//@            screen.setStatus("->[Option 3] do round robin with home country's carriers", false);
            //#endif
//@            apnSetting = roundRobinTest(homeMcc, gatewayIP, port);
//@            if (apnSetting != null && !apnSetting.equals(APN_NOT_FOUND)) {
//@                return apnSetting;
//@                //} else {
//@                //throw new RuntimeException("Couldn't find a working APN setting");
//@            }
//@            
//@            /** [4] do round robin for all the carriers in the current country **/
//@            /**     put the carriers in the current country on the top **/
//@            int currentMcc = RadioInfo.getMCC(RadioInfo.getCurrentNetworkIndex());
            //#ifdef NMSP_LOG
//@            log.debug("DiscoverApnUtil ->[Option 3] do round robin with current country's carriers on top");
            //#endif
//@            apnSetting = roundRobinTest(currentMcc, gatewayIP, port);
//@            if (apnSetting != null && !apnSetting.equals(APN_NOT_FOUND)) {
                //#ifdef NMSP_LOG
//@                log.debug("DiscoverApnUtil ->[Option 3] succeed apn [" + apnSetting + "]");
                //#endif
//@                return apnSetting;
//@            } else {
                //#ifdef NMSP_LOG
//@                log.error("DiscoverApnUtil ->[Option 3] failed");
                //#endif
//@                throw new RuntimeException("Couldn't find a working APN setting");
//@            }
//@        }
//@
//@        private static String getAPN(int mcc, int mnc) {
//@            int length = MCC_MNC_TABLE.length;
//@            for (int i = 0; i < length; i++) {
//@                int[] item = MCC_MNC_TABLE[i];
//@                if (mcc == item[0] && mnc == item[1]) {
//@                    Hashtable subTable = (Hashtable) APN_TABLE.get(new Integer(mcc));
//@                    if (subTable != null) {
//@                        return constructAPN((String) subTable.get(new Integer(item[2])));
//@                    }
//@                    return null;
//@                }
//@            }
//@            return null;
//@        }
//@
//@        private static String constructAPN(String rawStr) {
//@            if (rawStr == null)
//@                return APN_NOT_FOUND;
//@
//@            int first_bar = rawStr.indexOf('|');
//@            int last_bar  = rawStr.lastIndexOf('|');
//@            String apn = rawStr.substring(0, first_bar);
//@            String user = ((first_bar + 1 == last_bar)? "": rawStr.substring(first_bar + 1, last_bar));
//@            String pw = ((last_bar == rawStr.length() - 1)? "": rawStr.substring(last_bar + 1));
//@            String connectionStr = SEMICOLON + DEVICESIDE_TRUE + SEMICOLON + APN_EQUALS + apn;
//@
//@            if (user.equals(MOBILE_NUMBER))
//@                user = Phone.getDevicePhoneNumber(true);
//@            if (user != "")
//@                connectionStr += SEMICOLON + USERNAME_EQUALS + user;
//@            if (pw != "")
//@                connectionStr += SEMICOLON + PW_EQUALS + pw;
//@            return connectionStr;
//@        }
//@
//@        private static String roundRobinTest(int mcc, String gatewayIP, short port) {
//@            Hashtable subTable = (Hashtable) APN_TABLE.get(new Integer(mcc));
//@            if (subTable != null) {
//@                Enumeration enu = subTable.elements();
//@                while (enu.hasMoreElements()) {
//@                    String rawStr = (String) enu.nextElement();
//@                    String apnSetting = constructAPN(rawStr);
                    //#ifdef NMSP_LOG
//@                    log.debug("round robin: verify apn: [" + apnSetting + "]");
                    //#endif
//@                    if (apnSetting != null && !apnSetting.equals(APN_NOT_FOUND)) {
//@                        if (verifyAPN(apnSetting, gatewayIP, port))
//@                            return apnSetting;
//@                    }
//@                }
//@            }
//@
//@            return APN_NOT_FOUND;
//@        }
//@
//@        protected static boolean verifyAPN(String apnSetting, String gatewayIP, short port) {
//@            String connStr = "socket://" + gatewayIP + ":" + port + apnSetting;
            //#ifdef NMSP_LOG
//@            log.debug("DiscoverApnUtil connection string: [" + connStr + "]");
            //#endif
//@            SocketConnection socket = null;
//@            try {
//@                socket = (SocketConnection) Connector.open(connStr);
//@            } catch (IOException ioe) {
                //#ifdef NMSP_LOG
//@                log.error("DiscoverApnUtil IOExp on opening socket: [" + ioe.getMessage() + "]");
                //#endif
//@                return false;
//@            }
//@
//@            InputStream is = null;
//@            OutputStream os = null;
//@            try {
//@                is = socket.openInputStream();
//@            } catch (IOException ioe) {
                //#ifdef NMSP_LOG
//@                log.error("DiscoverApnUtil IOExp on opening IS: [" + ioe.getMessage() + "]");
                //#endif
//@                return false;
//@            }
//@            try {
//@                os = socket.openOutputStream();
//@            } catch (IOException ioe) {
                //#ifdef NMSP_LOG
//@                log.error("DiscoverApnUtil IOExp on opening OS: [" + ioe.getMessage() + "]");
                //#endif
//@                return false;
//@            }
//@
//@            byte[] msg = new byte[8];
//@            try {
//@                is.read(msg, 0, 8);
                //#ifdef NMSP_LOG
//@                log.debug("DiscoverApnUtil Len[8] Content["
//@                        + Integer.toHexString(msg[0]) + ", "
//@                        + Integer.toHexString(msg[1]) + ", "
//@                        + Integer.toHexString(msg[2]) + ", "
//@                        + Integer.toHexString(msg[3]) + ", "
//@                        + Integer.toHexString(msg[4]) + ", "
//@                        + Integer.toHexString(msg[5]) + ", "
//@                        + Integer.toHexString(msg[6]) + ", "
//@                        + Integer.toHexString(msg[7]) + "]");
                //#endif
//@            } catch (IOException ioe) {
                //#ifdef NMSP_LOG
//@                log.error("DiscoverApnUtil IOExp on reading socket: [" + ioe.getMessage() + "]");
                //#endif
//@                return false;
//@            }
//@            try {
//@                os.write(msg, 0, 8);
//@            } catch (IOException ioe) {
                //#ifdef NMSP_LOG
//@                log.error("DiscoverApnUtil IOExp on writing socket: [" + ioe.getMessage() + "]");
                //#endif
//@                return false;
//@            }
//@            try {
//@                os.flush();
//@            } catch (IOException ioe) {
                //#ifdef NMSP_LOG
//@                log.error("DiscoverApnUtil IOExp on flushing buffer: [" + ioe.getMessage() + "]");
                //#endif
//@                return false;
//@            }
//@
//@            try {
//@                is.close();
//@                os.close();
//@                socket.close();
//@            } catch (IOException ioe) {
                //#ifdef NMSP_LOG
//@                log.error("DiscoverApnUtil IOExp on closing socket: [" + ioe.getMessage() + "]");
                //#endif
//@                return false;
//@            }
//@
//@            if (msg[0] == 0xf
//@                    && msg[1] == 0
//@                    && msg[2] == 0
//@                    && msg[3] == 0
//@                    && msg[4] == 0
//@                    && msg[5] == 0
//@                    && msg[6] == 0
//@                    && msg[7] == 0) {
                //#ifdef NMSP_LOG
//@                log.debug("DiscoverApnUtil verification succeed");
                //#endif
//@
//@                return true;
//@            } else {
                //#ifdef NMSP_LOG
//@                log.debug("verification failed, hand shake failed");
                //#endif
//@                return false;
//@            }
//@        }
//@    }
//#endif
}
