/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client2.sdk.components.general;

import com.nuance.nmsp.client2.sdk.oem.DeviceInfoOEM;

//#ifdef NMSP_PLATFORM_ANDROID
import android.content.Context;
import com.nuance.id.NuanceId;
//#endif


public class ResourceUtil {

    /**
     * Bit field mask indicating IMEI hash to be included to generate the NuanceID
     */
    public static final int    BM_IMEI        = 0x01; //NuanceId.INCLUDE_IMEI;
    /**
     * Bit field mask indicating serial hash to be included to generate the NuanceID
     */
    public static final int    BM_SERIAL      = 0x02; //NuanceId.INCLUDE_SERIAL;
    /**
     * Bit field mask indicating MAC hash to be included to generate the NuanceID
     */
    public static final int    BM_MAC         = 0x04; //NuanceId.INCLUDE_MAC;
    /**
     * Bit field mask indicating Android ID hash to be included to generate the NuanceID
     */
    public static final int    BM_ANDROIDID   = 0x08; //NuanceId.INCLUDE_ANDROIDID;

    /**
     * Bit field mask indicating all 4 IDs (IMEI, Serial, MAC and Android ID) are to be included
     * to generate the NuanceID
     */
    public static final int    BM_ALL         = BM_IMEI | BM_SERIAL | BM_MAC | BM_ANDROIDID;

    /**
     * Gets the Nuance ID
     * @param context a valid Android Context.
     * @param idMasks Bit field indicating which IDs are used to generate NuanceID.
     *
     * @return Nuance ID string
     **/
    public static String getNuanceID(Object context, int idMasks)
    {
      //#ifdef NMSP_PLATFORM_ANDROID
           if (context instanceof Context)
           {
             NuanceId nuanceIdInst = new NuanceId((Context)context, idMasks);
             return nuanceIdInst.getId();
           }
           return null;
      //#else
//@        return null;
      //#endif
    }

    /**
     * Gets the Nuance ID including all ID masks
     * @param context a valid Android Context.
     *
     * @return Nuance ID string
     **/
    public static String getNuanceID(Object context)
    {
      //#ifdef NMSP_PLATFORM_ANDROID
        if (context instanceof Context)
        {
            NuanceId nuanceIdInst = new NuanceId((Context)context);
            return nuanceIdInst.getId();
        }
        return null;
      //#else
//@        return null;
      //#endif
    }

    /**
     * Gets the unique ID.
     **/
    //#ifdef NMSP_PLATFORM_ANDROID
    public static String getUniqueID(Context context)
    {
        return DeviceInfoOEM.getInstance(context).getUniqueID();
    }
    //#else
//@    public static String getUniqueID()
//@    {
//@        return DeviceInfoOEM.getInstance().getUniqueID();
//@    }
    //#endif


    /**
     * Gets the phone number.
     **/
    //#ifdef NMSP_PLATFORM_ANDROID
    public static String getPhoneNumber(Context context)
    {
        return DeviceInfoOEM.getInstance(context).getPhoneNumber();
    }
    //#else
//@    public static String getPhoneNumber()
//@    {
//@        return DeviceInfoOEM.getInstance().getPhoneNumber();
//@    }
    //#endif


    /**
     * Gets the operator name.
     **/
    //#ifdef NMSP_PLATFORM_ANDROID
    public static String getOperatorName(Context context)
    {
        return DeviceInfoOEM.getInstance(context).getOperatorName();
    }
    //#else
//@    public static String getOperatorName()
//@    {
//@        return DeviceInfoOEM.getInstance().getOperatorName();
//@    }
    //#endif

    /**
     * Gets the device model.
     **/
    public static String getDeviceModel()
    {
        return DeviceInfoOEM.getInstance().getDeviceModel();
    }

    /**
     * Gets the device sub model.
     **/
    public static String getDeviceSubModel()
    {
        return DeviceInfoOEM.getInstance().getDeviceSubModel();
    }

    /**
     * Gets the os type.
     **/
    public static String getOSType()
    {
        return DeviceInfoOEM.getInstance().getOSType();
    }

    /**
     * Gets the os version.
     **/
    public static String getOSVersion()
    {
        return DeviceInfoOEM.getInstance().getOSVersion();
    }
}
