/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client2.sdk.components.resource.common;

import com.nuance.nmsp.client2.sdk.components.core.calllog.SessionEventBuilder;

/**
 * Represents the Manager that manages the connection with the NMSP Gateway.
 *
 * @see ManagerListener
 */
public interface Manager
{
    /**
     * Shuts down the connection with the NMSP Gateway.
     */
    public void shutdown();

    /**
     * Log an application event by creating a {@link SessionEventBuilder}. This session event
     * is a child of the current SDK root session event. The application can call this method
     * multiple times to retrieve session event builders at different time.
     *
     * For example, logging user behaviors for different recognition transactions.
     *
     * The application can also use the returned {@link SessionEventBuilder} to log associated
     * sub events.
     *
     * @param appEventName Application event name. Must not be NULL or empty.
     * 
     * @param schemaVersion The call log schema version used by the client code. Could be Null
     *                      or empty string if there is no schema version for the client to use.
     *
     * @return {@link SessionEventBuilder} Application session event builder
     * @return null if the call logging feature is disabled
     *
     * @see {@link com.nuance.nmsp.client.sdk.common.defines.NMSP_DEFINES_CALLLOG_DISABLE}
     *
     */
    SessionEventBuilder logAppEvent(String appEventName, String schemaVersion);

    /**
     * Retrieves the root parent Id of the current Calllog session, the root parent Id is
     * Unique per Manager.
     *
     * @return the root parent id as a String
     * @return null if the call logging feature is disabled
     *
     * @see {@link com.nuance.nmsp.client.sdk.common.defines.NMSP_DEFINES_CALLLOG_DISABLE}
     */
    String getCalllogRootParentId();
    
    /**
     * Processes any pending call logs to the callback {@link com.nuance.nmsp.client2.sdk.components.resource.common.ManagerListener#callLogDataGenerated(byte[]) callLogDataGenerated}
     * This should be used whenever the client wants the logs to be flushed before the chunk 
     * size limit has been reached
     * 
     * There will be only one buffer to be produced once this function is called, because
     * the current internal call log data will always be smaller than the chunk size limit
     * otherwise it would have been already generated.
     *
     * In the case that there is no internal call log data at the moment when this function
     * is called, an empty buffer will be produced as well to notify the client this is the
     * end of the call log outputting.
     *
     * @see {@link com.nuance.nmsp.client.sdk.common.defines.NMSP_DEFINES_CALLLOG_DISABLE}
     *
     */
    void flushCallLogData();
}
