/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client2.sdk.components.resource.common;

import java.util.List;

/**
 * Defines methods that an application must implement to receive
 * notifications from the Manager.
 */
public interface ManagerListener {
	

    public static final short NMSP_REMOTE_DISCONNECT_NORMAL          = 0x0000;
    public static final short NMSP_REMOTE_DISCONNECT_TIMEOUT         = 0x0001;
    public static final short NMSP_REMOTE_DISCONNECT_SERVER_SHUTDOWN = 0x0002;
    public static final short NMSP_REMOTE_DISCONNECT_CRITICAL_ERROR  = 0x0003;
    public static final short NMSP_REMOTE_DISCONNECT_PING_TIME_OUT   = 0x0004;
    public static final short NMSP_REMOTE_DISCONNECT_SERVER_UNAVAILABLE = 0x0005;
    public static final short NMSP_REMOTE_DISCONNECT_ENDPOINT_EXCEPTION = 0x0006;
    public static final short NMSP_REMOTE_DISCONNECT_MAXAUDIO_PERSESSION= 0x0007;
    public static final short NMSP_REMOTE_DISCONNECT_MAXBYTE_PERAUDIO   = 0x0008;
    public static final short NMSP_REMOTE_DISCONNECT_CONNECT_FORBIDDEN = 0x0193;
    public static final short NMSP_REMOTE_DISCONNECT_UNKNOWN         = 0x0009;
    public static final short NMSP_REMOTE_DISCONNECT_SECURITY        = 0x000A;

    /**
     * A connection with NMSP Gateway has been established.
     * 
     * @param sessionId session ID
     * @param resource
     */
    public void connected(String sessionId, Resource resource);

    /**
     * The connection with NMSP Gateway has been terminated.
     * Expose reason code
     * 
     * @param resource the created resource object
     * @param reasonCode the server disconnected reason code defined at {@link #NMSP_REMOTE_DISCONNECT_NORMAL }
     * 
     */
    public void disconnected(Resource resource, short reasonCode); // TODO Do we need a reasonCode, yes!

    /**
     * Connection with NMSP Gateway failed.
     * 
     * @param resource
     * @param reasonCode the connection failure reason code defined at {@link #NMSP_REMOTE_DISCONNECT_NORMAL }
     */
    public void connectionFailed(Resource resource, short reasonCode); // TODO rename error() with a reason code?
    
    /**
     * Manager has been shut down and all the internal services have been stopped properly. 
     */
    public void shutdownCompleted();

    /**
     * @param callLogData  The binary representation of the call logs recorded by the associated
     *                     {@link Manager} object, could be NULL if call logging feature is disabled.
     * 
     * @param sessionEventIds   The pointer to an array of session event ids (as ASCII strings) packaged inside the
     *                          {@link callLogData}, could be NULL, if the buffer is NULL
     * @see {@link com.nuance.nmsp.client.sdk.common.defines.NMSP_DEFINES_CALLLOG_DISABLE}
     */
    
    public void callLogDataGenerated(byte[] callLogData, List sessionEventIds);
}
