/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client2.sdk.components.resource.common;

import java.util.Vector;

import com.nuance.nmsp.client2.sdk.components.general.Parameter;

/**
 * Defines methods that are implemented by all resources.
 * 
 */
public interface Resource {

    /**
     * Sets parameters on the given resource. 
     * 
     * <p>Supported parameter types are APP, NSS, GWLOG, NSSLOG, and SLOG (see the <em>Programmer's 
     * Guide</em> for information on session-related parameters). 
     * 
     * @param parameters a vector of {@link Parameter}
     * 
     * @return request ID
     * 
     * @throws ResourceException if an unexpected error occurred while the system was trying to perform an operation or access the resource
     */
    public long setParams(Vector parameters) throws ResourceException;

    /**
     * Gets parameters on the given resource. The method is synchronous; it blocks until a response is received by NMSP Gateway.
     * 
     * <p>Supported parameter types are APP and NSS (see the <em>Programmer's 
     * Guide</em> for information on session-related parameters). 
     * 
     * @param parameters a vector of {@link Parameter}
     * 
     * @return request ID
     * 
     * @throws ResourceException if an unexpected error occurred while the system was trying to perform an operation or access the resource
     */
    public long getParams(Vector parameters) throws ResourceException;

    /**
     * Frees the given resource on NMSP Gateway.
     * 
     * @param disconnectTimeout timeout, in milliseconds, before the resource is released
     * @throws ResourceException if an unexpected error occurred while the system was trying to perform an operation or access the resource
     */
    public void freeResource(int disconnectTimeout) throws ResourceException;
    
    /**
     * Sends an MDS message.
     * @param message message body
     * @return id of the transaction of sending the MDS message
     */
    public int sendMdsMessage(byte[] message);
    
    public void loadResource();
}
