/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client2.sdk.components.resource.common;

import java.util.Vector;

import com.nuance.nmsp.client2.sdk.components.core.internal.pdx.PDXDictionary;
import com.nuance.nmsp.client2.sdk.components.general.Parameter;

/**
 * Defines the methods that an application must implement to receive
 * notifications from the resource.
 */
public interface ResourceListener {

    /* Status Codes */
    public static final short STATUS_CODE_SUCCESS                        = 200;
    public static final short STATUS_CODE_PARTIAL_SUCCESS                = 201;
    public static final short STATUS_CODE_METHOD_NOT_ALLOWED             = 401;
    public static final short STATUS_CODE_METHOD_NOT_VALID_IN_THIS_STATE = 402;
    public static final short STATUS_CODE_UNSUPPORTED_PARAMETER          = 403;
    public static final short STATUS_CODE_ILLEGAL_VALUE_FOR_PARAMETER    = 404;
    public static final short STATUS_CODE_NOT_FOUND                      = 405;
    public static final short STATUS_CODE_MANDATORY_PARAMETER_MISSING    = 406;
    public static final short STATUS_CODE_METHOD_OR_OPERATION_FAILED     = 407;
    public static final short STATUS_CODE_UNRECOGNIZED_MESSAGE_ENTITY    = 408;
    public static final short STATUS_CODE_UNSUPPORTED_PARAMETER_VALUE    = 409;
    public static final short STATUS_CODE_OUT_OF_ORDER_NUMBER_ID         = 410;
    public static final short STATUS_CODE_SERVER_INTERNAL_ERROR          = 501;
    public static final short STATUS_CODE_PROTOCOL_VERSION_NOT_SUPPORTED = 502;
    public static final short STATUS_CODE_PROXY_TIMEOUT                  = 503;
    public static final short STATUS_CODE_MESSAGE_TOO_LARGE              = 504;
    public static final short STATUS_CODE_MDS_MESSAGE_NOT_SENT           = 601;
    public static final short STATUS_CODE_MDS_MESSAGE_RESPONSE_PASS      = 602;
    public static final short STATUS_CODE_MDS_MESSAGE_RESPONSE_FAIL      = 603;
    public static final short STATUS_CODE_MDS_MESSAGE_RECEIVED           = 604;
    public static final short STATUS_CODE_MDS_MESSAGE_RESPONSE_NOT_SENT  = 605;

    /* Event Codes */
    public static final short EVENT_CODE_CLIENT_DISCONNECTED_NORMALLY           = 0x0000;
    public static final short EVENT_CODE_CLIENT_SOCKET_SHUTDOWN_UNEXPECTEDLY    = 0x0001;
    public static final short EVENT_CODE_CLIENT_PING_TIMEOUT                    = 0x0002;
    public static final short EVENT_CODE_CLIENT_IDLE_TIMEOUT                    = 0x0003;
    public static final short EVENT_CODE_NSS_TERMINATED_SESSION_TIMEOUT         = 0x0004;
    public static final short EVENT_CODE_NSS_TERMINATED_SESSION_ERROR           = 0x0005;
    public static final short EVENT_CODE_GW_TERMINATED_SESSION_ERROR            = 0x0006;
    public static final short EVENT_CODE_APP_SERVER_SESSION_TERMINATED_NORMALLY = 0x0007;
    public static final short EVENT_CODE_APP_SERVER_SESSION_TERMINATED_ERROR    = 0x0008;

    /* Completion Cause */
    public static final short COMPLETION_CAUSE_RESOURCE_UNAVAILABLE             = 0x000E;
    public static final short COMPLETION_CAUSE_INVALID_REQUEST                  = 0x0258;
    public static final short COMPLETION_CAUSE_3RD_PARTY_APP_ERROR              = 0x0260; 
    public static final short COMPLETION_CAUSE_EMPTY	                        = 0x7FFF;

    /**
     * Parameters were set successfully.
     * 
     * @param statusCode status code
     * @param parameters a vector of {@link Parameter}
     * @param requestId request ID
     */
    public void setParameterCompleted(short statusCode, Vector parameters, long requestId);

    /**
     * Failed to set the parameters.
     * 
     * @param statusCode status code
     * @param completionCause reason that the request completed
     * @param requestId request ID
     */
    public void setParameterFailed(short statusCode, short completionCause, long requestId);

    /**
     * <code>getParams</code> result.
     * 
     * @param statusCode status code
     * @param parameters a vector of {@link Parameter}
     * @param requestId request ID
     */
    public void getParameterCompleted(short statusCode, Vector parameters, long requestId);

    /**
     * Failed to get the parameters.
     * 
     * @param statusCode status code
     * @param completionCause reason that the request completed
     * @param requestId request ID
     */
    public void getParameterFailed(short statusCode, short completionCause, long requestId);

    /**
     * The resource was freed on NMSP Gateway.
     * 
     * @param eventCode event code
     */
    public void resourceUnloaded(short eventCode);
    
    /**
     * Sending MDS message callback.
     * @param statusCode status code
     * @param transactionId id of the transaction of sending the message.
     */
    public void sendMdsMessageStatus(int statusCode, int transactionId);
    
    /**
     * Receiving MDS message callback.
     * @param statusCode status code
     * @param transactionId id of the transaction of receiving the message.
     * @return whether the message has been properly handled if statusCode
     * is STATUS_CODE_MDS_MESSAGE_RECEIVED.
     */
    public boolean receiveMdsMessageStatus(int statusCode, int transactionId, PDXDictionary message);
}
