/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client2.sdk.components.resource.internal.common;

import java.io.ByteArrayOutputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import com.nuance.nmsp.client2.sdk.common.defines.NMSPDefines;
import com.nuance.nmsp.client2.sdk.common.oem.api.MessageSystem;
import com.nuance.nmsp.client2.sdk.common.oem.api.MessageSystem.MessageData;
import com.nuance.nmsp.client2.sdk.common.protocols.ProtocolDefines;
import com.nuance.nmsp.client2.sdk.common.util.ByteConversion;
import com.nuance.nmsp.client2.sdk.common.util.Util;
import com.nuance.nmsp.client2.sdk.components.core.XMode;
import com.nuance.nmsp.client2.sdk.components.core.internal.pdx.PDXDictionary;
import com.nuance.nmsp.client2.sdk.components.general.Parameter;
import com.nuance.nmsp.client2.sdk.components.resource.common.Manager;
import com.nuance.nmsp.client2.sdk.components.resource.common.Resource;
import com.nuance.nmsp.client2.sdk.components.resource.common.ResourceException;
import com.nuance.nmsp.client2.sdk.components.resource.common.ResourceListener;
import com.nuance.nmsp.client2.sdk.oem.BluetoothSystemOEM;
//#ifdef NMSP_PLATFORM_BLACKBERRY
//@import net.rim.device.api.system.RuntimeStore;
//#endif
//#ifdef NMSP_LOG
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory.Log;
//#endif

public class ResourceImpl implements Resource, NMSPSessionListener, MessageSystem.MessageHandler {
    //#ifdef NMSP_LOG
    private static final Log log = LogFactory.getLog(ResourceImpl.class);
    //#endif
    
    protected NMSPSession session;
    //private String application;
    private ResourceListener resourceListener;
    private NMSPSessionListener sessionListener;
    protected NMSPDefines.Codec inputCodec;
    protected NMSPDefines.Codec outputCodec;
    public Vector parameters;
    private byte resourceType;
    protected MessageSystem messageSystem;
    protected Manager manager;


    protected static final int STATE_RESOURCE_UNLOADED 	= 0x00;
    protected static final int STATE_RESOURCE_LOADING 	= 0x01;
    protected static final int STATE_RESOURCE_LOADED 	= 0x02;
    protected int resourceState = STATE_RESOURCE_UNLOADED;
    
    protected Object syncObject;

    protected Hashtable requestIds;
    protected byte tranId = 0x0;

/**
 * This is the old key used int NMSP 3.0 SDK for BB V2
 * We need the building process to generate a different KEY for each Application later
 * "ID_NMSP_SDK_TRANSACTION_ID_COUNTER_NVC2"
 * private static final long ID_NMSP_SDK_TRANSACTION_ID_COUNTER = 0x3ed45c329ef34fb1L;
 **/

    protected static final long defaultReqId = 0x0000;
    private static long requestIdCounter = 0x0001;
    protected long resourceId;

    private Hashtable getParams;

    protected final static String RESOURCE_UNLOADED_EXCEPTION = "the resource was unloaded. ";

    protected static final byte CMD_LOAD_RESOURCE    = 0x00;
    protected static final byte CMD_SET_PARAM        = 0x01;
    protected static final byte CMD_GET_PARAM        = 0x02;
    protected static final byte CMD_FREE_RESOURCE    = 0x03;
    protected static final byte CMD_FREE_RESOURCE_ID = 0x04;
    protected static final byte CMD_BCP_SEND_MSG     = 0x05;
    protected static final byte CMD_BCP_SEND_MSG_RESPONSE = 0x06;

    protected static final byte LOAD_RESOURCE_REQUEST = 0x00;
    protected static final byte SET_PARAM_REQUEST     = 0x01;
    protected static final byte GET_PARAM_REQUEST     = 0x02;
    protected static final byte FREE_RESOURCE_REQUEST = 0x03;
    protected static final byte MDS_MESSAGE_REQUEST   = 0x04;
    protected static final byte MDS_RESPONSE_REQUEST  = 0x05;

    public static final byte NR9_RECO = 0x01;
    public static final byte NR9_TTS  = 0x02;
    public static final byte OPER     = 0x03;
    public static final byte DICT     = 0x04;
    public static final byte NMAS     = 0x05;
    public static final byte CALLLOG  = 0x06;
    
    // app thread
    public ResourceImpl(ManagerImpl manager, String application, ResourceListener resourceListener, Vector parameters, byte resourceType) {
        this.manager = manager;
        this.session = manager.getSession();
        //this.application = application;
        this.resourceListener = resourceListener;
        this.inputCodec = manager.getInputCodec();
        this.outputCodec = manager.getOutputCodec();
        this.resourceType = resourceType;
        this.messageSystem = manager.getMsgSys();
        this.syncObject = new Object();
        this.tranId = session.getTransactionId();
        this.requestIds = new Hashtable();
        this.getParams = new Hashtable();
        this.resourceId = session.getResourceId();
        
        this.parameters = new Vector();
        if (parameters != null) {
            // do a deep copy of the parameter vector
            for (int i = 0; i < parameters.size(); i++) {
                Parameter param = ((Parameter) parameters.elementAt(i)).clone();
                this.parameters.addElement(param);
            }
        }

		//if network is connected, set resource is loaded
		//else, set resource is unloaded.
		//so, if freeResource is called before resource is loaded, 
		//SDK throw exception.
        //if (session.getSessionId() == null)
        //	this.resourceLoaded = false;
        //else
        //	this.resourceLoaded = true;
    }

    public Manager getManager(){
        return this.manager;
    }
    
    private static Parameter.Type convertToType(String type) {
        if (type.equalsIgnoreCase("sdk")) {
            return Parameter.Type.SDK;
        } else if (type.equalsIgnoreCase("nmsp")) {
            return Parameter.Type.NMSP;
        } else if (type.equalsIgnoreCase("app")) {
            return Parameter.Type.APP;
        } else if (type.equalsIgnoreCase("nss")) {
            return Parameter.Type.NSS;
        } else if (type.equalsIgnoreCase("slog")) {
            return Parameter.Type.SLOG;
        } else if (type.equalsIgnoreCase("nsslog")) {
            return Parameter.Type.NSSLOG;
        } else if (type.equalsIgnoreCase("gwlog")) {
            return Parameter.Type.GWLOG;
        } else if (type.equalsIgnoreCase("svsp")) {
            return Parameter.Type.SVSP;
        } else if (type.equalsIgnoreCase("sip")) {
            return Parameter.Type.SIP;
        } else if (type.equalsIgnoreCase("sdp")) {
            return Parameter.Type.SDP;
        } else {
            return null;
        }
    }


    // sdk thread
    protected synchronized long getRequestId() {
        long reqId = requestIdCounter++;
        if (requestIdCounter == Long.MIN_VALUE)
            requestIdCounter = 0x0001;

        return reqId;
    }

    protected byte getResourceType() {
        return this.resourceType;
    }
    
    public int sendMdsMessage(byte[] message) {
        messageSystem.send(new MessageData(CMD_BCP_SEND_MSG, message), this,
                Thread.currentThread(), messageSystem.getVRAddr()[0]);
        
        return tranId;
    }
    
    public int sendMdsMessageResponse(int statusCode) {
        messageSystem.send(new MessageData(CMD_BCP_SEND_MSG_RESPONSE, new Integer(statusCode)), this,
                Thread.currentThread(), messageSystem.getVRAddr()[0]);
        
        return tranId;
    }
    
    private void handleSendMessage(byte[] payload) {
        long reqId = getRequestId();
        requestIds.put(new Long(reqId), new Byte(MDS_MESSAGE_REQUEST));
        session.postBcpMessage(ProtocolDefines.XMODE_BCP_COMMAND_BCP_MDS_MESSAGE, XMode.NET_CONTEXT_SEND_BCP_MDS_MESSAGE, payload, null, tranId, reqId, sessionListener, false, null);
    }
    
    private void handleSendMessageResponse(Integer statusCode) {
        byte[] payload = new byte[1];
        payload[0] = statusCode.byteValue();
        long reqId = getRequestId();
        
        requestIds.put(new Long(reqId), new Byte(MDS_RESPONSE_REQUEST));
        session.postBcpMessage(ProtocolDefines.XMODE_BCP_COMMAND_BCP_MDS_MESSAGE_RESPONSE, XMode.NET_CONTEXT_SEND_BCP_MDS_MESSAGE_RESPONSE, payload, null, tranId, reqId, sessionListener, false, null);
    }
    
    // app thread
    public void loadResource()
    {
        messageSystem.send(new MessageData(CMD_LOAD_RESOURCE, null),
                this, Thread.currentThread(), messageSystem.getVRAddr()[0]);
    }
    
    // sdk thread
    protected void handleLoadResource(NMSPSessionListener sessionListener, Vector parameters) {
        //#ifdef NMSP_LOG
        log.debug("loadResource");
        //#endif
        // There is no acknowledgment for loadResource. We assume it is successful
        
        /* Checking BT to adjust codecs */
        if ((new BluetoothSystemOEM(this.parameters)).isBluetoothHeadsetConnected()) {
            inputCodec = Util.adjustCodecForBluetooth(inputCodec);
            outputCodec = Util.adjustCodecForBluetooth(outputCodec);
            ((ManagerImpl)manager).setInputCodec(inputCodec);
            ((ManagerImpl)manager).setOutputCodec(outputCodec);
        }
        
        this.sessionListener = sessionListener;
        session.setSessionListener(sessionListener);

        if (this.resourceState == STATE_RESOURCE_UNLOADED) {
            if (session.getSessionId() != null) {
                // if the session has already been established and the resource has been unloaded
                // just switch the resource state to be loaded
                this.resourceState = STATE_RESOURCE_LOADED;
            } else {
                session.connect(inputCodec, outputCodec);
                this.resourceState = STATE_RESOURCE_LOADING;
            }
        } else {
            return;
        }

        
        long reqId = getRequestId();
        String payloadStr = "<lr>" + "<rid>" + reqId + "</rid>";
//        payloadStr += "<resid>" + resourceId + "</resid>";
        switch (resourceType) {
        case ResourceImpl.NMAS:
            return;
        case ResourceImpl.NR9_RECO:
            payloadStr += "<nr9><reco/></nr9>";
            break;
        case ResourceImpl.NR9_TTS:
            payloadStr += "<nr9><tts/></nr9>";
            break;
        case ResourceImpl.OPER:
            payloadStr += "<oper></oper>";
            break;
        case ResourceImpl.DICT:
            payloadStr += "<dict>";
            if (parameters != null) {
                for (int i = 0; i < parameters.size(); i++) {
                    Parameter param = (Parameter) parameters.elementAt(i);
                    if (param.getType() == Parameter.Type.SIP ||
                        param.getType() == Parameter.Type.SDP ||
                        param.getType() == Parameter.Type.APP ||
                        param.getType() == Parameter.Type.NSS) {
                        
                        String valueString = new String(param.getValue());
                        payloadStr += "<" + param.getType().toString() + " p=\"" + param.getName() + "\" v=\"" + Util.escapeXML(valueString) + "\"/>";
                    }
                }
            }
            payloadStr += "</dict>";
            break;
        }
                
        payloadStr += "</lr>";
        byte[] primaryPayload = payloadStr.getBytes();
        session.postBcpMessage(ProtocolDefines.XMODE_BCP_COMMAND_LOAD_RESOURCE, XMode.NET_CONTEXT_SEND_BCP_LOAD_RESOURCE,
                primaryPayload, null, tranId, reqId, sessionListener, false, null);
    }

    //TODO remove freeResource once we use reqId and freeResourceId
    // app thread
    public void freeResource(int disconnectTimeout) throws ResourceException {
        //#ifdef NMSP_LOG
        log.debug("freeResource, disconnect timeout: " + disconnectTimeout);
        //#endif
        synchronized (syncObject) {
            if (resourceState == STATE_RESOURCE_LOADED) {
            	resourceState = STATE_RESOURCE_UNLOADED;
                Integer dTimeout = new Integer(disconnectTimeout);
                messageSystem.send(
                        new MessageData(CMD_FREE_RESOURCE, dTimeout), this,
                        Thread.currentThread(), messageSystem.getVRAddr()[0]);
            } else {
                //#ifdef NMSP_LOG
                log.error("ResourceException " + RESOURCE_UNLOADED_EXCEPTION);
                //#endif
                throw new ResourceException(RESOURCE_UNLOADED_EXCEPTION);
            }
        }
    }
    
    // sdk thread
    // WLWL to delete
//    protected void sendBCPLog(byte[] l) {
//        //#ifdef NMSP_LOG
//        if (log.isDebugEnabled())
//            log.debug("sendBCPLog");
//        //#endif
//        session.bcpLog(tranId, l);
//    }

    // sdk thread
    private void handleFreeResource(int disconnectTimeout) {
        session.freeResource(tranId, disconnectTimeout, sessionListener);

        resourceListener.resourceUnloaded(ResourceListener.EVENT_CODE_CLIENT_DISCONNECTED_NORMALLY);
    }

 // app thread
    public void freeResourceId(int disconnectTimeout) throws ResourceException {
        //#ifdef NMSP_LOG
        log.debug("freeResourceId, disconnect timeout: " + disconnectTimeout);
        //#endif
        synchronized (syncObject) {
            if (resourceState == STATE_RESOURCE_LOADED) {
            	resourceState = STATE_RESOURCE_UNLOADED;
                Integer dTimeout = new Integer(disconnectTimeout);
                messageSystem.send(
                        new MessageData(CMD_FREE_RESOURCE_ID, dTimeout), this,
                        Thread.currentThread(), messageSystem.getVRAddr()[0]);
            } else {
                //#ifdef NMSP_LOG
                log.error("ResourceException " + RESOURCE_UNLOADED_EXCEPTION);
                //#endif
                throw new ResourceException(RESOURCE_UNLOADED_EXCEPTION);
            }
        }
    }

    // sdk thread
    private void handleFreeResourceId(int disconnectTimeout) {
        long reqId = getRequestId();
        String payloadStr = "<fr>" + "<rid>" + reqId + "</rid>";
        payloadStr += "<n>1</n>";
        payloadStr += "<resids>";
        payloadStr += "<res1><id>" + resourceId + "</id>";
        payloadStr += "<timeout>" + disconnectTimeout + "<timeout></res1>";
        payloadStr += "</resids>";
        payloadStr += "</fr>";
        byte[] primaryPayload = payloadStr.getBytes();
        session.postBcpMessage(ProtocolDefines.XMODE_BCP_COMMAND_FREE_RESOURCE_ID, 
                               XMode.NET_CONTEXT_SEND_BCP_FREE_RESOURCE_ID,
                               primaryPayload, null, tranId, reqId, sessionListener, true, null);
        
    }
    
    // sdk thread
    public void onBcpFreeResourceId() {
        resourceListener.resourceUnloaded(ResourceListener.EVENT_CODE_CLIENT_DISCONNECTED_NORMALLY);
    }

 // app thread
    public long getParams(Vector parameters) throws ResourceException {
        //#ifdef NMSP_LOG
        log.debug("getParams");
        //#endif
        if (parameters == null) {
            //#ifdef NMSP_LOG
            log.error("NullPointerException parameters is NULL. ");
            //#endif
            throw new NullPointerException("parameters can not be null!");
        }
        for (int i = 0; i < parameters.size(); i++) {
            Parameter parameter = (Parameter) parameters.elementAt(i);
            if(parameter.getType() == Parameter.Type.SDK ||
               parameter.getType() == Parameter.Type.SLOG ||
               parameter.getType() ==  Parameter.Type.NSSLOG ||
               parameter.getType() ==  Parameter.Type.GWLOG ||
               parameter.getType() ==  Parameter.Type.SIP ||
               parameter.getType() ==  Parameter.Type.SDP) {
                //#ifdef NMSP_LOG
                log.error("IllegalArgumentException Parameter type: " + parameter.getType() + " not allowed. ");
                //#endif
                throw new IllegalArgumentException("Parameter type: " + parameter.getType() + " not allowed. ");
            }
        }
        synchronized (syncObject) {
            long reqId = getRequestId();
            Object array[] = new Object[2];
            array[0] = parameters;
            array[1] = new Long(reqId);
            messageSystem.send(new MessageData(CMD_GET_PARAM, array),
                    this, Thread.currentThread(),
                    messageSystem.getVRAddr()[0]);
            return reqId;
        }
    }
    
    // sdk thread
    private void handleGetParams(Vector parameters, long reqId) {
    	
    	if (resourceState != STATE_RESOURCE_LOADED){    
    		resourceListener.getParameterFailed(ResourceListener.STATUS_CODE_METHOD_OR_OPERATION_FAILED,
    											ResourceListener.COMPLETION_CAUSE_RESOURCE_UNAVAILABLE,
    											reqId);
    	}
        byte[] binaryPayload = new byte[0];
        requestIds.put(new Long(reqId), new Byte(GET_PARAM_REQUEST));
        getParams.put(new Long(reqId), parameters);
        String payloadStr = "<gp>" + "<rid>" + reqId + "</rid>";
//      payloadStr += "<resid>" + resourceId + "</resid>";
        for (int i = 0; i < parameters.size(); i++) {
            Parameter parameter = (Parameter) parameters.elementAt(i);
            if(parameter.getType() == Parameter.Type.APP ||
               parameter.getType() ==  Parameter.Type.NMSP ||
               parameter.getType() ==  Parameter.Type.NSS ||
               parameter.getType() ==  Parameter.Type.SVSP) {
                    payloadStr += "<" + parameter.getType() + " p=\"" + parameter.getName() + "\"/>";
            }
        }
        payloadStr += "</gp>";
        session.postBcpMessage(ProtocolDefines.XMODE_BCP_COMMAND_GET_PARAMS, XMode.NET_CONTEXT_SEND_BCP_GET_PARAMS, payloadStr.getBytes(), binaryPayload, tranId, reqId, sessionListener, false, null);
    }

    // app thread
    public long setParams(Vector parameters) throws ResourceException {
        //#ifdef NMSP_LOG
        log.debug("setParams");
        //#endif
        if (parameters == null) {
            //#ifdef NMSP_LOG
            log.error("NullPointerException parameters is NULL. ");
            //#endif
            throw new NullPointerException("parameters can not be null!");
        }
        for (int i = 0; i < parameters.size(); i++) {
            Parameter parameter = (Parameter) parameters.elementAt(i);
            if(parameter.getType() == Parameter.Type.SDK) {
                //#ifdef NMSP_LOG
                log.error("IllegalArgumentException Parameter type: " + parameter.getType() + " not allowed. ");
                //#endif
                throw new IllegalArgumentException("Parameter type: " + parameter.getType() + " not allowed. ");
            }
        }
        synchronized (syncObject) {
            long reqId = getRequestId();
            Object array[] = new Object[2];
            array[0] = parameters;
            array[1] = new Long(reqId);
            messageSystem.send(new MessageData(CMD_SET_PARAM, array),
                    this, Thread.currentThread(),
                    messageSystem.getVRAddr()[0]);
            return reqId;
        }
    }
    
    // sdk thread
    private void handleSetParams(Vector parameters, long reqId) {
    	if (resourceState != STATE_RESOURCE_LOADED){    
    		resourceListener.setParameterFailed(ResourceListener.STATUS_CODE_METHOD_OR_OPERATION_FAILED,
    											ResourceListener.COMPLETION_CAUSE_RESOURCE_UNAVAILABLE,
    											reqId);
    		return;
    	}
    	
        boolean containsOnlyLogParams = true;
        ByteArrayOutputStream binaryPayload = new ByteArrayOutputStream();
        byte[] value;
        byte binaryLength[] = new byte[4];
        int binaryIndex = 1;
        requestIds.put(new Long(reqId), new Byte(SET_PARAM_REQUEST));
        String payloadStr = "<sp>" + "<rid>" + reqId + "</rid>";
//      payloadStr += "<resid>" + resourceId + "</resid>";
        for (int i = 0; i < parameters.size(); i++) {
            Parameter parameter = (Parameter) parameters.elementAt(i);

            if(parameter.getType() == Parameter.Type.APP ||
               parameter.getType() ==  Parameter.Type.NMSP ||
               parameter.getType() ==  Parameter.Type.NSS) {
                containsOnlyLogParams = false;
                String valueString = new String(parameter.getValue());
                payloadStr += "<" + parameter.getType() + " p=\"" + parameter.getName() + "\" v=\"" + Util.escapeXML(valueString) + "\"/>";
            }
            else {
                // Parameter.Type.GWLOG
                // Parameter.Type.NSSLOG
                // Parameter.Type.SLOG
                // Parameter.Type.SVSP
                if(parameter.getType() ==  Parameter.Type.SVSP) {
                    containsOnlyLogParams = false;
                }
                payloadStr += "<" + parameter.getType().toString() + " p=\"" + parameter.getName() + "\" v=\"" + binaryIndex++ + "\"/>";
                value = parameter.getValue();
                ByteConversion.intToBytes(value.length, binaryLength, 0);
                binaryPayload.write(binaryLength, 0, 4);
                binaryPayload.write(value, 0, value.length);
            }
        }
        payloadStr += "</sp>";

        session.postBcpMessage(ProtocolDefines.XMODE_BCP_COMMAND_SET_PARAMS, XMode.NET_CONTEXT_SEND_BCP_SET_PARAMS, payloadStr.getBytes(), binaryPayload.toByteArray(), tranId, reqId, sessionListener, containsOnlyLogParams, null);
        try {
            binaryPayload.close();
        } catch (Exception e) {
        }
    }

    // sdk thread
    public void onBcpEvent(byte transactionId, short eventCode) {
        //#ifdef NMSP_LOG
        log.debug("onBcpEvent, TID: " + transactionId + ", event code: " + eventCode);
        //#endif
        
        if(resourceState == STATE_RESOURCE_LOADED) {
        	resourceState = STATE_RESOURCE_UNLOADED;
            
            // go through all pending requests and declared them failed.
            Enumeration keys = requestIds.keys();
            while (keys.hasMoreElements()) {
                Long requestId = (Long) keys.nextElement();
                switch (((Byte)requestIds.remove(requestId)).byteValue()) {
                case SET_PARAM_REQUEST:
                    resourceListener.setParameterFailed(ResourceListener.STATUS_CODE_METHOD_OR_OPERATION_FAILED, ResourceListener.COMPLETION_CAUSE_RESOURCE_UNAVAILABLE, requestId.longValue());
                    break;
                case GET_PARAM_REQUEST:
                    resourceListener.getParameterFailed(ResourceListener.STATUS_CODE_METHOD_OR_OPERATION_FAILED, ResourceListener.COMPLETION_CAUSE_RESOURCE_UNAVAILABLE, requestId.longValue());
                    break;
                case MDS_MESSAGE_REQUEST:
                	resourceListener.sendMdsMessageStatus(ResourceListener.STATUS_CODE_MDS_MESSAGE_NOT_SENT, transactionId);
                	break;
                case MDS_RESPONSE_REQUEST:
                	resourceListener.receiveMdsMessageStatus(ResourceListener.STATUS_CODE_MDS_MESSAGE_RESPONSE_NOT_SENT, transactionId, null);
                	break;
                // case FREE_RESOURCE_REQUEST:
                // resourceListener.resourceUnloaded();
                // break;
                }
            }
            resourceListener.resourceUnloaded(eventCode);
            session.removeSessionListener(this);
        }     
    }

    // sdk thread
    public void onBcpGetParamsComplete(byte transactionId, long requestId,
            short statusCode, byte[] result) {
        //#ifdef NMSP_LOG
        log.debug("onBcpGetParamsComplete, TID: " + transactionId + ", RID: " + requestId);
        //#endif
        if(null == requestIds.remove(new Long(requestId))) {
            //#ifdef NMSP_LOG
            log.warn("onBcpGetParamsComplete, RID: " + requestId + " already removed!");
            //#endif
            getParams.remove(new Long(requestId));
            return;
        }
        Vector parameters = (Vector) getParams.remove(new Long(requestId));
        if (parameters == null) {
            //#ifdef NMSP_LOG
            log.error("Could not find the grammars associated with RID: " + requestId);
            //#endif
            return;
        }

        String params = new String(result);
        String param;
        int start = 0;
        int index;
        int paramIndex = 0;
        // ;value;value;type:name;value;
        if (params.startsWith(";")) {
            start = 1;
        }
        while ((index = params.indexOf(";", start)) != -1) {
            param = params.substring(start, index);
            if (param.indexOf(":") == -1) {
                // we have a value
                Parameter oldParam = (Parameter)parameters.elementAt(paramIndex);
                //replace the old param with a new one with a value
                parameters.setElementAt(new Parameter(oldParam.getName(), param.getBytes(), oldParam.getType()), paramIndex);
            }
            start = index + 1;
            paramIndex++;
        }
        // in case we don't have a trailing ';'
        if (start < params.length()) {
            param = params.substring(start);
            if (param.indexOf(":") == -1) {
                // we have a value
                Parameter oldParam = (Parameter)parameters.elementAt(paramIndex);
                //replace the old param with a new one with a value
                parameters.setElementAt(new Parameter(oldParam.getName(), param.getBytes(), oldParam.getType()), paramIndex);
            }
        }

        //resourceListener.getParameterCompleted(statusCode, parameters, requestId);
        if(statusCode == ResourceListener.STATUS_CODE_SUCCESS ||
            statusCode == ResourceListener.STATUS_CODE_PARTIAL_SUCCESS) {
            resourceListener.getParameterCompleted(statusCode, parameters, requestId);
        }
        else {
            resourceListener.getParameterFailed(statusCode, ResourceListener.COMPLETION_CAUSE_EMPTY, requestId);
        }
    }

    // sdk thread
    public void onBcpResponse(byte transactionId, long requestId,
            short statusCode, short requestState, short completionCause) {
        //#ifdef NMSP_LOG
        log.debug("onBcpResponse, TID: " + transactionId + ", RID: " + requestId + ", status code: " + statusCode
                  + " , request state: " + requestState + ", completion cause: " + completionCause);
        //#endif
        
        if (statusCode != ProtocolDefines.XMODE_BCP_RESPONSE_STATUS_OK) {
            Byte cmd = (Byte) requestIds.get(new Long(requestId));
            if (cmd != null) {
                switch (cmd.byteValue()) {
                case SET_PARAM_REQUEST:
                    requestIds.remove(new Long(requestId));
                    resourceListener.setParameterFailed(statusCode, completionCause, requestId);
                    break;
                case GET_PARAM_REQUEST:
                    requestIds.remove(new Long(requestId));
                    resourceListener.getParameterFailed(statusCode, completionCause, requestId);
                    break;
                case MDS_MESSAGE_REQUEST:
                	requestIds.remove(new Long(requestId));
                	resourceListener.sendMdsMessageStatus(ResourceListener.STATUS_CODE_MDS_MESSAGE_NOT_SENT, transactionId);
                	break;
                case MDS_RESPONSE_REQUEST:
                	requestIds.remove(new Long(requestId));
                	resourceListener.receiveMdsMessageStatus(ResourceListener.STATUS_CODE_MDS_MESSAGE_RESPONSE_NOT_SENT, transactionId, null);
                	break;
                // case FREE_RESOURCE_REQUEST:
                // break;
                }
            }
        }
    }

    // sdk thread
    public void onBcpSetParamsComplete(byte transactionId, long requestId,
            short statusCode, byte[] result) {
        //#ifdef NMSP_LOG
        log.debug("onBcpSetParamsComplete, TID: " + transactionId + ", RID: " + requestId + ", status code: " + statusCode);
        //#endif
        if(null == requestIds.remove(new Long(requestId))) {
            //#ifdef NMSP_LOG
            log.warn("onBcpSetParamsComplete, RID: " + requestId + " already removed!");
            //#endif
            return;
        }
        Vector parameters = new Vector();
        if (result != null) {
            String params = new String(result);
            String param;
            int start = 0;
            int index;
            int index2;
            // ;type.name;type.name;
            if (params.startsWith(";")) {
                start = 1;
            }
            while ((index = params.indexOf(";", start)) != -1) {
                param = params.substring(start, index);
                if ((index2 = param.indexOf(".")) != -1) {
                    parameters.addElement(new Parameter(param.substring(index2 + 1), convertToType(param.substring(0, index2))));

                }
                start = index + 1;
            }
            // in case we don't have a trailing ';'
            if (start < params.length()) {
                param = params.substring(start);
                if ((index2 = param.indexOf(".")) != -1) {
                    parameters.addElement(new Parameter(param.substring(index2 + 1), convertToType(param.substring(0, index2))));
                }
            }
        }

        //resourceListener.setParameterCompleted(statusCode, parameters, requestId);
        if(statusCode == ResourceListener.STATUS_CODE_SUCCESS ||
            statusCode == ResourceListener.STATUS_CODE_PARTIAL_SUCCESS) {
            resourceListener.setParameterCompleted(statusCode, parameters, requestId);
        }
        else {
            //TODO Change completion cause
            resourceListener.setParameterFailed(statusCode, ResourceListener.COMPLETION_CAUSE_EMPTY, requestId);
        }
    }

    // sdk thread
    public void onSessionConnected(byte[] sessionId) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("onSessionConnected, SID: " + sessionId);
        //#endif
        synchronized (syncObject) {
        	resourceState = STATE_RESOURCE_LOADED;
        }
    }

    // sdk thread
    public void onSessionDisconnected(short reasonCode) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("onSessionDisconnected, reason code: " + reasonCode);
        //#endif
        short eventCode;

        switch (reasonCode) {
        case XMode.XMODE_SOCKET_CLOSED_DISCONNECT:
            eventCode = ResourceListener.EVENT_CODE_CLIENT_DISCONNECTED_NORMALLY;
            break;
        case XMode.XMODE_SESSION_IDLE_FOR_TOO_LONG:
            eventCode = ResourceListener.EVENT_CODE_CLIENT_IDLE_TIMEOUT;
            break;
        case XMode.XMODE_SOCKET_CLOSED_OPEN_SOCKET_FAILED:
            eventCode = ResourceListener.EVENT_CODE_CLIENT_SOCKET_SHUTDOWN_UNEXPECTEDLY;
            break;
        case XMode.XMODE_SOCKET_CLOSED_OPEN_SOCKET_FAILED_SECURITY:
            eventCode = ResourceListener.EVENT_CODE_CLIENT_SOCKET_SHUTDOWN_UNEXPECTEDLY;
            break;
        case XMode.XMODE_SOCKET_CLOSED_COP_CONNECT_TIMEOUT:
            eventCode = ResourceListener.EVENT_CODE_CLIENT_SOCKET_SHUTDOWN_UNEXPECTEDLY;
            break;
        case XMode.XMODE_SOCKET_CLOSED_COP_DISCONNECT:
            eventCode = ResourceListener.EVENT_CODE_GW_TERMINATED_SESSION_ERROR;
            break;
        case XMode.XMODE_SOCKET_CLOSED_COP_CONNECT_FAILED:
            eventCode = ResourceListener.EVENT_CODE_CLIENT_SOCKET_SHUTDOWN_UNEXPECTEDLY;
            break;
        default:
            eventCode = ResourceListener.EVENT_CODE_CLIENT_SOCKET_SHUTDOWN_UNEXPECTEDLY;
        }

        // go through all pending requests and declared them failed.
        Enumeration keys = requestIds.keys();
        while (keys.hasMoreElements()) {
            Long requestId = (Long) keys.nextElement();
            switch (((Byte)requestIds.remove(requestId)).byteValue()) {
            case SET_PARAM_REQUEST:
                resourceListener.setParameterFailed(ResourceListener.STATUS_CODE_METHOD_OR_OPERATION_FAILED, ResourceListener.COMPLETION_CAUSE_RESOURCE_UNAVAILABLE, requestId.longValue());
                break;
            case GET_PARAM_REQUEST:
                resourceListener.getParameterFailed(ResourceListener.STATUS_CODE_METHOD_OR_OPERATION_FAILED, ResourceListener.COMPLETION_CAUSE_RESOURCE_UNAVAILABLE, requestId.longValue());
                break;
            case MDS_MESSAGE_REQUEST:
            	resourceListener.sendMdsMessageStatus(ResourceListener.STATUS_CODE_MDS_MESSAGE_NOT_SENT, 0);
            	break;
            case MDS_RESPONSE_REQUEST:
            	resourceListener.receiveMdsMessageStatus(ResourceListener.STATUS_CODE_MDS_MESSAGE_RESPONSE_NOT_SENT, 0, null);
            	break;
            // case FREE_RESOURCE_REQUEST:
            // resourceListener.resourceUnloaded();
            // break;
            }
        }

        synchronized (syncObject) {
            if(resourceState == STATE_RESOURCE_LOADED) {
                resourceListener.resourceUnloaded(eventCode);
            }
            resourceState = STATE_RESOURCE_UNLOADED;
        }
    }
    
    public boolean onBcpMdsMessage(int statusCode, byte transactionId, PDXDictionary message, boolean error, boolean sendResponse) {
    	synchronized (syncObject) {
    		boolean success = resourceListener.receiveMdsMessageStatus(statusCode, transactionId, message) && !error;
    		if (sendResponse)
    		{
	    		if (success) {
	    			sendMdsMessageResponse(0);
	    		}
	    		else {
	    			sendMdsMessageResponse(1);
	    		}
    		}
    		return success;
    	}
    }
    
    public void onBcpMdsMessageResponse(int statusCode, byte transactionId) {
    	synchronized (syncObject) {
    		resourceListener.sendMdsMessageStatus(statusCode, transactionId);
    	}
    }

    // Should be overridden by specific resource
    public void onBcpGenerateAudioComplete(byte transactionId, long requestId,
            long audioId, short completionCause) {
    }

    // Should be overridden by specific resource
    public void onBcpRecognitionComplete(byte transactionId, long requestId,
            short completionCause, byte[] recResult) {
    }

    // Should be overridden by specific resource
    public void onBcpRecognitionIntermediateResults(byte transactionId,
            long requestId, short completionCause, byte[] recResult) {

    }
    
    // Should be overridden by specific resource
    public void onBcpData(byte transactionId, long requestId, byte[] data) {
    }
    
    // Should be overridden by specific resource
    public void onMsgSent(String context, Object data) {
    }
    
    // Should be overridden by specific resource
    public void onMsgNotSent(String context, Object data) {
    }

    // Should be overridden by specific resource
    public void onVapSending() {
    }
    
    // Should be overridden by specific resource
    public void onVapPlayBeginReceived()
    {
    }
    
    // Should be overridden by specific resource
    public void onVapPlayReceived()
    {
    }
    
    // Should be overridden by specific resource
    public void onVapPlayEndReceived()
    {
    }

    public void handleMessage(Object msgData, Object msgSender) {
        MessageData messageData = (MessageData) msgData;
        Object array[];

        switch (messageData.command) {
        case CMD_LOAD_RESOURCE:
            handleLoadResource(this, null);
            break;
//call loadResource() in createComand, loadResource will reconnect
//        case CMD_CONNECT:
//            Object[] params = (Object[])messageData.data;
//            Codec inputCodec = (Codec)params[0];
//            Codec outputCodec = (Codec)params[1];
//            handleConnect(inputCodec, outputCodec);
//            break;
        case CMD_SET_PARAM:
            array = (Object[]) messageData.data;
            handleSetParams((Vector) array[0], ((Long)array[1]).longValue());
            break;
        case CMD_GET_PARAM:
            array = (Object[]) messageData.data;
            handleGetParams((Vector) array[0], ((Long)array[1]).longValue());
            break;
        case CMD_FREE_RESOURCE:
            handleFreeResource(((Integer) messageData.data).intValue());
            break;
        case CMD_FREE_RESOURCE_ID:
            handleFreeResourceId(((Integer) messageData.data).intValue());
            break;
        case CMD_BCP_SEND_MSG:
            byte[] payload = (byte[]) messageData.data;
            handleSendMessage(payload);
            break;
        case CMD_BCP_SEND_MSG_RESPONSE:
            Integer statusCode = (Integer) messageData.data;
            handleSendMessageResponse(statusCode);
            break;
        }
        
    }
    
    public static long getDefaultRequestId() {
        return defaultReqId;
    }
}
