/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client2.sdk.components.resource.internal.nmas;
//#ifdef NMSP_LOG
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory.Log;
//#endif

public class PDXMessageFactory {
    //#ifdef NMSP_LOG
    private static final Log log = LogFactory.getLog(PDXMessageFactory.class);
    //#endif

    public static PDXMessage createMessage(byte[] input) {
        //#ifdef NMSP_LOG3
//@        String inputHex = "";
//@        for(int i = 0; i < input.length; i++) {
//@            inputHex += Integer.toString( ( input[i] & 0xff ) + 0x100, 16).substring( 1 );
//@        }
//@        log.debug("PDXMessageFactory.createMessage() " + inputHex);
        //#endif
        PDXMessage message = null;
        short command;
        byte[] payload;

        command = (short) (((input[1] & 0xFF) << 8) + (input[0] & 0xFF));
        payload = new byte[input.length - 2];
        System.arraycopy(input, 2, payload, 0, payload.length);

        switch (command) {
        case PDXMessage.QueryResult:
            message = new PDXQueryResult(payload);
            break;
        case PDXMessage.QueryError:
            message = new PDXQueryError(payload);
            break;
        case PDXMessage.QueryRetry:
            message = new PDXQueryRetry(payload);
            break;
        // case PDXMessage.QueryNarrow:
        // message = new PDXQueryNarrow(payload);
        // break;
        // case PDXMessage.QueryDisambiguate:
        // message = new PDXQueryDisambiguate(payload);
        // break;
        // case PDXMessage.ScriptUpdateAvailable:
        // message = new PDXScriptUpdateAvailable(payload);
        // break;
        // case PDXMessage.UpdateChunk:
        // message = new PDXUpdateChunk(payload);
        // break;
        // case PDXMessage.NoScriptUpdateAvailable:
        // message = new PDXNoScriptUpdateAvailable();
        // break;
        default:
            //#ifdef NMSP_LOG
            log.error("PDXMessageFactory.createMessage() Unknown command: " + command + ". ");
            //#endif
            break;
        }
        return message;
    }
}
