/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client2.sdk.components.resource.internal.nmas;

//<DICT>
//<ASCII>uid</ASCII><UTF8>(phone uid)</UTF8>
//<ASCII>pdx_version</ASCII><UTF8>(pdx protocol version string)</UTF8>
//<ASCII>client_version</ASCII><UTF8>(embedded client version string)</UTF8>
//<ASCII>script_version</ASCII><UTF8>(script version string)</UTF8>
//<ASCII>language</ASCII><UTF8>(language code (enUS, enGB, ...) )</UTF8>
//<ASCII>region</ASCII><UTF8>??</UTF8>
//<ASCII>device_codec</ASCII><INT>(codec used to record audio on the device, enUS, enGB)</INT>
//<ASCII>dictation_language</ASCII><UTF8>(language used for dictation)</UTF8>
//<ASCII>lcd_width</ASCII><INT>(lcd width )</INT>
//<ASCII>lcd_height</ASCII><INT>(lcd height )</INT>
//<ASCII>carrier</ASCII><UTF8>(carrier name)</UTF8>
//<ASCII>phone_model</ASCII><UTF8>(phone model)</UTF8>
//<ASCII>phone_number</ASCII><UTF8>(phone number)</UTF8>
//<ASCII>original_session_id</ASCII><UTF8>(original id)</UTF8>
//<ASCII>new_session_id</ASCII><UTF8>(new id)</UTF8>
//<ASCII>application</ASCII><ASCII>(application name)</ASCII>
//<ASCII>command</ASCII><ASCII>(command name)</ASCII>
//</DICT>

import java.util.Enumeration;

import com.nuance.nmsp.client2.sdk.common.defines.*;
import com.nuance.nmsp.client2.sdk.components.core.internal.pdx.PDXClass;
import com.nuance.nmsp.client2.sdk.components.core.internal.pdx.PDXDictionary;
import com.nuance.nmsp.client2.sdk.components.core.pdx.Dictionary;
import com.nuance.nmsp.client2.sdk.components.resource.internal.common.NMSPSession;
//#ifdef NMSP_LOG
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory.Log;
//#endif

public class PDXQueryBegin extends PDXMessage {
    //#ifdef NMSP_LOG
    private static final Log log = LogFactory.getLog(PDXQueryBegin.class);
    //#endif

    public PDXQueryBegin(String uid, String pdxVersion, String sdk_version, String clientVersion,
                         String scriptVersion, String language, String region, NMSPDefines.Codec codec,
                         String dictationLanguage, short lcdWidth, short lcdHeight,
                         String carrier, String phoneModel, String phoneNumber,
                         String originalSessionId, byte[] newSessionId, String application,
                         String command, Dictionary optionalKeys) {
        super(PDXMessage.QueryBegin);
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("PDXQueryBegin()");
        //#endif
        put("uid", uid, PDXClass.UTF8);
        put("pdx_version", pdxVersion, PDXClass.UTF8);
        put("sdk_version", sdk_version, PDXClass.UTF8);
        put("client_version", clientVersion, PDXClass.UTF8);
        put("script_version", scriptVersion, PDXClass.UTF8);
        put("language", language, PDXClass.UTF8);
        put("region", region, PDXClass.UTF8);
        put("device_codec", codec.getValue(), PDXClass.INT);
        put("dictation_language", dictationLanguage, PDXClass.UTF8);
        put("lcd_width", lcdWidth, PDXClass.INT);
        put("lcd_height", lcdHeight, PDXClass.INT);
        if (carrier == null)
            put("carrier", new byte[0], PDXClass.NULL);
        else
            put("carrier", carrier, PDXClass.UTF8);
        put("phone_model", phoneModel, PDXClass.UTF8);
        put("phone_number", phoneNumber, PDXClass.UTF8);
        put("original_session_id", originalSessionId, PDXClass.ASCII);
        if(newSessionId != null) {
            put("new_session_id", NMSPSession.FormatUuid(newSessionId), PDXClass.UTF8);
        }
        put("application", application, PDXClass.ASCII);
        put("nmaid", application, PDXClass.ASCII);
        put("command", command, PDXClass.ASCII);
        
//        if(msgData_dict != null) 
//        	addAll(msgData_dict);
        
        if(optionalKeys != null) {
            Enumeration keys = optionalKeys.keys();
            while(keys.hasMoreElements()) {
                String key = (String)keys.nextElement();
                switch((((PDXDictionary)optionalKeys).getClass(key)).getType()) {
                case PDXClass.BOOLEAN:
                	put(key, optionalKeys.getBoolean(key), PDXClass.BOOLEAN);
                	break;
                case PDXClass.DOUBLE:
                	put(key, optionalKeys.getDouble(key), PDXClass.DOUBLE);
                	break;                
                case PDXClass.ASCII:
                    put(key, optionalKeys.getAsciiString(key), PDXClass.ASCII);
                    break;
                case PDXClass.BYTES:
                    put(key, optionalKeys.getByteString(key), PDXClass.BYTES);
                    break;
                case PDXClass.INT:
                    put(key, optionalKeys.getInteger(key), PDXClass.INT);
                    break;
                case PDXClass.UTF8:
                    put(key, optionalKeys.getUTF8String(key), PDXClass.UTF8);
                    break;
                case PDXClass.NULL:
                    put(key, (byte[])null, PDXClass.NULL);
                    break;
                case PDXClass.DICT:
                    addDictionary(key, optionalKeys.getDictionary(key));
                    break;
                case PDXClass.SEQ:
                    addSequence(key, optionalKeys.getSequence(key));
                    break;
                }
            }
        }
    }

    void setSessionId(byte[] sessionId) {
        //#ifdef NMSP_LOG
        log.debug("PDXQueryBegin.setSessionId()");
        //#endif
        put("new_session_id", sessionId, PDXClass.ASCII);
    }
}
