/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client2.sdk.components.resource.internal.nmas;
//#ifdef NMSP_LOG
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory.Log;
//#endif
import com.nuance.nmsp.client2.sdk.components.core.internal.pdx.PDXClass;

//<DICT>
//<ASCII>name</ASCII><UTF8>(Name of the parameter)</UTF8>
//<ASCII>type</ASCII><INT>(Enum of type of parameter)</INT>
//
//(Depending on type - either)
//<ASCII>buffer_id</ASCII><UTF8>(VAP Audio buffer identifier)</UTF8>
//<ASCII>text</ASCII><UTF8>(User entered text or choice)</UTF8>
//<ASCII>data</ASCII><UTF8>(User entered text)</UTF8>
//</DICT>

public class PDXQueryParameter extends PDXMessage {
    //#ifdef NMSP_LOG
    private static final Log log = LogFactory.getLog(PDXQueryParameter.class);
    //#endif

    public PDXQueryParameter(PDXParam param) {
        super(PDXMessage.QueryParameter);
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("PDXQueryParameter()");
        //#endif
        put("name", param.getName(), PDXClass.UTF8);
        if (param.getType() == PDXParam.TTS_PARAMETER) { // A TTS param is actually a TEXT param
            put("type", PDXParam.DICT_PARAMETER, PDXClass.INT);
        } else {
            put("type", param.getType(), PDXClass.INT);
        }

        switch (param.getType()) {
        case PDXParam.AUDIO_PARAMETER:
            put("buffer_id", ((PDXAudioParam) param).getBufferId(), PDXClass.INT);
            break;
        case PDXParam.CHOICE_PARAMETER:
            put("text", ((PDXChoiceParam) param).getChoicename(), PDXClass.UTF8);
            break;
        case PDXParam.DATA_PARAMETER:
            put("data", ((PDXDataParam) param).getData(), PDXClass.UTF8);
            break;
        case PDXParam.DICT_PARAMETER:
            put("dict", ((PDXDictParam) param).getContent(), PDXClass.DICT);
            break;
        case PDXParam.SEQ_PARAMETER_START:
        case PDXParam.SEQ_PARAMETER_CHUNK:
        case PDXParam.SEQ_PARAMETER_END:
            put("dict", ((PDXSeqParam) param).getContent(), PDXClass.DICT);
            break;
        case PDXParam.TEXT_PARAMETER:
            put("text", ((PDXTextParam) param).getText(), PDXClass.UTF8);
            break;
        case PDXParam.TTS_PARAMETER:
            put("dict", ((PDXTTSParam) param).getContent(), PDXClass.DICT);
            break;
        default:
            //#ifdef NMSP_LOG
            log.error("PDXQueryParameter() Unknown parameter type: " + param.getType() + ". ");
            //#endif
            break;
        }
    }
}
