/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client2.sdk.components.resource.nmas;

import java.util.Vector;

import com.nuance.nmsp.client2.sdk.components.general.Parameter;
import com.nuance.nmsp.client2.sdk.components.resource.common.Manager;
import com.nuance.nmsp.client2.sdk.components.resource.internal.common.ManagerImpl;
import com.nuance.nmsp.client2.sdk.components.resource.internal.nmas.NMASResourceImpl;
import com.nuance.nmsp.client2.sdk.components.resource.internal.nmas.PDXQueryResult;
//#ifdef NMSP_LOG
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory.Log;
//#endif

/**
 * Has a static method for creating an
 * {@link NMASResource} interface.
 * 
 * @see NMASResourceListener
 * 
 * @author Nuance Communication Inc.
 */
public class NMASResourceFactory {
    //#ifdef NMSP_LOG
    private static final Log log = LogFactory.getLog(NMASResourceFactory.class);
    //#endif

    private NMASResourceFactory() {
    }

    /**
     * Factory method to create the {@link NMASResource}.
     * 
     * @param manager the manager associated with a session
     * @param nmasListener the NMASResource listener for NMAS events
     * @param parameters
     *            vector of {@link Parameter} type SDK or NMSP
     * @param application the user application name
     * 
     * @return the created NMASResource
     */
    public static NMASResource createNMASResource(Manager manager, NMASResourceListener nmasListener, Vector parameters, String application, String uid, String accessToken) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("createNMASResource");
        //#endif
        if(manager == null) {
            //#ifdef NMSP_LOG
            log.error("manager is null");
            //#endif
            throw new NullPointerException("manager can not be null!");
        }
        if (nmasListener == null) {
            //#ifdef NMSP_LOG
            log.error("nmasListener is null");
            //#endif
            throw new NullPointerException("nmasListener can not be null!");
        }
        if (parameters != null) {
            for (int i = 0; i < parameters.size(); i++) {
                Parameter parameter = (Parameter) parameters.elementAt(i);
                if(parameter.getType() == Parameter.Type.APP ||
                   parameter.getType() == Parameter.Type.NSS ||
                   parameter.getType() == Parameter.Type.SLOG ||
                   parameter.getType() ==  Parameter.Type.NSSLOG ||
                   parameter.getType() ==  Parameter.Type.GWLOG ||
                   parameter.getType() ==  Parameter.Type.SVSP ||
                   parameter.getType() ==  Parameter.Type.SIP ||
                   parameter.getType() ==  Parameter.Type.SDP) {
                    //#ifdef NMSP_LOG
                    log.error("IllegalArgumentException Parameter type: " + parameter.getType() + " not allowed. ");
                    //#endif
                    throw new IllegalArgumentException("Parameter type: " + parameter.getType() + " not allowed. ");
                }
            }
        }
        return new NMASResourceImpl((ManagerImpl)manager, nmasListener, parameters, application, uid, accessToken);
    }
    
    /**
     * Factory method to create a {@link QueryResult} from raw data previously
     * obtained from {@link QueryResult#getRawInput}.
     * 
     * @param rawData the raw input data for creating the query result
     * 
     * @return the created QueryResult
     */
    public static QueryResult createQueryResult(byte [] rawData)
    {
        return new PDXQueryResult(rawData);
    }
}
