/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client2.sdk.oem;

import java.util.Set;
import java.util.Vector;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.content.pm.PackageManager;
import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioRecord;
import android.media.AudioTrack;
import android.media.AudioTrack.OnPlaybackPositionUpdateListener;
import android.media.MediaRecorder.AudioSource;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;

import com.nuance.nmsp.client2.sdk.common.defines.NMSPDefines;
import com.nuance.nmsp.client2.sdk.common.defines.NMSPDefines.Codec;
import com.nuance.nmsp.client2.sdk.common.oem.api.AudioSystem;
import com.nuance.nmsp.client2.sdk.common.oem.api.MessageSystem;
import com.nuance.nmsp.client2.sdk.common.oem.api.MessageSystem.MessageHandler;
import com.nuance.nmsp.client2.sdk.common.oem.api.TimerSystem.TimerSystemTask;
import com.nuance.nmsp.client2.sdk.components.general.Parameter;

//#ifdef NMSP_LOG
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory.Log;
//#endif
import com.nuance.nmsp.client2.sdk.oem.bluetooth.Audio;
import com.nuance.nmsp.client2.sdk.oem.bluetooth.Bluetooth;
import com.nuance.nmsp.client2.sdk.oem.bluetooth.BluetoothHeadset;
import com.nuance.nmsp.client2.sdk.oem.bluetooth.BluetoothHeadset.ServiceListener;


public class AudioSystemOEM implements AudioSystem, MessageHandler {

    //#ifdef NMSP_LOG
    // Logger
    private Log log = LogFactory.getLog(this.getClass());
    //#endif

    // Playback / recorder shared constants (first will be assigned by constructor)
    private static final int CHANNEL_CONFIGURATION = AudioFormat.CHANNEL_CONFIGURATION_MONO;
    private static final int PCM_ENCODING = AudioFormat.ENCODING_PCM_16BIT;

    // Playback constants. Assigned by contructor.
    private int PLAYBACK_AUDIO_STREAM;
    private int PLAYBACK_MODE;
    private int PLAYBACK_SAMPLE_RATE_IN_HZ;
    private int PLAYBACK_PACKET_DURATION_IN_MS;
    private int PLAYBACK_PCM_PACKET_SIZE_IN_SHORTS;
    private int PLAYBACK_PCM_PACKET_SIZE_IN_BYTES;
    private int PLAYBACK_PCM_BUFFER_SIZE_IN_SHORTS;
    private int PLAYBACK_PCM_BUFFER_SIZE_IN_BYTES;
    private int PLAYBACK_BUFFER_MIN_DURATION_IN_MS;
    private int PLAYBACK_BUFFER_MIN_SIZE_IN_SHORTS;
    //private final int PLAYBACK_VOC_PACKET_SIZE_IN_BYTES;
    private int PLAYBACK_PCM_MIN_AUDIO_DURATION_IN_MS;
    private int PLAYBACK_PCM_MIN_AUDIO_SIZE_IN_SHORTS;
    private int PLAYBACK_PCM_MIN_AUDIO_SIZE_IN_BYTES;

    // Recorder constants. Assigned by contructor.
    private int RECORDER_AUDIO_SOURCE;
    private int RECORDER_SAMPLE_RATE_IN_HZ;
    private int RECORDER_PCM_PACKET_SIZE_IN_SHORTS;
    private int RECORDER_PCM_PACKET_SIZE_IN_BYTES;
    private int RECORDER_BUFFER_SIZE_IN_BYTES;
    private int RECORDER_VOC_PACKET_SIZE_IN_BYTES;

    /**
     * Internal parameter for recorder audio packet size
    **/
    private static final String RECORD_PACKET_DURATION = "Audio_Packet_Duration";
    private int _recordPacketDurationInMS = 40;
    
    
    // Player and recorder
    private PlaybackThread mPlaybackThread;
    private Recorder mRecorder;

    // lock objects
    private static Object audioRecordLock = new Object();
    private static Object audioTrackLock = new Object();
    private static Object playbackStoppingLock = new Object();

    // End-pointer logic
    private EndPointer endPointer = null;
    /**
     * Internal Exception class
     *
     */
    public static class AudioSystemException extends Exception {
        private static final long serialVersionUID = 1L;

        public AudioSystemException(String detailMessage) {
            super(detailMessage);
        }
    }


        // Load speex library
        static {
        	try{
	            System.loadLibrary("nmsp_speex");
	            //System.loadLibrary("nmsp_opus");
	            System.loadLibrary("dmt_opus");
        	}catch(Exception e)
        	{
        		System.out.println("Exception loading audio library : " + e.getMessage());
        	}
        }
        
        long _opusEncoderHandle = 0;
        private static native long initializeOpusEncoder(int[] retError,
									                int samplingRate,
									                int bitRate,
									                int complexity,
											boolean vbr);

		private static native int encodeOpus(long encoder, short[] inputPcmBuffer, int[] inputPcmBufferLen, byte[] outputOpusBuffer, int[] outputOpusBufferLen);
		private static native void releaseOpusEncoder(long encoder);
		
		long _opusDecoderHandle = 0;
	    private static native long initializeOpusDecoder(int[] retError, int samplingRate);
		private static native int decodeOpus(long decoder, byte[] inputOpusBuffer, int[] inputOpusBufferLen, short[] outputPcmBuffer, int[] outputPcmBufferLen);
		private static native void releaseOpusDecoder(long decoder);


        public  static native int encodeInitSpeex(int mode, int samplingRate, int complexity, int quality, int vad);
    //  private static native int encodeGetFrameSizeSpeex();
        public static native int encodeSpeex(short[] inputFrame, byte[] outputBytes, int outputMaxLength);
        public static native void encodeCleanupSpeex();
        public static native int decodeInitSpeex(int mode, int enh, int samplingRate);
    //  private static native int decodeGetFrameSizeSpeex();
        public static native int decodeSpeex(byte[] inputBytes, int inputBytesLen, short[] outputFrame, int outputMaxLength);
        public static native void decodeCleanupSpeex();
        public static native String speexCodeToString(int speexCode);
        public static boolean isSpeexDecodeInit = false;

        private short[] decFrame;
        private byte[] nextSpeexPackage;

        // OPUS parameters
        private boolean 	PARAM_OPUS_ENABLE_VBR = false;
        private int 		PARAM_OPUS_ENC_QUALITY = 10;
        private int 		PARAM_OPUS_BITRATE = 28000;
        private int		PARAM_OPUS_PACKET_LENGTH = 20;


        // Speex Parameters (the first two will be assigned by constructor)
        private int PARAM_SPEEX_MODE;
        private int PARAM_SPEEX_ENC_QUALITY;
        // For now, those parameters are hardcoded
        private final int PARAM_SPEEX_ENC_COMPLEXITY = 3; // Set the CPU resources allowed for the encoder (1 to 10)
        private final int PARAM_SPEEX_ENC_VAD = 1; // Voice activity detection on (1) or off (0). Need to be set to 1
        private final int PARAM_SPEEX_DEC_ENH = 0; // Perceptual enhancer on (1) or off (0)

        private int PARAM_SAMPLING_RATE_IN_HZ;
        private final int PLAYBACK_MAX_PCM_PACKET_SIZE_IN_SHORTS = 512*1024*4;
        private final int PLAYBACK_MAX_SPEEX_PACKET_SIZE = 512*1024;

        // The codec type to use
        public static class CodecType {
            public static CodecType SPEEX = new CodecType();
            public static CodecType PCM = new CodecType();
            public static CodecType OPUS = new CodecType();

            private CodecType(){}
        }
        private CodecType CODEC_TYPE;

        // Speex lock objects
        public static Object speexEncodeLock = new Object();
        public static Object speexDecodeLock = new Object();

        public static int bufferOffset = 0;
        public static int maxOffset = 0;

        private boolean useEnergyLevel = false;

        private MessageSystem _msgSys;
        private boolean _continuesOnEndPointerAndTimerStopping = false;
        private boolean _capturingContinuesOnEndPointer = false;

        private final static Integer CALLBACK_TYPE_END_POINTER_STARTED      = new Integer(1);
        private final static Integer CALLBACK_TYPE_END_POINTER_STOPPED      = new Integer(2);
        private final static Integer CALLBACK_TYPE_ON_PERIODIC_NOTIFICATION = new Integer(3);

        private Context _context = null;
        private Bluetooth _bt = null;
        private boolean _isBluetoothEnabled = false;

        private NMSPDefines.Codec _codec;

        private void init(NMSPDefines.Codec codec)
        {
            if (codec == Codec.PCM_16_8K) {
                CODEC_TYPE = CodecType.PCM;
                PARAM_SAMPLING_RATE_IN_HZ = 8000;
                PARAM_SPEEX_MODE = -1; // Invalid value;
                PARAM_SPEEX_ENC_QUALITY = -1; // Invalid value;
            }else if (codec == Codec.PCM_16_11K) {
                CODEC_TYPE = CodecType.PCM;
                PARAM_SAMPLING_RATE_IN_HZ = 11000;
                PARAM_SPEEX_MODE = -1; // Invalid value;
                PARAM_SPEEX_ENC_QUALITY = -1; // Invalid value;
            }
            else if (codec == Codec.PCM_16_16K) {
                CODEC_TYPE = CodecType.PCM;
                PARAM_SAMPLING_RATE_IN_HZ = 16000;
                PARAM_SPEEX_MODE = -1; // Invalid value;
                PARAM_SPEEX_ENC_QUALITY = -1; // Invalid value;
            } else if (codec == Codec.PCM_16_32K) {
                CODEC_TYPE = CodecType.PCM;
                PARAM_SAMPLING_RATE_IN_HZ = 32000;
                PARAM_SPEEX_MODE = -1; // Invalid value;
                PARAM_SPEEX_ENC_QUALITY = -1; // Invalid value;
            } else if (codec == Codec.SPEEX_8K){
                CODEC_TYPE = CodecType.SPEEX;
                PARAM_SAMPLING_RATE_IN_HZ = 8000;
                PARAM_SPEEX_ENC_QUALITY = 6;
                PARAM_SPEEX_MODE = 0;
            } else if (codec == Codec.SPEEX_16K){
                CODEC_TYPE = CodecType.SPEEX;
                PARAM_SAMPLING_RATE_IN_HZ = 16000;
                PARAM_SPEEX_ENC_QUALITY = 8;
                PARAM_SPEEX_MODE = 1;
            } else if (codec == Codec.OPUS_16K) {
            	CODEC_TYPE = CodecType.OPUS;
            	PARAM_SAMPLING_RATE_IN_HZ = 16000;
            } else if (codec == Codec.OPUS_8K) {
            	CODEC_TYPE = CodecType.OPUS;
            	PARAM_SAMPLING_RATE_IN_HZ = 8000;
            }

            // -----------------------------------------
            // Adding new 11k speex support
            // SPEEX_SB_NR0..SPEEX_SB_NR10
            // we will treat them 16k
            // -----------------------------------------
            else if (codec == Codec.SPEEX_11K){
                CODEC_TYPE = CodecType.SPEEX;
                PARAM_SAMPLING_RATE_IN_HZ = 16000;
                PARAM_SPEEX_ENC_QUALITY = 8;
                PARAM_SPEEX_MODE = 1;
            }
            // ------------------------------------------------
            // Do we need to add variable size for
            // these codecs?
            // It seems variable size is not supported because
            // cpu limitations on device
            // ------------------------------------------------
            // Codec.SPEEX_VBR_SB_R0..Codec.SPEEX_VBR_SB_R10
            //else if (codec == Codec.SPEEX_VBR_SB_R0..10){
            //    CODEC_TYPE = CodecType.SPEEX;
            //    PARAM_SAMPLING_RATE_IN_HZ = 16000;
            //    PARAM_SPEEX_ENC_QUALITY = 0..10;
            //    PARAM_SPEEX_MODE = 1;
            //}

             else {
                // It should never happen since other Codecs are not available for Android
                //#ifdef NMSP_LOG
                if(log.isErrorEnabled())
                log.error("Codec " + codec + " is not handled, using PCM_16_8K by default.");
                //#endif

                CODEC_TYPE = CodecType.PCM;
                PARAM_SAMPLING_RATE_IN_HZ = 8000;
                PARAM_SPEEX_MODE = -1; // Invalid value;
                PARAM_SPEEX_ENC_QUALITY = -1; // Invalid value;
            }


            // Initialize playback constants
            PLAYBACK_MODE = AudioTrack.MODE_STREAM;
            PLAYBACK_SAMPLE_RATE_IN_HZ = PARAM_SAMPLING_RATE_IN_HZ;
            // by experiments: for both PCM 16 bit 8K and 16K
            // we have to stream at least 750 ms audio to the AudioTrack (native player)
            // in order to let it start playing back the streamed audio buffers
            PLAYBACK_PCM_MIN_AUDIO_DURATION_IN_MS = 750;
            PLAYBACK_PCM_MIN_AUDIO_SIZE_IN_SHORTS = PLAYBACK_PCM_MIN_AUDIO_DURATION_IN_MS * PLAYBACK_SAMPLE_RATE_IN_HZ / 1000;
            PLAYBACK_PCM_MIN_AUDIO_SIZE_IN_BYTES = PLAYBACK_PCM_MIN_AUDIO_SIZE_IN_SHORTS * 2;
            PLAYBACK_PACKET_DURATION_IN_MS = 300;
            PLAYBACK_PCM_PACKET_SIZE_IN_SHORTS = PLAYBACK_SAMPLE_RATE_IN_HZ * PLAYBACK_PACKET_DURATION_IN_MS / 1000;
            PLAYBACK_PCM_PACKET_SIZE_IN_BYTES = PLAYBACK_PCM_PACKET_SIZE_IN_SHORTS * 2;
            // Determine the playback buffer size. It's either 5 PCM packet or the minimum size handled by the phone.
            int targetPlaybackBufferSizeInBytes = PLAYBACK_PCM_PACKET_SIZE_IN_BYTES * 5;
            int minPlaybackBufferSizeInBytes = AudioTrack.getMinBufferSize(PLAYBACK_SAMPLE_RATE_IN_HZ, CHANNEL_CONFIGURATION, PCM_ENCODING);
            if ((minPlaybackBufferSizeInBytes != AudioTrack.ERROR_BAD_VALUE) &&
                (minPlaybackBufferSizeInBytes != AudioTrack.ERROR) &&
                (minPlaybackBufferSizeInBytes > targetPlaybackBufferSizeInBytes)){
                PLAYBACK_PCM_BUFFER_SIZE_IN_BYTES = minPlaybackBufferSizeInBytes;
            }
            else {
                PLAYBACK_PCM_BUFFER_SIZE_IN_BYTES = targetPlaybackBufferSizeInBytes;
            }
            PLAYBACK_PCM_BUFFER_SIZE_IN_SHORTS = PLAYBACK_PCM_BUFFER_SIZE_IN_BYTES / 2;
            PLAYBACK_BUFFER_MIN_DURATION_IN_MS = 500;
            PLAYBACK_BUFFER_MIN_SIZE_IN_SHORTS = PLAYBACK_SAMPLE_RATE_IN_HZ * PLAYBACK_BUFFER_MIN_DURATION_IN_MS / 1000;
            //PLAYBACK_VOC_PACKET_SIZE_IN_BYTES = PLAYBACK_PCM_PACKET_SIZE_IN_BYTES; // TODO: in reality, we need much less space

            // Initialize recording constants
            RECORDER_SAMPLE_RATE_IN_HZ = PARAM_SAMPLING_RATE_IN_HZ;
            RECORDER_PCM_PACKET_SIZE_IN_SHORTS = RECORDER_SAMPLE_RATE_IN_HZ * _recordPacketDurationInMS / 1000;
            RECORDER_PCM_PACKET_SIZE_IN_BYTES = RECORDER_PCM_PACKET_SIZE_IN_SHORTS * 2;
            // Determine the recorder buffer size. It's either 3 PCM packet or the minimum size handled by the phone.
            int targetRecorderBufferSizeInBytes = RECORDER_PCM_PACKET_SIZE_IN_BYTES * 3; // Need enough space (used to be 4096*2)
            int minRecorderBufferSizeInBytes = AudioRecord.getMinBufferSize(RECORDER_SAMPLE_RATE_IN_HZ, CHANNEL_CONFIGURATION, PCM_ENCODING);
            if ((minRecorderBufferSizeInBytes != AudioRecord.ERROR_BAD_VALUE) &&
                (minRecorderBufferSizeInBytes != AudioRecord.ERROR) &&
                (minRecorderBufferSizeInBytes > targetRecorderBufferSizeInBytes)){
                RECORDER_BUFFER_SIZE_IN_BYTES = minRecorderBufferSizeInBytes;
            }
            else {
                RECORDER_BUFFER_SIZE_IN_BYTES = targetRecorderBufferSizeInBytes;
            }

            RECORDER_VOC_PACKET_SIZE_IN_BYTES = RECORDER_PCM_PACKET_SIZE_IN_BYTES; // TODO: in reality, we need much less space
        }

        public AudioSystemOEM(MessageSystem msgSys, NMSPDefines.Codec codec, Vector parameters)  {
            _codec = codec;
            _msgSys = msgSys;

        if (parameters != null) {

            for (int i = 0; i < parameters.size() ; i++) {
                Parameter curParam = (Parameter) parameters.get(i);
                String curParamName = curParam.getName();
                if(curParam.getType()  == Parameter.Type.SDK)
                {
	                if (curParamName.equals(NMSPDefines.NMSP_DEFINES_USE_ENERGY_LEVEL)) {
	                    String curParamValue = new String(curParam.getValue());
	                    if (curParamValue.equalsIgnoreCase("TRUE")) {
	                        //#ifdef NMSP_LOG
	                        if(log.isDebugEnabled())
	                            log.debug("Use energy level is activated.");
	                        //#endif
	                        useEnergyLevel = true;
	                    }
	                }
	                else if (curParamName.equals(NMSPDefines.NMSP_DEFINES_RECORDER_CONTINUES_ON_ENDPOINTER_AND_TIMER_STOPPING)) {
	                    String curParamValue = new String(curParam.getValue());
	                    if (curParamValue.equalsIgnoreCase("TRUE")) {
	                        //#ifdef NMSP_LOG
	                        if(log.isDebugEnabled())
	                            log.debug("_continuesOnEndPointerAndTimerStopping is activated.");
	                        //#endif
	                        _continuesOnEndPointerAndTimerStopping = true;
	                    }
	                }
	                else if (curParamName.equals(NMSPDefines.NMSP_DEFINES_CAPTURING_CONTINUES_ON_ENDPOINTER)) {
	                    String curParamValue = new String(curParam.getValue());
	                    if (curParamValue.equalsIgnoreCase("TRUE")) {
	                        //#ifdef NMSP_LOG
	                        if(log.isDebugEnabled())
	                            log.debug("_capturingContinuesOnEndPointer is activated.");
	                        //#endif
	                    	_capturingContinuesOnEndPointer = true;
	                    }
	                }
	                else if (curParamName.equals(NMSPDefines.NMSP_DEFINES_ANDROID_CONTEXT)) {
                        _context = (Context)curParam.getValueRaw();
                        //#ifdef NMSP_LOG
                        if(log.isDebugEnabled())
                            log.debug("NMSP_DEFINES_ANDROID_CONTEXT is passed in as" + _context);
                        //#endif
                        //_bt = Bluetooth.createInstance(_context);
                    }
                    else if (curParamName.equals(RECORD_PACKET_DURATION)) {
                        int duration = Integer.parseInt(new String(curParam.getValue()));
                        if (duration > 0 ) {
                            _recordPacketDurationInMS = duration;
                        }
                    }
                    else if (curParamName.equals(NMSPDefines.NMSP_DEFINES_OPUS_BITRATE)) {
                    	PARAM_OPUS_BITRATE = Integer.parseInt(new String(curParam.getValue()));
                        //#ifdef NMSP_LOG
                        if(log.isDebugEnabled())
                            log.debug("NMSP_DEFINES_OPUS_BITRATE is passed in as" + PARAM_OPUS_BITRATE);
                        //#endif
                    }
                    else if (curParamName.equals(NMSPDefines.NMSP_DEFINES_OPUS_VBR_ENABLED)) {
                    	PARAM_OPUS_ENABLE_VBR = Boolean.parseBoolean(new String(curParam.getValue()));
                        //#ifdef NMSP_LOG
                        if(log.isDebugEnabled())
                            log.debug("NMSP_DEFINES_OPUS_VBR_ENABLED is passed in as" + PARAM_OPUS_ENABLE_VBR);
                        //#endif
                    }
                    else if (curParamName.equals(NMSPDefines.NMSP_DEFINES_OPUS_COMPLEXITY)) {
                    	PARAM_OPUS_ENC_QUALITY = Integer.parseInt(new String(curParam.getValue()));
                        //#ifdef NMSP_LOG
                        if(log.isDebugEnabled())
                            log.debug("NMSP_DEFINES_OPUS_COMPLEXITY is passed in as" + PARAM_OPUS_ENC_QUALITY);
                        //#endif
                    }
                    else if (curParamName.equals(NMSPDefines.NMSP_DEFINES_OPUS_PACKET_LENGTH)) {
                    	PARAM_OPUS_PACKET_LENGTH = Integer.parseInt(new String(curParam.getValue()));
                        //#ifdef NMSP_LOG
                        if(log.isDebugEnabled())
                            log.debug("NMSP_DEFINES_OPUS_PACKET_LENGTH is passed in as" + PARAM_OPUS_PACKET_LENGTH);
                        //#endif
                    }

                }
            }
        }

        //RECORD_PACKET_DURATION may affect codec parameters.
        init(codec);
        
        // Initialize end-pointer
        if(   CODEC_TYPE == CodecType.SPEEX 
           || CODEC_TYPE == CodecType.PCM){
	        endPointer = new EndPointerOEM();
	        endPointer.initialize(parameters);
        }else if ( CODEC_TYPE == CodecType.OPUS ){
        	endPointer = new ElvisEndPointer(PARAM_SAMPLING_RATE_IN_HZ);
        	endPointer.initialize(parameters);
        }
    }

    private class PlaybackThread extends Thread implements OnPlaybackPositionUpdateListener {
        //#ifdef NMSP_LOG
        // Logger
        private Log log = LogFactory.getLog(this.getClass());
        //#endif

        private static final int PLAY = 1;
        private static final int STOP = 2;
        private static final int ABORT = 3;

        private Handler mHandler;
        private AudioTrack mTrack;
        private byte[] mByteBuffer;
        private short[] mShortBuffer;
        private boolean mStopped;
        private int mBytesWritten;
        private int mShortsWritten;
        private boolean askMoreData = true;
        private int lastPlaybackPos;
        private boolean mStopping = false;
        private boolean mAborted = false;

        private AudioCallback audioCb = null;
        private DoneCallback doneCb = null;
        private StopCallback stopCb = null;
        private Object contextStart = null;
        private Object contextStop = null;

        //#ifdef NMSP_LOG
        private int sampleNumber = 0;
        //#endif

        private void onPlaybackError(AudioSystemException ex){
            synchronized (audioTrackLock) {
                //#ifdef NMSP_LOG
                            if(log.isErrorEnabled())
                            log.error(ex.getMessage());
                //#endif

                // Stop player if possible
                if (mTrack != null){
                	if(mTrack.getPlayState() != AudioTrack.PLAYSTATE_STOPPED)
                	{
                		mTrack.stop();
                	}
                    mTrack.release();
                    Looper looper = Looper.myLooper();
                    if (looper != null)
                        looper.quit();
                    askMoreData = false;
                }

                // Stop BT
                if (_bt != null) {
                    _bt.stopBluetoothSco();
                    // Cleanup BT
                    cleanupBT();
                }

                doneCb.doneCallback(AudioStatus.AUDIO_ERROR, contextStart);
            }
        }

        public void run() {
            Looper.prepare();

            synchronized (this) {
                mHandler = new Handler() {
                    public void handleMessage(Message msg) {
                        switch (msg.what) {
                        case PLAY:
                            try {
                                onPlay();
                            }
                            catch (AudioSystemException ex){
                                onPlaybackError(ex);
                                return;
                            }
                            break;
                        case STOP:
                            onStop();
                            break;
                            
                        case ABORT:
                            Looper looper = Looper.myLooper();
                            if (looper != null)
                                looper.quit();
                            mAborted = true;
                            break;
                        }
                    }
                };

                try {
                    createAudioTrack();
                }
                catch (AudioSystemException ex){
                    onPlaybackError(ex);
                    return;
                }
                notifyAll();
            }

            Looper.loop();

            synchronized (this) {
                mStopped = true;
                notifyAll();
            }
        }

        private void createAudioTrack() throws AudioSystemException {
            synchronized (audioTrackLock) {
                //#ifdef NMSP_LOG
                if(log.isDebugEnabled())
                log.debug("Creating Audio Track");
                //#endif

                mByteBuffer = new byte[PLAYBACK_PCM_PACKET_SIZE_IN_BYTES];
                //mShortBuffer = new short[FRAME_SIZE];
                mShortBuffer = new short[PLAYBACK_BUFFER_MIN_SIZE_IN_SHORTS];

                // Buffer size considerations.
                // From one side, using the minimum buffer size results in choppy
                // playback.]]

                // From the other side, playback does not actually start until at
                // least the first
                // 'buffer size' samples are written to the audio track. This is why
                // we have to padd
                // very short prompts with silence up to the 'buffer size'. If the
                // 'buffer size' is
                // too long then too much silence is added and that negatively
                // affects digits read out.

                try{
                    lastPlaybackPos = 0;

                    mTrack = new AudioTrack(PLAYBACK_AUDIO_STREAM,
                            PLAYBACK_SAMPLE_RATE_IN_HZ, CHANNEL_CONFIGURATION,
                            PCM_ENCODING, PLAYBACK_PCM_BUFFER_SIZE_IN_SHORTS, PLAYBACK_MODE);

                    if ((mTrack == null) || (mTrack.getState() != AudioTrack.STATE_INITIALIZED)){
                        throw new AudioSystemException("AudioTrack object has not been initialized correctly. One or several parameters used to create it must be wrong.");
                    }

                    mTrack.setPlaybackPositionUpdateListener(this);

                    mTrack.play();
                }
                catch (IllegalArgumentException ex) {
                    throw new AudioSystemException("Could not instanciate AudioTrack object.");
                }
            }
        }

        public void startPlayback(AudioCallback audioCb, DoneCallback doneCb, Object contextStart) throws AudioSystemException{
            if (_bt != null) {
                _bt.startBluetoothSco();
            }

            synchronized (this) {
                //#ifdef NMSP_LOG
                sampleNumber = 0;
                //#endif

                this.audioCb = audioCb;
                this.doneCb = doneCb;
                this.contextStart = contextStart;

                // Initialize speex in vocoder mode
                if (CODEC_TYPE == CodecType.SPEEX){
                    int ret;
                    synchronized (speexDecodeLock) {
                        ret = decodeInitSpeex(PARAM_SPEEX_MODE, PARAM_SPEEX_DEC_ENH, PARAM_SAMPLING_RATE_IN_HZ);
                        if (ret < 0){
                            throw new AudioSystemException("decodeInitSpeex failed with code: " + speexCodeToString(ret));
                        }
                        decFrame = new short[PLAYBACK_MAX_PCM_PACKET_SIZE_IN_SHORTS];
                        nextSpeexPackage = new byte[PLAYBACK_MAX_SPEEX_PACKET_SIZE];
                        isSpeexDecodeInit = true;
                    }

                    bufferOffset = 0;
                    maxOffset = 0;
                }
                if (CODEC_TYPE == CodecType.OPUS){
                	int[] initError = new int[1];
                	if(_opusDecoderHandle != 0){
                		releaseOpusDecoder(_opusDecoderHandle);
                	}
                	_opusDecoderHandle = initializeOpusDecoder(initError, PARAM_SAMPLING_RATE_IN_HZ);
                	if(initError[0] != 0){
                		throw new AudioSystemException("initializeOpusDecoder failed with error : " + initError[0]);
                	}
                    decFrame = new short[PLAYBACK_MAX_PCM_PACKET_SIZE_IN_SHORTS];
                    nextSpeexPackage = new byte[PLAYBACK_MAX_SPEEX_PACKET_SIZE];
                }
                start();

                while (isAlive() && mHandler == null) {
                    try {
                        wait();
                    } catch (InterruptedException e) {

                    }
                }

                if (!isAlive())
                    return;

                if (mHandler != null)
                    mHandler.sendMessage(mHandler.obtainMessage(PLAY));
            }
        }
        public void stopPlayback(StopCallback stopCb, Object contextStop) {
            synchronized (playbackStoppingLock)
            {
                // Set mStopping before locking audioTrackLock, because otherwise we
                // can run into a deadlock as onPlay() is looking for audio data, but
                // never finds any because we are blocking this thread.
                mStopping = true;
            }

            synchronized (audioTrackLock)
            {
                askMoreData = false;
            }

            synchronized (this) {
                this.contextStop = contextStop;
                this.stopCb = stopCb;

                if (!isAlive() || mStopped)
                    return;

                if (mHandler != null)
                    mHandler.sendMessage(mHandler.obtainMessage(STOP));

                while (isAlive() && !mStopped) {
                    try {
                        wait();
                    } catch (InterruptedException e) {
                    }
                }
            }
        }

        public void onMarkerReached(AudioTrack track) {
//        	if(log.isDebugEnabled())
//        		log.debug("++++++++++++========== onMarkerReached(). PlaybackHeadPosition:" + mTrack.getPlaybackHeadPosition());

            try {
                onPlay();
            }
            catch (AudioSystemException ex){
                onPlaybackError(ex);
            }
         }

        public void onPeriodicNotification(AudioTrack track) {

//        	if(log.isDebugEnabled())
//        		log.debug("++++++++++++========== onPeriodicNotification(). PlaybackHeadPosition" + mTrack.getPlaybackHeadPosition());
        }

        public int getEncodedBuffer(short[] mBuffer, int requestSamples, IntegerMutable l, Object contextStart) throws AudioSystemException
        {
            if (requestSamples == 0)
            {
                return 0;
            }
            //if (CODEC_TYPE == CodecType.SPEEX || CODEC_TYPE == CodecType.OPUS){
                int shortsToCopy = requestSamples;
                int leftOver = (maxOffset - bufferOffset);

                if (leftOver >= shortsToCopy)
                {
                    System.arraycopy(decFrame, bufferOffset, mBuffer, 0, shortsToCopy);
                    bufferOffset+=shortsToCopy;
                    return shortsToCopy;
                }
                else if (leftOver > 0)
                {
                    System.arraycopy(decFrame, bufferOffset, mBuffer, 0, leftOver);
                    bufferOffset= 0;
                }
                else
                {
                    bufferOffset = 0;
                }

                audioCb.audioCallback(null, nextSpeexPackage, new IntegerMutable(0), l, null, contextStart);

                int pcmSamples = 0;
                if (l.i > 0){
                	if(CODEC_TYPE == CodecType.SPEEX)
                	{
	                    synchronized (speexDecodeLock) {
	                        pcmSamples = decodeSpeex(nextSpeexPackage,l.i, decFrame, decFrame.length);
	                        maxOffset = pcmSamples;
	                        if (maxOffset > decFrame.length)
	                            maxOffset = decFrame.length;
	                    }
                	}else if(CODEC_TYPE == CodecType.OPUS){
                		int[] inSize = new int[1];
                		int[] outSize = new int[1];
                		inSize[0] = l.i;
                		outSize[0] = decFrame.length;
                		int errorCode = decodeOpus(_opusDecoderHandle, nextSpeexPackage, inSize, decFrame, outSize);
                		if (errorCode != 0){
                			//#ifdef NMSP_LOG
                			if(log.isErrorEnabled())
                				log.error("Call to decodeOpus() failed with code: " + pcmSamples);
                			//#endif
                			throw new AudioSystemException("Call to decodeOpus() failed with code: " + pcmSamples);
                		}
                		else {
                			//#ifdef NMSP_LOG
                			if(log.isDebugEnabled())
                				log.debug("Obtained " + pcmSamples + " shorts of PCM data after decoding SPEEX.");
                			//#endif
                		}
                		pcmSamples = outSize[0];
                        assert(decFrame.length == outSize[0]);
                        maxOffset = pcmSamples;
                        if (maxOffset > decFrame.length)
                            maxOffset = decFrame.length;              
                        //#ifdef NMSP_LOG
                        if(log.isDebugEnabled())
                        log.debug("Number of bytes returned by decodeOpus : " + outSize[0]);
                        //#endif
                	}
                }

                if (leftOver>0) return leftOver;
            //}
            return 0;
        }
            private void onPlay() throws AudioSystemException {
            synchronized (audioTrackLock) {
                //#ifdef NMSP_LOG
                if(log.isDebugEnabled())
                log.debug("++++++++++++========== onPlay()");
                //#endif
                // Get data
                if (CODEC_TYPE == CodecType.SPEEX) {
                    // SPEEX
                    int offset = 0;

                    if (askMoreData) {
                        // Keep asking for data until a buffer is full
                        IntegerMutable l = new IntegerMutable(PLAYBACK_PCM_PACKET_SIZE_IN_SHORTS);
                        short[] tempBuffer = new short[PLAYBACK_PCM_PACKET_SIZE_IN_SHORTS];
                        while (offset < PLAYBACK_PCM_PACKET_SIZE_IN_SHORTS)
                        {
                            synchronized (playbackStoppingLock)
                            {
                                if (mStopping) {
                                    askMoreData = false;
                                    offset = 0;
                                    break;
                                }
                            }

                            int requestSamples = PLAYBACK_PCM_PACKET_SIZE_IN_SHORTS - offset;
                            int process = getEncodedBuffer(tempBuffer, requestSamples, l, contextStart);
                                                        //sleep a bit if there is no audio data available
                            if (process == 0) {
                                try {
                                    sleep(300);
                                } catch (InterruptedException e) {}
                            }

                            if (l.i == -1)
                            {
                                //#ifdef NMSP_LOG
                                if(log.isDebugEnabled())
                                log.debug("No more data to playback.");
                                //#endif

                                askMoreData = false;
                                break;
                            }
                            else if ((l.i > 0) && process>0){
                                System.arraycopy(tempBuffer, 0, mShortBuffer, offset, process);
                                offset+=process;
                            }
                        }
                    }
                    int numShortsToPlay = offset;
                    if (numShortsToPlay > 0)

                    // If buffer is less than a frame, fill it with silence
                    if (numShortsToPlay < PLAYBACK_PCM_PACKET_SIZE_IN_SHORTS) {
                        //#ifdef NMSP_LOG
                        if(log.isDebugEnabled())
                        log.debug("Filling buffer with silence.");
                        //#endif

                        for (int i = numShortsToPlay; i < PLAYBACK_PCM_PACKET_SIZE_IN_SHORTS; i++)
                            mShortBuffer[i] = 0;
                    }

                    if (numShortsToPlay > 0) {
                        offset = 0;
                        while (numShortsToPlay != 0) {
                            int written = mTrack.write(mShortBuffer, offset, PLAYBACK_PCM_PACKET_SIZE_IN_SHORTS);
                            if (written <= 0)
                                break;

                            numShortsToPlay -= written;
                            offset += written;

                            mShortsWritten += written;
                        }

                        int ret = mTrack.setNotificationMarkerPosition(mShortsWritten);
                        if (ret != AudioTrack.SUCCESS){
                            throw new AudioSystemException("Call to AudioTrack.setNotificationMarkerPosition() SPEEX 0 failed with code: " + ret);
                        }

                        mHandler.sendMessage(mHandler.obtainMessage(PLAY));
                    }
                    else {
                        // pad 0s if the complete streamed audio is shorter than 750ms
                        if (mShortsWritten < PLAYBACK_PCM_MIN_AUDIO_SIZE_IN_SHORTS) {
                            int numShortsToPad = PLAYBACK_PCM_MIN_AUDIO_SIZE_IN_SHORTS - mShortsWritten;
                            short[] shortsToPad = new short[numShortsToPad];
                            for (int i = 0; i < numShortsToPad; i++)
                                shortsToPad[i] = 0x00;
                            int written = mTrack.write(shortsToPad, 0, numShortsToPad);
                            if (written > 0) {
                                mShortsWritten += written;
                            }

                            int ret = mTrack.setNotificationMarkerPosition(mShortsWritten);
                            if (ret != AudioTrack.SUCCESS){
                                throw new AudioSystemException("Call to AudioTrack.setNotificationMarkerPosition() SPEEX 1 failed with code: " + ret);
                            }

                            mHandler.sendMessage(mHandler.obtainMessage(PLAY));
                            return;
                        }

                        //#ifdef NMSP_LOG
                            if(log.isDebugEnabled())
                            log.debug("onPlay() AudioTrack status is :" + mTrack.getState());
                        //#endif
                        int pos = 0;
                        int prev_pos = 0;
                        // only get position when the audio track state is good
                        if (mTrack.getState() != AudioTrack.STATE_UNINITIALIZED) {
                        	pos = mTrack.getPlaybackHeadPosition();
                        	prev_pos = pos;
                        	if (pos == lastPlaybackPos){
                        		// There is a bug in Android, when pos is near the end of buffer, it's not
                        		// updated. So if marker position did not changed, we force it and consider the audio has been played.
                        		pos = mShortsWritten;
                        	}
                        	lastPlaybackPos = pos;
                        }
                        

                        if (pos >= mShortsWritten ||
                        	mTrack.getState() == AudioTrack.STATE_UNINITIALIZED) {
                            long delay = 0;
                            if (pos > prev_pos) {
                                // calculate the delay each frame contains 1000/Rate miliseconds' audio
                                delay =  (pos - prev_pos) * 1000 / PARAM_SAMPLING_RATE_IN_HZ;
                            }
                            //#ifdef NMSP_LOG
                			if(log.isDebugEnabled()) {
                            	log.debug("calculating the delay......");
                            	log.debug("parameter pos = " + pos);
                            	log.debug("parameter prev_pos = " + prev_pos);
                            	log.debug("parameter delay = " + delay);
                        	}
                        	//#endif
                            // it does help trying to stop the AudioTrack after the audio has been
                            // really played, even if the the delay is 0 milisecond, it will help
                            // to play back a very short audio buffer
                            _msgSys.scheduleTask(new TimerSystemTask() {
                                public void run() {
                                    //#ifdef NMSP_LOG
                					if(log.isDebugEnabled())
                						log.debug("++++++++++++========== stop mTrack: " + mTrack);
                                	//#endif
                					synchronized (playbackStoppingLock)
                					{
                						// It is important to check if we are already stopping, do not attempt
                						// to stop again
                						// There is a potential race condition that leads to a deadlock when
                						// this task fires after some delay and overlaps another TTS request.
                						// In that case audioTrackLock will be reused by multiple instances
                						// of AudioSystemOEM. 
                						if(!mStopping && !mAborted)
                						{
                                        	synchronized (audioTrackLock) {
                                        		if(mTrack.getState() != AudioTrack.STATE_UNINITIALIZED)
                                        		{
                                        			mTrack.stop();
                                        		}
                                            	mTrack.flush();
                                            	mTrack.release();

                                                // Stop BT
                                                if (_bt != null) {
                                                    _bt.stopBluetoothSco();
                                                    // Cleanup BT
                                                    cleanupBT();
                                                }

                                                doneCb.doneCallback(AudioStatus.AUDIO_OK, contextStart);
     					
                                                // Cleanup speex
                                                synchronized (speexDecodeLock) {
                                                    if(isSpeexDecodeInit)
                                                    {
                                                        decodeCleanupSpeex();
                                                        decFrame = null;
                                                        nextSpeexPackage = null;
                                                        isSpeexDecodeInit = false;
                                                    }
                                                    else
                                                    {
                                                    //#ifdef NMSP_LOG
                					                if(log.isDebugEnabled())
                						                log.debug("Speex was already cleaned up");
                                	                //#endif
                                                    }
                                                }
                                                mHandler.sendMessage(mHandler.obtainMessage(ABORT));
                                        	}							
                							
                						}
                                        else
                                        {
                                        //#ifdef NMSP_LOG
                					    if(log.isDebugEnabled())
                						    log.debug("AudioSystemOEM was already stopping");
                                	    //#endif

                                        }
                					}

                                	

                                }
                            }, delay);

                        }
                    }
                }else if (CODEC_TYPE == CodecType.OPUS)
                {
                    try{
                    	playOpus();
                    }catch(Exception e)
                    {
                        //#ifdef NMSP_LOG
                        if(log.isErrorEnabled())
                        log.debug("AudioSystemException while getting Encoded Buffer");
                        //#endif
                    }
                	
                }
                else
                {
                    // Retrieve data to play
                    // PCM
                    int numBytesToPlay = 0;

                    if (askMoreData) {
                        IntegerMutable l = new IntegerMutable(PLAYBACK_PCM_PACKET_SIZE_IN_BYTES);
                        audioCb.audioCallback(mByteBuffer, null, l, new IntegerMutable(0), null, contextStart);
                        numBytesToPlay = l.i;
                        //#ifdef NMSP_LOG
                            if(log.isDebugEnabled())
                            log.debug("PLAYBACK: Sample #" + (sampleNumber++) + ". Asked for " + PLAYBACK_PCM_PACKET_SIZE_IN_BYTES + " bytes of data, " + numBytesToPlay + " bytes received.");
                        //#endif
                    }

                    // If buffer is less than a packet, fill it with silence
                    if (numBytesToPlay == 0 && mBytesWritten < PLAYBACK_PCM_PACKET_SIZE_IN_BYTES) {
                        //#ifdef NMSP_LOG
                            if(log.isDebugEnabled())
                            log.debug("Filling buffer with silence.");
                        //#endif

                        numBytesToPlay = PLAYBACK_PCM_PACKET_SIZE_IN_BYTES - mBytesWritten;
                        for (int i = 0; i < numBytesToPlay; i++)
                            mByteBuffer[i] = 0;
                    }

                    if (numBytesToPlay != -1) {
                        int offset = 0;
                        while (numBytesToPlay != 0) {
                            int written = mTrack.write(mByteBuffer, offset, numBytesToPlay);
                            if (written <= 0)
                                break;

                            numBytesToPlay -= written;
                            offset += written;

                            mBytesWritten += written;
                        }

                        // Each sample is 2 bytes
                        int ret = mTrack.setNotificationMarkerPosition(mBytesWritten / 2);
                        if (ret != AudioTrack.SUCCESS){
                            throw new AudioSystemException("Call to AudioTrack.setNotificationMarkerPosition() PCM 0 failed with code: " + ret);
                        }

                        mHandler.sendMessage(mHandler.obtainMessage(PLAY));
                    } else {
                        // pad 0s if the complete streamed audio is shorter than 750ms
                        if (mBytesWritten < PLAYBACK_PCM_MIN_AUDIO_SIZE_IN_BYTES) {
                            int numBytesToPad = PLAYBACK_PCM_MIN_AUDIO_SIZE_IN_BYTES - mBytesWritten;
                            byte[] bytesToPad = new byte[numBytesToPad];
                            for (int i = 0; i < numBytesToPad; i++)
                                bytesToPad[i] = 0;
                            int written = mTrack.write(bytesToPad, 0, numBytesToPad);
                            if (written > 0) {
                                mBytesWritten += written;
                            }

                            // Each sample is 2 bytes
                            int ret = mTrack.setNotificationMarkerPosition(mBytesWritten / 2);
                            if (ret != AudioTrack.SUCCESS){
                                throw new AudioSystemException("Call to AudioTrack.setNotificationMarkerPosition() PCM 1 failed with code: " + ret);
                            }

                            mHandler.sendMessage(mHandler.obtainMessage(PLAY));
                            return;
                        }
                        
                        //#ifdef NMSP_LOG
                            if(log.isDebugEnabled())
                            log.debug("onPlay() AudioTrack status is :" + mTrack.getState());
                        //#endif
                        int pos = 0;
                        int prev_pos = 0;
                        // only get position when the audio track state is good
                        if (mTrack.getState() != AudioTrack.STATE_UNINITIALIZED) {
                        	pos = mTrack.getPlaybackHeadPosition();
                        	prev_pos = pos;
                        	if (pos == lastPlaybackPos){
                        		// There is a bug in Android, when pos is near the end of buffer, it's not
                        		// updated. So if marker position did not changed, we force it and consider the audio has been played.
                        		pos = mBytesWritten / 2;
                        	}
                        	lastPlaybackPos = pos;
                        }

                        if (pos >= (mBytesWritten / 2)) // Each sample is 2 bytes
                        {
                            long delay = 0;
                            if (pos > prev_pos) {
                                // calculate the delay each frame contains 1000/Rate miliseconds' audio
                                delay =  (pos - prev_pos) * 1000 / PARAM_SAMPLING_RATE_IN_HZ;
                            }
                            // Increase the minimal delay so that the stop we queue next does not trigger
                            // before the current buffer is fully played.
                            if(delay < PLAYBACK_PCM_MIN_AUDIO_DURATION_IN_MS)
                            {
                            	delay = PLAYBACK_PCM_MIN_AUDIO_DURATION_IN_MS;
                            }
                            // it does help trying to stop the AudioTrack after the audio has been
                            // really played, even if the the delay is 0 milisecond, it will help
                            // to play back a very short audio buffer
                            _msgSys.scheduleTask(new TimerSystemTask() {
                                public void run() {
                                	// Note that the code from speex differs and includes an additional lock
                                	// it might be required to add it here
                                	synchronized (audioTrackLock) {
                                		mTrack.stop();
                                		mTrack.flush();
                                		mTrack.release();
                                		
                                        // Stop BT
                                        if (_bt != null) {
                                            _bt.stopBluetoothSco();
                                            // Cleanup BT
                                            cleanupBT();
                                        }

                                        doneCb.doneCallback(AudioStatus.AUDIO_OK, contextStart);
                                        Looper looper = Looper.myLooper();
                                        if (looper != null)
                                            looper.quit();
                                        askMoreData = false;
                                	}
                                }
                            }, delay);
                        }

//fixed bug 24346, should not stop player if no more data.
/*                        if (numSample == -1)
                        {
                            mTrack.stop();
                            mTrack.release();
                            doneCb.doneCallback(AudioStatus.AUDIO_OK, contextStart);
                            Looper.myLooper().quit();
                            askMoreData = false;
                        }*/

                    }
                }
            }
        }
            
        private void playOpus() throws AudioSystemException
        {
            // OPUS
            int offset = 0;

            if (askMoreData) {
                // Keep asking for data until a buffer is full
                IntegerMutable l = new IntegerMutable(PLAYBACK_PCM_PACKET_SIZE_IN_SHORTS);
                short[] tempBuffer = new short[PLAYBACK_PCM_PACKET_SIZE_IN_SHORTS];
                while (offset < PLAYBACK_PCM_PACKET_SIZE_IN_SHORTS)
                {
                    synchronized (playbackStoppingLock)
                    {
                        if (mStopping) {
                            askMoreData = false;
                            offset = 0;
                            break;
                        }
                    }

                    int requestSamples = PLAYBACK_PCM_PACKET_SIZE_IN_SHORTS - offset;
                    int process = 0;

                	process = getEncodedBuffer(tempBuffer, requestSamples, l, contextStart);

                                                //sleep a bit if there is no audio data available
                    if (process == 0) {
                        try {
                            sleep(300);
                        } catch (InterruptedException e) {}
                    }

                    if (l.i == -1)
                    {
                        //#ifdef NMSP_LOG
                        if(log.isDebugEnabled())
                        log.debug("No more data to playback.");
                        //#endif

                        askMoreData = false;
                        break;
                    }
                    else if ((l.i > 0) && process>0){
                        System.arraycopy(tempBuffer, 0, mShortBuffer, offset, process);
                        offset+=process;
                    }
                }
            }
            int numShortsToPlay = offset;
            if (numShortsToPlay > 0)

            // If buffer is less than a frame, fill it with silence
            if (numShortsToPlay < PLAYBACK_PCM_PACKET_SIZE_IN_SHORTS) {
                //#ifdef NMSP_LOG
                if(log.isDebugEnabled())
                log.debug("Filling buffer with silence.");
                //#endif

                for (int i = numShortsToPlay; i < PLAYBACK_PCM_PACKET_SIZE_IN_SHORTS; i++)
                    mShortBuffer[i] = 0;
            }

            if (numShortsToPlay > 0) {
                offset = 0;
                while (numShortsToPlay != 0) {
                    int written = mTrack.write(mShortBuffer, offset, PLAYBACK_PCM_PACKET_SIZE_IN_SHORTS);
                    if (written <= 0)
                        break;

                    numShortsToPlay -= written;
                    offset += written;

                    mShortsWritten += written;
                }

                int ret = mTrack.setNotificationMarkerPosition(mShortsWritten);
                if (ret != AudioTrack.SUCCESS){
                    throw new AudioSystemException("Call to AudioTrack.setNotificationMarkerPosition() SPEEX 0 failed with code: " + ret);
                }

                mHandler.sendMessage(mHandler.obtainMessage(PLAY));
            }
            else {
                // pad 0s if the complete streamed audio is shorter than 750ms
                if (mShortsWritten < PLAYBACK_PCM_MIN_AUDIO_SIZE_IN_SHORTS) {
                    int numShortsToPad = PLAYBACK_PCM_MIN_AUDIO_SIZE_IN_SHORTS - mShortsWritten;
                    short[] shortsToPad = new short[numShortsToPad];
                    for (int i = 0; i < numShortsToPad; i++)
                        shortsToPad[i] = 0x00;
                    int written = mTrack.write(shortsToPad, 0, numShortsToPad);
                    if (written > 0) {
                        mShortsWritten += written;
                    }

                    int ret = mTrack.setNotificationMarkerPosition(mShortsWritten);
                    if (ret != AudioTrack.SUCCESS){
                        throw new AudioSystemException("Call to AudioTrack.setNotificationMarkerPosition() SPEEX 1 failed with code: " + ret);
                    }

                    mHandler.sendMessage(mHandler.obtainMessage(PLAY));
                    return;
                }

                //#ifdef NMSP_LOG
                    if(log.isDebugEnabled())
                    log.debug("onPlay() AudioTrack status is :" + mTrack.getState());
                //#endif
                int pos = 0;
                int prev_pos = 0;
                // only get position when the audio track state is good
                if (mTrack.getState() != AudioTrack.STATE_UNINITIALIZED) {
                	pos = mTrack.getPlaybackHeadPosition();
                	prev_pos = pos;
                	if (pos == lastPlaybackPos){
                		// There is a bug in Android, when pos is near the end of buffer, it's not
                		// updated. So if marker position did not changed, we force it and consider the audio has been played.
                		pos = mShortsWritten;
                	}
                	lastPlaybackPos = pos;
                }
                

                if (pos >= mShortsWritten ||
                	mTrack.getState() == AudioTrack.STATE_UNINITIALIZED) {
                    long delay = 0;
                    if (pos > prev_pos) {
                        // calculate the delay each frame contains 1000/Rate miliseconds' audio
                        delay =  (pos - prev_pos) * 1000 / PARAM_SAMPLING_RATE_IN_HZ;
                    }
                    //#ifdef NMSP_LOG
        			if(log.isDebugEnabled()) {
                    	log.debug("calculating the delay......");
                    	log.debug("parameter pos = " + pos);
                    	log.debug("parameter prev_pos = " + prev_pos);
                    	log.debug("parameter delay = " + delay);
                	}
                	//#endif
                    // it does help trying to stop the AudioTrack after the audio has been
                    // really played, even if the the delay is 0 milisecond, it will help
                    // to play back a very short audio buffer
                    _msgSys.scheduleTask(new TimerSystemTask() {
                        public void run() {
                            //#ifdef NMSP_LOG
        					if(log.isDebugEnabled())
        						log.debug("++++++++++++========== stop mTrack: " + mTrack);
                        	//#endif
        					synchronized (playbackStoppingLock)
        					{
        						// It is important to check if we are already stopping, do not attempt
        						// to stop again
        						// There is a potential race condition that leads to a deadlock when
        						// this task fires after some delay and overlaps another TTS request.
        						// In that case audioTrackLock will be reused by multiple instances
        						// of AudioSystemOEM. 
        						if(!mStopping && !mAborted)
        						{
                                	synchronized (audioTrackLock) {
                                		if(mTrack.getState() != AudioTrack.STATE_UNINITIALIZED)
                                		{
                                			mTrack.stop();
                                		}
                                    	mTrack.flush();
                                    	mTrack.release();

                                        // Stop BT
                                        if (_bt != null) {
                                            _bt.stopBluetoothSco();
                                            // Cleanup BT
                                            cleanupBT();
                                        }

                                        doneCb.doneCallback(AudioStatus.AUDIO_OK, contextStart);
					
                                        // Cleanup speex
                                        synchronized (speexDecodeLock) {
                                            if(isSpeexDecodeInit)
                                            {
                                                decodeCleanupSpeex();
                                                decFrame = null;
                                                nextSpeexPackage = null;
                                                isSpeexDecodeInit = false;
                                            }
                                            else
                                            {
                                            //#ifdef NMSP_LOG
        					                if(log.isDebugEnabled())
        						                log.debug("Speex was already cleaned up");
                        	                //#endif
                                            }
                                        }
                                        mHandler.sendMessage(mHandler.obtainMessage(ABORT));
                                	}							
        							
        						}
                                else
                                {
                                //#ifdef NMSP_LOG
        					    if(log.isDebugEnabled())
        						    log.debug("AudioSystemOEM was already stopping");
                        	    //#endif

                                }
        					}

                        	

                        }
                    }, delay);

                }
            }
        }

        private void onStop() {
            //#ifdef NMSP_LOG
         if(log.isDebugEnabled())
            log.debug("Calling AudioSystemOEM.onStop()");
            //#endif


            synchronized (audioTrackLock) {
                try{
                    mTrack.stop();
                }
                catch(IllegalStateException ex){
                    //#ifdef NMSP_LOG
                if(log.isDebugEnabled())
                log.debug("Could not stop audioTrack.");
                    //#endif
                }
                
                askMoreData = false;
                mTrack.flush();
                mTrack.release();
                
                // Stop BT
                if (_bt != null) {
                    _bt.stopBluetoothSco();
                    // Cleanup BT
                    cleanupBT();
                }

                // Cleanup speex
                if(CODEC_TYPE == CodecType.SPEEX){
                    synchronized (speexDecodeLock) {
                        if(isSpeexDecodeInit)
                        {
                            decodeCleanupSpeex();
                            decFrame = null;
                            nextSpeexPackage = null;
                            isSpeexDecodeInit = false;
                        }
                        else
                        {
                        //#ifdef NMSP_LOG
                     if(log.isDebugEnabled())
                         log.debug("Attempting to cleanup speex decode while it is already stopped");
                        //#endif
                        }
                    }
                }
                if(CODEC_TYPE == CodecType.OPUS){
                	if(_opusDecoderHandle != 0){
                		releaseOpusDecoder(_opusDecoderHandle);
                		_opusDecoderHandle = 0;
                	}
                }
                stopCb.stopCallback(AudioStatus.AUDIO_OK, contextStop);
                Looper looper = Looper.myLooper();
                if (looper != null)
                    looper.quit();
            }
        }
    }

    /**
     * RECORD
     */
    private class Recorder implements
            AudioRecord.OnRecordPositionUpdateListener, MessageHandler {
        //#ifdef NMSP_LOG
        // Logger
        private Log log = LogFactory.getLog(this.getClass());
        //#endif

        private AudioRecord audioRecord;
        private boolean recordingActive;
        private AudioCallback audioCb = null;
        private EndOfSpeechCallback endOfSpeechCb = null;
        private StartOfSpeechCallback startOfSpeechCb = null;
        private EndPointerStartedCallback endPointerStartedCb = null;
        private EndPointerStoppedCallback endPointerStoppedCb = null;
        private StopCallback stopCb = null;
        private ErrorCallback errorCb = null;
        private boolean _useEndPointer = false;
        private boolean _currEndPointerDone  = false;

        private Object contextStart = null;

        //#ifdef NMSP_LOG
        private int sampleNumber = 0;
        //#endif

        private void onRecorderError(AudioSystemException ex){
            //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
            log.error(ex.getMessage());
            //#endif
            // It is safe to check recordingActive outside of the synchronized block
            // because this is the only thread it is modified from.
            if (!recordingActive)
                return;

            audioRecord.stop();

            synchronized (audioRecordLock) {
                recordingActive = false;
                audioRecord.release();
                audioRecord = null;
            }

            errorCb.errorCallback(contextStart);

            // Cleanup speex
            if(CODEC_TYPE == CodecType.SPEEX){
                synchronized (speexEncodeLock) {
                    encodeCleanupSpeex();
                }
            }
            if(CODEC_TYPE == CodecType.OPUS){
            	releaseOpusEncoder(_opusEncoderHandle);
            	_opusEncoderHandle = 0;
            }

            // Stop BT
            if (_bt != null) {
                _bt.stopBluetoothSco();
                // Cleanup BT
                cleanupBT();
            }

            if(stopCb != null)
                stopCb.stopCallback(AudioStatus.AUDIO_ERROR, contextStart);
        }

        public void startRecording(boolean useEndPointer, AudioCallback audioCallback, StopCallback stopCallback, ErrorCallback errorCallback, StartOfSpeechCallback startOfSpeechCallback, EndOfSpeechCallback endOfSpeechCallback, EndPointerStartedCallback endPointerStartedCallback, EndPointerStoppedCallback endPointerStoppedCallback, Object context) throws AudioSystemException{
            if (_bt != null) {
                _bt.startBluetoothSco();
            }

            synchronized (audioRecordLock) {
                //#ifdef NMSP_LOG
                sampleNumber = 0;
                //#endif

                this.contextStart = context;
                this.audioCb = audioCallback;
                this.endOfSpeechCb = endOfSpeechCallback;
                this.startOfSpeechCb = startOfSpeechCallback;
                endPointerStartedCb = endPointerStartedCallback;
                endPointerStoppedCb = endPointerStoppedCallback;
                stopCb = stopCallback;
                errorCb = errorCallback;

                if(useEndPointer)
                    turnOnEndPointer(endPointerStartedCb, contextStart);

                // Initialize speex in vocoder mode
                if(CODEC_TYPE == CodecType.SPEEX){
                    int ret;
                    synchronized (speexEncodeLock) {
                        ret = encodeInitSpeex(PARAM_SPEEX_MODE, PARAM_SAMPLING_RATE_IN_HZ, PARAM_SPEEX_ENC_COMPLEXITY, PARAM_SPEEX_ENC_QUALITY, PARAM_SPEEX_ENC_VAD);
                    }
                    if (ret < 0){
                        throw new AudioSystemException("encodeInitSpeex failed with code:" + speexCodeToString(ret));
                    }
                }
                if(CODEC_TYPE == CodecType.OPUS){
                	int ret;
                	int[] retError = new int[1];
                	//synchronized (opusEncodeLock) {
                	_opusEncoderHandle = initializeOpusEncoder(retError, PARAM_SAMPLING_RATE_IN_HZ,  PARAM_OPUS_BITRATE, PARAM_OPUS_ENC_QUALITY, PARAM_OPUS_ENABLE_VBR);
                    	
                	//}
                }

                try {
                    audioRecord = new AudioRecord(RECORDER_AUDIO_SOURCE, RECORDER_SAMPLE_RATE_IN_HZ,
                              CHANNEL_CONFIGURATION, PCM_ENCODING,
                              RECORDER_BUFFER_SIZE_IN_BYTES);

                    if ((audioRecord == null) || (audioRecord.getState() != AudioRecord.STATE_INITIALIZED)){
                        throw new AudioSystemException("AudioRecord object has not been initialized correctly. One or several parameters used to create it must be wrong.");
                    }
                }
                catch (IllegalArgumentException ex){
                    throw new AudioSystemException("Could not instanciate AudioRecord object.");
                }

                recordingActive = true;
                audioRecord.setRecordPositionUpdateListener(this);
                int ret = audioRecord.setPositionNotificationPeriod(RECORDER_PCM_PACKET_SIZE_IN_SHORTS);
                if( ret != AudioRecord.SUCCESS){
                    throw new AudioSystemException("Call to AudioRecord.setPositionNotificationPeriod() failed with code:" + ret);
                }

                try {
                    audioRecord.startRecording();
                }
                catch(IllegalStateException ex){
                    throw new AudioSystemException("Call to AudioRecord.startRecording() failed.");
                }

                int readLen = 0;
                short[] pcmBuffer = new short[RECORDER_PCM_PACKET_SIZE_IN_SHORTS];
                do {
                    ret = audioRecord.read(pcmBuffer, readLen, RECORDER_PCM_PACKET_SIZE_IN_SHORTS-readLen);
                    if ((ret == AudioRecord.ERROR_INVALID_OPERATION) || (ret == AudioRecord.ERROR_BAD_VALUE)){
                        throw new AudioSystemException("Call to AudioRecord.read() failed with code:" + ret);
                    } else if (ret <= 0) // TODO: What should we do here? We want to avoid an infinite loop
                    {
                        break;
                    } else
                    {
                        readLen += ret;
                    }
                } while (readLen < RECORDER_PCM_PACKET_SIZE_IN_SHORTS);
            }
        }

        public void turnOnEndPointer(EndPointerStartedCallback callback, Object context) {
            endPointerStartedCb = callback;
            _msgSys.send(new Object[] {CALLBACK_TYPE_END_POINTER_STARTED}, this, _msgSys.getMyAddr(), _msgSys.getVRAddr()[0]);
        }

        public void turnOffEndPointer(EndPointerStoppedCallback callback, Object context) {
            endPointerStoppedCb = callback;
            _msgSys.send(new Object[] {CALLBACK_TYPE_END_POINTER_STOPPED}, this, _msgSys.getMyAddr(), _msgSys.getVRAddr()[0]);
        }

        public void onMarkerReached(AudioRecord arg0) {
            // do nothing, ignore this notification
        }

        public void onPeriodicNotification(AudioRecord recorder) {
            if(		CODEC_TYPE == CodecType.SPEEX
            	 || CODEC_TYPE == CodecType.OPUS ){
                // SPEEX
                synchronized (audioRecordLock) {
                    if (recordingActive) {
                        short[] pcmBuffer = new short[RECORDER_PCM_PACKET_SIZE_IN_SHORTS];

                        int sampleCount = 0;
                        do {
                            int ret = recorder.read(pcmBuffer, sampleCount, RECORDER_PCM_PACKET_SIZE_IN_SHORTS-sampleCount);
                            if ((ret == AudioRecord.ERROR_INVALID_OPERATION) || (ret == AudioRecord.ERROR_BAD_VALUE) || (ret <= 0))
                                break;
                            sampleCount += ret;
                        } while (sampleCount < RECORDER_PCM_PACKET_SIZE_IN_SHORTS);
                        _msgSys.send(new Object[] {CALLBACK_TYPE_ON_PERIODIC_NOTIFICATION, new Integer(sampleCount), pcmBuffer}, this, _msgSys.getMyAddr(), _msgSys.getVRAddr()[0]);
                    }
                }
            }
            else {
                // PCM
                synchronized (audioRecordLock) {
                    if (recordingActive) {
                        byte[] buffer = new byte[RECORDER_PCM_PACKET_SIZE_IN_BYTES];

                        int bytesRead = 0;
                        do {
                            int ret = recorder.read(buffer, bytesRead, RECORDER_PCM_PACKET_SIZE_IN_BYTES-bytesRead);
                            if ((ret == AudioRecord.ERROR_INVALID_OPERATION) || (ret == AudioRecord.ERROR_BAD_VALUE) || (ret <= 0))
                                break;
                            bytesRead += ret;
                        } while (bytesRead < RECORDER_PCM_PACKET_SIZE_IN_BYTES);
                        _msgSys.send(new Object[] {CALLBACK_TYPE_ON_PERIODIC_NOTIFICATION, new Integer(bytesRead), buffer}, this, _msgSys.getMyAddr(), _msgSys.getVRAddr()[0]);
                    }
                }
            }
        }

        public void onPeriodicNotification_internal(int numOfUnitsRead, Object audioData) {
            if (recordingActive) {
                if(CODEC_TYPE == CodecType.SPEEX){
                    // SPEEX
                    short[] pcmBuffer/* = new short[RECORDER_PCM_PACKET_SIZE_IN_SHORTS]*/;
                    byte[] vocData = new byte[RECORDER_VOC_PACKET_SIZE_IN_BYTES];

                    int sampleCount = 0;
                    //synchronized (audioRecordLock) {
                        if (!recordingActive)
                            return;

                        pcmBuffer = (short[])audioData;
                        sampleCount = /*recorder.read(pcmBuffer, 0, RECORDER_PCM_PACKET_SIZE_IN_SHORTS)*/numOfUnitsRead;
                    //}
                    if (sampleCount < 0 ){
                        onRecorderError(new AudioSystemException("Call to AudioRecord.read() failed with code: " + sampleCount));
                        return;
                    }

                    // Do nothing if there is no audio
                    if (sampleCount == 0){
                        //#ifdef NMSP_LOG
                        if(log.isDebugEnabled())
                        log.debug("AudioRecorder has no audio.");
                        //#endif
                        return;
                    }

                    // Encode to speex (first 1 or 2 bytes is length of actual encoded data, it will be removed by the codec)
                    int vocDataLen;
                    EndPointerOEM.EpType detection =  EndPointerOEM.EpType.DETECT_NOTHING;
                    synchronized (speexEncodeLock) {
                        if (!recordingActive)
                            return;
                        vocDataLen = encodeSpeex(pcmBuffer, vocData, RECORDER_VOC_PACKET_SIZE_IN_BYTES);
                        if(_useEndPointer && !_currEndPointerDone)
                            detection = endPointer.detectEndPointing(null);
                    }
                    if (vocDataLen < 0){
                        onRecorderError(new AudioSystemException("Call to encodeSpeex() failed with code: " + speexCodeToString(vocDataLen)));
                        return;
                    }
                    //#ifdef NMSP_LOG
                        if(log.isDebugEnabled())
                            log.debug("RECORDING: Sample #" + (sampleNumber++) + ". Got " + sampleCount + " shorts from recorder, sending " +  vocDataLen + " SPEEX vocoded bytes. ");
                    //#endif

                    //#ifdef NMSP_LOG
                    if(log.isDebugEnabled()) {
                        if (detection == EndPointerOEM.EpType.SPEECH_END)                    
                            log.debug("ENDPOINTING SPEECH_END");
                        else if (detection == EndPointerOEM.EpType.SPEECH_START)                    
                            log.debug("ENDPOINTING SPEECH_START");
                    }                   
                    //#endif

                    float audioLevel = -1;

                    if(useEnergyLevel)
                        audioLevel = getAudioLevels(pcmBuffer);

                    audioCb.audioCallback(null, vocData, new IntegerMutable(0), new IntegerMutable(vocDataLen),  audioLevel < 0 ? null : audioLevel, contextStart);
                    if (detection == EndPointerOEM.EpType.SPEECH_END) {
                        _currEndPointerDone = true;
                        if(!_continuesOnEndPointerAndTimerStopping) {
                            stopRecording(/*null*/stopCb, this.endOfSpeechCb,contextStart);
                        } else {
                            if (endOfSpeechCb != null)
                                endOfSpeechCb.endOfSpeechCallback(contextStart);
                        }
                    }
                    if (detection == EndPointerOEM.EpType.SPEECH_START) {
                        startOfSpeechCb.startOfSpeechCallback(contextStart);
                    }
                }
                else if(CODEC_TYPE == CodecType.OPUS){
                    recordOpusBuffer(numOfUnitsRead, audioData);
                }
                else {
                    // PCM
                    byte[] buffer/* = new byte[RECORDER_PCM_PACKET_SIZE_IN_BYTES]*/;

                    int bytesRead = 0;
                    //synchronized (audioRecordLock) {
                        if (!recordingActive)
                            return;

                        buffer = (byte[])audioData;
                        bytesRead = /*recorder.read(buffer, 0, RECORDER_PCM_PACKET_SIZE_IN_BYTES)*/numOfUnitsRead;
                    //}

                    if (bytesRead < 0 ){
                        onRecorderError(new AudioSystemException("Call to AudioRecord.read() failed with code: " + bytesRead));
                        return;
                    }

                    // Do nothing if there is no audio
                    if (bytesRead == 0){
                        //#ifdef NMSP_LOG
                        if(log.isDebugEnabled())
                        log.debug("AudioRecorder has no audio.");
                        //#endif
                        return;
                    }

                    //#ifdef NMSP_LOG
                    if(log.isDebugEnabled())
                    log.debug("RECORDING: Sample #" + (sampleNumber++) + ". Sending " + bytesRead + " bytes from recorder.");
                    //#endif

                    IntegerMutable l = new IntegerMutable(bytesRead);
                    audioCb.audioCallback(buffer, null, l, new IntegerMutable(0), null, contextStart);
                }

            }
        }
        
        private void recordOpusBuffer(int numOfUnitsRead, Object audioData)
        {
        	// OPUS
            short[] pcmBuffer/* = new short[RECORDER_PCM_PACKET_SIZE_IN_SHORTS]*/;
            byte[] vocData = new byte[RECORDER_VOC_PACKET_SIZE_IN_BYTES];

            int sampleCount = 0;
            // TODO : MAKE SURE THIS IS PROPERLY LOCKED
            //synchronized (audioRecordLock) {
                if (!recordingActive)
                    return;

                pcmBuffer = (short[])audioData;
                sampleCount = /*recorder.read(pcmBuffer, 0, RECORDER_PCM_PACKET_SIZE_IN_SHORTS)*/numOfUnitsRead;
            //}
            if (sampleCount < 0 ){
                onRecorderError(new AudioSystemException("Call to AudioRecord.read() failed with code: " + sampleCount));
                return;
            }

            // Do nothing if there is no audio
            if (sampleCount == 0){
                //#ifdef NMSP_LOG
                if(log.isDebugEnabled())
                log.debug("AudioRecorder has no audio.");
                //#endif
                return;
            }

            // Encode to speex (first 1 or 2 bytes is length of actual encoded data, it will be removed by the codec)
            int errorCode = 0;
            // TODO : MAKE SURE THIS IS PROPERLY LOCKED
            EndPointer.EpType detection =  EndPointer.EpType.DETECT_NOTHING;
            //synchronized (speexEncodeLock) {
                if (!recordingActive)
                    return;
                int[] inputPcmBufferLen = new int[1];
                int[] outputOpusBufferLen = new int[1];
                outputOpusBufferLen[0] = vocData.length;
                inputPcmBufferLen[0] = pcmBuffer.length;
                errorCode = encodeOpus(_opusEncoderHandle, pcmBuffer, inputPcmBufferLen, vocData, outputOpusBufferLen);
                if(_useEndPointer && !_currEndPointerDone){
                    detection = endPointer.detectEndPointing(pcmBuffer);                
                }
                System.out.println("Encoded opus size : " + outputOpusBufferLen[0] + " detection : " + detection);

            //}
            if (errorCode != 0){
                onRecorderError(new AudioSystemException("Call to encodeOpus() failed encodeOpus() returned : " + errorCode));
                return;
            }
            //#ifdef NMSP_LOG
                if(log.isDebugEnabled())
                    log.debug("RECORDING: Sample #" + (sampleNumber++) + ". Got " + sampleCount + " shorts from recorder, sending " +  outputOpusBufferLen[0] + " SPEEX vocoded bytes. ");
            //#endif

            //#ifdef NMSP_LOG
            if(log.isDebugEnabled()) {
                if (detection == EndPointerOEM.EpType.SPEECH_END)                    
                    log.debug("ENDPOINTING SPEECH_END");
                else if (detection == EndPointerOEM.EpType.SPEECH_START)                    
                    log.debug("ENDPOINTING SPEECH_START");
            }                   
            //#endif

            float audioLevel = -1;

            if(useEnergyLevel)
                audioLevel = getAudioLevels(pcmBuffer);

            audioCb.audioCallback(null, vocData, new IntegerMutable(0), new IntegerMutable(outputOpusBufferLen[0]),  audioLevel < 0 ? null : audioLevel, contextStart);
            if (detection == EndPointerOEM.EpType.SPEECH_END) {
                _currEndPointerDone = true;
                if(!_continuesOnEndPointerAndTimerStopping) {
                    stopRecording(/*null*/stopCb, this.endOfSpeechCb,contextStart);
                } else {
                    if (endOfSpeechCb != null)
                        endOfSpeechCb.endOfSpeechCallback(contextStart);
                }
            }
            if (detection == EndPointerOEM.EpType.SPEECH_START) {
                startOfSpeechCb.startOfSpeechCallback(contextStart);
            }
        	
        }

        private float getAudioLevels(short[] pcmBuffer)
        {
            float audioLevel = -1;

            float average = computerAverageLevel(pcmBuffer);

            audioLevel = (long)(90 + average);

            return audioLevel;
        }

        private float computerAverageLevel(short[] pcmData)
        {
            float average;//, peak = 0;
            long acc = 0, val;
            int i;
            double pwr, dbpwr;

            for (i = 0; i < pcmData.length; i++)
            {
                val = pcmData[i];
                acc += (val * val) >> 9;
            }

            pwr = acc / (32768.0 * 32768.0);
            if (pwr < 1e-9)
                dbpwr = -90;
            else
            {
                dbpwr = 10 * Math.log10(pwr);
                if (dbpwr > 0)
                    dbpwr = 0;
            }

            average = (float)dbpwr;
            //peak = (float)dbpwr;

            return average;
        }

        public void stopRecording(StopCallback stopCallback,EndOfSpeechCallback endOfSpeechCallback, Object context) {
            if (recordingActive == true) {
                boolean performCleanup = true;
                try{
                    Thread.sleep(_recordPacketDurationInMS);
                    // It is safe to check recordingActive outside of the synchronized block
                    // because this is the only thread it is modified from.
                    if (recordingActive != true) {
                        performCleanup = false;
                    } else {
                        audioRecord.stop();
                    }

                    synchronized (audioRecordLock) {
                        recordingActive = false;
                        if (performCleanup) {
                            audioRecord.release();
                            audioRecord = null;
                        }
                    }

                    if (performCleanup) {
                        // Cleanup speex
                        if(CODEC_TYPE == CodecType.SPEEX){
                            synchronized (speexEncodeLock) {
                                encodeCleanupSpeex();
                            }
                        }else if (CODEC_TYPE == CodecType.OPUS){
                        	if(_opusEncoderHandle != 0){
                        		releaseOpusEncoder(_opusEncoderHandle);
                        		_opusEncoderHandle = 0;
                        	}
                        }
                        
                    }
                }
                catch (IllegalStateException ex){
                    //#ifdef NMSP_LOG
                    if(log.isDebugEnabled())
                    log.debug("Could not stop audioRecord.");
                    //#endif
                }
                catch (Exception e){}

            }

            // Stop BT
            if (_bt != null) {
                _bt.stopBluetoothSco();
                // Cleanup BT
                cleanupBT();
            }

            if (endOfSpeechCallback != null)
                endOfSpeechCallback.endOfSpeechCallback(context);
            if (stopCallback != null)
                stopCallback.stopCallback(AudioStatus.AUDIO_OK, context);
        }

        public void handleMessage(Object msgData, Object msgSender) {
            Object[] params = (Object[])msgData;
            if(((Integer) params[0]).intValue() == CALLBACK_TYPE_END_POINTER_STARTED.intValue()) {
                if(!_useEndPointer) {
                    _useEndPointer = true;
                    endPointer.resetEndpointingDetection();
                    _currEndPointerDone = false;
                    if(endPointerStartedCb != null) endPointerStartedCb.endPointerStartedCallback(null);
                }
            }
            else if(((Integer) params[0]).intValue() == CALLBACK_TYPE_END_POINTER_STOPPED.intValue()) {
                if(_useEndPointer) {
                    _useEndPointer = false;
                    if(endPointerStoppedCb != null) endPointerStoppedCb.endPointerStoppedCallback(null);
                }
            }
            else if(((Integer) params[0]).intValue() == CALLBACK_TYPE_ON_PERIODIC_NOTIFICATION.intValue()) {
                onPeriodicNotification_internal(((Integer)params[1]).intValue(), params[2]);
            }
            return;
        }
    }

    private void cleanupBT() {
        if (_bt != null) {
            _bt.close();
            _bt = null;
        }
    }

    public void handleMessage(Object msgData, Object msgSender) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
        log.debug("---------------------- AudioSystemAndroid +++++ handleMessage() Thread:" + Thread.currentThread());
        //#endif

        return;
    }

    public boolean startRecording(InputDevice inputDevice,
            boolean useEndPointer, AudioCallback audioCallback, StopCallback stopCallback, ErrorCallback errorCallback, StartOfSpeechCallback startOfSpeechCallback, NoSpeechCallback noSpeechCallback,
            EndOfSpeechCallback endOfSpeechCallback, EndPointerStartedCallback endPointerStartedCallback, EndPointerStoppedCallback endPointerStoppedCallback, Object context) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
        log.debug("++++++++++++========== startRecording()");
        //#endif

        if (mRecorder != null) {
            //#ifdef NMSP_LOG
            if(log.isDebugEnabled())
            log.debug("previous recording still running!");
            //#endif
            return false;
        }
        if (audioCallback == null) {
            //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
            log.error("audioCallback cannot be null.");
            //#endif
            return false;
        }

        if (_context == null) {
            //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("inputDevice is BLUETOOTH_HEADSET, but ANDROID_CONTEXT parameter is not passed in!!!");
            //#endif
            return false;
        }
        
        boolean isBluetoothEnabled = true;
        if (_context != null) {
        	isBluetoothEnabled = (_context.checkCallingOrSelfPermission("android.permission.BLUETOOTH") == PackageManager.PERMISSION_GRANTED);
        }
        
        if (isBluetoothEnabled) {
	        _bt = Bluetooth.createInstance(_context);
	        if (_bt.isHeadsetConnected()) {
	            adjustCodecForBluetooth();
	        }
        }
        else {
        	_bt = null;
        }

        if (inputDevice == InputDevice.MICROPHONE) {
            // use VOICE_RECOGNITION to avoid AGC which has bad impacts to the recongition engine
            RECORDER_AUDIO_SOURCE = AudioSource.VOICE_RECOGNITION;
            cleanupBT();
        } else if (inputDevice == InputDevice.BLUETOOTH_HEADSET && _bt != null) {
            RECORDER_AUDIO_SOURCE = _bt.getRecordingSource();
            if (!_bt.isHeadsetConnected()) {
                cleanupBT();
            }
        } else {
            //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("Unexpected inputDevice.");
            //#endif
            cleanupBT();
            return false;
        }

        // Start recorder
        mRecorder = new Recorder();

        try {
            mRecorder.startRecording(useEndPointer, audioCallback, stopCallback, errorCallback, startOfSpeechCallback, endOfSpeechCallback, endPointerStartedCallback, endPointerStoppedCallback, context);
        } catch (AudioSystemException ex) {
            mRecorder.onRecorderError(ex);
            mRecorder = null;
            return false;
        }

        return true;
    }

    public void turnOnEndPointer(EndPointerStartedCallback callback, Object context) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
        log.debug("++++++++++++========== turnOnEndPointer()");
        //#endif
        if (callback == null) {
            // error
            return;
        }

        if (mRecorder == null) {
            // error
            return;
        } else {
            mRecorder.turnOnEndPointer(callback, context);
        }
    }

    public void turnOffEndPointer(EndPointerStoppedCallback callback, Object context) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
        log.debug("++++++++++++========== turnOnEndPointer()");
        //#endif
        if (callback == null) {
            // error
            return;
        }

        if (mRecorder == null) {
            // error
            return;
        } else {
            mRecorder.turnOffEndPointer(callback, context);
        }
    }

    public void pauseRecording(int pause) {
        // Not implemented
        //#ifdef NMSP_LOG
        log.debug("++++++++++++========== pauseRecording() is not implemented");
        //#endif
    }

    public void stopRecording(StopCallback stopCallback, Object context) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
        log.debug("++++++++++++========== stopRecording()");
        //#endif
        if (stopCallback == null) {
            // error
            return;
        }

        if (mRecorder == null) {
            stopCallback.stopCallback(AudioStatus.AUDIO_OK, context);
        } else {
            mRecorder.stopRecording(stopCallback, null, context);
        }
    }

    public boolean startPlayback(OutputDevice outputDevice,
            AudioCallback audioCallback, DoneCallback doneCallback,
            Object context) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
        log.debug("++++++++++++========== startPlayback()");
        //#endif

        if (audioCallback == null) {
            //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
            log.error("audioCallback cannot be null.");
            //#endif
            return false;
        }

        if (_context == null) {
            //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("inputDevice is BLUETOOTH_HEADSET, but ANDROID_CONTEXT parameter is not passed in!!!");
            //#endif
            return false;
        }
        
        boolean isBluetoothEnabled = true;
        if (_context != null) {
        	isBluetoothEnabled = (_context.checkCallingOrSelfPermission("android.permission.BLUETOOTH") == PackageManager.PERMISSION_GRANTED);
        }
        
        if (isBluetoothEnabled) {
	        _bt = Bluetooth.createInstance(_context);
	        if (_bt.isHeadsetConnected()) {
	            adjustCodecForBluetooth();
	        }
        }
        else {
        	_bt = null;
        }

        if (outputDevice == OutputDevice.SPEAKER) {
            PLAYBACK_AUDIO_STREAM = AudioManager.STREAM_MUSIC;
            cleanupBT();
        } else if (outputDevice == OutputDevice.BLUETOOTH_HEADSET && _bt != null) {
            PLAYBACK_AUDIO_STREAM = _bt.getPlaybackStream();
            if (!_bt.isHeadsetConnected()) {
                cleanupBT();
            }
        } else {
            //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("Unexpected outputDevice.");
            //#endif
            cleanupBT();
            return false;
        }

        mPlaybackThread = new PlaybackThread();

        try {
            mPlaybackThread.startPlayback(audioCallback, doneCallback, context);
        } catch (AudioSystemException ex) {
            mPlaybackThread.onPlaybackError(ex);
            mPlaybackThread = null;
            return false;
        }
        return true;
    }

    public boolean pausePlayback(int pause) {
        // Not implemented
        //#ifdef NMSP_LOG
        log.info("++++++++++++========== pausePlayback() is not implemented");
        //#endif

        return false;
    }

    public boolean previousPlayback() {
        // Not implemented
        //#ifdef NMSP_LOG
        log.info("++++++++++++========== previousPlayback() is not implemented");
        //#endif

        return false;
    }

    public boolean nextPlayback() {
        // Not implemented
        //#ifdef NMSP_LOG
        log.info("++++++++++++========== nextPlayback() is not implemented");
        //#endif

        return false;
    }

    public void stopPlayback(StopCallback stopCallback, Object context) {
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
        log.debug("++++++++++++========== stopPlayback()");
        //#endif

        if (mPlaybackThread != null)
            mPlaybackThread.stopPlayback(stopCallback, context);
        mPlaybackThread = null;
    }

    public InputDevice getInputDevice() {
        return InputDevice.MICROPHONE;
    }

    public OutputDevice getOutputDevice() {
        return OutputDevice.SPEAKER;
    }

    public Vector getParams(Vector parameters) {
        for (int i = 0; i < parameters.size(); i++) {
            Parameter parameter = (Parameter) parameters.elementAt(i);
            if (parameter.getType() == Parameter.Type.SDK)
            {
                if(parameter.getName().equals(NMSPDefines.NMSP_DEFINES_AUDIO_INPUTSOURCE)) {
                    switch (RECORDER_AUDIO_SOURCE) {
                    case AudioSource.MIC:
                        parameters.set(i, new Parameter(NMSPDefines.NMSP_DEFINES_AUDIO_INPUTSOURCE, "Microphone audio source".getBytes(),Parameter.Type.SDK ));
                        break;
                    case AudioSource.DEFAULT:
                        parameters.set(i, new Parameter(NMSPDefines.NMSP_DEFINES_AUDIO_INPUTSOURCE, "Default audio source".getBytes(),Parameter.Type.SDK ));
                        break;
                    case AudioSource.VOICE_RECOGNITION:
                        parameters.set(i, new Parameter(NMSPDefines.NMSP_DEFINES_AUDIO_INPUTSOURCE, "VoiceRecognition audio source".getBytes(),Parameter.Type.SDK ));
                        break;
                    default:
                        parameters.set(i, new Parameter(NMSPDefines.NMSP_DEFINES_AUDIO_INPUTSOURCE, "Unkown audio source".getBytes(),Parameter.Type.SDK ));
                    break;
                    }
                }
                if(parameter.getName().equals(NMSPDefines.NMSP_DEFINES_ENDPOINTER)) {
                    parameters.setElementAt(new Parameter(NMSPDefines.NMSP_DEFINES_ENDPOINTER,
                                                         (new Boolean(endPointer.isEndPointingActive())).toString().getBytes(),
                                                         Parameter.Type.SDK ), i);
                }
                else if(parameter.getName().equals(NMSPDefines.NMSP_DEFINES_RECORDER_CONTINUES_ON_ENDPOINTER_AND_TIMER_STOPPING)) {
                    parameters.setElementAt(new Parameter(NMSPDefines.NMSP_DEFINES_RECORDER_CONTINUES_ON_ENDPOINTER_AND_TIMER_STOPPING,
                                                    	 (new Boolean(_continuesOnEndPointerAndTimerStopping)).toString().getBytes(),
                                                    	 Parameter.Type.SDK ), i);
                }
                else if(parameter.getName().equals(NMSPDefines.NMSP_DEFINES_CAPTURING_CONTINUES_ON_ENDPOINTER)) {
                    parameters.setElementAt(new Parameter(NMSPDefines.NMSP_DEFINES_CAPTURING_CONTINUES_ON_ENDPOINTER,
                                                    	 (new Boolean(_capturingContinuesOnEndPointer)).toString().getBytes(),
                                                    	 Parameter.Type.SDK ), i);
                }
                else if(parameter.getName().equals(NMSPDefines.NMSP_DEFINES_USE_ENERGY_LEVEL)) {
                    parameters.set(i, new Parameter(NMSPDefines.NMSP_DEFINES_USE_ENERGY_LEVEL,
                                                    (new Boolean(useEnergyLevel)).toString().getBytes(),
                                                    Parameter.Type.SDK ));
                }
                else {
                    parameters.set(i, new Parameter(parameter.getName(),
                            "unsupported parameter".getBytes(),
                            parameter.getType()));
                }
            }
        }
        return parameters;
    }

    private void adjustCodecForBluetooth()
    {
        if (_codec == NMSPDefines.Codec.SPEEX_16K ||
                _codec == NMSPDefines.Codec.SPEEX_11K) {
            _codec = NMSPDefines.Codec.SPEEX_8K;
            init(_codec);
        }
        else if (_codec == NMSPDefines.Codec.PCM_16_11K ||
                _codec == NMSPDefines.Codec.PCM_16_16K ||
                _codec == NMSPDefines.Codec.PCM_16_22K ||
                _codec == NMSPDefines.Codec.PCM_16_32K) {
            _codec = NMSPDefines.Codec.PCM_16_8K;
            init(_codec);
        }
    }
}
