/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client2.sdk.oem;

import java.util.Vector;
import com.nuance.nmsp.client2.sdk.components.general.*;

//#ifdef NMSP_LOG
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory.Log;
//#endif

public class EndPointerOEM implements EndPointer{

	//#ifdef NMSP_LOG
	private Log log = LogFactory.getLog(this.getClass());
	//#endif
    


	private boolean isEndPointingActive = (VST_OEM_AUDIO_VAD_STOP_ON_END_OF_SPEECH_DEFAULT == 1);
	
	private class VadParams {
		int stopOnEndOfSpeech;
		int vadLongUtterance;
		int vadHistoryLength;
		int vadBeginLength;
		int vadBeginThreshold;
		int vadBeginDelay;
		int vadEndLength;
		int vadEndThreshold;
		int vadInterSpeechLength; 	
	}



	private VadParams vadparams = new VadParams();


	public void initialize(Vector audioParams) {
	
		vadparams.stopOnEndOfSpeech = VST_OEM_AUDIO_VAD_STOP_ON_END_OF_SPEECH_DEFAULT;
		vadparams.vadLongUtterance = VST_OEM_AUDIO_VAD_LONG_UTTERANCE_DEFAULT;
		vadparams.vadHistoryLength = VST_OEM_AUDIO_VAD_HISTORY_LEN_DEFAULT;
		vadparams.vadBeginLength = VST_OEM_AUDIO_VAD_BEGIN_LEN_DEFAULT;
		vadparams.vadBeginThreshold = VST_OEM_AUDIO_VAD_BEGIN_THRESHOLD_DEFAULT;
		vadparams.vadEndLength = VST_OEM_AUDIO_VAD_END_LEN_DEFAULT;
		vadparams.vadEndThreshold = VST_OEM_AUDIO_VAD_END_THRESHOLD_DEFAULT;
		vadparams.vadInterSpeechLength = VST_OEM_AUDIO_VAD_INTERSPEECH_LEN_DEFAULT; 
		vadparams.vadBeginDelay = VST_OEM_AUDIO_VAD_BEGIN_DELAY_DEFAULT;
	
	
		if (audioParams != null) {

			for (int i = 0; i < audioParams.size() ; i++) {
				Parameter curParam = (Parameter) audioParams.get(i);
				String curParamName = curParam.getName();
				
				if (curParamName.equals(VST_OEM_AUDIO_VAD_STOP_ON_END_OF_SPEECH)) {
	                String curParamValue = new String(curParam.getValue());
					if (curParamValue.equalsIgnoreCase("TRUE")) {
						//#ifdef NMSP_LOG
						if(log.isDebugEnabled())
							log.debug("Stop on end of speech is activated.");
						//#endif
						vadparams.stopOnEndOfSpeech = 1;
					}
				}
				else if (curParamName.equals(VST_OEM_AUDIO_VAD_LONG_UTTERANCE)) {
	                String curParamValue = new String(curParam.getValue());
					if (curParamValue.equalsIgnoreCase("TRUE")) {
						vadparams.vadLongUtterance = 1;
					}				
				}
				else if (curParamName.equals(VST_OEM_AUDIO_VAD_HISTORY_LEN)) {
	                String curParamValue = new String(curParam.getValue());
					vadparams.vadHistoryLength = Integer.parseInt(curParamValue);
				}				

				else if (curParamName.equals(VST_OEM_AUDIO_VAD_BEGIN_LEN)) {
	                String curParamValue = new String(curParam.getValue());
					vadparams.vadBeginLength = Integer.parseInt(curParamValue);
				}	
				else if (curParamName.equals(VST_OEM_AUDIO_VAD_BEGIN_THRESHOLD)) {
	                String curParamValue = new String(curParam.getValue());
					vadparams.vadBeginThreshold = Integer.parseInt(curParamValue);
				}	
				else if (curParamName.equals(VST_OEM_AUDIO_VAD_END_LEN)) {
	                String curParamValue = new String(curParam.getValue());
					vadparams.vadEndLength = Integer.parseInt(curParamValue);
				}	
				else if (curParamName.equals(VST_OEM_AUDIO_VAD_END_THRESHOLD)) {
	                String curParamValue = new String(curParam.getValue());
					vadparams.vadEndThreshold = Integer.parseInt(curParamValue);
				}	
				else if (curParamName.equals(VST_OEM_AUDIO_VAD_INTERSPEECH_LEN)) {
	                String curParamValue = new String(curParam.getValue());
					vadparams.vadInterSpeechLength = Integer.parseInt(curParamValue);
				}
				else if (curParamName.equals(VST_OEM_AUDIO_VAD_BEGIN_DELAY)) {
	                String curParamValue = new String(curParam.getValue());
					vadparams.vadBeginDelay = Integer.parseInt(curParamValue);
				}
			}
		}		
		isEndPointingActive = (vadparams.stopOnEndOfSpeech == 1);
		if (isEndPointingActive) {
			initializeEndPointer(vadparams.stopOnEndOfSpeech, vadparams.vadLongUtterance, vadparams.vadHistoryLength, 
					vadparams.vadBeginLength, vadparams.vadBeginThreshold, vadparams.vadBeginDelay, vadparams.vadEndLength, 
					vadparams.vadEndThreshold, vadparams.vadInterSpeechLength);
			resetEndpointingDetection();
		}
	}
	
	public boolean isEndPointingActive() {
		return isEndPointingActive;		
	}

	/*------------------------------------------------------------------------
	 *
	 * VAD detection (e.g. nothing, end or start of speech)
	 *
 	------------------------------------------------------------------------*/
	public EpType detectEndPointing(short[] data) {		
		if (isEndPointingActive) {
			EpType detection= EpType.getEpType(getCurrentEndPointerState());	
			return detection;	
		}
		return EpType.DETECT_NOTHING;
	}
	
	public void resetEndpointingDetection() {
		if (isEndPointingActive) {
			resetVad();
		}
	}

    private static native void initializeEndPointer(int stopOnEndOfSpeech, int vadLongUtterance, int vadHistoryLength, int vadBeginLength,
    		int vadBeginThreshold, int vadBeginDelay, int vadEndLength, int vadEndThreshold, int vadInterSpeechLength);	
    private static native int getCurrentEndPointerState();	
    private static native int resetVad();	
}
