/**
 *  Copyright 2006-2011, Nuance Communications Inc. All rights reserved.
 *
 *  Nuance Communications, Inc. provides this document without representation
 *  or warranty of any kind. The information in this document is subject to
 *  change without notice and does not represent a commitment by Nuance
 *  Communications, Inc. The software and/or databases described in this
 *  document are furnished under a license agreement and may be used or
 *  copied only in accordance with the terms of such license agreement.
 *  Without limiting the rights under copyright reserved herein, and except
 *  as permitted by such license agreement, no part of this document may be
 *  reproduced or transmitted in any form or by any means, including, without
 *  limitation, electronic, mechanical, photocopying, recording, or otherwise,
 *  or transferred to information storage and retrieval systems, without the
 *  prior written permission of Nuance Communications, Inc.
 *
 *  Nuance, the Nuance logo, Nuance Recognizer, and Nuance Vocalizer are
 *  trademarks or registered trademarks of Nuance Communications, Inc. or its
 *  affiliates in the United States and/or other countries. All other
 *  trademarks referenced herein are the property of their respective owners.
 **/

package com.nuance.nmsp.client2.sdk.oem;

import java.util.ArrayList;

import android.os.Looper;
import java.util.Hashtable;
//#ifdef NMSP_LOG
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory.Log;
//#endif
import com.nuance.nmsp.client2.sdk.common.oem.api.MessageSystem;
import com.nuance.nmsp.client2.sdk.common.oem.api.TimerSystem.TimerSystemTask;

public class MessageSystemOEM implements MessageSystem {
	//#ifdef NMSP_LOG
    private final static Log log = LogFactory.getLog(MessageSystemOEM.class);
    //#endif
    
    public static class Message {
        public Message(Object data, MessageHandler handler) {
            msgData = data;
            msgHandler = handler;
        }
        public Thread  msgSender;      /* Address of sender */
        public Thread  msgReceiver;    /* Address of receiver */
        public MessageHandler  msgHandler;     /* Msg Handler function - must not be NULL */
        public Object          msgData;        /* Msg specific data - may be NULL */
    }
    
    public class TimerRunnable implements Runnable
    {
        private TimerSystemTask _task;
        public TimerRunnable(TimerSystemTask task)
        {
            _task = task;
            _pendingTimerTasks.put(task, this);
        }
        public void run()
        {
            TimerRunnable r = _pendingTimerTasks.remove(_task);
            //#ifdef NMSP_LOG
            if(log.isDebugEnabled())
                log.debug("TIMER run() _pendingTimerTasks.size():" + _pendingTimerTasks.size() + ", this:" + this + ", r:" + r);
            //#endif
            if (r != null)
                r._task.run();
        }
    }
    
    private final AttachableHandler _handler;
    private final Thread _thread;
    private final Hashtable<TimerSystemTask, TimerRunnable> _pendingTimerTasks;
    
    public MessageSystemOEM() {
        _pendingTimerTasks = new Hashtable<TimerSystemTask, TimerRunnable>();
        _handler = new AttachableHandler();
        _thread = new Thread(new Runnable()
        {
            public void run()
            {
                Looper.prepare();
                _handler.attachToCurrentThread();
                Looper.loop();
            }
        });
        _thread.start();
    }

    public int getNumOfVRThreads() {
    	return 1;
    }
    
    public Message createMessage(Object msgData, MessageHandler msgHandler) {
    	return new Message(msgData, msgHandler);
    }

    // Get the VR thread address. This is only thread we have for now.
    public Object[] getVRAddr() {
        return new Object[] {Thread.currentThread()};
    }
    // not used for now.
    public Object getMyAddr() {
    	return Thread.currentThread();
    }
    
    public void scheduleTask(final TimerSystemTask task, long delay)
    {
        TimerRunnable r = new TimerRunnable(task);
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("TIMER _handler.postDelayed(" + r + ")");
        //#endif
        _handler.postDelayed(r, delay);
    }
    
    public boolean cancelTask(TimerSystemTask task) {
        TimerRunnable r = _pendingTimerTasks.remove(task);
        //#ifdef NMSP_LOG
        if(log.isDebugEnabled())
            log.debug("TIMER cancelTask() _pendingTimerTasks.size():" + _pendingTimerTasks.size());
        //#endif
        if (r != null)
        {
            //#ifdef NMSP_LOG
            if(log.isDebugEnabled())
                log.debug("TIMER _handler.removeCallbacks(" + r + ")");
            //#endif
            _handler.removeCallbacksOwn(r);
        }
        return r != null;
    }
    
    public void send(Object msgData, MessageHandler msgHandler, Object sender, Object receiver) {
    	final Message msg = createMessage(msgData, msgHandler);
        msg.msgReceiver = (Thread)receiver;
        msg.msgSender = (Thread)sender;

        _handler.post(new Runnable()
        {
            public void run()
            {
                //#ifdef NMSP_LOG
                if(log.isTraceEnabled())
                    log.trace("Executing Message");
                //#endif
                msg.msgHandler.handleMessage(msg.msgData, msg.msgSender);
                //#ifdef NMSP_LOG
                if(log.isTraceEnabled())
                    log.trace("Done Executing Message");
                //#endif
            }
        });
    }
    
    public void stop() {
        _handler.post(new Runnable()
        {
            public void run()
            {
                Looper.myLooper().quit();
            }
        });
    }
    
    public void setSessionId(byte[] sessionId) {
    }
	
	public static void terminate() 
	{
	}
}
