package com.nuance.nmsp.client2.sdk.oem.bluetooth;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.List;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.os.Looper;

import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory.Log;

// BluetoothHeadsetOEM
public class BluetoothHeadsetOEM
{
    //#ifdef NMSP_LOG
    private final static Log log = LogFactory.getLog(Bluetooth.class);
    //#endif

    private Context mContext;
    private boolean _reflected;
    private com.nuance.nmsp.client2.sdk.oem.bluetooth.BluetoothHeadset _reflectedHeadset = null;
    private boolean _serviceConnectionGaveUp = false;
    private boolean _serviceCbCalled = false;
    private Object _nativeHeadset = null;

    public static String ACTION_AUDIO_STATE_CHANGED;
    public static String EXTRA_AUDIO_STATE;
    public static int AUDIO_STATE_CONNECTED;

    private final Reflection r;

    private Object mHeadsetSyncObj = new Object();

    public BluetoothHeadsetOEM(Context context)
    {
        mContext = context;
        r = new Reflection();
        int sdk = AndroidVersion.SDK;
        if (sdk <= AndroidVersion.GINGERBREAD_MR1) {
            _reflected = true;

            ACTION_AUDIO_STATE_CHANGED = com.nuance.nmsp.client2.sdk.oem.bluetooth.BluetoothHeadset.ACTION_AUDIO_STATE_CHANGED;
            EXTRA_AUDIO_STATE = com.nuance.nmsp.client2.sdk.oem.bluetooth.BluetoothHeadset.EXTRA_AUDIO_STATE;
            AUDIO_STATE_CONNECTED = com.nuance.nmsp.client2.sdk.oem.bluetooth.BluetoothHeadset.AUDIO_STATE_CONNECTED;

            com.nuance.nmsp.client2.sdk.oem.bluetooth.BluetoothHeadset.ServiceListener serviceListener = new com.nuance.nmsp.client2.sdk.oem.bluetooth.BluetoothHeadset.ServiceListener() {
                public void onServiceConnected() {
                    //#ifdef NMSP_LOG
                    if(log.isInfoEnabled())
                        log.info("BluetoothHeadsetOEM reflected onServiceConnected()");
                    //#endif
                    synchronized(mHeadsetSyncObj) {
                        if (!_serviceConnectionGaveUp) {
                            _serviceCbCalled = true;
                            mHeadsetSyncObj.notify();
                        } else {
                            closeReflectedHeadset();
                        }
                    }
                }
                public void onServiceDisconnected() {
                    //#ifdef NMSP_LOG
                    if(log.isInfoEnabled())
                        log.info("BluetoothHeadsetOEM reflected onServiceDisconnected()");
                    //#endif
                    synchronized(mHeadsetSyncObj) {
                        closeReflectedHeadset();
                        if (!_serviceConnectionGaveUp) {
                            _serviceCbCalled = true;
                            mHeadsetSyncObj.notify();
                        }
                    }
                }
            };
            synchronized(mHeadsetSyncObj) {
                long startTime = System.currentTimeMillis();
                _reflectedHeadset = new com.nuance.nmsp.client2.sdk.oem.bluetooth.BluetoothHeadset(mContext, serviceListener);
                try
                {
                    mHeadsetSyncObj.wait(1000);
                } catch (InterruptedException e)
                {
                    //#ifdef NMSP_LOG
                    if(log.isErrorEnabled())
                        log.error("BluetoothHeadsetOEM reflected mHeadsetSyncObj.wait() threw exception:" + e);
                    //#endif
                }
                if (!_serviceCbCalled) {
                    //#ifdef NMSP_LOG
                    if(log.isErrorEnabled())
                        log.error("BluetoothHeadsetOEM reflected service NOT connected in time. Gave up!!!");
                    //#endif
                    _serviceConnectionGaveUp = true;
                    closeReflectedHeadset();
                }
                else {
                    //#ifdef NMSP_LOG
                    if(log.isInfoEnabled())
                        log.info("BluetoothHeadsetOEM reflected service connection took " + (System.currentTimeMillis() - startTime) + "ms");
                    //#endif
                }
            }
        } else {
            _reflected = false;

            // skip devices which don't support BT profile HFP
            if (android.os.Build.MANUFACTURER.equals("Amazon") &&
                android.os.Build.BRAND.equals("Amazon")/* &&
                android.os.Build.DEVICE.equals("thor")*/)
            {
                //#ifdef NMSP_LOG
                if(log.isInfoEnabled())
                    log.info("skipped HFP probing for this device");
                //#endif
                return;
            }
            
            Class<?> BtHsetClass = r.getClassForName("android.bluetooth.BluetoothHeadset");
            ACTION_AUDIO_STATE_CHANGED = (String) r.getFieldValue(BtHsetClass, "ACTION_AUDIO_STATE_CHANGED");
            EXTRA_AUDIO_STATE = (String) r.getFieldValue(BtHsetClass, "EXTRA_STATE");
            AUDIO_STATE_CONNECTED = (Integer) r.getFieldValue(BtHsetClass, "STATE_AUDIO_CONNECTED");

            BluetoothAdapter adapter;
            // On Android 3.x (specific 3.1), this method "BluetoothAdapter.getDefaultAdapter()"
            // tends to create a Handler so we'd have to create a Looper thread to do this task
            if (AndroidVersion.SDK == AndroidVersion.HONEYCOMB
                    || AndroidVersion.SDK == AndroidVersion.HONEYCOMB_MR1
                    || AndroidVersion.SDK == AndroidVersion.HONEYCOMB_MR2
                    )
            {
                final Object[] objects = new Object[2];
                new Thread(new Runnable()
                {
                    ///Override
                    public void run()
                    {
                        Looper.prepare();
                        objects[1] = BluetoothAdapter.getDefaultAdapter();
                        objects[0] = new Boolean(true);
                        Looper.loop();
                    }
                }).start();
                while (objects[0] == null)
                {
                    try {
                        Thread.sleep(10);
                    } catch (Exception e) {}
                }
                adapter = (BluetoothAdapter) objects[1];
            } else
            {
                adapter = BluetoothAdapter.getDefaultAdapter();
            }

            Class<?> sServiceListenerClass = r.getClassForName("android.bluetooth.BluetoothProfile$ServiceListener");

            Object serviceListenerProxy = Proxy.newProxyInstance(this.getClass().getClassLoader(),
                    new Class<?>[] {
                        sServiceListenerClass
                    }, new InvocationHandler() {
                        public Object invoke(Object proxy, Method method, Object[] args)
                                throws Throwable {
                            if (method.getName().equals("onServiceConnected")) {
                                //#ifdef NMSP_LOG
                                if(log.isInfoEnabled())
                                    log.info("BluetoothHeadsetOEM native onServiceConnected()");
                                //#endif
                                synchronized(mHeadsetSyncObj) {
                                    if (!_serviceConnectionGaveUp) {
                                        _serviceCbCalled = true;
                                        _nativeHeadset = args[1];
                                        mHeadsetSyncObj.notify();
                                    } else {
                                        closeNativeHeadset(args[1]);
                                    }
                                }
                            } else if (method.getName().equals("onServiceDisconnected")) {
                                //#ifdef NMSP_LOG
                                if(log.isInfoEnabled())
                                    log.info("BluetoothHeadsetOEM native onServiceDisconnected()");
                                //#endif
                                synchronized(mHeadsetSyncObj) {
                                    if (!_serviceConnectionGaveUp) {
                                        _serviceCbCalled = true;
                                        mHeadsetSyncObj.notify();
                                    }
                                }
                            }
                            return null;
                        }
                    });

            synchronized(mHeadsetSyncObj) {
                long startTime = System.currentTimeMillis();
                Class<?> BtProfileClass = r.getClassForName("android.bluetooth.BluetoothProfile");
                Class<?> BtAdapterClass = r.getClassForName("android.bluetooth.BluetoothAdapter");

                try {
                    Method sISEnbaled = r.getMethod(BtAdapterClass, "isEnabled");
                    boolean isEnabled = (Boolean)sISEnbaled.invoke(adapter);
                    if (!isEnabled) {
                        return;
                    }
                } 
                catch (Exception e) {
                    e.printStackTrace();
                }

                Method sGetProfileProxy = r.getMethod(BtAdapterClass, "getProfileProxy", Context.class, sServiceListenerClass,int.class);

                try {
                    sGetProfileProxy.invoke(adapter, context, serviceListenerProxy, r.getFieldValue(BtProfileClass, "HEADSET"));
                } catch (Exception e) {
                     e.printStackTrace();
                }

                try
                {
                    mHeadsetSyncObj.wait(1000);
                } catch (InterruptedException e)
                {
                    //#ifdef NMSP_LOG
                    if(log.isErrorEnabled())
                        log.error("BluetoothHeadsetOEM native mHeadsetSyncObj.wait() threw exception:" + e);
                    //#endif
                }
                if (!_serviceCbCalled) {
                    //#ifdef NMSP_LOG
                    if(log.isErrorEnabled())
                        log.error("BluetoothHeadsetOEM native service NOT connected in time. Gave up!!!");
                    //#endif
                    _serviceConnectionGaveUp = true;
                } else {
                    //#ifdef NMSP_LOG
                    if(log.isInfoEnabled())
                        log.info("BluetoothHeadsetOEM native service connection took " + (System.currentTimeMillis() - startTime) + "ms");
                    //#endif
                }
            }
        }
    }

    @SuppressWarnings("unchecked")
    public BluetoothDevice getConnectedDevice() {
        if (_reflected && _reflectedHeadset != null) {
            return _reflectedHeadset.getConnectedDevice();
        }
        else if (!_reflected && _nativeHeadset != null) {
            Class<?> BtProfileClass = r.getClassForName("android.bluetooth.BluetoothProfile");
            Method sGetConnectedDevices = r.getMethodOrNull(BtProfileClass, "getConnectedDevices");

            List<BluetoothDevice> devices = null;
            try {
                devices = (List<BluetoothDevice>) sGetConnectedDevices.invoke(_nativeHeadset);
            } catch (Exception e) {
                e.printStackTrace();
            }
            return devices == null ? null : (devices.size() > 0 ? devices.get(0) : null);
        }
        else {
            return null;
        }
    }

    public boolean startVoiceRecognition() {
        if (_reflected) {
            return _reflectedHeadset == null ? false : _reflectedHeadset.startVoiceRecognition();
        }
        else {
            //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("BluetoothHeadsetOEM startVoiceRecognition() called on native headset!!!");
            //#endif
            return false;
        }
    }

    public boolean stopVoiceRecognition() {
        if (_reflected) {
            return _reflectedHeadset == null ? false : _reflectedHeadset.stopVoiceRecognition();
        }
        else {
            //#ifdef NMSP_LOG
            if(log.isErrorEnabled())
                log.error("BluetoothHeadsetOEM stopVoiceRecognition() called on native headset!!!");
            //#endif
            return false;
        }
    }

    public void close() {
        if (_reflected) {
            closeReflectedHeadset();
        }
        else {
            if (_nativeHeadset != null) {
                closeNativeHeadset(_nativeHeadset);
                _nativeHeadset = null;
            }
        }
    }

    private void closeReflectedHeadset()
    {
        if (_reflectedHeadset != null)
        {
            _reflectedHeadset.close();
            _reflectedHeadset = null;
        }
    }

    private void closeNativeHeadset(Object proxy)
    {
        Class<?> BtProfileClass = r.getClassForName("android.bluetooth.BluetoothProfile");
        Class<?> BtAdapterClass = r.getClassForName("android.bluetooth.BluetoothAdapter");

        Method sCloseProfileProxy = r.getMethodOrNull(BtAdapterClass, "closeProfileProxy", int.class, BtProfileClass);

        // No need to reflect getDefaultAdapter it is available since API level 5
        BluetoothAdapter adapter = BluetoothAdapter.getDefaultAdapter();

        try {
            sCloseProfileProxy.invoke(adapter, (Integer) r.getFieldValue(BtProfileClass, "HEADSET"), proxy);
        }
        catch (Exception e) {
             e.printStackTrace();
        }
    }
}
