package com.nuance.nmsp.client2.sdk.oem.bluetooth;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

//#ifdef NMSP_LOG
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory;
import com.nuance.nmsp.client2.sdk.common.oem.api.LogFactory.Log;
//#endif


/**
 * Reflection utilities. Simplifies dynamic object creation, field access,
 * and method invocation.
 */
public class Reflection {
    //#ifdef NMSP_LOG
    // Logger
    private Log log = LogFactory.getLog(this.getClass());
    //#endif

    public Class<?> getClassForName(String name) {
        Class<?> c = null;
        try {
            c = Class.forName(name);
        } catch(ClassNotFoundException e) {
            e.printStackTrace();
            throw new RuntimeException("getClassForName failed", e);
        }
        return c;
    }

    public Object getFieldValue(Class<?> c, String fieldName) {
        Object value = null;
        try {
            value = c.getField(fieldName).get(null);
        } catch(Exception e) {
            e.printStackTrace();
            throw new RuntimeException("getFieldValue failed", e);
        }
        return value;
    }

    public Object getFieldValue(Class<?> c, String fieldName, Object defaultValue) {
        Object value = defaultValue;
        try {
            value = c.getField(fieldName).get(null);
        } catch(Exception e) {
            // ignore; return default value
        }
        return value;
    }

    public Constructor<?> getConstructor(Class<?> c, Class<?>... parameterTypes) {
        Constructor<?> ctor = null;
        try {
            ctor = c.getConstructor(parameterTypes);
        } catch(Exception e) {
            e.printStackTrace();
            throw new RuntimeException("getConstructor failed", e);
        }
        return ctor;
    }

    public Method getMethod(Class<?> c, String methodName, Class<?>... parameterTypes) {
        Method method = null;
        try {
            method = c.getMethod(methodName, parameterTypes);
        } catch(Exception e) {
            e.printStackTrace();
            throw new RuntimeException("getMethod failed", e);
        }
        return method;
    }

    public Method getMethodOrNull(Class<?> c, String methodName, Class<?>... parameterTypes) {
        Method method = null;
        try {
            method = c.getMethod(methodName, parameterTypes);
        } catch(Exception e) {
            // ignore
        }
        return method;
    }

    public Object newObject(String classname) {
        Object result = null;
        if(classname != null) {
            try {
                Class<?> clazz = Class.forName(classname);
                result = clazz.newInstance();
            } catch(ClassNotFoundException cnfe) {
                handleException(cnfe);
            } catch(IllegalAccessException iae) {
                handleException(iae);
            } catch(InstantiationException ie) {
                handleException(ie);
            }
        }
        return result;
    }

    public Constructor<?> getConstructorForName(String classname, Class<?>... paramTypes) {
        Constructor<?> result = null;
        Class<?> clazz = getClassForName(classname);
        if(clazz != null) {
            result = getConstructor(clazz, paramTypes);
        }
        return result;
    }

    public Object newObject(Constructor constructor, Object... params) {
        Object result = null;
        if(constructor != null) {
            try {
                result = constructor.newInstance(params);
            } catch(IllegalAccessException iae) {
                handleException(iae);
            } catch(InstantiationException ie) {
                handleException(ie);
            } catch (InvocationTargetException ite) {
                handleException(ite);
            }
        }
        return result;
    }

    private void handleException(Exception e) {
        //#ifdef NMSP_LOG
        if (log.isErrorEnabled())
            log.error("Exception received - ", e);
        //#endif
    }
}
