package com.xyz.sdk.http;

import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;

import com.xyz.sdk.http.interfaces.ProgressInteface;
import com.xyz.sdk.http.manager.HttpRequestManager;
import com.xyz.sdk.json.XYZJson;

import java.util.HashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

/**
 * Created by zhangzheng on 2016/2/18.
 */
public class XYZHttp<M> {
    public ExecutorService executorService = Executors.newCachedThreadPool();

    private XYZHttp() {
    }

    private static HashMap<String, SucessCallBack> sucessCallbacks = new HashMap<>();
    private static HashMap<String, FailureCallBack> failureCallbacks = new HashMap<>();
    private static HashMap<String, ErrorCallBack> errorCallbacks = new HashMap<>();
    private static HashMap<String, ComputeCallBack> computeCallbacks = new HashMap<>();
    private static HashMap<String, ProgressInteface> progressInterfaces = new HashMap<>();
    private static HashMap<String, Future> futures = new HashMap<>();
    private static HashMap<String, Class> classs = new HashMap<>();

    private static ErrorCallBack defaultErrorCallBack;
    private static FailureCallBack defaultFailureCallBack;

    public static void setDefaultErrorCallBack(ErrorCallBack defaultErrorCallBack) {
        XYZHttp.defaultErrorCallBack = defaultErrorCallBack;
    }

    public static void setDefaultFailureCallBack(FailureCallBack defaultFailureCallBack) {
        XYZHttp.defaultFailureCallBack = defaultFailureCallBack;
    }

    private String url;

    public void setUrl(String url) {
        this.url = url;
    }

    private Future preFuture;

    public void setPreFuture(Future preFuture) {
        this.preFuture = preFuture;
    }

    public interface SucessCallBack<T> {
        void sucess(T response);
    }

    public interface FailureCallBack<T> {
        void failure(T response);
    }

    public interface ErrorCallBack {
        void error();
    }

    public interface ComputeCallBack {
        void compute();
    }

    public static XYZHttp<String> get(final XYZUrl xyzUrl) {
        return request(xyzUrl, HttpRequestManager.RequestModel.GET);
    }

    public static XYZHttp<String> post(final XYZUrl xyzUrl) {
        return request(xyzUrl, HttpRequestManager.RequestModel.POST);
    }

    private static XYZHttp<String> request(final XYZUrl xyzUrl, final HttpRequestManager.RequestModel model) {
        final XYZHttp<String> httpManager = new XYZHttp<String>();
        Future<?> future = httpManager.executorService.submit(new Runnable() {
            @Override
            public void run() {
                String result = HttpRequestManager.request(xyzUrl, model);
                new ResultHandler(xyzUrl, httpManager, result).sendEmptyMessage(0);
            }
        });
        httpManager.setUrl(xyzUrl.getApi());
        httpManager.setPreFuture(futures.get(xyzUrl.getApi()));
        futures.put(xyzUrl.getApi(), future);
        return httpManager;
    }

    private static class ResultHandler extends Handler {
        private XYZUrl xyzUrl;
        private XYZHttp<String> httpManager;
        private String result;

        public ResultHandler(XYZUrl xyzUrl, XYZHttp<String> httpManager, String result) {
            super(Looper.getMainLooper());
            this.xyzUrl = xyzUrl;
            this.httpManager = httpManager;
            this.result = result;
        }

        @Override
        public void handleMessage(Message msg) {
            Log.i("result", result);
            httpManager.controlResult(xyzUrl.getApi(), result);
            clearAllSave(xyzUrl.getApi());
        }
    }

    public void controlResult(String url, String result) {
        Class aClass = classs.get(url);
        M bean = null;
        try {
            if (aClass == null) {
                bean = (M) result;
            } else {
                bean= (M) XYZJson.parseJson(result, aClass);
            }
        } catch (Exception e) {
        }

        if (computeCallbacks.get(url) != null) {
            computeCallbacks.get(url).compute();
        }

        if (progressInterfaces.get(url) != null) {
            progressInterfaces.get(url).dismiss();
        }

        if (bean == null || result == null || result.trim().equals("")) {
            if (errorCallbacks.get(url) != null) {
                errorCallbacks.get(url).error();
                return;
            }
            if(defaultErrorCallBack!=null){
                defaultErrorCallBack.error();
            }
            return;
        }
        if ((bean instanceof Response) && !((Response) bean).validate()) {
            if (failureCallbacks.get(url) != null) {
                failureCallbacks.get(url).failure(bean);
                return;
            }
            if(defaultFailureCallBack!=null){
                defaultFailureCallBack.failure(bean);
            }
            return;
        }
        SucessCallBack sucessCallBack = sucessCallbacks.get(url);
        if (sucessCallBack != null) {
            sucessCallBack.sucess(bean);
        }
    }

    public <T extends Response> XYZHttp<T> transformation(Class<T> cla) {
        classs.put(url, cla);
        XYZHttp<T> httpManager = new XYZHttp<T>();
        httpManager.setUrl(url);
        httpManager.setPreFuture(preFuture);
        return httpManager;
    }

    public XYZHttp<M> sucess(SucessCallBack<M> sucessCallback) {
        sucessCallbacks.put(url, sucessCallback);
        return this;
    }

    public XYZHttp<M> error(ErrorCallBack errorCallBack) {
        errorCallbacks.put(url, errorCallBack);
        return this;
    }

    public XYZHttp<M> failure(FailureCallBack<M> failureCallBack) {
        failureCallbacks.put(url, failureCallBack);
        return this;
    }

    public XYZHttp<M> clearRepeatRequest() {
        Future future = preFuture;
        if (future != null) {
            future.cancel(false);
        }
        return this;
    }

    public XYZHttp<M> joinProgress(ProgressInteface progress) {
        progressInterfaces.put(url, progress);
        progress.show();
        return this;
    }

    public void complete(ComputeCallBack compute) {
        computeCallbacks.put(url, compute);
    }

    private static void clearAllSave(String url) {
        sucessCallbacks.remove(url);
        futures.remove(url);
        classs.remove(url);
        computeCallbacks.remove(url);
        progressInterfaces.remove(url);
        failureCallbacks.remove(url);
        errorCallbacks.remove(url);
    }

}
