package com.xyz.sdk.json;

import com.xyz.sdk.json.bean.JsonParseBean;
import com.xyz.sdk.json.interfaces.IJsonParse;
import com.xyz.sdk.json.manager.JsonManager;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

/**
 * Created by zhangzheng on 2016/2/19.
 */
public class XYZJson {

    public static <T> T parseJson(String json, Class<T> cla) {
        try {
            IJsonParse jsonParse = JsonManager.getInstence().getJsonParse();
            if(jsonParse!=null){
                return jsonParse.parseJson(json,cla);
            }
            JsonParseBean jsonParseBean = parseJsonToBean(json);
            return parseJsonBean(jsonParseBean, cla);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    private static <T> T parseJsonBean(JsonParseBean jsonParseBean, Class<T> cla) {
        if(cla==null){
            return null;
        }
        T instence = null;
        try {
            instence = cla.newInstance();
        } catch (Exception e) {
            return null;
        }
        HashMap<String, Object> baseParams = jsonParseBean.getBaseParams();
        Set<String> keyTypes = baseParams.keySet();
        for (String keyType : keyTypes) {
            try {
                String[] split = keyType.split("-");
                String key = split[0];
                String type = split[1];
                Object value = baseParams.get(keyType);
                fullBean(instence, key, Integer.valueOf(type), value);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return instence;
    }

    private static void fullBean(Object bean, String key, int type, Object value) throws Exception {
        String newKey = key.substring(0, 1).toUpperCase() + key.replaceFirst("\\w", "");
        Method method;
        Field declaredField = bean.getClass().getDeclaredField(key);
        method = bean.getClass().getMethod("set" + newKey, declaredField.getType());
        if (value.getClass() == JsonParseBean.class) {
            Object object = parseJsonBean((JsonParseBean) value, declaredField.getType());
            method.invoke(bean, object);
        } else if ((value instanceof List) && ((List) value).size() > 0 && ((List) value).get(0).getClass() == JsonParseBean.class) {
            List<JsonParseBean> jsonBeans = (List<JsonParseBean>) value;
            List<Object> objects = new ArrayList<>();
            for (JsonParseBean jb : jsonBeans) {
                Object object = parseJsonBean(jb, getGenericClass(declaredField));
                objects.add(object);
            }
            method.invoke(bean, objects);
        } else {
            value = compatibleIntAndString(declaredField.getType(), value);
            method.invoke(bean, value);
        }
    }

    /**
     * 得到类的泛型类型
     * @param f
     * @return
     * @throws Exception
     */
    private static Class getGenericClass(Field f)  {
        Type fc = f.getGenericType(); // 关键的地方，如果是List类型，得到其Generic的类型

        if (fc instanceof ParameterizedType) //如果是泛型参数的类型
        {
            ParameterizedType pt = (ParameterizedType) fc;

            Class genericClazz = (Class) pt.getActualTypeArguments()[0]; //得到泛型里的class类型对象。

            return genericClazz;
        }
        return null;
    }

    /**
     * 兼容基本类型数据不一致
     * @param methodClass
     * @param value
     * @return
     */
    private static Object compatibleIntAndString(Class<?> methodClass, Object value) {
        if (methodClass == Integer.class && value.getClass() == String.class) {
            return Integer.valueOf((String) value);
        }else if(methodClass==Double.class&& value.getClass() == String.class){
            return Double.valueOf((String) value);
        }else if(methodClass==Float.class&& value.getClass() == String.class){
            return Float.valueOf((String) value);
        }else if (methodClass == String.class && (value.getClass() == Integer.class
                ||value.getClass()==Double.class||value.getClass()==Float.class)) {
            return value + "";
        }
        return value;
    }

    private static JsonParseBean parseJsonToBean(String json) throws JSONException {

        JSONTokener parse = new JSONTokener(json);
        JSONObject jsonObject = (JSONObject) parse.nextValue();
        JsonParseBean jsonParseBean = parseJsonObject(jsonObject);
        return jsonParseBean;
    }

    private static final JsonParseBean parseJsonObject(JSONObject jsonObject) throws JSONException {
        Iterator<String> keys = jsonObject.keys();
        JsonParseBean jsonBean = new JsonParseBean();
        while (keys.hasNext()) {
            String key = keys.next();
            Object value = jsonObject.get(key);
            if (value instanceof JSONObject) {
                JsonParseBean jsonParseBean = parseJsonObject((JSONObject) value);
                jsonBean.putObject(key, jsonParseBean);
            } else if (value instanceof JSONArray) {
                JSONArray ja = (JSONArray) value;
                if (ja.length() == 0) {
                    continue;
                }
                List array = parseJsonArray(ja);
                jsonBean.putArray(key, array);
            } else if (value instanceof Boolean) {
                jsonBean.putBoolean(key, (Boolean) value);
            } else if (value instanceof Integer) {
                jsonBean.putInt(key, (Integer) value);
            } else if (value instanceof String) {
                jsonBean.putString(key, (String) value);
            }else {
                jsonBean.putString(key,value+"");
            }
        }
        return jsonBean;
    }

    private static List parseJsonArray(JSONArray ja) throws JSONException {
        Object jaObject = ja.get(0);
        List array = new ArrayList();
        for (int i = 0; i < ja.length(); i++) {
            if (jaObject instanceof JSONObject) {
                JSONObject jo = (JSONObject) ja.get(i);
                JsonParseBean jsonParseBean = parseJsonObject(jo);
                array.add(jsonParseBean);
            } else {
                array.add(ja.get(i));
            }
        }
        return array;
    }
}
