package com.xyz.sdk.product.bean.plan;

import com.xyz.sdk.utils.StringUtils;

import java.util.ArrayList;

/**
 * 性别集合
 *
 * MALE:男
 * FEMALE:女
 *
 * @author wangxiulong
 *
 */
public enum Sex
{
    MALE("1", true, "男"), FEMALE("0", false, "女");

    private String code;
    private boolean value;
    private String description;

    private Sex(String c, boolean v, String d)
    {
        code = c;
        value = v;
        description = d;
    }

    public String code()
    {
        return code;
    }

    public boolean value()
    {
        return value;
    }

    public String description()
    {
        return description;
    }

    /**
     * 从字符串中解析性别信息
     *
     * @param str:1-男，0-女
     * @return
     */
    public static Sex parse(String str)
    {
        if(StringUtils.isNull(str)) {
            return MALE;
        } else {
            if("0".equals(str)) {
                return FEMALE;
            } else {
                return MALE;
            }
        }
    }

    /**
     * @param index	0:女 1：男
     * @return
     */
    public static Sex parse(int index)
    {
        if(index == 0)
        {
            return FEMALE;
        }
        else
        {
            return MALE;
        }
    }


    public static ArrayList<Sex> parse(ArrayList<String> list)
    {
        ArrayList<Sex> sexList = new ArrayList<Sex>();
        for(String str : list) {
            try {
                sexList.add(Sex.parse(str));
            } catch(Exception e) {
                continue;
            }
        }
        return sexList;
    }

    public static String[] sexDescriptions()
    {
        return new String[]{FEMALE.description(),MALE.description()};
    }

    /**
     * 从布尔数据中解析性别信息
     *
     * @param falg：true-男，false-女
     * @return
     */
    public static Sex parse(boolean falg) {
        if(falg) {
            return MALE;
        } else {
            return FEMALE;
        }
    }
    public boolean equals(Sex sex) {
        return this.code.equals(sex.code());
    }

    public boolean equals(String sex) {
        return this.code.equals(sex);
    }
}
