package com.xyz.sdk.product.manager;

import android.content.Context;
import android.content.res.XmlResourceParser;
import android.util.Xml;

import com.xyz.sdk.XYZSDKManager;
import com.xyz.sdk.json.XYZJson;
import com.xyz.sdk.product.bean.plan.AreaBase;
import com.xyz.sdk.product.bean.plan.City;
import com.xyz.sdk.product.bean.plan.Continent;
import com.xyz.sdk.product.bean.plan.CountryBase;
import com.xyz.sdk.product.bean.plan.County;
import com.xyz.sdk.product.bean.plan.Province;
import com.xyz.sdk.product.resources.PlanResourses;
import com.xyz.sdk.utils.XYZSdkUtils;

import org.xmlpull.v1.XmlPullParser;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by zhangzheng on 2016/2/23.
 */
public class AreaManager {
	private static AreaManager mInstance = new AreaManager();

	private Context mContext = null;

	private ArrayList<Province> provinces = null;

	private ArrayList<Province> provinces2 = null;

	private ArrayList<Continent> continents = null;

	private AreaManager() {
		mContext = XYZSDKManager.getContext();

		continents = new ArrayList<Continent>();

		initWorldArea(getWorldAreaJson());
	}

	public static AreaManager getInstance() {
		return mInstance;
	}

	public ArrayList<Province> getProvinces() {
		if (provinces == null || provinces.size() == 0) {
			provinces = initChinaArea(getChinaAreaJson());
		}
		return provinces;
	}

	public ArrayList<Province> getProvincesContairCounty() {
		if (provinces2 == null || provinces2.size() == 0) {
			provinces2 = initChinaArea(getChinaAreaJsonContairCounty());
		}
		return provinces2;
	}

	public Province getProvince(String code) {
		if (XYZSdkUtils.isEmpty(code)) {
			return null;
		}

		if (XYZSdkUtils.isEmpty(provinces)) {
			return null;
		}

		for (Province province : provinces) {
			String c = province.getCode();
			if (XYZSdkUtils.isEmpty(c)) {
				continue;
			}

			if (code.equals(c)) {
				return province;
			}
		}
		return null;
	}

	public static City getCity(Province province, String code) {
		if (XYZSdkUtils.isEmpty(province)) {
			return null;
		}

		if (XYZSdkUtils.isEmpty(code)) {
			return null;
		}

		ArrayList<City> citys = province.getLeaves();
		if (XYZSdkUtils.isEmpty(citys)) {
			return null;
		}

		for (City city : citys) {
			String c = city.getCode();
			if (XYZSdkUtils.isEmpty(c)) {
				continue;
			}

			if (code.equals(c)) {
				return city;
			}
		}
		return null;
	}

	public static County getCounty(City city, String code) {
		if (XYZSdkUtils.isEmpty(city)) {
			return null;
		}

		if (XYZSdkUtils.isEmpty(code)) {
			return null;
		}
		List<County> countys = city.getLeaves();
		if (countys == null) {
			return null;
		}
		for (County county : countys) {
			if (county.getCode().equals(code)) {
				return county;
			}
		}
		return null;
	}

	public ArrayList<Continent> getContinents() {
		return continents;
	}

	/**
	 * 从json数据中抽取中国地域信息
	 * 
	 * @param json
	 */
	private ArrayList<Province> initChinaArea(String json) {
		AreaBase area = (AreaBase) XYZJson.parseJson(json, AreaBase.class);
		ArrayList<Province> provinces = new ArrayList<>();
		if (area != null) {
			for (int i = 0; i < area.getAreas().size(); i++) {
				if (area.getAreas().get(i).getLeaves() != null) {
					provinces.add(area.getAreas().get(i));
				}
			}
		}
		return provinces;
	}

	/**
	 * 从xml文件中获取中国地域的json信息
	 * 
	 * @return
	 */
	private String getChinaAreaJson() {
		return getAreaJsonFromAssert("address.json");
	}

	private String getChinaAreaJsonContairCounty() {
		return getAreaJsonFromAssert("newarea.json");
	}

	private String getAreaJsonFromAssert(String assertName) {
		String json = "";
		try {
			InputStream in = XYZSdkUtils.getInputStreamFromClassPath(
					assertName, PlanResourses.class);
			byte[] buffer = new byte[in.available()];
			in.read(buffer);
			json = new String(buffer, "utf-8");
			return json;
		} catch (IOException e) {
			e.printStackTrace();
		}
		return json;
	}

	/**
	 * 从json数据中抽取世界地域信息
	 * 
	 * @param json
	 */
	private void initWorldArea(String json) {
		CountryBase area = (CountryBase) XYZJson.parseJson(json,
				CountryBase.class);
		if (area != null) {
			for (int i = 0; i < area.getAreas().size(); i++) {
				if (area.getAreas().get(i).getLeaves() != null) {
					continents.add(area.getAreas().get(i));
				}
			}
		}
	}

	/**
	 * 从xml文件中获取世界地域的json信息
	 * 
	 * @return
	 */
	private String getWorldAreaJson() {
		StringBuilder sb = new StringBuilder("");
		XmlPullParser xrp = Xml.newPullParser();
		InputStream inputStream = XYZSdkUtils.getInputStreamFromClassPath(
				"destinationstrings.xml", PlanResourses.class);
		try {
			xrp.setInput(inputStream, "UTF-8");
			while (xrp.getEventType() != XmlResourceParser.END_DOCUMENT) {
				if (xrp.getEventType() == XmlResourceParser.START_TAG) {
					String name = xrp.getName();
					if (name.equals("destination")) {
						sb.append(xrp.nextText());
					}
				}

				xrp.next();
			}
		} catch (Exception e) {
		}

		return sb.toString();
	}

	public void clearMemory() {
		if (provinces != null) {
			provinces.clear();
		} else if (provinces2 != null) {
			provinces2.clear();
		}
	}

	public void preLoadData() {
		new Thread(new Runnable() {
			@Override
			public void run() {
				provinces = initChinaArea(getChinaAreaJson());
				provinces2 = initChinaArea(getChinaAreaJsonContairCounty());
			}
		}).start();
	}

	public void preLoadData(final boolean isContairCounty) {
		new Thread(new Runnable() {
			@Override
			public void run() {
				if (!isContairCounty) {
					provinces = initChinaArea(getChinaAreaJson());
				} else {
					provinces2 = initChinaArea(getChinaAreaJsonContairCounty());
				}
			}
		}).start();
	}
}
