package com.xyz.sdk.product.manager;

import com.xyz.sdk.http.XYZHttp;
import com.xyz.sdk.http.model.reponse.PlanTrialResp;
import com.xyz.sdk.http.model.request.PlanTrialReq;
import com.xyz.sdk.product.interfaces.PlanTrialCompute;
import com.xyz.sdk.utils.StringUtils;

/**
 * Created by zhangzheng on 2016/2/23.
 * 保费试算管理类
 */
public class PlanTrialManager {

    private PlanTrialReqBuilder builder;
    private PlanTrialCompute computeListener;

    public void setPlanTrialReqBuilder(PlanTrialReqBuilder builder) {
        this.builder = builder;
    }

    public void setComputeListener(PlanTrialCompute computeListener) {
        this.computeListener = computeListener;
    }

    /**
     * 获得上一次未被回收的PlanTrialReqBuilder，但会缓存上一次的数据
     * @return
     */
    public PlanTrialReqBuilder getPlanTrialReqBuilder() {
        if(builder==null){
            builder=new PlanTrialReqBuilder();
        }
        return builder;
    }

    private PlanTrialManager(){}

    private static PlanTrialManager planTrialManager=new PlanTrialManager();

    public static PlanTrialManager getInstence(){
        return planTrialManager;
    }

    /**
     * 请求保费试算
     */
    public void requestPlanTrial(PlanTrialReqBuilder builder,PlanTrialCompute computeListener){
        this.builder = builder;
        this.computeListener=computeListener;
        builder.setPlanTrialManager(this);
        requestWeb(builder.createPlanTrialReq(), computeListener);
    }

    private void requestWeb(PlanTrialReq planTrial,final PlanTrialCompute computeListener){
        XYZHttp.get(planTrial).transformation(PlanTrialResp.class).sucess(new XYZHttp.SucessCallBack<PlanTrialResp>() {
            @Override
            public void sucess(PlanTrialResp planTrialResp) {
                if (computeListener != null) {
                    computeListener.onCompute(planTrialResp.getPlanId(), planTrialResp.getMarketPrice(), planTrialResp.getDiscountPrice());
                }
            }
        }).error(new XYZHttp.ErrorCallBack() {
            @Override
            public void error() {
                if (computeListener != null) {
                    computeListener.onCompute(null,null,null);
                }
            }
        }).failure(new XYZHttp.FailureCallBack<PlanTrialResp>() {
            @Override
            public void failure(PlanTrialResp response) {
                if (computeListener != null) {
                    computeListener.onCompute(null,null,null);
                }
            }
        });
    }

    private void compute(PlanTrialReqBuilder builder){
        if(computeListener!=null&&builder.isFull()){
            requestPlanTrial(builder,computeListener );
        }
    }

    public static class PlanTrialReqBuilder{
        private PlanTrialReq planTrial;
        private PlanTrialManager planTrialManager;

        private PlanTrialReq getPlanTrial() {
            if(planTrial==null){
                planTrial=new PlanTrialReq();
            }
            return planTrial;
        }

        private void setPlanTrialManager(PlanTrialManager planTrialManager) {
            this.planTrialManager = planTrialManager;
        }

        public PlanTrialReqBuilder setPlanTrialPlanId(String planId){
            getPlanTrial().setPlanId(planId);
            return this;
        }

        public PlanTrialReqBuilder setPlanTrialGender(String gender){
            getPlanTrial().setGender(gender);
            return this;
        }

        public PlanTrialReqBuilder setPlanTrialPeriodStart(String periodStart){
            getPlanTrial().setPeriodStart(periodStart);
            return this;
        }

        public PlanTrialReqBuilder setPlanTrialPeriodEnd(String periodEnd){
            getPlanTrial().setPeriodEnd(periodEnd);
            return this;
        }

        public PlanTrialReqBuilder setPlanTrialBirthday(String birthday){
            getPlanTrial().setBirthday(birthday);
            return this;
        }
        
        public boolean isFull(){
        	return getPlanTrial().isFull();
        }

        public void compute(){
            if(planTrialManager!=null){
                planTrialManager.compute(this);
            }
        }

        public PlanTrialReq createPlanTrialReq(){
            return getPlanTrial();
        }
    }
}
