package com.xyz.sdk.utils;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.telephony.TelephonyManager;
import android.util.Log;
import android.util.SparseArray;

import java.io.InputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class XYZSdkUtils {

    public static boolean isEmpty(Object obj) {
        return obj == null;
    }

    public static boolean isEmpty(String str) {
        return str == null || "".equals(str);
    }

    public static boolean isEmpty(List<?> list) {
        return list == null || list.size() == 0;
    }

    public static boolean isEmpty(SparseArray<?> array) {
        return array == null || array.size() == 0;
    }


    public static final String HOTLINE_NUMBER = "4006675599";

    private static long lastClickTime;

    public static boolean isFastDoubleClick() {
        return isFastDoubleClick(2000);
    }

    public static boolean isFastDoubleClick(int offsetTime) {
        long time = System.currentTimeMillis();
        long timeD = time - lastClickTime;
        if (0 < timeD && timeD < offsetTime) {
            return true;
        }
        lastClickTime = time;
        return false;
    }


    /**
     * 获取两个毫秒单位的日期间隔天数
     *
     * @param start 开始时间，单位毫秒
     * @param end   结束时间，单位毫秒
     * @return
     */
    public static int getBetweenDays(long start, long end) {
        long nd = 24 * 60 * 60 * 1000;
        long time = end - start;
        return (int) (time / nd);
    }

    /**
     * 将元数据前补零，补后的总长度为指定的长度，以字符串的形式返回
     *
     * @param sourceDate
     * @param formatLength
     * @return 重组后的数据
     */
    public static String frontCompWithZore(int sourceDate, int formatLength) {
      /*
       * 0 指前面补充零
	   * formatLength 字符总长度为 formatLength 
	   * d 代表为正数。 
	   */
        String newString = String.format("%0" + formatLength + "d", sourceDate);
        return newString;
    }

    public static boolean isLater(String time) {
        if (!isEmpty(time)) {
            //如果格式为yyyy-MM-dd则补充成完整格式 +空格HH
            if (!time.contains(" ")) {
                time = time + " " + "0";
            }
            try {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH");
                Date d = sdf.parse(time);
                long start = d.getTime();
                long current = Calendar.getInstance().getTime().getTime();
                if (start - current < 0) {
                    return true;
                }
            } catch (ParseException ex) {
                return false;
            }
        }

        return false;
    }


    /**
     * 根据value从Map中获取其key值
     *
     * @param value
     * @param hm
     * @return 对应key
     */
    public static String getHashMapKey(String value, Map<String, String> hm) {
        if (hm.containsValue(value)) {
            Iterator<?> it = hm.entrySet().iterator();
            while (it.hasNext()) {
                Map.Entry<String, String> obj = (Map.Entry<String, String>) it.next();
                if (value.equals(obj.getValue()))
                    return obj.getKey().toString();
            }
        }
        return "";
    }

    /**
     * 获取应用版本号
     *
     * @param context
     * @return 版本号
     */
    public static String getAppVersion(Context context) {
        String versionName = "1.0.0";
        try {
            // ---get the package info---
            PackageManager pm = context.getPackageManager();
            PackageInfo pi = pm.getPackageInfo(context.getPackageName(), 0);
            versionName = pi.versionName;
        } catch (Exception e) {
            Log.e("VersionInfo", "Exception", e);
        }
        return versionName;
    }

    public static int getAppVersionCode(Context context) {
        int versionCode=-1;
        try {
            // ---get the package info---
            PackageManager pm = context.getPackageManager();
            PackageInfo pi = pm.getPackageInfo(context.getPackageName(), 0);
            versionCode = pi.versionCode;
        } catch (Exception e) {
            Log.e("VersionInfo", "Exception", e);
        }
        return versionCode;
    }

    /**
     * 获得唯一的设备编号
     * @return
     */
    public static String getOnlyDeviceId(Context context){
        TelephonyManager telephonyManager = (TelephonyManager)context.getSystemService(Context.TELEPHONY_SERVICE);
        //获取本机MIEI号码（仅手机存在）
        String deviceId = telephonyManager.getDeviceId();

        if(!StringUtils.isNull(deviceId)){
            return deviceId;
        }
        //获取设备序列号
        String sn =  telephonyManager.getSimSerialNumber();

        if(!StringUtils.isNull(sn)){
            return sn;
        }

        //获取本机电话号码
        String phonenumber = telephonyManager.getLine1Number();

        if(!StringUtils.isNull(phonenumber)){
            return phonenumber;
        }

        WifiManager wifi = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
        WifiInfo info = wifi.getConnectionInfo();
        //获取wifi设备的mac码
        String mac = info.getMacAddress();

        if(!StringUtils.isNull(mac)){
            return mac;
        }

        //获取本机型号
        String phonetype = android.os.Build.MODEL;

        if(StringUtils.isNull(phonetype)){
            return phonetype;
        }

        return "";
    }

    /**
     * 网络是否可用
     *
     * @param context
     * @return
     */
    public static boolean isConnect(Context context) {
        ConnectivityManager connectivity = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        if (connectivity == null) {
            return false;
        } else {
            NetworkInfo[] info = connectivity.getAllNetworkInfo();
            if (info != null) {
                for (int i = 0; i < info.length; i++) {
                    if (info[i].isAvailable()) {
                        return true;
                    }
                }
            }
        }
        return false;
    }


    public static boolean isWifi(Context context) {
        ConnectivityManager connectivityManager = (ConnectivityManager) context
                .getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetInfo = connectivityManager.getActiveNetworkInfo();
        if (activeNetInfo != null
                && activeNetInfo.getType() == ConnectivityManager.TYPE_WIFI) {
            return true;
        }
        return false;
    }

    public static boolean is3G(Context context) {
        ConnectivityManager connectivityManager = (ConnectivityManager) context
                .getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetInfo = connectivityManager.getActiveNetworkInfo();
        if (activeNetInfo != null
                && activeNetInfo.getType() == ConnectivityManager.TYPE_MOBILE) {
            return true;
        }
        return false;
    }
    
    /**
     * 从class目录下读取资源文件
     * @param name
     * @param cla
     * @return
     */
    public static InputStream getInputStreamFromClassPath(String name,Class<?> cla) {

		String packageName = "/" + cla.getName().replace('.', '/');

		packageName = packageName.substring(0, packageName.length() - cla.getSimpleName().length());

		String allName = packageName + name;

		InputStream is = cla.getResourceAsStream(allName);

		return is;
	}

}
